function YADA
% YADA: Initializes the YADA GUI.
%
% USAGE:
%
%       YADA
%
% Accepts not inputs and returns no output.
%
% This file checks if the YADA files can be located by Matlab. Once the path is properly setup, it initializes
% the YADA GUI. When the GUI function exits, the GUI is deleted and any files stored in the tmp directory are
% deleted. Also, the Matlab path is reset to its original value.
%
% NOTE: To run YADA, this file should be executed from the Matlab prompt.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 17, 2006.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-07-2006: Updated function documentation.
%
% * 20-07-2006: The path is now stored in the variable ProgPath.
%
% * 21-07-2006: The YADA GUI call was added.
%
% * 24-07-2006: Updated function documentation.
%
% * 26-07-2006: Included tests for operating system and screen size.
%
% * 27-07-2006: Renamed FixPath to FixYADAPath! Made sure the path is only changed to include the YADA directories
%               while YADA is running. The original path is reset just before YADA exists. Also, all files copied to
%               the directory tmp are always deleted before YADA closes!
%
% * 01-08-2006: Added check for multiple instances of YADA.
%
% * 01-09-2006: Added a warning off for YADA.
%
% * 06-09-2006: Fixed the warning command for Matlab 7 and later.
%
% * 23-11-2006: YADAGUI no longer takes the ProgPath input. Moreover, FixYADAPath is not tested. The YADAPath
%               function is always run. The file "iswinxp.txt" is now deleted before running the YADAGUI function.
%
% * 02-02-2007: Updated the documentation.
%
% * 11-04-2007: Added a clear('all') call.
%
% * 02-11-2007: Updated the documentation.
%
% * 03-03-2008: Took the changed status of observed variable scenarios into account.
%
% * 04-03-2008: Took the changed status of shock scenarios into account.
%
% * 10-03-2008: Added the clear('all') command towards the end and the beginning.
%
% * 17-03-2008: The code now takes into account that the Waggoner-Zha type of conditional forecasting
%               has been stripped from the public version of YADA.
%
% * 18-03-2008: A slight change in directory structure where all add-ons for YADA are now stored
%               in sub-directories to "addons" rather than in "private".
%
% * 19-03-2008: Took into account that "CondPredictionKalmanSmoother" has been moved from the "filter"
%               directory.
%
% * 31-03-2008: Make sure that shellprint is removed from the bin directory.
%
% * 13-05-2008: Made sure that EditDataDLG is deleted from the gui directory.
%
% * 19-05-2008: Old YADA figures are now being deleted by YADA.
%
% * 21-05-2008: Updated the documentation.
%
% * 23-05-2008: Added the #function pragme.
%
% * 27-06-2008: Removed all code where files that have been moved are deleted in their old directories
%               when still found there.
%
% * 23-07-2008: The function now deletes "logStdbetaPDF.m" if found in "dist" directory. YADA now uses
%               a general beta prior.
%
% * 24-07-2008: Deletes "SimulateAnSchorfheideDSGEmodel.m" and "TestKalmanFilter.m" if found in the base
%               directory.
%
% * 18-08-2008: Deletes "DSGEPredictionOVSTheta.m" if found in the addons directory.
%
% * 25-08-2008: Removed a number of file checks and deletes.
%
% * 28-11-2008: Updated the documentation and added a dos('dir') check.
%
% * 04-12-2008: Changed the screen height requirement to a screen height nag!
%
% * 05-12-2008: Corrected spelling for the nag about the screen height. The screen height is now measured
%               through the 4th and 2nd elements of vector obtained through the get(0,'ScreenSize') command.
%
% * 08-12-2008: Added quotes around the path for checking if YADA.exe exists. Added code for checking if the
%               license has been accepted or not.
%
% * 05-01-2009: Updated the documentation.
%
% * 26-02-2009: The function "GraphEditDataDLG" has been moved to the "gui" directory. It's therefore deleted
%               from the addons directory is found to reside there.
%
% * 09-03-2009: Added quotation marks around paths for the dos del commands.
%
% * 30-06-2009: YADA.m now also tests if it is running on a 64-bit windows operating system.
%
% * 27-11-2009: The function deletes "DSGEPosteriorSampling.m" from the "montecarlo" directory if it find the
%               file.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-02-2010: Added a test for the maximum number of computational threads.
%
% * 12-02-2010: Added functions to the #function pragma.
%
% * 13-02-2010: Added more functions to the #function pragma.
%
% * 14-02-2010: Added "ViewMenuFunctions" to the #function pragma.
%
% * 15-02-2010: Added "ToolsMenuFunctions" to the #function pragma.
%
% * 18-03-2010: The function deletes "logInverseGammaPDF.m" and "logInverseWishartPDF.m" from the "dist"
%               directory if it finds them.
%
% * 11-05-2010: Added three menu-functions to the function pragma.
%
% * 15-06-2010: Added "DSGEVARForecastMenuFunctions" and "DSGEVARCorrToolsMenuFunctions" to the function
%               pragma.
%
% * 01-01-2011: Updated the documentation.
%
% * 09-01-2011: The function now allows YADA to run on unix and mac systems provided at least Matlab 7
%               is used.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 27-01-2011: Fixed unix check and path issue. Thanks to Mathias Trabandt for letting me play a bit
%               with his Mac! The unix/mac porting seems to be working.
%
% * 09-02-2011: The "YADAGUI" function is now called with try-catch code.
%
% * 08-04-2011: The "DefaultUicontrolbackgroundColor" property is set to (1/255)*[237 237 237] if the
%               uicontrol background color property if too dark, i.e., if in RGB terms both the maximum
%               and the average values are below 100/255.
%
% * 11-11-2011: Updated the documentation.
%
% * 02-01-2012: Updated the documentation.
%
% * 31-01-2012: Made sure that aim-files are delected from the current directory.
%
% * 21-08-2012: Took into account that "PredLikeModifiedHarmonic" has changed name to "PredLikeImportanceSampling".
%
% * 02-01-2013: Updated the documentation.
%
% * 04-01-2013: Checks to make sure that "GetDSGEVARyMoments.m", "PrintDSGEPredDensityHeight.m", and 
%               "DSGEPredDensityHeightTheta.m" are deleted if located.
%
% * 05-06-2013: Multithreading is only checked if the computer is running Windows XP.
%
% * 17-07-2013: Took into account that the Waggoner-Zha files have been moved from the "addons\wz" directory
%               to the "tools" directory.
%
% * 16-09-2013: The file "IsWinXP.txt" is only deleted if the entry in the file is unity.
%
% * 24-01-2014: YADA no longer checks how many computational threads are used.
%
% * 29-01-2014: YADA now assumes that the oldest used OS under Windows is XP.
%
% * 05-01-2015: Updated the documentation.
%
% * 14-01-2015: Made sure that "SequentialPredictiveLikelihoodEstimation.m" doesn't exist in the
%               "print" directory.
%
% * 18-02-2015: YADA no longer empties the tmp-directory using *.*, but selectively.
%
% * 19-02-2015: YADA no longer tries to delete "compute_aim_matrices.m" and "compute_aim_data.m"
%               from the "tmp" directory. This is handled by the function "YADADeleteTmpFiles".
%
% * 26-02-2015: Took into account that the function "PredLikeImportanceSampling" has been renamed
%               "PredLikeMCI", and that "DSGEVARPredLikeImportanceSampling" has been renamed
%               "DSGEVARPredLikeMCI".
%
% * 16-03-2015: Fixed a particular issue with the ishandle function, since it returns empty for
%               and empty variable. The code uses the isa function instead.
%
% * 17-03-2015: Switched from the isa function to the selfmade IsHandle which first tests if the
%               input is empty or not.
%
% * 08-06-2015: The code now checks for the existence of the groot function.
%
% * 11-06-2015: Fixed a bug where IsHandle is called before the YADAPath function was called.
%
% * 12-06-2015: Changed the DeleteFcn of YADAfig.
%
% * 20-01-2016: Updated the documentation.
%
% * 19-10-2016: Removed check for YADA.exe.
%
% * 09-02-2017: Updated the documentation.
%
% * 26-05-2017: Took into account that "DSGEPriorSampling.m" has been moved from the "montecarlo"
%               directory to the "prior" directory.
%
% * 29-08-2017: The function now makes sure that "ActionsMenuFunctions.m" id deleted if it appears
%               in the "rndfcn" directory.
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% make sure we have as much memory as possible
%
clear('all');
%
YADAfig = findobj('Type','figure','Tag','YADA');
if isempty(YADAfig)==0;
   if exist('isvalid')==0;
      if ishandle(YADAfig);
         %
         % delete this handle
         %
         set(YADAfig,'DeleteFcn','');
         delete(YADAfig);
         drawnow;
         pause(0.02);
      end;
   else;
      if isvalid(YADAfig);
         %
         % delete this handle
         %
         set(YADAfig,'DeleteFcn','');
         delete(YADAfig);
         drawnow;
         pause(0.02);
      end;
   end;
   clear('all');
end;
%
% check if the computer supports YADA
%
VStr = version;
if ((strcmp(computer,'PCWIN')==0)&(strcmp(computer,'PCWIN64')==0)&(str2num(VStr(1,1))<7))|(str2num(VStr(1,1:3))<5.3);
   disp('YADA only works for all computers using Matlab 7 and later or on Microsoft Windows computers');
   disp('with at least version 5.3 of Matlab.');
   return;
end;
if exist('groot')==0;
   GraphicsRoot = 0;
else;
   GraphicsRoot = groot;
end;
%
scrsz = get(GraphicsRoot,'ScreenSize');
if scrsz(4)+scrsz(2)-1<768;
   disp('YADA is best viewed on a screen with a minimum height of 768 pixels.');
end;
if isunix==1;
   [Status,Txt] = unix('who');
   if Status~=0;
      disp('YADA wasn''t able to run the "unix" command successfully.');
      return;
   end;
else;
   [Status,Txt] = dos('dir');
   if Status~=0;
      disp('YADA wasn''t able to run the "dos" command successfully.');
      return;
   end;
end;
%
% add minimum requirement of functions for the #function pragma in case someone has the brilliant
% idea of compiling YADA with the Matlab compiler (not recommended).
%
%#function CloseGUI ShowStatus StopProgressDLG CancelProgressDLG DisplayComment TextGUICopy TextGUIPrint CloseProgressDLG
%#function ActionsMenuFunctions BVARMenuFunctions DSGEVARMenuFunctions EditMenuFunctions FileMenuFunctions HelpMenuFunctions
%#function ToolsMenuFunctions ViewMenuFunctions CorrToolsMenuFunctions DecompToolsMenuFunctions ForecastToolsMenuFunctions
%#function DSGEVARForecastMenuFunctions DSGEVARCorrToolsMenuFunctions
%
% update the path and make sure the path is stored in the ProgPath string variable.
%
OrigPath = path;
YADAPath;
%
% Set warning level to off
%
if MatlabNumber>=7;
   WarnStr = warning;
else;
   [WarnStr,WarnFreq] = warning;
end;
warning('off');
%
% check if the user has accepted the GNU General Public License
%
AcceptLicense = 0;
if FileExist(FixFilePath([pwd '\AcceptLicense.txt']))==1;
   AcceptTxt = ReadText(FixFilePath([pwd '\AcceptLicense.txt']));
   if min(size(AcceptTxt))==1;
      AcceptTxt = AcceptTxt{1};
      AcceptTxt = strrep(AcceptTxt,' ','');
      AcceptLicense = str2double(AcceptTxt);
      if isnumeric(AcceptLicense)==0;
         AcceptLicense = 0;
      end;
   end;
end;
%
% check if we should change the defaultuicontrolbackgroundcolor
%
DefUIBGColor = 255*get(GraphicsRoot,'defaultuicontrolbackgroundcolor');
if (max(DefUIBGColor)<100)&((sum(DefUIBGColor)/3)<100);
   set(GraphicsRoot,'defaultuicontrolbackgroundcolor',(1/255)*[237 237 237]);
end;
%
% check if the license has been accepted
%
if AcceptLicense==0;
   [VersionOfYADA,VersionDateStr] = YADAVersion;
   TxtStr = strvcat('APPROPRIATE LEGAL NOTICES FOR YADA','This program is free software: you can redistribute it and/or modify it in accordance with the terms of the GNU General Public License (http://www.gnu.org/licenses/) as published by the Free Software Foundation, either Version 3 of the license, or (at your option) any later version.',' ','In order to use the program you must first accept the following disclaimer and limitation of liability.',' ','DISCLAIMER OF WARRANTY','By accepting the disclaimer you assume the entire risk related to your use of this program. The European Central Bank (ECB) provides this program ''as is'' and the ECB disclaims any warranties, whether express or implied, including, but not limited to, the implied warranties of merchantability and fitness for a particular purpose. The entire risk as to the quality and performance of the program is with you.',' ','LIMITATION OF LIABILITY','In no event shall the ECB be liable to you or to any third party for any direct, indirect, incidental, consequential, general, special or exemplary damages or lost profit resulting from any use of or inability to use the program (including but not limited to loss of data or data being rendered inaccurate or losses sustained by you or third parties or a failure of the program to operate with any other programs). Should the program prove defective, you assume the cost of all necessary servicing, repair and/or corrections.',' ','COPYRIGHT NOTICE',['This program is released under the GNU General Public License, Version 3, 29 June 2007 (http://www.gnu.org/licenses/). The current release of the program is version ' num2str(VersionOfYADA,'%0.2f') ' and it was last modified by the ECB on ' VersionDateStr '.'],' ',['Copyright  2006-' datestr(now,10) ' European Central Bank.'],'Email: anders.warne@ecb.europa.eu');
   Answer = Query(TxtStr,'logo',500,'Appropriate Legal Notices for YADA',600,'no',InitializeINIFile,{'I Accept' 'I Don''t Accept' 'View License'},125,FixFilePath([pwd '\gpl.txt']));
   if strcmp(Answer,'I Accept')==1;
      %
      % write to file 
      %
      fid = fopen(FixFilePath([pwd '\AcceptLicense.txt']),'wt');
      fprintf(fid,'1\n');
      fclose(fid);
      AcceptLicense = 1;
   end;
end;
if AcceptLicense==0;
   path(OrigPath);
   warning(WarnStr);
   clear('all');
   return;
end;
%
% delete the old iswinxp.txt file and create a new one. This was different OS's can
% use the same YADA directory, and copying the directory doesn't imply that the wrong
% iswinxp.txt file is examined.
%
if ispc==1;
   file = [pwd '\iswinxp.txt'];
   if FileExist(file)==1;
      stat = dos(['del /Q "' file '"']);
   end;
   %
   if FileExist([pwd '\addons\ovs\GraphEditDataDLG.m'])==1;
      stat = dos(['del /Q "' pwd '\addons\ovs\GraphEditDataDLG.m"']);
   end;
   %
   if FileExist([pwd '\montecarlo\DSGEPosteriorSampling.m'])==1;
      stat = dos(['del /Q "' pwd '\montecarlo\DSGEPosteriorSampling.m"']);
   end;
   %
   if FileExist([pwd '\dist\logInverseGammaPDF.m'])==1;
      stat = dos(['del /Q "' pwd '\dist\logInverseGammaPDF.m"']);
   end;
   %
   if FileExist([pwd '\dist\logInverseWishartPDF.m'])==1;
      stat = dos(['del /Q "' pwd '\dist\logInverseWishartPDF.m"']);
   end;
   %
   if FileExist([pwd '\montecarlo\PredLikeModifiedHarmonic.m'])==1;
      stat = dos(['del /Q "' pwd '\montecarlo\PredLikeModifiedHarmonic.m"']);
   end;
   %
   if FileExist([pwd '\dsge-var\DSGEVARPredLikeModifiedHarmonic.m'])==1;
      stat = dos(['del /Q "' pwd '\dsge-var\DSGEVARPredLikeModifiedHarmonic.m"']);
   end;
   %
   if FileExist([pwd '\dsge-var\GetDSGEVARyMoments.m'])==1;
      stat = dos(['del /Q "' pwd '\dsge-var\GetDSGEVARyMoments.m"']);
   end;
   %
   if FileExist([pwd '\print\PrintDSGEPredDensityHeight.m'])==1;
      stat = dos(['del /Q "' pwd '\print\PrintDSGEPredDensityHeight.m"']);
   end;
   %
   if FileExist([pwd '\tools\DSGEPredDensityHeightTheta.m'])==1;
      stat = dos(['del /Q "' pwd '\tools\DSGEPredDensityHeightTheta.m"']);
   end;
end;
%
% make sure that compute_aim_matrices.m and compute_aim_data.m do not exist
% in pwd
%
if FileExist(FixFilePath([pwd '/compute_aim_matrices.m']))==1;
   delete('compute_aim_matrices.m');
end;
if FileExist(FixFilePath([pwd '/compute_aim_data.m']))==1;
   delete('compute_aim_data.m');
end;
if FileExist(FixFilePath([pwd '\addons\wz\CondPredictionKalmanSmoother.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\CondPredictionKalmanSmoother.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\CondPredictionKalmanSmootherHt.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\CondPredictionKalmanSmootherHt.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\CondPredictionSmoother.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\CondPredictionSmoother.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\CondPredictionSmootherHt.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\CondPredictionSmootherHt.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsMixed.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsMixed.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsThetaMixed.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsThetaMixed.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsThetaWZ.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsThetaWZ.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsWZ.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\DSGECondPredictionPathsWZ.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsMixed.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsMixed.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsThetaMixed.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsThetaMixed.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsThetaWZ.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsThetaWZ.m']));
end;
if FileExist(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsWZ.m']))==1;
   delete(FixFilePath([pwd '\addons\wz\StateCondPredictionPathsWZ.m']));
end;
if FileExist(FixFilePath([pwd '\addons\sc\CondStateControls.m']))==1;
   delete(FixFilePath([pwd '\addons\sc\CondStateControls.m']));
end;
if FileExist(FixFilePath([pwd '\addons\sc\CondStateFunction.m']))==1;
   delete(FixFilePath([pwd '\addons\sc\CondStateFunction.m']));
end;
if FileExist(FixFilePath([pwd '\addons\sc\VerifyStateCondControls.m']))==1;
   delete(FixFilePath([pwd '\addons\sc\VerifyStateCondControls.m']));
end;
if FileExist(FixFilePath([pwd '\addons\sc\StateCondPredictionPaths.m']))==1;
   delete(FixFilePath([pwd '\addons\sc\StateCondPredictionPaths.m']));
end;
if FileExist(FixFilePath([pwd '\addons\sc\StateCondPredictionPathsTheta.m']))==1;
   delete(FixFilePath([pwd '\addons\sc\StateCondPredictionPathsTheta.m']));
end;
if FileExist(FixFilePath([pwd '\print\SequentialPredictiveLikelihoodEstimation.m']))==1;
   delete(FixFilePath([pwd '\print\SequentialPredictiveLikelihoodEstimation.m']));
end;
if FileExist(FixFilePath([pwd '\montecarlo\PredLikeImportanceSampling.m']))==1;
   delete(FixFilePath([pwd '\montecarlo\PredLikeImportanceSampling.m']));
end;
if FileExist(FixFilePath([pwd '\dsge-var\DSGEVARPredLikeImportanceSampling.m']))==1;
   delete(FixFilePath([pwd '\dsge-var\DSGEVARPredLikeImportanceSampling.m']));
end;
if FileExist(FixFilePath([pwd '\montecarlo\DSGEPriorSampling.m']))==1;
   delete(FixFilePath([pwd '\montecarlo\DSGEPriorSampling.m']));
end;
if FileExist(FixFilePath([pwd '\rndfcn\ActionsMenuFunctions.m']))==1;
   delete(FixFilePath([pwd '\rndfcn\ActionsMenuFunctions.m']));
end;
%
if DirExist(FixFilePath([pwd '\addons\wz']))==1;
   rmdir(FixFilePath([pwd '\addons\wz']));
end;
if DirExist(FixFilePath([pwd '\addons\sc']))==1;
   rmdir(FixFilePath([pwd '\addons\sc']));
end;
if FileExist(FixFilePath([pwd '\logic\IsWinXP.m']))==1;
   delete(FixFilePath([pwd '\logic\IsWinXP.m']));
end;
if FileExist(FixFilePath([pwd '\bin\iswinxp.exe']))==1;
   delete(FixFilePath([pwd '\bin\iswinxp.exe']));
end;
if FileExist(FixFilePath([pwd '\bin\shellcmd.exe']))==1;
   delete(FixFilePath([pwd '\bin\shellcmd.exe']));
end;
%
% check if the user has support for multithreading
%
%LastNum = 0;
%if (IsWinXP==1)&(exist('maxNumCompThreads')==2)&(maxNumCompThreads>1);
%   txt = strvcat(['The maximum number of computational threads that your Matlab settings allow is ' int2str(maxNumCompThreads) '. Are you absolutely sure you want to allow this many?'],' ','NOTE: If you answer No, then YADA will set it to 1 (recommended).');
%   answer = Query(txt,'question',300,'YADA - Multithreading Enabled',500,'no',InitializeINIFile);
%   if strcmp(lower(answer),'no')==1;
%      LastNum = maxNumCompThreads(1);
%   end;
%end;
%
% we can now start up the YADA GUI
%
DSGEModel = [];
try;
   YADAHandle = YADAGUI('init');
catch;
   YADAHandle = [];
end;
%
% Close the GUI
%
if (isempty(YADAHandle)==0)&(IsHandle(YADAHandle)==1);
   %
   % check which is the last model loaded.
   %
   controls = get(YADAHandle,'UserData');
   DSGEModel = get(controls.open,'UserData');
   %
   delete(YADAHandle);
else;
   %
   % search for YADA figure
   %
   YADAHandle = findobj('Type','figure','Tag','YADA');
   if isempty(YADAHandle)==0;
      controls = get(YADAHandle,'UserData');
      if (isempty(controls)==0)&(isstruct(controls)==1);
         if isfield(controls,'open')==1;
            DSGEModel = get(controls.open,'UserData');
         end;
         if isfield(controls,'leave')==1;
            set(controls.leave,'UserData','done');
            delete(YADAHandle);
         end;
      end;
   end;
end;
%
% Delete any files that were copied to tmp
%
if isempty(DSGEModel)==0;
   YADADeleteTmpFiles(DSGEModel);
end;
%
% Make sure that the path is reset to the OrigPath string variable that was created before YADA
% was initialized.
%
path(OrigPath);
%
% reset the warning state.
%
warning(WarnStr);
%
% make sure that memory is cleared
%
clear('all');

%
% end of YADA.m
%
