function PlotBVAREigenvalues(EV,HeaderStr,ModelStr,EstStr,CurrINI)
% PlotBVAREigenvalues: Plots the eigenvalues of the VAR system relative to the unit circle.
%
% USAGE:
%
%       PlotBVAREigenvalues(EV,HeaderStr,ModelStr,EstStr,CurrINI)
%
% REQUIRED INPUT: EV (vector) of dimension pk with the eigenvalues of the VAR model for p variables
%                    and k lags.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 25, 2009.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 02-01-2012: Updated the documentation.
%
% * 09-02-2012: Added the input variables "HeaderStr", "ModelStr", and "EstStr".
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% set up the figure
%
FigHandle = figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-600)/2 (CurrINI.scrsz(4)-500)/2 600 500], ...
                   'Tag','EigenvaluesBVAR', ...
                   'Name',[HeaderStr ' - Eigenvalues']);
%
SetFigureProperties(FigHandle,CurrINI);
%
% split the real and the imaginary parts
%
EigenValues = [real(EV) imag(EV)];
%
% Plot unit circle
%
XVals = pi*(0:0.5:2);
YVals = [0 1 0 -1 0 1 0; 1 0 1 0 -1 0 1];
PPVals = spline(XVals,YVals);
YYVals = ppval(PPVals, linspace(0,2*pi,101));
plot(YYVals(1,:),YYVals(2,:),'-b','Color',[0 0 1],'Marker','none','LineWidth',1);
hold('on');
plot(zeros(2,1),[min([min(EigenValues(:,2)) -1])-0.1 max([max(EigenValues(:,2)) 1])+0.1]','LineWidth',1,'Color',[0 0 0]);
hold('on');
plot([min([min(EigenValues(:,1)) -1])-0.1 max([max(EigenValues(:,1)) 1])+0.1]',zeros(2,1),'LineWidth',1,'Color',[0 0 0]);
hold('on');
plot(EigenValues(:,1),EigenValues(:,2),'LineStyle','none','Marker','+','LineWidth',2,'Color',[1 0 0]);
%
% Make axes into a square box
%
set(gca,'Units','pixels');
CurrPosition = get(gca,'Position');
NewLeft = (600-CurrPosition(4))/2;
set(gca,'Position',[NewLeft CurrPosition(2) CurrPosition(4) CurrPosition(4)]);
set(gca,'Units','normalized');
set(gca,'XLim',[min([min(EigenValues(:,1)) -1])-0.1   max([max(EigenValues(:,1)) 1])+0.1]);
set(gca,'YLim',[min([min(EigenValues(:,2)) -1])-0.1   max([max(EigenValues(:,2)) 1])+0.1]);
%
% determine the title
%
SetAxesFonts(CurrINI,'Title',['Eigenvalues of ' ModelStr ' at ' lower(EstStr)],'Xlabel','Real','YLabel','Imaginary');
%
% display the graph
%
SetToolbarImages(FigHandle,CurrINI.images);
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;
pause(0.02);

%
% end of PlotBVAREigenvalues.m
%
