function [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrentChain)
% LoadDSGEPosteriorDraws: Loads the data from posterior estimation.
%
% USAGE:
%
%       [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,...
%                                                                                          CurrINI,CurrentChain)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  CurrentChain (integer) the value of the parallel chain to load data from.
%
% REQUIRED OUTPUT: PostSample (matrix) of dimension (NumIterxNumParam) with all the non-discarded
%                            posterior draws of phi (the transformed parameters)
%
%                  thetaPostSample (matrix) of dimension (NumIterxNumParam) with all the non-discarded
%                            posterior draws of theta (the original parameters)
%
%                  LogPost (vector) of dimension (NumIterx1) with all the log posterior values for the
%                            non-discarded posterior draws. These posterior values are calculated
%                            using the "logPosteriorPhiDSGE" function.
%
%                  AcceptedDraws (vector) of dimension (NumIterx1) with the number of accepted draws.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  Weights (vector) of dimension NumIter x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: December 12, 2006.
%                        This version: August 24, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2007: The function now takes into account that the control "controls.posterior.burnin" displays the number
%               of burn-in draws rather than the percentage relative to the full posterior sample.
%
% * 10-04-2007: The function now taken into account that the variable "InverseHessianEstimator" affects the name of
%               the PostSampleFile.
%
% * 11-04-2007: Changed the AcceptedDraws vector to the number of accepted draws instead of the acceptance ratio.
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 18-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 14-07-2009: Added redraw command to the for-loop.
%
% * 27-11-2009: The function can now load posterior draws from the random walk Metropolis algorithm or from
%               the slice sampler, depending on which is currently selected.
%
% * 28-11-2009: Fixed a bug where the variable "BatchAcceptedDraws" was attempted to be loaded when running
%               the slice sampler.
%
% * 05-01-2010: Updated the documentation.
%
% * 04-03-2010: Added a test if the number of draws per batch is an integer or not.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 25-08-2015: The function should now take the Student-t proposal density based RWM
%               posterior sampler into account.
%
% * 27-08-2015: Fixed a forgotten issue when collecting information about the accepted draws when
%               DSGEModel.PosteriorSampler is equal to 3.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function should now take the fixed blocking RWM with normal proposal density
%               posterior sampler into account.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 19-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 11-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 23-08-2017: Added the output variable "Weights".
%
% * 24-08-2017: Corrected an bug when retrieving "Weights" from the posterior sample file.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of posterior sample batches to save per chain
%
BatchStr = get(controls.posterior.batch,'String');
NumBatches = str2num(StringTrim(strrep(BatchStr(DSGEModel.SampleBatchValue,:),',','')));
if NumBatches>NumPostDraws;
   NumBatches = NumPostDraws;
end;
NumBatchDraws = NumPostDraws/NumBatches;
if IsInteger(NumBatchDraws)==0;
   NumBatchDraws = NumPostDraws;
   NumBatches = 1;
end;
BatchIndex = (1:NumBatchDraws);
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 4. Number of burnin draws per chain
%
if DSGEModel.PosteriorSampler<=7;
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
   if NumBurnin>=NumPostDraws;
      txt = ['The number of burn-in draws (' StringTrim(BurninStr(DSGEModel.BurnInValue,:)) ') leaves no posterior draws (' StringTrim(DrawsStr(DSGEModel.PosteriorDrawsValue,:)) ') for inference. YADA will therefore use 0 burn-in draws.'];
      About(txt,'information','YADA - Burnin Draws Problem',180,500,CurrINI);
      NumBurnin = 0;
   end;
else;
   NumBurnin = 0;
end;
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% Now we turn to batch no 1. since we need to determine how many parameters there are in phi and theta.
%
if DSGEModel.PosteriorSampler==1;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==2;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==3;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==4;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==5;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==6;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==7;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(1,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==8;
	PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumPostDraws,'%0.0f') '.mat']);
end;
PSData = load(PostSampleFile);
NumParam = size(PSData.BatchPostSample,2);
%
% initialize the data matrices
%
PostSample = zeros(NumPostDraws,NumParam);
thetaPostSample = PostSample;
LogPost = zeros(NumPostDraws,1);
AcceptedDraws = LogPost;
Weights = [];
%
% load the first batch of data
%
PostSample(BatchIndex,:) = PSData.BatchPostSample;
thetaPostSample(BatchIndex,:) = PSData.BatchthetaPostSample;
LogPost(BatchIndex,1) = PSData.BatchLogPost;
if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
   AcceptedDraws(BatchIndex,1) = PSData.BatchAcceptedDraws;
elseif DSGEModel.PosteriorSampler==2;
   AcceptedDraws(BatchIndex,1) = PSData.BatchLogPostEvaluations;
end;
if DSGEModel.PosteriorSampler==8;
   Weights = PSData.Weights;
end;
%
% now we loop over all draws
%
for NBatch=2:NumBatches;
   BatchIndex = BatchIndex+NumBatchDraws;
   if DSGEModel.PosteriorSampler==1;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NBatch,'%0.0f') '.' num2str(NumBatches,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==8;
		PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumPostDraws,'%0.0f') '.mat']);
   end;
   PSData = load(PostSampleFile);
   PostSample(BatchIndex,:) = PSData.BatchPostSample;
   thetaPostSample(BatchIndex,:) = PSData.BatchthetaPostSample;
   LogPost(BatchIndex,1) = PSData.BatchLogPost;
   if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
      AcceptedDraws(BatchIndex,1) = PSData.BatchAcceptedDraws;
   elseif DSGEModel.PosteriorSampler==2;
      AcceptedDraws(BatchIndex,1) = PSData.BatchLogPostEvaluations;
   end;
   drawnow;
   pause(0.02);
end;

%
% end of LoadDSGEPosteriorDraws.m
%
