function DoneCalc = DSGEVARCoherence(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,IsPosterior,PME,lambda,DSGEModel,CurrINI,controls)
% DSGEVARCoherence: Computes coherence between observed variables of a the DSGE-VAR model for a sample of
%                   draws from the prior or the posterior distribution.
%
% USAGE:
%
%       DoneCalc = DSGEVARCoherence(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,IsPosterior,PME,lambda,...
%                                   DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the unique draws
%                           of the SigmaEpsilon parameters. Each row holds vech(Sigma)'.
%
%                  PhiMode (matrix) of dimension nxnp with fixed values for the Phi parameters
%                           that can be used as replacements for PhiDraws.
%
%                  SigmaModeEpsilon (matrix) of dimension nxn with fixed values of the SigmaEpsilon
%                           parameters that can be used as replacements for SigmaDraws.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                           main dialog.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 22, 2010.
%                        This version: March 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-06-2010: Updated the documentation.
%
% * 26-10-2010: Added info about the lag order on the wait dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
UsedDraws = size(PhiDraws,1);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'coherence');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\coherence". The computation of coherence distribution for a DSGE-VAR(' num2str(lambda,'%0.4f') ') model has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/coherence". The computation of coherence distribution for a DSGE-VAR(' num2str(lambda,'%0.4f') ') model has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% backup parameters
%
Phistar = PhiMode;
Dn = DuplicationMatrix(n);
Omega = (0:pi/299:pi);
f = length(Omega);
In = eye(n);
%
% compute pxf matrices with exp(-i*k*omega) and exp(i*k*omega), respectively, where k=1,...,p.
%
NegOmega = exp(-i*(1:p)'*Omega);
PosOmega = exp(i*(1:p)'*Omega);
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
%
% check if we should setup a wait handle
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' - Prior';
end;
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(UsedDraws,'%0.0f') ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean computation time:';
   %
   ProgressStructure.name = ['Coherence Distribution' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes coherence for the observed variables via a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Coherence Distribution' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
MeanSolutionTime = 0;
LastDraw = 0;
ThisSave = 0;
Coherence = [];
for ParamIndex=1:UsedDraws;
   LastDraw = LastDraw+1;
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         break;
      else;
         ProgressDLG([ParamIndex/UsedDraws MeanSolutionTime]);
      end;
   end;
   %
   % determine the new parameters
   %
   tic;
   SigmaEpsilon = reshape(Dn*SigmaDraws(ParamIndex,:)',n,n);
   if IsPositiveDefinite(SigmaEpsilon)==1;
      Phi = reshape(PhiDraws(ParamIndex,:)',n,(n*p)+k);
      Phi = Phi(:,k+1:(n*p)+k);
   else;
      SigmaEpsilon = SigmaModeEpsilon;
      Phi = Phistar;
   end;
   %
   PhiT = zeros(n,n*p);
   for j=1:p;
      PhiT(:,1+(n*(j-1)):n*j) = Phi(:,1+(n*(j-1)):n*j)';
   end;
   %
   Coherence(LastDraw).YCoherence = zeros(n*(n-1)/2,f);
   for freq=1:f;
      %
      % compute the population spectrum
      %
      InvPhiPos = (In-(Phi*kron(NegOmega(:,freq),In)))\In;
      InvPhiTNeg = (In-(PhiT*kron(PosOmega(:,freq),In)))\In;
      SyOmega = (1/(2*pi))*InvPhiPos*SigmaEpsilon*InvPhiTNeg;
      %
      % compute the coherence
      %
      num = 0;
      for j=1:n-1;
         for l=j+1:n;
            num = num+1;
            if (real(SyOmega(k,k))>1.0e-010)&(real(SyOmega(l,l))>1.0e-010);
               Coherence(LastDraw).YCoherence(num,freq) = (abs(SyOmega(j,l))^2)/(real(SyOmega(j,j))*real(SyOmega(l,l)));
            end;
            drawnow;
         end;
      end;
   end;
   MeanSolutionTime = (((ParamIndex-1)/ParamIndex)*MeanSolutionTime)+((1/ParamIndex)*toc);
   %
   % check if we should save to disk
   %
   if (LastDraw==SaveAfterDraws)|(ParamIndex==UsedDraws);
      ThisSave = ThisSave+1;
      LastDraw = 0;
      %
      % set up the file name and then save
      %
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
      end;
      save(file,'Coherence','Omega');
      %
      % restore the data structure
      %
      Coherence = [];
   end;
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEVARCoherence.m
%
