function CorrData = DSGEVARCondCorrDistribution(UsedDraws,CurrChain,IsPosterior,Weights,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
% DSGEVARCondCorrDistribution: Computes the mean and the quantiles of the conditional correlations from a DSGE-VAR.
%
% USAGE:
%
%       CorrData = DSGEVARCondCorrDistribution(UsedDraws,CurrChain,IsPosterior,Weights,PME,lambda,NumPaths,...
%                                              DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  UsedDraws (integer) with the number of parameter draws used for the correlations.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                              are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                              all MCMC samplers.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                              are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                              used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                              when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  NumPaths (integer) with the number of simulated paths per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                              model, such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                              main dialog.
%
% REQUIRED OUTPUT: CorrData (structure) with fields Mean and Quantiles and ShockNames. The length of
%                              CorrData is equal to the number of shocks.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 24, 2011.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
CorrData = [];
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
if NumPaths>1;
   Weights = kron(Weights,ones(NumPaths,1));
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
if NumPaths==0;
   NumFiles = ceil(UsedDraws/SaveAfterDraws);
   SampleTypeStr = 'Population';
   TotalPaths = UsedDraws;
else;
   NumFiles = UsedDraws;
   SampleTypeStr = 'Sample';
   TotalPaths = UsedDraws*NumPaths;
end;
p = DSGEModel.BVARLags;
NumQuants = length(DSGEModel.Percentiles);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
n = DSGEModel.n;
Dn = DuplicationMatrix(n);
PosMat = VechPositions(n);
CorrData(1).ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
%
if (NumPaths>0)|(TotalPaths>=1000);
   txt = ['Please wait while YADA computes the distributions for the ' lower(SampleTypeStr) '-based conditional correlations of the observed variables via a DSGE-VAR(lambda=' num2str(lambda,'%0.4f') ',p=' int2str(p) ') model. Current shock: ' StringTrim(CorrData(1).ShockNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Conditional Correlations - ' SampleTypeStr '-Based' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
end;
%
% we should probably shown a wait dialog
%
for j=1:n;
   if (NumPaths>0)|(TotalPaths>=1000);
      txt = ['Please wait while YADA computes the distributions for the ' lower(SampleTypeStr) '-based conditional correlations of the observed variables via a DSGE-VAR(lambda=' num2str(lambda,'%0.4f') ',p=' int2str(p) ') model. Current shock: ' StringTrim(CorrData(1).ShockNames(j,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      pause(0.02);
   end;
   %
   % initialize output
   %
   try;
      CorrData(j).Mean = zeros(n,n);
      TmpCorr = zeros(TotalPaths,(n*(n+1))/2);
      FirstDraw = 1;
      LastDraw = 0;
      for ThisSave=1:NumFiles;
         if NumPaths==0;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
            end;
         else;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
            end;
         end;
         MatData = load(file);
         DrawSize = length(MatData.CC);
         %
         % compute the mean
         %
         for i=1:DrawSize;
            CorrData(j).Mean = CorrData(j).Mean+((Weights(LastDraw+i)/TotalPaths)*MatData.CC(i).Shock(j).CondCorr);
         end;
         %
         % now the quantile data for the correlations
         %
         LastDraw = LastDraw+DrawSize;
         for i=FirstDraw:LastDraw;
            TmpCorr(i,:) = vech(MatData.CC(i-FirstDraw+1).Shock(j).CondCorr)';
         end;
         FirstDraw = FirstDraw+DrawSize;
      end;
      %
      % sort all columns in TmpCorr
      %
      TmpCorr = sort(TmpCorr);
      %
      % now we can determine the quantiles
      %
      if DSGEModel.ConfidenceRegionMethod==1;
         for i=1:NumQuants;
            CorrData(j).Quantiles(i).percent = DSGEModel.Percentiles(i);
            if DSGEModel.Percentiles(i)<50;
               QuantVal = ceil((DSGEModel.Percentiles(i)/100)*TotalPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(i)/100)*TotalPaths);
            end;
            CorrData(j).Quantiles(i).Mean = reshape(Dn*TmpCorr(QuantVal,:)',n,n);
         end;
      else;
         for i=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-i+1)-DSGEModel.Percentiles(i))*TotalPaths/100);
            HMat = zeros(n*(n+1)/2,2);
            for t=1:n*(n+1)/2;
               HMat(t,:) = HighestProbabilityDensityBand(TmpCorr(:,t),ConfDraws);
            end;
            CorrData(j).Quantiles(i).Mean = reshape(Dn*HMat(:,1),n,n);
            CorrData(j).Quantiles(NumQuants-i+1).Mean = reshape(Dn*HMat(:,2),n,n);
            CorrData(j).Quantiles(i).percent = DSGEModel.Percentiles(i);
            CorrData(j).Quantiles(NumQuants-i+1).percent = DSGEModel.Percentiles(NumQuants-i+1);
         end;
      end;
   catch
      for t=1:n*(n+1)/2;
         if t==1;
            CorrData(j).Mean = zeros(n,n);
         end;
         TmpCorr = zeros(TotalPaths,1);
         FirstDraw = 1;
         LastDraw = 0;
         for ThisSave=1:NumFiles;
            if NumPaths==0;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
               end;
            else;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
               end;
            end;
            MatData = load(file);
            DrawSize = length(MatData.CC);
            %
            % compute the mean
            %
            if t==1;
               for i=1:DrawSize;
                  CorrData(j).Mean = CorrData(j).Mean+((Weights(LastDraw+i)/TotalPaths)*MatData.CC(i).Shock(j).CondCorr);
               end;
            end;
            %
            % now the quantile data for the correlations
            %
            LastDraw = LastDraw+DrawSize;
            for i=FirstDraw:LastDraw;
               MT = vech(MatData.CC(i-FirstDraw+1).Shock(j).CondCorr)';
               TmpCorr(i,1) = MT(t);
            end;
            FirstDraw = FirstDraw+DrawSize;
         end;
         %
         % sort all columns in TmpCorr
         %
         TmpCorr = sort(TmpCorr);
         %
         % now we can determine the quantiles
         %
         if DSGEModel.ConfidenceRegionMethod==1;
            for i=1:NumQuants;
               CorrData(j).Quantiles(i).percent = DSGEModel.Percentiles(i);
               if DSGEModel.Percentiles(i)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(i)/100)*TotalPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(i)/100)*TotalPaths);
               end;
               CorrData(j).Quantiles(i).Mean(PosMat(t,1),PosMat(t,2)) = TmpCorr(QuantVal,1);
            end;
         else;
            for i=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-i+1)-DSGEModel.Percentiles(i))*TotalPaths/100);
               Bounds = HighestProbabilityDensityBand(TmpCorr,ConfDraws);
               CorrData(j).Quantiles(i).Mean(PosMat(t,1),PosMat(t,2)) = Bounds(1);
               CorrData(j).Quantiles(NumQuants-i+1).Mean(PosMat(t,1),PosMat(t,2)) = Bounds(2);
               CorrData(j).Quantiles(i).percent = DSGEModel.Percentiles(i);
               CorrData(j).Quantiles(NumQuants-i+1).percent = DSGEModel.Percentiles(NumQuants-i+1);
            end;
         end;
      end;
   end;
end;
%
% close the wait dialog
%
if (NumPaths>0)|(TotalPaths>=1000);
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;

%
% end of DSGEVARCondCorrDistribution.m
%
