function DoneCalc = DSGEVARConditionalCorrsSimulation(PhiDraws,A0Draws,CurrChain,IsPosterior,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
% DSGEVARConditionalCorrsSimulation: Computes sample-based conditional correlations from a DSGE-VAR model using
%                                    draws from the prior or the posterior distribution.
%
% USAGE:
%
%       DoneCalc = DSGEVARConditionalCorrsSimulation(PhiDraws,A0Draws,CurrChain,IsPosterior,PME,lambda,NumPaths, ...
%                                                    DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  A0Draws (matrix) of dimension UsedDraws x n^2 with the draws of the A0
%                           parameters. Each row holds vec(A0)'.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  NumPaths (integer) with the number of paths to simulate.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                           main dialog.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 21, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-01-2011: Finished the function.
%
% * 01-03-2011: Changed the calculation of the square-root matrix of "SigmaZ".
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the call to "DoublingAlgorithmLyapunov".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
UsedDraws = size(PhiDraws,1);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'condcorr');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\condcorr". The computation of conditional correlation distribution has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/condcorr". The computation of conditional correlation distribution has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
%
% matrix needed for stacking the VAR model
%
J = eye(n);
if p>1;
   J = [J;zeros(n*(p-1),n)];
end;
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% begin the loop
%
abort = '';
StopSimulation = 0;
MeanEstimationTime = 0;
CC = [];
NumFiles = UsedDraws;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(UsedDraws*NumPaths) ' Sample Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Sample-Based Conditional Correlation Dist' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the sample-based conditional correlation distribution for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Conditional Correlations Distribution' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
for ParamIndex=1:UsedDraws;
   %
   % determine the new parameters
   %
   Phi = reshape(PhiDraws(ParamIndex,:)',n,(n*p)+k);
   A0 = reshape(A0Draws(ParamIndex,:)',n,n);
   if k>0;
      Phi0 = Phi(:,1:k);
      Phi = Phi(:,k+1:(n*p)+k);
   else;
      Phi0 = [];
   end;
   if p>1;
      Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   end;
   %
   % turn to the paths
   %
   for i=1:NumPaths;
      CurrPath = i+(NumPaths*(ParamIndex-1));
      %
      % update the progress bar
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([CurrPath/(UsedDraws*NumPaths) CurrPath]);
         end;
      end;
      %
      % turn to the individual shocks
      %
      for j=1:n;
         %
         % initial values for Y
         %
         A0j = A0(:,j);
         if i==1;
            try;
               [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*A0j*A0j'*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
            catch;
               vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(A0j*A0j');
               SigmaZ = reshape(vSigmaZ,n*p,n*p);
            end;
            [SE,LE] = schur(SigmaZ);
            CholSigmaZ = SE*sqrt(abs(LE));
            if k==0;
               init(j).muY = CholSigmaZ*randn(n*p,NumPaths);
            elseif (k==1)&(max(X)==min(X));
               init(j).muY = (kron(ones(p,1),J'*((eye(n*p)-Phi)\J)*Phi0)*ones(1,NumPaths))+(CholSigmaZ*randn(n*p,NumPaths));
            else;
               init(j).muY = Y(:,1)*ones(1,NumPaths);
            end;
         end;
         %
         % simulate the data
         %
         Epsilon = A0j*randn(1,T);
         ymat = zeros(n,T);
         Y = init(j).muY(:,i);
         if k>0;
            for t=1:T;
               Y = (J*Phi0*x(:,t))+(Phi*Y)+(J*Epsilon(:,t));
               ymat(:,t) = J'*Y;
            end;
         else;
            for t=1:T;
               Y = (Phi*Y)+(J*Epsilon(:,t));
               ymat(:,t) = J'*Y;
            end;
         end;
         if k>0;
            MeanY = (ymat*x')/(x*x');
            ymat = ymat-(MeanY*x);
         else;
            MeanY = (1/T)*sum(ymat,2);
            ymat = ymat-(MeanY*ones(1,T));
         end;
         SigmaYj = (1/T)*ymat*ymat';
         CorrVec = sqrt(abs(diag(SigmaYj)));
         CorrVec(CorrVec<1.0e-12) = 1;
         CC(i).Shock(j).CondCorr = SigmaYj./(CorrVec*CorrVec');
         drawnow;
      end;
   end;
   %
   if StopSimulation==1;
      break;
   end;
   %
   % save to disk
   %
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ParamIndex) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\condcorr\DSGEVAR-CondCorr-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ParamIndex) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
   end;
   save(file,'CC');
   %
   % restore the data structure
   %
   CC = [];
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEVARConditionalCorrsSimulation.m
%
