function SVEs = DSGEVARCorrelationSimulationParam(Phi,SigmaEpsilon,NumPaths,lambda,EstStr,DSGEModel,CurrINI)
% DSGEVARCorrelationSimulationParam: Computes sample correlations based on a DSGE-VAR with
%                                    fixed parameter values.
%
% USAGE:
%
%       SVEs = DSGEVARCorrelationSimulationParam(Phi,SigmaEpsilon,NumPaths,lambda,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with parameters on exogenous variables and
%                      on lagged endogenous variables.
%
%                  SigmaEpsilon (matrix) of dimension nxn with residual covariances.
%
%                  NumPaths (integer) with the number of paths to simulate.
%
%                  lambda  (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                  EstStr (string) that takes the values "Initial Values", "Marginal Posterior mode", or
%                      "Joint Posterior Mode".
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                      input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SVEs (structure) with sample means and quantiles of the mean, covariance, autocorrelations,
%                      and skewness.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 16, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 25-11-2011: The function now also computes skewness.
%
% * 02-12-2011: The function also calculates kurtosis.
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the call to "DoublingAlgorithmLyapunov".
%
% * 27-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SVEs = [];
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% remove parameters on the exogenous variables
%
if k>0;
   Phi0 = Phi(:,1:k);
else;
   Phi0 = zeros(n,1);
end;
Phi = Phi(:,k+1:(n*p)+k);
PhiOld = Phi;
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
%
% compute population covariances and autocorrelations
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
%
AutoCovHorizon = min([SampleFreq*3 T-20 18]);
SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
%
% compute the data measures
%
if k>0;
   SVEs.Data.Mean = (y*x')/(x*x');
   y = y-(SVEs.Data.Mean*x);
else;
   SVEs.Data.Mean = (1/T)*sum(y,2);
   y = y-(SVEs.Data.Mean*ones(1,T));
end;
AutoCov = (1/T)*y*y';
StdVec = sqrt(diag(AutoCov));
SVEs.Data.Cov = AutoCov;
SVEs.Data.Std = StdVec;
SVEs.Data.Skew = ((1/T)*sum(y.*y.*y,2))./(StdVec.*StdVec.*StdVec);
SVEs.Data.Kurt = ((1/T)*sum(y.*y.*y.*y,2))./(StdVec.*StdVec.*StdVec.*StdVec);
SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(StdVec*StdVec'));
for c=1:AutoCovHorizon;
   ys = y(:,c+1:T);
   ylag = y(:,1:T-c);
   AutoCov = (1/T)*ys*ylag';
   CorrMat = AutoCov./(StdVec*StdVec');
   SVEs.Data.Corr(:,AutoCovHorizon+1+c) = vech(CorrMat');
   SVEs.Data.Corr(:,AutoCovHorizon+1-c) = vech(CorrMat);
end;
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% initial values for Y
%
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*SigmaEpsilon*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch;
   vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(SigmaEpsilon);
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
CholSigmaZ = chol(SigmaZ)';
if k==0;
   muY = CholSigmaZ*randn(n*p,NumPaths);
elseif (k==1)&(max(x)==min(x));
   muY = (kron(ones(p,1),J'*((eye(n*p)-Phi)\J)*Phi0)*ones(1,NumPaths))+(CholSigmaZ*randn(n*p,NumPaths));
else;
   muY = Y(:,1)*ones(1,NumPaths);
end;
%
% begin the loop
%
abort = '';
MeanEstimationTime = 0;
StopSimulation = 0;
SVE = [];
CholSigma = chol(SigmaEpsilon)';
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPaths,'%0.0f') ' simulation paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Autocorrelation Distribution - ' EstStr];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA simulates ' int2str(NumPaths) ' samples with data from a DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags to compute sample means, autocorrelations, and skewness of the observed variables from the simulated data.'];
   WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution - ' EstType],600,400,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
for j=1:NumPaths;
   tic;
   %
   % update the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         StopSimulation = 1;
         break;
      else;
         ProgressDLG([j/NumPaths MeanEstimationTime]);
      end;
   end;
   %
   % simulate the data
   %
   Epsilon = CholSigma*randn(n,T);
   ymat = zeros(n,T);
   Y = muY(:,j);
   if k>0;
      for t=1:T;
         Y = (J*Phi0*x(:,t))+(Phi*Y)+(J*Epsilon(:,t));
         ymat(:,t) = J'*Y;
      end;
   else;
      for t=1:T;
         Y = (Phi*Y)+(J*Epsilon(:,t));
         ymat(:,t) = J'*Y;
      end;
   end;
   if k>0;
      MeanY = (ymat*x')/(x*x');
      ymat = ymat-(MeanY*x);
   else;
      MeanY = (1/T)*sum(ymat,2);
      ymat = ymat-(MeanY*ones(1,T));
   end;
   SVE.CorrData(j).MeanY = MeanY;
   AutoCov = (1/T)*ymat*ymat';
   SVE.CorrData(j).StdY = sqrt(diag(AutoCov));
   SVE.CorrData(j).SkewY = ((1/T)*sum(ymat.*ymat.*ymat,2))./(SVE.CorrData(j).StdY.*SVE.CorrData(j).StdY.*SVE.CorrData(j).StdY);
   SVE.CorrData(j).KurtY = ((1/T)*sum(ymat.*ymat.*ymat.*ymat,2))./(SVE.CorrData(j).StdY.*SVE.CorrData(j).StdY.*SVE.CorrData(j).StdY.*SVE.CorrData(j).StdY);
   SVE.CorrData(j).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVE.CorrData(j).CorrY(:,AutoCovHorizon+1) = vech(AutoCov./(SVE.CorrData(j).StdY*SVE.CorrData(j).StdY'));
   for c=1:AutoCovHorizon;
      ys = ymat(:,c+1:T);
      ylag = ymat(:,1:T-c);
      AutoCov = (1/T)*ys*ylag';
      CorrMat = AutoCov./(SVE.CorrData(j).StdY*SVE.CorrData(j).StdY');
      SVE.CorrData(j).CorrY(:,AutoCovHorizon+1+c) = vech(CorrMat');
      SVE.CorrData(j).CorrY(:,AutoCovHorizon+1-c) = vech(CorrMat);
   end;
   MeanEstimationTime = (((j-1)/j)*MeanEstimationTime)+((1/j)*toc);
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
if StopSimulation==1;
   return;
end;
%
% compute the empirical distribution
%
NumQuants = length(DSGEModel.Percentiles);
%
% initialize model based mean, standard deviation, and autocorrelations
%
if k>0;
   SVEs.Mean = zeros(n,k);
else;
   SVEs.Mean = zeros(n,1);
end;
SVEs.Std = zeros(n,1);
SVEs.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
SVEs.Skew = zeros(n,1);
SVEs.Kurt = zeros(n,1);
for i=1:NumQuants;
   if k>0;
      SVEs.Quantiles(i).Mean = zeros(n,k);
   else;
      SVEs.Quantiles(i).Mean = zeros(n,1);
   end;
   SVEs.Quantiles(i).Std = zeros(n,1);
   SVEs.Quantiles(i).Skew = zeros(n,1);
   SVEs.Quantiles(i).Kurt = zeros(n,1);
   SVEs.Quantiles(i).Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
% setup a wait dialog
%
txt = ['Please wait while YADA computes the distributions for the DSGE-VAR(' num2str(lambda,'%0.4f') ') model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution - ' EstStr],600,400,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
% deal with mean and standard deviation
%
for i=1:n;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the DSGE-VAR(' num2str(lambda,'%0.4f') ') model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % set up a matrix for the data
   %
   if k>0;
      TempMean = zeros(NumPaths,k);
   else;
      TempMean = zeros(NumPaths,1);
   end;
   TempStd = zeros(NumPaths,1);
   TempSkew = zeros(NumPaths,1);
   TempKurt = zeros(NumPaths,1);
   for j=1:NumPaths;
      TempMean(j,:) = SVE.CorrData(j).MeanY(i,:);
      TempStd(j,1) = SVE.CorrData(j).StdY(i);
      TempSkew(j,1) = SVE.CorrData(j).SkewY(i);
      TempKurt(j,1) = SVE.CorrData(j).KurtY(i);
   end;
   drawnow;
   %
   % we can now calculate the mean
   %
   SVEs.Mean(i,:) = (1/NumPaths)*sum(TempMean,1);
   SVEs.Std(i,1) = (1/NumPaths)*sum(TempStd,1);
   SVEs.Skew(i,1) = (1/NumPaths)*sum(TempSkew,1);
   SVEs.Kurt(i,1) = (1/NumPaths)*sum(TempKurt,1);
   %
   % now we can sort these and create the quantiles from the sorted
   %
   TempMean = sort(TempMean);
   TempStd = sort(TempStd);
   TempSkew = sort(TempSkew);
   TempKurt = sort(TempKurt);
   if DSGEModel.ConfidenceRegionMethod==1;
      for l=1:NumQuants;
         if DSGEModel.Percentiles(l)<50;
            QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
         else;
            QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
         end;
         SVEs.Quantiles(l).Mean(i,:) = TempMean(QuantVal,:);
         SVEs.Quantiles(l).Std(i,1) = TempStd(QuantVal,1);
         SVEs.Quantiles(l).Skew(i,1) = TempSkew(QuantVal,1);
         SVEs.Quantiles(l).Kurt(i,1) = TempKurt(QuantVal,1);
      end;
   else;
      for l=1:floor(NumQuants/2);
         ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
         BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
         SVEs.Quantiles(l).Std(i) = BoundsStd(1);
         SVEs.Quantiles(NumQuants-l+1).Std(i) = BoundsStd(2);
         BoundsSkew = HighestProbabilityDensityBand(TempSkew,ConfDraws);
         SVEs.Quantiles(l).Skew(i) = BoundsSkew(1);
         SVEs.Quantiles(NumQuants-l+1).Skew(i) = BoundsSkew(2);
         BoundsKurt = HighestProbabilityDensityBand(TempKurt,ConfDraws);
         SVEs.Quantiles(l).Kurt(i) = BoundsKurt(1);
         SVEs.Quantiles(NumQuants-l+1).Kurt(i) = BoundsKurt(2);
         for k1=1:size(TempMean,2);
            BoundsMean = HighestProbabilityDensityBand(TempMean(:,k1),ConfDraws);
            SVEs.Quantiles(l).Mean(i,k1) = BoundsMean(1);
            SVEs.Quantiles(NumQuants-l+1).Mean(i,k1) = BoundsMean(2);
         end;
      end;
   end;
   drawnow;
end;
%
% now we need to deal with the autocorrelations
%
SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
for j=1:(2*AutoCovHorizon)+1;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the DSGE-VAR(' num2str(lambda,'%0.4f') ') model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current autocorrelation horizon: ' int2str(j-(AutoCovHorizon+1)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   TempCorr = zeros(NumPaths,(n*(n+1))/2);
   for i=1:NumPaths;
      TempCorr(i,:) = SVE.CorrData(i).CorrY(:,j)';
   end;
   drawnow;
   %
   % we can now calculate the mean
   %
   SVEs.Corr(:,j) = (1/NumPaths)*sum(TempCorr,1)';
   %
   % sort the data and determine the quantile values
   %
   TempCorr = sort(TempCorr);
   if DSGEModel.ConfidenceRegionMethod==1;
      for l=1:NumQuants;
         if DSGEModel.Percentiles(l)<50;
            QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
         else;
            QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
         end;
         SVEs.Quantiles(l).Corr(:,j) = TempCorr(QuantVal,:)';
      end;
   else;
      for l=1:floor(NumQuants/2);
         ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
         for t=1:(n*(n+1))/2;
            Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
            SVEs.Quantiles(l).Corr(t,j) = Bounds(1);
            SVEs.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
         end;
      end;
   end;
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARCorrelationSimulationParam.m
%
