function DSGEVARObsVarDecompDLG(selector,y,yMean,yInitial,yShocks,DateVector,ShockNames,DSGEVARShocks,EstStr,DecType,lambda,SampleStrMat,CurrINI,DSGEModel)
% DSGEVARObsVarDecompDLG: Displays historical observed variable decompositions based on a DSGE-VAR model.
%
% USAGE:
%
%       DSGEVARObsVarDecompDLG(selector,y,yMean,yInitial,yShocks,DateVector,ShockNames,EstStr,DecType,lambda,...
%                              SampleStrMat,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string) that takes the values 'init', 'samplebegin', 'sampleend',
%                          'showgraph', and 'done'.
%
% OPTIONAL INPUT: y (matrix) with the data on the observed variables.
%
%                 yMean (matrix) of dimension nxT with the term due to the exogenous variables in the
%                          decomposition of the endogenous variables.
%
%                 yInitial (matrix) of dimension nxT with the term due to the initial values in the
%                          decomposition of the endogenous variables.
%
%                 yShocks (matrix) of dimension nxTxq with the terms due to each structural shock or group in
%                          the decomposition of the endogenous variables. The 3rd dimension is the shock (group)
%                          dimension.
%
%                 DateVector (vector) with numerical dates for the sample.
%
%                 ShockNames (string matrix) with the names of the structural shocks or their groups.
%
%                 DSGEVARShocks (vector) which indicates which of the state shocks from the DSGE model that
%                          also appear in the DSGE-VAR model.
%
%                 EstStr (string) that takes the values 'Initial Values', 'Marginal Posterior Mode',
%                          or 'Joint Posterior Mode'.
%
%                 DecType (integer) that takes the value 1 if the decomposition concerns the original
%                          data and 2 if the annualized data.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                 SampleStrMat (string matrix) with the sample dates.
%
%                 CurrINI (structure) with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information about the
%                          model, such as input files, variable and sample data. Actual observations
%                          are not stored in this file.
%
% NOTE: All optional variables are required when "selector" is equal to 'init'.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 25, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-05-2010: Finished the function.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-04-2011: Added support for the "DSGEModel.ShockColors" variable.
%
% * 06-04-2011: "ShockAliases" is now used when the number of shock groups is equal to the
%               number of shocks.
%
% * 19-04-2011: Made sure that ShockGroups and colors are proerly used.
%
% * 03-01-2012: Updated the documentation.
%
% * 06-01-2012: Made sure that ShockGroupColors is only used when it has at least as many rows as
%               there are shock groups.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   CurrINI.y = y;
   CurrINI.yMean = yMean;
   CurrINI.yInitial = yInitial;
   CurrINI.yShocks = yShocks;
   CurrINI.DateVector = DateVector;
   CurrINI.ShockNames = ShockNames;
   CurrINI.EstStr = EstStr;
   CurrINI.DecType = DecType;
   CurrINI.lambda = lambda;
   CurrINI.DSGEVARShocks = DSGEVARShocks;
   if size(ShockNames,1)<DSGEModel.n;
      ShockNames = char(zeros(size(ShockNames)));
      for j=1:size(ShockNames,1);
         ShockNames(j,:) = strrep(CurrINI.ShockNames(j,:),'_',' ');
      end;
      CurrINI.ShockNames = ShockNames;
   end;
else;
   ObsVarDecompGUI = findobj('Type','figure','Tag','DSGEVARObsVarDecompDLG');
   controls = get(ObsVarDecompGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
switch selector;
   case 'init';
      ObsVarDecompGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','DSGEVARObsVarDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-600)/2 (CurrINI.scrsz(4)-135)/2 600 135], ...
         'Tag','DSGEVARObsVarDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Historical Observed Variable Decompositions - ' EstStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','DSGEVARObsVarDecompDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(ObsVarDecompGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 598 125],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 80 125 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARObsVarDecompDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[340 80 125 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARObsVarDecompDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 255 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      controls.blackedgebars = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 20 186 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','checkbox', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Draw bars with black edges', ...
         'Value',1, ...
         'Enable','on', ...
         'Tag','BlackEdgeBarsCheckBox');
      %
      controls.skipinitialvalue = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[210 20 220 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','checkbox', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Skip initial value share', ...
         'Value',0, ...
         'Enable','on', ...
         'Tag','SkipInitialValueShareCheckBox');
      % 
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARObsVarDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARObsVarDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(ObsVarDecompGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      if VariableValue<size(DSGEModel.VariableNames,1);
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      % check if the bars should have black edges or not
      %
      BlackEdgeBars = get(controls.blackedgebars,'Value');
      %
      DateVector = CurrINI.DateVector;
      ShockNames = CurrINI.ShockNames;
      %
      SampleBeg = get(controls.samplebeg,'Value');
      SampleEnd = get(controls.sampleend,'Value');
      %
      if CurrINI.DecType==1;
         DataType = 'Original';
      else;
         DataType = 'Annual';
      end;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                         'Tag',CurrVariable, ...
                         'Name',[DataType ' Historical Observed Variable Decomposition - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % setup the decomposition data
      %
      Ymat = zeros(size(CurrINI.yShocks,3),size(CurrINI.yShocks,2));
      for i=1:size(CurrINI.yShocks,3);
         Ymat(i,:) = CurrINI.yShocks(VariableValue,:,i);
      end;
      %
      if size(CurrINI.ShockNames,1)<DSGEModel.n;
         ShockGroups = DSGEModel.ShockGroups(CurrINI.DSGEVARShocks);
         Ymat = Ymat(unique(ShockGroups),:);
         if (isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)>=length(unique(ShockGroups)));
            ShockGroupColors = DSGEModel.ShockGroupColors(unique(ShockGroups),:);
         else;
            ShockGroupColors = CreateDefaultColors(size(Ymat,1));
         end;
      else;
         if isempty(DSGEModel.ShockColors)==0;
            ShockGroupColors = DSGEModel.ShockColors(DSGEModel.DSGEVARShocks,:);
         else;
            ShockGroups = DSGEModel.ShockGroups(CurrINI.DSGEVARShocks);
            if max(ShockGroups)==DSGEModel.n;
               ShockGroupColors = CreateDefaultColors(DSGEModel.n);
            else;
               ShockGroupColors = CreateShockColors(ShockGroups,DSGEModel.ShockGroupColors);
            end;
         end;
         if isequal(ShockNames,DSGEModel.StateShockNames(CurrINI.DSGEVARShocks,:))==1;
            ShockNames = DSGEModel.ShockAliases;
         end;
      end;
      %
      % check if the initial value term should be included
      %
      if get(controls.skipinitialvalue,'Value')==0;
         Ymat = [CurrINI.yInitial(VariableValue,:);Ymat];
         ShockGroupColors = [0.3 0.3 0.3;ShockGroupColors];
         ShockNames = strvcat('Initial value',ShockNames);
      end;
      %
      % Create the bars and collect the handles
      %
      if MatlabNumber>8.3;
         BarHandles = bar(DateVector',Ymat','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',DateVector',Ymat','stacked');
         else;
            BarHandles = bar(DateVector',Ymat','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,ShockGroupColors,BlackEdgeBars);
      %
      % plot the observed variable minus its "mean"
      %
      LineHandle = plot(DateVector,CurrINI.y(VariableValue,:)-CurrINI.yMean(VariableValue,:),'.-','Color',[0 0 0],'LineWidth',2);
      %
      if CurrINI.DecType==1;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) in deviation from mean at ' lower(CurrINI.EstStr) ' for lambda=' num2str(CurrINI.lambda,'%0.4f')]);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) in deviation from mean at ' lower(CurrINI.EstStr) ' for lambda=' num2str(CurrINI.lambda,'%0.4f')]);
      end;
      axis('tight');
      ylim = get(gca,'YLim');
      for h=1:length(BarHandles);
         if h==1;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min(min(YData(:,SampleBeg:SampleEnd)));
            ylim(2) = max(max(YData(:,SampleBeg:SampleEnd)));
         else;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min([min(YData(:,SampleBeg:SampleEnd)) ylim(1)]);
            ylim(2) = max([max(YData(:,SampleBeg:SampleEnd)) ylim(2)]);
         end;
      end;
      %
      % see if we should add a zero line
      %
      AddOn = 0.1*(ylim(2)-ylim(1));
      if AddOn==0;
         Ylim = get(gca,'YLim');
         AddOn = 0.1*(Ylim(2)-Ylim(1));
      end;
      ylim = [ylim(1)-AddOn ylim(2)+AddOn];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      %
      % make sure that only the selected sample is displayed
      %
      XData = get(BarHandles(1),'XData');
      if size(XData,1)>size(XData,2);
         XData = XData';
      end;
      set(gca,'XLim',[min(XData(:,SampleBeg)) max(XData(:,SampleEnd))]);
      %
      % set the legend
      %
      legHandle = legend([BarHandles LineHandle],strvcat(ShockNames,CurrVariable),2);
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      % check if we should fix the X label
      %
      SetXTickIntegers(gca);
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(ObsVarDecompGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(ObsVarDecompGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of DSGEVARObsVarDecompDLG.m
%
