function [Phi,SigmaEpsilon,Gammayy,GammayY,GammaYY,Status] = DSGEVARParameters(A,H,R,F,B0,lambda,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,initP,MaxIter,Tolerance,HSample)
% DSGEVARParameters: Computes the posterior VAR parameter matrices for the DSGE-VAR model.
%
% USAGE:
%
%       [Phi,SigmaEpsilon,Gammayy,GammayY,GammaYY,Status] = DSGEVARParameters(A,H,R,F,B0,lambda,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,initP,MaxIter,Tolerance,HSample)
%
% REQUIRED INPUT:  A (kxn) matrix with coefficients on the exogenous variables in the measurement equation.
%
%                  H (rxnxT) matrix with cofficients on the state variables in the measurement equation.
%
%                  R (nxn) matrix with the covariances of the measurement errors.
%
%                  F (rxr) transition matrix for the state variables.
%
%                  B0 (rxq) matrix with coefficients on the economic shocks in the state equation.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                             variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                             variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                             regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  initP (integer) which determines how the unconditional state variable covariance matrix
%                        is calculated. If unity, then the analytical vec-procedure is used, else the 
%                        doubling algorithm is applied.
%
%                  MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                        divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                        are located.
%
% REQUIRED OUTPUT: Phi (matrix) of dimension nx(np+k) with the posterior estimate of the parameters on
%                        deterministic and lagged endogenous variables.
%
%                  SigmaEpsilon (matrix) of dimension nxn with the posterior estimate of the residual
%                        covariance matrix.
%
%                  Gammayy (nxn) matrix with the sample average of the non-central population
%                        moments for y(t) and y(t).
%
%                  GammayY (nx(np+k)) matrix with the sample average of the non-central population
%                        moments for y(t) and Y(t), where Y(t) = [x(t)' y(t-1)' ... y(t-p)']'.
%
%                  GammaYY ((np+k)x(np+k)) matrix with the sample average of the non-central population
%                        moments for Y(t) and Y(t).
%
% OPTIONAL OUTPUT: Status (boolean) which takes on the value 1 if the computations could be performed
%                        and zero otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 25, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-01-2010: Updated the documentation.
%
% * 02-02-2010: Changed the output variable "GammaBaryy" to the output variables "Gammayy",
%               "GammayY", and "GammaYY" from the prior distribution.
%
% * 10-02-2010: Made sure that "Status" is the 6th output variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
Phi = [];
SigmaEpsilon = [];
Gammayy = [];
GammayY = [];
GammaYY = [];
if nargout==6;
   Status = 0;
end;
%
% compute the prior moments from the DSGE model parameters
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,initP,MaxIter,Tolerance,HSample);
%
% check if we should report invertibility problems
%
if PriorStatus==0;
   return;
end;
%
% check if lambda is finite or not
%
if isinf(lambda)==0;
   tau = lambda/(1+lambda);
   GammaBaryy = (tau*Gammayy)+((1-tau)*GammaHatyy);
   GammaBaryY = (tau*GammayY)+((1-tau)*GammaHatyY);
   GammaBarYY = (tau*GammaYY)+((1-tau)*GammaHatYY);
   %
   % check if GammaBarYY is invertible
   %
   if rank(GammaBarYY)<size(GammaBarYY,1);
      return;
   end;
   %
   % compute the output
   %
   Phi = GammaBaryY/GammaBarYY;
   SigmaEpsilon = GammaBaryy-((GammaBaryY/GammaBarYY)*GammaBaryY');
else;
   Phi = GammayY/GammaYY;
   SigmaEpsilon = Gammayy-((GammayY/GammaYY)*GammayY');
end;
%
if nargout==6;
   Status = 1;
end;

%
% end of DSGEVARParameters.m
%
