function VecParmHat = DSGEVARPhiHatSigmaHat(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,p,DetProductMoments,HSample,ParamType,OrderQZ)
% DSGEVARPhiHatSigmaHat: Computes the column vectorization of the DSGE-VAR prior matrix GammayY, and the vech-type of column
%                        vectorization of the posterior DSGE-VAR matrices Gammayy and GammaYY.
%
% USAGE:
%
%       VecParmHat = DSGEVARPhiHatSigmaHat(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist, ...
%                              ModelParameters,DSGEModel,AIMData,lambda,p,DetProductMoments,HSample,ParamType,OrderQZ)
%
% REQUIRED INPUT:  phi (vector) with the transformed parameters of the DSGE model.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                             are located.
%
%                  ParamType (boolean) that takes the value 1 if vech(Gammayy) should be returned, 2 if
%                             vec(GammayY) should be returned, and 3 if vech(GammaYY) should be returned.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
% REQUIRED OUTPUT: VecParmHat (vector) whose dimension and content depends on "ParamType".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 2, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-02-2010: The function now computed vectorizations of Gammayy, GammayY, and GammaYY
%               instead of PhiHat and SigmaHatEpsilon.
%
% * 17-12-2010: Added the input variable "OrderQZ".
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 26-04-2012: Fixed the sum(isnan(theta)) test.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
VecParmHat = [];
%
% transform to theta
%
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
if (sum(isinf(theta))>=1)|(sum(isnan(theta))>=1);
   logPost = 1000000;
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% See if we can solve the model
%
[A,H,R,F,B0] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,OrderQZ);
%
% and now the prior
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if PriorStatus==0;
   return;
end;
if ParamType==1;
   VecParmHat = vech(Gammayy);
   return;
elseif ParamType==2;
   VecParmHat = vec(GammayY);
   return;
elseif ParamType==3;
   VecParmHat = vech(GammaYY);
end;

%
% end of DSGEVARPhiHatSigmaHat.m
%
