function [JointPDH,MargPDH,LaplaceMargLike,PredVars,MargPlugin,JointPlugin] = DSGEVARPredictiveLikelihoodParam(theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,YData,X,IsOriginal,IsPlugin,StepLength,DSGEModel,CurrINI)
% DSGEVARPredictiveLikelihoodParam: Computes the joint and marginal predictive likelihood for selected subset of variables
%                                   for fixed parameters values using the Laplace approximation of the marginal likelihood
%                                   in a DSGE-VAR.
%
% USAGE:
%
%       [JointPDH,MargPDH,LaplaceMargLike,PredVars,MargPlugin,JointPlugin] = DSGEVARPredictiveLikelihoodParam(theta, ...
%                        thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,AIMData, ...
%                        lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,YData,X, ...
%                        IsOriginal,IsPlugin,StepLength,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                                 0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of
%                                 transformation that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                 truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                 and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                 "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices
%                                 with the parameter values for the distributions. In the case of the normal, beta, gamma,
%                                 logistic and gumbel these parameter values are the mean and the standard deviation.
%                                 For the invgamma it is the s and q parameters, where q is a positive integer (degrees
%                                 of freedom). For the left truncated normal there's a third parameter, the lower bound.
%                                 For the uniform the parameter values are the lower and the upper bound. For the student-t
%                                 they are the location (mode), the scale and the degrees of freedom parameter, while for
%                                 the Cauchy only the location and scale parameters are needed. For the Pareto the
%                                 parameter values are for the mode and the shape parameter.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                                 for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                             variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                             variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                             regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                             are located.
%
%                  logGPR (scalar) with the value of the log of the gamma product function ratios. In the case of
%                             the joint posterior mode it is equal to logCPC with the value of the constant term
%                             for the log of the concentrated posterior density.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                             annualized values shoudl be ptredicted.
%
%                  IsPlugin (boolean) that takes the value 1 if the plugin estimate only should be calculated
%                             and 0 if both the plugin and the Laplace approximation should be dealt with.
%
%                  StepLength (scalar) with the step length used when computing minus the Hessian with
%                             finite differences.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: JointPDH (vector) of dimension h with the height of the joint predictive likelihood for the
%                                 h forecast periods using Laplace approximations.
%
%                  MargPDH (vector) of dimension h with the height of the marginal predictive likelihood for the
%                                 h forecast periods using Laplace approximations.
%
%                  LaplaceMargLike (scalar) with the value of the Laplace approximation of the log marginal
%                                 likelihood for the historical sample.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                                 variables for each prediction period.
%
%                  MargPlugin (matrix) of dimension h with the value of the marginal conditional likelihood
%                                 for the h forecast periods (a plugin estimate).
%
%                  JointPlugin (matrix) of dimension h with the value of the joint conditional likelihood
%                                 for the h forecast periods (a plugin estimate).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 4, 2012.
%                        This version: June 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-04-2012: Added the "ModeValue" input variable, removed the "ModeType" variable, and finished the
%               original data part of the function.
%
% * 15-05-2012: Added the output variables "MargPlugin" and "JointPlugin".
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 29-08-2012: Made sure that the correct Hessian matrices are calculated.  Added the "IsPlugin" input
%               variable which controls if the Laplace approximation should be computed in addition to
%               the plugin estimate.
%
% * 30-08-2012: The functions "logMargPredLikeDSGEVAR" and "logPredLikeDSGEVAR" no longer take the "logGPR"
%               input variable.
%
% * 24-09-2012: Updated the documentation.
%
% * 18-10-2012: Began work on the annualized predictive likelihood computations.
%
% * 19-10-2012: Finished coding the annualized predictive likelihood computations.
%
% * 25-10-2012: Removed the "ModeValue" input variable.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 12-06-2016: The function now partly supports the system prior.
%
% * 14-06-2017: The function now fully supports the system prior.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
JointPDH = [];
MargPDH = [];
LaplaceMargLike = [];
PredVars = [];
MargPlugin = [];
JointPlugin = [];
%
% Make sure that the ModelParameter structure takes the correct values.
% This is not necessary to do when initial values are given to the function.
% In that case, more substantial checking is required!
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% Update any parameteters that need updating
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% See if we can solve the model
%
try;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   end;
catch;
   if DSGEModel.ModelSolver==1;
      ErrorStr = ['There was an error running AiM. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==2;
      ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==3;
      ErrorStr = ['There was an error running Gensys. YADA received the following message: ' lasterr];
   end;
   About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,500,CurrINI);
   drawnow;
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
try;
   eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
catch;
   %
   % display an error message
   %
   ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' lasterr];
   About(ErrorStr,'error','Error - Measurement Equation',150,500,CurrINI);
   return;
end;
%
% Get the sub-sample
%
h = size(YData,2);
if length(size(H))==3;
   if size(H,3)<HSample(2)+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations and forecasts (T+h = ' int2str(HSample(2)+h) '). YADA has therefore aborted from computing the predictive likelihood.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   try;
      eval(['logSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.SystemPriorFile) '.m". Message caught is: ' lasterr];
      About(ErrorStr,'error','Error - System Prior Density',150,500,CurrINI);
      return;
   end;
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
   else;
      txt = 'YADA needs to compute the system prior mode before it can compute the predictive likelihood.';
      About(txt,'information','Predictive Likelihood',200,500,CurrINI);
      return;
   end;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
% compute the posterior mode of the VAR parameters at theta
%
[Phi,SigmaEpsilon,A0,IdentifyStatus,Status] = GetDSGEVARParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
if isinf(lambda)==0;
   SigmaEpsilon = (((1+lambda)*T)/(((1+lambda)*T)+n+1))*SigmaEpsilon;
end;
%
% fix the parameter vector and count the number of elements
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
if isinf(lambda)==0;
   phi = [vec(Phi);vech(SigmaEpsilon);phi];
   Dn = DuplicationMatrix(n);
else;
   Dn = [];
end;
NumParam = length(phi);
if IsPlugin==0;
   LogPostKernel = -logFullPosteriorKernelDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,Dn,GammaHatyy,GammaHatyY,GammaHatYY,T,n,p,npk,DetProductMoments,HSample,CurrINI.OrderQZ);
   %
   VechHessian = YADAHessianMatrix('logFullPosteriorKernelDSGEVAR',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,Dn,GammaHatyy,GammaHatyY,GammaHatYY,T,n,p,npk,DetProductMoments,HSample,CurrINI.OrderQZ);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   LaplaceMargLike = MargLikeLaplace(LogPostKernel,HessianMat,0)-LogMargLikeSystemLaplace;
   logdetHessian = LogDet(HessianMat);
end;
%
% initialize the remaining output
%
MargPDH = zeros(h,1);
JointPDH = MargPDH;
Y = vec(fliplr(DSGEModel.Y(:,HSample(2)-p+1:HSample(2))));
J = zeros(n*p,n);
J(1:n,:) = eye(n);
k = DSGEModel.k;
%
if IsOriginal==1;
   %
   % compute the marginal and joint predictive posterior
   %
   [MargPDH,PredVars] = logMargPredLikeDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YData,X,k,h,0);
   JointPDH = -logPredLikeDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YData,X,k,h,0)-LogMargLikeSystemLaplace;
   MargPDH = -MargPDH-LogMargLikeSystemLaplace;
   %
   MargPlugin = MargPDH;
   JointPlugin = JointPDH;
   if IsPlugin==0;
      %
      % compute Hessian matrices
      %
      VechHessianMargPred = YADAHessianMatrix('logMargPredLikeDSGEVAR',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YData,X,k,h,1);
      drawnow;
      VechHessianJointPred = YADAHessianMatrix('logPredLikeDSGEVAR',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YData,X,k,h,1);
      drawnow;
      %
      % compute the Laplace approximations
      %
      for i=1:h;
         %
         % marginal predictive likelihoods
         %
         HessMargth = GeneralizedCholeski(reshape(VechHessianMargPred(i,:),NumParam,NumParam));
         MargPDH(i) = MargPDH(i)+((1/2)*(logdetHessian-LogDet(HessMargth)));
         %
         % joint predictive likelihoods
         %
         HessJointth = GeneralizedCholeski(reshape(VechHessianJointPred(i,:),NumParam,NumParam));
         JointPDH(i) = JointPDH(i)+((1/2)*(logdetHessian-LogDet(HessJointth)));
      end;
   end;
else;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,HSample(2)-2:HSample(2));
      DSGEModel.Frequency = 4;
   else;
      Yinit = DSGEModel.Y(:,HSample(2)-10:HSample(2));
      DSGEModel.Frequency = 12;
   end;
   DSGEModel.Yinit = Yinit;
   %
   % no need to make sure that we take NaN's over the historical sample into account appropriately
   % since the DSGE-VAR doesn't support them
   %
   YDataAnnual = Annualize([Yinit YData],DSGEModel.annual,ones(n,1));
   [MargPDH,PredVars] = logMargPredLikeDSGEVARAnnual(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YDataAnnual,X,k,h,0);
   MargPDH = -MargPDH-LogMargLikeSystemLaplace;
   JointPDH = -logPredLikeDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YData,X,k,h,0)-LogMargLikeSystemLaplace;
   %
   MargPlugin = MargPDH;
   JointPlugin = JointPDH;
   if IsPlugin==0;
      %
      % compute Hessian matrices
      %
      VechHessianMargPred = YADAHessianMatrix('logMargPredLikeDSGEVARAnnual',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YDataAnnual,X,k,h,1);
      drawnow;
      VechHessianJointPred = YADAHessianMatrix('logPredLikeDSGEVAR',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,CurrINI.OrderQZ,J,Y,YData,X,k,h,1);
      drawnow;
      %
      % compute the Laplace approximations
      %
      for i=1:h;
         %
         % marginal predictive likelihoods
         %
         HessMargth = GeneralizedCholeski(reshape(VechHessianMargPred(i,:),NumParam,NumParam));
         MargPDH(i) = MargPDH(i)+((1/2)*(logdetHessian-LogDet(HessMargth)));
         %
         % joint predictive likelihoods
         %
         HessJointth = GeneralizedCholeski(reshape(VechHessianJointPred(i,:),NumParam,NumParam));
         JointPDH(i) = JointPDH(i)+((1/2)*(logdetHessian-LogDet(HessJointth)));
      end;
   end;
end;

%
% end of DSGEVARPredictiveLikelihoodParam.m
%
