function ShockRes = DSGEVARShockCorrelationDistribution(DSGEModel,CurrINI,controls,UsedDraws,CurrChain,IsPosterior,Weights,PME,lambda)
% DSGEVARShockCorrelationDistribution: Computes the mean value of the structural shocks as well as quantiles by loading data from disk.
%
% USAGE:
%
%       ShockRes = DSGEVARShockCorrelationDistribution(DSGEModel,CurrINI,controls,UsedDraws,CurrChain,...
%                                                      IsPosterior,Weights,PME,lambda)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                            main dialog.
%
%                  UsedDraws (integer) with the number of draws of the DSGE-VAR parameters.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                            are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
% REQUIRED OUTPUT: ShockRes (structure) with fields ,
%                            "Quantiles" is a vector structure  and "percent".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 19, 2014.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-03-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
ShockRes = [];
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
p = DSGEModel.BVARLags;
ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
for j=1:DSGEModel.n;
   ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
end;
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
NumQuants = length(DSGEModel.Percentiles);
%
% load the first file
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-ShockCorrelations-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-ShockCorrelations-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
end;
ShockData = load(file);
%
n = DSGEModel.n;
ShockRes.SampleMean = zeros(n,1);
ShockRes.StdShocks = zeros(n,1);
ShockRes.CorrShocks = zeros(n,n);
ShockRes.ShockNames = ShockNames;
for i=1:NumQuants;
   ShockRes.Quantiles(i).SampleMean = zeros(n,1);
   ShockRes.Quantiles(i).StdShocks = zeros(n,1);
   ShockRes.Quantiles(i).CorrShocks = zeros(n,n);
   ShockRes.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
Weightsn = Weights*ones(1,n);
%
% setup a wait dialog
%
txt = ['Please wait while YADA computes the average structural shock means, standard deviations, and correlations as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(ShockNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Structural Shock Correlations' PriorHeader],500,200,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
for i=1:n;
   TempMean = zeros(UsedDraws,1);
   TempStd = zeros(UsedDraws,1);
   TempCorr = zeros(UsedDraws,n);
   %
   txt = ['Please wait while YADA computes the average structural shock means, standard deviations, and correlations as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(ShockNames(i,:)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   CurrSaves = 0;
   for ThisSave=1:NumFiles;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-ShockCorrelations-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-ShockCorrelations-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
      end;
      ShockData = load(file);
      CurrShocks = ShockData.Shocks;
      NumSaves = length(CurrShocks);
      for s=1:NumSaves;
         TempMean(CurrSaves+s,:) = CurrShocks(s).SampleMean(i);
         TempStd(CurrSaves+s,:) = CurrShocks(s).StdShocks(i);
         TempCorr(CurrSaves+s,:) = CurrShocks(s).CorrShocks(i,:);
      end;
      CurrSaves = CurrSaves+NumSaves;
      drawnow;
   end;
   %
   % we can now calculate the mean
   %
   ShockRes.SampleMean(i) = (1/UsedDraws)*sum(TempMean.*Weights,1);
   ShockRes.StdShocks(i) = (1/UsedDraws)*sum(TempStd.*Weights,1);
   ShockRes.CorrShocks(i,:) = (1/UsedDraws)*sum(TempCorr.*Weightsn,1);
   %
   % now we can sort these and create the quantiles from the sorted
   %
   TempMean = sort(TempMean);
   TempStd = sort(TempStd);
   TempCorr = sort(TempCorr);
   if DSGEModel.ConfidenceRegionMethod==1;
      for l=1:NumQuants;
         if DSGEModel.Percentiles(l)<50;
            QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
         else;
            QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
         end;
         ShockRes.Quantiles(l).SampleMean(i) = TempMean(QuantVal);
         ShockRes.Quantiles(l).StdShocks(i) = TempStd(QuantVal);
         ShockRes.Quantiles(l).CorrShocks(i,:) = TempCorr(QuantVal,:);
      end;
   else;
      for l=1:floor(NumQuants/2);
         ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
         BoundsMean = HighestProbabilityDensityBand(TempMean,ConfDraws);
         ShockRes.Quantiles(l).SampleMean(i) = BoundsMean(1);
         ShockRes.Quantiles(NumQuants-l+1).SampleMean(i) = BoundsMean(2);
         BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
         ShockRes.Quantiles(l).StdShocks(i) = BoundsStd(1);
         ShockRes.Quantiles(NumQuants-l+1).StdShocks(i) = BoundsStd(2);
         for j=1:n;
            BoundsCorr = HighestProbabilityDensityBand(TempCorr(:,j),ConfDraws);
            ShockRes.Quantiles(l).CorrShocks(i,j) = BoundsCorr(1);
            ShockRes.Quantiles(NumQuants-l+1).CorrShocks(i,j) = BoundsCorr(2);
         end;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARShockCorrelationDistribution.m
%
