function VDs = DSGEVARVDsDistribution(DSGEModel,CurrINI,controls,ShockType,UsedDraws,h,VDType,CurrChain,IsPosterior,Weights,PME,lambda)
% DSGEVARVDsDistribution: Computes the mean value of variance decompositions as well as quantiles by loading data from disk.
%
% USAGE:
%
%       VDs = DSGEVARVDsDistribution(DSGEModel,CurrINI,controls,ShockType,UsedDraws,h,VDType,CurrChain,...
%                                    IsPosterior,Weights,PME,lambda)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                            main dialog.
%
%                  ShockType (string vector) taking the values "individual" or "group".
%
%                  UsedDraws (integer) with the number of draws of the DSGE-VAR parameters.
%
%                  h (integer) with the response horizon.
%
%                  VDType (integer) that takes the value 1 when using the original data and
%                            2 when using the levels.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                            are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
% REQUIRED OUTPUT: VDs (structure) with fields "FEVD", "LRVD" and "Quantiles". The "FEVD" field holds a 3D matrix of dimension
%                                 n x q x h, where n is the number of observed variables, q the number of shocks or shock
%                                 groups, and h the impulse response horizon. The values are the posterior or prior mean
%                                 variance decomposition. The "LRVD" fields is an nxq matrix with the posterior or prior mean
%                                 of the long-run variance decompositions. The field "Quantiles" is a vector structure with
%                                 fields "FEVD", "LRVD", and "percent", where "FEVD" holds a 3D matrix of dimension n x q x h
%                                 with values from the percent quantile of the marginal distribution of the variance decompositions,
%                                 "LRVD" is an n x q matrix with the long-run results,and "percent" holds the percentile value
%                                 in percent.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 21, 2010.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-05-2010: Updated the documentation.
%
% * 27-05-2010: Removed an unnecessary variable "AddWidth".
%
% * 22-10-2010: The wait dialog now informs the user about the number of lags.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
VDs = [];
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
Weightsh = Weights*ones(1,h);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if VDType==1;
   DataType = 'Original';
elseif VDType==2;
   DataType = 'Levels';
end;
%
% set up the shocks to use
%
p = DSGEModel.BVARLags;
n = DSGEModel.n;
ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
if strcmp(ShockType,'individual')==1;
   IMatrix = [];
   q = n;
   ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
   ShockStr = 'shock';
else;
   IMatrix = SubIdentityMatrix(ShockGroups);
   q = size(IMatrix,2);
   ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
   ShockStr = 'shock group';
   for j=1:size(ShockNames,1);
      ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
   end;
end;
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
NumQuants = length(DSGEModel.Percentiles);
%
% initialize output
%
VDs.FEVDs = zeros(n,q,h);
VDs.LRVD = zeros(n,q);
for i=1:NumQuants;
   VDs.Quantiles(i).FEVDs = zeros(n,q,h);
   VDs.Quantiles(i).LRVD = zeros(n,q);
   VDs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean variance decompositions as well as percentile values of the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
else;
   txt = ['Please wait while YADA computes the prior mean variance decompositions as well as percentile values of the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
end;
WaitHandle = WaitDLG(txt,'information',[DataType ' Variance Decompositions' PriorHeader],500,300,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:n;
   for j=1:q;
      %
      % update the wait dialog
      %
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean variance decompositions as well as percentile values of the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      else;
         txt = ['Please wait while YADA computes the prior mean variance decompositions as well as percentile values of the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(UsedDraws,h);
         TempLR = zeros(UsedDraws,1);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-VAR-VD-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-VAR-VD-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
            end;
            VDData = load(file);
            VD = VDData.VarDec;
            NumSaves = length(VD);
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            %
            if isempty(IMatrix)==1;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = VD(s).FEVDs(i,j,:);
                  TempLR(CurrSaves+s,1) = VD(s).LRVD(i,j);
               end;
            else;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = IMatrix(:,j)'*reshape(VD(s).FEVDs(i,:,:),n,h);
                  TempLR(CurrSaves+s,1) = VD(s).LRVD(i,:)*IMatrix(:,j);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % sort the data
         %
         VDs.FEVDs(i,j,:) = (1/UsedDraws)*sum(TempVar.*Weightsh);
         VDs.LRVD(i,j) = (1/UsedDraws)*sum(TempLR.*Weights);
         TempVar = sort(TempVar);
         TempLR = sort(TempLR);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
               end;
               VDs.Quantiles(l).FEVDs(i,j,:) = TempVar(QuantVal,:);
               VDs.Quantiles(l).LRVD(i,j) = TempLR(QuantVal,1);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
               BoundsLR = HighestProbabilityDensityBand(TempLR,ConfDraws);
               VDs.Quantiles(l).LRVD(i,j) = BoundsLR(1);
               VDs.Quantiles(NumQuants-l+1).LRVD(i,j) = BoundsLR(2);
               for t=1:h;
                  Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
                  VDs.Quantiles(l).FEVDs(i,j,t) = Bounds(1);
                  VDs.Quantiles(NumQuants-l+1).FEVDs(i,j,t) = Bounds(2);
               end;
            end;
         end;
      catch;
         for t=1:h;
            TempVar = zeros(UsedDraws,1);
            TempLR = zeros(UsedDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-VAR-VD-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-VAR-VD-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
               end;
               VDData = load(file);
               VD = VDData.VarDec;
               NumSaves = length(VD);
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               %
               if isempty(IMatrix)==1;
                  for s=1:NumSaves;
                     TempVar(CurrSaves+s,1) = VD(s).FEVDs(i,j,t);
                     TempLR(CurrSaves+s,1) = VD(s).LRVD(i,j);
                  end;
               else;
                  for s=1:NumSaves;
                     MT = IMatrix(:,j)'*reshape(VD(s).FEVDs(i,:,:),n,h);
                     TempVar(CurrSaves+s,1) = MT(t);
                     TempLR(CurrSaves+s,1) = VD(s).LRVD(i,:)*IMatrix(:,j);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % sort the data
            %
            VDs.FEVDs(i,j,t) = (1/UsedDraws)*sum(TempVar.*Weights);
            VDs.LRVD(i,j) = (1/UsedDraws)*sum(TempLR.*Weights);
            TempVar = sort(TempVar);
            TempLR = sort(TempLR);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  end;
                  VDs.Quantiles(l).FEVDs(i,j,t) = TempVar(QuantVal,1);
                  VDs.Quantiles(l).LRVD(i,j) = TempLR(QuantVal,1);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
                  BoundsLR = HighestProbabilityDensityBand(TempLR,ConfDraws);
                  VDs.Quantiles(l).LRVD(i,j) = BoundsLR(1);
                  VDs.Quantiles(NumQuants-l+1).LRVD(i,j) = BoundsLR(2);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  VDs.Quantiles(l).FEVDs(i,j,t) = Bounds(1);
                  VDs.Quantiles(NumQuants-l+1).FEVDs(i,j,t) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARVDsDistribution.m
%
