function [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda)
% GetLambdaSelectionData: Determines the data to show in the lambda selection dialog and the default
%                         value for lambda.
%
% USAGE:
%
%       [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda)
%
% REQUIRED INPUT:  MatFile (string vector) with the full path and name of the mat-file where
%                          the default value for lambda should be located.
%
%                  lambda (vector) with lambda values that the user can choose from.
%
% REQUIRED OUTPUT: LambdaValuesStr (string matrix) with the different lambda values that the
%                          user can choose from.
%
%                  SuggestedLambdaPosition (integer) whoose value at at least 1 and at most
%                          equal to the length of lambda. The function tries to pick lambda
%                          such that the marginal likelihood for the DSGE-VAR is maximized.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 21, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-02-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 27-02-2011: Took into account that "lambda" need not be a field of "ModeData", i.e, when
%               the DSGE posterior mode mat-file is used.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if FileExist(MatFile)==1;
   try;
      ModeData = load(MatFile);
      if isfield(ModeData,'lambda')==1;
         LambdaData = ModeData.lambda;
         Positions = ismember(LambdaData,lambda).*(1:length(LambdaData));
         Positions = Positions(Positions>0);
         if isempty(Positions)==0;
            [MaxMargLike,Position] = max(ModeData.LaplaceMargLike);
            SuggestedLambdaPosition = (lambda==LambdaData(Position)).*(1:length(lambda));
            SuggestedLambdaPosition = SuggestedLambdaPosition(SuggestedLambdaPosition>0);
            if isempty(SuggestedLambdaPosition)==1;
               SuggestedLambdaPosition = length(lambda);
            end;
         else;
            SuggestedLambdaPosition = length(lambda);
         end;
      else;
         SuggestedLambdaPosition = length(lambda);
      end;
   catch;
      SuggestedLambdaPosition = length(lambda);
   end;
else;
   SuggestedLambdaPosition = length(lambda);
end;
LambdaValuesStr = '';
for i=1:length(lambda);
   LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
end;

%
% end of GetLambdaSelectionData.m
%
