function [value,CurrChain,UsePME,Lambda] = GetVARDSGEPostDrawsSelection(DialogName,DSGEModel,controls,CurrINI)
% GetVARDSGEPostDrawsSelection: determines the parameters necessary for loading posterior draws for tha
%                               VAR parameters of the DSGE-VAR.
%
% USAGE:
%
%       [value,CurrChain,UsePME,Lambda] = GetVARDSGEPostDrawsSelection(DialogName,DSGEModel,controls,CurrINI)
%
% REQUIRED INPUT:  DialogName (string) with the name to use for the dialogs.
%
%                  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrChain (integer) the value of the parallel chain to load data from.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: value (integer) with the selected position for the available lambda hyperparameter.
%
%                  CurrChain (integer) with the number of the selected Markov chain with posterior draws.
%
%                  UsePME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 9, 2010.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 11-09-2017: Made sure that posterior mode estimator doesn't have to be selected under SMC with likelihood
%               tempering posterior sampler.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
value = 0;
CurrChain = 1;
UsePME = [];
Lambda = [];
%
% check if we have multiple markov chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
if NumChains>1;
   %
   % ask which chain to load
   %
   ChainsStr = '';
   for i=1:NumChains;
      ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
   end;
   [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection',DialogName,'','',CurrINI);
else;
   CurrChain = 1;
   action = 'ok';
end;
if strcmp(lower(action),'cancel')==0;
   %
   % determine which results are available
   %
   [status,PME] = VerifyVARDSGEPosteriorDraws(DSGEModel,CurrChain,controls);
   if status==1;
      %
      % check which posterior mode estimators the use can choose between
      %
      AvailablePME = 1-[isempty(PME(1).Lambda) isempty(PME(2).Lambda) isempty(PME(3).Lambda)];
      if sum(AvailablePME)>1;
         PMEStr = '';
         if AvailablePME(1)==1;
            PMEStr = 'DSGE model posterior mode';
         end;
         if AvailablePME(2)==1;
            PMEStr = strvcat(PMEStr,'DSGE-VAR marginal posterior mode');
         end;
         if AvailablePME(3)==1;
            PMEStr = strvcat(PMEStr,'DSGE-VAR joint posterior mode');
         end;
         %
         % display a selection dialog
         %
         if DSGEModel.PosteriorSampler<=7;
            [action,SelectedPME] = SelectionDlg(PMEStr,1,'Select the posterior mode estimator used in posterior sampling:','Estimator Selection',DialogName,'','',CurrINI);
         else;
            action = 'OK';
            SelectedPME = 1;
         end;
      else;
         UsePME = AvailablePME.*(1:3);
         UsePME = UsePME(UsePME>0);
         action = 'OK';
         SelectedPME = [];
      end;
      if strcmp(lower(action),'ok')==1;
         %
         % check if we have multiple lambda values to choose between
         %
         if isempty(SelectedPME)==0;
            PMEStr = StringTrim(PMEStr(SelectedPME,:));
            if strcmp(PMEStr,'DSGE model posterior mode')==1;
               UsePME = 1;
            elseif strcmp(PMEStr,'DSGE-VAR marginal posterior mode')==1;
               UsePME = 2;
            else;
               UsePME = 3;
            end;
         end;
         Lambda = PME(UsePME).Lambda;
         if length(Lambda)>1;
            LambdaValuesStr = '';
            for i=1:length(Lambda);
               LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(Lambda(i),'%0.4f')]);
            end;
            [action,value] = SelectionDlg(LambdaValuesStr,length(Lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',DialogName,'','',CurrINI);
            if strcmp(lower(action),'ok')==0;
               value = 0;
            end;
         else;
            value = 1;
         end;
      end;
   else;
      txt = ['YADA couldn''t locate any posterior draws for the VAR model parameters on disk for posterior chain number ' int2str(CurrChain) '. Please choose a different Markov chain.'];
      About(txt,'information',DialogName,200,500,CurrINI);
   end;
end;

%
% end of GetVARDSGEPostDrawsSelection.m
%
