function [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrentChain)
% LoadDSGEARPosteriorDraws: Loads the data from posterior sampling of the DSGE model parameters based on a DSGE-VAR.
%
% USAGE:
%
%       [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,...
%                                                                                  CurrINI,lambda,PME,CurrentChain)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  CurrentChain (integer) the value of the parallel chain to load data from.
%
% REQUIRED OUTPUT: PostSample (matrix) of dimension (NumIterxNumParam) with all the non-discarded
%                            posterior draws of phi (the transformed parameters)
%
%                  thetaPostSample (matrix) of dimension (NumIterxNumParam) with all the non-discarded
%                            posterior draws of theta (the original parameters)
%
%                  LogPost (vector) of dimension (NumIterx1) with all the log posterior values for the
%                            non-discarded posterior draws. These posterior values are calculated
%                            using the "logPosteriorPhiDSGE" function.
%
%                  AcceptedDraws (vector) of dimension (NumIterx1) with the number of accepted draws.
%
%                  NumBurnin (integer) with the number of brun-in draws.
%
%                  Weights (vector) of dimension NumIter x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 26, 2010.
%                        This version: September 8, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-02-2010: Updated the documentation.
%
% * 03-03-2010: Took into account that posterior samplign output for the DSGE-VAR no longer
%               uses the selected shocks as an indicator for the filename of result files
%               when dealing with the DSGE model parameters.
%
% * 04-03-2010: Added a test if the number of draws per batch is an integer or not.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 20-01-2015: Adapted the function for use with YADA's sequential tools.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 14-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 08-09-2017: Added the output variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

p = DSGEModel.BVARLags;
%
%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of posterior sample batches to save per chain
%
BatchStr = get(controls.posterior.batch,'String');
NumBatches = str2num(StringTrim(strrep(BatchStr(DSGEModel.SampleBatchValue,:),',','')));
if NumBatches>NumPostDraws;
   NumBatches = NumPostDraws;
end;
NumBatchDraws = NumPostDraws/NumBatches;
if IsInteger(NumBatchDraws)==0;
   NumBatchDraws = NumPostDraws;
   NumBatches = 1;
end;
BatchIndex = (1:NumBatchDraws);
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 4. Number of burnin draws per chain
%
if DSGEModel.PosteriorSampler<=7;
	BurninStr = get(controls.posterior.burnin,'String');
	NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
	if NumBurnin>=NumPostDraws;
	   if isempty(CurrINI.Sequence)==1;
	      txt = ['The number of burn-in draws (' StringTrim(BurninStr(DSGEModel.BurnInValue,:)) ') leaves no posterior draws (' StringTrim(DrawsStr(DSGEModel.PosteriorDrawsValue,:)) ') for inference. YADA will therefore use 0 burn-in draws.'];
	      About(txt,'information','YADA - Burnin Draws Problem',180,500,CurrINI);
	   else;
	      SetLogText('Number of burnin draws has been reset to 0.');
	   end;
	   NumBurnin = 0;
	end;
else;
	NumBurnin = 0;
end;
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% Now we turn to batch no 1. since we need to determine how many parameters there are in phi and theta.
%
if DSGEModel.PosteriorSampler==1;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==2;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==3
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==4;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==5;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==6;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==7;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==8;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
end;
PSData = load(PostSampleFile);
NumParam = size(PSData.BatchPostSample,2);
%
% initialize the data matrices
%
PostSample = zeros(NumPostDraws,NumParam);
thetaPostSample = PostSample;
LogPost = zeros(NumPostDraws,1);
AcceptedDraws = LogPost;
Weights = [];
%
% load the first batch of data
%
PostSample(BatchIndex,:) = PSData.BatchPostSample;
thetaPostSample(BatchIndex,:) = PSData.BatchthetaPostSample;
LogPost(BatchIndex,1) = PSData.BatchLogPost;
if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
   AcceptedDraws(BatchIndex,1) = PSData.BatchAcceptedDraws;
elseif DSGEModel.PosteriorSampler==2;
   AcceptedDraws(BatchIndex,1) = PSData.BatchLogPostEvaluations;
end;
if DSGEModel.PosteriorSampler==8;
   Weights = PSData.Weights;
end;
%
% now we loop over all draws
%
for NBatch=2:NumBatches;
   BatchIndex = BatchIndex+NumBatchDraws;
   if DSGEModel.PosteriorSampler==1;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==8;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   end;
   PSData = load(PostSampleFile);
   PostSample(BatchIndex,:) = PSData.BatchPostSample;
   thetaPostSample(BatchIndex,:) = PSData.BatchthetaPostSample;
   LogPost(BatchIndex,1) = PSData.BatchLogPost;
   if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
      AcceptedDraws(BatchIndex,1) = PSData.BatchAcceptedDraws;
   elseif DSGEModel.PosteriorSampler==2;
      AcceptedDraws(BatchIndex,1) = PSData.BatchLogPostEvaluations;
   end;
   drawnow;
   pause(0.02);
end;

%
% end of LoadDSGEVARPosteriorDraws.m
%
