function [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrentChain,IsSimulate)
% LoadVARDSGEPosteriorDraws: Loads the data from posterior sampling of the VAR model parameters based on a DSGE-VAR.
%
% USAGE:
%
%       [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = ...
%                                       LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrentChain,IsSimulate)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  CurrentChain (integer) the value of the parallel chain to load data from.
%
%                  IsSimulate (boolean) that takes on the value 1 if the parameter draws will
%                            be used for simulation exercises (such as prediction), and 0
%                            otherwise.
%
% REQUIRED OUTPUT: PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                            VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the draws of the
%                            unique Sigma paremeters. Each row holds vech(Sigma)'.
%
%                  A0Draws (matrix) of dimension UsedDraws x n^2 with the draws of the A0
%                            parameters. Each row holds vec(A0)'. If the structural shocks
%                            can't be identified, then A0Draws is empty.
%
%                  thetaPostSample (matrix) of dimension UsedDraws x NumParan with the original
%                            parameters from the DSGE model.
%
%                  PostSample (matrix) of dimension UsedDraws x NumParam with the transformed
%                            parameters of the DSGE model.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
% OPTIONAL OUTPUT: PhiMode (matrix) of dimension nx(np+k) with the mode values of the parameters
%                            on exogenous and lagged endogenous variables.
%
%                  SigmaModeEpsilon (marix) of dimension nxn with the mode values of the residual
%                            covariance matrix parameters.
%
%                  A0Mode (matrix) of dimension nxn with the mode values of the A0 matrix. This
%                            matrix is empty if the structural shocks of the VAR could not be
%                            identified.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 9, 2010.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-06-2010: Added the "IsSimulate" input variables.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 25-11-2011: Checked to make sure that posterior mode estimates are loaded.
%
% * 03-01-2012: Updated the documentation.
%
% * 11-05-2012: Added the output variable "thetaPostSample".
%
% * 14-05-2012: Added the output variable "PostSample".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 14-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 08-09-2017: Took into account that "LoadDSGEVARPosteriorDraws" provides the "Weights" output variable.
%
% * 11-09-2017: Added the "Weights" as an output variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
A0Draws = [];
thetaPostSample = [];
PostSample = [];
Weights = [];
%
p = DSGEModel.BVARLags;
%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% remove burnin draws
%
if DSGEModel.PosteriorSampler<=7
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
   NumPostDraws = NumPostDraws-NumBurnin;
else;
   NumBurnin = 0;
end;
%
% and the share used
%
if DSGEModel.PosteriorSampler<=7;
   DrawsPercentStr = get(controls.posterior.usepostdraws,'String');
   DrawsPercent = str2num(strrep(StringTrim(DrawsPercentStr(DSGEModel.PostDrawsPercentValue,:)),'%',''));
   UsedDraws = ceil(DrawsPercent*NumPostDraws/100);
else;
   UsedDraws = NumPostDraws;
end;
%
% 2. Number of posterior sample batches to save per chain
%
if DSGEModel.PosteriorSampler<=7;
   BatchStr = get(controls.posterior.batch,'String');
   NumBatches = str2num(StringTrim(strrep(BatchStr(DSGEModel.SampleBatchValue,:),',','')));
   if NumBatches>UsedDraws;
      NumBatches = UsedDraws;
   end;
   NumBatchDraws = UsedDraws/NumBatches;
   if IsInteger(NumBatchDraws)==0;
      NumBatchDraws = UsedDraws;
      NumBatches = 1;
   end;
else;
   NumBatchDraws = NumPostDraws;
   NumBatches = 1;
end;

BatchIndex = (1:NumBatchDraws);
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% Now we turn to batch no 1. since we need to determine how many parameters there are in phi and theta.
%
PostSampleStr = [int2str(DSGEModel.InverseHessianEstimator) '.' int2str(DSGEModel.PosteriorSampler)];
if DSGEModel.PosteriorSampler==3;
   PostSampleStr = [PostSampleStr '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==4;
   PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks)];
elseif DSGEModel.PosteriorSampler==5;
   PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==6;
   PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2))];
elseif DSGEModel.PosteriorSampler==7;
   PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==8;
   PostSampleStr = [int2str(DSGEModel.PosteriorSampler) '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm)];
end;
%
PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrentChain) '.' int2str(NumChains) '-1.' int2str(NumBatches) '-' int2str(UsedDraws) '.' int2str(NumPostDraws) '.mat']);
PSData = load(PostSampleFile);
nnpk = size(PSData.BatchPhiDraws,2);
nn12 = size(PSData.BatchSigmaDraws,2);
nn = size(PSData.BatchA0Draws,2);
%
PhiDraws = zeros(UsedDraws,nnpk);
SigmaDraws = zeros(UsedDraws,nn12);
if nn>0;
   A0Draws = zeros(UsedDraws,nn);
end;
%
% load the first batch of data
%
PhiDraws(BatchIndex,:) = PSData.BatchPhiDraws;
SigmaDraws(BatchIndex,:) = PSData.BatchSigmaDraws;
if nn>0;
   A0Draws(BatchIndex,:) = PSData.BatchA0Draws;
end;
if nargout==9;
   PhiMode = PSData.PhiMode;
   SigmaModeEpsilon = PSData.SigmaModeEpsilon;
   A0Mode = PSData.A0Mode;
end;
%
% now we loop over all draws
%
for NBatch=2:NumBatches;
   BatchIndex = BatchIndex+NumBatchDraws;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '-' int2str(UsedDraws) '.' int2str(NumPostDraws) '.mat']);
   PSData = load(PostSampleFile);
   PhiDraws(BatchIndex,:) = PSData.BatchPhiDraws;
   SigmaDraws(BatchIndex,:) = PSData.BatchSigmaDraws;
   if nn>0;
      A0Draws(BatchIndex,:) = PSData.BatchA0Draws;
   end;
   drawnow;
   pause(0.02);
end;
%
if (isfield(PSData,'PostSample')==1)&(isempty(PSData.PostSample)==0);
   thetaPostSample = PSData.thetaPostSample;
   PostSample = PSData.PostSample;
   if isfield(PSData,'Weights')==1;
      Weights = PSData.Weights;
   end;
else;
   [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrentChain);
   NumDraws = length(LogPost);
   clear('LogPost','AcceptedDraws');
   drawnow;
   thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
   PostSample = PostSample(NumBurnin+1:NumDraws,:);
   NumDraws = NumDraws-NumBurnin;
   %
   % make sure that we select "UsedDraws" draws
   %
   if UsedDraws<NumDraws;
      if DSGEModel.RandomizeDraws==0;
         DrawFreq = round(NumDraws/UsedDraws);
         Draws = (1:DrawFreq:NumDraws);
         if length(Draws)==(UsedDraws+1);
            Draws = [Draws UsedDraws];
         elseif length(Draws)>UsedDraws;
            Draws = Draws(1:UsedDraws);
         end;
      else;
         %
         % initialize the random number generator
         %
         if DSGEModel.RandomNumberValue==1;
            randn('state',0);
            rand('state',0);
         else;
            randn('state',sum(100*clock));
            rand('state',sum(100*clock));
         end;
         %
         Draws = ceil(rand(1,UsedDraws)*NumDraws);
      end;
      thetaPostSample = thetaPostSample(Draws,:);
      PostSample = PostSample(Draws,:);
      %
      % append the draws to the last posterior draws output file
      % for the VAR parameters
      %
      save(PostSampleFile,'thetaPostSample','PostSample','-APPEND');
   elseif UsedDraws>NumDraws;
      %
      % unable to determine suitable theta draws
      %
      thetaPostSample = [];
      PostSample = [];
   end;
end;
%
% check if we should select a subset of the posterior draws
%
if (DSGEModel.PosteriorSampler<=7)&(IsSimulate==1);
   PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
   NumPostDrawsUsage = str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',',''));
   if NumPostDrawsUsage<UsedDraws;
      if DSGEModel.RandomizeDraws==0;
         DrawFreq = round(UsedDraws/NumPostDrawsUsage);
         SelectDraws = (1:DrawFreq:UsedDraws);
         if length(SelectDraws)+1==NumPostDrawsUsage;
            SelectDraws = [SelectDraws UsedDraws];
         elseif length(SelectDraws)>NumPostDrawsUsage;
            SelectDraws = SelectDraws(1:NumPostDrawsUsage);
         end;
      else;
         SelectDraws = ceil(rand(1,NumPostDrawsUsage)*UsedDraws);
      end;
      PhiDraws = PhiDraws(SelectDraws,:);
      SigmaDraws = SigmaDraws(SelectDraws,:);
      if nn>0;
         A0Draws = A0Draws(SelectDraws,:);
      end;
      if isempty(thetaPostSample)==0;
         thetaPostSample = thetaPostSample(SelectDraws,:);
         PostSample = PostSample(SelectDraws,:);
      end;
   end;
end;

%
% end of LoadVARDSGEPosteriorDraws.m
%
