function [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile)
% LoadVARDSGEPriorDraws: Loads the data from prior sampling of the VAR model parameters based on a DSGE-VAR.
%
% USAGE:
%
%       [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  PriorDrawsFile (string) with the path and filename of the mat-file that
%                            contains the prior draws.
%
% REQUIRED OUTPUT: PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                            VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the draws of the
%                             unique Sigma paremeters. Each row holds vech(Sigma)'.
%
%                  A0Draws (matrix) of dimension UsedDraws x n^2 with the draws of the A0
%                             parameters. Each row holds vec(A0)'. If the structural shocks
%                             can't be identified, then A0Draws is empty.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 16, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% determine parameters
%
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% load the data
%
PriorData = load(PriorDrawsFile);
UsedDraws = length(PriorData.PriorDraws);
%
PriorDraws = PriorData.PriorDraws;
%
PhiDraws = zeros(UsedDraws,n*((n*p)+k));
SigmaDraws = zeros(UsedDraws,n*(n+1)/2);
if (isfield(PriorDraws(1),'A0')==1)&(isempty(PriorDraws(1).A0)==0);
   A0Draws = zeros(UsedDraws,n*n);
   IdentifyStatus = 1;
else;
   A0Draws = [];
   IdentifyStatus = 0;
end;
%
% now loop over all the entries
%
for j=1:UsedDraws;
   PhiDraws(j,:) = vec(PriorDraws(j).Phi)';
   SigmaDraws(j,:) = vech(PriorDraws(j).SigmaEpsilon)';
   if IdentifyStatus==1;
      A0Draws(j,:) = vec(PriorDraws(j).A0)';
   end;
   drawnow;
end;

%
% end of LoadVARDSGEPriorDraws.m
%
