function [Status,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal,PredictionVariables,SelectedYData] = SequentialPredictiveLikelihoodDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal,PredictionVariables,SelectedYData,RecomputePL)
% SequentialPredictiveLikelihoodDSGEVAR: Can be used for computing the predictive likelihood of DSGE-VARs
%                                        sequentially.
%
% USAGE:
%
%       [Status,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal, ...
%                 PredictionVariables,SelectedYData] = SequentialPredictiveLikelihoodDSGEVAR(DSGEModel,CurrINI,controls, ...
%                                                                      value,CurrChain,PME,Lambda,IsOriginal,IsMarginal, ...
%                                                                      PredictionVariables,SelectedYData,RecomputePL)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  value (integer) with the selected position for the available lambda hyperparameter.
%
%                  CurrChain (integer) with the number of the selected Markov chain with posterior draws.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
%                  IsOriginal (boolean) that takes the value 1 if the predictive likelihood
%                            should be computed for the original data and 0 if for annualized
%                            data. When empty, the function checks which alternative should
%                            be considered.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive
%                            likelihood should be computed and 0 for the joint. When empty,
%                            the function checks which alternative should be considered.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for. When empty, the function
%                            checks which alternative should be considered.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed
%                            variables.
%
%                  RecomputePL (boolean) which takes the value 1 if the predictive likelihood
%                            should be recomputed and 0 if old calculations should be used. When
%                            empty. the function checks which alternative should be considered.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the computations could be successfully
%                            carries out, 0 if not, and -1 if they were aborted by the user.
%
%                  value (integer) with the selected position for the available lambda hyperparameter.
%
%                  CurrChain (integer) with the number of the selected Markov chain with posterior draws.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
%                  IsOriginal (boolean) that takes the value 1 if the predictive likelihood
%                            should be computed for the original data and 0 if for annualized
%                            data. When empty, the function checks which alternative should
%                            be considered.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive
%                            likelihood should be computed and 0 for the joint. When empty,
%                            the function checks which alternative should be considered.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for. When empty, the function
%                            checks which alternative should be considered.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed
%                            variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 20, 2015.
%                        This version: September 11, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-01-2015: Finished the function.
%
% * 27-01-2015: Made sure that the final results are saved to a mat-file.
%
% * 06-02-2015: Corrected a typo.
%
% * 26-02-2015: Took into account that the function "DSGEVARPredLikeImportanceSampling" has had
%               its name changed to "DSGEVARPredLikeMCI", and that this function supports two
%               additional output variables "MinMaxCondLike" and "TestStd".
%
% * 31-03-2015: Added the possible results -1 for Status.
%
% * 15-06-2015: The function now deals with the input and output variables "SelectedYData" for handling different
%               setf of actuals of the observed variables.
%
% * 21-01-2016: Updated the documentation.
%
% * 17-03-2017: Took into account that "PrintDSGEPredictiveLikelihood" now takes five additional input variables.
%
% * 11-09-2017: Took into account that "LoadVARDSGEPosteriorDraws" provides the output variable "Weights", and that
%               "DSGEVARPredLikeMCI" takes the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Status = 0;
OriginalAnnual = [(strcmp(get(controls.dsgevartools.forecast.unconditional.original.height.posterior,'Enable'),'on')==1) (strcmp(get(controls.dsgevartools.forecast.unconditional.annual.height.posterior,'Enable'),'on')==1)];
if isempty(IsOriginal)==1;
   if OriginalAnnual(2)==1;
      txt = 'Would you like to calculate predictive likelihoods for the original data or the annualized data of the DSGE-VAR models?';
      answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',550,'yes',CurrINI,{'Original' 'Annualized'});
      if strcmp(lower(answer),'original')==1;
         IsOriginal = 1;
      else;
         IsOriginal = 0;
      end;
   else;
      IsOriginal = 1;
   end;
end;
if IsOriginal==1;
   DataStr = 'Original';
else;
   DataStr = 'Annualized';
end;
%
if isempty(value)==1;
   [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataStr ' Predictive Likelihood - Posterior Distribution'],DSGEModel,controls,CurrINI);
end;
if (isempty(value)==1)|(value==0);
   SetLogText('Unable to load posterior data. Aborting calculation.');
   value = [];
   ChrrChain = [];
   PME = [];
   Lambda = [];
   return;
end;
lambda = Lambda(value);
%
% load the posterior draws
%
[PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
UsedDraws = size(PhiDraws,1);
[NumDraws,NumParam] = size(thetaPostSample);
if UsedDraws<100;
   clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
   SetLogText(strvcat('YADA requires at least 100 draws of the VAR parameters to compute the predictive',['likelihood. You currently only have ' int2str(UsedDraws) ' draws of the these parameters with lambda=' num2str(lambda) '.']));
   return;
end;
%
% check if the posterior mode innovation covariance matrix is positive definite
%
if IsPositiveDefinite(SigmaModeEpsilon)~=1;
   clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
   SetLogText(strvcat('The posterior mode estimate of the residual covariance matrix','is not positive semidefinite. Aborting calculation.'));
   return;
end;
%
% check if the number of VAR parameters matches the number of DSGE model parameters
%
if NumDraws~=UsedDraws;
   clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
   SetLogText(strvcat('The number of posterior draws for VAR and DSGE model parameters don''t match.','Aborting calculation.'));
   return;
end;
%
% we don't need the posterior draws of A0
%
clear('A0Draws','PostSample','thetaPostSample');
%
% determine string with 1's and 0's for the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% determine the historical sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% check how many time periods to use for predictive distributions
%
MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% check the maximum when we take NaNs into account
%
YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
%
% check which actuals should be used
%
if isempty(SelectedYData)==1;
   if isempty(DSGEModel.Actuals)==1;
      SelectedYData = 1;
      DataAction = 'ok';
      YDataStr = 'Actuals from original sample';
   else;
      YDataStr = 'Actuals from original sample';
      m = length(DSGEModel.Actuals);
      for i=1:m;
         YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
      end;
      [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Posterior Distribution','','',CurrINI);
      if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
         YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
      end;
   end;
else;
   DataAction = 'ok';
   YDataStr = 'Actuals from original sample';
   if isempty(DSGEModel.Actuals)==0;
      m = length(DSGEModel.Actuals);
      for i=1:m;
         YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
      end;
   end;
   if SelectedYData>1;
      YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
   end;
end;
%
% test if we should continue
%
if strcmp(lower(DataAction),'ok')==0;
   SetLogText('Aborting calculation.');
   Status = -1;
   return;
end;
MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
[MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
MaxFH = min(MaxFH,MaxPH);
YData = YData(:,1:MaxFH);
%
% matrix with exogenous variables
%
X = [];
if DSGEModel.k>0;
   if NumPredX<MaxFH;
      X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
   end;
   if NumPredX>0;
      X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
   end;
end;
%
% check if we should compute the marginal or the joint predictive likelihood
%
if isempty(IsMarginal)==1;
   txt = 'Would you like to compute the marginal predictive likelikehood or the joint predictive likelihood of DSGE-VARs?';
   answer = Query(txt,'question',400,'Predictive Likelihood - Posterior Distribution',500,'yes',CurrINI,{'Marginal' 'Joint'});
   if strcmp(lower(answer),'marginal')==1;
      IsMarginal = 1;
   else;
      IsMarginal = 0;
   end;
end;
%
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
%
% check the PredictionVariables
%
if isempty(PredictionVariables)==1;
   PredictionVariables = ones(1,DSGEModel.n);
   PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
   [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
   if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
      SetLogText('No prediction variables selected. Aborting calculation.');
      Status = -1;
      return;
   elseif strcmp(lower(Action),'ok')==0;
      SetLogText('Aborting calculation.');
      Status = -1;
      return;
   end;
end;
%
% check if the selected variables to predict can indeed be used.
%
if max(PredictionVariables)>DSGEModel.n;
   SetLogText('Prediction variables do not match. Aborting calculation.');
   return;
end;
DSGEModel.PredictionVariables = PredictionVariables;
%
% need to compute certain values
%
n = DSGEModel.n;
p = DSGEModel.BVARLags;
npk = (n*p)+DSGEModel.k;
%
% setup the sample
%
[y,Y,HSample,xx,XX] = GetDSGEVARData(DSGEModel);
%
% check if older calculations exists given that such may be used
%
tic;
if (RecomputePL==0)&(DSGEVARPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,1,lambda,PME,MaxFH,UsedDraws,CurrChain,controls)==1);
   DoneCalc = 1;
else;
   %
   % compute the predictive likelihood
   %
   DoneCalc = DSGEVARPredictiveLikelihood(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,npk,HSample,YData,X,SelectedYData,IsOriginal,1,IsMarginal,CurrChain,PME,DSGEModel,CurrINI,controls);
end;
clear('PhiDraws','SigmaDraws');
if DoneCalc==0;
   SetLogText('Aborting calculation.');
   toc;
   return;
end;
%
% we can now do the remaining modified harmonic mean estimates.
%
[LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd] = DSGEVARPredLikeMCI(SelectedYData,NumDraws,MaxFH,IsOriginal,IsMarginal,1,Weights,lambda,PME,CurrChain,DSGEModel,controls);
CompTime = toc;
if isempty(LogPredLike)==1;
   SetLogText('Calculating predictive likelihood failed.');
   return;
end;
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
%
PredictionSampleStrMat = CreateFullSample(DSGEModel);
PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
%
DrawsStr = get(controls.posterior.draws,'String');
TotalDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
BurninStr = get(controls.posterior.burnin,'String');
NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
%
% write the results to text file
%
ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr 'PredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(UsedDraws) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.txt']);
%
PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,[],[],[],[],PredictionSampleStrMat,'',YDataStr(SelectedYData,:),IsOriginal,IsMarginal,1,UsedDraws,NumBurnin,TotalDraws,lambda,CompTime,DSGEModel,CurrINI);
%
% save output also to a mat-file
%
MatFile = strrep(ResFile,'.txt','.mat');
YNames = DSGEModel.VariableNames;
save(MatFile,'LogPredLike','StdErr','PredVars','YNames','PredictionSampleStrMat','SelectedYData','YDataStr');
%
VarStr = StringTrim(vecstr(DSGEModel.VariableNames(PredVars{1},:),', '));
%
if IsOriginal==1;
   DataString = 'original data:';
else;
   DataString = 'annualized data:';
end;
%
if FileExist(ResFile)==1;
   ResultsStr = '';
   for i=1:length(LogPredLike);
      ResultsStr = strvcat(ResultsStr,sprintf('%2.0f: %0.4f (%0.4f)',i,LogPredLike(i),StdErr(i)));
   end;
   DivStr = MultiCharStr('-',size(ResultsStr,2));
   SetLogText(strvcat(['Computation time: ' SecondsToHoursStr(CompTime) ' (hrs:min:sec)'],' ',['DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags'],['Variables: ' VarStr(1,1:length(VarStr)-1)],' ',[MargStr ' predictive likelihood for ' DataString],DivStr,ResultsStr,' ','Output written to:',['Text file: ' GetFilename(ResFile)],['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(ResFile)],' '));
   Status = 1;
end;

%
% end of SequentialPredictiveLikelihoodDSGEVAR.m
%
