function [logPost,logPostT] = logConcPosteriorThetaDSGEVAR4Time(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCLC,OrderQZ,NumWorkers)
% logConcPosteriorThetaDSGEVAR4Time: Computes the the height of the log concentrated posterior density for the theta
%                                    parameters when the likelihood function of a VAR is used.
%
% USAGE:
%
%       [logPost,logPostT] = logConcPosteriorThetaDSGEVAR4Time(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions, ...
%                                                  thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk, ...
%                                                  GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCLC,...
%                                                  OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  theta (vector) with all parameters to be estimated.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  ParemeterBounds (matrix) of dimension NumParam x 2 with the lower and upper bounds of the parameters.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                             variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                             variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                             regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                             are located.
%
%                  logCLC (scalar) with the value of the constant term for the log of the concentrated
%                             posterior density.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and 0 otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: logPost (scalar) equal to minus the log height of the posterior.
%
% OPTIONAL OUTPUT: logPostT (vector) with the value of the log posterior ar different time periods.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 12, 2011.
%                        This version: August 30, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-07-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 12-06-2017: Took into account that the log-likelihood functions provide the Solution output variable,
%               and supports the system prior.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 30-08-2017: Added the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% check if the bounds are satisfied by theta
%
if (any(theta<ParameterBounds(:,1))==1)|(any(theta>ParameterBounds(:,2))==1);
   logPost = 1000000;
   if nargout==2;
      logPostT = (1000000/T)*ones(T,1);
   end;
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% Calculate the log-likelihood
%
if nargout==2;
   if isempty(logCLC)==0;
      [logLikeValue,logLikeT,Solution,mcode,status] = logConcLikelihoodDSGEVAR4Time(ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCLC,DSGEModel.Data.y,DSGEModel.Data.Y,OrderQZ,NumWorkers);
   else;
      %
      % lambda is infinite
      %
      [logLikeValue,logLikeT,Solution,mcode,status] = logLikelihoodDSGEVARInf4Time(ModelParameters,DSGEModel,AIMData,T,n,p,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,DSGEModel.Data.y,DSGEModel.Data.Y,OrderQZ,NumWorkers);
   end;
else;
   if isempty(logCLC)==0;
      [logLikeValue,Solution,mcode,status] = logConcLikelihoodDSGEVAR(ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCLC,OrderQZ,NumWorkers);
   else;
      %
      % lambda is infinite
      %
      [logLikeValue,Solution,mcode,status] = logLikelihoodDSGEVARInf(ModelParameters,DSGEModel,AIMData,T,n,p,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,OrderQZ,NumWorkers);
   end;
end;
%
% add things up and take into account that the function will be minimized
%
if (mcode==1)&(status==1);
   %
   % calculate the value of the log prior density at theta
   %
   logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
   if isnan(logPrior)==0;
      %
      % check if we should compute the system prior
      %
      if FileExist(DSGEModel.SystemPriorFile)==0;
         %
         % compute the function value
         %
         logPost = -(logPrior+logLikeValue);
         if nargout==2;
            logPostT = -(logPrior+vec(logLikeT));
         end;
      else;
         DSGEModel.Solution = Solution;
         eval(['logSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ);']);
         if isnan(logSP)==0;
            %
            % compute the function value
            %
            logPost = -(logPrior+logLikeValue+logSP);
            if nargout==2;
               logPostT = -(logPrior+logSP+vec(logLikeT));
            end;
         else;
            logPost = 1000000;
            if nargout==2;
               logPostT = (1000000/T)*ones(T,1);
            end;
         end;
      end;
   else;
      logPost = 1000000;
      if nargout==2;
         logPostT = (1000000/T)*ones(T,1);
      end;
   end;
else;
   logPost = 1000000;
   if nargout==2;
      logPostT = (1000000/T)*ones(T,1);
   end;
end;

%
% end of logConcPosteriorThetaDSGEVAR4Time.m
%
