function lnSP = SystemPriorFile(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ)
% SystemPriorFile: Computes the value of the log prior for system features. The input variables
%                  can be used to locate a solution of the model via the DSGEModel structure, or
%                  to solve the DSGE model with the "SolveDSGEModel" function in YADA.
%
% USAGE:
%
%       lnSP = SystemPriorFile(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ)
%
% REQUIRED INPUT:  theta (vector) with the original parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  AIMData (structure) with data collected by AIM about the model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero
%                                 otherwise.
%
% REQUIRED OUTPUT: lnSP (scalar) with the height of the log prior density of a feature conditional on
%                                 the model parameters.
%
% NOTE: This example function computes the population standard deviations of the observed variables and
%       computes the height of the inverted Gamma prior for the standard deviation of inflation. 
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 12, 2017.
%                        This version: June 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-05-2017: Updated the documentation.
%
% * 26-05-2017: Changed the default value of lnSP from NaN to the log of eps.
%
% * 22-06-2017: Changed the input variable "CurrINI" to "OrderQZ", since this is the only field
%               of that structure which is required.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

lnSP = log(eps);
if (isfield(DSGEModel,'Solution')==1)&(isfield(DSGEModel.Solution,'F')==1)&(isempty(DSGEModel.Solution.F)==0);
   F = DSGEModel.Solution.F;
   B0 = DSGEModel.Solution.B0;
   H = DSGEModel.Solution.H;
   R = DSGEModel.Solution.R;
else;
   [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,OrderQZ);
   if mcode~=1;
      return;
   end;
end;
[Sigma,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
if length(size(H))==2;
   YCov = (H'*Sigma*H)+R;
else;
   Ht = H(:,:,size(H,3));
   YCov = (Ht'*Sigma*Ht)+R;
end;
%
% locate the position of inflation among the observed variables
%
InfPos = loc(DSGEModel.VariableNames,'INFL');
%
% the value of the standard deviation of inflation
%
sigmaInf = sqrt(abs(YCov(InfPos,InfPos)));
%
% compute the height of the inverted Gamma distribution at sigmaInf
%
lnSP = logInvertedGammaPDF(sigmaInf,0.265,10);

%
% end of SystemPriorFile.m
%
