function [A,H,R] = LPTMeasurementEqFile(ModelParameters,StateVarNames,VariableNames,XVariableNames,n,r,k)
% LPTMeasurementEqFile: Shows how to set up the measurement equations for the Leeper-Plante-Traum model.
%
% USAGE:
%
%       [A,H,R] = LPTMeasurementEqFile(ModelParameters,StateVarNames,VariableNames,XVariableNames,n,r,k)
%
%
% REQUIRED INPUTS:  ModelParameters (structure) with fields given by all the model parameters
%                                   and the fields taking scalar values. All model parameters
%                                   includes the parameters to be estimated, the parameters that
%                                   are calibrated, and the parameters that are functions of
%                                   the above.
%
%                   StateVarNames (string matrix) with rows being the names of the variables
%                                   as they have been assigned in the AiM model file. This will
%                                   typically include all state variables, as well as the shocks
%                                   to the state variables (and possibly a constant).
%
%                   VariableNames (string matrix) where each row contains a name of an observed
%                                   variables.
%
%                   XVariableNames (string matrix) where each row contains a name of an exogenous
%                                   variable.
%
%                   n (integer) with the number of observed variables.
%
%                   r (integer) with the number of state variables.
%
%                   k (integer) with the number of exogenous variables.
%
% REQUIRED OUTPUTS: A (kxn) matrix with parameters on the exogenous variables. Notice that k=
%                                   number of exogenous variables, n=number of observed variables
%                                   in the measurement equation.
%
%                   H (rxn) matrix with parameters on the state variables. Here, r=number of state
%                                   variables.
%
%                   R (nxn) matrix with covariances for the measurement errors. This matrix has
%                                   to be positive semi-definite!
%
%
% NOTICE: The measurement equations are expressed as:
%
%             y(t) = A'*x(t) + H'*xi(t) + w(t)
%
%         where y (nx1) is the vector of observed variable, x (kx1) is the vector of exogenous
%         variables, xi (rx1) the vector of state variables, and w (nx1) the vector of measurement
%         errors, with covariance matrix R.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 15, 2015.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-07-2015: Updated the documentation.
%
% * 17-07-2015: Fixed the links between the tax revenue observed variables and the
%               model tax rate variables.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% the model doesn't have any deterministic variables
%
A = [];
%
% The parameters on the state variables.
%
% NOTICE: There are no actual parameters from the ModelParameters structure being used here.
%         However, the StateVarNames string along with the loc functions is used to locate
%         the positions of the state variables. These positions correspond to the relevant
%         rows in the H matrix.
%
H = zeros(r,n);
H(loc(StateVarNames,'chat'),loc(VariableNames,'lnC')) = 1;
H(loc(StateVarNames,'ihat'),loc(VariableNames,'lnI')) = 1;
H(loc(StateVarNames,'lhat'),loc(VariableNames,'lnH')) = 1;
H(loc(StateVarNames,'bhat'),loc(VariableNames,'lnB')) = 1;
H(loc(StateVarNames,'ghat'),loc(VariableNames,'lnG')) = 1;
H(loc(StateVarNames,'taukhat'),loc(VariableNames,'lnTk')) = 1;
H(loc(StateVarNames,'yhat'),loc(VariableNames,'lnTk')) = 1;
H(loc(StateVarNames,'taulhat'),loc(VariableNames,'lnTl')) = 1;
H(loc(StateVarNames,'yhat'),loc(VariableNames,'lnTl')) = 1;
H(loc(StateVarNames,'tauchat'),loc(VariableNames,'lnTc')) = 1;
H(loc(StateVarNames,'chat'),loc(VariableNames,'lnTc')) = 1;
H(loc(StateVarNames,'zhat'),loc(VariableNames,'lnZ')) = 1;
%
% The covariance matrix for the measurement errors. Here, we have no measurement errors!
%
R = zeros(n,n);

%
% end of LPTMeasurementEqFile.m
%
