function [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,X,K1,K2,A,H,R,F,B0)
% CondPredictionKalmanSmootherHt: Calculates smoothed predictions of the state variables and their covariance
%                                 matrix using conditioning assumptions.
%
% USAGE:
%
%       [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,X,K1,K2,A,H,R,F,B0)
%
% REQUIRED INPUT:  KsitT (matrix) of dimension rxg with smoothed values of the states at time
%                        T-g+1 until T.
%
%                  PtT (matrix) of dimension rxrxg with smoothed values of the state covariance at
%                        times T-g+1 until T.
%
%                  wtT (matrix) of dimension nxg with smooth values of the measurement errors
%                        at time T-g+1 until T.
%
%                  Z (matrix) of dimension mxg with the conditioning assumptions.
%
%                  U (matrix) of dimension mxg with initial conditions for the conditioning.
%
%                  X (matrix) of dimension kxg with ecogenous variables.
%
%                  K1 (matrix) of dimension nxm for mapping the observed variables into
%                        the conditioning data.
%
%                  K2 (matrix) of dimension nxm for mapping lags of the observed variables into
%                        the conditioning data. Matrix may be 3-dimensional where the 3rd
%                        allows for a lag-varying K2.
%
%                  A (matrix) of dimension kxn with the parameters on the exogenous variables in
%                        the measurement equation.
%
%                  H (matrix) of dimension rxn with parameters on the state variabels in the
%                        measurement equation.
%
%                  R (matrix) of dimension nxn with the covariances of the measurement errors
%                        in the measurement equation.
%
%                  F (matrix) of dimension rxr with state transitions parameters for the state
%                        equation.
%
%                  B0 (matrix) of dimension rxq with parameters on the economic shocks in the
%                        state equation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 15, 2008.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-07-2008: Fixed a bug where the third dimension of H was not measured from the size
%               function.
%
% * 18-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 04-01-2011: Updated the documentation.
%
% * 17-05-2011: Speeded up the code thanks to Ingvar Strid's suggestions. That is, the function now uses
%               the De Jong algorithm for the smoother. Fixed a bug concerning the first value of Zpred.
%
% * 02-01-2012: Updated the documentation.
%
% * 13-01-2012: Support for missing observations.
%
% * 16-01-2012: Fixed an error using pos in ZError. Function supports 3-dimensional K2 matrix.
%
% * 17-01-2012: The default value for lag-varying K2 matrices is zero so that missing lags are
%               replaced with this value.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
KsiTTz = [];
PTTz = [];
%
% some important dimensional parameters
%
k = size(X,1);
[r,n,Hg] = size(H);
q = size(B0,2);
[m,g] = size(Z);
%
% some parameter matrices that are needed
%
% - FTIlde and QTilde for the stacked state equation
% - KStar for mapping the observed variables into the conditioning assumptions
% - HStar is used when re-defining the measurement errors as state variables
%
FStar = [F zeros(r,n);zeros(n,r+n)];
QStar = [B0*B0' zeros(r,n);zeros(n,r) R];
for t=1:size(H,3);
   HStar(:,:,t) = [H(:,:,t);eye(n)];
end;
%
if g>1;
   if length(size(K2))==2;
      KStar = [K1;kron(ones(g-1,1),K2)];
   else;
      KStar = K1;
      for j=1:g-1;
         if j<=size(K2,3);
            KStar = [KStar;K2(:,:,j)];
         else;
            KStar = [KStar;zeros(size(K1))];
         end;
      end;
   end;
   FTilde = [FStar zeros(r+n,(g-1)*(r+n));eye((g-1)*(r+n)) zeros((g-1)*(r+n),r+n)];
   QTilde = [QStar zeros(r+n,(g-1)*(r+n));zeros((g-1)*(r+n),g*(r+n))];
else;
   KStar = K1;
   FTilde = FStar;
   QTilde = QStar;
end;
HTilde = zeros((n+r)*g,size(KStar,2),g);
for i=1:g;
   HHelpTilde = zeros((n+r)*g,n*g);
   for j=1:g;
      HHelpTilde(1+((n+r)*(j-1)):(n+r)*j,1+(n*(j-1)):n*j) = HStar(:,:,g+(i-1)-(j-1));
   end;
   HTilde(:,:,i) = HHelpTilde*KStar;
end;
%
% parameters on exogenous variables
%
if k>0;
   Xmat = zeros(n*g,g);
   for i=1:g;
      for j=1:g;
         Xmat(1+((j-1)*n):j*n,i) = A'*X(:,g+i-j);
      end;
   end;
   ATilde = [KStar;eye(m)];
   XStar = ATilde'*[Xmat;U];
else;
   XStar = U;
end;
%
% make sure that missing obs for measurement errors are zero
%
wtT(isnan(wtT)) = 0;
%
% set up the initial values
%
KsiTTStar = zeros(g*(r+n),1);
PTTTilde = zeros(g*(r+n),g*(r+n));
for i=1:g;
   KsiTTStar(1+((i-1)*(r+n)):i*(r+n),1) = [KsitT(:,g-i+1);wtT(:,g-i+1)];
   PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((i-1)*(r+n)):i*(r+n)) = [PtT(:,:,g-i+1) zeros(r,n);zeros(n,r) R];
end;
%
% dealing with off-diagonal blocks
%
for i=1:g;
   Fpower = FStar;
   for j=i+1:g;
      PMat = Fpower*PTTTilde(1+((j-i-1)*(r+n)):(j-i)*(r+n),1+((j-i-1)*(r+n)):(j-i)*(r+n));
      PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((j-1)*(r+n)):j*(r+n)) = PMat;
      PTTTilde(1+((j-1)*(r+n)):j*(r+n),1+((i-1)*(r+n)):i*(r+n)) = PMat';
      Fpower = Fpower*FStar;
   end;
end;
%
% initialize forecast, update, and smoother matrices
%
KsiStarTT1 = zeros(g*(r+n),g);
KsiStarTT = KsiStarTT1;
KsiStarTTh = KsiStarTT1;
PTildeTT1 = zeros(g*(r+n),g*(r+n),g);
PTildeTT = PTildeTT1;
PTildeTTh = PTildeTT1;
Zpred = zeros(m,g);
%
% update the initil conditions
%
KsiStarTT1(:,1) = FTilde*KsiTTStar;
PTildeTT1(:,:,1) = (FTilde*PTTTilde*FTilde')+QTilde;
Zpred(:,1) = XStar(:,1)+(HTilde(:,:,1)'*KsiStarTT1(:,1));
%
% run the Kalman forecast and update parts
%
for t=2:g;
   pos = ~isnan(Z(:,t-1));
   n1 = sum(pos);
   if n1>0;
      InvMSEZ = eye(n1)/(HTilde(:,pos,t-1)'*PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1));
      ZError = Z(pos,t-1)-Zpred(pos,t-1);
      KsiStarTT(:,t-1) = KsiStarTT1(:,t-1)+(PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ*ZError);
      PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1)-(PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ*HTilde(:,pos,t-1)'*PTildeTT1(:,:,t-1));
      %
      % calculate the Kalman gain
      %
      KalmanGain = FTilde*PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ;
      %
      % the state vector
      %
      KsiStarTT1(:,t) = (FTilde*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
      FTildeDev = FTilde-(KalmanGain*HTilde(:,pos,t-1)');
      PTildeTT1(:,:,t) = (FTildeDev*PTildeTT1(:,:,t-1)*FTildeDev')+QTilde;
      Zpred(:,t) = XStar(:,t)+(HTilde(:,:,t)'*KsiStarTT1(:,t));
   else;
      KsiStarTT(:,t-1) = KsiStarTT1(:,t-1);
      PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1);
      KsiStarTT1(:,t) = FTilde*KsiStarTT1(:,t-1);
      PTildeTT1(:,:,t) = (FTilde*PTildeTT1(:,:,t-1)*FTilde')+QTilde;
      Zpred(:,t) = XStar(:,t)+(HTilde(:,:,t)'*KsiStarTT1(:,t));
   end;
end;
pos = ~isnan(Z(:,g));
n1 = sum(pos);
if n1>0;
   InvMSEZ = eye(n1)/(HTilde(:,pos,g)'*PTildeTT1(:,:,g)*HTilde(:,pos,g));
   ZError = Z(pos,g)-Zpred(pos,g);
   rg1g = HTilde(:,pos,g)*InvMSEZ*ZError;
   Ng1g = HTilde(:,pos,g)*InvMSEZ*HTilde(:,pos,g)';
else;
   rg1g = 0;
   Ng1g = 0;
end;
KsiStarg1g = KsiStarTT1(:,g)+(PTildeTT1(:,:,g)*rg1g);
PStarg1g = PTildeTT1(:,:,g)-(PTildeTT1(:,:,g)*Ng1g*PTildeTT1(:,:,g));
KsiTTz = KsiStarg1g(1+((g-1)*(r+n)):r+((g-1)*(r+n)));
PTTz = PStarg1g(1+((g-1)*(r+n)):r+((g-1)*(r+n)),1+((g-1)*(r+n)):r+((g-1)*(r+n)));

%
% end of CondPredictionKalmanSmootherHt.m
%
