function [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,FirstPeriod,LastPeriod,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos,AllowNaNs,KalmanAlgorithm)
% RunKalmanFilter: Generic function for running Kalman filter functions under a wide variety of circumstances.
%
% USAGE:
%
%       [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,FirstPeriod,...
%                                                      LastPeriod,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,...
%                                                      AllowUnitRoot,c,StationaryPos,AllowNaNs,KalmanAlgorithm)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT1 with observed data.
%
%                  X (matrix) of dimension kxT1 matrix with exogenous of predetermined data.
%
%                  FirstPeriod (integer) with the first period to use for the data.
%
%                  LastPeriod (integer) with the last period to use for the data.
%
%                  A (matrix) of dimension kxn with parameter values on the k exogenous variables in the measurement equation.
%
%                  H (matrix) of dimension rxn (or rxnxT) with parameter values on the r state variables (ksi) in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  B0 (matrix) of dimension rxq with parameters on the standardized innovations in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  KsiLast (vector) of dimension rx1 with initial values for the state vector.
%
%                  initP (integer) indicating method of determining initial value for P(1|0), the MSE matrix for
%                        ksi(1|0). If initP=1 (default), the P(1|0) is given by the unconditional covariance matrix of ksi,
%                        calculated through the vec operator. Since this operation can be excessively slow for large systems
%                        (when r^2 is a large number) the Doubling Algorthm can be chosen to approximate the unconditional
%                        variance. This algorithm is called if initP=2. If initP=3, then P(1|0) = c*eye(r). Finally, if
%                        initP=4, then P(1|0) = c*eye(r) with c -> infinity.
%                  NOTE: If F has eigenvalues outside the unit circle, then initP is always set to 3!
%
%                  MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                        divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                  StartPeriod (integer) indicating first observation to use when calculating the log-likelihood value. The
%                        default value is 1.
%
%                  AllowUnitRoot (boolean) which is 1 if unit roots should be allowed for 0 otherwise.
%
%                  c (positive real number) indicating the variance of P(1|0) when initP=3. The default value is 1000.
%
%                  StationaryPos (vector) with positions in the state vector of the stationary variables.
%
%                  AllowNaNs (boolean) that takes the value 1 if NaNs among the observed variables should be allowed
%                        or not.
%
%                  KalmanAlgorithm (integer) that takes the value 1 if the standard Kalman filter should be used,
%                        2 if the square-root filter should be applied, and 3 if the univariate filter should be run.
%
% REQUIRED OUTPUT: lnL (scalar) with the value of the log-likelihood function.
%
%                  status (boolean) indicating the covariance stationarity of the state variables. If all the
%                        eigenvalues of F are inside the unit circle, then status=0 and the log-likelihood function
%                        is calculated. If not, then status=1 and initP is set to 3. In addition, status can be
%                        -1 if ths forecast covariance matrix is not positive definite for some time period.
%
% OPTIONAL OUTPUT: lnLt (vector) of dimension 1xT with the time t values of the log-densities that sum to the value lnL.
%
%                  Yerror (matrix) of dimension nxT with the 1-step ahead forecast errors of the observed variables.
%
%                  Yhat (matrix) of dimension nxT with the predictions [y(StartPeriod|StartPeriod-1) ... y(T|T-1).
%
%                  SigmaSqRoot (matrix) of dimension nxnxT with the lower traingular square root of the inverse of
%                        the 1-step ahead forecast error covariance matrix of the observed variables.
%
%                  MSEY (matrix) of dimension nxnxT with the mean squared error matrix for y(t|t-1).
%
%                  InvMSEY (matrix) of dimension nxnxT with the inverted 1-step ahead forecast error covariances of
%                        the observed variables.
%
%                  KalmanGain (matrix) of dimension rxnxT with the Kalman gain estimates from the square root filter.
%
%                  Ksitt1 (matrix) of dimension rxT with the state variable predictions.
%
%                  Ptt1 (matrix) of dimension rxrxT with covariances for the state variable prediction errors.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 10, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-03-2011: Added support for "KalmanFilterMO".
%
% * 14-03-2011: Added support for all missing observations Kalman filtering functions.
%
% * 19-05-2011: Made sure that "Yerror", etc, have the correct number of elements also
%               when the standard Kalman filter algorithm is applied.
%
% * 04-10-2011: Started adding support for the univariate Kalman filter.
%
% * 05-10-2011: Took into account that the univariate Kalman filter provides 4 output variables
%               with cell arrays with vectors and matrices. The names of the output variables
%               have been selected to match those from the square root filter.
%
% * 06-10-2011: Added more code for running the univariate Kalman filters.
%
% * 11-10-2011: Continued adding code for the univariate Kalman filter.
%
% * 12-10-2011: Finished adding code for univariate Kalman filters.
%
% * 17-10-2011: Started work on allowing for diffuse initialization of the Kalman filter.
%
% * 18-10-2011: Continued work on diffuse initialization.
%
% * 20-10-2011: Added more code for using diffuse initialization.
%
% * 21-10-2011: Added further code for making use of diffuse initialization.
%
% * 24-10-2011: Added yet even more code for using diffuse initialization with the Kalman filter.
%
% * 25-10-2011: Finished adding code for diffuse initialization.
%
% * 03-01-2012: Updated the documentation.
%
% * 15-11-2012: Added code for using the Chandrasekhar recursions.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

Y = Y(:,FirstPeriod:LastPeriod);
if isempty(X)==0;
   X = X(:,FirstPeriod:LastPeriod);
end;
if length(size(H))==3;
   H = H(:,:,FirstPeriod:LastPeriod);
end;
Q = B0*B0';
r = size(B0,1);
r1 = length(StationaryPos);
%
% check if we're only here to calculate the log-likelihood function
%
if initP<=3;
   if nargout==2;
      %
      % check if we have missing observations or not
      %
      if AllowNaNs==0;
         if length(size(H))==2;
            if KalmanAlgorithm==1;
               if r==r1;
                  [lnL,status] = KalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
               else;
                  [lnL,status] = UnitRootKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
               end;
            elseif KalmanAlgorithm==2;
               if r==r1;
                  [lnL,status] = SquareRootKalmanFilter(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
               else;
                  [lnL,status] = UnitRootSquareRootKalmanFilter(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
               end;
            elseif KalmanAlgorithm==3;
               if r==r1;
                  [lnL,status] = UnivariateKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
               else;
                  [lnL,status] = UnitRootUnivariateKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
               end;
            elseif KalmanAlgorithm==4;
               [lnL,status] = ChandrasekharRecursions(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod);
            end;
         else;
            if KalmanAlgorithm==1;
               if r==r1;
                  [lnL,status] = KalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
               else;
                  [lnL,status] = UnitRootKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
               end;
            elseif KalmanAlgorithm==2;
               if r==r1;
                  [lnL,status] = SquareRootKalmanFilterHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
               else;
                  [lnL,status] = UnitRootSquareRootKalmanFilterHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
               end;
            elseif KalmanAlgorithm==3;
               if r==r1;
                  [lnL,status] = UnivariateKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
               else;
                  [lnL,status] = UnitRootUnivariateKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
               end;
            end;
         end;
         return;
      end;
      %
      % missing observations code
      %
      if length(size(H))==2;
         if KalmanAlgorithm==1;
            if r==r1;
               [lnL,status] = KalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status] = UnitRootKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==2;
            if r==r1;
               [lnL,status] = SquareRootKalmanFilterMO(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status] = UnitRootSquareRootKalmanFilterMO(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==3;
            if r==r1;
               [lnL,status] = UnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status] = UnitRootUnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         end;
      else;
         if KalmanAlgorithm==1;
            if r==r1;
               [lnL,status] = KalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status] = UnitRootKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==2;
            if r==r1;
               [lnL,status] = SquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status] = UnitRootSquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==3;
            if r==r1;
               [lnL,status] = UnivariateKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status] = UnitRootUnivariateKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         end;
      end;
      return;
   end;
   %
   % initialize some possibly not provided output arguments
   %
   Yerror = [];
   SigmaSqRoot = [];
   InvMSEY = [];
   KalmanGain = [];
   [n,T] = size(Y);
   %
   % we should compute all the outputs
   %
   if AllowNaNs==0;
      if length(size(H))==2;
         if KalmanAlgorithm==1;
            if r==r1;
               [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = KalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = UnitRootKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
            Yerror = Y(:,StartPeriod:T)-Yhat;
            SigmaSqRoot = zeros(n,n,T-StartPeriod+1);
            InvMSEY = SigmaSqRoot;
            KalmanGain = zeros(r,n,T-StartPeriod+1);
         elseif KalmanAlgorithm==2;
            if r==r1;
               [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = SquareRootKalmanFilter(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = UnitRootSquareRootKalmanFilter(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==3;
            if r==r1;
               [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnivariateKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnitRootUnivariateKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==4;
            [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = ChandrasekharRecursions(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod);
            Yerror = Y(:,StartPeriod:T)-Yhat;
            SigmaSqRoot = zeros(n,n,T-StartPeriod+1);
            InvMSEY = SigmaSqRoot;
            KalmanGain = zeros(r,n,T-StartPeriod+1);
         end;
      else;
         if KalmanAlgorithm==1;
            if r==r1;
               [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = KalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = UnitRootKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
            Yerror = Y(:,StartPeriod:T)-Yhat;
            SigmaSqRoot = zeros(n,n,T-StartPeriod+1);
            InvMSEY = SigmaSqRoot;
            KalmanGain = zeros(r,n,T-StartPeriod+1);
         elseif KalmanAlgorithm==2;
            if r==r1;
               [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = SquareRootKalmanFilterHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = UnitRootSquareRootKalmanFilterHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         elseif KalmanAlgorithm==3;
            if r==r1;
               [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnivariateKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
            else;
               [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnitRootUnivariateKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
            end;
         end;
      end;
      return;
   end;
   %
   % missing observations code
   %
   if length(size(H))==2;
      if KalmanAlgorithm==1;
         if r==r1;
            [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = KalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
         else;
            [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = UnitRootKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
         end;
         Yerror = Y(:,StartPeriod:T)-Yhat;
         SigmaSqRoot = zeros(n,n,T-StartPeriod+1);
         InvMSEY = SigmaSqRoot;
         KalmanGain = zeros(r,n,T-StartPeriod+1);
      elseif KalmanAlgorithm==2;
         if r==r1;
            [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = SquareRootKalmanFilterMO(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
         else;
            [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = UnitRootSquareRootKalmanFilterMO(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
         end;
      elseif KalmanAlgorithm==3;
         if r==r1;
            [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
         else;
            [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnitRootUnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
         end;
      end;
   else;
      if KalmanAlgorithm==1;
         if r==r1;
            [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = KalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
         else;
            [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = UnitRootKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
         end;
         Yerror = Y(:,StartPeriod:T)-Yhat;
         SigmaSqRoot = zeros(n,n,T-StartPeriod+1);
         InvMSEY = SigmaSqRoot;
         KalmanGain = zeros(r,n,T-StartPeriod+1);
      elseif KalmanAlgorithm==2;
         if r==r1;
            [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = SquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
         else;
            [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = UnitRootSquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
         end;
      elseif KalmanAlgorithm==3;
         if r==r1;
            [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnivariateKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c);
         else;
            [lnL,status,lnLt,Yhat,MSEY,SigmaSqRoot,KalmanGain,Yerror,InvMSEY,Ksitt1,Ptt1] = UnitRootUnivariateKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c,StationaryPos);
         end;
      end;
   end;
   return;
end;
%
% diffuse initialization
%
if nargout==2;
   %
   % check if we have missing observations or not
   %
   if AllowNaNs==0;
      if length(size(H))==2;
         if KalmanAlgorithm==1;
            [lnL,status] = DiffuseKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
         elseif KalmanAlgorithm==2;
            [lnL,status] = DiffuseSquareRootKalmanFilter(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
         elseif KalmanAlgorithm==3;
            [lnL,status] = DiffuseUnivariateKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
         end;
      else;
         if KalmanAlgorithm==1;
            [lnL,status] = DiffuseKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
         elseif KalmanAlgorithm==2;
            [lnL,status] = DiffuseSquareRootKalmanFilterHt(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
         elseif KalmanAlgorithm==3;
            [lnL,status] = DiffuseUnivariateKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
         end;
      end;
      return;
   end;
   %
   % missing observations code
   %
   if length(size(H))==2;
      if KalmanAlgorithm==1;
         [lnL,status] = DiffuseKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==2;
         [lnL,status] = DiffuseSquareRootKalmanFilterMO(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==3;
         [lnL,status] = DiffuseUnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      end;
   else;
      if KalmanAlgorithm==1;
         [lnL,status] = DiffuseKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==2;
         [lnL,status] = DiffuseSquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==3;
         [lnL,status] = DiffuseUnivariateKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      end;
   end;
   return;
end;
%
% initialize some possibly not provided output arguments
%
Yerror = [];
InvMSEY = [];
%
% we should compute all the outputs
%
if AllowNaNs==0;
   if length(size(H))==2;
      if KalmanAlgorithm==1;
         [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==2;
         [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseSquareRootKalmanFilter(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==3;
         [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseUnivariateKalmanFilter(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      end;
   else;
      if KalmanAlgorithm==1;
         [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==2;
         [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseSquareRootKalmanFilterHt(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      elseif KalmanAlgorithm==3;
         [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseUnivariateKalmanFilterHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
      end;
   end;
   return;
end;
%
% missing observations code
%
if length(size(H))==2;
   if KalmanAlgorithm==1;
      [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
   elseif KalmanAlgorithm==2;
      [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseSquareRootKalmanFilterMO(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
   elseif KalmanAlgorithm==3;
      [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseUnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
   end;
else;
   if KalmanAlgorithm==1;
      [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
   elseif KalmanAlgorithm==2;
      [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseSquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
   elseif KalmanAlgorithm==3;
      [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaSqRoot,KalmanGain] = DiffuseUnivariateKalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos);
   end;
end;

%
% end of RunKalmanFilter.m
%
