function [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat,rtt,rtT] = SquareRootSmootherMOHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0)
% SquareRootSmootherMOHt: Computes the update and smooth estimates of the state variables and their covariances
%                         using a square root filter. Allows for missing observations and a time-varying
%                         matrix with coefficients on the state variables in the measurement equation.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat,rtt,rtT] = SquareRootSmootherMOHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,...
%                                                                                 Ksitt1,Ptt1,H,F,R,B0)
%
% REQUIRED INPUT:  Yerror (matrix) of dimension nxT with the 1-step ahead forecast errors of the observed variables.
%
%                  SigmaSqRoot (matrix) of dimension nxnx(T-StartPeriod+1) with the lower traingular square root
%                              of the inverse of the 1-step ahead forecast error covariance matrix of the observed
%                              variables.
%
%                  InvMSEY (matrix) of dimension nxnxT with the inverse of the 1-step ahead forecast error
%                              covariance matrix, computed from the square root filter.
%
%                  KalmanGain (matrix) of dimension rxnxT with the Kalman gains matrix from the square root filter.
%
%                  Ksitt1 (matrix) of dimension rxT with the 1-step ahead forecasts of the state variables.
%
%                  Ptt1 (matrix) of dimension rxrxT with the covariance matrix of the 1-step ahead forecasts
%                              of the state variables.
%
%                  H (matrix) of dimension rxnxT with parameter values on the r state variables in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
% REQUIRED OUTPUT: Ksitt (matrix) of dimension rxT with the updated predictions of the state vector.
%
%                  Ptt (matrix) of dimension rxrxT with the covariances of the updated predictions of the state vector.
%
%                  KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the covariances of the smooth predictions of the state vector.
%
%                  Ksit1t (rxT) matrix with 1-step smooth predictions of F*ksi(t-1) conditional on time t data.
%
% OPTIONAL OUTPUT: rtvec (vector) of dimension rx1 with the r(t)-vector for period 1.
%
%                  NtMat (matrix) of dimension rxrxT with the Nt matrices.
%
%                  rtt (matrix) of dimension rxT with innovations for the Kalman updater.
%
%                  rtT (matrix) of dimension rxT with innovations for the Kalman smoother.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 15, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-03-2011: Updated the documentation.
%
% * 24-03-2011: Fixed a missing transpose character on HT.
%
% * 14-10-2011: Added the optional output variables "rtt" and "rtT".
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% determine some dimentions
%
[r,T] = size(Ksitt1);
Q = B0*B0';
%
% Initializa the output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
Ksit1t = zeros(r,T);
NtMatTmp = zeros(r,r,T);
rttTmp = zeros(r,T);
rtTTmp = zeros(r,T);
%
% we loop from period T and backwards to t=1
%
for t=(T:-1:1);
   %
   % check for missing observations
   %
   pos = (~isnan(Yerror(:,t)));
   n1 = sum(pos);
   if n1>0;
      HT = H(:,:,t)';
      Ptt1H = Ptt1(:,:,t)*H(:,pos,t);
      %
      % calculate the updated state prediction and its MSE
      %
      Ksitt(:,t) = Ksitt1(:,t)+(Ptt1H*InvMSEY(pos,pos,t)*Yerror(pos,t));
      Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1H*InvMSEY(pos,pos,t)*Ptt1H');
      Ksit1t(:,t) = Ksitt1(:,t)+((Ptt1(:,:,t)-Q)*H(:,pos,t)*InvMSEY(pos,pos,t)*Yerror(pos,t));
      ut = H(:,pos,t)*InvMSEY(pos,pos,t)*Yerror(pos,t);
      if t<T;
         Lt = F-(KalmanGain(:,pos,t)*HT(pos,:));
         rt = (H(:,pos,t)*InvMSEY(pos,pos,t)*Yerror(pos,t))+(Lt'*rt);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
         Ntilde = [H(:,pos,t)*SigmaSqRoot(pos,pos,t)' Lt'*Nstar];
         [G,NS] = qr(Ntilde');
         Nstar = NS(1:r,:)';
         Nt = Nstar*Nstar';
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
      else;
         %
         % Updates and smooth predictions and MSEs have the same values for t=T
         %
         KsitT(:,T) = Ksitt(:,T);
         PtT(:,:,T) = Ptt(:,:,T);
         rt = H(:,pos,t)*InvMSEY(pos,pos,T)*Yerror(pos,T);
         Ntilde = [H(:,pos,t)*SigmaSqRoot(pos,pos,T)' zeros(r,r)];
         [G,NS] = qr(Ntilde');
         Nstar = NS(1:r,:)';
         Nt = Nstar*Nstar';
      end;
   else;
      Ksitt(:,t) = Ksitt1(:,t);
      Ptt(:,:,t) = Ptt1(:,:,t);
      Ksit1t(:,t) = Ksitt1(:,t);
      ut = zeros(r,1);
      if t<T;
         rt = F'*rt;
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
         Ntilde = F'*Nstar;
         [G,NS] = qr(Ntilde');
         Nstar = NS(1:r,:)';
         Nt = Nstar*Nstar';
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
      else;
         KsitT(:,T) = Ksitt(:,T);
         PtT(:,:,T) = Ptt(:,:,T);
         rt = zeros(r,1);
         Nstar = zeros(r,r);
         Nt = Nstar;
      end;
   end;
   NtMatTmp(:,:,t) = Nt;
   rttTmp(:,t) = ut;
   rtTTmp(:,t) = rt;
   %
   % make sure that the diagonal of Ptt and PtT are non-negative
   %
   Ptt(:,:,t) = diag(abs(diag(Ptt(:,:,t))))+tril(Ptt(:,:,t),-1)+tril(Ptt(:,:,t),-1)';
   PtT(:,:,t) = diag(abs(diag(PtT(:,:,t))))+tril(PtT(:,:,t),-1)+tril(PtT(:,:,t),-1)';
end;
if nargout>=6;
   rtvec = rt;
   if nargout>=7;
      NtMat = NtMatTmp;
      if nargout==9;
         rtt = rttTmp;
         rtT = rtTTmp;
      end;
   end;
end;

%
% end of SquareRootSmootherMOHt.m
%
