function [XMargPrior,YMargPrior,BoundsMatrix,XSystemPrior,YSystemPrior] = GraphSystemPriorDensity(DSGEModel,CurrINI)
% GraphSystemPriorDensity: Creates plots of the marginal and system prior densities for all original parameters.
%                          When the output arguments are listed, then the function simply resturns matrices with
%                          x-axis and y-axis values for the densities.
%
% USAGE:
%
%       [XMargPrior,YMargPrior,BoundsMatrix,XSystemPrior,YSystemPrior] = GraphSystemPriorDensity(DSGEModel,CurrINI)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  CurrINI (structure), with initialization information.
%
% OPTIONAL OUTPUT: XMargPrior (matrix) of dimension NumParamxNumPoints. It contains the x-axis data for all
%                            the marginal prior densities.
%
%                  YMargPrior (matrix) of dimension NumParamxNumPoints. It contains the y-axis data for all
%                            the marginal prior densities.
%
%                  BoundsMatrix (matrix) of dimension NumParamx2. It contains information about possible
%                            upper and lower bounds of the parameters.
%
%                  XSystemPrior (matrix) of dimension NumParamxNumSPoints. It contains the x-axis data for all
%                            the prior densities conditional on the system prior data.
%
%                  YSystemPrior (matrix) of dimension NumParamxNumSPoints. It contains the y-axis data for all
%                            the prior densities conditional on the system prior data.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 1, 2017.
%                        This version: June 2, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-06-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargout==5;
   XMargPrior = [];
   YMargPrior = [];
   BoundsMatrix = [];
   XSystemPrior = [];
   YSystemPrior = [];
end;
%
% check if prior draws already exist
%
ErrorStr = '';
PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
if FileExist(PriorDrawsFile)==1;
   %
   % check if the file contains the correct information
   %
   PriorDrawsData = load(PriorDrawsFile);
   if isfield(PriorDrawsData,'LogMargLikeSystemLaplace')==0;
      ErrorStr = 'The prior draws file exist, but has not been saved with the data from having a system prior. You must run prior sampling before YADA can graph the prior densities when these include a system prior.';
   end;
else;
   ErrorStr = 'The prior draws file doesn''t exist. You must run prior sampling before YADA can graph the prior densities when these include a system prior.';
end;
%
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','Graph Prior Densities',300,500,CurrINI);
   drawnow;
   return;
end;
%
% plot the prior draws
%
thetaDraws = PriorDrawsData.thetaDraws;
if nargout==0;
   GraphPosteriorDensity(thetaDraws',0,[],[],[],[],[],[],[],DSGEModel,CurrINI);
else;
   [thetaMargMode,UpdateMargMode,XMargPrior,YMargPrior,BoundsMatrix,XSystemPrior,YSystemPrior] = GraphPosteriorDensity(thetaDraws',0,[],[],[],[],[],[],[],DSGEModel,CurrINI);
end;

%
% end of GraphSystemPriorDensity.m
%
