function DSGEModel = ImportYADASettings(DSGEModel,NewDSGEModel,controls)
% ImportYADASettings: Changes certains settings in DSGEModel based on NewDSGEModel settings.
%
% USAGE:
%
%       DSGEModel = ImportYADASettings(DSGEModel,NewDSGEModel,controls)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  NewDSGEModel (structure) with perhaps different settings than those
%                            found in DSGEModel.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 8, 2009.
%                        This version: September 14, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-11-2009: Took the field "PosteriorSampler" into account.
%
% * 06-01-2010: Updated the documentation.
%
% * 14-12-2010: Took the fields "CsminwelExtraRuns", "KalmanAlgorithm", and "ModelSolver" into account.
%
% * 04-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 05-07-2011: Made sure that the function supported valid "MaximizeAlgorithmValue" entries.
%
% * 07-07-2011: Changed the upper bound for testing valid optimizers from 2 to 4 since newrat
%               has been added.
%
% * 12-07-2011: Changed the upper bound for "MaximizeAlgorithmValue" to 6 now that gmhmaxlike
%               is on board.
%
% * 04-01-2012: Updated the documentation.
%
% * 09-02-2012: Added the field "StationaryVAR".
%
% * 24-08-2012: Added the field "ConfidenceRegionMethod".
%
% * 02-01-2013: Updated the documentation.
%
% * 17-07-2013: The function now fully supports the Waggoner-Zha approach to conditional forecasting.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: Took the fields "StudenttDegFree" and "FixedNumParamBlocks" into account.
%
% * 13-04-2017: Took the field "BlockSize" into account.
%
% * 10-08-2017: Took the fields "TemperingStagesValue" and "TemperingLambdaValue" into account.
%
% * 11-08-2017: Took the fields "NumParamBlocks", "MixedDistWeightValue", "SMCInitialScaleFactor",
%               "TargetAcceptanceRateValue", and "ResamplingAlgorithm" into account.
%
% * 15-08-2017: Took the field "SMCNumMHSteps" into account.
%
% * 14-09-2017: Took the field "ResamplingThresholdValue" into account.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% reset all settings that are not directly related to the DSGE model
%
DSGEModel.CsminwelExtraRuns = NewDSGEModel.CsminwelExtraRuns;
DSGEModel.KalmanAlgorithm = NewDSGEModel.KalmanAlgorithm;
DSGEModel.ModelSolver = NewDSGEModel.ModelSolver;
DSGEModel.UseDoublingAlgorithm = NewDSGEModel.UseDoublingAlgorithm;
DSGEModel.DAMaximumIterationsValue = NewDSGEModel.DAMaximumIterationsValue;
DSGEModel.ShowProgress = NewDSGEModel.ShowProgress;
DSGEModel.ShowProgressClock = NewDSGEModel.ShowProgressClock;
DSGEModel.OverwriteDraws = NewDSGEModel.OverwriteDraws;
DSGEModel.MarginalLikelihoodValue = NewDSGEModel.MarginalLikelihoodValue;
DSGEModel.SequentialStartIteration = NewDSGEModel.SequentialStartIteration;
DSGEModel.SequentialStepLength = NewDSGEModel.SequentialStepLength;
DSGEModel.SequentialML = NewDSGEModel.SequentialML;
DSGEModel.CoverageStart = NewDSGEModel.CoverageStart;
DSGEModel.CoverageIncrement = NewDSGEModel.CoverageIncrement;
DSGEModel.CoverageEnd = NewDSGEModel.CoverageEnd;
DSGEModel.DAToleranceValue = NewDSGEModel.DAToleranceValue;
DSGEModel.RiccatiMaxIterations = NewDSGEModel.RiccatiMaxIterations;
DSGEModel.RiccatiToleranceValue = NewDSGEModel.RiccatiToleranceValue;
DSGEModel.StateCovConst = NewDSGEModel.StateCovConst;
%
StateCStr = get(controls.dsge.statecovariance,'String');
DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
%
if isfield(NewDSGEModel,'PosteriorSampler')==1;
   DSGEModel.PosteriorSampler = NewDSGEModel.PosteriorSampler;
   DSGEModel.StudenttDegFree = NewDSGEModel.StudenttDegFree;
   DSGEModel.FixedNumParamBlocks = NewDSGEModel.FixedNumParamBlocks;
   DSGEModel.BlockSize = NewDSGEModel.BlockSize;
else;
   DSGEModel.PosteriorSampler = 1;
   DSGEModel.StudenttDegFree = [];
   DSGEModel.FixedNumParamBlocks = [];
   DSGEModel.BlockSize = [];
end;
DSGEModel.PosteriorDrawsValue = NewDSGEModel.PosteriorDrawsValue;
DSGEModel.SampleBatchValue = NewDSGEModel.SampleBatchValue;
DSGEModel.ParallelChainsValue = NewDSGEModel.ParallelChainsValue;
DSGEModel.BurnInValue = NewDSGEModel.BurnInValue;
DSGEModel.MHInitialScaleFactor = NewDSGEModel.MHInitialScaleFactor;
DSGEModel.MHScaleFactor = NewDSGEModel.MHScaleFactor;
DSGEModel.RandomWeightValue = NewDSGEModel.RandomWeightValue;
DSGEModel.OptMaxIterationsValue = NewDSGEModel.OptMaxIterationsValue;
DSGEModel.TemperingStagesValue = NewDSGEModel.TemperingStagesValue;
DSGEModel.TemperingLambdaValue = NewDSGEModel.TemperingLambdaValue;
DSGEModel.SMCNumMHSteps = NewDSGEModel.SMCNumMHSteps;
DSGEModel.NumParamBlocks = NewDSGEModel.NumParamBlocks;
DSGEModel.MixedDistWeightValue = NewDSGEModel.MixedDistWeightValue;
DSGEModel.SMCInitialScaleFactor = NewDSGEModel.SMCInitialScaleFactor;
DSGEModel.TargetAcceptanceRateValue = NewDSGEModel.TargetAcceptanceRateValue;
DSGEModel.ResamplingAlgorithm = NewDSGEModel.ResamplingAlgorithm;
DSGEModel.ResamplingThresholdValue = NewDSGEModel.ResamplingThresholdValue;
DSGEModel.OptToleranceValue = NewDSGEModel.OptToleranceValue;
DSGEModel.MaximizeAlgorithmValue = NewDSGEModel.MaximizeAlgorithmValue;
if (DSGEModel.MaximizeAlgorithmValue>6)&(FileExist(FixFilePath([pwd '\optim\fmin\matlab7\YADAfminunc7.m']))==0)&(FileExist(FixFilePath([pwd '\optim\fmin\matlab5\YADAfminunc5.m']))==0)&(FileExist(FixFilePath([pwd '\optim\fmin\matlab75\YADAfminunc7.m']))==0);
   DSGEModel.MaximizeAlgorithmValue = 1;
end;
DSGEModel.StepLengthHessian = NewDSGEModel.StepLengthHessian;
DSGEModel.NumberOfGridPoints = NewDSGEModel.NumberOfGridPoints;
DSGEModel.FiniteDifferenceHessian = NewDSGEModel.FiniteDifferenceHessian;
DSGEModel.CheckOptimum = NewDSGEModel.CheckOptimum;
DSGEModel.CheckTransformedOptimum = NewDSGEModel.CheckTransformedOptimum;
DSGEModel.GridWidth = NewDSGEModel.GridWidth;
DSGEModel.IRHorizon = NewDSGEModel.IRHorizon;
DSGEModel.PriorKernel = NewDSGEModel.PriorKernel;
DSGEModel.KernelDensityValue = NewDSGEModel.KernelDensityValue;
DSGEModel.KernelDensityEstimator = NewDSGEModel.KernelDensityEstimator;
DSGEModel.PosteriorDensityValue = NewDSGEModel.PosteriorDensityValue;
DSGEModel.RandomNumberValue = NewDSGEModel.RandomNumberValue;
DSGEModel.MaxForecastHorizon = NewDSGEModel.MaxForecastHorizon;
DSGEModel.BVARLags = NewDSGEModel.BVARLags;
DSGEModel.HarmonicLagDecayValue = NewDSGEModel.HarmonicLagDecayValue;
DSGEModel.CrossEqTightnessValue = NewDSGEModel.CrossEqTightnessValue;
DSGEModel.OverallTightnessValue = NewDSGEModel.OverallTightnessValue;
DSGEModel.PriorType = NewDSGEModel.PriorType;
DSGEModel.PriorLevelMeanValue = NewDSGEModel.PriorLevelMeanValue;
DSGEModel.PriorDiffMeanValue = NewDSGEModel.PriorDiffMeanValue;
DSGEModel.OmegaPriorType = NewDSGEModel.OmegaPriorType;
DSGEModel.WishartType = NewDSGEModel.WishartType;
DSGEModel.VarianceTightnessValue = NewDSGEModel.VarianceTightnessValue;
DSGEModel.WishartDFValue = NewDSGEModel.WishartDFValue;
DSGEModel.NumPredPathsValue = NewDSGEModel.NumPredPathsValue;
DSGEModel.PostDrawsUsageValue = NewDSGEModel.PostDrawsUsageValue;
DSGEModel.RandomizeDraws = NewDSGEModel.RandomizeDraws;
DSGEModel.ConfidenceBandBaseColor = NewDSGEModel.ConfidenceBandBaseColor;
DSGEModel.PostDrawsPercentValue = NewDSGEModel.PostDrawsPercentValue;
if NewDSGEModel.InverseHessianEstimator~=4;
   DSGEModel.InverseHessianEstimator = NewDSGEModel.InverseHessianEstimator;
end;
DSGEModel.MaxCorrelationValue = NewDSGEModel.MaxCorrelationValue;
DSGEModel.AdjustPredictionPaths = NewDSGEModel.AdjustPredictionPaths;
DSGEModel.KsiUseCondData = NewDSGEModel.KsiUseCondData;
DSGEModel.ShockControlMethod = NewDSGEModel.ShockControlMethod;
DSGEModel.ModifiedHessian = NewDSGEModel.ModifiedHessian;
DSGEModel.AIMTolerance = NewDSGEModel.AIMTolerance;
DSGEModel.AllowUnitRoot = NewDSGEModel.AllowUnitRoot;
DSGEModel.InitializeHessian = NewDSGEModel.InitializeHessian;
DSGEModel.StationaryVAR = NewDSGEModel.StationaryVAR;
DSGEModel.ConfidenceRegionMethod = NewDSGEModel.ConfidenceRegionMethod;

%
% end of ImportYADASettings.m
%
