function ObsWeightDecompDLG(selector,StateDecomp,VarStr,EstStr,DSGEModel,CurrINI)
% ObsWeightDecompDLG: Displays graphs of observation weight decompositions of estimated
%                     state variable and shocks.
%
% USAGE:
%
%       ObsWeightDecompDLG(selector,StateDecomp,VarStr,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph",
%                          and "done".
%
% OPTIONAL INPUT: StateDecomp (structure) whose fields depend on if state variables or shock have been decomposed.
%
%                 VarStr (string vector) that takes on the values "State Variables" and "Shocks".
%
%                 EstStr (string vector) that takes on the values "Initial Values" and "Posterior Mode".
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as
%                          input files, variable and sample data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: September 24, 2009.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-09-2009: Added all the plotting code.
%
% * 29-09-2009: Removed the last character of "VarStr" for the string value of the variablestext
%               field of the controls structure.
%
% * 06-01-2010: Updated the documentation.
%
% * 25-05-2010: Corrected a text issue on the dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 16-03-2011: Corrected an issue for the observed variable groups when the number of colors
%               could be less than the number of groups.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.VarStr = VarStr;
   DSGEModel.StateDecomp = StateDecomp;
   DSGEModel.EstStr = EstStr;
else;
   GraphGUI = findobj('Type','figure','Tag','ObsWeightDecompDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
% create the dialog
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ObsWeightDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-520)/2 (CurrINI.scrsz(4)-160)/2 520 160], ...
         'Tag','ObsWeightDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Observation Weight Decompositions - ' VarStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ObsWeightDecompDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 518 150],'Graphics Selections',45,[0.5 1],'on',CurrINI);
      %
      % sample selection controls
      %
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 105 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 110 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateDecomp.SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsWeightDecompDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 110 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateDecomp.SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsWeightDecompDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(StateDecomp.SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 70 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String',['Select ' lower(VarStr(1,1:length(VarStr)-1)) ':'], ...
         'HorizontalAlignment','left', ...
         'Tag','StateShockVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 75 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateDecomp.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','StateShockVarPopup',...
         'Value',1);
      %
      controls.blackedgebars = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw bars with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgeBarsCheckBox');
      %
      controls.useobsvargroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use observed variable groups', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','UseObsVarGorupsCheckBox');
      %
      if max(DSGEModel.ObsVarGroups)<DSGEModel.n;
         set(controls.useobsvargroups,'Value',1);
      end;
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsWeightDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsWeightDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.StateDecomp.VariableNames(VariableValue,:));
      if VariableValue<size(DSGEModel.StateDecomp.VariableNames,1);
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      % check if the bars should have black edges or not
      %
      BlackEdgeBars = get(controls.blackedgebars,'Value');
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      UseObsVarGroups = get(controls.useobsvargroups,'Value');
      %
      % collect data
      %
      StateDecomp = DSGEModel.StateDecomp;
      DateVector = StateDecomp.DateVector;
      if UseObsVarGroups==1
         if (isempty(DSGEModel.ObsVarGroupColors)==0)&(size(DSGEModel.ObsVarGroupColors,1)==size(DSGEModel.ObsVarGroupNames,1));
            GroupColors = DSGEModel.ObsVarGroupColors;
         else;
            GroupColors = CreateDefaultColors(DSGEModel.n);
         end;
      else;
         GroupColors = CreateDefaultColors(DSGEModel.n);
      end;
      %
      % initialize
      %
      if UseObsVarGroups==1;
         ForecastDecomp = zeros(max(DSGEModel.ObsVarGroups)+DSGEModel.UseOwnInitialState,length(DateVector));
         UpdateDecomp = zeros(max(DSGEModel.ObsVarGroups)+DSGEModel.UseOwnInitialState,length(DateVector));
         SmoothDecomp = zeros(max(DSGEModel.ObsVarGroups)+DSGEModel.UseOwnInitialState,length(DateVector));
         Groups = DSGEModel.ObsVarGroups;
         GroupNames = '';
         for i=1:size(DSGEModel.ObsVarGroupNames,1);
            GroupNames = strvcat(GroupNames,StringTrim(strrep(DSGEModel.ObsVarGroupNames(i,:),'_',' ')));
         end;
      else;
         ForecastDecomp = zeros(DSGEModel.n+DSGEModel.UseOwnInitialState,length(DateVector));
         UpdateDecomp = zeros(DSGEModel.n+DSGEModel.UseOwnInitialState,length(DateVector));
         SmoothDecomp = zeros(DSGEModel.n+DSGEModel.UseOwnInitialState,length(DateVector));
         Groups = (1:DSGEModel.n);
         GroupNames = DSGEModel.VariableNames;
      end;
      %
      % check if the initial state vector is zero
      %
      if DSGEModel.UseOwnInitialState==1;
         GroupColors = [0.3 0.3 0.3;GroupColors];
         GroupNames = strvcat('Initial State',GroupNames);
      end;
      %
      if strcmp(lower(DSGEModel.VarStr),'state variables')==1;
         %
         % state variable data
         %
         ksitt1 = StateDecomp.Ksitt1(VariableValue,:);
         ksitt = StateDecomp.Ksitt(VariableValue,:);
         ksitT = StateDecomp.KsitT(VariableValue,:);
         %
         % get the contribution of the initial state vector
         %
         if DSGEModel.UseOwnInitialState==1;
            ForecastDecomp(1,:) = StateDecomp.StateForecastDecomp(VariableValue,:,DSGEModel.n+1);
            UpdateDecomp(1,:) = StateDecomp.StateUpdateDecomp(VariableValue,:,DSGEModel.n+1);
            SmoothDecomp(1,:) = StateDecomp.StateSmoothDecomp(VariableValue,:,DSGEModel.n+1);
         end;
         %
         % collect the contributions of the individual groups/variables
         %
         for i=1:DSGEModel.n;
            ForecastDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:) = ForecastDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:)+StateDecomp.StateForecastDecomp(VariableValue,:,i);
            UpdateDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:) = UpdateDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:)+StateDecomp.StateUpdateDecomp(VariableValue,:,i);
            SmoothDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:) = SmoothDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:)+StateDecomp.StateSmoothDecomp(VariableValue,:,i);
         end;
         %
         % plotting state variables
         %
         FigHandle = figure('Visible','off', ...
                'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                'Tag',CurrVariable, ...
                'Name',['Observation Weight Decomposition - Forecasted ' CurrVariable]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         FigHandles = FigHandle;
         %
         % Create the bars and collect the handles
         %
         if MatlabNumber>8.3;
            BarHandles = bar(DateVector',ForecastDecomp','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',DateVector',ForecastDecomp','stacked');
            else;
               BarHandles = bar(DateVector',ForecastDecomp','stacked');
            end;
         end;
         %
         % remake the bar patches such that all bars are displayed properly.
         %
         BarHandles = RemakeBars(BarHandles,GroupColors,BlackEdgeBars);
         %
         % plot the data
         %
         LineHandle = plot(DateVector,ksitt1,'.-','Color',[0 0 0],'LineWidth',2);
         %
         SetAxesFonts(CurrINI,'Title',['Forecast estimate of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
         %
         axis('tight');
         ylim = get(gca,'YLim');
         AddOn = 0.1*(ylim(2)-ylim(1));
         ylim = [ylim(1)-AddOn ylim(2)+AddOn];
         set(gca,'YLim',ylim);
         if (ylim(1)<0)&(ylim(2)>0);
            hold('on');
            plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
            %
            % make sure the zero line is displayed below the observed variable
            %
            kids = get(gca,'Children');
            if size(kids,1)>size(kids,2);
               kids = kids';
            end;
            kids = [kids(2:length(kids)) kids(1)]';
            set(gca,'Children',kids);
         end;
         %
         % make sure that only the selected sample is displayed
         %
         XData = get(BarHandles(1),'XData');
         if size(XData,1)>size(XData,2);
            XData = XData';
         end;
         set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
         %
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,[CurrVariable ' (t|t-1)']),2);
         SetAxesFonts(CurrINI,legHandle);
         hold('off');
         %
         % check if we should fix the X label
         %
         SetXTickIntegers(gca);
         %
         % 2. update estimates
         %
         FigHandle = figure('Visible','off', ...
                'Position',[(CurrINI.scrsz(3)-750)/2+22 (CurrINI.scrsz(4)-500)/2-22 750 500], ...
                'Tag',CurrVariable, ...
                'Name',['Observation Weight Decomposition - Updated ' CurrVariable]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         FigHandles = [FigHandles FigHandle];
         %
         % Create the bars and collect the handles
         %
         if MatlabNumber>8.3;
            BarHandles = bar(DateVector',UpdateDecomp','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',DateVector',UpdateDecomp','stacked');
            else;
               BarHandles = bar(DateVector',UpdateDecomp','stacked');
            end;
         end;
         %
         % remake the bar patches such that all bars are displayed properly.
         %
         BarHandles = RemakeBars(BarHandles,GroupColors,BlackEdgeBars);
         %
         % plot the data
         %
         LineHandle = plot(DateVector,ksitt,'.-','Color',[0 0 0],'LineWidth',2);
         %
         SetAxesFonts(CurrINI,'Title',['Update estimate of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
         %
         axis('tight');
         ylim = get(gca,'YLim');
         AddOn = 0.1*(ylim(2)-ylim(1));
         ylim = [ylim(1)-AddOn ylim(2)+AddOn];
         set(gca,'YLim',ylim);
         if (ylim(1)<0)&(ylim(2)>0);
            hold('on');
            plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
            %
            % make sure the zero line is displayed below the observed variable
            %
            kids = get(gca,'Children');
            if size(kids,1)>size(kids,2);
               kids = kids';
            end;
            kids = [kids(2:length(kids)) kids(1)]';
            set(gca,'Children',kids);
         end;
         %
         % make sure that only the selected sample is displayed
         %
         XData = get(BarHandles(1),'XData');
         if size(XData,1)>size(XData,2);
            XData = XData';
         end;
         set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
         %
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,[CurrVariable ' (t|t)']),2);
         SetAxesFonts(CurrINI,legHandle);
         hold('off');
         %
         % check if we should fix the X label
         %
         SetXTickIntegers(gca);
         %
         % 3. smooth estimate
         %
         FigHandle = figure('Visible','off', ...
                'Position',[(CurrINI.scrsz(3)-750)/2+44 (CurrINI.scrsz(4)-500)/2-44 750 500], ...
                'Tag',CurrVariable, ...
                'Name',['Observation Weight Decomposition - Smoothed ' CurrVariable]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         FigHandles = [FigHandles FigHandle];
         %
         % Create the bars and collect the handles
         %
         if MatlabNumber>8.3;
            BarHandles = bar(DateVector',SmoothDecomp','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',DateVector',SmoothDecomp','stacked');
            else;
               BarHandles = bar(DateVector',SmoothDecomp','stacked');
            end;
         end;
         %
         % remake the bar patches such that all bars are displayed properly.
         %
         BarHandles = RemakeBars(BarHandles,GroupColors,BlackEdgeBars);
         %
         % plot the data
         %
         LineHandle = plot(DateVector,ksitT,'.-','Color',[0 0 0],'LineWidth',2);
         %
         SetAxesFonts(CurrINI,'Title',['Smooth estimate of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
         %
         axis('tight');
         ylim = get(gca,'YLim');
         AddOn = 0.1*(ylim(2)-ylim(1));
         ylim = [ylim(1)-AddOn ylim(2)+AddOn];
         set(gca,'YLim',ylim);
         if (ylim(1)<0)&(ylim(2)>0);
            hold('on');
            plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
            %
            % make sure the zero line is displayed below the observed variable
            %
            kids = get(gca,'Children');
            if size(kids,1)>size(kids,2);
               kids = kids';
            end;
            kids = [kids(2:length(kids)) kids(1)]';
            set(gca,'Children',kids);
         end;
         %
         % make sure that only the selected sample is displayed
         %
         XData = get(BarHandles(1),'XData');
         if size(XData,1)>size(XData,2);
            XData = XData';
         end;
         set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
         %
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,[CurrVariable ' (t|T)']),2);
         SetAxesFonts(CurrINI,legHandle);
         hold('off');
         %
         % check if we should fix the X label
         %
         SetXTickIntegers(gca);
      else;
         %
         % shock data
         %
         etatt = StateDecomp.etatt(VariableValue,:);
         etatT = StateDecomp.etatT(VariableValue,:);
         %
         % get the contribution of the initial state vector
         %
         if DSGEModel.UseOwnInitialState==1;
            UpdateDecomp(1,:) = StateDecomp.ShockUpdateDecomp(VariableValue,:,DSGEModel.n+1);
            SmoothDecomp(1,:) = StateDecomp.ShockSmoothDecomp(VariableValue,:,DSGEModel.n+1);
         end;
         %
         % collect the contributions of the individual groups/variables
         %
         for i=1:DSGEModel.n;
            UpdateDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:) = UpdateDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:)+StateDecomp.ShockUpdateDecomp(VariableValue,:,i);
            SmoothDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:) = SmoothDecomp(Groups(i)+DSGEModel.UseOwnInitialState,:)+StateDecomp.ShockSmoothDecomp(VariableValue,:,i);
         end;
         %
         % 1. update estimates
         %
         FigHandle = figure('Visible','off', ...
                'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                'Tag',CurrVariable, ...
                'Name',['Observation Weight Decomposition - Updated ' CurrVariable]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         FigHandles = FigHandle;
         %
         % Create the bars and collect the handles
         %
         if MatlabNumber>8.3;
            BarHandles = bar(DateVector',UpdateDecomp','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',DateVector',UpdateDecomp','stacked');
            else;
               BarHandles = bar(DateVector',UpdateDecomp','stacked');
            end;
         end;
         %
         % remake the bar patches such that all bars are displayed properly.
         %
         BarHandles = RemakeBars(BarHandles,GroupColors,BlackEdgeBars);
         %
         % plot the data
         %
         LineHandle = plot(DateVector,etatt,'.-','Color',[0 0 0],'LineWidth',2);
         %
         SetAxesFonts(CurrINI,'Title',['Update estimate of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
         %
         axis('tight');
         ylim = get(gca,'YLim');
         AddOn = 0.1*(ylim(2)-ylim(1));
         ylim = [ylim(1)-AddOn ylim(2)+AddOn];
         set(gca,'YLim',ylim);
         if (ylim(1)<0)&(ylim(2)>0);
            hold('on');
            plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
            %
            % make sure the zero line is displayed below the observed variable
            %
            kids = get(gca,'Children');
            if size(kids,1)>size(kids,2);
               kids = kids';
            end;
            kids = [kids(2:length(kids)) kids(1)]';
            set(gca,'Children',kids);
         end;
         %
         % make sure that only the selected sample is displayed
         %
         XData = get(BarHandles(1),'XData');
         if size(XData,1)>size(XData,2);
            XData = XData';
         end;
         set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
         %
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,[CurrVariable ' (t|t)']),2);
         SetAxesFonts(CurrINI,legHandle);
         hold('off');
         %
         % check if we should fix the X label
         %
         SetXTickIntegers(gca);
         %
         % 2. smooth estimate
         %
         FigHandle = figure('Visible','off', ...
                'Position',[(CurrINI.scrsz(3)-750)/2+22 (CurrINI.scrsz(4)-500)/2-22 750 500], ...
                'Tag',CurrVariable, ...
                'Name',['Observation Weight Decomposition - Smoothed ' CurrVariable]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         FigHandles = [FigHandles FigHandle];
         %
         % Create the bars and collect the handles
         %
         if MatlabNumber>8.3;
            BarHandles = bar(DateVector',SmoothDecomp','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',DateVector',SmoothDecomp','stacked');
            else;
               BarHandles = bar(DateVector',SmoothDecomp','stacked');
            end;
         end;
         %
         % remake the bar patches such that all bars are displayed properly.
         %
         BarHandles = RemakeBars(BarHandles,GroupColors,BlackEdgeBars);
         %
         % plot the data
         %
         LineHandle = plot(DateVector,etatT,'.-','Color',[0 0 0],'LineWidth',2);
         %
         SetAxesFonts(CurrINI,'Title',['Smooth estimate of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
         %
         axis('tight');
         ylim = get(gca,'YLim');
         AddOn = 0.1*(ylim(2)-ylim(1));
         ylim = [ylim(1)-AddOn ylim(2)+AddOn];
         set(gca,'YLim',ylim);
         if (ylim(1)<0)&(ylim(2)>0);
            hold('on');
            plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
            %
            % make sure the zero line is displayed below the observed variable
            %
            kids = get(gca,'Children');
            if size(kids,1)>size(kids,2);
               kids = kids';
            end;
            kids = [kids(2:length(kids)) kids(1)]';
            set(gca,'Children',kids);
         end;
         %
         % make sure that only the selected sample is displayed
         %
         XData = get(BarHandles(1),'XData');
         if size(XData,1)>size(XData,2);
            XData = XData';
         end;
         set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
         %
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,[CurrVariable ' (t|T)']),2);
         SetAxesFonts(CurrINI,legHandle);
         hold('off');
         %
         % check if we should fix the X label
         %
         SetXTickIntegers(gca);
      end;
      %
      % display the graphs
      %
      for i=1:length(FigHandles);
         SetToolbarImages(FigHandles(i),CurrINI.images);
         set(FigHandles(i),'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of ObsWeightDecompDLG.m
%
