function VerifyStateCondControls(DSGEModel,controls)
% VerifyStateCondControls:
%
% USAGE:
%
%       VerifyStateCondControls(DSGEModel,controls)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 controls (structure) with the handles to all controls on the main
%                           YADA dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 14, 2009.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-07-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 04-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 02-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 09-02-2017: Updated the documentation.
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% test if the conditioning data is valid
%
if CheckStateCondData(DSGEModel)==0;
   set(controls.predict.stateconditional_original_initialvalues,'Enable','off');
   set(controls.predict.stateconditional_annual_initialvalues,'Enable','off');
   set(controls.predict.stateconditional_transformed_initialvalues,'Enable','off');
   set(controls.predict.stateconditional_original_prior,'Enable','off');
   set(controls.predict.stateconditional_annual_prior,'Enable','off');
   set(controls.predict.stateconditional_transformed_prior,'Enable','off');
   set(controls.predict.stateconditional_original_postmode,'Enable','off');
   set(controls.predict.stateconditional_annual_postmode,'Enable','off');
   set(controls.predict.stateconditional_transformed_postmode,'Enable','off');
   set(controls.predict.stateconditional_original_posterior,'Enable','off');
   set(controls.predict.stateconditional_original_predevent,'Enable','off');
   set(controls.predict.stateconditional_annual_posterior,'Enable','off');
   set(controls.predict.stateconditional_annual_predevent,'Enable','off');
   set(controls.predict.stateconditional_transformed_posterior,'Enable','off');
   set(controls.predict.stateconditional_transformed_predevent,'Enable','off');
   return;
end;
%
% determine some useful variables
%
MaxAnnual = max(DSGEModel.annual);
TransStatus = 1-isempty(DSGEModel.YTransformation);
%
% 1. initial values
%
if VerifyPosteriorModeEstimation(DSGEModel)==1;
   set(controls.predict.stateconditional_original_initialvalues,'Enable','on');
   if MaxAnnual>1;
      set(controls.predict.stateconditional_annual_initialvalues,'Enable','on');
   else;
      set(controls.predict.stateconditional_annual_initialvalues,'Enable','off');
   end;
   if TransStatus==1;
      set(controls.predict.stateconditional_transformed_initialvalues,'Enable','on');
   else;
      set(controls.predict.stateconditional_transformed_initialvalues,'Enable','off');
   end;
else;
   set(controls.predict.stateconditional_original_initialvalues,'Enable','off');
   set(controls.predict.stateconditional_annual_initialvalues,'Enable','off');
   set(controls.predict.stateconditional_transformed_initialvalues,'Enable','off');
end;
%
% 2. prior distribution
%
PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
if FileExist(PriorDrawsFile)==1;
   set(controls.predict.stateconditional_original_prior,'Enable','on');
   if MaxAnnual>1;
      set(controls.predict.stateconditional_annual_prior,'Enable','on');
   else;
      set(controls.predict.stateconditional_annual_prior,'Enable','off');
   end;
   if TransStatus==1;
      set(controls.predict.stateconditional_transformed_prior,'Enable','on');
   else;
      set(controls.predict.stateconditional_transformed_prior,'Enable','off');
   end;
else;
   set(controls.predict.stateconditional_original_prior,'Enable','off');
   set(controls.predict.stateconditional_annual_prior,'Enable','off');
   set(controls.predict.stateconditional_transformed_prior,'Enable','off');
end;
%
% 3. posterior mode
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
if FileExist(ModeFile)==1;
   set(controls.predict.stateconditional_original_postmode,'Enable','on');
   if MaxAnnual>1;
      set(controls.predict.stateconditional_annual_postmode,'Enable','on');
   else;
      set(controls.predict.stateconditional_annual_postmode,'Enable','off');
   end;
   if TransStatus==1;
      set(controls.predict.stateconditional_transformed_postmode,'Enable','on');
   else;
      set(controls.predict.stateconditional_transformed_postmode,'Enable','off');
   end;
else;
   set(controls.predict.stateconditional_original_postmode,'Enable','off');
   set(controls.predict.stateconditional_annual_postmode,'Enable','off');
   set(controls.predict.stateconditional_transformed_postmode,'Enable','off');
end;
%
% 4. posterior distribution and prediction events
%
if VerifyDSGEPosteriorDraws(DSGEModel,1,controls)==1;
   set(controls.predict.stateconditional_original_posterior,'Enable','on');
   set(controls.predict.stateconditional_original_predevent,'Enable','on');
   if MaxAnnual>1;
      set(controls.predict.stateconditional_annual_posterior,'Enable','on');
      set(controls.predict.stateconditional_annual_predevent,'Enable','on');
   else;
      set(controls.predict.stateconditional_annual_posterior,'Enable','off');
      set(controls.predict.stateconditional_annual_predevent,'Enable','off');
   end;
   if TransStatus==1;
      set(controls.predict.stateconditional_transformed_posterior,'Enable','on');
      set(controls.predict.stateconditional_transformed_predevent,'Enable','on');
   else;
      set(controls.predict.stateconditional_transformed_posterior,'Enable','off');
      set(controls.predict.stateconditional_transformed_predevent,'Enable','off');
   end;
else;
   set(controls.predict.stateconditional_original_posterior,'Enable','off');
   set(controls.predict.stateconditional_original_predevent,'Enable','off');
   set(controls.predict.stateconditional_annual_posterior,'Enable','off');
   set(controls.predict.stateconditional_annual_predevent,'Enable','off');
   set(controls.predict.stateconditional_transformed_posterior,'Enable','off');
   set(controls.predict.stateconditional_transformed_predevent,'Enable','off');
end;

%
% end of VerifyStateCondControls.m
%
