function ZLBPredictiveDistributionDLG(selector,DSGEModel,CurrINI,ZLBYMeanPrediction,ZLBYQuantiles,KsitT,ZLBKsiMeanPrediction,ZLBKsiQuantiles,NameStr,AnnualStr,IsPosterior,IsConditional)
% ZLBPredictiveDistributionDLG: Displays predictive distributions of the observed and the state variables along with the mean
%                               predictions and when possible the observed values, when the monetary policy rate is subject
%                               to the zero lower bound.
%
% USAGE:
%
%       ZLBPredictiveDistributionDLG(selector,DSGEModel,CurrINI,ZLBYMeanPrediction,ZLBYQuantiles,KsitT, ...
%                                    ZLBKsiMeanPrediction,ZLBKsiQuantiles,NameStr,AnnualStr,IsPosterior,IsConditional)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 ZLBYMeanPrediction (matrix) of dimension nxh with the mean predictions of the observed
%                           variables.
%
%                 ZLBYQuantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                           is an nxh matrix with the quantiles of the prediction paths. The second field
%                           is "percent" that gives the percentile value as a string vector.
%
%                 KsitT (matrix) of dimension rxTh with smooth estimates of the state variables.
%
%                 ZLBKsiMeanPrediction (matrix) of dimension rxh with the mean predictions of the state
%                           variables.
%
%                 ZLBKsiQuantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                           is an rxh matrix with the quantiles of the prediction paths. The second field
%                           is "percent" that gives the percentile value as a string vector.
%
%                 NameStr (string vector) taking on the values "Posterior Mode", "Initial Values", or "".
%
%                 AnnualStr (string vector) taking on the values "Original", "Annual", or "Transformed".
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
%                 IsConditional (boolean) that is 1 if conditional forecasts have been computed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: September 10, 2014.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-09-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.ZLBYMeanPrediction = ZLBYMeanPrediction;
   DSGEModel.ZLBYQuantiles = ZLBYQuantiles;
   DSGEModel.KsitT = KsitT;
   DSGEModel.ZLBKsiMeanPrediction = ZLBKsiMeanPrediction;
   DSGEModel.ZLBKsiQuantiles = ZLBKsiQuantiles;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.IsPosterior = IsPosterior;
   DSGEModel.IsConditional = IsConditional;
   %
   DSGEModel.PriorHeader = '';
   if IsPosterior==0;
      DSGEModel.PriorHeader = ' For Prior';
   end;
   %
   % Check where the sample of prediction data actually begins
   %
   [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,DSGEModel.PredDateVector,DSGEModel.PredSampleStrMat] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
   %
   % check if we have observations for the whole prediction sample
   %
      DSGEModel.ShowAdjustOption = 0;
      DSGEModel.ShowKernel = 0;
else
   PredictiveDistributionGUI = findobj('Type','figure','Tag','CondPredictiveDistributionDLG');
   PredictiveControls = get(PredictiveDistributionGUI,'UserData');
   CurrINI = get(PredictiveControls.varnames,'UserData');
   DSGEModel = get(PredictiveControls.variables,'UserData');
end;
%
%
switch selector;
   case 'init';
      PredictiveDistributionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ZLBPredictiveDistributionDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-140)/2 500 140], ...
         'Tag','CondPredictiveDistributionDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['ZLB Predictive Distributions' DSGEModel.PriorHeader ' - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ZLBPredictiveDistributionDLG done', ...
         'ToolBar','none');
      %
      if isempty(NameStr)==1;
         set(PredictiveDistributionGUI,'Name',['ZLB Predictive Distribution With ' ModelStr ' Model']);
      end;
      %
      if MatlabNumber>=7.0;
         set(PredictiveDistributionGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 130],'Multistep Conditional Forecasts',45,[0.5 1],'on',CurrINI);
      %
      PredictiveControls.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Prediction Sample: ' PredStartYear ':' PredStartPeriod ' - ' PredEndYear ':' PredEndPeriod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      if (strcmp(lower(AnnualStr),'original')==1)|(strcmp(lower(AnnualStr),'transformed')==1);;
         if DSGEModel.KalmanFirstObservation>1;
            [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
         else;
            NewStartYear = DSGEModel.SubBeginYear;
            NewStartPeriod = DSGEModel.SubBeginPeriod;
         end;
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,max(DSGEModel.annual)+DSGEModel.KalmanFirstObservation-2);
      end;
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      [FirstPeriod,LastPeriod,SampleStrMat] = CreateSubSample(DSGEModel);
      DSGEModel = OldDSGEModel;
      %
      % check if we need to change the sample from the top
      %
      if strcmp(lower(AnnualStr),'transformed')==1;
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         T2 = size(YData,1);
         if DSGEModel.T-T2>FirstPeriod+DSGEModel.KalmanFirstObservation-1;
            SampleStrMat = SampleStrMat(DSGEModel.T-T2-(FirstPeriod+DSGEModel.KalmanFirstObservation-1):size(SampleStrMat,1),:);
         end;
      end;
      %
      % data sample
      %
      PredictiveControls.startdatatext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Start period for variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','StartDataText');
      %
      PredictiveControls.startdata = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 55 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Tag','StartDataPopup',...
         'Value',1);
      %
      % Select variable
      %
      PredictiveControls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableNamesText');
      %
      PredictiveControls.varnames = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames), ...
         'HorizontalAlignment','center', ...
         'Tag','VariableNamesPopup',...
         'Value',1);
      %
      % Create graph button
      %
      PredictiveControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ZLBPredictiveDistributionDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      PredictiveControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ZLBPredictiveDistributionDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(PredictiveDistributionGUI,'UserData',PredictiveControls,...
         'HandleVisibility','callback');
   case 'showgraph'
      VariableValue = get(PredictiveControls.varnames,'Value');
      VariableStr = get(PredictiveControls.varnames,'String');
      CurrVariable = StringTrim(VariableStr(VariableValue,:));
      if VariableValue<size(VariableStr,1);
         set(PredictiveControls.varnames,'Value',VariableValue+1);
      else;
         set(PredictiveControls.varnames,'Value',1);
      end;
      if DSGEModel.IsConditional==1;
         CondStr = 'Conditional ';
      else;
         CondStr = 'Unconditional ';
      end;
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      KsiFirstPeriod = FirstPeriod;
      FirstPeriod = FirstPeriod+DSGEModel.KalmanFirstObservation+get(PredictiveControls.startdata,'Value')-2;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                     'Tag',[CurrVariable '-' DSGEModel.NameStr], ...
                     'Name',['ZLB ' CondStr 'Predictions' DSGEModel.PriorHeader ' - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      %
      if VariableValue<=DSGEModel.n;
         MeanPrediction = DSGEModel.ZLBYMeanPrediction;
         Quantiles = DSGEModel.ZLBYQuantiles;
         if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
            xData = DateVector(FirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
            yData = DSGEModel.Y(VariableValue,FirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
         elseif strcmp(lower(DSGEModel.AnnualStr),'annual')==1;
            Y = Annualize(DSGEModel.Y(:,FirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector))),DSGEModel.annual,DSGEModel.annualscale);
            xData = DateVector(1,max(DSGEModel.annual)+FirstPeriod-1:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
            yData = Y(VariableValue,:);
         else;
            %
            % we're using the transformed data
            %
            T1 = min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector));
            OldDateVector = DateVector;
            DateVector = DateVector(FirstPeriod:T1);
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y(:,1:T1)',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            Y = YData';
            T2 = size(Y,2);
            if T2==T1;
               Y = Y(:,FirstPeriod:T1);
            else;
               if FirstPeriod-(T1-T2)>=1;
                  Y = Y(:,FirstPeriod-(T1-T2):T1);
               else;
                  DateVector = DateVector(T1-T2+1:T1);
               end;
            end;
            xData = DateVector;
            yData = Y(VariableValue,:);
         end;
         YStr = CurrVariable;
      else;
         MeanPrediction = DSGEModel.ZLBKsiMeanPrediction;
         Quantiles = DSGEModel.ZLBKsiQuantiles;
         VariableValue = VariableValue-DSGEModel.n;
         xData = DateVector(KsiFirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
         yData = DSGEModel.KsitT(VariableValue,KsiFirstPeriod:min(size(DSGEModel.KsitT,2),LastPeriod+length(DSGEModel.PredDateVector)));
         if length(xData)>length(yData);
            xData = xData(1:length(yData))
         end;
         YStr = ['Smoothed ' CurrVariable];
      end;
      %
      % first we plot the predictive bounds
      %
      NumBounds = floor(length(Quantiles)/2);
      Handles = [];
      LegendStr = '';
      for i=1:NumBounds;
         Lower = Quantiles(i).Quantile(VariableValue,:);
         Upper = Quantiles(length(Quantiles)-i+1).Quantile(VariableValue,:);
         PatchColor = ((1+(2*(i-1)))/length(Quantiles))*DSGEModel.ConfidenceBandBaseColor;
         ConfBound = Quantiles(length(Quantiles)-i+1).percent-Quantiles(i).percent;
         ConfPatch = patch([DSGEModel.PredDateVector fliplr(DSGEModel.PredDateVector)],[Upper fliplr(Lower)],PatchColor);
         set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
         hold('on');
         Handles = [Handles ConfPatch];
         LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
      end;
      set(gca,'Layer','Top');
      %
      % next we plot the observed variable
      %
      if isempty(xData)==0
         YHandle = plot(xData,yData,'+-','Color',[0.75 0 0],'LineWidth',2);
         hold('on');
      else;
         YHandle = [];
         YStr = '';
      end;
      %
      % plot the mean prediction
      %
      PredHandle = plot(DSGEModel.PredDateVector,MeanPrediction(VariableValue,:),'--','Color',[0 0 0.75],'LineWidth',2);
      Handles = [YHandle PredHandle Handles];
      LegendStr = strvcat(YStr,'Mean predictions',LegendStr);
      %
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      if (strcmp(lower(DSGEModel.AnnualStr),'original')==1)|(strcmp(lower(DSGEModel.AnnualStr),'transformed')==1);;
         set(gca,'YLim',ylim,'XLim',[DateVector(FirstPeriod) max(DSGEModel.PredDateVector)]);
      else;
         set(gca,'YLim',ylim,'XLim',[DateVector(1,max(DSGEModel.annual)+FirstPeriod-1) max(DSGEModel.PredDateVector)]);
      end;
      %
      % add a vertical line from the forecast point
      %
      fp = plot((((3*DateVector(LastPeriod))-DateVector(LastPeriod-1))/2)*[1 1],ylim,':','Color',[0 0 0]);
      hold('on');
      %
      if (ylim(1)<0)&(ylim(2)>0);
         %
         % add a zero line
         %
         zl = plot(get(gca,'XLim'),[0 0],'-','Color',[0 0 0]);
      else;
         zl = [];
      end;
      if isempty(YHandle)==0;
         set(gca,'Children',[PredHandle;YHandle;zl;fp;fliplr(Handles(3:length(Handles)))']);
      else;
         set(gca,'Children',[PredHandle;zl;fp;fliplr(Handles(2:length(Handles)))']);
      end;
      if isempty(DSGEModel.NameStr)==0;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (' lower(DSGEModel.AnnualStr) ' data) and out-of-sample ' lower(CondStr) ' prediction paths - ' lower(DSGEModel.NameStr) ' of the parameters']);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (' lower(DSGEModel.AnnualStr) ' data) and out-of-sample ' lower(CondStr) ' prediction paths']);
      end;
      hold('off');
      %
      % add the legend
      %
      legHandle = legend(Handles,LegendStr,2);
      SetAxesFonts(CurrINI,legHandle);
      %
      SetXTickIntegers(gca);
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      %
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(PredictiveControls.done,'UserData','done');
      delete(PredictiveDistributionGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(PredictiveDistributionGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(PredictiveControls.done);
   end;
   waitfor(PredictiveControls.done,'UserData','done');
end;

%
% end of ZLBPredictiveDistributionDLG.m
%
