function status = IsPositiveSemiDefinite(Sigma,OnePositiveEigenvalue)
% IsPositiveSemiDefinite: Tests if a matrix is positive semidefinite by checking that the number of rows
%                         and columns match, that it is symmetric, and that all eigenvalues are non-negative.
%
% USAGE:
%
%       status = IsPositiveSemiDefinite(Sigma,BoolOnePositive)
%
% REQUIRED INPUT:  Sigma (matrix) of dimension n1xn2.
%
% OPTIONAL INPUT:  OnePositiveEigenvalue (boolean) which if unity means that the rank of Sigma must be at least
%                                        1. If zero, then Sigma can also be a zero matrix. The default is 1 and
%                                        hence that Sigma can only be considered positive semidefinite if it has
%                                        at least rank 1.
%
% REQUIRED OUTPUT: status (boolean) that takes on the value 1 if Sigma is positive semidefinite and 0
%                                        otherwise. The function requires that the rank is at least unity.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 18, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-02-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==1;
   OnePositiveEigenvalue = 1;
end;
%
% initialize
%
status = 0;
[n1,n2] = size(Sigma);
if n1~=n2;
   return;
end;
r = rank(Sigma);
if (OnePositiveEigenvalue==1)&(r==0);
   return;
end;
if r==size(Sigma,1);
   status = IsPositiveDefinite(Sigma);
else;
   if isequal(Sigma,Sigma')==1;
      EV = eig(Sigma);
      status = (r==sum(EV>0));
   end;
end;

%
% end of IsPositiveSemiDefinite.m
%
