function status = StateCVDExist(DSGEModel,NumDraws,TotalDraws,CurrChain,IsPosterior)
% StateCVDExist: Tests if the conditional variance decompositions for the state variables have already been
%                calculated for the DSGE model.
%
% USAGE:
%
%       status = StateCVDExist(DSGEModel,NumDraws,TotalDraws,CurrChain,IsPosterior)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                           parameters.
%
%                  TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                           that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
% REQUIRED OUTPUT: status (boolean) that indicates if the conditional variance decompositions have already been
%                           calculated or not.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European central Bank.
%
%                       First version: May 4, 2017.
%                        This version: May 5, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-05-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
NumFiles = ceil(NumDraws/min(NumDraws,1000));
%
% Determine the impulse response horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
h = SampleFreq*DSGEModel.IRHorizon;
for ThisSave=1:NumFiles;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-State-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-State-Prior-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
   end;
   if FileExist(file)==0;
      return;
   end;
end;
status = 1;

%
% end of StateCVDExist.m
%
