function BVARMenuFunctions(selector)
% BVARMenuFunctions: switch function for running the function on the BVAR menu.
%
% USAGE:
%
%       BVARMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 12, 2010.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-02-2010: Updated the documentation.
%
% * 16-02-2010: Changed the order of the input variables for "YADAEnableControls".
%
% * 17-02-2010: Fixed the "YADA" estimation log text.
%
% * 10-03-2010: Made sure that controls.tab6 gets focus when the matlab version supports this.
%
% * 22-07-2010: Took into account that "SummarizeDSGEPredictions" now provides the "TotalVar" output
%               variable.
%
% * 23-07-2010: Added a call to "NormalDensityValues" for a normal approximation of the marginal
%               predictive densities.
%
% * 18-11-2010: Took into account that "BVARPredictionPathsPostMode" now provides the output variable
%               "YObsEventData".
%
% * 19-11-2010: And now that is 8 columns.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" suppots unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 04-03-2011: Took into account that several handles on the BVAR predictions menu have new names.
%
% * 07-03-2011: Finished the conditional predictive distribution at posterior mode.
%
% * 08-03-2011: Finished the conditional predictive distribution for the posterior draws.
%
% * 02-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter value when
%               forecasting can be unity.
%
% * 29-08-2012: Took into account that "PredictionPathDistribution" now takes the input variable
%               "ConfidenceRegionMethod".
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 01-09-2014: Made sure that "BVARPredictionDistribution" receives the input variable "ConfidenceRegionMethod",
%               supported via the DSGEModel structure.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
lasterr = '';
%
switch selector
   case 'BVARPostMode'
      if get(controls.about,'UserData')==0;
         %
         % make the output tab visible
         %
         oldtab = get(controls.tab1,'UserData');
         set(maingui,'CurrentObject',controls.tab6);
         YADAGUI('tab');
         if MatlabNumber>=7;
            uicontrol(controls.tab6);
         end;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Fix the log text control
         %
         LogText = get(controls.estimation_log,'String');
         if isempty(LogText)==0;
            SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
         end;
         %
         % run the Gibbs sampling algorithm
         %
         BVARPosteriorModeEstimation(DSGEModel,CurrINI,controls);
         %
         set(controls.about,'UserData',0);
%         set(maingui,'CurrentObject',getfield(controls,['tab' num2str(oldtab)]));
%         YADAGUI('tab');
      end;
   case 'Gibbs'
      if get(controls.about,'UserData')==0;
         %
         % make the output tab visible
         %
         oldtab = get(controls.tab1,'UserData');
         set(maingui,'CurrentObject',controls.tab6);
         YADAGUI('tab');
         if MatlabNumber>=7;
            uicontrol(controls.tab6);
         end;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Fix the log text control
         %
         LogText = get(controls.estimation_log,'String');
         if isempty(LogText)==0;
            SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
         end;
         %
         % run the Gibbs sampling algorithm
         %
         BVARPosteriorSampling(DSGEModel,CurrINI,controls);
         %
         set(controls.about,'UserData',0);
%         set(maingui,'CurrentObject',getfield(controls,['tab' num2str(oldtab)]));
%         YADAGUI('tab');
      end;
   case 'BVARPostModeResults'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeBVAR-' DSGEModel.NameOfModel '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '.txt']);
         TextGUI(ModeTextFile,['Bayesian VAR Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'BVARGibbsResults'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         DrawsStr = get(controls.posterior.draws,'String');
         NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
         GibbsTextFile = FixFilePath([DSGEModel.OutputDirectory '\BVAR-GibbsSummary-' DSGEModel.NameOfModel '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(NumPostDraws,'%0.0f') '.txt']);
         TextGUI(GibbsTextFile,['Bayesian VAR Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'BVAREigenvalues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-BVAR-' DSGEModel.NameOfModel '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            %
            % compute the eigenvalues
            %
            EV = BVAREigenvaluesPostMode(ModeData.Pi);
            %
            % display the eigenvalues
            %
            PlotBVAREigenvalues(EV,'Bayesian VAR','VAR model','posterior mode',CurrINI);
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'BVARPredictPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictions
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-BVAR-' DSGEModel.NameOfModel '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            %
            % Collect the data we need from ModeData
            %
            Psi = ModeData.Psi;
            Pi = ModeData.Pi;
            Omega = ModeData.Omega;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%            DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,max(size(DSGEModel.PredictedX,2)*(isempty(DSGEModel.BVARX)==0),240*(isempty(DSGEModel.BVARX)==1)));
            %
            % matrix with exogenous variables
            %
            X = [];
            if isempty(DSGEModel.BVARX)==0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(DSGEModel.BVARX,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(DSGEModel.BVARX,1:NumPredX)];
               end;
            end;
            %
            % check if we should annualize or not
            %
            if gcbo==controls.bvarpredict.unconditional.original.postmode;
               AnnualizeData = 0;
               AnnualStr = 'original';
            else;
               AnnualizeData = 1;
               AnnualStr = 'annual';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % ask if we should conduct prediction event calculations
            %
            if NumPredPaths>1;
               if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                  DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
               end;
               txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
               answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               %
               % make sure that the prediction event length does not exceed the maximum prediction horizon
               %
               for i=1:DSGEModel.n;
                  if DSGEModel.PredictionEvent(i,3)>MaxPH;
                     DSGEModel.PredictionEvent(i,3) = MaxPH;
                  end;
               end;
               %
               % display a dialog for setting up an prediction event, but using only the selected BVAR endogenous variables
               %
               [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.PredictionEvent(DSGEModel.BVARY,:),MaxPH,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.PredictionEvent(DSGEModel.BVARY,:) = PredictionEvent;
                  DSGEModel.RunPredictionEvent = 1;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
            else;
               DSGEModel.RunPredictionEvent = 0;
            end;
            %
            % we can now compute the predictive distribution for the endogenous variables
            % up to period MaxPh
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
            else;
               randn('state',sum(100*clock));
            end;
            %
            % run the prediction path calculation function
            %
            [PredPaths,PredMean,PredEventData,YObsEventData,KernelX,KernelY] = BVARPredictionPathsPostMode(Psi,Pi,Omega,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData);
            if isempty(PredPaths)==0;
               %
               % the computations went OK. Compute point estimates from predictive distribution and
               % quantiles
               %
               [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
               %
               % compute the mean error bias in percent
               %
               MeanErrorBias = max(max(abs((PredMean-MeanPrediction)./PredMean)));
               if DSGEModel.AdjustPredictionPaths==0;
                  %
                  % ask if the prediction distribution should be corrected
                  %
                  txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust the predictive distribution such that it matches the theoretical mean?'];
                  answer = Query(txt,'question',140,'Mean Error Bias - Predictive Distributions',500,'yes',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % adjust the quantiles to match
                  %
                  for i=1:length(Quantiles);
                     OlVal = Quantiles(i).Quantile;
                     Quantiles(i).Quantile = Quantiles(i).Quantile+(PredMean-MeanPrediction);
                  end;
                  MeanPrediction = PredMean;
               end;
               %
               % compute the forecast errors based on the mean predictions
               %
               PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,0,'BVAR',DSGEModel);
               %
               % check if the user wants to save the data
               %
               txt = ['Would you like to save the predictive distributions from the Bayesian VAR for the ' AnnualStr ' data at posterior mode to file?'];
               answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                  file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-BVAR-' AnnualStr '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.mat']);
                  YNames = DSGEModel.VariableNames(DSGEModel.BVARY,:);
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                  Y = DSGEModel.Y(DSGEModel.BVARY,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                  if length(DateVector)>size(Y,2);
                     DateVector = DateVector(1,1:size(Y,2));
                  end;
                  EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                  EstimationStartPeriodOrig = EstimationStartPeriod;
                  YOrig = Y;
                  DateVectorOrig = DateVector;
                  if AnnualizeData==1;
                     Y = Annualize(Y,DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
                     EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                     DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                  end;
                  if isempty(PredEventData)==1;
                     save(file,'MeanPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                  else;
                     save(file,'MeanPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData','YObsEventData');
                  end;
                  %
                  % create a matlab m-file to read the output.
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\PDBVAR' AnnualStr num2str(DSGEModel.PriorType,'%0.0f') num2str(DSGEModel.OmegaPriorType,'%0.0f') num2str(MaxPH,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'PostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables in the BVAR\n%%\n');
                  fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                  fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                  fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                  fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                  fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                  if isempty(KernelX)==0;
                     fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                     fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                     KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                     KVal = 2;
                  else;
                     KStr = '';
                     KVal = 0;
                  end;
                  fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                  fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                  fprintf(fid,'%%\n%% observed variables\n%%\n');
                  fprintf(fid,'Y = DataStructure.Y;\n');
                  fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  if isempty(PredEventData)==0;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                     NumEntries = int2str(10+KVal);
                     ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with similar information but for mean predictions given by the observations), ';
                  else;
                     NumEntries = int2str(7+KVal);
                     ExtraEntry = '';
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  txt = ['The predictive distributions from the Bayesian VAR have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Predictive Distributions',500,600,CurrINI);
               end;
               if isempty(PredEventData)==0;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-BVAR-' AnnualStr '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames(DSGEModel.BVARY,:),PredEventData,YObsEventData,DSGEModel.PredictionEvent(DSGEModel.BVARY,:),[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                  end;
               end;
               %
               % Open a dialog for displaying predictive distributions
               %
               PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,KernelX,KernelY,'Posterior Mode','BVAR',AnnualStr,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARCondPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictions
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-BVAR-' DSGEModel.NameOfModel '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            %
            % Collect the data we need from ModeData
            %
            Psi = ModeData.Psi;
            Pi = ModeData.Pi;
            Omega = ModeData.Omega;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%            DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,max(size(DSGEModel.PredictedX,2)*(isempty(DSGEModel.BVARX)==0),240*(isempty(DSGEModel.BVARX)==1)));
            %
            % matrix with exogenous variables
            %
            X = [];
            if isempty(DSGEModel.BVARX)==0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(DSGEModel.BVARX,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(DSGEModel.BVARX,1:NumPredX)];
               end;
            end;
            %
            % check if we should annualize or not
            %
            if gcbo==controls.bvarpredict.conditional.original.postmode;
               AnnualizeData = 0;
               AnnualStr = 'original';
            else;
               AnnualizeData = 1;
               AnnualStr = 'annual';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckConditioningData(DSGEModel);
            %
            % check if we should adjust the Z matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod);
            %
            % ask if we should conduct prediction event calculations
            %
            if NumPredPaths>1;
               if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                  DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
               end;
               txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
               answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               %
               % make sure that the prediction event length does not exceed the maximum prediction horizon
               %
               for i=1:DSGEModel.n;
                  if DSGEModel.PredictionEvent(i,3)>MaxPH;
                     DSGEModel.PredictionEvent(i,3) = MaxPH;
                  end;
               end;
               %
               % display a dialog for setting up an prediction event, but using only the selected BVAR endogenous variables
               %
               [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.PredictionEvent(DSGEModel.BVARY,:),MaxPH,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.PredictionEvent(DSGEModel.BVARY,:) = PredictionEvent;
                  DSGEModel.RunPredictionEvent = 1;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
            else;
               DSGEModel.RunPredictionEvent = 0;
            end;
            %
            % we can now compute the predictive distribution for the endogenous variables
            % up to period MaxPh
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
            else;
               randn('state',sum(100*clock));
            end;
            %
            % run the prediction path calculation function
            %
            [PredPaths,PredMean,MeanShocks,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,KernelX,KernelY] = BVARCondPredictionPathsPostMode(Psi,Pi,Omega,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData);
            if isempty(PredPaths)==0;
               %
               % the computations went OK. Compute point estimates from predictive distribution and
               % quantiles
               %
               [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
               %
               % compute the mean error bias in percent
               %
               MeanErrorBias = max(max(abs((PredMean-MeanPrediction)./PredMean)));
               if DSGEModel.AdjustPredictionPaths==0;
                  %
                  % ask if the prediction distribution should be corrected
                  %
                  txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust the conditional predictive distribution such that it matches the theoretical mean?'];
                  answer = Query(txt,'question',140,'Mean Error Bias - Conditional Predictive Dist',500,'yes',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % adjust the quantiles to match
                  %
                  for i=1:length(Quantiles);
                     OlVal = Quantiles(i).Quantile;
                     Quantiles(i).Quantile = Quantiles(i).Quantile+(PredMean-MeanPrediction);
                  end;
                  MeanPrediction = PredMean;
               end;
               %
               % compute the forecast errors based on the mean predictions
               %
               PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,0,'BVAR',DSGEModel);
               %
               % check if we have modesty statistics to deal with
               %
               if AnnualizeData==0;
                  %
                  % check how often we have a greater value of column 1 than column 2 in MultiModestyStat
                  %
                  MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/NumPredPaths;
                  MeanMultiModesty = (1/NumPredPaths)*sum(MultiModestyStat,1);
                  MeanUniModesty = (1/NumPredPaths)*sum(UniModestyStat,1);
                  if exist('YADAchi2cdf')==2;
                     ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY));
                  else;
                     if exist('chi2cdf')==2;
                        ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY));
                     else;
                        ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY));
                     end;
                  end;
                  PrintBVARModestyStatistics(DSGEModel,CurrINI,MaxPH,NumPeriods,1,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,UniModestyStatLZ,MeanShocks);
               else;
                  MultiPValue = [];
                  MeanMultiModesty = [];
                  MeanUniModesty = [];
                  ChiSqPValue = [];
               end;
               %
               % check if the user wants to save the data
               %
               txt = ['Would you like to save the conditional predictive distributions from the Bayesian VAR for the ' AnnualStr ' data at posterior mode to file?'];
               answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-BVAR-' AnnualStr '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.mat']);
                  YNames = DSGEModel.VariableNames(DSGEModel.BVARY,:);
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                  Y = DSGEModel.Y(DSGEModel.BVARY,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                  if length(DateVector)>size(Y,2);
                     DateVector = DateVector(1,1:size(Y,2));
                  end;
                  EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                  EstimationStartPeriodOrig = EstimationStartPeriod;
                  YOrig = Y;
                  DateVectorOrig = DateVector;
                  if AnnualizeData==1;
                     Y = Annualize(Y,DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
                     EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                     DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                  end;
                  MeanUniModestyLZ = UniModestyStatLZ;
                  if isempty(PredEventData)==1;
                     save(file,'MeanPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                  else;
                     save(file,'MeanPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData','YObsEventData');
                  end;
                  %
                  % create a matlab m-file to read the output.
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\CPDBVAR' AnnualStr int2str(DSGEModel.PriorType) int2str(DSGEModel.OmegaPriorType) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'PostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables in the BVAR\n%%\n');
                  fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                  fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                  fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                  fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                  fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                  if isempty(KernelX)==0;
                     fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                     fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                     KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                     KVal = 2;
                  else;
                     KStr = '';
                     KVal = 0;
                  end;
                  fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                  fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                  fprintf(fid,'%%\n%% observed variables\n%%\n');
                  fprintf(fid,'Y = DataStructure.Y;\n');
                  fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  if isempty(PredEventData)==0;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                     NumEntries = int2str(10+KVal);
                     ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with similar information but for mean predictions given by the observations), ';
                  else;
                     NumEntries = int2str(7+KVal);
                     ExtraEntry = '';
                  end;
                  if AnnualizeData==0;
                     fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                     fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                     fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                     fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                     fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                     fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                     fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                     fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                     fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                     fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                     NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                     ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  txt = ['The conditional predictive distributions from the Bayesian VAR have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Conditional Predictive Distributions',500,600,CurrINI);
               end;
               if isempty(PredEventData)==0;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-BVAR-' AnnualStr '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames(DSGEModel.BVARY,:),PredEventData,YObsEventData,DSGEModel.PredictionEvent(DSGEModel.BVARY,:),[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                  end;
               end;
               %
               % Open a dialog for displaying predictive distributions
               %
               CondPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,KernelX,KernelY,'Posterior Mode','BVAR',AnnualStr,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARPredictPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            NumDraws = length(LogPsiDensity);
            %
            % clear what is not needed
            %
            clear('LogPsiDensity','LogPiDensity');
            %
            % set up out-of-sample data for the exogenous variable
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%            DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,max(size(DSGEModel.PredictedX,2)*(isempty(DSGEModel.BVARX)==0),240*(isempty(DSGEModel.BVARX)==1)));
            %
            % matrix with exogenous variables
            %
            X = [];
            if isempty(DSGEModel.BVARX)==0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(DSGEModel.BVARX,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(DSGEModel.BVARX,1:NumPredX)];
               end;
            end;
            %
            % initialize the random number generator
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
               rand('state',0);
            else;
               randn('state',sum(100*clock));
               rand('state',sum(100*clock));
            end;
            %
            % select which posterior draws to use
            %
            PsiPostSample = PsiPostSample(NumBurnin+1:NumDraws,:);
            PiPostSample = PiPostSample(NumBurnin+1:NumDraws,:);
            OmegaPostSample = OmegaPostSample(NumBurnin+1:NumDraws,:);
            NumDraws = NumDraws-NumBurnin;
            %
            % check how many draws we should pick among these
            %
            PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
            NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
            %
            % get the draws to use
            %
            if DSGEModel.RandomizeDraws==0;
               DrawFreq = round(NumDraws/NumPostDrawsUsage);
               SelectDraws = (1:DrawFreq:NumDraws);
               if length(SelectDraws)+1==NumPostDrawsUsage;
                  SelectDraws = [SelectDraws NumDraws];
               elseif length(SelectDraws)>NumPostDrawsUsage;
                  SelectDraws = SelectDraws(1:NumPostDrawsUsage);
               end;
               PsiPostSample = PsiPostSample(SelectDraws,:);
               PiPostSample = PiPostSample(SelectDraws,:);
               OmegaPostSample = OmegaPostSample(SelectDraws,:);
            else;
               PsiPostSample = PsiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               PiPostSample = PiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               OmegaPostSample = OmegaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
            end;
            %
            % are we using annualized data or the original
            %
            if gcbo==controls.bvarpredict.unconditional.original.posterior;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'Predictive Distributions For BVAR - Original Data';
            else;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'Predictive Distributions For BVAR - Annualized Data';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if results have already been stored to disk
            %
            if BVARPredictionsExist(DSGEModel.OutputDirectory,'',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData)==1;
               txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
               answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               %
               % load the data from disk
               %
               [PredEventData,PredictionEvent,ShockMean,ShockCov,ParameterCov] = LoadBVARPredictionUncertainty(DSGEModel.OutputDirectory,'',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData);
               DoneCalc = 1;
            else;
               %
               % ask if we should conduct prediction event calculations
               %
               if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                  DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
               end;
               txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
               answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  %
                  % make sure that the prediction event length does not exceed the maximum prediction horizon
                  %
                  for i=1:DSGEModel.n;
                     if DSGEModel.PredictionEvent(i,3)>MaxPH;
                        DSGEModel.PredictionEvent(i,3) = MaxPH;
                     end;
                  end;
                  %
                  % display a dialog for setting up an prediction event, but using only the selected BVAR endogenous variables
                  %
                  [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.PredictionEvent(DSGEModel.BVARY,:),MaxPH,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PredictionEvent(DSGEModel.BVARY,:) = PredictionEvent;
                     DSGEModel.RunPredictionEvent = 1;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
               %
               % run the prediction path calculation function
               %
               [DoneCalc,PredEventData,ShockCov,ParameterCov] = BVARPredictionPaths(PsiPostSample,PiPostSample,OmegaPostSample,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData);
            end;
            if DoneCalc==1;
               %
               % the computations went OK. Compute point estimates from predictive distribution and
               % quantiles
               %
               [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = BVARPredictionDistribution(DSGEModel.OutputDirectory,'',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,DSGEModel.Percentiles,DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.ConfidenceRegionMethod,CurrINI);
               %
               % compute the forecast errors based on the mean predictions
               %
               PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,0,'BVAR',DSGEModel);
               %
               % check if the user wants to save the data
               %
               txt = ['Would you like to save the predictive distributions from the Bayesian VAR for the ' AnnualStr ' data to file?'];
               answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                  file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-BVAR-' AnnualStr '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                  YNames = DSGEModel.VariableNames(DSGEModel.BVARY,:);
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                  Y = DSGEModel.Y(DSGEModel.BVARY,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                  if length(DateVector)>size(Y,2);
                     DateVector = DateVector(1,1:size(Y,2));
                  end;
                  EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                  EstimationStartPeriodOrig = EstimationStartPeriod;
                  YOrig = Y;
                  DateVectorOrig = DateVector;
                  if AnnualizeData==1;
                     Y = Annualize(Y,DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
                     EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                     DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                  end;
                  if isempty(PredEventData)==1;
                     save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','ShockCov','ParameterCov');
                  else;
                     save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','ShockCov','ParameterCov','PredictionEvent','PredEventData');
                  end;
                  %
                  % create a matlab m-file to read the output.
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\PDBVAR' AnnualStr num2str(DSGEModel.PriorType,'%0.0f') num2str(DSGEModel.OmegaPriorType,'%0.0f') num2str(MaxPH,'%0.0f') num2str(NumPostDrawsUsage,'%0.0f') num2str(NumDraws,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables in the BVAR\n%%\n');
                  fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                  fprintf(fid,'%%\n%% 3D matrix (pxpxh) with the sample covariance of the predictions of the observed variables in the BVAR\n%%\n');
                  fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                  fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                  fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                  fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                  fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                  fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                  fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                  fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                  fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                  fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                  fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                  fprintf(fid,'%%\n%% observed variables\n%%\n');
                  fprintf(fid,'Y = DataStructure.Y;\n');
                  fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  if AnnualizeData==0;
                     fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to residual/shock uncertainty\n%%\n');
                     fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                     fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to parameter uncertainty\n%%\n');
                     fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                     CovDecompStr = 'ShockCov (3D error covariance matrix with part due to shock uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                  else;
                     CovDecompStr = '';
                  end;
                  if isempty(PredEventData)==0;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     if AnnualizeData==0;
                        NumEntries = '14';
                     else;
                        NumEntries = '12';
                     end;
                     ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                  else;
                     if AnnualizeData==0;
                        NumEntries = '12';
                     else;
                        NumEntries = '10';
                     end;
                     ExtraEntry = '';
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  txt = ['The predictive distributions from the Bayesian VAR have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Predictive Distributions',500,600,CurrINI);
               end;
               if isempty(PredEventData)==0;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-BVAR-' AnnualStr '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames(DSGEModel.BVARY,:),PredEventData,[],DSGEModel.PredictionEvent(DSGEModel.BVARY,:),[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                  end;
               end;
               %
               % summarize the information in the uncertainty variables more conveniently
               %
               if isempty(ShockCov)==0;
                  [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(length(DSGEModel.BVARY),length(DSGEModel.BVARY),MaxPH),ShockCov,zeros(length(DSGEModel.BVARY),length(DSGEModel.BVARY)),ParameterCov);
                  %
                  % Open a dialog for displaying predictive distributions
                  %
                  UData.ShockVar = ShockVar;
                  UData.ParameterVar = ParameterVar;
                  [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
               else;
                  UData = [];
               end;
               PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,KernelX,KernelY,'','BVAR',AnnualStr,1,UData);
            end;
            %
            % clear everything that remains of the posterior draws
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARCondPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            NumDraws = length(LogPsiDensity);
            %
            % clear what is not needed
            %
            clear('LogPsiDensity','LogPiDensity');
            %
            % set up out-of-sample data for the exogenous variable
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%            DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,max(size(DSGEModel.PredictedX,2)*(isempty(DSGEModel.BVARX)==0),240*(isempty(DSGEModel.BVARX)==1)));
            %
            % matrix with exogenous variables
            %
            X = [];
            if isempty(DSGEModel.BVARX)==0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(DSGEModel.BVARX,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(DSGEModel.BVARX,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckConditioningData(DSGEModel);
            %
            % check if we should adjust the Z matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod);
            %
            % initialize the random number generator
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
               rand('state',0);
            else;
               randn('state',sum(100*clock));
               rand('state',sum(100*clock));
            end;
            %
            % select which posterior draws to use
            %
            PsiPostSample = PsiPostSample(NumBurnin+1:NumDraws,:);
            PiPostSample = PiPostSample(NumBurnin+1:NumDraws,:);
            OmegaPostSample = OmegaPostSample(NumBurnin+1:NumDraws,:);
            NumDraws = NumDraws-NumBurnin;
            %
            % check how many draws we should pick among these
            %
            PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
            NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
            %
            % get the draws to use
            %
            if DSGEModel.RandomizeDraws==0;
               DrawFreq = round(NumDraws/NumPostDrawsUsage);
               SelectDraws = (1:DrawFreq:NumDraws);
               if length(SelectDraws)+1==NumPostDrawsUsage;
                  SelectDraws = [SelectDraws NumDraws];
               elseif length(SelectDraws)>NumPostDrawsUsage;
                  SelectDraws = SelectDraws(1:NumPostDrawsUsage);
               end;
               PsiPostSample = PsiPostSample(SelectDraws,:);
               PiPostSample = PiPostSample(SelectDraws,:);
               OmegaPostSample = OmegaPostSample(SelectDraws,:);
            else;
               PsiPostSample = PsiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               PiPostSample = PiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               OmegaPostSample = OmegaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
            end;
            %
            % are we using annualized data or the original
            %
            if gcbo==controls.bvarpredict.conditional.original.posterior;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'Conditional Predictive Dist For BVAR - Original Data';
            else;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'Conditional Predictive Dist For BVAR - Annualized Data';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if results have already been stored to disk
            %
            if BVARPredictionsExist(DSGEModel.OutputDirectory,'Cond',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData)==1;
               txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
               answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               %
               % load the data from disk
               %
               [PredEventData,PredictionEvent,ShockMean,ShockCov,ParameterCov] = LoadBVARPredictionUncertainty(DSGEModel.OutputDirectory,'Cond',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData);
               DoneCalc = 1;
            else;
               %
               % ask if we should conduct prediction event calculations
               %
               if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                  DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
               end;
               txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
               answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  %
                  % make sure that the prediction event length does not exceed the maximum prediction horizon
                  %
                  for i=1:DSGEModel.n;
                     if DSGEModel.PredictionEvent(i,3)>MaxPH;
                        DSGEModel.PredictionEvent(i,3) = MaxPH;
                     end;
                  end;
                  %
                  % display a dialog for setting up an prediction event, but using only the selected BVAR endogenous variables
                  %
                  [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.PredictionEvent(DSGEModel.BVARY,:),MaxPH,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PredictionEvent(DSGEModel.BVARY,:) = PredictionEvent;
                     DSGEModel.RunPredictionEvent = 1;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
               %
               % run the prediction path calculation function
               %
               [DoneCalc,PredEventData,ShockMean,ShockCov,ParameterCov] = BVARCondPredictionPaths(PsiPostSample,PiPostSample,OmegaPostSample,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData);
            end;
            %
            if DoneCalc==1;
               %
               % the computations went OK. Compute point estimates from predictive distribution and
               % quantiles
               %
               [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = BVARPredictionDistribution(DSGEModel.OutputDirectory,'Cond',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,DSGEModel.Percentiles,DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.ConfidenceRegionMethod,CurrINI);
               %
               % compute the forecast errors based on the mean predictions
               %
               PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,0,'BVAR',DSGEModel);
               %
               % check if we should evaluate modesty test data
               %
               if AnnualizeData==0;
                  [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = BVARModestyStatistics(DSGEModel.OutputDirectory,DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,length(DSGEModel.BVARY));
                  if exist('YADAchi2cdf')==2;
                     ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY));
                  else;
                     if exist('chi2cdf')==2;
                        ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY));
                     else;
                        ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY));
                     end;
                  end;
                  %
                  % print results to file
                  %
                  PrintBVARModestyStatistics(DSGEModel,CurrINI,MaxPH,NumPeriods,NumPostDrawsUsage,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean);
               else;
                  ChiSqPValue = [];
                  MultiPValue = [];
                  MeanMultiModesty = [];
                  MeanUniModesty = [];
                  MeanUniModestyLZ = [];
               end;
               txt = ['Would you like to save the conditional predictive distributions from the Bayesian VAR for the ' AnnualStr ' data to file?'];
               answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-BVAR-' AnnualStr '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                  YNames = DSGEModel.VariableNames(DSGEModel.BVARY,:);
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                  Y = DSGEModel.Y(DSGEModel.BVARY,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                  if length(DateVector)>size(Y,2);
                     DateVector = DateVector(1,1:size(Y,2));
                  end;
                  EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                  EstimationStartPeriodOrig = EstimationStartPeriod;
                  YOrig = Y;
                  DateVectorOrig = DateVector;
                  if AnnualizeData==1;
                     Y = Annualize(Y,DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
                     EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                     DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                  end;
                  if isempty(PredEventData)==1;
                     save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','ShockCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                  else;
                     save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','ShockCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData');
                  end;
                  %
                  % create a matlab m-file to read the output.
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\CPDBVAR' AnnualStr int2str(DSGEModel.PriorType) int2str(DSGEModel.OmegaPriorType) int2str(MaxPH) int2str(NumPostDrawsUsage) int2str(NumDraws) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables in the BVAR\n%%\n');
                  fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                  fprintf(fid,'%%\n%% 3D matrix (pxpxh) with the sample covariance of the predictions of the observed variables in the BVAR\n%%\n');
                  fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                  fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                  fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                  fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                  fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                  fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                  fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                  fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                  fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                  fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                  fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                  fprintf(fid,'%%\n%% observed variables\n%%\n');
                  fprintf(fid,'Y = DataStructure.Y;\n');
                  fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  if AnnualizeData==0;
                     fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to residual/shock uncertainty\n%%\n');
                     fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                     fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to parameter uncertainty\n%%\n');
                     fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                     fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                     fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                     fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                     fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                     fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                     fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                     fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                     fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                     fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                     fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                     CovDecompStr = 'ShockCov (3D error covariance matrix with part due to shock uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), ';
                  else;
                     CovDecompStr = '';
                  end;
                  if isempty(PredEventData)==0;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     if AnnualizeData==0;
                        NumEntries = '19';
                     else;
                        NumEntries = '12';
                     end;
                     ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                  else;
                     if AnnualizeData==0;
                        NumEntries = '17';
                     else;
                        NumEntries = '10';
                     end;
                     ExtraEntry = '';
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  txt = ['The conditional predictive distributions from the Bayesian VAR have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Conditional Predictive Distributions',500,600,CurrINI);
               end;
               if isempty(PredEventData)==0;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-BVAR-' AnnualStr '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames(DSGEModel.BVARY,:),PredEventData,[],DSGEModel.PredictionEvent(DSGEModel.BVARY,:),[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                  end;
               end;
               %
               % summarize the information in the uncertainty variables more conveniently
               %
               if isempty(ShockCov)==0;
                  [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(length(DSGEModel.BVARY),length(DSGEModel.BVARY),MaxPH),ShockCov,zeros(length(DSGEModel.BVARY),length(DSGEModel.BVARY)),ParameterCov);
                  %
                  % Open a dialog for displaying predictive distributions
                  %
                  UData.ShockVar = ShockVar;
                  UData.ParameterVar = ParameterVar;
                  [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
               else;
                  UData = [];
               end;
               %
               % Open a dialog for displaying predictive distributions
               %
               CondPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,Quantiles,KernelX,KernelY,'','BVAR',AnnualStr,1,UData);
            end;
            %
            % clear everything that remains of the posterior draws
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARPredictionEvent'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            NumDraws = length(LogPsiDensity);
            %
            % clear what is not needed
            %
            clear('LogPsiDensity','LogPiDensity');
            %
            % set up out-of-sample data for the exogenous variable
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%            DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,max(size(DSGEModel.PredictedX,2)*(isempty(DSGEModel.BVARX)==0),240*(isempty(DSGEModel.BVARX)==1)));
            %
            % matrix with exogenous variables
            %
            X = [];
            if isempty(DSGEModel.BVARX)==0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(DSGEModel.BVARX,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(DSGEModel.BVARX,1:NumPredX)];
               end;
            end;
            %
            % initialize the random number generator
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
               rand('state',0);
            else;
               randn('state',sum(100*clock));
               rand('state',sum(100*clock));
            end;
            %
            % select which posterior draws to use
            %
            PsiPostSample = PsiPostSample(NumBurnin+1:NumDraws,:);
            PiPostSample = PiPostSample(NumBurnin+1:NumDraws,:);
            OmegaPostSample = OmegaPostSample(NumBurnin+1:NumDraws,:);
            NumDraws = NumDraws-NumBurnin;
            %
            % check how many draws we should pick among these
            %
            PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
            NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
            %
            % get the draws to use
            %
            if DSGEModel.RandomizeDraws==0;
               DrawFreq = round(NumDraws/NumPostDrawsUsage);
               SelectDraws = (1:DrawFreq:NumDraws);
               if length(SelectDraws)+1==NumPostDrawsUsage;
                  SelectDraws = [SelectDraws NumDraws];
               elseif length(SelectDraws)>NumPostDrawsUsage;
                  SelectDraws = SelectDraws(1:NumPostDrawsUsage);
               end;
               PsiPostSample = PsiPostSample(SelectDraws,:);
               PiPostSample = PiPostSample(SelectDraws,:);
               OmegaPostSample = OmegaPostSample(SelectDraws,:);
            else;
               PsiPostSample = PsiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               PiPostSample = PiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               OmegaPostSample = OmegaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
            end;
            %
            % are we using annualized data or the original
            %
            if gcbo==controls.bvarpredict.unconditional.original.predevent;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'Predictive Distributions For Bayesian VAR - Original Data';
            else;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'Predictive Distributions For Bayesian VAR - Annualized Data';
            end;
            %
            % ask if we should conduct prediction event calculations
            %
            if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
               DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
            end;
            %
            % make sure that the prediction event length does not exceed the maximum prediction horizon
            %
            for i=1:DSGEModel.n;
               if DSGEModel.PredictionEvent(i,3)>MaxPH;
                  DSGEModel.PredictionEvent(i,3) = MaxPH;
               end;
            end;
            %
            % display a dialog for setting up an prediction event, but using only the selected BVAR endogenous variables
            %
            [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.PredictionEvent(DSGEModel.BVARY,:),MaxPH,CurrINI);
            %
            % check if the cancel button was pressed or not
            %
            if strcmp(lower(action),'ok')==1;
               DSGEModel.PredictionEvent(DSGEModel.BVARY,:) = PredictionEvent;
               DSGEModel.RunPredictionEvent = 1;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % check if results have already been stored to disk
               %
               if BVARPredictionsExist(DSGEModel.OutputDirectory,'',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData)==1;
                  txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % load the data from disk
                  %
                  PredEventData = LoadBVARPredictionPaths(DSGEModel.PredictionEvent(DSGEModel.BVARY,:),DSGEModel.OutputDirectory,'',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,CurrINI);
                  DoneCalc = 1;
               else;
                  %
                  % run the prediction path calculation function
                  %
                  [DoneCalc,PredEventData,ShockCov,ParameterCov] = BVARPredictionPaths(PsiPostSample,PiPostSample,OmegaPostSample,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData);
               end;
               if DoneCalc==1;
                  %
                  % check if summary results have previously been saved to disk
                  %
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-BVAR-' AnnualStr '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                  if FileExist(file)==1;
                     save(file,'PredictionEvent','PredEventData','-APPEND');
                     %
                     % create a matlab m-file to read the output.
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\PDBVAR' AnnualStr num2str(DSGEModel.PriorType,'%0.0f') num2str(DSGEModel.OmegaPriorType,'%0.0f') num2str(MaxPH,'%0.0f') num2str(NumPostDrawsUsage,'%0.0f') num2str(NumDraws,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables in the BVAR\n%%\n');
                     fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                     fprintf(fid,'%%\n%% 3D matrix (pxpxh) with the sample covariance of the predictions of the observed variables in the BVAR\n%%\n');
                     fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                     fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                     fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                     fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                     fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                     fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                     fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                     fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                     fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                     fprintf(fid,'%%\n%% observed variables\n%%\n');
                     fprintf(fid,'Y = DataStructure.Y;\n');
                     fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     if AnnualizeData==0;
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to residual/shock uncertainty\n%%\n');
                        fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to parameter uncertainty\n%%\n');
                        fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                     end;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                  end;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-BVAR-' AnnualStr '-' num2str(DSGEModel.PriorType,'%0.0f') '.' num2str(DSGEModel.OmegaPriorType,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames(DSGEModel.BVARY,:),PredEventData,[],DSGEModel.PredictionEvent(DSGEModel.BVARY,:),[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                  end;
               end;
            end;
            %
            % clear everything that remains of the posterior draws
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARCondPredictionEvent'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            NumDraws = length(LogPsiDensity);
            %
            % clear what is not needed
            %
            clear('LogPsiDensity','LogPiDensity');
            %
            % set up out-of-sample data for the exogenous variable
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%            DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,max(size(DSGEModel.PredictedX,2)*(isempty(DSGEModel.BVARX)==0),240*(isempty(DSGEModel.BVARX)==1)));
            %
            % matrix with exogenous variables
            %
            X = [];
            if isempty(DSGEModel.BVARX)==0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(DSGEModel.BVARX,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(DSGEModel.BVARX,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckConditioningData(DSGEModel);
            %
            % check if we should adjust the Z matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod);
            %
            % initialize the random number generator
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
               rand('state',0);
            else;
               randn('state',sum(100*clock));
               rand('state',sum(100*clock));
            end;
            %
            % select which posterior draws to use
            %
            PsiPostSample = PsiPostSample(NumBurnin+1:NumDraws,:);
            PiPostSample = PiPostSample(NumBurnin+1:NumDraws,:);
            OmegaPostSample = OmegaPostSample(NumBurnin+1:NumDraws,:);
            NumDraws = NumDraws-NumBurnin;
            %
            % check how many draws we should pick among these
            %
            PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
            NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
            %
            % get the draws to use
            %
            if DSGEModel.RandomizeDraws==0;
               DrawFreq = round(NumDraws/NumPostDrawsUsage);
               SelectDraws = (1:DrawFreq:NumDraws);
               if length(SelectDraws)+1==NumPostDrawsUsage;
                  SelectDraws = [SelectDraws NumDraws];
               elseif length(SelectDraws)>NumPostDrawsUsage;
                  SelectDraws = SelectDraws(1:NumPostDrawsUsage);
               end;
               PsiPostSample = PsiPostSample(SelectDraws,:);
               PiPostSample = PiPostSample(SelectDraws,:);
               OmegaPostSample = OmegaPostSample(SelectDraws,:);
            else;
               PsiPostSample = PsiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               PiPostSample = PiPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
               OmegaPostSample = OmegaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
            end;
            %
            % are we using annualized data or the original
            %
            if gcbo==controls.bvarpredict.conditional.original.predevent;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'Conditional Predictive Dist For BVAR - Original Data';
            else;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'Conditional Predictive Dist For BVAR - Annualized Data';
            end;
            %
            % setup prediction event calculations
            %
            if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
               DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
            end;
            %
            % make sure that the prediction event length does not exceed the maximum prediction horizon
            %
            for i=1:DSGEModel.n;
               if DSGEModel.PredictionEvent(i,3)>MaxPH;
                  DSGEModel.PredictionEvent(i,3) = MaxPH;
               end;
            end;
            %
            % display a dialog for setting up an prediction event, but using only the selected BVAR endogenous variables
            %
            [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames(DSGEModel.BVARY,:),DSGEModel.PredictionEvent(DSGEModel.BVARY,:),MaxPH,CurrINI);
            %
            % check if the cancel button was pressed or not
            %
            if strcmp(lower(action),'ok')==1;
               DSGEModel.PredictionEvent(DSGEModel.BVARY,:) = PredictionEvent;
               DSGEModel.RunPredictionEvent = 1;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % check if results have already been stored to disk
               %
               if BVARPredictionsExist(DSGEModel.OutputDirectory,'Cond',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData)==1;
                  txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % load the data from disk
                  %
                  PredEventData = LoadBVARPredictionPaths(DSGEModel.PredictionEvent(DSGEModel.BVARY,:),DSGEModel.OutputDirectory,'Cond',DSGEModel.NameOfModel,DSGEModel.PriorType,DSGEModel.OmegaPriorType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,CurrINI);
                  DoneCalc = 1;
               else;
                  %
                  % run the prediction path calculation function
                  %
                  [DoneCalc,PredEventData,ShockMean,ShockCov,ParameterCov] = BVARCondPredictionPaths(PsiPostSample,PiPostSample,OmegaPostSample,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData);
               end;
               if DoneCalc==1;
                  %
                  % check if summary results have previously been saved to disk
                  %
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-BVAR-' AnnualStr '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                  if FileExist(file)==1;
                     save(file,'PredictionEvent','PredEventData','-APPEND');
                     %
                     % create a matlab m-file to read the output.
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\CPDBVAR' AnnualStr int2str(DSGEModel.PriorType) int2str(DSGEModel.OmegaPriorType) int2str(MaxPH) int2str(NumPostDrawsUsage) int2str(NumDraws) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables in the BVAR\n%%\n');
                     fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                     fprintf(fid,'%%\n%% 3D matrix (pxpxh) with the sample covariance of the predictions of the observed variables in the BVAR\n%%\n');
                     fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                     fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                     fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                     fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(DSGEModel.Percentiles));
                     fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                     fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                     fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                     fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                     fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                     fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                     fprintf(fid,'%%\n%% observed variables\n%%\n');
                     fprintf(fid,'Y = DataStructure.Y;\n');
                     fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     if AnnualizeData==0;
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to residual/shock uncertainty\n%%\n');
                        fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                        fprintf(fid,'%%\n%% 3D Error covariance matrix (pxpxh) part due to parameter uncertainty\n%%\n');
                        fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                        fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                        fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                        fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                        fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                        fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                        fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                        fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                        fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                        fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                        fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                     end;
                     fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                     fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                     fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                     fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                  end;
                  %
                  % write prediction event results to file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\CondPredictiveDist-BVAR-' AnnualStr '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                  [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                  PrintPredictionEvent(file,DSGEModel.VariableNames(DSGEModel.BVARY,:),PredEventData,[],DSGEModel.PredictionEvent(DSGEModel.BVARY,:),[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                  %
                  % displays the file if it exists
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                  end;
               end;
            end;
            %
            % clear everything that remains of the posterior draws
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARPsiRawDraws'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % send the data to the raw plotting function
            %
            drawnow;
            RawBVARPosteriorPlots('Psi',PsiPostSample,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARPiRawDraws'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % send the data to the raw plotting function
            %
            drawnow;
            RawBVARPosteriorPlots('Pi',PiPostSample,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVAROmegaRawDraws'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % send the data to the raw plotting function
            %
            drawnow;
            RawBVARPosteriorPlots('Omega',OmegaPostSample,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'MargLikelihoodChib'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % run the sequential estimation routine
            %
            drawnow;
            PlotMargLikeChib(LogPsiDensity,LogPiDensity,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARMeanMedianConvergence'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % determine which data to use
            %
            if gcbo==controls.bvarpsimeanconvergence;
               PostSample = PsiPostSample;
               ParamStr = 'Psi';
               MomentType = 'Mean';
            elseif gcbo==controls.bvarpimeanconvergence;
               PostSample = PiPostSample;
               ParamStr = 'Pi';
               MomentType = 'Mean';
            elseif gcbo==controls.bvaromegameanconvergence;
               PostSample = OmegaPostSample;
               ParamStr = 'Omega';
               MomentType = 'Mean';
            elseif gcbo==controls.bvarpsimedianconvergence;
               PostSample = PsiPostSample;
               ParamStr = 'Psi';
               MomentType = 'Median';
            elseif gcbo==controls.bvarpimedianconvergence;
               PostSample = PiPostSample;
               ParamStr = 'Pi';
               MomentType = 'Median';
            elseif gcbo==controls.bvaromegamedianconvergence;
               PostSample = OmegaPostSample;
               ParamStr = 'Omega';
               MomentType = 'Median';
            end;
            %
            % send the data to the posterior densities plotting function
            %
            drawnow;
            PosteriorBVARMeanMedianPlots(PostSample,ParamStr,MomentType,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'BVARModestyStatistics'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % retrieve the results
         %
         ModestyFile = get(gcbo,'UserData');
         LabelStr = strrep(strrep(get(gcbo,'Label'),'&',''),'...','');
         %
         TextGUI(ModestyFile,['Modesty Statistics - ' LabelStr],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,1);
         %
         set(controls.about,'UserData',0);
      end;
   case 'BVARPsiPriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % contruct the plots
         %
         drawnow;
         GraphBVARPriorDensity('Psi',DSGEModel,CurrINI)
         set(controls.about,'UserData',0);
      end;
   case 'BVARPiPriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % contruct the plots
         %
         drawnow;
         GraphBVARPriorDensity('Pi',DSGEModel,CurrINI)
         set(controls.about,'UserData',0);
      end;
   case 'BVAROmegaPriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % contruct the plots
         %
         drawnow;
         GraphBVARPriorDensity('Omega',DSGEModel,CurrINI)
         set(controls.about,'UserData',0);
      end;
   case 'BVARPsiPosteriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % send the data to the posterior densities plotting function
            %
            drawnow;
            GraphBVARPosteriorDensity('Psi',PsiPostSample,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVARPiPosteriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % send the data to the posterior densities plotting function
            %
            drawnow;
            GraphBVARPosteriorDensity('Pi',PiPostSample,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'BVAROmegaPosteriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Check if the data actually exists
         %
         if VerifyBVARPosteriorDraws(DSGEModel,controls)==1;
            %
            % load the data
            %
            [PsiPostSample,PiPostSample,OmegaPostSample,LogPsiDensity,LogPiDensity,NumBurnin] = LoadBVARPosteriorDraws(DSGEModel,CurrINI);
            %
            % send the data to the posterior densities plotting function
            %
            drawnow;
            GraphBVARPosteriorDensity('Omega',OmegaPostSample,NumBurnin,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('PsiPostSample','PiPostSample','OmegaPostSample','LogPsiDensity','LogPiDensity');
         else;
            txt = 'Some of the files for loading the posterior data for the Bayesian VAR are not available.';
            About(txt,'error','Bayesian VAR - Missing Posterior Draws',150,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of BVARMenuFunctions.m
%
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        