function DSGEVARCorrToolsMenuFunctions(selector)
% DSGEVARCorrToolsMenuFunctions: switch function for running functions on the DSGE-VAR menu.
%
% USAGE:
%
%       DSGEVARMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 15, 2010.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-06-2010: Began work on the posterior mode code for observed variable correlations
%               for the time domain.
%
% * 17-06-2010: Finished the posterior mode case and the initial values. Began work on
%               observed variable correlations for the posterior distribution.
%
% * 18-06-2010: Finished the posterior distribution case for the observed variable
%               correlations. Added the prior distribution case as well.
%
% * 21-06-2010: Completed coherence for the posterior mode case.
%
% * 22-06-2010: Completed coherence for the initial values, the prior and the posterior
%               distribution of the same functionality.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 20-01-2011: Began work on conditional correlations for the marginal and joint posterior
%               mode estimates.
%
% * 21-01-2011: Continued work on the conditional correlations.
%
% * 24-01-2011: Further work on the conditional correlations.
%
% * 25-01-2011: Finished the conditional correlations for the prior draws.
%
% * 28-02-2011: Added code to handle the "DSGE Posterior Mode" cases.
%
% * 25-11-2011: The function now supports skewness computations for observed variable correlations.
%
% * 02-12-2011: The function now supports kurtosis calculations for the observed variables.
%
% * 02-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter value can
%               be unity.
%
% * 11-05-2012: Took into account that "LoadVARDSGEPosteriorDraws" now provides "thetaPostSample" as an
%               output variable.
%
% * 14-05-2012: Took into account that the same function now also provides "PostSample".
%
% * 03-01-2013: Updated the documentation.
%
% * 23-05-2013: Added information when asking if sample moments should be simulated.
%
% * 28-01-2014: Updated the documentation.
%
% * 13-03-2014: Updated the documentation concerning lambda-values.
%
% * 14-03-2014: Added cases for shock correlations and began word on the posterior mode estimates.
%
% * 17-03-2014: Continued work on the code for the shock correlations, finishing the posterior mode codee.
%
% * 18-03-2014: Finished the initial values code and began with the posterior draws code for the
%               shock correlations.
%
% * 19-03-2014: Finished the code for the posteror distribution draws of the shock correlations.
%
% * 20-03-2014: Finished the code for the prior distribution draws for the shock correlations.
%
% * 25-03-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 11-09-2017: Took into account that "LoadVARDSGEPosteriorDraws" provides the output variable "Weights".
%               The functions "DSGEVARCoherenceDistribution", "DSGEVARCorrelation Distribution",
%               "DSGEVARCorrSimulationDistribution", "DSGEVARCondCorrDistribution", and
%               "DSGEVARShockCorrelationDistribution" require the input variable "Weights".
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
lasterr = '';
%
switch selector
   case 'CoherenceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Coherence - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  if IsPositiveDefinite(SigmaEpsilon)==1;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     %
                     % compute coherence
                     %
                     [SOmega,Omega] = DSGEVARCoherenceParam(Phi,SigmaEpsilon,lambdai,'Initial Values',DSGEModel,CurrINI);
                     if isempty(SOmega)==0;
                        %
                        % check if the user wants to save the results to disk
                        %
                        txt = 'Would you like to save the coherence results for the observed variables at initial values to file?';
                        answer = Query(txt,'question',140,'Save - Coherence',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Coherence-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-InitialValues.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SOmega','VariableNames','Omega');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCoherence' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% matrix with coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                           fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                           fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                           fprintf(fid,'Omega = DataStructure.Omega;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The coherence results for the observed variables at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: VariableNames (string matrix with the names of the observed variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), and Omega (vector with the different frequencies from 0 to pi). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Coherence - Initial Values',400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the results
                        %
                        CoherenceDistributionDLG('init',SOmega,Omega,[],DSGEModel.VariableNames,'Observed','Initial Values',['DSGE-VAR(' num2str(lambdai) ') '],DSGEModel,CurrINI);
                     end;
                  else;
                     txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive definite. YADA is therefore unable to compute the coherence statistic.'];
                     About(txt,'information','Coherence - Initial Values',300,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CoherencePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         %
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               if IsPositiveDefinite(SigmaEpsilon)==1;
                  %
                  % determine which model to consider
                  %
                  [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Coherence Distribution - Prior',DSGEModel,CurrINI);
                  if isempty(lambda)==0;
                     T = HSample(2)-HSample(1)+1;
                     if isinf(lambda)==0;
                        SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     %
                     % load the prior draws data
                     %
                     [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                     UsedDraws = size(PhiDraws,1);
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % check if calculations have already been performed
                     %
                     if DSGEVARCoherenceExist(UsedDraws,lambda,[],[],0,DSGEModel,controls)==1;
                        txt = 'YADA has located coherence results for the observed variables on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Coherence Distribution - Prior',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     %
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % compute the distribution
                        %
                        DoneCalc = DSGEVARCoherence(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,[],0,[],lambda,DSGEModel,CurrINI,controls);
                     end;
                     %
                     % clear all draws
                     %
                     clear('PhiDraws','SigmaDraws');
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the impulse responses
                        %
                        [SOmega,Omega,Quantiles] = DSGEVARCoherenceDistribution(UsedDraws,lambda,[],[],0,[],DSGEModel,CurrINI,controls);
                        %
                        % ask if we should save results to disk
                        %
                        txt = ['Would you like to save the prior distributions of the coherence statistics for the observed variables to file?'];
                        answer = Query(txt,'question',140,'Save - Coherence Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Coherence-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-PriorDist.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SOmega','VariableNames','Omega','Quantiles');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCoherence' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) 'Prior.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% matrix with mean coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                           fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                           fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                           fprintf(fid,'Omega = DataStructure.Omega;\n');
                           fprintf(fid,'%%\n%% vector structure with quantile data through the fields SOmega and percent\n%%\n');
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The prior distribution of the coherence statistics for the observed variables have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: VariableNames (string matrix with the names of the observed variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), Omega (vector with the different frequencies from 0 to pi), and Quantiles (vector structure with quantile data in the fields SOmega and percent). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Coherence Distribution - Prior',400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the results
                        %
                        CoherenceDistributionDLG('init',SOmega,Omega,Quantiles,DSGEModel.VariableNames,'Observed','Prior Distribution',['DSGE-VAR(' num2str(lambda) ') '],DSGEModel,CurrINI);
                     end;
                  end;
               else;
                  txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive definite. YADA is therefore unable to compute the coherence statistic.'];
                  About(txt,'information','Coherence Distribution - Prior',300,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CoherencePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.obscorr.frequencydomain.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.obscorr.frequencydomain.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.obscorr.frequencydomain.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Coherence - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  SigmaEpsilon = ModeData.SigmaModeEpsilon;
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               end;
               if ModeStatus==1;
                  if IsPositiveDefinite(SigmaEpsilon)==1;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     %
                     % compute coherence
                     %
                     [SOmega,Omega] = DSGEVARCoherenceParam(Phi,SigmaEpsilon,lambdai,ModeType,DSGEModel,CurrINI);
                     if isempty(SOmega)==0;
                        %
                        % check if the user wants to save the results to disk
                        %
                        txt = ['Would you like to save the coherence results for the observed variables at ' lower(ModeType) ' to file?'];
                        answer = Query(txt,'question',140,'Save - Coherence',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Coherence-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' ModeShort '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SOmega','VariableNames','Omega');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCoherence' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) ModeShort '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% matrix with coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                           fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                           fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                           fprintf(fid,'Omega = DataStructure.Omega;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The coherence results for the observed variables at ' lower(ModeType) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: VariableNames (string matrix with the names of the observed variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), and Omega (vector with the different frequencies from 0 to pi). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',['Coherence - ' ModeType],400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the results
                        %
                        CoherenceDistributionDLG('init',SOmega,Omega,[],DSGEModel.VariableNames,'Observed',ModeType,['DSGE-VAR(' num2str(lambdai) ') '],DSGEModel,CurrINI);
                     end;
                  else;
                     txt = ['The ' lower(ModeType) ' estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive definite. YADA is therefore unable to compute the coherence statistic.'];
                     About(txt,'information',['Coherence - ' ModeType],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CoherencePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Coherence Distribution - Posterior',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample','PostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if IsPositiveDefinite(SigmaModeEpsilon)==1;
                  if DSGEModel.k>0;
                     PhiMode = PhiMode(:,DSGEModel.k+1:size(PhiMode,2));
                  end;
                  %
                  % we don't need the posterior draws of A0
                  %
                  clear('A0Draws');
                  %
                  % check if calculations have already been performed
                  %
                  if DSGEVARCoherenceExist(UsedDraws,lambda,PME,CurrChain,1,DSGEModel,controls)==1;
                     txt = 'YADA has located coherence results for the observed variables on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,'Coherence Distribution - Posterior',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the distribution
                     %
                     DoneCalc = DSGEVARCoherence(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                  end;
                  %
                  % clear all draws
                  %
                  clear('PhiDraws','SigmaDraws');
                  %
                  if DoneCalc==1;
                     %
                     % compute the distribution for the impulse responses
                     %
                     [SOmega,Omega,Quantiles] = DSGEVARCoherenceDistribution(UsedDraws,lambda,PME,CurrChain,1,Weights,DSGEModel,CurrINI,controls);
                     %
                     % ask if we should save results to disk
                     %
                     txt = ['Would you like to save the posterior distributions of the coherence statistics for the observed variables to file?'];
                     answer = Query(txt,'question',140,'Save - Coherence Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Coherence-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-PosteriorDist.mat']);
                        VariableNames = DSGEModel.VariableNames;
                        save(file,'SOmega','VariableNames','Omega','Quantiles');
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCoherence' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) 'Posterior.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% matrix with mean coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                        fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                        fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                        fprintf(fid,'Omega = DataStructure.Omega;\n');
                        fprintf(fid,'%%\n%% vector structure with quantile data through the fields SOmega and percent\n%%\n');
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The posterior distribution of the coherence statistics for the observed variables have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: VariableNames (string matrix with the names of the observed variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), Omega (vector with the different frequencies from 0 to pi), and Quantiles (vector structure with quantile data in the fields SOmega and percent). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Coherence Distribution - Posterior',400,500,CurrINI);
                        drawnow;
                     end;
                     %
                     % display the results
                     %
                     CoherenceDistributionDLG('init',SOmega,Omega,Quantiles,DSGEModel.VariableNames,'Observed','Posterior Distribution',['DSGE-VAR(' num2str(lambda) ') '],DSGEModel,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive definite. YADA is therefore unable to compute the coherence statistic.'];
                  About(txt,'information','Coherence Distribution - Posterior',300,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the coherence statistic. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Coherence Distribution - Posterior',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Observed Variable Correlations - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  if IsPositiveDefinite(SigmaEpsilon)==1;
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     if NumPredPaths>1;
                        txt = 'Would you like to examine model based sample moments rather than model based population moments? Note: the model based sample moments require simulation of data.';
                        answer = Query(txt,'question',150,'Observed Variable Correlations - Initial Values',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'no')==1;
                        SimulateData = 0;
                        NumPredPaths = 0;
                     else;
                        SimulateData = 1;
                     end;
                     %
                     if SimulateData==0;
                        SVEs = DSGEVARCorrelationParam(Phi,SigmaEpsilon,DSGEModel);
                     else;
                        SVEs = DSGEVARCorrelationSimulationParam(Phi,SigmaEpsilon,NumPredPaths,lambdai,'Initial Values',DSGEModel,CurrINI);
                     end;
                     if isempty(SVEs)==0;
                        if SimulateData==0;
                           AutoCorrType = 'Population-based';
                           BoxStr = 'Graphics - Population Autocorrelations';
                        else;
                           AutoCorrType = 'Sample-based';
                           BoxStr = 'Graphics - Sample Autocorrelations';
                        end;
                        %
                        % ask if the user wants to save to results to disk
                        %
                        txt = ['Would you like to save the ' lower(AutoCorrType) ' mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions for the initial values to file?'];
                        answer = Query(txt,'question',140,'Save - Observed Variable Correlations',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-AutoCorrelation-' AutoCorrType '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(NumPredPaths) '-InitialValues.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SVEs','VariableNames');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARAC' AutoCorrType(1) strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(NumPredPaths) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The ' lower(AutoCorrType) ' autocorrelation data for a DSGE-VAR(' num2str(lambdai) ') model using the initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, and possible quantile data), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Observed Variable Correlations - Initial Values',200,500,CurrINI);
                        end;
                        %
                        % display the autocorrelations
                        %
                        AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,['DSGE-VAR(' num2str(lambdai) ') Autocorrelation Functions - Initial Values'],BoxStr,DSGEModel,CurrINI);
                        %
                        % we can now present the results
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelation-' AutoCorrType '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(NumPredPaths) '-InitialValues.txt']);
                        PrintDSGEVARAutoCorrelations(ResFile,lambdai,DSGEModel,SVEs,CurrINI,'Initial Values',NumPredPaths,1);
                        if FileExist(ResFile)==1;
                           txt = ['Would you like to view the data and ' lower(AutoCorrType) ' mean and standard deviations of the observed variables for the DSGE-VAR(' num2str(lambdai) ') model?'];
                           answer = Query(txt,'question',200,'Observed Variable Correlations - Initial Values',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              TextGUI(ResFile,['DSGEVAR(' num2str(lambdai) ') Mean & Standard Deviation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                           end;
                        end;
                     end;
                  else;
                     txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive definite. YADA is therefore unable to compute the observed variable correlations.'];
                     About(txt,'information','Observed Variable Correlations - Initial Values',300,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         %
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               if IsPositiveDefinite(SigmaEpsilon)==1;
                  %
                  % determine which model to consider
                  %
                  [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Observed Variable Correlations - Prior',DSGEModel,CurrINI);
                  if isempty(lambda)==0;
                     T = HSample(2)-HSample(1)+1;
                     if isinf(lambda)==0;
                        SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     %
                     % load the prior draws data
                     %
                     [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                     UsedDraws = size(PhiDraws,1);
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % ask if the user wants to simulate data
                     %
                     txt = 'Would you like to examine model based sample moments (prior predictive densities) rather than model based population moments? Note: the model based sample moments require simulation of data, while the population moments only use draws from the prior distribution of the VAR parameters.';
                     answer = Query(txt,'question',150,'Observed Variable Correlations - Prior',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        SimulateData = 0;
                        NumPredPaths = 0;
                     else;
                        SimulateData = 1;
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     end;
                     %
                     % First we check if the calculations have already been performed
                     %
                     if DSGEVARCorrelationsExist(UsedDraws,[],0,[],lambda,NumPredPaths,DSGEModel,controls)==1;
                        txt = 'YADA has located results for the observed variable correlations on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Observed Variable Correlations - Prior',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     %
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % compute the distributions
                        %
                        if SimulateData==0;
                           DoneCalc = DSGEVARCorrelationEstimation(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,[],0,[],lambda,DSGEModel,CurrINI,controls);
                        else;
                           DoneCalc = DSGEVARCorrelationSimulation(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,[],0,[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                        end;
                     end;
                     %
                     % clear all parameter draws
                     %
                     clear('PhiDraws','SigmaDraws');
                     if DoneCalc==1;
                        %
                        % compute the distribution for the correlations
                        %
                        if SimulateData==0;
                           SVEs = DSGEVARCorrelationDistribution(UsedDraws,[],0,[],[],lambda,DSGEModel,CurrINI,controls);
                           AutoCorrType = 'Population-based';
                           BoxStr = 'Graphics - Population Autocorrelations';
                        else;
                           SVEs = DSGEVARCorrSimulationDistribution(UsedDraws,[],0,[],[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                           AutoCorrType = 'Sample-based';
                           BoxStr = 'Graphics - Sample Autocorrelations';
                        end;
                        %
                        % check if results should be written to file
                        %
                        txt = ['Would you like to save the ' lower(AutoCorrType) ' mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions to file?'];
                        answer = Query(txt,'question',140,'Save - Observed Variable Correlations',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-AutoCorrPriorDist-' AutoCorrType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) '-' int2str(NumPredPaths) '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SVEs','VariableNames');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARACPriorDist' AutoCorrType(1) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(UsedDraws) int2str(NumPredPaths) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The ' lower(AutoCorrType) ' autocorrelation data for a DSGE-VAR(' num2str(lambda) ') model have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, and quantile data), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Observed Variable Correlations - Prior',300,500,CurrINI);
                        end;
                        %
                        % display the autocorrelations
                        %
                        AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,['DSGE-VAR(' num2str(lambda) ') Autocorrelation Functions - Prior'],BoxStr,DSGEModel,CurrINI);
                        %
                        % we can now present the results
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-AutoCorrDist-' AutoCorrType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) '-' int2str(NumPredPaths) '.txt']);
                        PrintDSGEVARAutoCorrelations(ResFile,lambda,DSGEModel,SVEs,CurrINI,'Prior Distribution',NumPredPaths,UsedDraws);
                        if FileExist(ResFile)==1;
                           txt = ['Would you like to view the data and ' lower(AutoCorrType) ' mean and standard deviations of the observed variables for the DSGE-VAR(' num2str(lambda) ') model?'];
                           answer = Query(txt,'question',200,'Observed Variable Correlations - Prior',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              TextGUI(ResFile,['DSGEVAR(' num2str(lambda) ') Mean & Standard Deviation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,1);
                           end;
                        end;
                     end;
                  end;
               else;
                  txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive definite. YADA is therefore unable to compute the observed variable correlations.'];
                  About(txt,'information','Observed Variable Correlations - Prior',300,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.obscorr.timedomain.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.obscorr.timedomain.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.obscorr.timedomain.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Observed Variable Correlations - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  SigmaEpsilon = ModeData.SigmaModeEpsilon;
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               end;
               if ModeStatus==1;
                  if IsPositiveDefinite(SigmaEpsilon)==1;
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     if NumPredPaths>1;
                        txt = 'Would you like to examine model based sample moments rather than model based population moments? Note: the model based sample moments require simulation of data.';
                        answer = Query(txt,'question',150,['Observed Variable Correlations - ' ModeType],500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'no')==1;
                        SimulateData = 0;
                        NumPredPaths = 0;
                     else;
                        SimulateData = 1;
                     end;
                     %
                     if SimulateData==0;
                        SVEs = DSGEVARCorrelationParam(Phi,SigmaEpsilon,DSGEModel);
                     else;
                        SVEs = DSGEVARCorrelationSimulationParam(Phi,SigmaEpsilon,NumPredPaths,lambdai,ModeType,DSGEModel,CurrINI);
                     end;
                     if isempty(SVEs)==0;
                        if SimulateData==0;
                           AutoCorrType = 'Population-based';
                           BoxStr = 'Graphics - Population Autocorrelations';
                        else;
                           AutoCorrType = 'Sample-based';
                           BoxStr = 'Graphics - Sample Autocorrelations';
                        end;
                        %
                        % ask if the user wants to save to results to disk
                        %
                        txt = ['Would you like to save the ' lower(AutoCorrType) ' mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions for the ' lower(ModeType) ' parameter estimate to file?'];
                        answer = Query(txt,'question',140,'Save - Observed Variable Correlations',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-AutoCorrelation-' AutoCorrType '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(NumPredPaths) '-' ModeShort '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SVEs','VariableNames');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARAC' AutoCorrType(1) strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(NumPredPaths) ModeShort '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The ' lower(AutoCorrType) ' autocorrelation data for a DSGE-VAR(' num2str(lambdai) ') model using ' lower(ModeType) ' parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, and possible quantile data), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',['Observed Variable Correlations - ' ModeType],200,500,CurrINI);
                        end;
                        %
                        % display the autocorrelations
                        %
                        AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,['DSGE-VAR(' num2str(lambdai) ') Autocorrelation Functions - ' ModeType],BoxStr,DSGEModel,CurrINI);
                        %
                        % we can now present the results
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelation-' AutoCorrType '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(NumPredPaths) '-' ModeShort '.txt']);
                        PrintDSGEVARAutoCorrelations(ResFile,lambdai,DSGEModel,SVEs,CurrINI,ModeType,NumPredPaths,1);
                        if FileExist(ResFile)==1;
                           txt = ['Would you like to view the data and ' lower(AutoCorrType) ' mean and standard deviations of the observed variables for the DSGE-VAR(' num2str(lambdai) ') model?'];
                           answer = Query(txt,'question',200,['Observed Variable Correlations - ' ModeType],500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              TextGUI(ResFile,['DSGEVAR(' num2str(lambdai) ') Mean & Standard Deviation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],[ModeType ' Parameter Values'],50,CurrINI,1,1);
                           end;
                        end;
                     end;
                  else;
                     txt = ['The ' lower(ModeType) ' estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive definite. YADA is therefore unable to compute the observed variable correlations.'];
                     About(txt,'information',['Observed Variable Correlations - ' ModeType],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Observed Variable Correlations - Posterior',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % ask if the user wants to simulate data
            %
            txt = 'Would you like to examine model based sample moments (posterior predictive densities) rather than model based population moments? Note: the model based sample moments require simulation of data, while the population moments only use draws from the posterior distribution of the VAR parameters.';
            answer = Query(txt,'question',150,'Observed Variable Correlations - Posterior',500,'no',CurrINI);
            if strcmp(lower(answer),'no')==1;
               SimulateData = 0;
               NumPredPaths = 0;
            else;
               SimulateData = 1;
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            end;
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,SimulateData);
            clear('thetaPostSample','PostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if IsPositiveDefinite(SigmaModeEpsilon)==1;
                  %
                  % we don't need the posterior draws of A0
                  %
                  clear('A0Draws');
                  %
                  % First we check if the calculations have already been performed
                  %
                  if DSGEVARCorrelationsExist(UsedDraws,CurrChain,1,PME,lambda,NumPredPaths,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the observed variable correlations on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,'Observed Variable Correlations - Posterior',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the distributions
                     %
                     if SimulateData==0;
                        DoneCalc = DSGEVARCorrelationEstimation(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                     else;
                        DoneCalc = DSGEVARCorrelationSimulation(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,1,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                     end;
                  end;
                  %
                  % clear all parameter draws
                  %
                  clear('PhiDraws','SigmaDraws');
                  if DoneCalc==1;
                     %
                     % compute the distribution for the correlations
                     %
                     if SimulateData==0;
                        SVEs = DSGEVARCorrelationDistribution(UsedDraws,CurrChain,1,Weights,PME,lambda,DSGEModel,CurrINI,controls);
                        AutoCorrType = 'Population-based';
                        BoxStr = 'Graphics - Population Autocorrelations';
                     else;
                        SVEs = DSGEVARCorrSimulationDistribution(UsedDraws,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                        AutoCorrType = 'Sample-based';
                        BoxStr = 'Graphics - Sample Autocorrelations';
                     end;
                     %
                     % check if results should be written to file
                     %
                     txt = ['Would you like to save the ' lower(AutoCorrType) ' mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions to file?'];
                     answer = Query(txt,'question',140,'Save - Observed Variable Correlations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-AutoCorrDist-' AutoCorrType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumPredPaths) '.mat']);
                        VariableNames = DSGEModel.VariableNames;
                        save(file,'SVEs','VariableNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARACDist' AutoCorrType(1) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(CurrChain) int2str(UsedDraws) int2str(NumPredPaths) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                        fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The ' lower(AutoCorrType) ' autocorrelation data for a DSGE-VAR(' num2str(lambda) ') model have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, and quantile data), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Observed Variable Correlations - Posterior',300,500,CurrINI);
                     end;
                     %
                     % display the autocorrelations
                     %
                     AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,['DSGE-VAR(' num2str(lambda) ') Autocorrelation Functions - Posterior'],BoxStr,DSGEModel,CurrINI);
                     %
                     % we can now present the results
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-AutoCorrDist-' AutoCorrType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumPredPaths) '.txt']);
                     PrintDSGEVARAutoCorrelations(ResFile,lambda,DSGEModel,SVEs,CurrINI,'Posterior Distribution',NumPredPaths,UsedDraws);
                     if FileExist(ResFile)==1;
                        txt = ['Would you like to view the data and ' lower(AutoCorrType) ' mean and standard deviations of the observed variables for the DSGE-VAR(' num2str(lambda) ') model?'];
                        answer = Query(txt,'question',200,'Observed Variable Correlations - Posterior',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           TextGUI(ResFile,['DSGEVAR(' num2str(lambda) ') Mean & Standard Deviation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,1);
                        end;
                     end;
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive definite. YADA is therefore unable to compute the observed variable correlations.'];
                  About(txt,'information','Observed Variable Correlations - Posterior',300,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the observed variable correlations. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Observed Variable Correlations - Posterior',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Conditional Correlations - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     if NumPredPaths>1;
                        txt = 'Would you like to examine model based sample correlations rather than model based population correlations? Note: the model based sample correlations require simulation of data.';
                        answer = Query(txt,'question',150,'Conditional Correlations - Initial Values',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'no')==1;
                        NumPredPaths = 0;
                     end;
                     CondCorr = DSGEVARConditionalCorrsParam(Phi,A0,lambdai,DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:),NumPredPaths,'Initial Values',DSGEModel,CurrINI);
                     if isempty(CondCorr)==0;
                        %
                        % ask if we should save the results to disk
                        %
                        if NumPredPaths>0
                           txt = 'Would you like to save the mean and quantiles of the conditional correlation distribution at initial values to file?';
                           answer = Query(txt,'question',140,'Save - Conditional Correlation Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrelationDist-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' int2str(NumPredPaths)  '-InitialValues.mat']);
                              VariableNames = DSGEModel.VariableNames;
                              ShockNames = CondCorr(1).ShockNames;
                              for i=1:length(CondCorr);
                                 ConditionalCorrs(i).Mean = CondCorr(i).Mean;
                                 ConditionalCorrs(i).Quantiles = CondCorr(i).Quantiles;
                              end;
                              %
                              save(file,'VariableNames','ShockNames','ConditionalCorrs');
                              %
                              % write an m-file
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCCDist' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) SelectedShockStr int2str(NumPredPaths) 'InitVal.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of shocks\n%%\n');
                              fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                              fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              txt = ['The sample-based conditional correlations data at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the sample conditional correlations), ShockNames (string matrix with the names of the shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Conditional Correlations - Initial Values',200,500,CurrINI);
                           end;
                        end;
                        %
                        % we can now present the results
                        %
                        if NumPredPaths==0;
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorr-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-InitialValues.txt']);
                        else;
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorr-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(NumPredPaths) '-InitialValues.txt']);
                        end;
                        PrintCondCorrelations(ResFile,DSGEModel,CondCorr,CurrINI,'initial','DSGE-VAR',NumPredPaths,1);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                        end;
                     end;
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the conditional correlations.'];
                     About(txt,'information','Conditional Correlations - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         %
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               if IsPositiveDefinite(SigmaEpsilon)==1;
                  %
                  % determine which model to consider
                  %
                  [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Conditional Correlations - Prior',DSGEModel,CurrINI);
                  if isempty(lambda)==0;
                     %
                     % load the prior draws data
                     %
                     [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                     UsedDraws = size(PhiDraws,1);
                     if isempty(A0Draws)==0;
                        %
                        % we don't need the posterior draws of Sigma
                        %
                        clear('SigmaDraws');
                        %
                        % ask if the user wants to simulate data
                        %
                        txt = 'Would you like to examine model based sample correlations (prior predictive densities) rather than model based population correlations? Note: the model based sample moments require simulation of data, while the population moments only use draws from the prior distribution of the VAR parameters.';
                        answer = Query(txt,'question',150,'Conditional Correlations - Prior',500,'no',CurrINI);
                        if strcmp(lower(answer),'no')==1;
                           SimulateData = 0;
                           NumPredPaths = 0;
                        else;
                           SimulateData = 1;
                           NumPredPathsStr = get(controls.forecast.predpaths,'String');
                           NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                        end;
                        %
                        % we don't need the posterior draws of SigmaEpsilon
                        %
                        clear('SigmaDraws');
                        %
                        if DSGEVARCondCorrExist(UsedDraws,[],0,[],lambda,NumPredPaths,DSGEModel,controls)==1;
                           txt = 'YADA has located results for the conditional correlations on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,'Conditional Correlations - Prior',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        %
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                        else;
                           %
                           % compute the distributions
                           %
                           if SimulateData==0;
                              DoneCalc = DSGEVARConditionalCorrsEstimation(PhiDraws,A0Draws,[],0,[],lambda,DSGEModel,CurrINI,controls);
                           else;
                              DoneCalc = DSGEVARConditionalCorrsSimulation(PhiDraws,A0Draws,[],0,[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                           end;
                        end;
                        %
                        % determine the distribution for the correlations
                        %
                        CorrData = DSGEVARCondCorrDistribution(UsedDraws,[],0,[],[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                        %
                        % ask if we should save the results to disk
                        %
                        if NumPredPaths==0;
                           SampleTypeStr = 'Population';
                           AddFileName = '';
                        else;
                           SampleTypeStr = 'Sample';
                           AddFileName = ['-' int2str(NumPredPaths)];
                        end;
                        %
                        txt = ['Would you like to save the mean and quantiles of the ' lower(SampleTypeStr) '-based conditional correlation distribution to file?'];
                        answer = Query(txt,'question',140,'Save - Conditional Correlations',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrPrior' SampleTypeStr 'Dist-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) AddFileName '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           ShockNames = CorrData(1).ShockNames;
                           for i=1:length(CorrData);
                              ConditionalCorrs(i).Mean = CorrData(i).Mean;
                              ConditionalCorrs(i).Quantiles = CorrData(i).Quantiles;
                           end;
                           %
                           save(file,'VariableNames','ShockNames','ConditionalCorrs');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCCP' SampleTypeStr(1) 'Dist' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(UsedDraws) strrep(AddFileName,'-','') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of state shocks\n%%\n');
                           fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The ' lower(SampleTypeStr) '-based conditional correlations data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the ' lower(SampleTypeStr) ' conditional correlations), ShockNames (string matrix with the names of the nonzero state shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Conditional Correlations - Prior',200,500,CurrINI);
                        end;
                        %
                        % print the output to file
                        %
                        if NumPredPaths==0;
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrPriorDist-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(UsedDraws) '.txt']);
                        else;
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrPriorDist-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(UsedDraws) '.' int2str(NumPredPaths) '.txt']);
                        end;
                        PrintCondCorrelations(ResFile,DSGEModel,CorrData,CurrINI,'prior distribution','DSGE-VAR',NumPredPaths,UsedDraws);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['DSGEVAR(' num2str(lambda) ') Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,1);
                        end;
                     else;
                        clear('PhiDraws','SigmaDraws');
                        txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the prior distribution of the conditional correlations.'];
                        About(txt,'information','Conditional Correlations - Prior',200,500,CurrINI);
                     end;
                  end;
               else;
                  txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive definite. YADA is therefore unable to compute the conditional correlations.'];
                  About(txt,'information','Conditional Correlations - Prior',300,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.condcorr.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.condcorr.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.condcorr.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Conditional Correlations - ' ModeType],'','',CurrINI);
            drawnow;
            pause(0.02);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     A0 = ModeData.A0Mode;
                     %
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     if NumPredPaths>1;
                        txt = 'Would you like to examine model based sample correlations rather than model based population correlations? Note: the model based sample correlations require simulation of data.';
                        answer = Query(txt,'question',150,['Conditional Correlations - ' ModeType],500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'no')==1;
                        NumPredPaths = 0;
                     end;
                     %
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     end;
                     %
                     % compute the conditional correlations
                     %
                     CondCorr = DSGEVARConditionalCorrsParam(Phi,A0,lambdai,ShockNames,NumPredPaths,ModeType,DSGEModel,CurrINI);
                     if isempty(CondCorr)==0;
                        %
                        % ask if we should save the results to disk
                        %
                        if NumPredPaths>0
                           txt = ['Would you like to save the mean and quantiles of the conditional correlation distribution at ' lower(ModeType) ' to file?'];
                           answer = Query(txt,'question',140,'Save - Conditional Correlation Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrelationDist-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' int2str(NumPredPaths)  '-' ModeShort '.mat']);
                              VariableNames = DSGEModel.VariableNames;
                              for i=1:length(CondCorr);
                                 ConditionalCorrs(i).Mean = CondCorr(i).Mean;
                                 ConditionalCorrs(i).Quantiles = CondCorr(i).Quantiles;
                              end;
                              %
                              save(file,'VariableNames','ShockNames','ConditionalCorrs');
                              %
                              % write an m-file
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCCDist' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) SelectedShockStr int2str(NumPredPaths) ModeShort '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of shocks\n%%\n');
                              fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                              fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              txt = ['The sample-based conditional correlations data at ' lower(ModeType) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the sample conditional correlations), ShockNames (string matrix with the names of the shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information',['Conditional Correlations - ' ModeType],200,500,CurrINI);
                           end;
                        end;
                        %
                        % we can now present the results
                        %
                        if NumPredPaths==0;
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorr-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' ModeShort '.txt']);
                        else;
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorr-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(NumPredPaths) '-' ModeShort '.txt']);
                        end;
                        PrintCondCorrelations(ResFile,DSGEModel,CondCorr,CurrINI,lower(ModeType),'DSGE-VAR',NumPredPaths,1);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],[ModeType ' Parameter Values'],50,CurrINI,1,1);
                        end;
                     end;
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to compute the conditional correlations.'];
                     About(txt,'information',['Conditional Correlations - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Conditional Correlations - Posterior',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % ask if the user wants to simulate data
            %
            txt = 'Would you like to examine model based sample correlations (posterior predictive densities) rather than model based population correlations? Note: the model based sample correlations require simulation of data, while the population correlations only use draws from the posterior distribution of the VAR parameters.';
            answer = Query(txt,'question',150,'Conditional Correlations - Posterior',500,'no',CurrINI);
            if strcmp(lower(answer),'no')==1;
               SimulateData = 0;
               NumPredPaths = 0;
            else;
               SimulateData = 1;
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            end;
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,SimulateData);
            clear('thetaPostSample','PostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if isempty(A0Draws)==0;
                  %
                  % we don't need the posterior draws of SigmaEpsilon
                  %
                  clear('SigmaDraws');
                  %
                  if DSGEVARCondCorrExist(UsedDraws,CurrChain,1,PME,lambda,NumPredPaths,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the conditional correlations on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,'Conditional Correlations - Posterior',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the distributions
                     %
                     if SimulateData==0;
                        DoneCalc = DSGEVARConditionalCorrsEstimation(PhiDraws,A0Draws,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                     else;
                        DoneCalc = DSGEVARConditionalCorrsSimulation(PhiDraws,A0Draws,CurrChain,1,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                     end;
                  end;
                  if DoneCalc==1;
                     %
                     % determine the distribution for the correlations
                     %
                     CorrData = DSGEVARCondCorrDistribution(UsedDraws,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                     %
                     % ask if we should save the results to disk
                     %
                     if NumPredPaths==0;
                        SampleTypeStr = 'Population';
                        AddFileName = '';
                     else;
                        SampleTypeStr = 'Sample';
                        AddFileName = ['-' int2str(NumPredPaths)];
                     end;
                     %
                     txt = ['Would you like to save the mean and quantiles of the ' lower(SampleTypeStr) '-based conditional correlation distribution to file?'];
                     answer = Query(txt,'question',140,'Save - Conditional Correlations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorr' SampleTypeStr 'Dist-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) AddFileName '.mat']);
                        VariableNames = DSGEModel.VariableNames;
                        ShockNames = CorrData(1).ShockNames;
                        for i=1:length(CorrData);
                           ConditionalCorrs(i).Mean = CorrData(i).Mean;
                           ConditionalCorrs(i).Quantiles = CorrData(i).Quantiles;
                        end;
                        %
                        save(file,'VariableNames','ShockNames','ConditionalCorrs');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCC' SampleTypeStr(1) 'Dist' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(CurrChain) int2str(UsedDraws) strrep(AddFileName,'-','') '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of state shocks\n%%\n');
                        fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The ' lower(SampleTypeStr) '-based conditional correlations data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the ' lower(SampleTypeStr) ' conditional correlations), ShockNames (string matrix with the names of the nonzero state shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Conditional Correlations - Posterior',200,500,CurrINI);
                     end;
                     %
                     % print the output to file
                     %
                     if NumPredPaths==0;
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrDist-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(UsedDraws) '.txt']);
                     else;
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondCorrDist-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(UsedDraws) '.' int2str(NumPredPaths) '.txt']);
                     end;
                     PrintCondCorrelations(ResFile,DSGEModel,CorrData,CurrINI,'posterior distribution','DSGE-VAR',NumPredPaths,UsedDraws);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,['DSGEVAR(' num2str(lambda) ') Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior distribution of the conditional correlations.'];
                  About(txt,'information','Conditional Correlations - Posterior',200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior distribution of the conditional correlations. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Conditional Correlations - Posterior',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrelationsInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         %
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Structural Shock Correlations - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                     for j=1:DSGEModel.n;
                        ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                     end;
                     %
                     % get the data and sample information
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     %
                     % compute the shocks and their correlations
                     %
                     Shocks = (A0\eye(size(y,1)))*(y-(Phi*Y));
                     T = size(Shocks,2);
                     SampleMean = (1/T)*sum(Shocks,2);
                     Shocks = Shocks-(SampleMean*ones(1,T));
                     CovarianceMatrix = (1/T)*Shocks*Shocks';
                     %
                     StDev = sqrt(diag(CovarianceMatrix));
                     Correlations = CovarianceMatrix./(StDev*StDev');
                     %
                     prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     prt_str = '%s';
                     for i=1:length(SampleMean);
                        prt_str = [prt_str '     ' prt_val];
                     end;
                     %
                     txt = ['Would you like to save to file the sample estimates of the first and second moments of the structural shocks based on the initial values of the DSGE model parameters for the DSGE-VAR(' num2str(lambdai) ') model?'];
                     answer = Query(txt,'question',140,'Save - Structural Shock Correlations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                        SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                        StDev = diag(StDev);
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ShockCorrelations-' num2str(lambdai) '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-InitialValues.mat']);
                        save(file,'ShockNames','SampleMean','StDev','Correlations');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARSC' strrep(num2str(lambdai),'.','_') int2str(DSGEModel.BVARLags) SelectedShockStr 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the structural shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% vector with the sample mean for the initial values\n%%\n');
                        fprintf(fid,'SampleMean = DataStructure.SampleMean;\n');
                        fprintf(fid,'%%\n%% vector with the sample standard deviation\n%%\n');
                        fprintf(fid,'StDev = DataStructure.StDev;\n');
                        fprintf(fid,'%%\n%% matrix with the sample correlations\n%%\n');
                        fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The sample estimates of the first and second moments of the structural shocks at the initial values of the DSGE model parameters for the DSGE-VAR(' num2str(lambdai) ') model with ' int2str(DSGEModel.BVARLags) ' lags have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: ShockNames (string matrix with the names of the structural shocks), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), and Correlations (matrix with the sample correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Structural Shock Correlations - Initial Values',200,500,CurrINI);
                     end;
                     %
                     txt = strvcat(' ','STRUCTURAL SHOCKS','=================',' ','Using the initial parameter values.',' ');
                     %
                     txt = strvcat(txt,'Sample Mean matrix:','-------------------',' ');
                     for i=1:length(SampleMean);
                        txt = strvcat(txt,sprintf(['%s     ' prt_val],ShockNames(i,:),SampleMean(i)));
                     end;
                     %
                     txt = strvcat(txt,' ','Sample Covariance matrix:','-------------------------',' ');
                     for i=1:length(SampleMean);
                        txt = strvcat(txt,sprintf(prt_str,ShockNames(i,:),CovarianceMatrix(:,i)));
                     end;
                     txt = strvcat(txt,' ','Sample Correlation matrix:','--------------------------',' ');
                     for i=1:length(SampleMean);
                        txt = strvcat(txt,sprintf(prt_str,ShockNames(i,:),Correlations(:,i)));
                     end;
                     %
                     TextGUI(txt,['DSGE-VAR(' num2str(lambdai) ') Structural Shocks - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not estimate the structural shocks.'];
                     About(txt,'information','Structural Shock Correlations - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrelationsPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine which model to consider
         %
         [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Structural Shock Correlations - Prior',DSGEModel,CurrINI);
         if isempty(lambda)==0;
            %
            % load the prior draws data
            %
            [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
            UsedDraws = size(PhiDraws,1);
            if isempty(A0Draws)==0;
               clear('SigmaDraws');
               %
               % check if posterior draws already exist on disk
               %
               if DSGEVARShockCorrelationsExist(UsedDraws,[],0,[],lambda,DSGEModel,controls)==1;
                  txt = 'YADA has located results for the structural shock correlations on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Structural Shock Correlations - Prior',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the structural shocks
                  %
                  DoneCalc = DSGEVARStructuralShockCorrelations(PhiDraws,A0Draws,[],0,[],lambda,DSGEModel,CurrINI,controls);
               end;
               %
               % clear everything
               %
               clear('PhiDraws','A0Draws');
               if DoneCalc==1;
                  %
                  % sort the results
                  %
                  ShockRes = DSGEVARShockCorrelationDistribution(DSGEModel,CurrINI,controls,UsedDraws,[],0,[],[],lambda);
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the distributions for the structural shock correlations to file?';
                  answer = Query(txt,'question',140,'Save - Structural Shock Corrleations',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ShockCorrPriorDist-' num2str(lambda,'%0.4f') '-' int2str(UsedDraws) '.mat']);
                     %
                     save(file,'ShockRes');
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARShockCorrPriorDist' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(UsedDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields ShockNames containing the names of the shocks,\n%% SampleMean with the means of the shocks, StdShocks with the means of\n%% the standard deviations of the shocks, CorrShocks with the means of\n%% the correlation matrix, and vector field Quantiles with subfields\n%% SampleMean, StdShocks, CorrShocks, and percent holding quantile data\n%% and quantile percent.\n%%\n');
                     fprintf(fid,'ShockRes = DataStructure.ShockRes;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The distribution of the structural shock correlations has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: ShockRes (a structure with the means, standard deviations, correlations, and quantile data for the structural shocks based on the prior distribution). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Structural Shock Correlations - Prior',200,500,CurrINI);
                  end;
                  %
                  % print the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ShockCorrPriorDist-' num2str(lambda,'%0.4f') '-' int2str(UsedDraws) '.txt']);
                  PrintDSGEVARShockCorrelations(ResFile,lambda,DSGEModel,ShockRes,CurrINI,'Prior',[],UsedDraws);
                  %
                  % display the results
                  %
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Structural Shock Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Prior Distributions',50,CurrINI,1,1);
                  end;
               end;
            else;
               clear('PhiDraws','SigmaDraws');
               txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the prior distribution of the structural shock correlations.'];
               About(txt,'information','Structural Shock Correlations - Prior',200,500,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrelationsPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.shockcorr.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeTxt = 'marginal posterior mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.shockcorr.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeTxt = 'joint posterior mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.shockcorr.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeTxt = 'DSGE posterior mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Structural Shock Correlations - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     end;
                     %
                     % locate the positions in DSGEModel.StateShockNames that correspond to ShockNames
                     %
                     Positions = zeros(DSGEModel.n,1);
                     for j=1:DSGEModel.n;
                        Positions(j) = LocateStr(DSGEModel.StateShockNames,StringTrim(ShockNames(j,:)));
                     end;
                     Positions = Positions(Positions>0);
                     ShockNames = DSGEModel.ShockAliases(Positions,:);
                     for j=1:DSGEModel.n;
                        ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                     end;
                     %
                     % get the data and sample information
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     %
                     % compute the shocks and their correlations
                     %
                     Shocks = (A0\eye(size(y,1)))*(y-(Phi*Y));
                     T = size(Shocks,2);
                     SampleMean = (1/T)*sum(Shocks,2);
                     Shocks = Shocks-(SampleMean*ones(1,T));
                     CovarianceMatrix = (1/T)*Shocks*Shocks';
                     %
                     StDev = sqrt(diag(CovarianceMatrix));
                     Correlations = CovarianceMatrix./(StDev*StDev');
                     %
                     prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     prt_str = '%s';
                     for i=1:length(SampleMean);
                        prt_str = [prt_str '     ' prt_val];
                     end;
                     %
                     txt = ['Would you like to save to file the sample estimates of the first and second moments of the structural shocks based on the ' ModeTxt ' estimates for the DSGE-VAR(' num2str(lambdai) ') model?'];
                     answer = Query(txt,'question',140,'Save - Structural Shock Correlations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        StDev = diag(StDev);
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ShockCorrelations-' num2str(lambdai) '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ModeShort '.mat']);
                        save(file,'ShockNames','SampleMean','StDev','Correlations');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARSC' strrep(num2str(lambdai),'.','_') int2str(DSGEModel.BVARLags) SelectedShockStr ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the structural shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% vector with the sample mean for the %s estimates\n%%\n',ModeTxt);
                        fprintf(fid,'SampleMean = DataStructure.SampleMean;\n');
                        fprintf(fid,'%%\n%% vector with the sample standard deviation\n%%\n');
                        fprintf(fid,'StDev = DataStructure.StDev;\n');
                        fprintf(fid,'%%\n%% matrix with the sample correlations\n%%\n');
                        fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The sample estimates of the first and second moments of the structural shocks at ' ModeTxt ' for the DSGE-VAR(' num2str(lambdai) ') model with ' int2str(DSGEModel.BVARLags) ' lags have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: ShockNames (string matrix with the names of the structural shocks), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), and Correlations (matrix with the sample correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',['Structural Shock Correlations - ' ModeType],200,500,CurrINI);
                     end;
                     %
                     txt = strvcat(' ','STRUCTURAL SHOCKS','=================',' ',['Using the parameter values at ' ModeTxt '.'],' ');
                     %
                     txt = strvcat(txt,'Sample Mean matrix:','-------------------',' ');
                     for i=1:length(SampleMean);
                        txt = strvcat(txt,sprintf(['%s     ' prt_val],ShockNames(i,:),SampleMean(i)));
                     end;
                     %
                     txt = strvcat(txt,' ','Sample Covariance matrix:','-------------------------',' ');
                     for i=1:length(SampleMean);
                        txt = strvcat(txt,sprintf(prt_str,ShockNames(i,:),CovarianceMatrix(:,i)));
                     end;
                     txt = strvcat(txt,' ','Sample Correlation matrix:','--------------------------',' ');
                     for i=1:length(SampleMean);
                        txt = strvcat(txt,sprintf(prt_str,ShockNames(i,:),Correlations(:,i)));
                     end;
                     %
                     TextGUI(txt,['DSGE-VAR(' num2str(lambdai) ') Structural Shocks - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],ModeType,50,CurrINI,1,1);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not estimate the structural shocks.'];
                     About(txt,'information',['Structural Shock Correlations - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrelationsPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % select DSGE-VAR model
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Structural Shock Correlations - Posterior',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if isempty(A0Draws)==0;
                  %
                  % clear the sigma parameter draws from memory
                  %
                  clear('SigmaModeEpsilon');
                  %
                  % check if posterior draws already exist on disk
                  %
                  if DSGEVARShockCorrelationsExist(UsedDraws,CurrChain,1,PME,lambda,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the structural shock correlations on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,'Structural Shock Correlations - Posterior',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the structural shocks
                     %
                     DoneCalc = DSGEVARStructuralShockCorrelations(PhiDraws,A0Draws,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                  end;
                  %
                  % clear everything
                  %
                  clear('PhiDraws','A0Draws');
                  if DoneCalc==1;
                     DrawsStr = get(controls.posterior.draws,'String');
                     NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                     %
                     % sort the results
                     %
                     ShockRes = DSGEVARShockCorrelationDistribution(DSGEModel,CurrINI,controls,UsedDraws,CurrChain,1,Weights,PME,lambda);
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the distributions for the structural shock correlations to file?';
                     answer = Query(txt,'question',140,'Save - Structural Shock Corrleations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ShockCorrDist-' num2str(lambda,'%0.4f') '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
                        %
                        save(file,'ShockRes');
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARShockCorrDist' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% structure with fields ShockNames containing the names of the shocks,\n%% SampleMean with the means of the shocks, StdShocks with the means of\n%% the standard deviations of the shocks, CorrShocks with the means of\n%% the correlation matrix, and vector field Quantiles with subfields\n%% SampleMean, StdShocks, CorrShocks, and percent holding quantile data\n%% and quantile percent.\n%%\n');
                        fprintf(fid,'ShockRes = DataStructure.ShockRes;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The distribution of the structural shock correlations has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: ShockRes (a structure with the means, standard deviations, correlations, and quantile data for the structural shocks based on the posterior distribution). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Structural Shock Correlations - Posterior',200,500,CurrINI);
                     end;
                     %
                     % print the results to file
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ShockCorrDist-' num2str(lambda,'%0.4f') '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.txt']);
                     PrintDSGEVARShockCorrelations(ResFile,lambda,DSGEModel,ShockRes,CurrINI,'Posterior',NumDraws,UsedDraws);
                     %
                     % display the results
                     %
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,['Structural Shock Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distributions',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior distribution of the structural shock correlations.'];
                  About(txt,'information','Structural Shock Correlations - Posterior',200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior distribution of the structural shock correlations. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Structural Shock Correlations - Posterior',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of DSGEVARCorrToolsMenuFunctions.m
%
  ,