function controls = DSGEVARMenuControls(maingui,controls)
% DSGEVARMenuControls: Sets up the DSGE-VAR menu.
%
% USAGE:
%
%       controls = DSGEVARMenuControls(maingui,controls)
%
% REQUIRED INPUT:  maingui (handle) to YADA's main dialog.
%
%                  controls (structure) with handles to the controls on the YADA main dialog.
%
% REQUIRED OUTPUT: controls (structure) with handles to the controls on the YADA main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 3, 2010.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2010: Updated the documentation.
%
% * 05-03-2010: Corrected callback names.
%
% * 09-03-2010: Posterior densities are plotted by the same callback function for all types
%               of VAR model parameters.
%
% * 19-03-2010: Added controls for comparing marginal likelihood across DSGE-VAR and a DSGE
%               with the Chib & Jeliazkov estimator as well as the modified harmonic mean
%               estimator.
%
% * 12-03-2010: Added controls for comparing the estimated parameters along with their
%               posterior confidence intervals.
%
% * 15-04-2010: Added controls for computing impulse responses from the posterior draws.
%
% * 16-04-2010: Added controls for computing impulse responses from the prior draws.
%
% * 26-04-2010: Added control for computing the structural shocks from the posterior draws.
%
% * 19-05-2010: Added controls for computing forecast error variance decompositions.
%
% * 24-05-2010: Added controls for observed variable decompositions.
%
% * 26-05-2010: Added controls for observed variable paths.
%
% * 28-05-2010: Added controls for spectral decompositions.
%
% * 01-06-2010: Added controls for unconditional forecasts.
%
% * 15-06-2010: Added controls for observed variable correlations.
%
% * 21-06-2010: Added controls for coherence for the observed variables.
%
% * 03-11-2010: Added controls for simulating data.
%
% * 05-01-2011: Updated the documentation.
%
% * 18-01-2011: Added controls for correlation decompositions.
%
% * 20-01-2011: Added controls for conditional correlations.
%
% * 18-02-2011: Added controls for conditional forecasting.
%
% * 27-02-2011: Added control for "DSGE posterior mode" case for "View|VAR Model", and for
%               "Tools|SImulate Data"
%
% * 28-02-2011: Added further controls for the "DSGE posterior mode" case for the DSGE-VAR
%               model.
%
% * 02-03-2011: Added Modesty statistics controls.
%
% * 03-03-2011: Made sure that the multiple chain controls for the modesty statistics have the
%               correct callback function.
%
% * 26-04-2011: Added call to DSGE-VAR scenario controls.
%
% * 02-01-2012: Updated the documentation.
%
% * 09-02-2012: Added controls for the "Eigenvalues" function.
%
% * 24-02-2012: Added controls for the "Sequential Marginal Likelihood" functions.
%
% * 19-03-2012: Added controls for the "Laplace Approximation" on the sequential marginal likelihood item.
%
% * 02-04-2012: Added controls for the "Predictive Likelihood" calculations.
%
% * 03-04-2012: Added controls for the annualized "Predictive Likelihood" calculations.
%
% * 11-05-2012: Added a separator.
%
% * 10-12-2012: Added the Sims, Waggoner and Zha version with a truncated elliptical distribution of the
%               modified harmonic mean estimator of the log marginal likelihood.
%
% * 13-12-2012: Added a control for comparing marginal likelihoods with the Sims, Waggoner and Zha version
%               of the modified harmonic mean estimator.
%
% * 02-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 21-02-2014: Added a control for plotting the log posterior kernel values of the DSGE-VAR model for
%               when the marginalized likelihood function is used.
%
% * 24-02-2014: Added controls for scatter-plotting the posterior draws of the DSGE model parameters.
%
% * 25-02-2014: Added controls for convergence measure based on the acceptance ratio, the posterior mean,
%               and the posterior median, as well as for the CUSUM plots.
%
% * 26-02-2014: Added a control for the MANOVA convergence statistics.
%
% * 07-03-2014: Added controls for the Laplace posterior densities.
%
% * 11-03-2014: Added controls for plotting the log-likelhood function.
%
% * 14-03-2014: Added controls for shock correlations.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 23-06-2017: Added controls for the marginal predictive moments for the original data using
%               prior and posterior draws.
%
% * 11-09-2017: Added control for raw posterior plots of the Weights from SMC with likelihood
%               tempering.
%
% * 12-09-2017: Added control for comparing marginal likelihood with SMC estimates.
%
% * 15-09-2017: Added control for SMC adaption graphs.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% DSGE-VAR menu
%
controls.dsgevarmenu = uimenu('Parent',maingui, ...
   'Label','&DSGE-VAR', ...
   'Tag','DSGEVARMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Estimate posterior mode                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarmargmodemenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','&Estimate Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARMargPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalPosteriorModeMenu');
%
controls.dsgevarjointmodemenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','Estimate &Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARJointPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                Posterior sampling for DSGE model parameters                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarpostsamplingmenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','&DSGE Posterior Sampling...', ...
   'Callback','DSGEVARMenuFunctions PostSampling', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorSamplingMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                 Posterior sampling for VAR model parameters                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarvarpostsamplingmenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','VAR &Posterior Sampling...', ...
   'Callback','DSGEVARMenuFunctions VARPostSampling', ...
   'Enable','off', ...
   'Tag','DSGEVARVARPosteriorSamplingMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                               Prior sampling                                %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarpriorsamplingmenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','Prior Sa&mpling...', ...
   'Callback','DSGEVARMenuFunctions PriorSampling', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARPriorSamplingMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                    View                                     %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarviewparent = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','&View', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARViewMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             VAR model sub-menu                              %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.varmodelparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','&VAR Equations', ...
   'Enable','on', ...
   'Tag','DSGEVAREquationsMenu');
%
% Initial parameter values
%
controls.dsgevarview.varmodel.initialvalues = uimenu('Parent',controls.dsgevarview.varmodelparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions VARModelInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVAREquationsInitialValueMenu');
%
% DSGE model posterior mode
%
controls.dsgevarview.varmodel.dsgemode = uimenu('Parent',controls.dsgevarview.varmodelparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions VARModelPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVAREquationsDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevarview.varmodel.margmode = uimenu('Parent',controls.dsgevarview.varmodelparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions VARModelPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVAREquationsMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevarview.varmodel.jointmode = uimenu('Parent',controls.dsgevarview.varmodelparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions VARModelPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVAREquationsJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                 Eigenvalues                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.eigenvaluesparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','&Eigenvalues', ...
   'Enable','on', ...
   'Tag','DSGEVAREigenvaluesMenu');
%
%
% Initial parameter values
%
controls.dsgevarview.eigenvalues.initialvalues = uimenu('Parent',controls.dsgevarview.eigenvaluesparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions EigenvaluesInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVAREigenvaluesInitialValueMenu');
%
% DSGE model posterior mode
%
controls.dsgevarview.eigenvalues.dsgemode = uimenu('Parent',controls.dsgevarview.eigenvaluesparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions EigenvaluesPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVAREigenvaluesDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevarview.eigenvalues.margmode = uimenu('Parent',controls.dsgevarview.eigenvaluesparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions EigenvaluesPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVAREigenvaluesMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevarview.eigenvalues.jointmode = uimenu('Parent',controls.dsgevarview.eigenvaluesparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions EigenvaluesPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVAREigenvaluesJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Posterior mode results                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.postmoderesparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','&Posterior Mode Results', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARPosteriorModeResultsMenu');
%
% marginal posterior mode
%
controls.dsgevarview.postmoderes.margmode = uimenu('Parent',controls.dsgevarview.postmoderesparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions PostModeResults', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalPosteriorModeResultsMenu');
%
% joint posterior mode
%
controls.dsgevarview.postmoderes.jointmode = uimenu('Parent',controls.dsgevarview.postmoderesparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions PostModeResults', ...
   'Enable','off', ...
   'Tag','DSGEVARJointPosteriorModeResultsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Posterior mode summary                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.postmodesumparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Posterior &Mode Summary', ...
   'Enable','on', ...
   'Tag','DSGEVARPosteriorModeSummaryMenu');
%
% marginal posterior mode
%
controls.dsgevarview.postmodesum.margmode = uimenu('Parent',controls.dsgevarview.postmodesumparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions PostModeResults', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalPosteriorModeSummaryMenu');
%
% joint posterior mode
%
controls.dsgevarview.postmodesum.jointmode = uimenu('Parent',controls.dsgevarview.postmodesumparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions PostModeResults', ...
   'Enable','off', ...
   'Tag','DSGEVARJointPosteriorModeSummaryMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         Optimization error summary                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.opterrorparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Optimi&zation Error Summary', ...
   'Enable','on', ...
   'Tag','DSGEVAROptimizationErrorSummaryMenu');
%
% marginal posterior mode
%
controls.dsgevarview.opterror.margmode = uimenu('Parent',controls.dsgevarview.opterrorparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions OptErrorSummary', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalOptimizationErrorSummaryMenu');
%
% joint posterior mode
%
controls.dsgevarview.opterror.jointmode = uimenu('Parent',controls.dsgevarview.opterrorparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions OptErrorSummary', ...
   'Enable','off', ...
   'Tag','DSGEVARJointOptimizationErrorSummaryMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         Posterior sampling summary                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.posteriorsamplingparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Posterior &Sampling Summary', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','DSGEVARPosteriorSamplingParentMenu');
%
% DSGE parameter sampling
%
controls.dsgevarview.posteriorsamplingsummary = uimenu('Parent',controls.dsgevarview.posteriorsamplingparent, ...
   'Label','&DSGE...', ...
   'Callback','DSGEVARMenuFunctions PosteriorSamplingSummary', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorSamplingDSGESummaryMenu');
%
% VAR parameter sampling
%
controls.dsgevarview.varposteriorsamplingsummary = uimenu('Parent',controls.dsgevarview.posteriorsamplingparent, ...
   'Label','&VAR...', ...
   'Callback','DSGEVARMenuFunctions VARPosteriorSamplingSummary', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorSamplingVARSummaryMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             Raw posterior draws                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.rawdrawsparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Ra&w Posterior Draws', ...
   'Enable','on', ...
   'Tag','DSGEVARRawPosteriorDrawsParentMenu');
%
% original parameters
%
controls.dsgevarview.rawdraws.originalparameters = uimenu('Parent',controls.dsgevarview.rawdrawsparent, ...
   'Label','&Original Parameters...', ...
   'Callback','DSGEVARMenuFunctions RawPosteriorDraws', ...
   'Enable','off', ...
   'Tag','DSGEVARRawOriginalPosteriorDrawsMenu');
%
% transformed parameters
%
controls.dsgevarview.rawdraws.transformedparameters = uimenu('Parent',controls.dsgevarview.rawdrawsparent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','DSGEVARMenuFunctions RawPosteriorDraws', ...
   'Enable','off', ...
   'Tag','DSGEVARRawTransformedPosteriorDrawsMenu');
%
controls.dsgevarview.rawdraws.logposterior = uimenu('Parent',controls.dsgevarview.rawdrawsparent, ...
   'Label','&Log Posterior Kernel...', ...
   'Callback','DSGEVARMenuFunctions RawPosteriorDraws', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARLogPosteriorRawPlotsMenu');
%
controls.dsgevarview.rawdraws.weights = uimenu('Parent',controls.dsgevarview.rawdrawsparent, ...
   'Label','&Particle Weights...', ...
   'Callback','DSGEVARMenuFunctions RawPosteriorDraws', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARWeightsRawPlotsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        Scatter-plot posterior draws                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.scatterplotsparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Scatt&er-Plot Posterior Draws', ...
   'Enable','on', ...
   'Tag','DSGEVARScatterPlotsParentMenu');
%
controls.dsgevarview.scatterplots.originalparameters = uimenu('Parent',controls.dsgevarview.scatterplotsparent, ...
   'Label','&Original Parameters...', ...
   'Callback','DSGEVARMenuFunctions ScatterPlots', ...
   'Enable','off', ...
   'Tag','DSGEVAROriginalScatterPlotsMenu');
%
controls.dsgevarview.scatterplots.transformedparameters = uimenu('Parent',controls.dsgevarview.scatterplotsparent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','DSGEVARMenuFunctions ScatterPlots', ...
   'Enable','off', ...
   'Tag','DSGEVARTransformedScatterPlotsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                       Marginal likelihood estimation                        %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.sequentialmarglikeparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Se&quential Marginal Likelihood', ...
   'Enable','on', ...
   'Tag','DSGEVARMarginalLikelihoodParentMenu');
%
% Laplace approximation
%
controls.dsgevarview.sequentialmarglike.laplaceparent = uimenu('Parent',controls.dsgevarview.sequentialmarglikeparent, ...
   'Label','&Laplace Approximation', ...
   'Enable','on', ...
   'Tag','DSGEVARMarginalLikelihoodLaplaceMenu');
%
% initial values
%
controls.dsgevarview.sequentialmarglike.laplace.initialvalues = uimenu('Parent',controls.dsgevarview.sequentialmarglike.laplaceparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions LaplaceInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodLaplaceInitValMenu');
%
% DSGE posterior mode
%
controls.dsgevarview.sequentialmarglike.laplace.dsgemode = uimenu('Parent',controls.dsgevarview.sequentialmarglike.laplaceparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions LaplacePosteriorMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodLaplaceDSGEPostModeMenu');
%
% Marginal posterior mode
%
controls.dsgevarview.sequentialmarglike.laplace.margmode = uimenu('Parent',controls.dsgevarview.sequentialmarglike.laplaceparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions LaplacePosteriorMode', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodLaplaceMargPostModeMenu');
%
% Joint posterior mode
%
controls.dsgevarview.sequentialmarglike.laplace.jointmode = uimenu('Parent',controls.dsgevarview.sequentialmarglike.laplaceparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions LaplacePosteriorMode', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodLaplaceJointPostModeMenu');
%
% Chib and Jeliazkov
%
controls.dsgevarview.sequentialmarglike.chibjeliazkov = uimenu('Parent',controls.dsgevarview.sequentialmarglikeparent, ...
   'Label','&Chib and Jeliazkov...', ...
   'Callback','DSGEVARMenuFunctions ChibJeliazkov', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodCJMenu');
%
% Geweke's  modified harmonic mean
%
controls.dsgevarview.sequentialmarglike.mhm = uimenu('Parent',controls.dsgevarview.sequentialmarglikeparent, ...
   'Label','&Modified Harmonic Mean...', ...
   'Callback','DSGEVARMenuFunctions ModifiedHarmonicMean', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodMHMMenu');
%
% Sims, Waggoner and Zha's version
%
controls.dsgevarview.sequentialmarglike.swz = uimenu('Parent',controls.dsgevarview.sequentialmarglikeparent, ...
   'Label','&Sims, Waggoner and Zha...', ...
   'Callback','DSGEVARMenuFunctions TruncatedElliptical', ...
   'Enable','off', ...
   'Tag','DSGEVARMarginalLikelihoodSWZMHMMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                 convergence                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergenceparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Con&vergence', ...
   'Enable','on', ...
   'Tag','DSGEVARConvergenceMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                 CUSUM plots                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.cusumplotsparent = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','&CUSUM Plots', ...
   'Enable','on', ...
   'Tag','DSGEVARCusumPlotsMenu');
%
controls.dsgevarview.convergence.cusumplots.logposterior = uimenu('Parent',controls.dsgevarview.convergence.cusumplotsparent, ...
   'Label','&Log Posterior Kernel...', ...
   'Callback','DSGEVARMenuFunctions CUSUMLogPosterior', ...
   'Enable','off', ...
   'Tag','DSGEVARCusumPlotsLogPosteriorMenu');
%
controls.dsgevarview.convergence.cusumplots.original = uimenu('Parent',controls.dsgevarview.convergence.cusumplotsparent, ...
   'Label','&Original Parameters...', ...
   'Callback','DSGEVARMenuFunctions CUSUMParameters', ...
   'Enable','off', ...
   'Tag','DSGEVARCusumPlotsOriginalParametersMenu');
%
controls.dsgevarview.convergence.cusumplots.transformed = uimenu('Parent',controls.dsgevarview.convergence.cusumplotsparent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','DSGEVARMenuFunctions CUSUMParameters', ...
   'Enable','off', ...
   'Tag','DSGEVARCusumPlotsTransformedParametersMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        Separated partial means test                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.spmtparent = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','&Separated Partial Means Test', ...
   'Enable','on', ...
   'Tag','SeparatedPartialMeansTestMenu');
%
controls.dsgevarview.convergence.spmt.logposterior = uimenu('Parent',controls.dsgevarview.convergence.spmtparent, ...
   'Label','&Log Posterior Kernel...', ...
   'Callback','DSGEVARMenuFunctions SPMTLogPosterior', ...
   'Enable','off', ...
   'Tag','DSGEVARSPMTLogPosteriorMenu');
%
controls.dsgevarview.convergence.spmt.original = uimenu('Parent',controls.dsgevarview.convergence.spmtparent, ...
   'Label','&Original Parameters...', ...
   'Callback','DSGEVARMenuFunctions SPMTParameters', ...
   'Enable','off', ...
   'Tag','DSGEVARCusumPlotsOriginalParametersMenu');
%
controls.dsgevarview.convergence.spmt.transformed = uimenu('Parent',controls.dsgevarview.convergence.spmtparent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','DSGEVARMenuFunctions SPMTParameters', ...
   'Enable','off', ...
   'Tag','DSGEVARCusumPlotsTransformedParametersMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                              Acceptance ratio                               %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.acceptancerate = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','Acceptance &Ratio...', ...
   'Callback','DSGEVARMenuFunctions ConvergeAcceptance', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARAcceptanceRatioConvergeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                SMC adaption                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.smcadaption = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','SMC A&daption...', ...
   'Callback','DSGEVARMenuFunctions SMCAdapation', ...
   'Enable','off', ...
   'Tag','DSGEVARSMCAdaptionConvergeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                   MANOVA                                    %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.manova = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','Multivariate &ANOVA...', ...
   'Callback','DSGEVARMenuFunctions ConvergeMANOVA', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARMANOVAConvergeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                          Recursive posterior mean                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.posteriormean = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','&Posterior Mean...', ...
   'Callback','DSGEVARMenuFunctions ConvergeMean', ...
   'Enable','off', ...
   'Tag','DSGEVARConvergeMeanMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         Recursive posterior median                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.convergence.posteriormedian = uimenu('Parent',controls.dsgevarview.convergenceparent, ...
   'Label','Posterior &Median...', ...
   'Callback','DSGEVARMenuFunctions ConvergeMedian', ...
   'Enable','off', ...
   'Tag','DSGEVARConvergeMedianMenu');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             Modesty statistics                              %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.modestyparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Modest&y Statistics', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','DSGEVARModestyStatisticsParentMenu');
%
% Initial values
%
controls.dsgevarview.modesty.initialvalues = uimenu('Parent',controls.dsgevarview.modestyparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions ModestyStatistics', ...
   'UserData','', ...
   'Enable','off', ...
   'Tag','DSGEVARModestyStatisticsInitialValuesMenu');
%
% prior distribution
%
controls.dsgevarview.modesty.prior = uimenu('Parent',controls.dsgevarview.modestyparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions ModestyStatistics', ...
   'UserData','', ...
   'Enable','off', ...
   'Tag','DSGEVARModestyStatisticsPriorDistributionMenu');
%
% DSGE posterior mode
%
controls.dsgevarview.modesty.dsgemode = uimenu('Parent',controls.dsgevarview.modestyparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ModestyStatistics', ...
   'UserData','', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARModestyStatisticsDSGEPosteriorModeMenu');
%
% Marginal posterior mode
%
controls.dsgevarview.modesty.margmode = uimenu('Parent',controls.dsgevarview.modestyparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ModestyStatistics', ...
   'UserData','', ...
   'Enable','off', ...
   'Tag','DSGEVARModestyStatisticsMarginalPosteriorModeMenu');
%
% Joint posterior mode
%
controls.dsgevarview.modesty.jointmode = uimenu('Parent',controls.dsgevarview.modestyparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ModestyStatistics', ...
   'UserData','', ...
   'Enable','off', ...
   'Tag','DSGEVARModestyStatisticsJointPosteriorModeMenu');
%
% posterior draws
%
for i=1:10;
   ModHandle = uimenu('Parent',controls.dsgevarview.modestyparent, ...
      'Label',['Chain &' int2str(i) '...'], ...
      'Callback','DSGEVARMenuFunctions ModestyStatistics', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag',['ModestyChain' int2str(i) 'Menu']);
   if i==1;
      set(ModHandle,'Separator','on');
   end;
   if i==10;
      set(ModHandle,'Label','Chain 1&0...');
   end;
   controls.dsgevarview.modesty = setfield(controls.dsgevarview.modesty,['chain' num2str(i,'%0.0f')],ModHandle);
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        Compare estimated parameters                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.compareparametersparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Compare Est&imated Parameters', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARCompareParametersModeMenu');
%
% marginal posterior mode
%
controls.dsgevarview.compareparameters.margmode = uimenu('Parent',controls.dsgevarview.compareparametersparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CompareEstimatedParameters', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareParametersMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevarview.compareparameters.jointmode = uimenu('Parent',controls.dsgevarview.compareparametersparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CompareEstimatedParameters', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareParametersJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevarview.compareparameters.posterior = uimenu('Parent',controls.dsgevarview.compareparametersparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions CEPPosteriorDist', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareParametersPosteriorDistModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                            Check posterior mode                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.checkpostmodeparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Chec&k Posterior Mode', ...
   'Enable','on', ...
   'Tag','DSGEVARCheckPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevarview.checkpostmode.margmode = uimenu('Parent',controls.dsgevarview.checkpostmodeparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CheckPosteriorMode', ...
   'Enable','off', ...
   'Tag','DSGEVARCheckMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevarview.checkpostmode.jointmode = uimenu('Parent',controls.dsgevarview.checkpostmodeparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CheckPosteriorMode', ...
   'Enable','off', ...
   'Tag','DSGEVARCheckJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         Compare marginal likelihood                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.marglikeparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','C&ompare Marginal Likelihood', ...
   'Enable','on', ...
   'Tag','DSGEVARCompareMarginalLikelihoodMenu');
%
% Chib and Jeliazkov
%
controls.dsgevarview.marglike.chibjeliazkov = uimenu('Parent',controls.dsgevarview.marglikeparent, ...
   'Label','&Chib and Jeliazkov...', ...
   'Callback','DSGEVARMenuFunctions CompareChib', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareMarginalLikelihoodChibJeliazkovMenu');
%
% Modified Harmonic Mean
%
controls.dsgevarview.marglike.modifiedharmonic = uimenu('Parent',controls.dsgevarview.marglikeparent, ...
   'Label','&Modified Harmonic Mean...', ...
   'Callback','DSGEVARMenuFunctions CompareModifiedHarmonic', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareMarginalLikelihoodModifiedHarmonicMenu');
%
% Sims, Waggoner and Zha's version of the modified harmonic mean
%
controls.dsgevarview.marglike.swzmodifiedharmonic = uimenu('Parent',controls.dsgevarview.marglikeparent, ...
   'Label','&Sims, Waggoner and Zha...', ...
   'Callback','DSGEVARMenuFunctions CompareSWZModifiedHarmonic', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareMarginalLikelihoodSWZModifiedHarmonicMenu');
%
% Sequential Monte Carlo
%
controls.dsgevarview.marglike.smc = uimenu('Parent',controls.dsgevarview.marglikeparent, ...
   'Label','S&equential Monte Carlo...', ...
   'Callback','DSGEVARMenuFunctions CompareSMCMargLike', ...
   'Enable','off', ...
   'Tag','DSGEVARCompareMarginalLikelihoodSMCMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                               Prior densities                               %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.priordensityparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','P&rior Densities', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARPriorDensityMenu');
%
% VAR parameters
%
controls.dsgevarview.priordensity.varparameter = uimenu('Parent',controls.dsgevarview.priordensityparent, ...
   'Label','&VAR Parameters...', ...
   'Callback','DSGEVARMenuFunctions VARPriorDensity', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensityVARParamMenu');
%
% Sigma parameters
%
controls.dsgevarview.priordensity.sigmaparameter = uimenu('Parent',controls.dsgevarview.priordensityparent, ...
   'Label','&Sigma Parameters...', ...
   'Callback','DSGEVARMenuFunctions VARPriorDensity', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensitySigmaParamMenu');
%
% A0 parameters
%
controls.dsgevarview.priordensity.a0parameter = uimenu('Parent',controls.dsgevarview.priordensityparent, ...
   'Label','&A0 Parameters...', ...
   'Callback','DSGEVARMenuFunctions VARPriorDensity', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensityA0ParamMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                   Laplace approximation of the posterior                    %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.viewlaplaceposteriorparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','&Laplace Posterior Densities', ...
   'Enable','on', ...
   'Tag','DSGEVARViewLaplacePosteriorMenu');
%
% marginal posterior mode
%
controls.dsgevarview.viewlaplaceposterior.margmode = uimenu('Parent',controls.dsgevarview.viewlaplaceposteriorparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ViewLaplacePosterior', ...
   'Enable','off', ...
   'Tag','DSGEVARViewLaplaceMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevarview.viewlaplaceposterior.jointmode = uimenu('Parent',controls.dsgevarview.viewlaplaceposteriorparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ViewLaplacePosterior', ...
   'Enable','off', ...
   'Tag','DSGEVARViewLaplaceJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             Posterior densities                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarview.postdensityparent = uimenu('Parent',controls.dsgevarviewparent, ...
   'Label','Pos&terior Densities', ...
   'Enable','on', ...
   'Tag','DSGEVARPosteriorDensityMenu');
%
% DSGE Model parameters
%
controls.dsgevarview.postdensity.dsgeparameter = uimenu('Parent',controls.dsgevarview.postdensityparent, ...
   'Label','&DSGE Parameters...', ...
   'Callback','DSGEVARMenuFunctions DSGEPosteriorDensity', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensityDSGEParamMenu');
%
% VAR parameters
%
controls.dsgevarview.postdensity.varparameter = uimenu('Parent',controls.dsgevarview.postdensityparent, ...
   'Label','&VAR Parameters...', ...
   'Callback','DSGEVARMenuFunctions VARPosteriorDensity', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensityVARParamMenu');
%
% Sigma parameters
%
controls.dsgevarview.postdensity.sigmaparameter = uimenu('Parent',controls.dsgevarview.postdensityparent, ...
   'Label','&Sigma Parameters...', ...
   'Callback','DSGEVARMenuFunctions VARPosteriorDensity', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensitySigmaParamMenu');
%
% A0 parameters
%
controls.dsgevarview.postdensity.a0parameter = uimenu('Parent',controls.dsgevarview.postdensityparent, ...
   'Label','&A0 Parameters...', ...
   'Callback','DSGEVARMenuFunctions VARPosteriorDensity', ...
   'Enable','off', ...
   'Tag','DSGEVARPosteriorDensityA0ParamMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                    Tools                                    %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartoolsparent = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','&Tools', ...
   'Enable','on', ...
   'Tag','DSGEVARToolsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                Simulate data                                %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.simulatedataparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','Sim&ulate Data', ...
   'Enable','on', ...
   'Tag','DSGEVARSimulateDataMenu');
%
% initial values
%
controls.dsgevartools.simulatedata.initialvalues = uimenu('Parent',controls.dsgevartools.simulatedataparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARSimulateDataInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARSimulateDataInitialValuesMenu');
%
% DSGE model posterior mode
%
controls.dsgevartools.simulatedata.dsgemode = uimenu('Parent',controls.dsgevartools.simulatedataparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARSimulateDataPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARSimulateDataDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.simulatedata.margmode = uimenu('Parent',controls.dsgevartools.simulatedataparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARSimulateDataPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARSimulateDataMarginalPosteriorModeMenu');
%
controls.dsgevartools.simulatedata.jointmode = uimenu('Parent',controls.dsgevartools.simulatedataparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARSimulateDataPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARSimulateDataJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                       Observed variable correlations                        %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.obscorrparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','&Observed Variable Correlations', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARObsCorrMenu');
%
% Frequency domain
%
controls.dsgevartools.obscorr.frequencydomainparent = uimenu('Parent',controls.dsgevartools.obscorrparent, ...
   'Label','&Frequency Domain', ...
   'Enable','on', ...
   'Tag','DSGEVARObsCorrFrequencyDomainMenu');
%
% initial values
%
controls.dsgevartools.obscorr.frequencydomain.initialvalues = uimenu('Parent',controls.dsgevartools.obscorr.frequencydomainparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CoherenceInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARCoherenceInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.obscorr.frequencydomain.prior = uimenu('Parent',controls.dsgevartools.obscorr.frequencydomainparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CoherencePriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARCoherencePriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.obscorr.frequencydomain.dsgemode = uimenu('Parent',controls.dsgevartools.obscorr.frequencydomainparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CoherencePostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARCoherenceDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.obscorr.frequencydomain.margmode = uimenu('Parent',controls.dsgevartools.obscorr.frequencydomainparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CoherencePostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARCoherenceMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.obscorr.frequencydomain.jointmode = uimenu('Parent',controls.dsgevartools.obscorr.frequencydomainparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CoherencePostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARCoherenceJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.obscorr.frequencydomain.posterior = uimenu('Parent',controls.dsgevartools.obscorr.frequencydomainparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CoherencePosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARCoherencePosteriorDistMenu');
%
% Time domain
%
controls.dsgevartools.obscorr.timedomainparent = uimenu('Parent',controls.dsgevartools.obscorrparent, ...
   'Label','&Time Domain', ...
   'Enable','on', ...
   'Tag','DSGEVARObsCorrTimeDomainMenu');
%
% initial values
%
controls.dsgevartools.obscorr.timedomain.initialvalues = uimenu('Parent',controls.dsgevartools.obscorr.timedomainparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ObsVarCorrInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARObsCorrTimeDomainInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.obscorr.timedomain.prior = uimenu('Parent',controls.dsgevartools.obscorr.timedomainparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ObsVarCorrPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARObsCorrTimeDomainPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.obscorr.timedomain.dsgemode = uimenu('Parent',controls.dsgevartools.obscorr.timedomainparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ObsVarCorrPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARObsCorrTimeDomainDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.obscorr.timedomain.margmode = uimenu('Parent',controls.dsgevartools.obscorr.timedomainparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ObsVarCorrPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARObsCorrTimeDomainMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.obscorr.timedomain.jointmode = uimenu('Parent',controls.dsgevartools.obscorr.timedomainparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ObsVarCorrPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARObsCorrTimeDomainJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.obscorr.timedomain.posterior = uimenu('Parent',controls.dsgevartools.obscorr.timedomainparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ObsVarCorrPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARObsCorrTimeDomainPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                          Conditional correlations                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.condcorrparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','&Conditional Correlations', ...
   'Enable','on', ...
   'Tag','DSGEVARCondCorrMenu');
%
%
% initial values
%
controls.dsgevartools.condcorr.initialvalues = uimenu('Parent',controls.dsgevartools.condcorrparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CondCorrInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARCondCorrInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.condcorr.prior = uimenu('Parent',controls.dsgevartools.condcorrparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CondCorrPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARCondCorrPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.condcorr.dsgemode = uimenu('Parent',controls.dsgevartools.condcorrparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CondCorrPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARCondCorrDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.condcorr.margmode = uimenu('Parent',controls.dsgevartools.condcorrparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CondCorrPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARCondCorrMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.condcorr.jointmode = uimenu('Parent',controls.dsgevartools.condcorrparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CondCorrPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARCondCorrJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.condcorr.posterior = uimenu('Parent',controls.dsgevartools.condcorrparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions CondCorrPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARCondCorrPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             Shock correlations                              %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.shockcorrparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','&Structural Shock Correlations', ...
   'Enable','on', ...
   'Tag','DSGEVARShockCorrelationsMenu');
%
% initial values
%
controls.dsgevartools.shockcorr.initialvalues = uimenu('Parent',controls.dsgevartools.shockcorrparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ShockCorrelationsInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARShockCorrelationsInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.shockcorr.prior = uimenu('Parent',controls.dsgevartools.shockcorrparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ShockCorrelationsPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARShockCorrelationsPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.shockcorr.dsgemode = uimenu('Parent',controls.dsgevartools.shockcorrparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ShockCorrelationsPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARShockCorrelationsDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
%
controls.dsgevartools.shockcorr.margmode = uimenu('Parent',controls.dsgevartools.shockcorrparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ShockCorrelationsPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARShockCorrelationsMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.shockcorr.jointmode = uimenu('Parent',controls.dsgevartools.shockcorrparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ShockCorrelationsPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARShockCorrelationsJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.shockcorr.posterior = uimenu('Parent',controls.dsgevartools.shockcorrparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARCorrToolsMenuFunctions ShockCorrelationsPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARShockCorrelationsPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                              Structural shocks                              %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.shocksparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','Structural Shoc&ks', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARStructuralShocksMenu');
%
% initial values
%
controls.dsgevartools.shocks.initialvalues = uimenu('Parent',controls.dsgevartools.shocksparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions ShocksInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARShocksInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.shocks.prior = uimenu('Parent',controls.dsgevartools.shocksparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions ShocksPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARShocksPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.shocks.dsgemode = uimenu('Parent',controls.dsgevartools.shocksparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ShocksPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARShocksDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.shocks.margmode = uimenu('Parent',controls.dsgevartools.shocksparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ShocksPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARShocksMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.shocks.jointmode = uimenu('Parent',controls.dsgevartools.shocksparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ShocksPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARShocksJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.shocks.posterior = uimenu('Parent',controls.dsgevartools.shocksparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions ShocksPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARShocksPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                      Graph the log-likelihood function                      %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.loglikelihoodparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Separator','on', ...
   'Label','&Log-likelihood Function', ...
   'Tag','LogLikelihoodMenu');
%
controls.dsgevartools.loglikelihood.initialvalues = uimenu('Parent',controls.dsgevartools.loglikelihoodparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions LogLikelihoodInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARLogLikelihoodInitialValuesMenu');
%
controls.dsgevartools.loglikelihood.dsgemode = uimenu('Parent',controls.dsgevartools.loglikelihoodparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions LogLikelihoodPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARLogLikelihoodDSGEPosteriorModeMenu');
%
controls.dsgevartools.loglikelihood.margmode = uimenu('Parent',controls.dsgevartools.loglikelihoodparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions LogLikelihoodPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARLogLikelihoodMarginalPosteriorModeMenu');
%
controls.dsgevartools.loglikelihood.jointmode = uimenu('Parent',controls.dsgevartools.loglikelihoodparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions LogLikelihoodPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARLogLikelihoodJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                  Scenarios                                  %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if FileExist(FixFilePath([pwd '\addons\ovs\OVSControls.m']))==1;
   controls = OVSControls(controls,'dsgevarmenu');
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                          Predictive Distributions                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.forecastparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','&Predictive Distributions', ...
   'Enable','on', ...
   'Tag','DSGEVARForecastParentMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Unconditional forecasts                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.forecast.unconditionalparent = uimenu('Parent',controls.dsgevartools.forecastparent, ...
   'Label','&Unconditional', ...
   'Tag','DSGEVARUnconditionalForecastParentMenu');
%
% original data
%
controls.dsgevartools.forecast.unconditional.originalparent = uimenu('Parent',controls.dsgevartools.forecast.unconditionalparent, ...
   'Label','&Original Data', ...
   'Tag','DSGEVARUnconditionalForecastOriginalParentMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annualparent = uimenu('Parent',controls.dsgevartools.forecast.unconditionalparent, ...
   'Label','&Annualized Data', ...
   'Tag','DSGEVARUnconditionalForecastAnnualParentMenu');
%
% transformed data
%
controls.dsgevartools.forecast.unconditional.transformedparent = uimenu('Parent',controls.dsgevartools.forecast.unconditionalparent, ...
   'Label','&Transformed Data', ...
   'Tag','DSGEVARUnconditionalForecastTransformedParentMenu');
%
% predictive likelihood
%
controls.dsgevartools.forecast.unconditional.original.heightparent = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','Predictive &Likelihood', ...
   'Enable','on', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalParentMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.heightparent = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','Predictive &Likelihood', ...
   'Enable','on', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualParentMenu');
%
% initial values
%
controls.dsgevartools.forecast.unconditional.original.height.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.heightparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightInitialValues', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalInitialValuesMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.height.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annual.heightparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightInitialValues', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.forecast.unconditional.original.height.prior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.heightparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalPriorDistMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.height.prior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annual.heightparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.forecast.unconditional.original.height.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.heightparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalDSGEPosteriorModeMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.height.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annual.heightparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.forecast.unconditional.original.height.margmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.heightparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalMarginalPosteriorModeMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.height.margmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annual.heightparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.forecast.unconditional.original.height.jointmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.heightparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalJointPosteriorModeMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.height.jointmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annual.heightparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.forecast.unconditional.original.height.posterior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.heightparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightOriginalPosteriorDistMenu');
%
% annualized data
%
controls.dsgevartools.forecast.unconditional.annual.height.posterior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annual.heightparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredHeightPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveDensityHeightAnnualPosteriorDistMenu');
%
% marginal predictive moments
%
controls.dsgevartools.forecast.unconditional.original.momentsparent = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','M&arginal Predictive Moments', ...
   'Enable','on', ...
   'Tag','DSGEVARUnconditionalPredictiveMomentsOriginalParentMenu');
%
% prior distribution
%
controls.dsgevartools.forecast.unconditional.original.moments.prior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.momentsparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UnconMargPredMomentsPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveMomentsOriginalPriorDistMenu');
%
% posterior distribution
%
controls.dsgevartools.forecast.unconditional.original.moments.posterior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.original.momentsparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UnconMargPredMomentsPosteriorDist', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalPredictiveMomentsOriginalPosteriorDistMenu');
%
% initial values
%
controls.dsgevartools.forecast.unconditional.original.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondInitialValues', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARUnconditionalForecastOriginalInitialValuesMenu');
%
controls.dsgevartools.forecast.unconditional.annual.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondInitialValues', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARUnconditionalForecastAnnualInitialValuesMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.forecast.unconditional.original.prior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastOriginalPriorDistMenu');
%
controls.dsgevartools.forecast.unconditional.annual.prior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastAnnualPriorDistMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.prior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.forecast.unconditional.original.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastOriginalDSGEPosteriorModeMenu');
%
controls.dsgevartools.forecast.unconditional.annual.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastAnnualDSGEPosteriorModeMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.forecast.unconditional.original.margmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastOriginalMarginalPosteriorModeMenu');
%
controls.dsgevartools.forecast.unconditional.annual.margmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastAnnualMarginalPosteriorModeMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.margmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.forecast.unconditional.original.jointmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastOriginalJointPosteriorModeMenu');
%
controls.dsgevartools.forecast.unconditional.annual.jointmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastAnnualJointPosteriorModeMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.jointmode = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.forecast.unconditional.original.posterior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastOriginalPosteriorDistMenu');
%
controls.dsgevartools.forecast.unconditional.annual.posterior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastAnnualPosteriorDistMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.posterior = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedPosteriorDistMenu');
%
% prediction event
%
controls.dsgevartools.forecast.unconditional.original.predevent = uimenu('Parent',controls.dsgevartools.forecast.unconditional.originalparent, ...
   'Label','Prediction &Event...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredictionEvent', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastOriginalPredictionEventMenu');
%
controls.dsgevartools.forecast.unconditional.annual.predevent = uimenu('Parent',controls.dsgevartools.forecast.unconditional.annualparent, ...
   'Label','Prediction &Event...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredictionEvent', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastAnnualPredictionEventMenu');
%
controls.dsgevartools.forecast.unconditional.transformed.predevent = uimenu('Parent',controls.dsgevartools.forecast.unconditional.transformedparent, ...
   'Label','Prediction &Event...', ...
   'Callback','DSGEVARForecastMenuFunctions UncondPredictionEvent', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARUnconditionalForecastTransformedPredictionEventMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                            Conditional forecasts                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.forecast.conditionalparent = uimenu('Parent',controls.dsgevartools.forecastparent, ...
   'Label','&Conditional', ...
   'Tag','DSGEVARConditionalForecastParentMenu');
%
% original data
%
controls.dsgevartools.forecast.conditional.originalparent = uimenu('Parent',controls.dsgevartools.forecast.conditionalparent, ...
   'Label','&Original Data', ...
   'Tag','DSGEVARConditionalForecastOriginalParentMenu');
%
% annualized data
%
controls.dsgevartools.forecast.conditional.annualparent = uimenu('Parent',controls.dsgevartools.forecast.conditionalparent, ...
   'Label','&Annualized Data', ...
   'Tag','DSGEVARConditionalForecastAnnualParentMenu');
%
% transformed data
%
controls.dsgevartools.forecast.conditional.transformedparent = uimenu('Parent',controls.dsgevartools.forecast.conditionalparent, ...
   'Label','&Transformed Data', ...
   'Tag','DSGEVARConditionalForecastTransformedParentMenu');
%
% initial values
%
controls.dsgevartools.forecast.conditional.original.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions CondInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalInitialValuesMenu');
%
controls.dsgevartools.forecast.conditional.annual.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions CondInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualInitialValuesMenu');
%
controls.dsgevartools.forecast.conditional.transformed.initialvalues = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARForecastMenuFunctions CondInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.forecast.conditional.original.prior = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalPriorDistMenu');
%
controls.dsgevartools.forecast.conditional.annual.prior = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualPriorDistMenu');
%
controls.dsgevartools.forecast.conditional.transformed.prior = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.forecast.conditional.original.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalDSGEPosteriorModeMenu');
%
controls.dsgevartools.forecast.conditional.annual.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualDSGEPosteriorModeMenu');
%
controls.dsgevartools.forecast.conditional.transformed.dsgemode = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.forecast.conditional.original.margmode = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalMarginalPosteriorModeMenu');
%
controls.dsgevartools.forecast.conditional.annual.margmode = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualMarginalPosteriorModeMenu');
%
controls.dsgevartools.forecast.conditional.transformed.margmode = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.forecast.conditional.original.jointmode = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalJointPosteriorModeMenu');
%
controls.dsgevartools.forecast.conditional.annual.jointmode = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualJointPosteriorModeMenu');
%
controls.dsgevartools.forecast.conditional.transformed.jointmode = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.forecast.conditional.original.posterior = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalPosteriorDistMenu');
%
controls.dsgevartools.forecast.conditional.annual.posterior = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualPosteriorDistMenu');
%
controls.dsgevartools.forecast.conditional.transformed.posterior = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedPosteriorDistMenu');
%
% prediction event
%
controls.dsgevartools.forecast.conditional.original.predevent = uimenu('Parent',controls.dsgevartools.forecast.conditional.originalparent, ...
   'Label','Prediction &Event...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPredictionEvent', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastOriginalPredictionEventMenu');
%
controls.dsgevartools.forecast.conditional.annual.predevent = uimenu('Parent',controls.dsgevartools.forecast.conditional.annualparent, ...
   'Label','Prediction &Event...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPredictionEvent', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastAnnualPredictionEventMenu');
%
controls.dsgevartools.forecast.conditional.transformed.predevent = uimenu('Parent',controls.dsgevartools.forecast.conditional.transformedparent, ...
   'Label','Prediction &Event...', ...
   'Callback','DSGEVARForecastMenuFunctions CondPredictionEvent', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARConditionalForecastTransformedPredictionEventMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                               Decompositions                                %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decompositionparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','&Decompositions', ...
   'Enable','on', ...
   'Separator','on', ...
   'Tag','DSGEVARDecompositionsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                      Observed variable decompositions                       %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decomposition.ovd.parent = uimenu('Parent',controls.dsgevartools.decompositionparent, ...
   'Label','&Observed Variables', ...
   'Enable','on', ...
   'Tag','DSGEVARObsVarDecMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Decompositions submenu                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decomposition.ovd.decomp.parent = uimenu('Parent',controls.dsgevartools.decomposition.ovd.parent, ...
   'Label','&Decompositions', ...
   'Enable','on', ...
   'Tag','DSGEVAROVDecompMenu');
%
% original data
%
controls.dsgevartools.decomposition.ovd.decomp.original.parent = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.parent, ...
   'Label','&Original Data', ...
   'Enable','on', ...
   'Tag','DSGEVAROVDecompOriginalMenu');
%
% annualized data
%
controls.dsgevartools.decomposition.ovd.decomp.annual.parent = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.parent, ...
   'Label','&Annualized Data', ...
   'Enable','on', ...
   'Tag','DSGEVAROVDecompAnnualMenu');
%
% initial values
%
controls.dsgevartools.decomposition.ovd.decomp.original.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.original.parent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompOriginalInitialValuesMenu');
%
controls.dsgevartools.decomposition.ovd.decomp.annual.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.annual.parent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompAnnualInitialValuesMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.decomposition.ovd.decomp.original.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.original.parent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompOriginalDSGEPosteriorModeMenu');
%
controls.dsgevartools.decomposition.ovd.decomp.annual.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.annual.parent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompAnnualDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.decomposition.ovd.decomp.original.margmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.original.parent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompOriginalMarginalPosteriorModeMenu');
%
controls.dsgevartools.decomposition.ovd.decomp.annual.margmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.annual.parent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompAnnualMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.decomposition.ovd.decomp.original.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.original.parent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompOriginalJointPosteriorModeMenu');
%
controls.dsgevartools.decomposition.ovd.decomp.annual.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.decomp.annual.parent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarDecompPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVAROVDecompAnnualJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                Paths submenu                                %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decomposition.ovd.paths.parent = uimenu('Parent',controls.dsgevartools.decomposition.ovd.parent, ...
   'Label','&Paths', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','DSGEVAROVPathsMenu');
%
% original data
%
controls.dsgevartools.decomposition.ovd.paths.original.parent = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.parent, ...
   'Label','&Original Data', ...
   'Enable','on', ...
   'Tag','DSGEVAROVPathsOriginalMenu');
%
% annualized data
%
controls.dsgevartools.decomposition.ovd.paths.annual.parent = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.parent, ...
   'Label','&Annualized Data', ...
   'Enable','on', ...
   'Tag','DSGEVAROVPathsAnnualMenu');
%
% initial values
%
controls.dsgevartools.decomposition.ovd.paths.original.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.original.parent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsOriginalInitialValuesMenu');
%
controls.dsgevartools.decomposition.ovd.paths.annual.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.annual.parent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsAnnualInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.decomposition.ovd.paths.original.prior = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.original.parent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsOriginalPriorDistMenu');
%
controls.dsgevartools.decomposition.ovd.paths.annual.prior = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.annual.parent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsAnnualPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.decomposition.ovd.paths.original.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.original.parent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsOriginalDSGEPosteriorModeMenu');
%
controls.dsgevartools.decomposition.ovd.paths.annual.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.annual.parent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsAnnualDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.decomposition.ovd.paths.original.margmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.original.parent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsOriginalMarginalPosteriorModeMenu');
%
controls.dsgevartools.decomposition.ovd.paths.annual.margmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.annual.parent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsAnnualMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.decomposition.ovd.paths.original.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.original.parent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsOriginalJointPosteriorModeMenu');
%
controls.dsgevartools.decomposition.ovd.paths.annual.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.annual.parent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsAnnualJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.decomposition.ovd.paths.original.posterior = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.original.parent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsOriginalPosteriorDistMenu');
%
controls.dsgevartools.decomposition.ovd.paths.annual.posterior = uimenu('Parent',controls.dsgevartools.decomposition.ovd.paths.annual.parent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions ObsVarPathsPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVAROVPathsAnnualPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         Correlation decompositions                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decomposition.corrdecomp.parent = uimenu('Parent',controls.dsgevartools.decompositionparent, ...
   'Label','Co&rrelation Decompositions', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','DSGEVARCorrelationDecompMenu');
%
%
% initial values
%
controls.dsgevartools.decomposition.corrdecomp.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.corrdecomp.parent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions CorrDecompInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARCorrelationDecompInitialValuesMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.decomposition.corrdecomp.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.corrdecomp.parent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CorrDecompPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARCorrelationDecompDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.decomposition.corrdecomp.margmode = uimenu('Parent',controls.dsgevartools.decomposition.corrdecomp.parent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CorrDecompPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARCorrelationDecompMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.decomposition.corrdecomp.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.corrdecomp.parent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions CorrDecompPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARCorrelationDecompJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Spectral decompositions                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decomposition.spectraldecomp.parent = uimenu('Parent',controls.dsgevartools.decompositionparent, ...
   'Label','&Spectral Decompositions', ...
   'Enable','on', ...
   'Tag','DSGEVARSpectralDecompMenu');
%
% initial values
%
controls.dsgevartools.decomposition.spectraldecomp.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.spectraldecomp.parent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions SpectralDecompInitialValues', ...
   'Enable','off', ...
   'Tag','DSGEVARSpectralDecompInitialValuesMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.decomposition.spectraldecomp.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.spectraldecomp.parent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions SpectralDecompPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARSpectralDecompDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.decomposition.spectraldecomp.margmode = uimenu('Parent',controls.dsgevartools.decomposition.spectraldecomp.parent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions SpectralDecompPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARSpectralDecompMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.decomposition.spectraldecomp.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.spectraldecomp.parent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions SpectralDecompPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARSpectralDecompJointPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           Variance decompositions                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.decomposition.fevd.parent = uimenu('Parent',controls.dsgevartools.decompositionparent, ...
   'Label','&Variance Decompositions', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','DSGEVARVarianceDecompositionsMenu');
%
% Original data
%
controls.dsgevartools.decomposition.fevd.originalparent = uimenu('Parent',controls.dsgevartools.decomposition.fevd.parent, ...
   'Label','&Original Data', ...
   'Enable','on', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalMenu');
%
% levels data
%
controls.dsgevartools.decomposition.fevd.levelsparent = uimenu('Parent',controls.dsgevartools.decomposition.fevd.parent, ...
   'Label','&Levels Data', ...
   'Enable','on', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsMenu');
%
% initial values
%
controls.dsgevartools.decomposition.fevd.original.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.fevd.originalparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions FEVDInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalInitialValuesMenu');
%
controls.dsgevartools.decomposition.fevd.levels.initialvalues = uimenu('Parent',controls.dsgevartools.decomposition.fevd.levelsparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions FEVDInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.decomposition.fevd.original.prior = uimenu('Parent',controls.dsgevartools.decomposition.fevd.originalparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions FEVDPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalPriorDistMenu');
%
controls.dsgevartools.decomposition.fevd.levels.prior = uimenu('Parent',controls.dsgevartools.decomposition.fevd.levelsparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions FEVDPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.decomposition.fevd.original.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.fevd.originalparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions FEVDPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalDSGEPosteriorModeMenu');
%
controls.dsgevartools.decomposition.fevd.levels.dsgemode = uimenu('Parent',controls.dsgevartools.decomposition.fevd.levelsparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions FEVDPostMode', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.decomposition.fevd.original.margmode = uimenu('Parent',controls.dsgevartools.decomposition.fevd.originalparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions FEVDPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalMarginalPosteriorModeMenu');
%
controls.dsgevartools.decomposition.fevd.levels.margmode = uimenu('Parent',controls.dsgevartools.decomposition.fevd.levelsparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions FEVDPostMode', ...
   'Separator','off', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.decomposition.fevd.original.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.fevd.originalparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions FEVDPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalJointPosteriorModeMenu');
%
controls.dsgevartools.decomposition.fevd.levels.jointmode = uimenu('Parent',controls.dsgevartools.decomposition.fevd.levelsparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions FEVDPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.decomposition.fevd.original.posterior = uimenu('Parent',controls.dsgevartools.decomposition.fevd.originalparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions FEVDPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsOriginalPosteriorDistMenu');
%
controls.dsgevartools.decomposition.fevd.levels.posterior = uimenu('Parent',controls.dsgevartools.decomposition.fevd.levelsparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions FEVDPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARVarianceDecompositionsLevelsPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                              Impulse responses                              %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevartools.irfunctionparent = uimenu('Parent',controls.dsgevartoolsparent, ...
   'Label','&Impulse Responses', ...
   'Enable','on', ...
   'Tag','DSGEVARImpulseResponseMenu');
%
% original data
%
controls.dsgevartools.irfunction.originalparent = uimenu('Parent',controls.dsgevartools.irfunctionparent, ...
   'Label','&Original Data', ...
   'Enable','on', ...
   'Tag','DSGEVARImpulseResponseOriginalMenu');
%
% annualized data
%
controls.dsgevartools.irfunction.annualparent = uimenu('Parent',controls.dsgevartools.irfunctionparent, ...
   'Label','&Annualized Data', ...
   'Enable','on', ...
   'Tag','DSGEVARImpulseResponseAnnualizedMenu');
%
% levels data
%
controls.dsgevartools.irfunction.levelsparent = uimenu('Parent',controls.dsgevartools.irfunctionparent, ...
   'Label','&Levels Data', ...
   'Enable','on', ...
   'Tag','DSGEVARImpulseResponseLevelsMenu');
%
% initial values
%
controls.dsgevartools.irfunction.original.initialvalues = uimenu('Parent',controls.dsgevartools.irfunction.originalparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseOriginalInitialValuesMenu');
%
controls.dsgevartools.irfunction.annual.initialvalues = uimenu('Parent',controls.dsgevartools.irfunction.annualparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseAnnualizedInitialValuesMenu');
%
controls.dsgevartools.irfunction.levels.initialvalues = uimenu('Parent',controls.dsgevartools.irfunction.levelsparent, ...
   'Label','&Initial Values...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionInitVal', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseLevelsInitialValuesMenu');
%
% prior distribution
%
controls.dsgevartools.irfunction.original.prior = uimenu('Parent',controls.dsgevartools.irfunction.originalparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseOriginalPriorDistMenu');
%
controls.dsgevartools.irfunction.annual.prior = uimenu('Parent',controls.dsgevartools.irfunction.annualparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseAnnualizedPriorDistMenu');
%
controls.dsgevartools.irfunction.levels.prior = uimenu('Parent',controls.dsgevartools.irfunction.levelsparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseLevelsPriorDistMenu');
%
% DSGE posterior mode
%
controls.dsgevartools.irfunction.original.dsgemode = uimenu('Parent',controls.dsgevartools.irfunction.originalparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARImpulseResponseOriginalDSGEPosteriorModeMenu');
%
controls.dsgevartools.irfunction.annual.dsgemode = uimenu('Parent',controls.dsgevartools.irfunction.annualparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARImpulseResponseAnnualizedDSGEPosteriorModeMenu');
%
controls.dsgevartools.irfunction.levels.dsgemode = uimenu('Parent',controls.dsgevartools.irfunction.levelsparent, ...
   'Label','DSGE &Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARImpulseResponseLevelsDSGEPosteriorModeMenu');
%
% marginal posterior mode
%
controls.dsgevartools.irfunction.original.margmode = uimenu('Parent',controls.dsgevartools.irfunction.originalparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARImpulseResponseOriginalMarginalPosteriorModeMenu');
%
controls.dsgevartools.irfunction.annual.margmode = uimenu('Parent',controls.dsgevartools.irfunction.annualparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARImpulseResponseAnnualizedMarginalPosteriorModeMenu');
%
controls.dsgevartools.irfunction.levels.margmode = uimenu('Parent',controls.dsgevartools.irfunction.levelsparent, ...
   'Label','&Marginal Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Separator','off', ...
   'Tag','DSGEVARImpulseResponseLevelsMarginalPosteriorModeMenu');
%
% joint posterior mode
%
controls.dsgevartools.irfunction.original.jointmode = uimenu('Parent',controls.dsgevartools.irfunction.originalparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseOriginalJointPosteriorModeMenu');
%
controls.dsgevartools.irfunction.annual.jointmode = uimenu('Parent',controls.dsgevartools.irfunction.annualparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseAnnualizedJointPosteriorModeMenu');
%
controls.dsgevartools.irfunction.levels.jointmode = uimenu('Parent',controls.dsgevartools.irfunction.levelsparent, ...
   'Label','&Joint Posterior Mode...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPostMode', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseLevelsJointPosteriorModeMenu');
%
% posterior distribution
%
controls.dsgevartools.irfunction.original.posterior = uimenu('Parent',controls.dsgevartools.irfunction.originalparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseOriginalPosteriorDistMenu');
%
controls.dsgevartools.irfunction.annual.posterior = uimenu('Parent',controls.dsgevartools.irfunction.annualparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseAnnualizedPosteriorDistMenu');
%
controls.dsgevartools.irfunction.levels.posterior = uimenu('Parent',controls.dsgevartools.irfunction.levelsparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','DSGEVARMenuFunctions IRFunctionPosteriorDist', ...
   'Enable','off', ...
   'Tag','DSGEVARImpulseResponseLevelsPosteriorDistMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                            Select shocks to use                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarshocksmenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','&Set DSGE-VAR Shocks...', ...
   'Callback','DSGEVARMenuFunctions SetDSGEVARShocks', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','DSGEVARShockSelectionMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        Select lag order for DSGE-VAR                        %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.dsgevarlagordermenu = uimenu('Parent',controls.dsgevarmenu, ...
   'Label','&Lag Order...', ...
   'Callback','DSGEVARMenuFunctions DSGEVARLagOrder', ...
   'Enable','on', ...
   'Tag','DSGEVARLagOrderMenu');

%
% end of DSGEVARMenuControls.m
%
