function [LogPredLike,StdErr,PredVars] = PredLikeImportanceSampling(NumDraws,h,IsOriginal,IsMarginal,IsPosterior,NumBurnin,TotalDraws,CurrChain,DSGEModel)
% PredLikeImportanceSampling: Computes the predictive likelihood and the marginal likelihood for parameter draws using the
%                             importance sampling estimator for a DSGE model.
%
% USAGE:
%
%       [LogPredLike,StdErr,PredVars] = PredLikeImportanceSampling(NumDraws,h,IsOriginal,IsMarginal,IsPosterior,NumBurnin, ...
%                                                                  TotalDraws,CurrChain,DSGEModel)
%
% REQUIRED INPUT:  NumDraws (integer) with the number of draws used.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                             annualized values should be ptredicted.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                             distribution and 0 otherwise.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) for the selected MCMC chain when posterior draws are used.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
% REQUIRED OUTPUT: LogPredLike (vector) of dimension h with the predictive likelihood for the forecast
%                             periods.
%
%                  StdErr (vector) of dimension h with the numerical standard errors of the log predictive
%                             likelihood.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2015 European Central Bank.
%
%                       First version: April 27, 2012.
%                        This version: January 14, 2015.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-04-2012: Finished the function.
%
% * 14-05-2012: Updated the documentation.
%
% * 21-08-2012: Changed estimator to an importance sampling estimator.
%
% * 18-10-2012: Added numerical standard errors based on Newey-West.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
LogPredLike = [];
PredVars = [];
StdErr = [];
%
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
end;
PredData = load(file);
PredVars = PredData.PredVars;
LogPosteriorData = PredData.LogPosteriorData;
clear('PredData');
%
% compute the predictive likelihood as the average of the conditional likelihood evaluated at the
% posterior values of the parameters.
%
LogPosteriorData = exp(LogPosteriorData);
PredLike = sum(LogPosteriorData)'/NumDraws;
LogPredLike = log(PredLike);
%
% compute numerical standard errors with Newey-West
%
StdErrMat = NeweyWestCovMat(LogPosteriorData,floor(NumDraws^(1/2.01)));
StdErr = sqrt(diag(StdErrMat))./PredLike;

%
% end of PredLikeImportanceSampling.m
%
