function [PostMod,PostVar,Scale,PostMean,fval,gh,fcount,fdiff,abort,OptResults,ScreenMessages,phiEstimates,OptTime] = YADAgmhmaxlik(ObjFun,xparam1,mh_bounds,num,iScale,MeanPar,VarCov,RandomNumberValue,waitbar,ShowClock,waitbartext,CurrINI,varargin)  
%function [PostMod,PostVar,Scale,PostMean,fval,gh,fcount,fdiff,abort,OptResults,ScreenMessages,phiEstimates,OptTime] = YADAgmhmaxlik(ObjFun,xparam1,mh_bounds,num,iScale,MeanPar,VarCov,RandomNumberValue,waitbar,ShowClock,waitbartext,CurrINI,varargin)  
% (Dirty) Global minimization routine of (minus) a likelihood (or posterior density) function. 
% 
% INPUTS 
%   o ObjFun     [char]     string specifying the name of the objective function.
%   o xparam1    [double]   (p*1) vector of parameters to be estimated.
%   o mh_bounds  [double]   (p*2) matrix defining lower and upper bounds for the parameters.
%   o num        [integer]  scalar specifying the number of MH iterations in step 2.
%   o iScale     [double]   scalar specifying the initial of the jumping distribution's scale parameter.
%   o info       [char]     string, empty or equal to 'LastCall'. [removed]
%   o MeanPar    [double]   (p*1) vector specifying the initial posterior mean.
%   o VarCov     [double]   (p*p) matrix specifying the initial posterior covariance matrix.
%  
% OUTPUTS 
%   o PostMod    [double]   (p*1) vector, evaluation of the posterior mode.
%   o PostVar    [double]   (p*p) matrix, evaluation of the posterior covariance matrix.
%   o Scale      [double]   scalar specifying the scale parameter that should be used in 
%                           an eventual metropolis-hastings algorithm. 
%   o PostMean   [double]   (p*1) vector, evaluation of the posterior mean.  
%
% ALGORITHM 
%   Metropolis-Hastings with an constantly updated covariance matrix for
%   the jump distribution. The posterior mean, variance and mode are
%   updated (in step 2) with the following rules:
%
%   \[ 
%       \mu_t = \mu_{t-1} + \frac{1}{t}\left(\theta_t-\mu_{t-1}\right) 
%   \]    
%
%   \[ 
%       \Sigma_t = \Sigma_{t-1} + \mu_{t-1}\mu_{t-1}'-\mu_{t}\mu_{t}' + 
%                  \frac{1}{t}\left(\theta_t\theta_t'-\Sigma_{t-1}-\mu_{t-1}\mu_{t-1}'\right) 
%   \]
%
%   and
%
%   \[
%       \mathrm{mode}_t = \left\{
%                       \begin{array}{ll}
%                         \theta_t, & \hbox{if } p(\theta_t|\mathcal Y) > p(\mathrm{mode}_{t-1}|\mathcal Y) \\
%                         \mathrm{mode}_{t-1}, & \hbox{otherwise.}
%                       \end{array}
%                     \right. 
%   \]
%
%   where $t$ is the iteration, $\mu_t$ the estimate of the posterior mean
%   after $t$ iterations, $\Sigma_t$ the estimate of the posterior
%   covariance matrix after $t$ iterations, $\mathrm{mode}_t$ is the
%   evaluation of the posterior mode after $t$ iterations and
%   $p(\theta_t|\mathcal Y)$ is the posterior density of parameters
%   (specified by the user supplied function "fun").       
%
% SPECIAL REQUIREMENTS
%   None.

% Copyright (C) 2006-2010 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This version has been prepared for YADA by Anders Warne.
%
% First version: 12-07-2011
% Last modified: 09-06-2015
%

%
% initialize the output
%
PostMod = [];
PostVar = [];
Scale = [];
PostMean = [];
OptTime = [];
gh = [];
fcount = 0;
fdiff = NaN;
abort = '';
OptResults = [];
phiEstimates = xparam1;
if RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
NumAccepted = 0;
%
%global bayestopt_ estim_params_ options_

%options_.lik_algo = 1;
npar = length(xparam1);

NumberOfIterations = num;
%MaxNumberOfTuningSimulations   = 200000;
%MaxNumberOfClimbingSimulations = 200000;
%AcceptanceTarget               = 1/3;
MaxNumberOfTuningSimulations   = min(200000,num*10);
MaxNumberOfClimbingSimulations = min(200000,num*10);
AcceptanceTarget               = 1/4;

CovJump = VarCov;
ModePar = xparam1;

%% [1] I tune the scale parameter.
%if exist('OCTAVE_VERSION') || options_.console_mode
%    diary off;
%    disp(' ');
%else
%    hh = waitbar(0,'Tuning of the scale parameter...');
%    set(hh,'Name','Tuning of the scale parameter.')
%end
jj  = 1;
isux = 0;
jsux = 0;
test = 0;
ix2 = ModePar;% initial condition!
ilogpo2 = -feval(ObjFun,ix2,varargin{:}); % initial posterior density
MaxValue = ilogpo2;
fval = -MaxValue;
fcount = 1;
mlogpo2 = ilogpo2;
try;
%    dd = transpose(chol(CovJump));
   dd = chol(CovJump)';
catch;
   dd = eye(length(CovJump));
end;
initScale = iScale;
%
% setup a waitdialog
%
if strcmp(StringTrim(waitbartext),'')==0;
   HeaderStr = waitbartext;
else;
   HeaderStr = 'Monte Carlo Optimization with gmhmaxlik';
end;
txt = ['Please wait while YADA attempts to tune the scale factor for estimation of the posterior covariance matrix. A maximum of ' int2str(MaxNumberOfTuningSimulations) ' parameter draws are used during the initial tuning stage.'];
if waitbar==1;
   WaitHandle = WaitDLG(txt,'information',HeaderStr,500,200,CurrINI,1);
else;
   WaitHandle = WaitDLG(txt,'information',HeaderStr,500,200,CurrINI,0);
end;
WaitControls = get(WaitHandle,'UserData');
%
SetLogText(strvcat(' ',['Tuning of the scale factor with max ' int2str(MaxNumberOfTuningSimulations) ' parameter draws...'],[' * Initial scale factor value:       ' num2str(iScale)]));
ScreenMessages = strvcat(['Tuning of the scale factor with max ' int2str(MaxNumberOfTuningSimulations) ' parameter draws...'],[' * Initial scale factor value:       ' num2str(iScale)]);
tic;
StopEstimation = 0;
for j=1:MaxNumberOfTuningSimulations;
   if waitbar==1;
      abort = get(WaitControls.cancel,'UserData');
      if strcmp(abort,'cancel')==1;
         StopEstimation = 1;
         break;
      end;
   end;
   proposal = iScale*dd*randn(npar,1) + ix2;
   if (all(proposal>mh_bounds(:,1))==1)&(all(proposal<mh_bounds(:,2))==1);
      logpo2 = -feval(ObjFun,proposal,varargin{:});
      fcount = fcount+1;
   else
      logpo2 = -Inf;
   end
   % I move if the proposal is enough likely...
   if (logpo2>-Inf)&(log(rand)<logpo2-ilogpo2);
      ix2 = proposal; 
      if logpo2>mlogpo2;
         dx = proposal-ModePar;
         ModePar = proposal;
         fdiff = logpo2-mlogpo2;
         MaxValue = logpo2;
         phiEstimates = [phiEstimates ModePar];
         [gh,badg] = YADAnumgrad(ObjFun,ModePar,varargin{:});
         NumAccepted = NumAccepted+1;
         ScreenMessages = strvcat(ScreenMessages,['   Acceptance number: ' int2str(NumAccepted)],['   - Norm of dx:   ' num2str(norm(dx))],['   - Scale factor: ' num2str(iScale)]);
         OptResults = [OptResults;[NumAccepted logpo2 fdiff max(abs(gh))]];
         mlogpo2 = logpo2;
      end
      ilogpo2 = logpo2;
      isux = isux+1;
      jsux = jsux+1;
   end; % ... otherwise I don't move.
%   prtfrc = j/MaxNumberOfTuningSimulations;
%    if exist('OCTAVE_VERSION') || options_.console_mode
%        if mod(j, 10)==0
%            if exist('OCTAVE_VERSION')
%                printf('Tuning of the scale parameter (%f%% done):: Acceptance rates: %f [%f]\r',prtfrc*100,isux/j,jsux/jj);
%            else
%                fprintf('   Tuning of the scale parameter (%f \b%% done):: Acceptance rates: %f [%f]\r',prtfrc*100,isux/j,jsux/jj);
%            end
%        end
%    else
%        waitbar(prtfrc,hh,sprintf('Acceptance rates: %f [%f]',isux/j,jsux/jj));
%    end
   if  (j/500)==round(j/500);
      test1 = jsux/jj;
      cfactor = test1/AcceptanceTarget;
      if cfactor>0;
         iScale = iScale*cfactor;
      else;
         iScale = iScale/10;
      end;
      jsux = 0;
      jj = 0;
      if (cfactor>0.90)&(cfactor<1.10);
         test = test+1;
      end;
      if test>4;
         break;
      end;
   end;
   jj = jj+1;
   drawnow;
end;
%
% collect some info
%
TuneTime = toc;
OptTime = TuneTime;
if StopEstimation==1;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
   return;
end;
%
CurrTarget = (isux/j);
%iScale = (AcceptanceTarget/CurrTarget)*iScale;
%if exist('OCTAVE_VERSION') || options_.console_mode
%    diary on;
%else
%    close(hh);
%end
%% [2] One block metropolis, I update the covariance matrix of the jumping distribution
%if exist('OCTAVE_VERSION') || options_.console_mode
%    diary off;
%else
%    hh = waitbar(0,'Metropolis-Hastings...');
%    set(hh,'Name','Estimation of the posterior covariance...')
%end
isux = 0;
ilogpo2 = -feval(ObjFun,ix2,varargin{:});
fcount = fcount+1;
SetLogText(strvcat([' * Tuned scale factor value:         ' num2str(iScale)],[' * Tuning time:                      ' SecondsToHoursStr(TuneTime) ' (hrs:min:sec)'],[' * Acceptance rate during tuning:    ' num2str(CurrTarget,'%0.4f')],[' * Targeted acceptance rate:         ' num2str(AcceptanceTarget,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')],' ','Estimation of the posterior covariance...'));
ScreenMessages = strvcat(ScreenMessages,[' * Tuned scale factor value:         ' num2str(iScale)],[' * Tuning time:                      ' SecondsToHoursStr(TuneTime) ' (hrs:min:sec)'],[' * Acceptance rate during tuning:    ' num2str(CurrTarget,'%0.4f')],[' * Targeted acceptance rate:         ' num2str(AcceptanceTarget,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')],' ','Estimation of the posterior covariance...');
%
% setup a progress dialog if asked to
%
if waitbar==1;
   %
   % close the wait dialog first
   %
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
   ProgressStructure.title = ['Progress for ' int2str(NumberOfIterations) ' Parameter Draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = ShowClock;
   ProgressStructure.label = 'Objective Function Change:';
   %
   if strcmp(StringTrim(waitbartext),'')==0;
      ProgressStructure.name = waitbartext;
   else;
      ProgressStructure.name = 'Monte Carlo Optimization with gmhmaxlik';
   end;
   ProgressStructure.CurrINI = CurrINI;
   mywait = ProgressDLG(0,ProgressStructure);
   set(mywait,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   %
   % show a wait dialog
   %
   txt = ['Please wait while YADA attempts to estimate the posterior mode with gmhmaxlik. A total of ' int2str(NumberOfIterations) ' parameter draws are used during the estimation of the posterior covariance matrix.'];
   set(WaitControls.text,'String',txt);
end;
%
tic;
for j=1:NumberOfIterations;
   if waitbar==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         if j<=NumberOfIterations;
            ProgressDLG([j/NumberOfIterations fdiff]);
         end;
      end;
   end;
   proposal = iScale*dd*randn(npar,1) + ix2;
   if (all(proposal>mh_bounds(:,1))==1)&(all(proposal<mh_bounds(:,2))==1);
      logpo2 = -feval(ObjFun,proposal,varargin{:});
      fcount = fcount+1;
   else;
      logpo2 = -Inf;
   end;
   % I move if the proposal is enough likely...
   if (logpo2>-Inf)&(log(rand)<logpo2-ilogpo2);
      ix2 = proposal;
      if logpo2>mlogpo2;
         dx = proposal-ModePar;
         ModePar = proposal;
         fdiff = logpo2-mlogpo2;
         MaxValue = logpo2;
         phiEstimates = [phiEstimates ModePar];
         [gh,badg] = YADAnumgrad(ObjFun,ModePar,varargin{:});
         NumAccepted = NumAccepted+1;
         ScreenMessages = strvcat(ScreenMessages,['   Acceptance number: ' int2str(NumAccepted)],['   - Norm of dx:   ' num2str(norm(dx))]);
         OptResults = [OptResults;[NumAccepted logpo2 fdiff max(abs(gh))]];
         mlogpo2 = logpo2;
      end
      ilogpo2 = logpo2;
      isux = isux+1;
      jsux = jsux+1;
   end % ... otherwise I don't move.    
%   prtfrc = j/NumberOfIterations;
%    if exist('OCTAVE_VERSION') || options_.console_mode
%        if mod(j, 10)==0
%            if exist('OCTAVE_VERSION')
%                printf('Estimation of the posterior covariance (%f%% done):: Acceptance rates: %f\r',prtfrc,isux/j);
%            else
%                fprintf('   Estimation of the posterior covariance (%f \b%% done):: Acceptance rates: %f     \r',prtfrc*100,isux/j);
%            end
%        end
%    else
%        waitbar(prtfrc,hh,sprintf('Acceptance rate: %f',isux/j));
%    end
    % I update the covariance matrix and the mean:
   oldMeanPar = MeanPar;
   MeanPar = oldMeanPar + (1/j)*(ix2-oldMeanPar);
   CovJump = CovJump + oldMeanPar*oldMeanPar' - MeanPar*MeanPar' + ...
             (1/j)*(ix2*ix2' - CovJump - oldMeanPar*oldMeanPar');
   drawnow;
end;
EstTime = toc;
OptTime = OptTime+EstTime;
%
% make sure we close any dialogs
%
if waitbar==1;
   if IsHandle(mywait)==1;
      set(mywait,'UserData','done');
      close(mywait);
      drawnow;
      pause(0.02);
   end;
   if strcmp(lower(abort),'cancel')==1;
      return;
   end;
   %
   % startup a wait dialog again
   %
   txt = ['Please wait while YADA attempts to retune the scale factor before making the attempt to climb to the summit. A maximum of ' int2str(MaxNumberOfTuningSimulations) ' parameter draws are used during the retuning stage.'];
   WaitHandle = WaitDLG(txt,'information',HeaderStr,500,200,CurrINI,1);
   WaitControls = get(WaitHandle,'UserData');
else;
   txt = ['Please wait while YADA attempts to retune the scale factor before making the attempt to climb to the summit. A maximum of ' int2str(MaxNumberOfTuningSimulations) ' parameter draws are used during the retuning stage.'];
   set(WaitControls.text,'String',txt)
end;
%if exist('OCTAVE_VERSION') || options_.console_mode
%    diary on;
%else
%    close(hh);
%end
PostVar = CovJump;
PostMean = MeanPar;
%% [3 & 4] I tune the scale parameter (with the new covariance matrix) if
%% this is the last call to the routine, and I climb the hill (without
%% updating the covariance matrix)...
%if strcmpi(info,'LastCall')
%    if exist('OCTAVE_VERSION') || options_.console_mode
%        diary off;
%    else
%        hh = waitbar(0,'Tuning of the scale parameter...');
%        set(hh,'Name','Tuning of the scale parameter.')
%    end
ilogpo2 = -feval(ObjFun,ix2,varargin{:}); % initial posterior density
fcount = fcount+1;
SetLogText(strvcat([' * Estimation time:                  ' SecondsToHoursStr(EstTime) ' (hrs:min:sec)'],[' * Acceptance rate:                  ' num2str(isux/NumberOfIterations,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')],' ',['Retuning the scale factor with max ' int2str(MaxNumberOfTuningSimulations) ' parameter draws...']));
ScreenMessages = strvcat(ScreenMessages,[' * Estimation time:                  ' SecondsToHoursStr(EstTime) ' (hrs:min:sec)'],[' * Acceptance rate:                  ' num2str(isux/NumberOfIterations,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')],' ',['Retuning the scale factor with max ' int2str(MaxNumberOfTuningSimulations) ' parameter draws...']);
%
jj  = 1;
isux = 0;
jsux = 0;
test = 0;
dd = chol(CovJump)';
tic;
for j=1:MaxNumberOfTuningSimulations;
   if waitbar==1;
      abort = get(WaitControls.cancel,'UserData');
      if strcmp(abort,'cancel')==1;
         StopEstimation = 1;
         break;
      end;
   end;
   proposal = iScale*dd*randn(npar,1) + ix2;
   if (all(proposal>mh_bounds(:,1))==1)&(all(proposal<mh_bounds(:,2))==1);
      logpo2 = -feval(ObjFun,proposal,varargin{:});
      fcount = fcount+1;
   else
      logpo2 = -Inf;
   end
   % I move if the proposal is enough likely...
   if (logpo2>-Inf)&(log(rand)<logpo2-ilogpo2);
      ix2 = proposal;
      if logpo2>mlogpo2;
         dx = proposal-ModePar;
         ModePar = proposal;
         fdiff = logpo2-mlogpo2;
         phiEstimates = [phiEstimates ModePar];
         MaxValue = logpo2;
         [gh,badg] = YADAnumgrad(ObjFun,ModePar,varargin{:});
         NumAccepted = NumAccepted+1;
         ScreenMessages = strvcat(ScreenMessages,['   Acceptance number: ' int2str(NumAccepted)],['   - Norm of dx:   ' num2str(norm(dx))],['   - Scale factor: ' num2str(iScale)]);
         OptResults = [OptResults;[NumAccepted logpo2 fdiff max(abs(gh))]];
         mlogpo2 = logpo2;
      end
      ilogpo2 = logpo2;
      isux = isux+1;
      jsux = jsux+1;
   end; % ... otherwise I don't move.
%     prtfrc = j/MaxNumberOfTuningSimulations;
%        if exist('OCTAVE_VERSION') || options_.console_mode
%            if mod(j, 10)==0
%                if exist('OCTAVE_VERSION')
%                    printf('Tuning of the scale parameter (%f%% done):: Acceptance rates: %f [%f]\r',prtfrc*100,isux/j,jsux/jj);
%                else
%                    fprintf('   Tuning of the scale parameter (%f \b%% done):: Acceptance rates: %f [%f]            \r',prtfrc*100,isux/j,jsux/jj);
%                end
%            end
%        else
%            waitbar(prtfrc,hh,sprintf('Acceptance rates: %f [%f]',isux/j,jsux/jj));
%        end
   if (j/1000)==round(j/1000);
      test1 = jsux/jj;  
      cfactor = test1/AcceptanceTarget;
      iScale = iScale*cfactor;
      jsux = 0;
      jj = 0;
      if (cfactor>0.90)&(cfactor<1.10);
         test = test+1;
      end;
      if test>4;
         break;
      end;
   end;
   jj = jj+1;
   drawnow;
end;
TuneTime = toc;
OptTime = OptTime+TuneTime;
%
% check if user selected to stop further computations
%
if StopEstimation==1;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
   return;
end;
%
ilogpo2 = -feval(ObjFun,ModePar,varargin{:}); 
CurrTarget = (isux/j);
%iScale = (AcceptanceTarget/CurrTarget)*iScale;
%
SetLogText(strvcat([' * Tuned scale factor value:         ' num2str(iScale)],[' * Tuning time:                      ' SecondsToHoursStr(TuneTime) ' (hrs:min:sec)'],[' * Acceptance rate during retuning:  ' num2str(CurrTarget,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')],' ',['Attempting to climb the summit with max ' int2str(MaxNumberOfClimbingSimulations) ' parameter draws...']));
ScreenMessages = strvcat(ScreenMessages,[' * Tuned scale factor value:         ' num2str(iScale)],[' * Tuning time:                      ' SecondsToHoursStr(TuneTime) ' (hrs:min:sec)'],[' * Acceptance rate during retuning:  ' num2str(CurrTarget,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')],' ',['Attempting to climb the summit with max ' int2str(MaxNumberOfClimbingSimulations) ' parameter draws...']);
%
txt = ['Please wait while YADA attempts to climb to the summit of the posterior density. A maximum of ' int2str(MaxNumberOfClimbingSimulations) ' parameter draws are used during this final stage.'];
set(WaitControls.text,'String',txt)
%
%    if exist('OCTAVE_VERSION') || options_.console_mode
%        diary on;
%    else
%        close(hh);
%    end
Scale = iScale;
    %%
    %% Now I climb the hill
    %%
%    climb = 1;
%    if climb
%        if exist('OCTAVE_VERSION') || options_.console_mode
%            diary off;
%        else
%            hh = waitbar(0,' ');
%            set(hh,'Name','Now I am climbing the hill...')
%        end
jj  = 1;
jsux = 0;
isux = 0;
test = 0;
tic;
for j=1:MaxNumberOfClimbingSimulations;
   if waitbar==1;
      abort = get(WaitControls.cancel,'UserData');
      if strcmp(abort,'cancel')==1;
         StopEstimation = 1;
         break;
      end;
   end;
   proposal = iScale*dd*randn(npar,1) + ModePar;
   if (all(proposal>mh_bounds(:,1))==1)&(all(proposal<mh_bounds(:,2))==1);
      logpo2 = -feval(ObjFun,proposal,varargin{:});
      fcount = fcount+1;
   else
      logpo2 = -Inf;
   end
   if logpo2>mlogpo2; % I move if the proposal is higher...
      dx = proposal-ModePar;
      ModePar = proposal;
      fdiff = logpo2-mlogpo2;
      phiEstimates = [phiEstimates ModePar];
      MaxValue = logpo2;
      [gh,badg] = YADAnumgrad(ObjFun,ModePar,varargin{:});
      NumAccepted = NumAccepted+1;
      ScreenMessages = strvcat(ScreenMessages,['   Acceptance number: ' int2str(NumAccepted)],['   - Norm of dx:   ' num2str(norm(dx))],['   - Scale factor: ' num2str(iScale)]);
      OptResults = [OptResults;[NumAccepted logpo2 fdiff max(abs(gh))]];
      mlogpo2 = logpo2;
      jsux = jsux+1;
      isux = isux+1;
   end; % otherwise I don't move...
%            prtfrc = j/MaxNumberOfClimbingSimulations;
%            if exist('OCTAVE_VERSION') || options_.console_mode
%                if mod(j, 10)==0
%                    if exist('OCTAVE_VERSION')
%                        printf('Now I am climbing the hill (%f%% done):: %f Jumps / MaxStepSize %f\r',prtfrc*100,jsux,sqrt(max(diag(iScale*CovJump))));
%                    else
%                        fprintf('   Now I am climbing the hill (%f \b%% done):: %f Jumps / MaxStepSize %f                   \r',prtfrc*100,jsux,sqrt(max(diag(iScale*CovJump))));
%                    end
%                end
%            else
%                waitbar(prtfrc,hh,sprintf('%f Jumps / MaxStepSize %f',jsux,sqrt(max(diag(iScale*CovJump)))));
%            end
   if (j/200)==round(j/200);
      if jsux<=1;
         test = test+1;
      else;
         test = 0;
      end;
      jsux = 0;
      jj = 0;
      if test>4; % If I do not progress enough I reduce the scale parameter
                   % of the jumping distribution (cooling down the system).
         iScale = iScale/1.10;
      end;
      if sqrt(max(diag(iScale*CovJump)))<10^(-4);
          break; % Steps are too small!
      end
  end
  jj = jj+1;
  drawnow;
end
EstTime = toc;
OptTime = OptTime+EstTime;
%
% close the wait dialog
%
if IsHandle(WaitHandle)==1;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
SetLogText(strvcat([' * Summit climbing time:             ' SecondsToHoursStr(EstTime) ' (hrs:min:sec)'],[' * Acceptance rate:                  ' num2str(isux/j,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')]));
ScreenMessages = strvcat(ScreenMessages,[' * Summit climbing time:             ' SecondsToHoursStr(EstTime) ' (hrs:min:sec)'],[' * Acceptance rate:                  ' num2str(isux/j,'%0.4f')],[' * Value of the objective function:  ' num2str(MaxValue,'%0.6f')]);
%        if exist('OCTAVE_VERSION') || options_.console_mode
%            diary on;
%        else
%            close(hh);
%        end
%    end %climb
%else
%    Scale = iScale;
%end
PostMod = ModePar;
fval = -MaxValue;

%
% end of YADAgmhmaxlik.m
%
