function PosteriorModeEstimation(DSGEModel,CurrINI,controls)
% PosteriorModeEstimation: Runs the posterior mode estimation procedures.
%
% USAGE:
%
%       PosteriorModeEstimation(DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure), with initialization information.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: None currently.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 10, 2006
%                        This version: August 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-08-2006: Added the check optimum features.
%
% * 14-08-2006: Added some initial features regarding additional and calibrated parameters.
%
% * 15-08-2006: Added reading the sub-sample setting and the "CurrINI.GridWidth" option (when checking the optimum)
%
% * 16-08-2006: Removed an unncessary file exist check before copying the measurement equation file to the tmp directory.
%
% * 17-08-2006: Added the two new inputs for the measurement equation file with string matrices containing the names
%               of the observed variables and of the exogenous variables.
%
% * 18-08-2006: Changed the StrTrim function to StringStrim.
%
% * 21-08-2006: Added automatic saving of graphs to "GraphDirectory".
%
% * 22-08-2006: Added calculations of the largest eigenvalue of F, the state transition matrix, at the mode and at the
%               initial values.
%
% * 23-08-2006: Added collection of results from the optimization routines through the iterations. Forecast errors are
%               now also computed the observed variables.
%
% * 24-08-2006: Removed the output "InitialValues" from the call to "VerifyPriorData". Furthermore, an unnecessary call
%               to the measurement equation function was removed.
%
% * 25-08-2006: Added changes required to handle a uniform prior.
%
% * 28-08-2006: Added the input CloseSelf=1 to the TextGUI call. This allows the latter function to avoid using the waitfor
%               command.
%
% * 14-09-2006: Added the new inputs for the measurement equation call.
%
% * 15-09-2006: Checking the ModelParameters structure. No changes made!
%
% * 21-09-2006: Checked the PriorDist.normal matrix. No changes made! Bug fixed in VerifyPriorData function.
%
% * 22-09-2006: The AiMSolver call is now run twice to figure out the solution time.
%
% * 16-10-2006: Collected information about the initialized parameters names in order to separate them from the
%               additional parameters that are updated.
%
% * 17-10-2006: Added a try, catch call when attempting to load mat-files. This is needed if matlab can't
%               read the mat-file, e.g., because it was create with a later version of matlab.
%
% * 20-10-2006: Added the call to the summary output from the estimation.
%
% * 25-10-2006: The field KalmanFirstObservation to the DSGEModel structure now uses the round function instead of the
%               ceil function.
%
% * 26-10-2006: Added the DSGEModel fields "FirstPeriod" and "LastPeriod" to the data stored on disk from the posterior
%               mode estimation.
%
% * 09-11-2006: Added information sent to output window about the estimation sample. The function also checks the gradient
%               of the log posterior at the initial values to capture potential bad gradients.
%
% * 10-11-2006: The value of the log prior and the log Jacobian at the initial value are now also sent to the output window.
%
% * 20-11-2006: The initial value for the state vector is now user defined through the variable DSGEModel.InitialStateVector.
%
% * 22-11-2006: Additional output from failed optimizations are now written to file.
%
% * 24-11-2006: Took the name changes "logPosteriorPhiDSGEOptimize" to "logPosteriorPhiDSGE", and "logPosteriorThetaDSGEOptimize"
%               to "logPosteriorThetaDSGE" into account. The "phiEstimates" matrix is now obtained from the optimizer and stored
%               in the output file.
%
% * 06-12-2006: Made sure that the "CalculateDSGEStateVariables" function receives the new input variable.
%
% * 07-12-2006: Took the changes is fields for CurrINI and DSGEModel into account, where CurrINI.MaximumIterationsValue
%               is now given by DSGEModel.OptMaxIterationsValue, CurrINI.ToleranceValue is now DSGEModel.OptToleranceValue.
%               Moreover, CurrINI.MaximizeAlgorithmValue is now DSGEModel.MaximizeAlgorithmValue; CurrINI.StepLengthHessian is
%               DSGEModel.StepLengthHessian; CurrINI.NumberOfGridPoints is DSGEModel.NumberOfGridPoints;
%               CurrINI.FiniteDifferenceHessian is DSGEModel.FiniteDifferenceHessian; CurrINI.CheckOptimum is
%               DSGEModel.CheckOptimum; CurrINI.CheckTransformedOptimum is DSGEModel.CheckTransformedOptimum; CurrINI.GridWidth
%               is DSGEModel.GridWidth.
%
% * 20-02-2007: The matrix with partial derivatives "ThetaPhiPartial", i.e., d Theta/ d phi', is now saved to the posterior
%               mode results file.
%
% * 09-03-2007: Made sure that "CurrINI" is sent to the "CalculateDSGEMaximumEigenvalue" function.
%
% * 20-04-2007: Added the input "phiFittedStd" for the call to the "PrintPosteriorModeSummary" function.
%
% * 12-11-2007: Updated the documentation.
%
% * 11-12-2007: Introduced the variable "NumParam" that replaced calls to the length function for theta, phi,
%               thetaMode and phiMode.
%
% * 21-12-2007: Specified unit-root state processes can now be handled.
%
% * 02-01-2008: Removed code using "DSGEModel.InitializeKalman".
%
% * 07-01-2007: Added drawnow calls after Query and About calls.
%
% * 16-05-2008: The function now attempts to collect data from AiM output from the tmp directory for error
%               message reporting. Took the "DSGEModel.AIMTolerance" input for AiMSolver into account.
%
% * 19-05-2008: Made sure that the "AiMErrorData.mat" file is cleaned from the tmp directory after estimation
%               has finished.
%
% * 20-05-2008: The gradient vector is now sent to the "PrintOptimizationErrorReport" function.
%
% * 23-05-2008: Updated the documentation.
%
% * 27-05-2008: The mcode value 7 is now supported.
%
% * 03-07-2008: Made sure that the measurement equation receives only the active state variable names.
%               Added try-catch code for AiMSolver.
%
% * 22-07-2008: Took the lower bound for the gamma prior into account.
%
% * 23-07-2008: Took the change from 0-1 lower and upper bounds to general bounds for the beta distribution
%               into account.
%
% * 23-10-2008: The method for computing the total number of parameters have been changed.
%
% * 21-01-2009: Updated the documentation.
%
% * 30-01-2009: Added code for dealing with user choice of method to initialize the inverse Hessian
%               when using the csminwel optimization routine.
%
% * 09-02-2009: Changed the input "length(DSGEModel.StateVariablePositions)" to "size(F,1)" when calling the
%               measurement equation function. This required that the call to the measurement equation function
%               was moved to after trying the solve the DSGE model.
%
% * 13-02-2009: The mcode values 7 and 8 are now properly mentioned when encountered.
%
% * 09-03-2009: Added quotation marks around paths for the dos del and copy commands.
%
% * 13-03-2009: The function calls "MomentToParamLogisticPDF" to retrieve the location and scale
%               parameters of the logistic distribution from the mean and standard deviation.
%
% * 20-03-2009: The call to the function "PriorHessianMatrix" now accepts the boolean input variable
%               "DSGEModel.RandomNumberValue".
%
% * 02-04-2009: Took into account that the function "MomentToParamLogisticPDF" now also supports
%               a shape parameter. Added the call to "MomentToParamGumbelPDF".
%
% * 07-04-2009: Changed the maximum height of the "Error - " message boxes for problem files.
%
% * 30-04-2009: The code now checks the Kalman filter status if the log posterior value is 1000000.
%               The outcome of this test is shown both in the estimation log control and on the
%               About dialog.
%
% * 18-05-2009: Added the values of the input variables "RecEst" and "HeaderStr" for "CalculateDSGEStateVariables".
%
% * 16-09-2009: Fixed a bug where the function wouldn't run the Markov chain when "AllowUnitRoot" is
%               unity and the model has unit roots without specifying where the roots are located.
%
% * 24-09-2009: The code now checks if the inverse Hessian estimator at the posterior mode is positive
%               definite.
%
% * 25-09-2009: The code now asks if the user wants to have a new inverse Hessian when the one obtained from
%               the optimization routine is not positive definite. The new inverse Hessian is positive
%               definite as long as the old inverse Hessian does not have any zero eigenvalues. The new
%               inverse Hessian is computed by taking an eigenvalue decomposition of the old and letting
%               the eigenvalues be replaced with the absolute values of the eigenvalues. Moreover, the
%               code keeps the diagonal of the old inverse Hessian if it has only positive elements by
%               renormalizing the new matrix The function "CreatePositiveDefinite"is used.
%
% * 05-01-2010: Updated the documentation.
%
% * 15-10-2010: The eigenvalues and eigenvectors of the inverse Hessian at the posterior mode is now investigated.
%               If these are too small, then the resulting eigenvectors and eigenvalues are reported in the
%               posterior mode results file.
%
% * 21-01-2010: The function "PrintOptimizationErrorReport" now accepts the input variable "TxtFileName".
%               The function "VillaniCheckOptimum" now accepts the "ModelString" variable.
%
% * 26-01-2010: Took into account that "VillaniCheckOptimum" now accepts the "MargStr" variable.
%
% * 05-02-2010: The function can now save more data to disk when the optimization routine has failed
%               to generate a mode. Took into account that "YADAcsminwel" now outputs the string matrix
%               "ScreenMessages". The function also takes into account that "VillaniCheckOptimum" now
%               accepts the "OtherParameters" input variable.
%
% * 07-02-2010: The function "PrintPosteriorModeResults" now accepts the input variable "ScreenMessages"
%               from csminwel.
%
% * 17-02-2010: In case the inverse Hessian from the optimization algorithm is not positive definite,
%               the function now computes a finite difference based inverse Hessian automatically.
%
% * 24-02-2010: The function now always writes an optimization error summary when the inverse Hessian is
%               not positive definite and a positive definite matrix can't be created.
%
% * 08-03-2010: Made sure that a value for the lambda hyperparameter is provided to the function
%               "PrintOptimizationErrorReport".
%
% * 15-10-2010: The csminwel optimization can now be rerun with the latest parameter values provided
%               that the original run converged according to csminwel's criterion and the maximum value
%               of the gradient is larger than the tolerance level.
%
% * 19-10-2010: Changed the additional runs so that (i) the covergence criterion is based on the change
%               if the log posterior from one csminwel run to another, (ii) the user can choose a different
%               tolerance level for this test, and (iii) the user can abort.
%
% * 08-12-2010: The AiM solution time "SolveTime" is now computed until the state-space form has been
%               obtained.
%
% * 14-12-2010: Took the "CsminwelExtraRuns" variable into account.
%
% * 16-12-2010: Took into account that "logLikelihoodDSGE", "logPosteriorPhiDSGE" and "logPosteriorThetaDSGE"
%               now take the input variable "OrderQZ" from the CurrINI structure. The function now supports the
%               Klein and Sims DSGE model solvers.
%
% * 23-12-2010: The function now supports the square root filter for computing the log-likelihood.
%
% * 05-01-2011: Updated the documentation.
%
% * 09-01-2011: Changed the dos delete commands to delete commands.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-01-2011: Made sure that "ModeFile" suppots unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 24-06-2011: Added try-catch code around the call to csminwel.
%
% * 29-06-2011: Simplified the code with return statements.
%
% * 05-07-2011: Started work to make the function compatible with optimization of the original parameters.
%
% * 06-07-2011: Made sure that the log Jacobian is added to the log posterior when the latter takes
%               the value 1000000.
%
% * 07-07-2011: Started work on integrating newrat into YADA.
%
% * 08-07-2011: Continued integrating newrat.
%
% * 11-07-2011: Finished work on integrating newrat.
%
% * 12-07-2011: Started integrating gmhmaxlik into YADA.
%
% * 13-07-2011: Continued work on integrating gmhmaxlik.
%
% * 14-07-2011: Fixed a bug when calling YADAnewrat instead of YADAgmhmaxlik for the original parameters
%               when Dynare's gmhmaxlik is the selected optimization algorithm.
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 13-10-2011: Took into account that "KalmanAlgorithm" now supports the univariate filter and smoother.
%
% * 09-11-2011: Took into account that the Kalman filter now supports diffuse initialization.
%
% * 07-12-2011: The function now takes NaN's into account when computing mean 1-step ahead forecast errors
%               and the squares of these.
%
% * 05-01-2012: Updated the documentation.
%
% * 22-03-2012: The Laplace approximation of the log marginal likelihood is now computed using the finite
%               difference estimator of the inverse Hessian.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 16-11-2012: The function takes the use of the Chandrasekhar recursions into account.
%
% * 03-01-2013: Updated the documentation.
%
% * 10-04-2013: Fixed a bug where the newrat routine could provide an infinite value for the objective function
%               while this issue wasn't reported as an optimzation error by YADA.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 02-12-2014: Began adapting the function for sequential estimation.
%
% * 03-12-2014: Took the data in "CurrINI.Sequence.QueryAnswers" into account. Took into account that
%               "VillaniCheckOptimum" takes the boolean input variable "ShowFigs".
%
% * 14-01-2015: Updated the documentation.
%
% * 06-02-2015: Fixed a bug where "VillaniCheckOptimum" wasn't taking "ShowFigs into account when analysing
%               the mode for the original parameters, i.e., when using the function "logPosteriorThetaDSGE".
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 07-05-2015: Added information of the Laplace approximation at phi mode to the log text output.
%
% * 19-06-2015: Fixed a bug when retrieving an element from OptResults after running YADAgmhmaxlik.
%
% * 02-09-2015: Fixed the information displayed when the kalmanstatus is -1.
%
% * 27-10-2015: Added query to check if user wants to run csminwel after gmhmaxlik.
%
% * 30-10-2015: Checked for a bug when checking if csminwel improves on the gmhmaxlik result.
%
% * 02-11-2015: Added checks for the inverse Hessian being positive definite before running csminwel after gmhmaxlik.
%
% * 03-11-2015: Added "OptResults" to the data saved in the posterior mode mat-file.
%
% * 22-02-2016: Updated the documentation.
%
% * 17-10-2016: Fixed a typo.
%
% * 22-03-2017: Updated the documentation.
%
% * 06-04-2017: Made it possible to skip the finite difference calculation of the inverse Hessian
%               and thereby the calculation of the Laplace approximation of the marginal likelihood.
%
% * 23-05-2017: Took into account that VillaniCheckOptimum now requires an additional input variable.
%
% * 01-06-2017: Began work on adding the system prior to the calculations.
%
% * 06-06-2017: Took into account that "logLikelihoodDSGE" now provides the "Solution" output variable.
%
% * 13-06-2017: Made sure that the marginal likelihood with the Laplace approximation always takes the
%               system prior into account.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 23-06-2017: The PrintPosteriorModeSummary function now receives the maximum of the absolute
%               Gradient vector.
%
% * 10-08-2017: Made sure that ModelParameters is updated after the posterior mode has been computed.
%
% * 29-08-2017: Took into account that "logPosteriorPhiDSGE", "logPosteriorTheta", and "logLikelihoodDSGE"
%               require the input variable "NumWorkers", which is here required to be 1.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

EstimationStatus = 0;
ScreenMessages = '';
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% set ShowFigs
%
if isempty(CurrINI.Sequence)==1;
   ShowFigs = 1;
else;
   ShowFigs = 0;
end;
%
% Check the file with prior information
%
SetLogText(strvcat(' ','Initializing posterior mode estimation...',[' * Checking "' GetFilename(DSGEModel.PriorFile) '" with data on prior...']));
%
[status,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI);
if (status==1)&(isempty(theta)==1);
   SetLogText(strvcat('   STATUS: No parameters to estimate!',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = 'All parameters of your model have status calibrated. Hence, YADA doesn''t have any parameters to estimate.';
      About(txt,'information','Calibrated DSGE Model',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif (status~=1)|(isempty(theta)==1);
   SetLogText(strvcat('   STATUS: Bad prior file or inadequate data.',' '));
   return;
end;
%
OriginalModelParameters = ModelParameters;
%
% Transform PriorDist.gamma and PriorDist.beta to the relevant parameters that are used by the logGammaPDF and
% the logBetaPDF functions
%
if isempty(PriorDist.beta)==0;
   [a,b] = MomentToParamStdbetaPDF((PriorDist.beta(:,1)-PriorDist.beta(:,3))./(PriorDist.beta(:,4)-PriorDist.beta(:,3)),PriorDist.beta(:,2)./(PriorDist.beta(:,4)-PriorDist.beta(:,3)));
   PriorDist.beta_ab = [a b];
else;
   PriorDist.beta_ab = [];
end;
%
if isempty(PriorDist.gamma)==0;
   [a,b] = MomentToParamGammaPDF(PriorDist.gamma(:,1)-PriorDist.gamma(:,3),PriorDist.gamma(:,2));
   PriorDist.gamma_ab = [a b];
else;
   PriorDist.gamma_ab = [];
end;
if isempty(PriorDist.logistic)==0;
   [a,b] = MomentToParamLogisticPDF(PriorDist.logistic(:,1),PriorDist.logistic(:,2),PriorDist.logistic(:,3));
   PriorDist.logistic_ab = [a b PriorDist.logistic(:,3)];
else;
   PriorDist.logistic_ab = [];
end;
if isempty(PriorDist.gumbel)==0;
   [a,b] = MomentToParamGumbelPDF(PriorDist.gumbel(:,1),PriorDist.gumbel(:,2));
   PriorDist.gumbel_ab = [a b];
else;
   PriorDist.gumbel_ab = [];
end;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   IsSystemPrior = 1;
else;
   IsSystemPrior = 0;
end;
ErrorStr = '';
ErrorFile = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.AIMDataFile;
end;
%
% The structure AIMData contains fields such as neq, nlead and nlag
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if DSGEModel.RunInitializeFirst==1;
      %
      % check if the file with additional parameters that only needs to be updated is OK
      %
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
      %
      % we need to run the file DSGEModel.InitializeParameterFile first
      %
      try;
         firstInitializePos = length(fieldnames(ModelParameters))+1;
         eval(['ModelParameters = ' GetName(DSGEModel.InitializeParameterFile) '(ModelParameters);']);
         lastInitializePos = length(fieldnames(ModelParameters));
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' lasterr];
         ErrorFile = DSGEModel.InitializeParameterFile;
      end;
      if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ErrorStr)==1);
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' lasterr];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
   else;
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' lasterr];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
      if (FileExist(DSGEModel.InitializeParameterFile)==1)&(isempty(ErrorStr)==1);
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
         try;
            firstInitializePos = length(fieldnames(ModelParameters))+1;
            eval(['ModelParameters = ' GetName(DSGEModel.InitializeParameterFile) '(ModelParameters);']);
            lastInitializePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' lasterr];
            ErrorFile = DSGEModel.InitializeParameterFile;
         end;
      end;
   end;
end;
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% we can go on with checking if the initial values permit a unique solution of the DSGE model
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent','   solution for initial parameter values...'));
%
% we can now run the DSGE model solver:
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'AiM';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
end;
%
% To measure the solution time more accurately we call it twice, with the first call only being
% performed for checking if the model has a unique convergent solution at the initial values.
%
try;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   end;
catch;
   if DSGEModel.ModelSolver==1;
      ErrorStr = ['There was an error running AiM. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==2;
      ErrorStr = ['There was an error running Klein''s solver. YADA received the following message: ' lasterr];
   elseif DSGEModel.ModelSolver==3;
      ErrorStr = ['There was an error running Sims'' gensys solver. YADA received the following message: ' lasterr];
   end;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error','AiM Error',CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   else;
      SetLogText(ErrorStr);
   end;
   return;
end;
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% check for explosive or unsupported unit roots
%
if isempty(DSGEModel.UnitRootStates)==1;
   F1 = F;
else;
   KeepPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   F1 = F(KeepPos,KeepPos);
end;
%
if ~((max(abs(eig(F1)))<1)&(IsAlmostEqual(max(abs(eig(F1))),1)==0))&~((isempty(DSGEModel.UnitRootStates)==1)&(max(abs(eig(F1)))<=1));
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F1)))) '. The state equation is therefore non-stationary. You need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check how long it takes to solve the model
%
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
end;
SolveTime = toc;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% check the measurement equation file for internal validity
%
SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:      ' SolverStr],['   Maximum absolute error: ' num2str(MaxAbsError)],['   Solution time:          ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
try;
   eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
catch;
   ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.MeasurementEquationFile;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% checking the system prior file if it exists
%
if IsSystemPrior==1;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior can be calculated...'));
   try;
      eval(['lnSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
   catch;
      ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' lasterr];
      ErrorFile = DSGEModel.SystemPriorFile;
      SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
         drawnow;
      end;
      return;
   end;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior mode has been calculated...'));
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
      SetLogText('   STATUS: OK');
   else;
      %
      % we need to first compute the system prior mode
      %
      [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos],AIMData,controls,DSGEModel,CurrINI);
      if SystemPriorStatus==0;
         SetLogText('Posterior mode estimation has been aborted.');
         return;
      end;
   end;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% The model has a unique solution for the initial values
%
MaxIterStr = get(controls.optimize.maximumiterations,'String');
MaxIterStr = StringTrim(strrep(MaxIterStr(DSGEModel.OptMaxIterationsValue,:),',',''));
MaxIter = str2num(MaxIterStr);
%
ToleranceStr = get(controls.optimize.tolerance,'String');
ToleranceStr = StringTrim(ToleranceStr(DSGEModel.OptToleranceValue,:));
ToleranceValue = str2num(ToleranceStr);
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
%
if DSGEModel.KalmanAlgorithm==1;
   AlgorithmText = '   Kalman algorithm:       Standard filter';
elseif DSGEModel.KalmanAlgorithm==2;
   AlgorithmText = '   Kalman algorithm:       Square root filter';
elseif DSGEModel.KalmanAlgorithm==3;
   AlgorithmText = '   Kalman algorithm:       Univariate filter';
elseif DSGEModel.KalmanAlgorithm==4;
   AlgorithmText = '   Kalman algorithm:       Chandrasekhar recursions';
end;
if DSGEModel.UseDoublingAlgorithm==1;
   InitialAlgorithmText = '   Initialization method:  Analytic';
elseif DSGEModel.UseDoublingAlgorithm==2;
   InitialAlgorithmText = '   Initialization method:  Numeric (doubling algorithm)';
elseif DSGEModel.UseDoublingAlgorithm==3;
   InitialAlgorithmText = '   Initialization method:  Constant times identity';
elseif DSGEModel.UseDoublingAlgorithm==4;
   InitialAlgorithmText = '   Initialization method:  Diffuse';
end;
SetLogText(strvcat('   STATUS: OK',' * Checking if the log posterior can be calculated...'));
%
% Collect additional parameters in ParameterNames.additional
%
ParameterNames = ReadAdditionalParameterNames(ParameterNames,ModelParameters,OriginalModelParameters,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos]);
%
% set up vectors with calibrated parameters and additional parameters. These are only used for printing to the output file
%
[thetaCalibrated,thetaAdditionalInitial] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% Set up the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood with the Kalman filter
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
NumParam = length(theta);
%
% get the upper and lower bounds of the parameters
%
ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
ParameterBounds = ParameterBounds(:,1:2);
%
% check the parameter target type
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   OptParameters = 'Transformed';
else;
   OptParameters = 'Original';
end;
%
% check if the log posterior can be computed
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
ErrorStr = '';
try;
   tic;
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      logPost = logPosteriorPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   else;
      logPost = logPosteriorThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      if logPost~=1000000;
         logPost = logPost-logJacobian(phi,thetaIndex,UniformBounds);
      end;
   end;
   LogPostTime = toc;
   LogPosteriorPhiIV = -logPost;
catch;
   ErrorStr = ['Unable to calculate the log posterior. Message caught is: ' lasterr];
   SetLogText(strvcat('   STATUS: Unable to calculate log posterior.',' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error','Error - Log Posterior',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
if ((exist('logPost')==0)|(logPost==1000000));
   %
   % check the log likelihood
   %
   drawnow;
   if (exist('logPost')==1)&(logPost==1000000);
      [logLikeValue,Solution,aimstatus,kalmanstatus] = logLikelihoodDSGE(ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
      if kalmanstatus==1;
         AddOnTxt = 'The state variables are not stationary at the initial values.';
         AddLogTxt = '           Non-stationary state variables detected.';
      elseif kalmanstatus==-1;
         AddOnTxt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
         AddLogTxt = '           Forecast covariance matrix in Kalman filter is not positive definite';
      else;
         AddOnTxt = 'You probably need to change your initial parameter values.';
         AddLogTxt = '';
      end;
   else;
      AddLogTxt = '';
      AddOnTxt = 'You probably need to change your initial parameter values.';
   end;
   %
   SetLogText(strvcat('   STATUS: Unable to compute log posterior at initial values',AddLogTxt,' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['Unable to compute the log posterior at the initial values. ' AddOnTxt];
      About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Compute a numerical gradient
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   [g,badg] = YADAnumgrad('logPosteriorPhiDSGE',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
else;
   [g,badg] = YADAnumgrad('logPosteriorThetaDSGE',theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
end;
if badg~=0;
   SetLogText(strvcat('   STATUS: Bad gradient of log posterior at initial values',' '));
   [badpar,maxpos] = min(abs(g));
   %
   if badpar>0;
      ValueStr = num2str(badpar);
   else;
      ValueStr = 'NaN';
   end;
   %
   ParamStr = ['"' StringTrim(ParameterNames.estimated(maxpos(1),:)) '"'];
   AddParamStr = ' ';
   if length(maxpos)>=2;
      AddParamStr = 's ';
      for i=2:length(maxpos);
         ParamStr = [ParamStr ', "' StringTrim(ParameterNames.estimated(maxpos(i),:)) '"'];
      end;
   end;
   %
   if isempty(CurrINI.Sequence)==1;
      txt = ['Unable to compute a meaningful numerical gradient of the log posterior at the initial values. The largest absolute value or an improper value (NaN or Inf) of the gradient is found for the parameter' AddParamStr ParamStr ', with value of ' ValueStr '. You probably need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Starting the optimization
%
if (DSGEModel.MaximizeAlgorithmValue==1)|(DSGEModel.MaximizeAlgorithmValue==2);
   %
   % We use Sims' csminwel routine with transformed parameters.
   %
   OptAlgorithm = 'csminwel';
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      eval(['LogSPIV = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   % compute the initial inverse Hessian
   %
   if DSGEModel.InitializeHessian==1;
      InitialInvHessMat = 10*eye(NumParam);
   elseif DSGEModel.InitializeHessian==2;
      InitialInvHessMat = 1*eye(NumParam);
   elseif DSGEModel.InitializeHessian==3;
      InitialInvHessMat = 10^(-1)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==4;
      InitialInvHessMat = 10^(-2)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==5;
      InitialInvHessMat = 10^(-3)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==6;
      InitialInvHessMat = 10^(-4)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==7;
      InitialInvHessMat = 10^(-5)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==8;
      %
      % sample from the marginal prior
      %
      InitialInvHessMat = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
   end;
   %
   % Deelete old error file if present
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogPostTime) ' seconds'],AlgorithmText,InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   try;
      tic;
      if DSGEModel.MaximizeAlgorithmValue==1;
         [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorPhiDSGE',phi,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
            thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      else;
         [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorThetaDSGE',theta,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
            thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         %
         % make sure that the outputs are transformed into phi-space
         %
         phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
         phiEstimates = zeros(size(thetaEstimates));
         for i=1:size(thetaEstimates,2);
            phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
         end;
         ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
         InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
      end;
      MaxTime = toc;
      GradientVector = abs(gh);
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         WaitControls = get(WaitHandle,'UserData');
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
   end;
   %
   % check if the user would like to continue optimization using the new parameter values as input
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter)&(max(GradientVector)>ToleranceValue)&(DSGEModel.CsminwelExtraRuns>1);
      MaxExtraRuns = DSGEModel.CsminwelExtraRuns-1;
      LastFH = -fh;
      if isempty(CurrINI.Sequence)==1;
         txt = strvcat(['According to the convergence criterion used by csminwel, i.e., the change in the value of the log posterior (' num2str(FuncDiff) ') is less than or equal to the tolerance value (' num2str(ToleranceValue) '), the optimization has converged. However, the maximum value of the gradient (' num2str(max(GradientVector)) ') is greater than the tolerance value. Would you like to continue running csminwel using the latest values for the parameters along with a reinitialized inverse Hessian as input?'],' ',['NOTE: The maximum number of extra runs of csminwel is set to ' int2str(MaxExtraRuns) '. YADA will also abort if the new value of the log posterior is smaller than the previous value. In the event that YADA aborts from the extra csminwel runs it will use the original csminwel results.']);
         answer = Query(txt,'question',500,'Posterior Mode Estimation - csminwel',600,'no',CurrINI);
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
      if strcmp(lower(answer),'yes')==1;
         %
         % select a suitable convergence criterior
         %
         TolStr = get(controls.optimize.tolerance,'String');
         [action,TolVal] = SelectionDlg(TolStr,DSGEModel.OptToleranceValue,'Tolerance value for the between runs change of optimal value given by csminwel:','Select Tolerance Value','Posterior Mode Estimation - csminwel','','',CurrINI);
         if strcmp(lower(action),'ok')==1;
            NewToleranceValue = num2str(StringTrim(TolStr(TolVal,:)));
            %
            % store all original results
            %
            OldResults.fh = fh;
            OldResults.phiMode = phiMode;
            OldResults.GradientVector = GradientVector;
            OldResults.InverseHessian = InverseHessian;
            OldResults.NumIter = NumIter;
            OldResults.fcount = fcount;
            OldResults.FuncDiff = FuncDiff;
            OldResults.OptResults = OptResults;
            OldResults.ScreenMessages = ScreenMessages;
            OldResults.phiEstimates = phiEstimates;
            OldResults.ReturnCode = ReturnCode;
            OldResults.MaxTime = MaxTime;
            NewResults = OldResults;
            NewResults.LastRun = 0;
            %
            NumExtraRuns = 1;
            done = 0;
            TotalIter = 0;
            while done==0;
               %
               % run csminwel again
               %
               SetLogText(strvcat(' ',['Extra csminwel run number ' int2str(NumExtraRuns) ' out of ' int2str(MaxExtraRuns)],[' * Last value of the log posterior:  ' num2str(-fh,'%0.4f')]));
               tic;
               if DSGEModel.MaximizeAlgorithmValue==1;
                  [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorPhiDSGE',phiMode,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               else;
                  [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorThetaDSGE',thetaMode,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,ParameterBoundsBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                  %
                  % make sure that the outputs are transformed into phi-space
                  %
                  phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
                  phiEstimates = zeros(size(thetaEstimates));
                  for i=1:size(thetaEstimates,2);
                     phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
                  end;
                  ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
                  InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
               end;
               MaxTime = MaxTime+toc;
               GradientVector = abs(gh);
               SetLogText(strvcat([' * New value of the log posterior:   ' num2str(-fh,'%0.4f')],[' * Number of iterations:             ' int2str(NumIter)],[' * Maximum value of the gradient:    ' num2str(max(GradientVector))],[' * Original value of the gradient:   ' num2str(max(OldResults.GradientVector))],[' * Tolerance value:                  ' num2str(NewToleranceValue)]));
               %
               % collect the results
               %
               TotalIter = TotalIter+NumIter;
               if isequal(InverseHessian,InitialInvHessMat)==0;
                  NewResults.LastRun = NumExtraRuns;
                  NewResults.fh = fh;
                  NewResults.phiMode = phiMode;
                  NewResults.GradientVector = GradientVector;
                  NewResults.InverseHessian = InverseHessian;
                  NewResults.phiEstimates = [NewResults.phiEstimates phiEstimates];
                  NewResults.ReturnCode = ReturnCode;
                  NewResults.NumIter = NewResults.NumIter+NumIter;
                  NewResults.fcount = NewResults.fcount+fcount;
                  NewResults.FuncDiff = FuncDiff;
                  NewResults.OptResults = [NewResults.OptResults;OptResults];
                  NewResults.ScreenMessages = strvcat(NewResults.ScreenMessages,' ',['Addition run number: ' int2str(NumExtraRuns)],ScreenMessages);
               end;
               NewResults.MaxTime = MaxTime;
               PosTestValue = (-fh-LastFH>=0);
               SmallTestValue = (-fh-LastFH<=NewToleranceValue);
               ConvTest = (PosTestValue==1)&(SmallTestValue);
               if (ConvTest==1)&(TotalIter>NumExtraRuns)&(strcmp(lower(AbortEstimation),'cancel')==0);
                  done = 1;
               elseif strcmp(lower(AbortEstimation),'cancel')==1;
                  done = 2;
               elseif PosTestValue==0;
                  done = 3;
               elseif NumExtraRuns==MaxExtraRuns;
                  done = 4;
               else;
                  NumExtraRuns = NumExtraRuns+1;
                  LastFH = -fh;
               end;
            end;
            if done==1;
               if (NewResults.LastRun>0)&(OldResults.fh-NewResults.fh>0);
                  if isempty(CurrINI.Sequence)==1;
                     txt = ['YADA has finished the extra runs. The new results are taken from extra run number ' int2str(NewResults.LastRun) '. The value of the log posterior has changed from ' num2str(-OldResults.fh,'%0.4f') ' to ' num2str(-NewResults.fh,'%0.4f') ', while the maximum value of the gradient was originally ' num2str(max(OldResults.GradientVector)) ' and is now ' num2str(max(NewResults.GradientVector)) '. At the same time, the smallest eigenvalue of the inverse Hessian was originally ' num2str(min(abs(eig(OldResults.InverseHessian)))) ' and is now ' num2str(min(abs(eig(NewResults.InverseHessian)))) '. Would you like to use the new results?'];
                     answer = Query(txt,'question',500,'Posterior Mode Estimation - csminwel',600,'no',CurrINI);
                  else;
                     answer = CurrINI.Sequence.QueryAnswers{2};
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     NumIter = NewResults.NumIter;
                     fcount = NewResults.fcount;
                     OptResults = NewResults.OptResults;
                     ScreenMessages = NewResults.ScreenMessages;
                     phiEstimates = NewResults.phiEstimates;
                     InverseHessian = NewResults.InverseHessian;
                     phiMode = NewResults.phiMode;
                     FuncDiff = NewResults.FuncDiff;
                     fh = NewResults.fh;
                     GradientVector = NewResults.GradientVector;
                     ReturnCode = NewResults.ReturnCode;
                  else;
                     fh = OldResults.fh;
                     phiMode = OldResults.phiMode;
                     GradientVector = OldResults.GradientVector;
                     InverseHessian = OldResults.InverseHessian;
                     NumIter = OldResults.NumIter;
                     fcount = OldResults.fcount;
                     FuncDiff = OldResults.FuncDiff;
                     OptResults = OldResults.OptResults;
                     ScreenMessages = OldResults.ScreenMessages;
                     phiEstimates = OldResults.phiEstimates;
                     ReturnCode = OldResults.ReturnCode;
                     MaxTime = OldResults.MaxTime;
                  end;
               else;
                  fh = OldResults.fh;
                  phiMode = OldResults.phiMode;
                  GradientVector = OldResults.GradientVector;
                  InverseHessian = OldResults.InverseHessian;
                  NumIter = OldResults.NumIter;
                  fcount = OldResults.fcount;
                  FuncDiff = OldResults.FuncDiff;
                  OptResults = OldResults.OptResults;
                  ScreenMessages = OldResults.ScreenMessages;
                  phiEstimates = OldResults.phiEstimates;
                  ReturnCode = OldResults.ReturnCode;
                  MaxTime = OldResults.MaxTime;
                  SetLogText(strvcat(' ','No real improvement: YADA will use the original csminwel results'));
               end;
            else;
               if done==4;
                  SetLogText(strvcat(' ','No real improvement: YADA will use the original csminwel results'));
               elseif done==3;
                  SetLogText(strvcat(' ','Lower log posterior value: YADA will use the original csminwel results'));
               elseif done==2;
                  SetLogText(strvcat(' ','User aborted: YADA will use the original csminwel results'));
               end;
               fh = OldResults.fh;
               phiMode = OldResults.phiMode;
               GradientVector = OldResults.GradientVector;
               InverseHessian = OldResults.InverseHessian;
               NumIter = OldResults.NumIter;
               fcount = OldResults.fcount;
               FuncDiff = OldResults.FuncDiff;
               OptResults = OldResults.OptResults;
               ScreenMessages = OldResults.ScreenMessages;
               phiEstimates = OldResults.phiEstimates;
               ReturnCode = OldResults.ReturnCode;
               MaxTime = OldResults.MaxTime;
            end;
            clear('OldResults','NewResults');
         else;
            SetLogText(strvcat(' ','User skipped extra runs: YADA will use the original csminwel results'));
         end;
      end;
   end;
   %
   % gh is the numerical gradient, fh the function value at minimum
   %
   % ReturnCode: 0 = minimum found
   %             1 = zero gradient
   %        2 or 4 = back and forth on step length never finished
   %             3 = smallest step still improving too slow
   %             5 = largest step still improving too fast
   %             6 = smallest step still improving too slow, reversed gradient
   %             7 = warning: possible inaccuracy in H matrix
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
      %
      % we're OK
      %
      LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         LogPostDensity = -fh;
      else;
         LogPostDensity = -fh+LogJacobianMode;
      end;
      %
      % Calculate the log posterior at theta
      %
      thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
      %
      LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
      %
      % make sure ModelParameters is updated
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
      if IsSystemPrior==1;
         eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
         SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
      else;
         LogSPMode = 0;
         SystemPriorStr = '';
      end;
      LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
      %
      SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
      %
      EstimationStatus = 1;
      %
   elseif strcmp(lower(AbortEstimation),'cancel')==1;
      %
      % posteriod mode estimation was aborted
      %
      SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
      %
      % maximum number of iterations was reached
      %
      txt = ['The maximum number of iterations was reached without csminwel converging. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Maximum number of iteration reached.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
   elseif ReturnCode==1;
      txt = ['Csminwel return code: zero gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: zero gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: zeros gradient.';
   elseif (ReturnCode==2)|(ReturnCode==4);
      txt = ['Csminwel return code: back and forth on step length never finished. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: back and forth on step length never finished.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: back and forth on step length never finished.';
   elseif ReturnCode==3;
      txt = ['Csminwel return code: smallest step still improving too slow. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: smallest step still improving too slow.';
   elseif ReturnCode==5;
      txt = ['Csminwel return code: largest step still improving too fast. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: largest step still improving too fast.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: largest step still improving too fast.';
   elseif ReturnCode==6;
      txt = ['Csminwel return code: smallest step still improving too slow, reversed gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow, reversed gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: smallest step still improving too slow, reversed gradient.';
   elseif ReturnCode==7;
      txt = ['Csminwel return code: possible inaccuracy in inverse Hessian. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: possible inaccuracy in inverse Hessian.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: possible inaccuracy in inverse Hessian.';
   elseif ReturnCode==-1;
      SetLogText(strvcat(' ','Csminwel error: optimization ended abruptly.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==3)|(DSGEModel.MaximizeAlgorithmValue==4);
   %
   % use Marco Ratto's newrat
   %
   OptAlgorithm = 'newrat';
   %
   % compute the initial inverse Hessian
   %
   if DSGEModel.InitializeHessian==1;
      HessianMat = 10^(-1)*eye(NumParam);
   elseif DSGEModel.InitializeHessian==2;
      HessianMat = eye(NumParam);
   elseif DSGEModel.InitializeHessian==3;
      HessianMat = 10*eye(NumParam);
   elseif DSGEModel.InitializeHessian==4;
      HessianMat = 10^2*eye(NumParam);
   elseif DSGEModel.InitializeHessian==5;
      HessianMat = 10^3*eye(NumParam);
   elseif DSGEModel.InitializeHessian==6;
      HessianMat = 10^4*eye(NumParam);
   elseif DSGEModel.InitializeHessian==7;
      HessianMat = 10^5*eye(NumParam);
   elseif DSGEModel.InitializeHessian==8;
      %
      % sample from the prior
      %
      HessianMat = eye(NumParam)/PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
   end;
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      eval(['LogSPIV = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   % create prior standard deviations
   %
   PriorStd = sqrt(diag(PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue))));
   %
   % call the function
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogPostTime) ' seconds'],AlgorithmText,InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Ratto''s newrat for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   %
   % run the estimation
   %
   if DSGEModel.MaximizeAlgorithmValue==3;
      ParamLUB = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
   else;
      ParamLUB = ParameterBounds;
   end;
   %
   try;
      tic;
      if DSGEModel.MaximizeAlgorithmValue==3;
         [phiMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAnewrat('logPosteriorPhiDSGE',phi,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - newrat',CurrINI, ...
            thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      else;
         [thetaMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAnewrat('logPosteriorThetaDSGE',theta,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - newrat',CurrINI, ...
            thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         %
         % make sure that the outputs are transformed into phi-space
         %
         phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
         phiEstimates = zeros(size(thetaEstimates));
         for i=1:size(thetaEstimates,2);
            phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
         end;
         ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
         InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
      end;
      %
      MaxTime = toc;
      GradientVector = abs(gh);
      if isinf(fh)==1;
         ReturnCode = -2;
         ErrorStr = 'The value of the objective function is infinite. Optimization aborted.';
      elseif isnan(fh)==1;
         ReturnCode = -3;
         ErrorStr = 'The objective function is a NaN value. Optimization aborted.';
      end;
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         WaitControls = get(WaitHandle,'UserData');
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running newrat. YADA caught the following error message: ' lasterr];
   end;
   %
   % gh is the numerical gradient, fh the function value at minimum
   %
   % ReturnCode: 0 = minimum found
   %             1 = zero gradient
   %        2 or 4 = back and forth on step length never finished
   %             3 = smallest step still improving too slow
   %             5 = largest step still improving too fast
   %             6 = smallest step still improving too slow, reversed gradient
   %             7 = warning: possible inaccuracy in H matrix
   %            -2 infinite value of function
   %            -3 NaN value of function
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
      %
      % we're OK
      %
      LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         LogPostDensity = -fh;
      else;
         LogPostDensity = -fh+LogJacobianMode;
      end;
      %
      % Calculate the log posterior at theta
      %
      thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
      %
      LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
      %
      % make sure ModelParameters is updated
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
      if IsSystemPrior==1;
         eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
         SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
      else;
         LogSPMode = 0;
         SystemPriorStr = '';
      end;
      LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
      %
      SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
      %
      EstimationStatus = 1;
      %
   elseif strcmp(lower(AbortEstimation),'cancel')==1;
      %
      % posteriod mode estimation was aborted
      %
      SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
      %
      % maximum number of iterations was reached
      %
      txt = ['The maximum number of iterations was reached without newrat converging. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Maximum number of iteration reached.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
   elseif ReturnCode==1;
      txt = ['Newrat return code: zero gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: zero gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: zeros gradient.';
   elseif (ReturnCode==2)|(ReturnCode==4);
      txt = ['Newrat return code: back and forth on step length never finished. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: back and forth on step length never finished.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: back and forth on step length never finished.';
   elseif ReturnCode==3;
      txt = ['Newrat return code: smallest step still improving too slow. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: smallest step still improving too slow.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: smallest step still improving too slow.';
   elseif ReturnCode==5;
      txt = ['Newrat return code: largest step still improving too fast. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: largest step still improving too fast.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: largest step still improving too fast.';
   elseif ReturnCode==6;
      txt = ['Newrat return code: smallest step still improving too slow, reversed gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: smallest step still improving too slow, reversed gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: smallest step still improving too slow, reversed gradient.';
   elseif ReturnCode==7;
      txt = ['Newrat return code: possible inaccuracy in inverse Hessian. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: possible inaccuracy in inverse Hessian.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: possible inaccuracy in inverse Hessian.';
   elseif ReturnCode==-1;
      SetLogText(strvcat(' ','Newrat error: optimization ended abruptly.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
         drawnow;
      end;
      return;
   elseif ReturnCode==-2;
      SetLogText(strvcat(' ','Newrat error: infinite value of objective function.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
         drawnow;
      end;
      return;
   elseif ReturnCode==-3;
      SetLogText(strvcat(' ','Newrat error: NaN value of objective function.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==5)|(DSGEModel.MaximizeAlgorithmValue==6);
   %
   % use Dynare's gmhmaxlik
   %
   OptAlgorithm = 'gmhmaxlik';
   %
   % determine the initial scale factor
   %
   ScaleStr = get(controls.posterior.scalefactor,'String');
   c = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
   %
   % determine the number of draws
   %
   DrawsStr = get(controls.posterior.draws,'String');
   NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
   NumDraws = NumPostDraws-NumBurnin;
   %
   % initial covariance matrix
   %
   [InitialInvHessian,ParamMean] = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));;
   %
   % parameter bounds
   %
   if DSGEModel.MaximizeAlgorithmValue==5;
      ParamLUB = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
   else;
      ParamLUB = ParameterBounds;
   end;
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      eval(['LogSPIV = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   %
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogPostTime) ' seconds'],AlgorithmText,InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Dynare''s gmhmaxlik for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Number of Monte Carlo draws:      ' int2str(NumDraws)],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   %
   try;
      if DSGEModel.MaximizeAlgorithmValue==5;
         [phiMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,MaxTime] = YADAgmhmaxlik('logPosteriorPhiDSGE',phi,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
            thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      else;
         [thetaMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,MaxTime] = YADAgmhmaxlik('logPosteriorThetaDSGE',theta,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
            thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         %
         % make sure that the outputs are transformed into phi-space
         %
         phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
         phiEstimates = zeros(size(thetaEstimates));
         for i=1:size(thetaEstimates,2);
            phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
         end;
         ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
         ThetaInverseHessian = InverseHessian;
         InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
      end;
      %
      GradientVector = abs(gh);
      ReturnCode = 0;
      NumIter = NumDraws;
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            if IsHandle(WaitHandle)==1;
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         if IsHandle(WaitHandle)==1;
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running gmhmaxlik. YADA caught the following error message: ' lasterr];
   end;
   %
   % check what we should do
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
      %
      % check if the user wants to run csminwel using the gmhmaxlik results as input
      %
      if isempty(CurrINI.Sequence)==1;
         txt = 'Would you like to run csminwel once using the gmhmaxlik results as input?';
         answer = Query(txt,'question',200,'YADA - Csminwel Optimization',500,'no',CurrINI);
      else;
         answer = CurrINI.Sequence.QueryAnswers{5};
      end;
      if strcmp(lower(answer),'yes')==1;
         OldResults.fh = fh;
         OldResults.phiMode = phiMode;
         OldResults.GradientVector = GradientVector;
         OldResults.InverseHessian = InverseHessian;
         OldResults.NumIter = NumIter;
         OldResults.fcount = fcount;
         OldResults.FuncDiff = FuncDiff;
         OldResults.OptResults = OptResults;
         OldResults.ScreenMessages = ScreenMessages;
         OldResults.phiEstimates = phiEstimates;
         OldResults.ReturnCode = ReturnCode;
         OldResults.MaxTime = MaxTime;
         NewResults = OldResults;
         NewResults.LastRun = 0;
         if DSGEModel.MaximizeAlgorithmValue==5;
            if IsPositiveDefinite(InverseHessian)==0;
               OldResults.InverseHessian = abs(diag(diag(OldResults.InverseHessian)));
               SetLogText(strvcat(' ','NOTE: The inverse Hessian is not positive definite. Using only diagonal elements.'));
            end;
         else;
            OldResults.ThetaInverseHessian = ThetaInverseHessian;
            if IsPositiveDefinite(ThetaInverseHessian)==0;
               OldResults.ThetaInverseHessian = abs(diag(diag(OldResults.ThetaInverseHessian)));
               SetLogText(strvcat(' ','NOTE: The inverse Hessian is not positive definite. Using only diagonal elements.'));
            end;
         end;
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==5;
               [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorPhiDSGE',phiMode,OldResults.InverseHessian,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            else;
               [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorThetaDSGE',thetaMode,OldResults.ThetaInverseHessian,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(thetaEstimates));
               for i=1:size(thetaEstimates,2);
                  phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
               end;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
            end;
            if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
               MaxTime = MaxTime+toc;
               GradientVector = abs(gh);
               SetLogText(strvcat([' * Old value of the log posterior:   '  num2str(-OldResults.fh,'%0.4f')],' ',' * Running csminwel to climb even higher',[' * New value of the log posterior:   ' num2str(-fh,'%0.4f')],[' * Number of iterations:             ' int2str(NumIter)]));
               NewResults.LastRun = NumIter;
               NewResults.fh = fh;
               NewResults.phiMode = phiMode;
               NewResults.GradientVector = GradientVector;
               NewResults.InverseHessian = InverseHessian;
               NewResults.phiEstimates = [NewResults.phiEstimates phiEstimates];
               NewResults.ReturnCode = ReturnCode;
               NewResults.NumIter = NewResults.NumIter+NumIter;
               NewResults.fcount = NewResults.fcount+fcount;
               NewResults.FuncDiff = FuncDiff;
               NewResults.OptResults = [NewResults.OptResults;OptResults];
               NewResults.ScreenMessages = strvcat(NewResults.ScreenMessages,' ','Addition csminwel runs:',ScreenMessages);
               %
               PosTestValue = (-fh+OldResults.fh>=0);
               if (PosTestValue==1)&(IsPositiveDefinite(InverseHessian)==1);
                  NumIter = NewResults.NumIter;
                  fcount = NewResults.fcount;
                  OptResults = NewResults.OptResults;
                  ScreenMessages = NewResults.ScreenMessages;
                  phiEstimates = NewResults.phiEstimates;
                  SetLogText(' * Using the new results from csminwel');
               else;
                  fh = OldResults.fh;
                  phiMode = OldResults.phiMode;
                  GradientVector = OldResults.GradientVector;
                  InverseHessian = OldResults.InverseHessian;
                  NumIter = OldResults.NumIter;
                  fcount = OldResults.fcount;
                  FuncDiff = OldResults.FuncDiff;
                  OptResults = OldResults.OptResults;
                  ScreenMessages = OldResults.ScreenMessages;
                  phiEstimates = OldResults.phiEstimates;
                  ReturnCode = OldResults.ReturnCode;
                  MaxTime = OldResults.MaxTime;
                  if PosTestValue==1;
                     SetLogText(strvcat('NOTE: Bad inverse Hessian for csminwel',' * Using the results from gmhmaxlik only'));
                  else;
                     SetLogText(strvcat('NOTE: Log posterior kernel value not improving',' * Using the results from gmhmaxlik only'));
                  end;
               end;
            else;
               toc;
               fh = OldResults.fh;
               phiMode = OldResults.phiMode;
               GradientVector = OldResults.GradientVector;
               InverseHessian = OldResults.InverseHessian;
               NumIter = OldResults.NumIter;
               fcount = OldResults.fcount;
               FuncDiff = OldResults.FuncDiff;
               OptResults = OldResults.OptResults;
               ScreenMessages = OldResults.ScreenMessages;
               phiEstimates = OldResults.phiEstimates;
               ReturnCode = OldResults.ReturnCode;
               MaxTime = OldResults.MaxTime;
               SetLogText(' * Using the results from gmhmaxlik only');
            end;
         catch;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               else;
                  WaitHandle = findobj('Type','figure','Tag','WaitDLG');
                  if IsHandle(WaitHandle)==1;
                     WaitControls = get(WaitHandle,'UserData');
                     set(WaitControls.text,'UserData','done');
                     delete(WaitHandle);
                     drawnow;
                     pause(0.02);
                  end;
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               if IsHandle(WaitHandle)==1;
                  WaitControls = get(WaitHandle,'UserData');
                  set(WaitControls.text,'UserData','done');
                  delete(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            end;
            ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
         end;
      else;
         OldResults = [];
      end;
      %
      % we're OK
      %
      LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         LogPostDensity = -fh;
      else;
         LogPostDensity = -fh+LogJacobianMode;
      end;
      %
      % Calculate the log posterior at theta
      %
      thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
      %
      LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
      %
      % make sure ModelParameters is updated
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
      if IsSystemPrior==1;
         eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
         SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
      else;
         LogSPMode = 0;
         SystemPriorStr = '';
      end;
      LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
      %
      SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Objective function change:        ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(size(OptResults,1),4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
      %
      EstimationStatus = 1;
      %
   elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')==1);
      %
      % the user selected to abort posterior mode estimation
      %
      SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   elseif ReturnCode==-1;
      SetLogText(strvcat(' ','Gmhmaxlik error: optimization ended abruptly.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
   %
   % use matlab's fminunc
   %
   OptAlgorithm = 'fminunc';
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      eval(['LogSPIV = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   % set the options
   %
   Options = optimset('MaxIter',MaxIter,'MaxFunEvals',100*MaxIter,'Diagnostics','off','LargeScale','off','TolFun',ToleranceValue,'TolX',ToleranceValue);
   %
   % call the function
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogPostTime) ' seconds'],AlgorithmText,InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Matlab''s fminunc for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   if MatlabNumber>=7;
      try;
         tic;
         if DSGEModel.MaximizeAlgorithmValue==7;
            [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logPosteriorPhiDSGE',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logPosteriorThetaDSGE',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(OUTPUT.phiEstimates));
            for i=1:size(OUTPUT.phiEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(OUTPUT.phiEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            OUTPUT.phiEstimates = phiEstimates;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
         end;
         MaxTime = toc;
         GradientVector = abs(Grad);
         AbortEstimation = OUTPUT.abort;
         NumIter = OUTPUT.iterations;
         OptResults = OUTPUT.OptResults;
      catch;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
         ExitFlag = NaN;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running fminunc. YADA caught the following error message: ' lasterr];
      end;
      if (ExitFlag>0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
         %
         % we're OK
         %
         FuncDiff = OUTPUT.convergence;
         LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
         if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
            LogPostDensity = -fh;
            thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
         else;
            LogPostDensity = -fh+LogJacobianMode;
         end;
         InverseHessian = HessianMatrix\eye(NumParam);
         phiEstimates = OUTPUT.phiEstimates;
         %
         % compute the value of the prior and of the likelihood         
         %
         LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
         %
         % make sure ModelParameters is updated
         %
         ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
         if IsSystemPrior==1;
            eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
            SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
         else;
            LogSPMode = 0;
            SystemPriorStr = '';
         end;
         LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
         %
         SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
         %
         EstimationStatus = 1;
         %
      elseif strcmp(lower(AbortEstimation),'cancel')==1;
         %
         % posteriod mode estimation was aborted
         %
         SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
      elseif ExitFlag==0;
         %
         % Reached maximum number of iteration
         %
         txt = ['The maximum number of function evaluations or iterations was reached without fminunc converging (ExitFlag is 0). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ','Maximum number of function evaluations or iteration reached.',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = 'Fminunc return code: the maximum number of function evaluations or iteration was reached without fminunc coverging (ExitFlag is 0).';
      elseif ExitFlag<0;
         %
         % no convergence
         %
         txt = ['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) '). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ',['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) ').'],[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = ['Fminunc return code: No convergence (ExitFlag is ' int2str(ExitFlag) ').'];
      elseif isnan(ExitFlag)==1;
         SetLogText(strvcat(' ','Fminunc error: optimization ended abruptly.'));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
            drawnow;
         end;
         return;
      end;
   else;
      try;
         tic;
         if DSGEModel.MaximizeAlgorithmValue==7;
            [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logPosteriorPhiDSGE',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logPosteriorThetaDSGE',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(OUTPUT.phiEstimates));
            for i=1:size(OUTPUT.phiEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(OUTPUT.phiEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            OUTPUT.phiEstimates = phiEstimates;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
         end;
         MaxTime = toc;
         AbortEstimation = OUTPUT.abort;
         NumIter = OUTPUT.iterations;
         OptResults = OUTPUT.OptResults;
         GradientVector = abs(Grad);
      catch;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
         ExitFlag = NaN;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running fminunc. YADA caught the following error message: ' lasterr];
      end;
      if (ExitFlag>0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
         %
         % we're OK
         %
         FuncDiff = OUTPUT.convergence;
         LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
         if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
            LogPostDensity = -fh;
            thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
         else;
            LogPostDensity = -fh+LogJacobianMode;
         end;
         InverseHessian = HessianMatrix\eye(NumParam);
         phiEstimates = OUTPUT.phiEstimates;
         %
         % compute the value of the prior density and the likelihood         
         %
         LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
         %
         % make sure ModelParameters is updated
         %
         ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
         if IsSystemPrior==1;
            eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
            SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
         else;
            LogSPMode = 0;
            SystemPriorStr = '';
         end;
         LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
         %
         SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
         %
         EstimationStatus = 1;
         %
      elseif strcmp(lower(AbortEstimation),'cancel')==1;
         %
         % posteriod mode estimation was aborted
         %
         SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
      elseif ExitFlag==0;
         %
         % Reached maximum number of iteration
         %
         txt = ['The maximum number of function evaluations or iterations was reached without fminunc converging (ExitFlag is 0). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ','Maximum number of function evaluations or iteration reached.',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = 'Fminunc return code: the maximum number of function evaluations or iteration was reached without fminunc coverging (ExitFlag is 0).';
      elseif ExitFlag<0;
         %
         % no convergence
         %
         txt = ['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) '). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ',['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) ').'],[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = ['Fminunc return code: No convergence (ExitFlag is ' int2str(ExitFlag) ').'];
      elseif isnan(ExitFlag)==1;
         SetLogText(strvcat(' ','Fminunc error: optimization ended abruptly.'));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
            drawnow;
         end;
         return;
      end;
   end;
end;
%
% check if estimation was unsuccessful for some reason
%
if (EstimationStatus==0)&(strcmp(lower(AbortEstimation),'cancel')==0);
   %
   % write error information to file
   %
   SetLogText(strvcat('Writing optimization error summary to disk.',['   Directory: ' DSGEModel.OutputDirectory],['   File name: OptErrorReport-' DSGEModel.NameOfModel '.txt']));
   %
   % transform phiMode to thetaMode taking into account that certain entries may be Inf or Nan's
   %
   NotFinite = 1-isfinite(phiMode);
   if sum(NotFinite)>=1;
      HelpIndex = (1:NumParam)';
      SwapParams = (NotFinite .* HelpIndex);
      SwapParams = SwapParams(SwapParams>0);
      phiModeOld = phiMode;
      phiMode(SwapParams) = 1;
   end;
   thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
   %
   % compute the partial of theta
   %
   phiGradientVector = abs((PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds)\eye(NumParam))*GradientVector);
   %
   % Now we have to reset all those elelemts that were either NaN or Inf
   %
   if sum(NotFinite)>=1;
      thetaMode(SwapParams) = phiModeOld(SwapParams);
      phiGradientVector(SwapParams) = abs(phiModeOld(SwapParams));
   end;
   %
   % check what we can save to mat-file
   %
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      LogPostDensity = -fh;
   else;
      LogPostDensity = -fh+logJacobian(phiMode,thetaIndex,UniformBounds);
   end;
   if (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
      if (isempty(HessianMatrix)==0)&(rank(HessianMatrix)==size(HessianMatrix,2));
         InverseHessian = HessianMatrix\eye(length(phiMode));
      else;
         InverseHessian = [];
      end;
      phiEstimates = OUTPUT.phiEstimates;
   end;
   %
   % see if we should check the optimum
   %
   if (DSGEModel.CheckOptimum==1)&(isempty(InverseHessian)==0)&(IsPositiveDefinite(InverseHessian)==1);
      %
      % Check if GraphDirectory exists
      %
      GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
      MinMaxGrid = zeros(NumParam,2);
      stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
      %
      % create lower and upper bounds from InverseHessian and phiMode
      %
      MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
      MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
      try;
         phiFittedStd = VillaniCheckOptimum('logPosteriorPhiDSGE',phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature Of Log Posterior - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',ShowFigs,1, ...
            thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      catch;
         phiFittedStd = NaN;
      end;
   else;
      phiFittedStd = NaN;
   end;
   %
   % Save to disk
   %
   ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
   stat = MakeDir(DSGEModel.OutputDirectory,'mode');
   if stat==1;
      SetLogText(strvcat(' ','Saving optimization data to disk using directory:',['   "' ModeFileDir '"...']));
      ModeFile = FixFilePath([ModeFileDir '\PosteriorModeError-' DSGEModel.NameOfModel '.mat']);
      save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','ParameterNames','LogPostDensity','InverseHessian','phiFittedStd');
   end;
   %
   % print an error report
   %
   TxtFileName = FixFilePath([DSGEModel.OutputDirectory '\OptErrorReport-' DSGEModel.NameOfModel '.txt']);
   PrintOptimizationErrorReport(TxtFileName,0,[],DSGEModel,CurrINI,AIMData,ParameterNames,PriorDist,thetaDist,thetaMode,theta,phiGradientVector,GradientVector,rank(R),OptResults,ScreenMessages,ReturnCodeText);
   %
   % display the file
   %
   if isempty(CurrINI.Sequence)==1;
      TextGUI(TxtFileName,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   end;
   return;
elseif (EstimationStatus==0)&(strcmp(lower(AbortEstimation),'cancel')==1);
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   return;
end;
%
% estimation went well. prepare output
%
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% Record the first observation used by the Kalman filter when calculating the log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
% Compute the largest eigenvalue of F in the state equation at the mode and at the initial values
%
MaxEigenvalueIV = CalculateDSGEMaximumEigenvalue(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
MaxEigenvalueMode = CalculateDSGEMaximumEigenvalue(thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI);
%
% Compute the partial derivatives of the original parameters wrt the transformed parameters, evaluated at the
% posterior mode
%
ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
%
% Check if we should compute the inverse Hessian with finite difference methods
%
if DSGEModel.FiniteDifferenceHessian==1;
   SetLogText(strvcat(' ','Computing posterior standard deviations from inverse Hessian','using finite differences...'));
   %
   % for the transformed parameters
   %
   VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phiMode,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   phiInverseHessianFinDiff = HessianMat\eye(NumParam);
   phiPostModeStdFinDiff = sqrt(diag(phiInverseHessianFinDiff));
   %
   % for the orginal parameters
   %
   VechHessian = YADAHessianMatrix('logPosteriorThetaDSGE',thetaMode,StepLength,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   thetaInverseHessianFinDiff = HessianMat\eye(NumParam);
   thetaPostModeStdFinDiff = sqrt(diag(thetaInverseHessianFinDiff));
else;
   if isempty(CurrINI.Sequence)==1;
      txt = 'Would you like to compute the inverse Hessian using finite differences and use it to estimate the marginal likelihood with the Laplace approximation?';
      answer = Query(txt,'question',500,'Posterior Mode Estimation - Finite Differences',600,'yes',CurrINI);
   else;
      answer = CurrINI.Sequence.QueryAnswers{4};
   end;
   if strcmp(lower(answer),'yes')==1;
      SetLogText(strvcat(' ','Computing an inverse Hessian using finite differences...'));
      VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phiMode,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
      if IsPositiveDefinite(HessianMat)==1;
         phiInverseHessianFinDiff = HessianMat\eye(NumParam);
         SetLogText('* Matrix is positive definite.');
      else;
         phiInverseHessianFinDiff = NaN;
         SetLogText('* Matrix is not positive definite.');
      end;
   else;
      phiInverseHessianFinDiff = NaN;
   end;
   %
   phiPostModeStdFinDiff = NaN;
   thetaInverseHessianFinDiff = NaN;
   thetaPostModeStdFinDiff = NaN;
end;
OldInverseHessian = InverseHessian;
if IsPositiveDefinite(InverseHessian)==0;
   %
   % try to compute an inverse Hessian using finite differences
   %
   SetLogText(strvcat(' ','WARNING: The inverse Hessian at the posterior mode is not','         positive definite. Replacing it with estimator','         from finite differences.'));
   if DSGEModel.FiniteDifferenceHessian==1;
      InverseHessian = phiPostModeStdFinDiff;
   else;
      VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phiMode,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
      InverseHessian = HessianMat\eye(NumParam);
   end;
end;
if IsPositiveDefinite(InverseHessian)==0;
   SetLogText(strvcat(' ','WARNING: The inverse Hessian at the posterior mode','         is not positive definite.'));
   DgInvHess = diag(InverseHessian);
   if length(DgInvHess(DgInvHess>0))==length(DgInvHess);
      if isempty(CurrINI.Sequence)==1;
         txt = 'The inverse Hessian at the posterior mode is not positive definite. Do you want YADA to attempt to create a positive definite matrix from the inverse Hessian?';
         answer = Query(txt,'question',200,'YADA - Bad Inverse Hessian',500,'no',CurrINI);
      else;
         answer = CurrINI.Sequence.QueryAnswers{3};
      end;
      if strcmp(lower(answer),'yes')==1;
         SetLogText('         Attempting the create positive definite matrix...');
         [InvHess,pdstatus] = CreatePositiveDefinite(InverseHessian);
         if pdstatus==1;
            SetLogText(strvcat('         STATUS: Successful. Replacing old inverse Hessian'));
            InverseHessian = InvHess;
         else;
            SetLogText(strvcat('         STATUS: Failed.'));
            EstimationStatus = 0;
         end;
      end;
   end;
end;
%
% Check for problems
%
if EstimationStatus==0;
   SetLogText(strvcat(' ','Writing optimization error summary to disk.',['   Directory: ' DSGEModel.OutputDirectory],['   File name: OptErrorReport-' DSGEModel.NameOfModel '.txt']));
   %
   % compute the partial of theta
   %
   phiGradientVector = abs((PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds)\eye(NumParam))*GradientVector);
   if (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
      phiEstimates = OUTPUT.phiEstimates;
   end;
   phiFittedStd = NaN;
   %
   % Save to disk
   %
   ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
   stat = MakeDir(DSGEModel.OutputDirectory,'mode');
   if stat==1;
      SetLogText(strvcat(' ','Saving optimization data to disk using directory:',['   "' ModeFileDir '"...']));
      ModeFile = FixFilePath([ModeFileDir '\PosteriorModeError-' DSGEModel.NameOfModel '.mat']);
      save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','ParameterNames','LogPostDensity','InverseHessian','phiFittedStd');
   end;
   %
   % print an error report
   %
   TxtFileName = FixFilePath([DSGEModel.OutputDirectory '\OptErrorReport-' DSGEModel.NameOfModel '.txt']);
   ReturnCodeText = 'YADA return code: Inverse Hessian is not positive definite.';
   PrintOptimizationErrorReport(TxtFileName,0,[],DSGEModel,CurrINI,AIMData,ParameterNames,PriorDist,thetaDist,thetaMode,theta,phiGradientVector,GradientVector,rank(R),OptResults,ScreenMessages,ReturnCodeText);
   %
   % display the file
   %
   if isempty(CurrINI.Sequence)==1;
      TextGUI(TxtFileName,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   end;
   SetLogText(' ');
   return;
end;
%
% Check if we should check the optimum
%
if (DSGEModel.CheckOptimum==1)&(IsPositiveDefinite(InverseHessian)==1);
   %
   % make sure we remove the log marginal likelihood of the system prior from the log
   % marginal likelihood of the data
   %
   if isnan(phiInverseHessianFinDiff)==0;
      LogMargLike = MargLikeLaplace(LogPostDensity,phiInverseHessianFinDiff,1)-LogMargLikeSystemLaplace;
      LapStr = 'finite diff.';
   else;
      LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
      LapStr = 'optimization';
   end;
   SetLogText(strvcat(' ',['Computing the Laplace approximation using ' LapStr ' estimator...'],[' * Log marg. likelihood at phi mode: ' num2str(LogMargLike,'%0.4f')],' ','Checking the optimum...',' * Transformed parameters'));
   %
   % Check if GraphDirectory exists
   %
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   MinMaxGrid = zeros(NumParam,2);
   stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
   %
   % create lower and upper bounds from InverseHessian and phiMode
   %
   MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
   MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
   phiFittedStd = VillaniCheckOptimum('logPosteriorPhiDSGE',phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',ShowFigs,1, ...
      thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
      SetLogText(' * Original parameters');
      MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),thetaIndex,UniformBounds,LowerBound);
      MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),thetaIndex,UniformBounds,LowerBound);
      thetaFittedStd = VillaniCheckOptimum('logPosteriorThetaDSGE',thetaMode,[],MinMaxGrid,ThetaPhiPartial*InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta','','',ShowFigs,1, ...
         thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   else;
      thetaFittedStd = NaN;
   end;
else;
   if IsPositiveDefinite(InverseHessian)==1;
      if isnan(phiInverseHessianFinDiff)==0;
         LogMargLike = MargLikeLaplace(LogPostDensity,phiInverseHessianFinDiff,1)-LogMargLikeSystemLaplace;
         LapStr = 'finite diff.';
      else;
         LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
         LapStr = 'optimization';
      end;
      SetLogText(strvcat(' ',['Computing the Laplace approximation using ' LapStr ' estimator...'],[' * Log marg. likelihood at phi mode: ' num2str(LogMargLike,'%0.4f')]));
   end;
   phiFittedStd = NaN;
   thetaFittedStd = NaN;
end;
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% check the parameters with most extreme correlations based on the inverse Hessian
%
ExtremeCorrBound = 0.7;
CorrHessian = diag(1./sqrt(diag(InverseHessian)))*InverseHessian*diag(1./sqrt(diag(InverseHessian)));
CorrHessian = tril(CorrHessian,-1);
[RowIndex,ColIndex] = find(abs(CorrHessian)>=ExtremeCorrBound);
if isempty(RowIndex)==0;
   ExtremeCorrParameters = cell(length(RowIndex),3);
   for i=1:length(RowIndex)
      ExtremeCorrParameters{i,1} = ParameterNames.estimated(RowIndex(i),:);
      ExtremeCorrParameters{i,2} = ParameterNames.estimated(ColIndex(i),:);
      ExtremeCorrParameters{i,3} = CorrHessian(RowIndex(i),ColIndex(i));
   end;
else;
   ExtremeCorrParameters = [];
end;
%
% check other possible rank deficiencies with the inverse Hessian at the mode
%
[EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(CorrHessian);
SmallEigenValues = EigenValues(EigenValues<0.01);
if isempty(SmallEigenValues)==0;
   p1 = length(SmallEigenValues);
   p = size(EigenVectorMatrix,2);
   SmallEigenVectors = EigenVectorMatrix(:,p-p1+1:p);
else;
   SmallEigenVectors = [];
end;
%
% calculate the log-likelihood at theta and the log prior
%
ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
[LogLikeDensity,Solution,mcode,status] = logLikelihoodDSGE(ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
LogPriorDensity = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
LogJacobianValue = logJacobian(phiMode,thetaIndex,UniformBounds);
LogPriorDensityIV = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
%
% Compute forecasts of the observed variables at mode and at initial values
%
StateVarStructureMode = CalculateDSGEStateVariables(thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
StateVarStructureIV = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
%
% take YNaN into account when computing mean forecast errors and mean squared forecast errors
%
if DSGEModel.YNaNs==0;
   T = length(StateVarStructureMode.Y);
   MSEYMode = (1/T)*(StateVarStructureMode.Y-StateVarStructureMode.Yhat)*(StateVarStructureMode.Y-StateVarStructureMode.Yhat)';
   MSEYIV = (1/T)*(StateVarStructureIV.Y-StateVarStructureIV.Yhat)*(StateVarStructureIV.Y-StateVarStructureIV.Yhat)';
   MFEYMode = (1/T)*sum(StateVarStructureMode.Y-StateVarStructureMode.Yhat,2);
   MFEYIV = (1/T)*sum(StateVarStructureIV.Y-StateVarStructureIV.Yhat,2);
else;
   pos = ~isnan(StateVarStructureMode.Y);
   Tvec = sum(pos,2);
   TMatrix = diag(Tvec);
   for i=1:DSGEModel.n-1;
      for j=2:DSGEModel.n;
         TMatrix(i,j) = min(Tvec([i j]));
         TMatrix(j,i) = TMatrix(i,j);
      end;
   end;
   FEMode = zeros(size(pos));
   FEIV = FEMode;
   FEMode(pos) = StateVarStructureMode.Y(pos)-StateVarStructureMode.Yhat(pos);
   FEIV(pos) = StateVarStructureIV.Y(pos)-StateVarStructureIV.Yhat(pos);
   MSEYMode = (ones(DSGEModel.n,DSGEModel.n)./TMatrix).*(FEMode*FEMode');
   MSEYIV = (ones(DSGEModel.n,DSGEModel.n)./TMatrix).*(FEIV*FEIV');
   MFEYMode = (ones(DSGEModel.n,1)./Tvec).*sum(FEMode,2);
   MFEYIV = (ones(DSGEModel.n,1)./Tvec).*sum(FEIV,2);
end;
%
% get the mode values of any additional parameters
%
[thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% Save to disk
%
ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
stat = MakeDir(DSGEModel.OutputDirectory,'mode');
if stat==1;
   SetLogText(strvcat(' ','Saving posterior mode data to disk using directory:',['   "' ModeFileDir '"...']));
   ModeFile = FixFilePath([ModeFileDir '\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
   save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','OptResults','thetaCalibrated','thetaAdditionalInitial','thetaAdditionalMode','ParameterNames','ExtremeCorrParameters','LogPostDensity','LogLikeDensity',...
        'LogPriorDensity','LogJacobianValue','InverseHessian','OldInverseHessian','ThetaPhiPartial','thetaIndex','ParameterBounds','LowerBound','thetaPositions','thetaDist','PriorDist','ModelParameters','UniformBounds',...
        'DSGEModel','AIMData','phiInverseHessianFinDiff','phiPostModeStdFinDiff','thetaInverseHessianFinDiff','thetaPostModeStdFinDiff',...
        'phiFittedStd','thetaFittedStd','KalmanFirstObservation','FirstPeriod','LastPeriod','SmallEigenValues','SmallEigenVectors','LogSPIV','LogSPMode','LogMargLikeSystemLaplace');
end;
%
% write output to file
%
SetLogText(strvcat(' ','Writing posterior mode estimation results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: PosteriorModeResults-' DSGEModel.NameOfModel '.txt']));
PrintPosteriorModeResults(DSGEModel,CurrINI,ParameterNames,phiMode,thetaMode,phi,theta,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,phiInverseHessianFinDiff,phiFittedStd,thetaFittedStd,phiPostModeStdFinDiff,thetaPostModeStdFinDiff,LogPostDensity,LogLikeDensity,LogPriorDensity,LogJacobianValue,ExtremeCorrParameters,SmallEigenValues,SmallEigenVectors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,MaxEigenvalueIV,MaxEigenvalueMode,OptResults,ScreenMessages,LogPosteriorPhiIV,LogPosteriorThetaIV,LogPriorDensityIV,rank(R),MFEYMode,MSEYMode,MFEYIV,MSEYIV,LogSPIV,LogSPMode,IsSystemPrior,LogMargLikeSystemLaplace);
%
% write summary information to file
%
SetLogText(strvcat(' ','Writing posterior mode estimation summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: PosteriorModeSummary-' DSGEModel.NameOfModel '.txt']));
PrintPosteriorModeSummary(DSGEModel,CurrINI,ParameterNames,thetaMode,theta,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,phiInverseHessianFinDiff,phiFittedStd,LogPostDensity,LogLikeDensity,LogPriorDensity,LogJacobianValue,ExtremeCorrParameters,SmallEigenValues,SmallEigenVectors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,max(GradientVector),MaxEigenvalueIV,MaxEigenvalueMode,LogPosteriorPhiIV,LogPosteriorThetaIV,LogPriorDensityIV,rank(R),MFEYMode,MSEYMode,MFEYIV,MSEYIV,LogSPIV,LogSPMode,IsSystemPrior,LogMargLikeSystemLaplace);
SetLogText('All done!');
%
% open one of the results files
%
if isempty(CurrINI.Sequence)==1;
   txt = 'Would you like to view only a summary of the posterior mode estimation output? [No means all printed results]';
   answer = Query(txt,'question',120,'Output Selection',500,'yes',CurrINI);
   drawnow;
   if strcmp(lower(answer),'yes')==1;
      ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeSummary-' DSGEModel.NameOfModel '.txt']);
   else;
      ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeResults-' DSGEModel.NameOfModel '.txt']);
   end;
   TextGUI(ModeTextFile,['Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
else;
   SetLogText(' ');
end;

%
% end of PosteriorModeEstimation.m
%
