function [logPost,logLike] = logPosteriorPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,OrderQZ,NumWorkers)
% logPosteriorPhiDSGE: computes the value of the log posterior at phi when we take
%                              the log of the Jacobian in the transformation theta = g(phi)
%                              into account.
%
% USAGE:
%
%       [logPost,logLike] = logPosteriorPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist, ...
%                                               PriorDist,ModelParameters,DSGEModel,AIMData,OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  phi (vector) with all parameters to be estimated.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: logPost (scalar) the value of the log posterior at phi when we take the log of the Jacobian in the
%                             transformation theta = g(phi) into account, i.e., the log likelihood plus the log prior
%                             of the transformed parameters.
%
% OPTIONAL OUTPUT: logLike (scalar) with the value of the log likelihood.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 8, 2006.
%                        This version: August 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-08-2006: Fixed the logPost value when mcode~=1 or status~=0.
%
% * 25-08-2006: Changed the order of computing the parts logLikeValue, logPrior, and logJac. Furthermore, updating
%               of the additional parameters was moved from logLikelihoodDSGE to logPosteriorPhiDSGEOptimize to make
%               sure that ModelParameters is re-evaluated in one function only. The input argument UniformBounds was
%               also added to allowe for some parameters to have a uniform prior distribution.
%
% * 28-08-2006: Added the test if logPrior is a NaN or not.
%
% * 15-09-2006: Checking parameters. No changes made.
%
% * 22-09-2006: Added the drawnow command to make sure that the screen is refreshed each time the function is called.
%
% * 11-10-2006: Added a test if some of the theta parameters are infinite or NaN's. If so, the output is set to 1000000.
%
% * 24-11-2006: Changed the name from "logPosteriorPhiDSGEOptimize" to "logPosteriorPhiDSGE".
%
% * 15-03-2007: Updated the documentation.
%
% * 29-03-2007: Updated the documentation.
%
% * 12-11-2007: Updated the documentation.
%
% * 11-12-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 23-07-2008: Updated the documentation.
%
% * 21-01-2009: Updated the documentation.
%
% * 13-03-2009: Updated the documentation.
%
% * 02-04-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 21-01-2010: Updated the documentation.
%
% * 11-02-2010: Added the optional output variable "logLike".
%
% * 27-02-2010: Updated the documentation.
%
% * 30-08-2010: Added a test if the log-likelihood value is infinite because of a singular covariance
%               matrix of the 1-step ahead forecast errors for the observed variables.
%
% * 16-12-2010: Added the "OrderQZ" in put variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 27-03-2012: Updated the documentation.
%
% * 26-04-2012: Fixed the sum(isnan(theta)) test.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 01-06-2017: The function now also takes a system prior into account.
%
% * 06-06-2017: Took into account that "logLikelihoodDSGE" now provides the "Solution" output variable.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 29-08-2017: Added the "NumWorkers" input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

drawnow;
if nargout==2;
   logLike = NaN;
end;
%
% transform to theta
%
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
if (sum(isinf(theta))>=1)|(sum(isnan(theta))>=1);
   logPost = 1000000;
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% Calculate the log-likelihood
%
[logLikeValue,Solution,mcode,status] = logLikelihoodDSGE(ModelParameters,DSGEModel,AIMData,OrderQZ,NumWorkers);
%
% add things up and take into account that the function will be minimized
%
if (mcode==1)&(status==0)&(isinf(logLikeValue)==0);
   %
   % calculate the value of the log prior density at theta
   %
   logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
   if isnan(logPrior)==0;
      %
      % calculate the log of the Jacobian
      %
      logJac = logJacobian(phi,thetaIndex,UniformBounds);
      %
      % check if we should compute the system prior
      %
      if FileExist(DSGEModel.SystemPriorFile)==0;
         %
         % compute the function value
         %
         logPost = -(logPrior+logJac+logLikeValue);
         if nargout==2;
            logLike = logLikeValue;
         end;
      else;
         DSGEModel.Solution = Solution;
         eval(['logSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ);']);
         if isnan(logSP)==0;
            %
            % compute the function value
            %
            logPost = -(logPrior+logJac+logLikeValue+logSP);
            if nargout==2;
               logLike = logLikeValue;
            end;
         else;
            logPost = 1000000;
         end;
      end;
   else;
      logPost = 1000000;
   end;
else;
   logPost = 1000000;
end;

%
% end of logPosteriorPhiDSGE.m
%
