function PrintDSGEMomentsResponse(ResFile,MomentsResp,ParameterNames,ParamNumber,OutputType,CurrChain,IsPosterior,NumDraws,TotalDraws,DSGEModel,CurrINI)
% PrintDSGEMomentsResponse: Prints the numerical partial derivatives of the means, standard deviations, and correlations
%                           with respect to a change in one structural parameters using parameter draws from the prior
%                           or the posterior distribution.
%
% USAGE:
%
%       PrintDSGEMomentsResponse(ResFile,MomentsResp,ParameterNames,ParamNumber,OutputType,CurrChain,IsPosterior,...
%                                NumDraws,TotalDraws,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string vector) with the full path+name of the file to be printed.
%
%                 MomentsResp (structure) with the fields "MeanResp", "StdResp" and "CorrResp" containing
%                             the mean responses. For the observed variables the dimensions of these fields
%                             are nxk, nx1 and (n(n-1))/2x1, with k being the number of exogenous variables,
%                             while for the state variables the dimensions are empty, rx1 and (r(r-1))/2x1.
%                             The field "Quantiles" contains the quantile information on these matrices and
%                             vectors in sub-fields as well as the sub-field "percent".
%
%                 ParameterNames (string matrix) with the names of the parameters.
%
%                 ParamNumber (integer) with the position of the parameter in theta to change.
%
%                 OutputType (boolean) that takes the value 1 if responses to the moments should be computed
%                             for the observed variables, and 0 if the responses should be computed for the
%                             state variables.
%
%                 CurrChain (integer) with the number for the current Markov chain.
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                 NumDraws (integer) with total number of parameter draws actually used.
%
%                 TotalDraws (integer) with the total number of draws that could have been used.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 25, 2014.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-03-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if IsPosterior==1;
   EstStr = 'Posterior';
else;
   EstStr = 'Prior';
end;
k = DSGEModel.k;
if OutputType==1;
   VarStr = 'Observed';
   [n,vn] = size(DSGEModel.VariableNames);
   VariableNames = DSGEModel.VariableNames;
else;
   VarStr = 'State';
   [n,vn] = size(DSGEModel.StateVariableNames);
   VariableNames = DSGEModel.StateVariableNames;
end;
%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*    M O M E N T   R E S P O N S E S   T O   A   C H A N G E   I N   O N E     *\n');
fprintf(fid,'* S T R U C T U R A L   P A R A M E T E R   O F   T H E   D S G E   M O D E L  *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% initializing the formatting of output numbers. we usually apply the 12.6f format.
%
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
if isempty(TotalDraws)==0
   fprintf(fid,'Number of available parameter draws: %0.0f\n',TotalDraws);
end;
fprintf(fid,'Number of parameter draws used:      %0.0f\n',NumDraws);
fprintf(fid,'Parameter distribution:              %s\n\n',EstStr);
fprintf(fid,'NOTE: The moment responses are calculated as numerical partial derivatives of the\n');
fprintf(fid,'      (mean,) standard deviation and correlation of the %s variables with \n',lower(VarStr));
fprintf(fid,'      respect to the structural parameter: "%s".\n\n',StringTrim(ParameterNames(ParamNumber,:)));
if vn<20;
   AddNameStr = SpaceStr(20-vn);
else;
   AddNameStr = '';
end;
%
% determine all the quantile data
%
NumBounds = floor(length(MomentsResp.Quantiles)/2);
QuantVec = zeros(NumBounds,1);
if (OutputType==1)&(k>0);
   QuantMean = zeros(NumBounds,2*k,n);
end;
QuantStd = zeros(NumBounds,2,n);
if n>1;
   QuantCorr = zeros(NumBounds,2,(n*(n-1))/2);
end;
for nb=1:NumBounds;
   QuantVec(NumBounds-nb+1,1) = MomentsResp.Quantiles(length(DSGEModel.Percentiles)-nb+1).percent-MomentsResp.Quantiles(nb).percent;
   for i=1:n;
      QuantStd(NumBounds-nb+1,1,i) = MomentsResp.Quantiles(nb).StdResp(i);
      QuantStd(NumBounds-nb+1,2,i) = MomentsResp.Quantiles(length(DSGEModel.Percentiles)-nb+1).StdResp(i);
      if (OutputType==1)&(k>0);
         for j=1:k
            QuantMean(NumBounds-nb+1,1+(2*(j-1)),i) = MomentsResp.Quantiles(nb).MeanResp(i,j);
            QuantMean(NumBounds-nb+1,2*j,i) = MomentsResp.Quantiles(length(DSGEModel.Percentiles)-nb+1).MeanResp(i,j);
         end;
      end;
   end;
   if n>1;
      for i=1:(n*(n-1))/2;
         QuantCorr(NumBounds-nb+1,1,i) = MomentsResp.Quantiles(nb).CorrResp(i);
         QuantCorr(NumBounds-nb+1,2,i) = MomentsResp.Quantiles(length(DSGEModel.Percentiles)-nb+1).CorrResp(i);
      end;
   end;
end;
%
% print the effects on the means
%
if (OutputType==1)&(k>0);
   PrtStr = '%s%s';
   PrtQuantStr = '%3.0f percent         ';
   TopStr = MultiCharStr('=',21+(30*k));
   DivStr = MultiCharStr('-',21+(30*k));
   kn = size(DSGEModel.XVariableNames,2);
   if kn<15;
      AddXVar = SpaceStr(15-kn);
   else;
      AddXVar = '';
   end;
   ConstStr = SpaceStr(20);
   for i=1:k;
      PrtStr = [PrtStr '           ' PrtVal '       '];
      PrtQuantStr = [PrtQuantStr '   [' PrtVal ',' PrtVal ']'];
      ConstStr = [ConstStr '        ' AddXVar DSGEModel.XVariableNames(i,:) '       '];
   end;
   fprintf(fid,[TopStr '\n']);
   fprintf(fid,' Response of the means:\n');
   fprintf(fid,[ConstStr '\n']);
   fprintf(fid,'            Variable \n');
   fprintf(fid,[TopStr '\n']);
   for i=1:n;
      fprintf(fid,[PrtStr '\n'],AddNameStr,VariableNames(i,:),MomentsResp.MeanResp(i,:)');
      for j=1:NumBounds;
         fprintf(fid,[PrtQuantStr '\n'],QuantVec(j),QuantMean(j,:,i)');
      end;
      if i<n;
         fprintf(fid,[DivStr '\n']);
      else;
         fprintf(fid,[TopStr '\n\n']);
      end;
   end;
end;
%
% print the partial derivatives of the standard deviations
%
PrtStr = ['%s%s           ' PrtVal ' \n'];
PrtQuantStr = ['%3.0f percent            [' PrtVal ',' PrtVal '] \n'];
TopStr = MultiCharStr('=',51);
DivStr = MultiCharStr('-',51);
fprintf(fid,[TopStr '\n']);
fprintf(fid,' Responses of the standard devs:\n');
fprintf(fid,'            Variable \n');
fprintf(fid,[TopStr '\n']);
for i=1:n;
   fprintf(fid,PrtStr,AddNameStr,VariableNames(i,:),MomentsResp.StdResp(i));
   for j=1:NumBounds;
      fprintf(fid,PrtQuantStr,QuantVec(j),QuantStd(j,:,i)');
   end;
   if i<n;
      fprintf(fid,[DivStr '\n']);
   else;
      fprintf(fid,[TopStr '\n\n']);
   end;
end;
%
% print the partial derivatives of the correlations
%
if n>1;
   VarStrMat = '';
   for j=1:n-1;
      for i=j+1:n;
         VarStrMat = strvcat(VarStrMat,[VariableNames(j,:) ' ' VariableNames(i,:)]);
      end;
   end;
   PrtStr = ['%s%s%s           ' PrtVal ' \n'];
   PrtQuantStr = ['%3.0f percent                                 [' PrtVal ',' PrtVal '] \n'];
   TopStr = MultiCharStr('=',72);
   DivStr = MultiCharStr('-',72);
   fprintf(fid,[TopStr '\n']);
   fprintf(fid,' Responses of the correlations:\n');
   fprintf(fid,'          Variable 1/Variable 2\n');
   fprintf(fid,[TopStr '\n']);
   for i=1:(n*(n-1))/2;
      fprintf(fid,PrtStr,AddNameStr,VarStrMat(i,:),AddNameStr,MomentsResp.CorrResp(i));
      for j=1:NumBounds;
         fprintf(fid,PrtQuantStr,QuantVec(j),QuantCorr(j,:,i)');
      end;
      if i<(n*(n-1))/2;
         fprintf(fid,[DivStr '\n']);
      else;
         fprintf(fid,[TopStr '\n\n']);
      end;
   end;
end;
%
% bottom file data
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEMomentsResponse.m
%
