function PrintStateConditionalCVD(ResFile,DSGEModel,CurrINI,CVDs,PrintGroups,NumDraws,TotalDraws,CurrChain,IsPosterior)
% PrintStateConditionalCVD: Writes the conditional forecast error variance decompositions for the state
%                           variables to a text file.
%
% USAGE:
%
%       PrintStateConditionalCVD(DSGEModel,CurrINI,CVDs,PrintGroups,NumDraws,TotalDraws,CurrChain,IsPosterior)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 CVDs (structure) with fields FEVD (posterior mean), Quantiles(j).FEVD,
%                           and Quantiles(j).percent with the quantile and percent values
%                           of the posterior distribution.
%
%                 PrintGroups (boolean) which determines if shocks groups should be printed (1) or not (0).
%
%                 NumDraws (integer) with the number of draws from the posterior distribution
%                           of the DSGE model parameters that were used in the computations.
%
%                 TotalDraws (integer) with the total number of draws from the posterior
%                           distribution of the DSGE model parameters that could have been
%                           used for the computations.
%
%                 CurrChain (integer) with the value of the current MCMC chain.
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 5, 2017.
%                        This version: May 5, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*    C O N D I T I O N A L   V A R I A N C E   D E C O M P O S I T I O N S     *\n');
fprintf(fid,'*                F O R   T H E   S T A T E   V A R I A B L E S                 *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Number of parameter draws used:      %0.0f \n',NumDraws);
if IsPosterior==1;
   fprintf(fid,'Number of available parameter draws: %0.0f \n',TotalDraws);
   fprintf(fid,'Current MCMC chain:                  %0.0f \n\n',CurrChain);
else;
   fprintf(fid,'\n');
end;
fprintf(fid,'NOTE: The conditional forecast error variance decomposition neglect the initial\n      state variable uncertainty. They may therefore differ from the forecast\n      error variance decompositions.\n\n');
%
if IsPosterior==1;
   fprintf(fid,'      Posterior mean values of conditional variance decompositions for the state\n      variables along with quantiles.\n\n');
else;
   fprintf(fid,'      Prior mean values of conditional variance decompositions for the state\n      variables along with quantiles.\n\n');
end;
%
[n,q,h] = size(CVDs.FEVD);
vq = size(DSGEModel.VariableNames,2);
if PrintGroups==0;
   NamesMatrix = DSGEModel.StateShockNames;
else;
   NamesMatrix = char(zeros(size(DSGEModel.ShockGroupNames)));
   for i=1:q;
      NamesMatrix(i,:) = strrep(DSGEModel.ShockGroupNames(i,:),'_',' ');
   end;
end;
NamesMatrix = strvcat(NamesMatrix,'Sum');
vn = size(NamesMatrix,2);
if vn<15;
   AddNameStr = SpaceStr(15-vn);
else;
   AddNameStr = '';
end;
if vq<20;
   AddShockStr = SpaceStr(20-vq);
else;
   AddShockStr = '';
end;
NameStr = '                    ';
MEStr = '';
PrtStr = '%s';
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
QPrtStr = '       %4.1f quantile   ';
CPrtStr = '     %2.1f%% confidence      ';
CPrtVal = '%0.5f';
for i=1:q+1;
   NameStr = [NameStr '   ' AddNameStr NamesMatrix(i,:)];
   PrtStr = [PrtStr '      ' PrtVal];
   if i<=q;
      QPrtStr = [QPrtStr '      ' PrtVal];
      CPrtStr = [CPrtStr ' [' CPrtVal ',' CPrtVal ']'];
   end;
end;
%
for i=1:n;
   MEStr = strvcat(MEStr,sprintf('%s%s',AddShockStr,DSGEModel.StateVariableNames(i,:)));
end;
if DSGEModel.ConfidenceRegionMethod==1;
   FirstStr = MultiCharStr('=',length(NameStr)+4);
   DivStr = MultiCharStr('-',length(NameStr)+4);
   MeanStr = MultiCharStr('',length(NameStr)+4);
else;
   FirstStr = MultiCharStr('=',length(NameStr)+5);
   DivStr = MultiCharStr('-',length(NameStr)+5);
   MeanStr = MultiCharStr('',length(NameStr)+5);
end;
%
% check if we should relate the shocks to shock groups
%
if PrintGroups==1;
   for i=1:q;
      iGroup = (DSGEModel.ShockGroups==i) .* (1:length(DSGEModel.ShockGroups));
      iGroup = iGroup(iGroup>0);
      ShockNames = StrWrap(vecstr(DSGEModel.StateShockNames(iGroup,:),' '),60);
      GroupName = StringTrim(NamesMatrix(i,:));
      if size(GroupName,2)<25;
         AddSpace = SpaceStr(25-size(GroupName,2));
      else;
         AddSpace = '';
      end;
      fprintf(fid,'%s:%s %s\n',GroupName,AddSpace,ShockNames(1,:));
      for j=2:size(ShockNames,1);
         if strcmp(StringTrim(ShockNames(j,:)),'')==0;
            fprintf(fid,'%s%s\n',SpaceStr(27),ShockNames(j,:));
         end;
      end;
   end;
   fprintf(fid,'\n');
end;
%
% Write to output for horizon h
%
for j=1:h;
   fprintf(fid,'%s\n',FirstStr);
   if j<h;
      fprintf(fid,'Forecast Horizon: %3.0f\n',j);
   else;
      fprintf(fid,'Forecast Horizon: Long run\n');
   end;
   fprintf(fid,'%s\n',NameStr);
   for i=1:n;
      fprintf(fid,'%s\n',DivStr);
      fprintf(fid,[PrtStr '\n'],MEStr(i,:),[abs(CVDs.FEVD(i,:,j)) sum(abs(CVDs.FEVD(i,:,j)))]');
      fprintf(fid,'%s\n',MeanStr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for k=1:length(CVDs.Quantiles);
            fprintf(fid,[QPrtStr '\n'],CVDs.Quantiles(k).percent,abs(CVDs.Quantiles(k).FEVD(i,:,j))');
         end;
      else;
         for k=(length(CVDs.Quantiles)/2:-1:1);
            fprintf(fid,[CPrtStr '\n'],CVDs.Quantiles(length(CVDs.Quantiles)-k+1).percent-CVDs.Quantiles(k).percent,vec([abs(CVDs.Quantiles(k).FEVD(i,:,j));abs(CVDs.Quantiles(length(CVDs.Quantiles)-k+1).FEVD(i,:,j))]));
         end;
      end;
   end;
   fprintf(fid,'%s\n\n',FirstStr);
end;
%
% we're all done now
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintStateConditionalCVD.m
%
