function [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,ParameterPositions,AIMData,controls,DSGEModel,CurrINI)
% ComputeSystemPriorMode: Estimates the prior mode for the system prior and writen the results to disk.
%
% USAGE:
%
%       [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(theta,thetaIndex,thetaDist,LowerBound,...
%                                                           ModelParameters,thetaPositions,PriorDist,ParameterNames,...
%                                                           UniformBounds,ParameterPositions,AIMData,controls,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector), with initial values of the parameters to be estimated.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                             of the model parameter in position i of theta. The field "distribution" has a string
%                             vector giving the assumed prior distribution of the parameter.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                             "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each field returns
%                             a string matrix with the parameter names.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                             for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  ParameterPositions (vector) of dimension 1x4 with the entries:
%
%                                    [firstUpdatePosition lastUpdatePosition firstInitializePosition lastInitializePosition]
%
%                  AIMData (structure) with data collected by AIM about the model.
%
%                  controls (structure)  whose fields provide handles to all the UI
%                             controls in YADA's main dialog.
%
%                  DSGEModel (structure) (structure) whose fields contain all required information about the model, such
%                             as input files, variable and sample data.
%
%                  CurrINI (structure with initialization information.
%
% REQUIRED OUTPUT: SystemPriorStatus (boolean) that takes the value 1 if the estimation was successful and 0 otherwise.
%
%                  LogMargLikeSystemLaplace (scalar) with the log marginal likelihood of the system prior using the
%                             Laplace approximation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 1, 2017.
%                        This version: June 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-06-2017: Updated the documentation.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SystemPriorStatus = 0;
LogMargLikeSystemLaplace = [];
%
SetLogText(strvcat('   STATUS: NO',' * Checking the marginal log prior...'));
%
% transform the original parameters to the transformed ;)
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
NumParam = length(theta);
%
% fist we must check the marginal prior
%
LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
LogThetaPriorIV = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
%
% check if the log prior of the transformed parameters can be computed
%
if (isnan(LogJacobianIV)==1)|(isnan(LogThetaPriorIV)==1);
   ErrorStr = ['The marginal log prior of the original parameters is ' num2str(LogThetaPriorIV) ', while the log Jacobian is ' num2str(LogJacobianIV) '. YADA is therefore unable to compute the marginal log prior of the transformed parameters.'];
   SetLogText(strvcat('   STATUS: The marginal log prior is not a number.',' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error','Error - Log Prior',400,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
SetLogText(strvcat('   STATUS: OK',' * Checking the speed of computing the full system log prior...'));
%
% check the parameter target type
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   OptParameters = 'Transformed';
else;
   OptParameters = 'Original';
end;
%
% Collect additional parameters in ParameterNames.additional
%
ParameterNames = ReadAdditionalParameterNames(ParameterNames,ModelParameters,ModelParameters,ParameterPositions);
%
% set up vectors with calibrated parameters and additional parameters. These are only used for printing to the output file
%
[thetaCalibrated,thetaAdditionalInitial] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% get the upper and lower bounds of the parameters
%
ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
ParameterBounds = ParameterBounds(:,1:2);
tic;
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   logPrior = logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
else;
   logPrior = logSystemPriorThetaDSGE(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
   logPrior = logPrior-LogJacobianIV;
end;
LogPriorTime = toc;
%
SetLogText(strvcat(['   Computation time:       ' num2str(LogPriorTime) ' seconds'],' * Checking the numerical gradient of the full system log prior'));
%
% Compute a numerical gradient
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   [g,badg] = YADAnumgrad('logSystemPriorPhiDSGE',phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
else;
   [g,badg] = YADAnumgrad('logSystemPriorThetaDSGE',theta,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
end;
%
if badg~=0;
   SetLogText(strvcat('   STATUS: Bad gradient of log prior at initial values',' '));
   [badpar,maxpos] = min(abs(g));
   %
   if badpar>0;
      ValueStr = num2str(badpar);
   else;
      ValueStr = 'NaN';
   end;
   %
   ParamStr = ['"' StringTrim(ParameterNames.estimated(maxpos(1),:)) '"'];
   AddParamStr = ' ';
   if length(maxpos)>=2;
      AddParamStr = 's ';
      for i=2:length(maxpos);
         ParamStr = [ParamStr ', "' StringTrim(ParameterNames.estimated(maxpos(i),:)) '"'];
      end;
   end;
   %
   if isempty(CurrINI.Sequence)==1;
      txt = ['Unable to compute a meaningful numerical gradient of the log posterior at the initial values. The largest absolute value or an improper value (NaN or Inf) of the gradient is found for the parameter' AddParamStr ParamStr ', with value of ' ValueStr '. You probably need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% We use Sims' csminwel routine with transformed parameters.
%
OptAlgorithm = 'csminwel';
%
eval(['logSP = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
LogSystemPriorIV = logSP;
%
% compute the initial inverse Hessian
%
if DSGEModel.InitializeHessian==1;
   InitialInvHessMat = 10*eye(NumParam);
elseif DSGEModel.InitializeHessian==2;
   InitialInvHessMat = 1*eye(NumParam);
elseif DSGEModel.InitializeHessian==3;
   InitialInvHessMat = 10^(-1)*eye(NumParam);
elseif DSGEModel.InitializeHessian==4;
   InitialInvHessMat = 10^(-2)*eye(NumParam);
elseif DSGEModel.InitializeHessian==5;
   InitialInvHessMat = 10^(-3)*eye(NumParam);
elseif DSGEModel.InitializeHessian==6;
   InitialInvHessMat = 10^(-4)*eye(NumParam);
elseif DSGEModel.InitializeHessian==7;
   InitialInvHessMat = 10^(-5)*eye(NumParam);
elseif DSGEModel.InitializeHessian==8;
   %
   % sample from the prior
   %
   InitialInvHessMat = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
end;
%
% make sure that we have the R matrix from the measurement equations to determine the number of measurement errors
%
eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(DSGEModel.StateVariableNames,1),DSGEModel.k);']);
%
% The model has a unique solution for the initial values
%
MaxIterStr = get(controls.optimize.maximumiterations,'String');
MaxIterStr = StringTrim(strrep(MaxIterStr(DSGEModel.OptMaxIterationsValue,:),',',''));
MaxIter = str2num(MaxIterStr);
%
ToleranceStr = get(controls.optimize.tolerance,'String');
ToleranceStr = StringTrim(ToleranceStr(DSGEModel.OptToleranceValue,:));
ToleranceValue = str2num(ToleranceStr);
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   SetLogText(strvcat('   STATUS: OK',['   Log system prior value: ' num2str(LogSystemPriorIV,'%0.4f')],['   Log theta prior value:  ' num2str(LogThetaPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],['   Total log prior value:  ' num2str(LogSystemPriorIV+LogThetaPriorIV+LogJacobianIV,'%0.4f')],' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' int2str(NumParam)],[' * Total number of parameters:       ' int2str(size(ParameterNames.all,1))],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   try;
      tic;
      [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logSystemPriorPhiDSGE',phi,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Prior Mode Estimation - csminwel',CurrINI, ...
            thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
      GradientVector = abs(gh);
      ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
      MaxTime = toc;
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         WaitControls = get(WaitHandle,'UserData');
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
   end;
else;
   SetLogText(strvcat('   STATUS: OK',['   Log system prior value: ' num2str(LogSystemPriorIV,'%0.4f')],['   Log theta prior value:  ' num2str(LogThetaPriorIV,'%0.4f')],['   Total log prior value:  ' num2str(LogSystemPriorIV+LogThetaPriorIV,'%0.4f')],' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' int2str(NumParam)],[' * Total number of parameters:       ' int2str(size(ParameterNames.all,1))],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')]));
   try;
      tic;
      [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logSystemPriorThetaDSGE',theta,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Prior Mode Estimation - csminwel',CurrINI, ...
            thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
      %
      % make sure that the outputs are transformed into phi-space
      %
      phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
      phiEstimates = zeros(size(thetaEstimates));
      for i=1:size(thetaEstimates,2);
         phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
      end;
      ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
      InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
      GradientVector = abs(gh);
      MaxTime = toc;
   catch;
      if DSGEModel.ShowProgress==1;
         WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         WaitHandle = findobj('Type','figure','Tag','WaitDLG');
         WaitControls = get(WaitHandle,'UserData');
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
      ReturnCode = -1;
      AbortEstimation = '';
      NumIter = [];
      GradientVector = [];
      ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' lasterr];
   end;
end;
if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
   %
   % we're OK
   %
   thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
   LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
   LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      LogJointPriorDensity = -fh;
   else;
      LogJointPriorDensity = -fh+LogJacobianMode;
   end;
   LogSystemPriorMode = LogJointPriorDensity-(LogPriorMode+LogJacobianMode);
   %
   SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Total log prior of phi at mode:   ' num2str(LogJointPriorDensity,'%0.4f')],[' * Total log prior of theta at mode: ' num2str(LogJointPriorDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],[' * Log system prior at mode:         ' num2str(LogSystemPriorMode,'%0.4f')]));
   EstimationStatus = 1;
elseif strcmp(lower(AbortEstimation),'cancel')==1;
   %
   % posteriod mode estimation was aborted
   %
   SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   return;
elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
   %
   % maximum number of iterations was reached
   %
   txt = ['The maximum number of iterations was reached without csminwel converging. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Maximum number of iteration reached.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','Optimization Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif ReturnCode==1;
   txt = ['Csminwel return code: zero gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: zero gradient.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','Optimization Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif (ReturnCode==2)|(ReturnCode==4);
   txt = ['Csminwel return code: back and forth on step length never finished. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: back and forth on step length never finished.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','Optimization Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif ReturnCode==3;
   txt = ['Csminwel return code: smallest step still improving too slow. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow.'));
   About(txt,'information','Optimization Problem',200,500,CurrINI);
   drawnow;
   return;
elseif ReturnCode==5;
   txt = ['Csminwel return code: largest step still improving too fast. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: largest step still improving too fast.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','Optimization Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif ReturnCode==6;
   txt = ['Csminwel return code: smallest step still improving too slow, reversed gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow, reversed gradient.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','Optimization Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif ReturnCode==7;
   txt = ['Csminwel return code: possible inaccuracy in inverse Hessian. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
   SetLogText(strvcat(' ','Csminwel return code: possible inaccuracy in inverse Hessian.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','Optimization Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
elseif ReturnCode==-1;
   SetLogText(strvcat(' ','Csminwel error: optimization ended abruptly.'));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','Optimization Problem',400,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
SetLogText(strvcat(' ','Computing the inverse Hessian with finite differences...'));
%
% compute the finite difference estimate of the inverse Hessian
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   %
   % for the transformed parameters
   %
   VechHessian = YADAHessianMatrix('logSystemPriorPhiDSGE',phiMode,StepLength,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   InverseHessianFinDiff = HessianMat\eye(NumParam);
else;
   %
   % for the original parameters
   %
   VechHessian = YADAHessianMatrix('logSystemPriorThetaDSGE',thetaMode,StepLength,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   InverseHessianFinDiff = HessianMat\eye(NumParam);
   InverseHessianFinDiff = (ThetaPhiPartial\InverseHessianFinDiff)/ThetaPhiPartial;
end;
%
% compute the Laplace approximation
%
phi = phiMode;
if IsPositiveDefinite(InverseHessianFinDiff)==1;
   LogMargLikeSystemLaplace = MargLikeLaplace(LogJointPriorDensity,InverseHessianFinDiff,1);
else;
   LogMargLikeSystemLaplace = MargLikeLaplace(LogJointPriorDensity,InverseHessian,1);
end;
%
SetLogText(strvcat([' * Log sys prior marg like, Laplace: ' num2str(LogMargLikeSystemLaplace,'%0.4f')],' ','Checking the optimum...',' * Transformed parameters'));
%
% Check if GraphDirectory exists
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
MinMaxGrid = zeros(NumParam,2);
stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
%
% create lower and upper bounds from InverseHessian and phiMode
%
MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
phiFittedStd = VillaniCheckOptimum('logSystemPriorPhiDSGE',phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Prior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',0,0, ...
   thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
%
SetLogText(' * Original parameters');
MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),thetaIndex,UniformBounds,LowerBound);
MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),thetaIndex,UniformBounds,LowerBound);
thetaFittedStd = VillaniCheckOptimum('logSystemPriorThetaDSGE',thetaMode,[],MinMaxGrid,ThetaPhiPartial*InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Prior Mode - Original Parameters',CurrINI,GraphDirectory,'theta','','',0,0, ...
   thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,ParameterBounds,LowerBound,AIMData,DSGEModel,CurrINI);
%
% now we can save the results to the prior mode file
%
ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
stat = MakeDir(DSGEModel.OutputDirectory,'mode');
if stat==1;
   SetLogText(strvcat(' ','Saving prior mode data to disk using directory:',['   "' ModeFileDir '"...'],' ','System prior mode estimation completed.'));
   ModeFile = FixFilePath([ModeFileDir '\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   save(ModeFile,'phi','LogMargLikeSystemLaplace','InverseHessian','InverseHessianFinDiff','phiFittedStd','thetaFittedStd','R','theta','thetaMode','ParameterNames','thetaDist','PriorDist','LowerBound','thetaPositions','ModelParameters','thetaIndex','UniformBounds');
else;
   SetLogText(strvcat(' ','Unable to create the directory:',['   "' ModeFileDir '"...'],' ','System prior mode results have not been saved to disk.'));
end;
%
% we're done and have been successful
%
SystemPriorStatus = 1;

%
% end of ComputeSystemPriorMode.m
%
