function ProjectFunctions(selector)
% ProjectFunctions: switch function for running the hypothetical example
%                   function on the project menu.
%
% USAGE:
%
%       ProjectFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 9, 2009.
%                        This version: September 4, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-07-2009: Finished the example function.
%
% * 08-07-2009: Updated the documentation.
%
% * 18-09-2009: Made sure that "lasterr" is reset to an empty string.
%
% * 25-09-2009: The function now displays the error message caught when attempting to load
%               posterior mode values under initial values and loading failed.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 23-08-2017: Took into account that LoadDSGEPosteriorDraws now provides the output variable "Weights".
%
% * 04-09-2017: Ensure that Weights is used properly by the various functions.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
lasterr = '';
%
switch selector
   case 'spatdist_init_values'
      %
      % initial parameter values
      %
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            txt = 'Are you sure you want to compute spatial distortions before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Spatial Distortions - Initial Values',500,'no',CurrINI);
         end;
         if strcmp(lower(answer),'yes')==1;
            %
            % user is willing to run spatial distortions
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
                  else;
                     About(ErrorStr,'error','Spatial Distortions - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
            if InitStatus==1;
               %
               % we can now try to compute the spatial distortions
               %
               [SpatDist,status,kalmanstatus] = DSGESpatialDistortionsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
               if isempty(SpatDist)==0;
                  %
                  % check if user wants to save results to disk
                  %
                  txt = 'Would you like to save the spatial distortions at initial values to file?';
                  answer = Query(txt,'question',140,'Save - Spatial Distortions',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     YNames = DSGEModel.VariableNames;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\SpatialDistortions-' DSGEModel.NameOfModel '-InitialValues.mat']);
                     save(file,'YNames','StateVariableNames','SpatDist');
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SpatDist' DSGEModel.NameOfModel 'InitVal.m']);
                     fid = fopen(mfile,'wt');
                     %
                     % write code for m-file
                     %
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStruc = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStruc.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStruc.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% matrix with spatial distortions results\n%%\n');
                     fprintf(fid,'SpatDist = DataStruc.SpatDist;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     % display a dialog with information
                     %
                     txt = ['The spatial distortions at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), and SpatDist (matrix with spatial distortions). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Spatial Distortions',200,500,CurrINI);
                  end;
                  %
                  % write output fo file
                  %
                  PrintSpatialDistortions(DSGEModel,CurrINI,SpatDist,[],1,1,1,0);
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-InitialValues.txt']);
                  %
                  if FileExist(file)==1;
                     TextGUI(file,'Spatial Distortions',[(CurrINI.scrsz(3)-min(1000,CurrINI.scrsz(3)))/2 32 min(1000,CurrINI.scrsz(3)) CurrINI.scrsz(4)-100],'Initial Values',100,CurrINI,1,0);
                  end;
                  %
                  % view the results
                  %
                  SpatialDistortionsDLG('init',DSGEModel,CurrINI,SpatDist,'Initial Values');
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'spatdist_prior_dist'
      %
      % prior distribution values
      %
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Spatial Distortions - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ParameterNames = ModeData.ParameterNames;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'error','Spatial Distortions - Prior',200,500,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % set default choice of parameters that can vary across draws
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               DSGEModel.ScenarioParameters = ones(1,length(theta));
            end;
            positions = DSGEModel.ScenarioParameters .* (1:length(theta));
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','Spatial Distortions','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               DSGEModel.ScenarioParameters = zeros(1,length(theta));
               DSGEModel.ScenarioParameters(positions) = ones(1,length(positions));
               %
               % continue with the next step
               %
               if SpatDistExist(DSGEModel,DSGEModel.ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0)==1;
                  txt = 'YADA has located results for the spatial distortions on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Spatial Distortions',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  DoneCalc = DSGESpatialDistortions(theta,thetaDraws',thetaPositions,ModelParameters,DSGEModel.ScenarioParameters,0,NumPriorDraws,1,DSGEModel,CurrINI);
               end;
               if DoneCalc==1;
                  %
                  % compute the marginal distributions for prior
                  %
                  SpatDist = DSGESpatDistDistribution(DSGEModel,CurrINI,DSGEModel.ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                  %
                  % check if user wants to save results to disk
                  %
                  txt = 'Would you like to save the mean, the quantiles, and the shortest confidence band of the spatial distortions to file?';
                  answer = Query(txt,'question',140,'Save - Spatial Distortions',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(DSGEModel.ScenarioParameters),' ','');
                     YNames = DSGEModel.VariableNames;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\SpatialDPriorDist-' DSGEModel.NameOfModel '-' SelParamStr '.' int2str(NumDraws) '.mat']);
                     save(file,'YNames','StateVariableNames','SpatDist');
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SDPriorDist' DSGEModel.NameOfModel SelParamStr int2str(NumDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     %
                     % write code for m-file
                     %
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStruc = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStruc.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStruc.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% structure with spatial distortions results\n%%\n');
                     fprintf(fid,'SpatDist = DataStruc.SpatDist;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     % display a dialog with information
                     %
                     txt = ['The spatial distortions data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), and SpatDist (structure with data on mean, quantiles, and shortest confidence bands). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Spatial Distortions',200,500,CurrINI);
                  end;
                  %
                  % write output fo file
                  %
                  PrintSpatialDistortions(DSGEModel,CurrINI,SpatDist,DSGEModel.ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0);
                  %
                  SelParamStr = strrep(num2str(SelectedParameters),' ','');
                  file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(size(thetaPostSample,1)) '.' int2str(NumDraws) '.txt']);
                  %
                  if FileExist(file)==1;
                     TextGUI(file,'Spatial Distortions',[(CurrINI.scrsz(3)-min(1000,CurrINI.scrsz(3)))/2 32 min(1000,CurrINI.scrsz(3)) CurrINI.scrsz(4)-100],'Prior Distribution',100,CurrINI,1,0);
                  end;
                  %
                  % view the results
                  %
                  SpatialDistortionsDLG('init',DSGEModel,CurrINI,SpatDist,'Prior Distribution');
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the spatial distortions distribution.';
               About(txt,'information','Spatial Distortions',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'spatdist_post_mode'
      %
      % posterior mode values
      %
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % we can now try to compute spatial distortions
            %
            SpatDist = DSGESpatialDistortionsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
            if isempty(SpatDist)==0;
               %
               % check if user wants to save results to disk
               %
               txt = 'Would you like to save the spatial distortions at posterior mode values to file?';
               answer = Query(txt,'question',140,'Save - Spatial Distortions',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  YNames = DSGEModel.VariableNames;
                  StateVariableNames = DSGEModel.StateVariableNames;
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\SpatialDistortions-' DSGEModel.NameOfModel '-PosteriorMode.mat']);
                  save(file,'YNames','StateVariableNames','SpatDist');
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\SpatDist' DSGEModel.NameOfModel 'PostMode.m']);
                  fid = fopen(mfile,'wt');
                  %
                  % write code for m-file
                  %
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStruc = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStruc.YNames;\n');
                  fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                  fprintf(fid,'StateVariableNames = DataStruc.StateVariableNames;\n');
                  fprintf(fid,'%%\n%% matrix with spatial distortions results\n%%\n');
                  fprintf(fid,'SpatDist = DataStruc.SpatDist;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  % display a dialog with information
                  %
                  txt = ['The spatial distortions at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), and SpatDist (matrix with spatial distortions). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Spatial Distortions',200,500,CurrINI);
               end;
               %
               % write output fo file
               %
               PrintSpatialDistortions(DSGEModel,CurrINI,SpatDist,[],1,1,1,1);
               %
               file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-PosteriorMode.txt']);
               %
               if FileExist(file)==1;
                  TextGUI(file,'Spatial Distortions',[(CurrINI.scrsz(3)-min(1000,CurrINI.scrsz(3)))/2 32 min(1000,CurrINI.scrsz(3)) CurrINI.scrsz(4)-100],'Posterior Mode',100,CurrINI,1,0);
               end;
               %
               % view the results
               %
               SpatialDistortionsDLG('init',DSGEModel,CurrINI,SpatDist,'Posterior Mode');
            end;
         else;
            About(ErrorStr,'error','Spatial Distortions - Posterior Mode',200,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'spatdist_post_dist'
      %
      % posterior distribution values
      %
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % prepare for using draws from posterior
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            if NumChains>1;
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Spatial Distortions','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the posterior draws exist
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  drawnow;
                  %
                  % assume that DSGEModel.PostDrawsPercentValue determines the number of
                  % posterior draws
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                     end;
                  end;
                  %
                  % set default choice of parameters that can vary across draws
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     DSGEModel.ScenarioParameters = ones(1,length(theta));
                  end;
                  positions = DSGEModel.ScenarioParameters .* (1:length(theta));
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','Spatial Distortions','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     DSGEModel.ScenarioParameters = zeros(1,length(theta));
                     DSGEModel.ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % continue with the next step
                     %
                     if SpatDistExist(DSGEModel,DSGEModel.ScenarioParameters,size(thetaPostSample,1),NumDraws,CurrChain,1)==1;
                        txt = 'YADA has located results for the spatial distortions on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Spatial Distortions',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = DSGESpatialDistortions(theta,thetaPostSample,thetaPositions,ModelParameters,DSGEModel.ScenarioParameters,1,NumDraws,CurrChain,DSGEModel,CurrINI);
                     end;
                     if DoneCalc==1;
                        %
                        % compute the marginal distributions for posterior
                        %
                        SpatDist = DSGESpatDistDistribution(DSGEModel,CurrINI,DSGEModel.ScenarioParameters,size(thetaPostSample,1),NumDraws,CurrChain,1,Weights);
                        %
                        % check if user wants to save results to disk
                        %
                        txt = 'Would you like to save the mean, the quantiles, and the shortest confidence band of the spatial distortions to file?';
                        answer = Query(txt,'question',140,'Save - Spatial Distortions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(DSGEModel.ScenarioParameters),' ','');
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\SpatialDDist-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
                           save(file,'YNames','StateVariableNames','SpatDist');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SDDist' DSGEModel.NameOfModel int2str(CurrChain) SelParamStr int2str(NumDraws) int2str(TotalDraws) '.m']);
                           fid = fopen(mfile,'wt');
                           %
                           % write code for m-file
                           %
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStruc = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStruc.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStruc.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% structure with spatial distortions results\n%%\n');
                           fprintf(fid,'SpatDist = DataStruc.SpatDist;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           % display a dialog with information
                           %
                           txt = ['The spatial distortions data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), and SpatDist (structure with data on mean, quantiles, and shortest confidence bands). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Spatial Distortions',200,500,CurrINI);
                        end;
                        %
                        % write output fo file
                        %
                        PrintSpatialDistortions(DSGEModel,CurrINI,SpatDist,DSGEModel.ScenarioParameters,size(thetaPostSample,1),NumDraws,CurrChain,1);
                        %
                        SelParamStr = strrep(num2str(SelectedParameters),' ','');
                        file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(size(thetaPostSample,1)) '.' int2str(NumDraws) '.txt']);
                        %
                        if FileExist(file)==1;
                           TextGUI(file,'Spatial Distortions',[(CurrINI.scrsz(3)-min(1000,CurrINI.scrsz(3)))/2 32 min(1000,CurrINI.scrsz(3)) CurrINI.scrsz(4)-100],'Posterior Distribution',100,CurrINI,1,0);
                        end;
                        %
                        % view the results
                        %
                        SpatialDistortionsDLG('init',DSGEModel,CurrINI,SpatDist,'Posterior Distribution');
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the spatial distortions distribution.';
                     About(txt,'information','Spatial Distortions',150,500,CurrINI);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'error','Spatial Distortions - Posterior',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
end
%
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);
VerifyProjectMenu(DSGEModel,controls);

%
% end of ProjectFunctions.m
%
