function [Indexes,NK] = ResidualResampling(Weights,N)
% ResidualResampling: Computes the indexes of the resampled particles for SMC using residual resampling
%                     based on the original normalized weights.
%
% USAGE:
%
%       Indexes = ResidualResampling(Weights,N)
%
% REQUIRED INPUT:  Weights (vector) of dimension N with the normalized weights.
%
% OPTIONAL INPUT:  N (integer) with the number of weights to consider. Could be less than the
%                    length of Weights, but defaults to this when missing.
%
% REQUIRED OUTPUT: Indexed (vector) of dimension N with the positions of the resamples particles.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 17, 2017.
%                        This version: August 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-08-2017: Added the optional input variable N.
%
% * 21-08-2017: Added the output variable NK.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==1;
   N = length(Weights);
end;
Indexes = zeros(N,1);
%
% allocate Ns copies of index s for s=1,...,N
%
nc = floor(Weights);
NC = cumsum(nc);
K = NC(N);
%
% indexes of particles to copy
%
k = 1;
for i=1:K;
   while NC(k)<i;
      k = k+1;
   end;
   Indexes(i) = k;
end;
%
% compute the additional number of indexes with stratified resampling
%
WeightsB = Weights-nc;
NK = N-K;
Indexes(K+1:N) = StratifiedResampling(WeightsB,NK);

%
% end of ResidualResampling.m
%
