function MomentsResp = DSGEMomentsResponseDistribution(ParameterNames,ParamNumber,OutputType,CurrChain,IsPosterior,Weights,NumDraws,TotalDraws,DSGEModel,CurrINI)
% DSGEMomentsResponseDistribution: Computes the mean and the quantiles from the distribution of the effects on the means,
%                                  standard deviations and correlations of the observed or state variables from a change
%                                  in a structural parameter.
%
% USAGE:
%
%       MomentsResp = DSGEMomentsResponseDistribution(ParameterNames,ParamNumber,OutputType,CurrChain,IsPosterior,...
%                                                     Weights,NumDraws,TotalDraws,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  ParameterNames (string matrix) with the names of the parameters.
%
%                  ParamNumber (integer) with the position of the parameter in theta to change.
%
%                  OutputType (boolean) that takes the value 1 if responses to the moments should be computed
%                             for the observed variables, and 0 if the responses should be computed for the
%                             state variables.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                             likelihood tempering posterior sampler.
%
%                  NumDraws (integer) with total number of parameter draws actually used.
%
%                  TotalDraws (integer) with the total number of draws that could have been used.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: MomentsResp (structure) with the fields "MeanResp", "StdResp" and "CorrResp" containing
%                             the mean responses. For the observed variables the dimensions of these fields
%                             are nxk, nx1 and (n(n-1))/2x1, with k being the number of exogenous variables,
%                             while for the state variables the dimensions are empty, rx1 and (r(r-1))/2x1.
%                             The field "Quantiles" contains the quantile information on these matrices and
%                             vectors in sub-fields as well as the sub-field "percent".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 25, 2014.
%                        This version: September 4, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-03-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 24-08-2017: Added the Weights input variable.
%
% * 04-09-2017: Speeded up the Weights computations.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
MomentsResp = [];
ParamStr = StringTrim(ParameterNames(ParamNumber,:));
SaveAfterDraws = min(NumDraws,100);
NumFiles = ceil(NumDraws/SaveAfterDraws);
NumQuants = length(DSGEModel.Percentiles);
%
% load one file to record the size of the data
%
if OutputType==1;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-1-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
   end;
else;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-1-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
   end;
end;
PCData = load(file);
%
% make sure the mcodevec entry can be used as a logical variable
%
mcodevec = logical(PCData.mcodevec);
ND = sum(mcodevec);
n = size(PCData.PD(1).StdResp,1);
k = DSGEModel.k;
if (OutputType==1)&(k>0);
   MomentsResp.MeanResp = zeros(n,k);
   Weightsk = Weights*ones(1,k);
end;
MomentsResp.StdResp = zeros(n,1);
if n>1;
   MomentsResp.CorrResp = zeros((n*(n-1))/2,1);
   Weightsn = Weights*ones(1,(n*(n-1))/2);
end;
for i=1:NumQuants;
   if (OutputType==1)&(k>0);
      MomentsResp.Quantiles(i).MeanResp = zeros(n,k);
   end;
   MomentsResp.Quantiles(i).StdResp = zeros(n,1);
   if n>1;
      MomentsResp.Quantiles(i).CorrResp = zeros((n*(n-1))/2,1);
   end;
   MomentsResp.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
if OutputType==1;
   VariableNames = DSGEModel.VariableNames;
else;
   VariableNames = DSGEModel.StateVariableNames;
end;
%
% start with the nxT matrices
%
PriorHeader = 'Posterior';
if IsPosterior==0;
   PriorHeader = 'Prior';
end;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the distribution of the partial derivatives of the means, standard deviations and correlations from a change in the parameter "' ParamStr '" as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(VariableNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Parameter Change - ' PriorHeader ' Distribution'],600,250,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
for i=1:n;
   txt = ['Please wait while YADA computes the distribution of the partial derivatives of the means, standard deviations and correlations from a change in the parameter "' ParamStr '" as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(VariableNames(i,:)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   if (OutputType==1)&(k>0);
      TempMean = zeros(NumDraws,k);
   end;
   TempStd = zeros(NumDraws,1);
   CurrSaves = 0;
   if (i==1)&(n>1);
      TempCorr = zeros(NumDraws,(n*(n-1))/2);
   end;
   %
   % load the data
   %
   for ThisSave=1:NumFiles;
      if OutputType==1;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
         end;
      else;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
         end;
      end;
      PCData = load(file);
      NumSaves = length(PCData.PD);
      for s=1:NumSaves;
         if (OutputType==1)&(k>0);
            TempMean(CurrSaves+s,:) = PCData.PD(s).MeanResp(i,:);
         end;
         TempStd(CurrSaves+s) = PCData.PD(s).StdResp(i);
         if (i==1)&(n>1);
            TempCorr(CurrSaves+s,:) = PCData.PD(s).CorrResp';
         end;
      end;
      CurrSaves = CurrSaves+NumSaves;
      drawnow;
   end;
   if (OutputType==1)&(k>0);
      MomentsResp.MeanResp(i,:) = (1/ND)*sum(TempMean.*Weightsk,1);
      TempMean = sort(TempMean(mcodevec,:));
   end;
   MomentsResp.StdResp(i) = (1/ND)*sum(TempStd.*Weights);
   TempStd = sort(TempStd(mcodevec));
   if (i==1)&(n>1);
      MomentsResp.CorrResp = (1/ND)*sum(TempCorr.*Weightsn,1)';
      TempCorr = sort(TempCorr(mcodevec,:));
   end;
   if DSGEModel.ConfidenceRegionMethod==1;
       for l=1:NumQuants;
         if DSGEModel.Percentiles(l)<50;
            QuantVal = ceil((DSGEModel.Percentiles(l)/100)*ND);
         else;
            QuantVal = floor((DSGEModel.Percentiles(l)/100)*ND);
         end;
         if (OutputType==1)&(k>0);
            MomentsResp.Quantiles(l).MeanResp(i,:) = TempMean(QuantVal,:);
         end;
         MomentsResp.Quantiles(l).StdResp(i) = TempStd(QuantVal);
         if (i==1)&(n>1);
            MomentsResp.Quantiles(l).CorrResp = TempCorr(QuantVal,:)';
         end;
      end;
   else;
      for l=1:floor(NumQuants/2);
         ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*ND/100);
         if (OutputType==1)&(k>0);
            for j=1:k;
               Bounds = HighestProbabilityDensityBand(TempMean(:,j),ConfDraws);
               MomentsResp.Quantiles(l).MeanResp(i,j) = Bounds(1);
               MomentsResp.Quantiles(NumQuants-l+1).MeanResp(i,j) = Bounds(2);
            end;
         end;
         BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
         MomentsResp.Quantiles(l).StdResp(i) = BoundsStd(1);
         MomentsResp.Quantiles(NumQuants-l+1).StdResp(i) = BoundsStd(2);
         if (i==1)&(n>1);
            for j=1:(n*(n-1))/2;
               Bounds = HighestProbabilityDensityBand(TempCorr(:,j),ConfDraws);
               MomentsResp.Quantiles(l).CorrResp(j) = Bounds(1);
               MomentsResp.Quantiles(NumQuants-l+1).CorrResp(j) = Bounds(2);
            end;
         end;
      end;
   end;
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEMomentsResponseDistribution.m
%
