function DoneCalc = DSGEPredictiveLikelihood(thetaSample,theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,YData,X,BackYData,SelectedYData,IsOriginal,IsPosterior,IsMarginal,FirstPeriod,LastPeriod,TotalDraws,NumBurnin,CurrChain,DSGEModel,CurrINI)
% DSGEPredictiveLikelihood: Computes the joint or the marginal predictive likelihood for selected subset of variables
%                           for prior or posterior draws of parameters such that the data can be used by the importance
%                           sampling estimator.
%
% USAGE:
%
%       DoneCalc = DSGEPredictiveLikelihood(thetaSample,CovProb,theta,thetaPositions,thetaIndex,thetaDist,PriorDist, ...
%                               LowerBound,UniformBounds,ModelParameters,YData,X,BackYData,SelectedYData,IsOriginal, ...
%                               IsPosterior,IsMarginal,FirstPeriod,LastPeriod,TotalDraws,NumBurnin,CurrChain, ...
%                               DSGEModel,CurrINI)
%
% REQUIRED INPUT:  thetaSample (matrix) of dimension NumDrawsxNumParam with the sample of original parameters.
%
%                  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                                 0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of
%                                 transformation that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                 truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                 and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                 "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices
%                                 with the parameter values for the distributions. In the case of the normal, beta, gamma,
%                                 logistic and gumbel these parameter values are the mean and the standard deviation.
%                                 For the invgamma it is the s and q parameters, where q is a positive integer (degrees
%                                 of freedom). For the left truncated normal there's a third parameter, the lower bound.
%                                 For the uniform the parameter values are the lower and the upper bound. For the student-t
%                                 they are the location (mode), the scale and the degrees of freedom parameter, while for
%                                 the Cauchy only the location and scale parameters are needed. For the Pareto the
%                                 parameter values are for the mode and the shape parameter.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                                 for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                                 sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  BackYData (matrix) of dimension nx2 with the realization for the observed variables in the last
%                                 two historical time periods. Used for back and nowcasts.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed variables.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                                 annualized values should be ptredicted.
%
%                  IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                                 distribution and 0 otherwise.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                                 computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  FirstPeriod (integer) with the first period of the historical sample.
%
%                  LastPeriod (integer) with the last period of the historical sample.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  CurrChain (integer) for the selected MCMC chain when posterior draws are used.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 26, 2012.
%                        This version: August 30, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-04-2012: Finished the original data part of the function.
%
% * 11-05-2012: Added the annualized data code to the function.
%
% * 21-08-2012: Took into account that an importance sampling estimator is used for the
%               predictive likelihood.
%
% * 29-08-2012: Took into account that "logPredLikePriorDSGE" and "logPredLikePriorDSGEAnnual"
%               no longer supplied the output variable "lnPrior".
%
% * 24-10-2012: Removed calls to "logPredLikePriorDSGE" and "logPredLikePriorDSGEAnnual" which included
%               the "lnPrior" output argument.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 05-02-2015: Took into account that About dialogs shouldn't be displayed when running the function
%               in a sequential model run.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 12-06-2015: The function now requires the input variable "SelectedYData".
%
% * 22-02-2016: Updated the documentation.
%
% * 14-03-2017: Added the input variable "BackYData".
%
% * 30-08-2017: Took into account that "logPredLikePriorDSGE" and "logPredLikePriorDSGEAnnual" require
%               the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
NumDraws = size(thetaSample,1);
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if isempty(CurrINI.Sequence)==1;
      if ispc==1;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of the predictive likelihood has therefore been aborted.'];
      else;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of the predictive likelihood has therefore been aborted.'];
      end;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   else;
      if ispc==1;
         DirText = ['"' DSGEModel.OutputDirectory '\predictions"'];
      else;
         DirText = ['"' DSGEModel.OutputDirectory '/predictions"'];
      end;
      SetLogText(['Unable to create the directory: ' DirText]);
   end;
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
h = size(YData,2);
n = DSGEModel.n;
k = DSGEModel.k;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
   else;
      SetLogText('Error when trying to solve the DSGE model. Calculations aborted.');
   end;
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   else;
      SetLogText(strvcat(['The ' SolverStr ' solver provided the return code:'],mcodeStr,'Calculations aborted.'));
   end;
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      if isempty(CurrINI.Sequence)==1;
         ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from computing the ' lower(MargStr) ' predictive likelihood.'];
         About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      else;
         SetLogText(strvcat('Number of time periods for H(t) is less than the number of observations.','Calculations aborted.'));
      end;
      return;
   end;
end;
%
% compute backup values of the conditional likelihood, the likelihood and the prior
%
if IsOriginal==1;
   [lnPLmode,lnLmode,MargBackNowcastmode,PredVars,mcode,status] = logPredLikePriorDSGE(theta,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,BackYData,h,IsMarginal,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
else;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      DSGEModel.Frequency = 4;
   else;
      Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      DSGEModel.Frequency = 12;
   end;
   DSGEModel.Yinit = Yinit;
   %
   % make sure that we take NaN's over the historical sample into account appropriately
   %
   if DSGEModel.YNaNs==0;
      DSGEModel.YAnn = DSGEModel.Y;
   else;
      Y = DSGEModel.Y;
      %
      % make sure that Y has NaN's that conform to the pattern implied by annualization
      %
      NaNMatrix = isnan(Y);
      NaNMatrix = Annualize([zeros(DSGEModel.n,DSGEModel.Frequency-1) NaNMatrix],DSGEModel.annual,ones(n,1));
      DSGEModel.YAnn(logical(NaNMatrix)) = NaN;
   end;
   YDataAnnual = Annualize([Yinit YData],DSGEModel.annual,ones(n,1));
   YData(isnan(YDataAnnual)) = NaN;
   DSGEModel.Y = DSGEModel.YAnn;
   [lnPLmode,lnLmode,PredVars,mcode,status] = logPredLikePriorDSGEAnnual(theta,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,YDataAnnual,X,h,IsMarginal,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
end;
%
if status~=0;
   if status==0;
      ErrorMsg = 'Unable to compute the likelihood.';
   else;
      ErrorMsg = 'The forecast covariance matrix is not positive definite for all periods.';
   end;
   if isempty(CurrINI.Sequence)==1;
      txt = ['There was an error running the Kalman filter for the historical sample. The following message was received: ' ErrorMsg];
      About(txt,'information','YADA - Kalman Filter',200,500,CurrINI);
   else;
      SetLogText(strvcat('Kalman filter error:',ErrorMsg,'Calculations aborted.'));
   end;
   return;
end;
%
% we can now run the routine for all draws
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' Draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = [MargStr ' Predictive Likelihood' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the ' lower(MargStr) ' predictive likelihood for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Predictive Likelihood ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
%
LogPosteriorData = zeros(NumDraws,h);
if (IsOriginal==1)&(IsMarginal==1);
   LogPosteriorBackNowcastData = zeros(NumDraws,size(BackYData,2));
else;
   LogPosteriorBackNowcastData = [];
end;
%
for ParamIndex=1:NumDraws;
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         StopEstimation = 1;
         break;
      else;
         ProgressDLG([ParamIndex/NumDraws ParamIndex]);
      end;
   end;
   theta = thetaSample(ParamIndex,:)';
   if IsOriginal==1;
      [lnPL,lnL,MargBackNowcast,PredVars,mcode,status] = logPredLikePriorDSGE(theta,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,BackYData,h,IsMarginal,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   else;
      [lnPL,lnL,PredVars,mcode,status] = logPredLikePriorDSGEAnnual(theta,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,YDataAnnual,X,h,IsMarginal,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   end;
   %
   % check if we should plug in the backup values
   %
   if (mcode~=1)|(status~=0);
      lnPL = lnPLmode;
      if (IsOriginal==1)&(IsMarginal==1);
         MargBackNowcast = MargBackNowcastmode;
      end;
   end;
   LogPosteriorData(ParamIndex,:) = lnPL';
   if (IsOriginal==1)&(IsMarginal==1);
      LogPosteriorBackNowcastData(ParamIndex,:) = MargBackNowcast;
   end;
   drawnow;
end;
%
if StopEstimation==0;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
   end;
   save(file,'LogPosteriorData','LogPosteriorBackNowcastData','PredVars');
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
% we made it to the end
%
DoneCalc = 1;

%
% end of DSGEPredictiveLikelihood.m
%
