function SVEs = DSGESVESimulatedDistribution(DSGEModel,CurrINI,Selector,SelectedParameters,NumDraws,TotalDraws,CurrChain,NumPaths,IsPosterior,Weights)
% DSGESVESimulatedDistribution:  Computes the mean value of X as well as quantiles by loading data from disk.
%                                Based on simulated data, X is here sample based moments for the observed
%                                variables or the the state variables.
%
% USAGE:
%
%       SVEs = DSGESVESimulatedDistribution(DSGEModel,CurrINI,Selector,SelectedParameters,NumDraws,...
%                                           TotalDraws,CurrChain,NumPaths,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  Selector (integer) that is:
%                                            5 - if the function should work with observed variable correlations.
%                                            6 - if the function should work with state variable correlations.
%
%                  SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                     one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                     parameters.
%
%                  TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                                     that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  NumPaths (integer) with the number of data paths that have been simulated per parameter value.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: SVEs (structure) with fields that depend on Selector. For the observed variable correlations the fields
%                                     are: "MeanY", "StdY", "CorrY", "SkewY". In addition, it yields the fields "Data.Mean",
%                                     "Data.Std", "Data.Corr", "Data.Skew", and "Data.Kurt". For the state variable
%                                     correlations the fields are "MeanKsi", "StdKsi" and "CorrKsi". In addition, the
%                                     vector field "Quantiles" is used, with subfields "percent" and field names determined
%                                     by the Selector.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 28, 2008.
%                        This version: August 24, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-02-2008: Added the code for "Selector" value 6.
%
% * 03-03-2008: Made sure that the number of exogenous variables is properly taken into account for the observed
%               variable correlations.
%
% * 01-04-2008: Took into account that "NumFiles" has changed. Moreover, the filename now depends on value for
%               "Selector".
%
% * 23-05-2008: Updated the documentation.
%
% * 22-01-2009: Updated the documentation.
%
% * 23-03-2009: Added the input variable "IsPosterior".
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 17-03-2011: The function now supports missing observations when computing data based moments.
%
% * 25-11-2011: The function now takes skewness computations into account.
%
% * 02-12-2011: The function now also takes kurtosis computations into account.
%
% * 21-12-2011: Made sure that the data is corrected for the mean correctly when computing the mean.
%
% * 05-01-2012: Updated the documentation.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 21-08-2013: Added code for computing p-values of prior and posterior predictive checks on the
%               moments of the oberved data.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 24-08-2017: Added the Weights input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
SVEs = [];
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% fix the information needed to setup the file names
%
SelParamStr = strrep(num2str(SelectedParameters),' ','');
%
%NumFiles = ceil(NumDraws/min(NumDraws,1000));
NumFiles = ceil(NumDraws/1);
NumQuants = length(DSGEModel.Percentiles);
%
% we need to first determine the size of the data
%
if Selector==5;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-1-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
   end;
else;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-1-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
   end;
end;
SVEData = load(file);
if Selector<=4;
   SVEs.DateVector = SVEData.DateVector;
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' - Prior';
end;
%
if Selector==5;
   %
   % we have the observed variable correlations
   %
   Y = SVEData.Y;
   [n T] = size(Y);
   %
   % compute the data based mean, standard deviation, and autocorrelation
   %
   X = SVEData.X;
   k = min(size(X));
   if DSGEModel.YNaNs==0;
      if isempty(X)==0;
         SVEs.Data.Mean = (Y*X')/(X*X');
         %
         % de-mean Y using estimate of the mean from the data rather than A'.
         %
         Y = Y-(SVEs.Data.Mean*X);
      else;
         SVEs.Data.Mean = (1/T)*sum(Y,2);
         Y = Y-(SVEs.Data.Mean*ones(1,T));
      end;
      SVEs.Data.Std = sqrt(diag((1/T)*Y*Y'));
      SVEs.Data.Skew = ((1/T)*sum(Y.*Y.*Y,2))./(SVEs.Data.Std.*SVEs.Data.Std.*SVEs.Data.Std);
      SVEs.Data.Kurt = ((1/T)*sum(Y.*Y.*Y.*Y,2))./(SVEs.Data.Std.*SVEs.Data.Std.*SVEs.Data.Std.*SVEs.Data.Std);
      AutoCovHorizon = (size(SVEData.SVE.CorrData(1,1).CorrY,2)-1)/2;
      %
      SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
      SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(((1/T)*Y*Y')./(SVEs.Data.Std*SVEs.Data.Std'));
      for i=1:AutoCovHorizon;
         Y = SVEData.Y(:,1+i:size(SVEData.Y,2));
         YL = SVEData.Y(:,1:length(SVEData.Y)-i);
         if isempty(X)==0;
            X = SVEData.X(:,1+i:size(SVEData.X,2));
            Y = Y-(((Y*X')/(X*X'))*X);
            XL = SVEData.X(:,1:size(SVEData.X,2)-i);
            YL = YL-(((Y*X')/(X*X'))*XL);
         else;
            Y = Y-(SVEs.Data.Mean*ones(1,size(Y,2)));
            YL = YL-(SVEs.Data.Mean*ones(1,size(YL,2)));
         end;
         Corr = ((1/T)*Y*YL')./(SVEs.Data.Std*SVEs.Data.Std');
         %
         % first vector is lagged and second is leaded.
         %
         SVEs.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
         SVEs.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
      end;
   else;
      pos = (~isnan(Y));
      NaNY = isnan(Y);
      Y(NaNY) = 0;
      Tvec = T./sum(pos,2);
      if isempty(X)==0;
         SVEs.Data.Mean = diag(Tvec)*(Y*X')/(X*X');
         %
         % de-mean Y using estimate of the mean from the data rather than A'.
         %
         Y = Y-(SVEs.Data.Mean*X);
      else;
         SVEs.Data.Mean = Tvec.*((1/T)*sum(Y,2));
         Y = Y-(SVEs.Data.Mean*ones(1,T));
      end;
      Y(NaNY) = 0;
      TMat = ones(n,n)./TMatrix(sum(pos,2));
      SVEs.Data.Std = sqrt(diag(TMat.*(Y*Y')));
      SVEs.Data.Skew = ((Tvec/T).*sum(Y.*Y.*Y,2))./(SVEs.Data.Std.*SVEs.Data.Std.*SVEs.Data.Std);
      SVEs.Data.Kurt = ((Tvec/T).*sum(Y.*Y.*Y.*Y,2))./(SVEs.Data.Std.*SVEs.Data.Std.*SVEs.Data.Std.*SVEs.Data.Std);
      AutoCovHorizon = (size(SVEData.SVE.CorrData(1,1).CorrY,2)-1)/2;
      SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
      SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(((1/T)*Y*Y')./(SVEs.Data.Std*SVEs.Data.Std'));
      for i=1:AutoCovHorizon;
         Y = SVEData.Y(:,1+i:size(SVEData.Y,2));
         YL = SVEData.Y(:,1:length(SVEData.Y)-i);
         pos = (~isnan(Y));
         NaNY = isnan(Y);
         NaNYL = isnan(YL);
         Y(NaNY) = 0;
         YL(NaNYL) = 0;
         Tvec = size(Y,2)./sum(pos,2);
         if isempty(X)==0;
            X = SVEData.X(:,1+i:size(SVEData.X,2));
            Y = Y-(diag(Tvec)*((Y*X')/(X*X'))*X);
            XL = SVEData.X(:,1:size(SVEData.X,2)-i);
            YL = YL-(((Y*X')/(X*X'))*XL);
         else;
            Y = Y-(SVEs.Data.Mean*ones(1,size(Y,2)));
            YL = YL-(SVEs.Data.Mean*ones(1,size(YL,2)));
         end;
         Y(NaNY) = 0;
         YL(NaNYL) = 0;
         Corr = (TMat.*(Y*YL'))./(SVEs.Data.Std*SVEs.Data.Std');
         %
         % first vector is lagged and second is leaded.
         %
         SVEs.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
         SVEs.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
      end;
   end;
   %
   % initialize model based mean, standard deviation, and autocorrelations
   %
   if k>0;
      SVEs.Mean = zeros(n,k);
   else;
      SVEs.Mean = zeros(n,1);
   end;
   SVEs.Std = zeros(n,1);
   SVEs.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Skew = zeros(n,1);
   SVEs.Kurt = zeros(n,1);
   SVEs.PredCheck.Mean = SVEs.Mean;
   SVEs.PredCheck.Std = SVEs.Std;
   SVEs.PredCheck.Skew = SVEs.Skew;
   SVEs.PredCheck.Kurt = SVEs.Kurt;
   for i=1:NumQuants;
      if k>0;
         SVEs.Quantiles(i).Mean = zeros(n,k);
      else;
         SVEs.Quantiles(i).Mean = zeros(n,1);
      end;
      SVEs.Quantiles(i).Std = zeros(n,1);
      SVEs.Quantiles(i).Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
      SVEs.Quantiles(i).Skew = zeros(n,1);
      SVEs.Quantiles(i).Kurt = zeros(n,1);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution' PriorHeader],600,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   Weight = kron(Weights,ones(NumPaths,1));
   for i=1:n;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up a matrix for the data
      %
      if k>0;
         TempMean = zeros(NumDraws*NumPaths,k);
      else;
         TempMean = zeros(NumDraws*NumPaths,1);
      end;
      TempStd = zeros(NumDraws*NumPaths,1);
      TempSkew = zeros(NumDraws*NumPaths,1);
      TempKurt = zeros(NumDraws*NumPaths,1);
      CurrSaves = 0;
      for ThisSave=1:NumFiles;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
         end;
         SVEData = load(file);
         SVE = SVEData.SVE;
         NumSaves = size(SVE.CorrData,1);
         if (i==1)&(ThisSave==1);
            IsSkewness = isfield(SVE(1).CorrData,'SkewY');
         end;
         %
         % load data from CurrSaves to NumSaves+1-CurrSaves
         %
         for s=1:NumSaves;
            for p=1:NumPaths;
               TempMean(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrData(s,p).MeanY(i,:);
               TempStd(CurrSaves+p+((s-1)*NumPaths),1) = SVE.CorrData(s,p).StdY(i,1);
               if IsSkewness==1;
                  TempSkew(CurrSaves+p+((s-1)*NumPaths),1) = SVE.CorrData(s,p).SkewY(i,1);
                  TempKurt(CurrSaves+p+((s-1)*NumPaths),1) = SVE.CorrData(s,p).KurtY(i,1);
               else;
                  TempSkew(CurrSaves+p+((s-1)*NumPaths),1) = NaN;
                  TempKurt(CurrSaves+p+((s-1)*NumPaths),1) = NaN;
               end;
            end;
            drawnow;
         end;
         CurrSaves = CurrSaves+(NumSaves*NumPaths);
         drawnow;
      end;
      %
      % we can now calculate the mean
      %
      if k>0;
         SVEs.Mean(i,:) = (1/(NumDraws*NumPaths))*sum(TempMean.*(Weight*ones(1,k)),1);
      else;
         SVEs.Mean(i,1) = (1/(NumDraws*NumPaths))*sum(TempMean.*Weight,1);
      end;
      SVEs.Std(i,1) = (1/(NumDraws*NumPaths))*sum(TempStd.*Weight,1);
      SVEs.Skew(i,1) = (1/(NumDraws*NumPaths))*sum(TempSkew.*Weight,1);
      SVEs.Kurt(i,1) = (1/(NumDraws*NumPaths))*sum(TempKurt.*Weight,1);
      %
      % we can now compute the predictive check p-values
      %
      for j=1:k;
         SVEs.PredCheck.Mean(i,j) = (1/(NumDraws*NumPaths))*sum((TempMean(:,j)>SVEs.Data.Mean(i,j)).*Weight);
      end;
      SVEs.PredCheck.Std(i) = (1/(NumDraws*NumPaths))*sum((TempStd>SVEs.Data.Std(i)).*Weight);
      SVEs.PredCheck.Skew(i) = (1/(NumDraws*NumPaths))*sum((TempSkew>SVEs.Data.Skew(i)).*Weight);
      SVEs.PredCheck.Kurt(i) = (1/(NumDraws*NumPaths))*sum((TempKurt>SVEs.Data.Kurt(i)).*Weight);
      %
      % now we can sort these and create the quantiles from the sorted
      %
      if DSGEModel.ConfidenceRegionMethod==1;
         for j=1:size(TempMean,2);
            TempMeanj = sort(TempMean(:,j));
            if j==1;
               TempStd = sort(TempStd);
               TempSkew = sort(TempSkew);
               TempKurt = sort(TempKurt);
            end;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
               end;
               SVEs.Quantiles(l).Mean(i,j) = TempMeanj(QuantVal,1);
               if j==1;
                  SVEs.Quantiles(l).Std(i,1) = TempStd(QuantVal,1);
                  SVEs.Quantiles(l).Skew(i,1) = TempSkew(QuantVal,1);
                  SVEs.Quantiles(l).Kurt(i,1) = TempKurt(QuantVal,1);
               end;
            end;
            drawnow;
         end;
      else;
         TempMean = sort(TempMean);
         TempStd = sort(TempStd);
         TempSkew = sort(TempSkew);
         TempKurt = sort(TempKurt);
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*NumPaths/100);
            BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
            SVEs.Quantiles(l).Std(i) = BoundsStd(1);
            SVEs.Quantiles(NumQuants-l+1).Std(i) = BoundsStd(2);
            BoundsSkew = HighestProbabilityDensityBand(TempSkew,ConfDraws);
            SVEs.Quantiles(l).Skew(i) = BoundsSkew(1);
            SVEs.Quantiles(NumQuants-l+1).Skew(i) = BoundsSkew(2);
            BoundsKurt = HighestProbabilityDensityBand(TempKurt,ConfDraws);
            SVEs.Quantiles(l).Kurt(i) = BoundsKurt(1);
            SVEs.Quantiles(NumQuants-l+1).Kurt(i) = BoundsKurt(2);
            for t=1:size(TempMean,2);
               BoundsMean = HighestProbabilityDensityBand(TempMean(:,t),ConfDraws);
               SVEs.Quantiles(l).Mean(i,t) = BoundsMean(1);
               SVEs.Quantiles(NumQuants-l+1).Mean(i,t) = BoundsMean(2);
            end;
         end;
      end;
   end;
   %
   % now we need to deal with the autocorrelations
   %
   SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
   for j=1:(2*AutoCovHorizon)+1;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. The data are loaded sequentially from disk and it may therefore take some time. Current autocorrelation horizon: ' int2str(j-(AutoCovHorizon+1)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempCorr = zeros(NumDraws*NumPaths,(n*(n+1))/2);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
            end;
            SVEData = load(file);
            SVE = SVEData.SVE;
            NumSaves = size(SVE.CorrData,1);
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            for s=1:NumSaves;
               for p=1:NumPaths;
                  TempCorr(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrData(s,p).CorrY(:,j)';
               end;
               drawnow;
            end;
            CurrSaves = CurrSaves+(NumSaves*NumPaths);
            drawnow;
         end;
         %
         % compute the mean
         %
         SVEs.Corr(:,j) = (1/(NumDraws*NumPaths))*sum(TempCorr.*(Weight*ones(1,(n*(n+1))/2)),1)';
         %
         % sort the data and determine the quantile values
         %
         TempCorr = sort(TempCorr);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
               end;
               SVEs.Quantiles(l).Corr(:,j) = TempCorr(QuantVal,:)';
            end;
         else;
             for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*NumPaths/100);
               for t=1:n*(n+1)/2;
                  Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
                  SVEs.Quantiles(l).Corr(t,j) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
               end;
            end;
         end;
      catch;
         for i1=1:n*(n+1)/2;
            TempCorr = zeros(NumDraws*NumPaths,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-OVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
               end;
               SVEData = load(file);
               SVE = SVEData.SVE;
               NumSaves = size(SVE.CorrData,1);
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               for s=1:NumSaves;
                  for p=1:NumPaths;
                     TempCorr(CurrSaves+p+((s-1)*NumPaths),1) = SVE.CorrData(s,p).CorrY(i1,j)';
                  end;
                  drawnow;
               end;
               CurrSaves = CurrSaves+(NumSaves*NumPaths);
               drawnow;
            end;
            %
            % compute the mean
            %
            SVEs.Corr(i1,j) = (1/(NumDraws*NumPaths))*sum(TempCorr.*Weight,1)';
            %
            % sort the data and determine the quantile values
            %
            TempCorr = sort(TempCorr);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
                  end;
                  SVEs.Quantiles(l).Corr(i1,j) = TempCorr(QuantVal,1);
               end;
            else;
                for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*NumPaths/100);
                  Bounds = HighestProbabilityDensityBand(TempCorr,ConfDraws);
                  SVEs.Quantiles(l).Corr(i1,j) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).Corr(i1,j) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
elseif Selector==6;
   %
   % we have the state variable correlations
   %
   r = length(SVEData.SVE.CorrState(1,1).MeanKsi);
   SVEs.MeanKsi = zeros(r,1);
   SVEs.StdKsi = zeros(r,1);
   SVEs.CorrKsi = zeros(r,r);
   for i=1:NumQuants;
      SVEs.Quantiles(i).MeanKsi = zeros(r,1);
      SVEs.Quantiles(i).StdKsi = zeros(r,1);
      SVEs.Quantiles(i).CorrKsi = zeros(r,r);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the mean state variable correlations for model-based simulated data as well as percentile values for the state variable correlations. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.StateVariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['State Variable Distribution' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   Weight = kron(Weights,ones(NumPaths,1));
   for i=1:r;
      %
      % update the wait dialog
      %
      txt = ['Please wait while YADA computes the mean state variable correlations for model-based simulated data as well as percentile values for the state variable correlations. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(DSGEModel.StateVariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up matrices for storing the data
      %
      try;
         TempMean = zeros(NumDraws*NumPaths,1);
         TempStd = zeros(NumDraws*NumPaths,1);
         TempCorr = zeros(NumDraws*NumPaths,r);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
            end;
            SVEData = load(file);
            SVE = SVEData.SVE;
            NumSaves = size(SVE.CorrState,1);
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            for s=1:NumSaves;
               for p=1:NumPaths;
                  TempMean(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrState(s,p).MeanKsi(i,1);
                  TempStd(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrState(s,p).StdKsi(i,1);
                  TempCorr(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrState(s,p).CorrKsi(i,:);
               end;
               drawnow;
            end;
            CurrSaves = CurrSaves+(NumSaves*NumPaths);
            drawnow;
         end;
         %
         % we can now calculate the mean
         %
         SVEs.MeanKsi(i,1) = (1/(NumDraws*NumPaths))*sum(TempMean.*Weight);
         SVEs.StdKsi(i,1) = (1/(NumDraws*NumPaths))*sum(TempStd.*Weight);
         SVEs.CorrKsi(i,:) = (1/(NumDraws*NumPaths))*sum(TempCorr.*(Weight*ones(1,r)),1);
         %
         % now we can sort these and create the quantiles from the sorted
         %
         TempMean = sort(TempMean);
         TempStd = sort(TempStd);
         TempCorr = sort(TempCorr);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
               end;
               SVEs.Quantiles(l).MeanKsi(i) = TempMean(QuantVal,1);
               SVEs.Quantiles(l).StdKsi(i) = TempStd(QuantVal,1);
               SVEs.Quantiles(l).CorrKsi(i,:) = TempCorr(QuantVal,:);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*NumPaths/100);
               BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
               SVEs.Quantiles(l).StdKsi(i) = BoundsStd(1);
               SVEs.Quantiles(NumQuants-l+1).StdKsi(i) = BoundsStd(2);
               BoundsMean = HighestProbabilityDensityBand(TempMean,ConfDraws);
               SVEs.Quantiles(l).MeanKsi(i) = BoundsMean(1);
               SVEs.Quantiles(NumQuants-l+1).MeanKsi(i) = BoundsMean(2);
               for t=1:r;
                  Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
                  SVEs.Quantiles(l).CorrKsi(i,t) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).CorrKsi(i,t) = Bounds(2);
               end;
            end;
         end;
      catch;
         for r1=1:r;
            if r1==1;
               TempMean = zeros(NumDraws*NumPaths,1);
               TempStd = zeros(NumDraws*NumPaths,1);
            end;
            TempCorr = zeros(NumDraws*NumPaths,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SVE-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SVE-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(TotalDraws,'%0.0f') '-' num2str(NumPaths,'%0.0f') '.mat']);
               end;
               SVEData = load(file);
               SVE = SVEData.SVE;
               NumSaves = size(SVE.CorrState,1);
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               for s=1:NumSaves;
                  for p=1:NumPaths;
                     if r1==1;
                        TempMean(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrState(s,p).MeanKsi(i,1);
                        TempStd(CurrSaves+p+((s-1)*NumPaths),:) = SVE.CorrState(s,p).StdKsi(i,1);
                     end;
                     TempCorr(CurrSaves+p+((s-1)*NumPaths),r1) = SVE.CorrState(s,p).CorrKsi(i,r1);
                  end;
                  drawnow;
               end;
               CurrSaves = CurrSaves+(NumSaves*NumPaths);
               drawnow;
            end;
            %
            % we can now calculate the mean
            %
            if r1==1;
               SVEs.MeanKsi(i,1) = (1/(NumDraws*NumPaths))*sum(TempMean.*Weight);
               SVEs.StdKsi(i,1) = (1/(NumDraws*NumPaths))*sum(TempStd.*Weight);
               TempMean = sort(TempMean);
               TempStd = sort(TempStd);
            end;
            SVEs.CorrKsi(i,r1) = (1/(NumDraws*NumPaths))*sum(TempCorr.*Weight,1);
            TempCorr = sort(TempCorr);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws*NumPaths);
                  end;
                  if r1==1;
                     SVEs.Quantiles(l).MeanKsi(i) = TempMean(QuantVal,1);
                     SVEs.Quantiles(l).StdKsi(i) = TempStd(QuantVal,1);
                  end;
                  SVEs.Quantiles(l).CorrKsi(i,r1) = TempCorr(QuantVal,1);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws*NumPaths/100);
                  if r1==1;
                     BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
                     SVEs.Quantiles(l).StdKsi(i) = BoundsStd(1);
                     SVEs.Quantiles(NumQuants-l+1).StdKsi(i) = BoundsStd(2);
                     BoundsMean = HighestProbabilityDensityBand(TempMean,ConfDraws);
                     SVEs.Quantiles(l).MeanKsi(i) = BoundsMean(1);
                     SVEs.Quantiles(NumQuants-l+1).MeanKsi(i) = BoundsMean(2);
                  end;
                  Bounds = HighestProbabilityDensityBand(TempCorr(:,1),ConfDraws);
                  SVEs.Quantiles(l).CorrKsi(i,r1) = Bounds(1);
                  SVEs.Quantiles(NumQuants-l+1).CorrKsi(i,r1) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGESVESimulatedDistribution.m
%
