function CVDs = DSGEStateCVDDistribution(DSGEModel,CurrINI,UseGroups,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% DSGEStateCVDDistribution: Computes the mean value of conditional variance decompositions for the state variables
%                           as well as quantiles by loading data from disk.
%
% USAGE:
%
%       CVDs = DSGEStateCVDDistribution(DSGEModel,CurrINI,UseGroups,NumDraws,TotalDraws,CurrChain,...
%                                       IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  UseGroups (boolean) that is 1 if shock groups should be used and 0 otherwise.
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                 parameters.
%
%                  TotalDraws (integer) with the total number of draws from the posterior distribution of the
%                                 parameters that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: CVDs (structure) with fields "FEVD" and "Quantiles". The "FEVD" field holds a 3D matrix of dimension
%                                 r x q x h+1, where r is the number of state variables, q the number of shocks,
%                                 and h the impulse response horizon. The values are the posterior mean conditional
%                                 variance decomposition. The field "Quantiles" is a vector structure with fields "FEVD" and
%                                 "percent", where "FEVD" holds a 3D matrix of dimension r x q x h+1 with values from
%                                 the percent quantile of the marginal distribution of the conditional variance decomposition
%                                 and "percent" holds the percentile value in percent. The h+1:st r x q matrix is the long
%                                 run.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 4, 2017.
%                        This version: August 25, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-05-2017: Finished the function.
%
% * 25-08-2017: Added the "Weights" input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
CVDs = [];
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
NumFiles = ceil(NumDraws/min(NumDraws,1000));
%
% Determine the impulse response horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
h = SampleFreq*DSGEModel.IRHorizon;
%
if UseGroups==0;
   q = length(DSGEModel.StateShockPositions);
   ShockNames = DSGEModel.StateShockNames;
   IMatrix = eye(q);
else;
   q = max(DSGEModel.ShockGroups);
   ShockNames = char(zeros(size(DSGEModel.ShockGroupNames)));
   for i=1:q;
      ShockNames(i,:) = lower(strrep(DSGEModel.ShockGroupNames(i,:),'_',' '));
   end;
   IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
end;
r = length(DSGEModel.StateVariablePositions);
NumQuants = length(DSGEModel.Percentiles);
%
% initialize output
%
CVDs.FEVD = zeros(r,q,h+1);
for i=1:NumQuants;
   CVDs.Quantiles(i).FEVD = zeros(r,q,h+1);
   CVDs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
AddWidth = 0;
if IsPosterior==0;
   PriorHeader = ' For Prior';
   AddWidth = 50;
end;
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean conditional variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (state variable,shock): (' StringTrim(DSGEModel.StateVariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
else;
   txt = ['Please wait while YADA computes the prior mean conditional variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (state variable,shock): (' StringTrim(DSGEModel.StateVariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
end;
WaitHandle = WaitDLG(txt,'information',['Conditional Variance Decompositions' PriorHeader ' - State Variables'],500+AddWidth,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:r;
   for j=1:q;
      %
      % update the wait dialog
      %
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean conditional variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (state variable,shock): (' StringTrim(DSGEModel.StateVariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      else;
         txt = ['Please wait while YADA computes the prior mean conditional variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (state variable,shock): (' StringTrim(DSGEModel.StateVariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(NumDraws,h+1);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-State-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-State-Prior-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
            end;
            CVDData = load(file);
            CVD = CVDData.CVD;
            NumSaves = length(CVD);
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            for s=1:NumSaves;
               %
               % check if we should study data for shock groups or not.
               %
               if UseGroups==0
                  FEVD = CVD(s).FEVD;
               else;
                  FEVD = zeros(r,q,h+1);
                  for k=1:h+1;
                     FEVD(:,:,k) = CVD(s).FEVD(:,:,k)*IMatrix;
                  end;
               end;
               TempVar(CurrSaves+s,:) = FEVD(i,j,:);
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % now we sort and compute the mean the data
         %
         for k=1:h+1;
            if max(abs(TempVar(:,k)))>1.0e-10;
               CVDs.FEVD(i,j,k) = mean(TempVar(:,k).*Weights);
            end;
            TempVar(:,k) = sort(TempVar(:,k));
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempVar(QuantVal,k))>1.0e-10;
                     CVDs.Quantiles(l).FEVD(i,j,k) = TempVar(QuantVal,k);
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar(:,k),ConfDraws);
                  CVDs.Quantiles(l).FEVD(i,j,k) = Bounds(1);
                  CVDs.Quantiles(NumQuants-l+1).FEVD(i,j,k) = Bounds(2);
               end;
            end;
            drawnow;
         end;
      catch;
         for k1=1:h+1;
            TempVar = zeros(NumDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-State-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\cvd\DSGE-CVD-State-Prior-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
               end;
               CVDData = load(file);
               CVD = CVDData.CVD;
               NumSaves = length(CVD);
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               for s=1:NumSaves;
                  %
                  % check if we should study data for shock groups or not.
                  %
                  if UseGroups==0
                     FEVD = CVD(s).FEVD;
                  else;
                     FEVD = zeros(r,q,h+1);
                     for k=1:h+1;
                        FEVD(:,:,k) = CVD(s).FEVD(:,:,k)*IMatrix;
                     end;
                  end;
                  TempVar(CurrSaves+s,1) = FEVD(i,j,k1);
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % now we sort and compute the mean the data
            %
            if max(abs(TempVar))>1.0e-10;
               CVDs.FEVD(i,j,k1) = mean(TempVar.*Weights);
            end;
            TempVar = sort(TempVar);
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempVar(QuantVal))>1.0e-10;
                     CVDs.Quantiles(l).FEVD(i,j,k1) = TempVar(QuantVal,1);
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  CVDs.Quantiles(l).FEVD(i,j,k1) = Bounds(1);
                  CVDs.Quantiles(NumQuants-l+1).FEVD(i,j,k1) = Bounds(2);
               end;
            end;
            drawnow;
         end;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEStateCVDDistribution.m
%
