function [Status,CurrChain,IsSubSample] = SequentialPredictiveMoments(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputeMoments)
% SequentialPredictiveMoments: Can compute the unconditional predictive mean and covariance matrix using posterior draws
%                              for DSGE models.
%
% USAGE:
%
%       [Status,CurrChain,IsSubSample] = SequentialPredictiveMoments(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,...
%                                                                    RecomputeMoments)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%                            When empty, the function checks which alternative should be considered.
%
%                  RecomputeMoments (boolean) which takes the value 1 if the predictive moments
%                            should be recomputed and 0 if old calculations should be used.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the computations could be successfully
%                            carries out, and 0 otherwise.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 4, 2014.
%                        This version: August 25, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-02-2015: Continued work on the function from the call to "DSGEPredictiveMoments".
%
% * 06-02-2015: Finished the function from the point where output is written to file and to screen.
%
% * 09-02-2015: Changed the name of the output files such that the prediction sample is used.
%
% * 10-02-2015: Made sure that "YData" is stored in the mat-file.
%
% * 31-03-2015: Made sure that toc command is collected into a variable.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 20-06-2017: Fixed a reference to predictive likelihood instead of preditive moments.
%
% * 17-08-2017: Removed command that checks if the data in the forecast sample are available or just NaNs.
%
% * 23-08-2017: Took into account that LoadDSGEPosteriorDraws now provides the output variable "Weights".
%
% * 25-08-2017: Took into account that "DSGEPredictiveMoments" requires the input variable "Weights".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Status = 0;
%
% first attempt to load some posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
HeaderStr = 'YADA - Bad MAT File';
try;
   ModeData = load(ModeFile);
catch;
   ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
end;
%
% check if file loading was successful
%
if isempty(ErrorStr)==0;
   SetLogText('Unable to load posterior mode data. Aborting calculation.');
   return;
end;
%
% if there are multiple chains, check which one to use if necessary
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
if (isempty(CurrChain)==1)&(NumChains>1);
   %
   % ask which chain to load
   %
   ChainsStr = '';
   for i=1:NumChains;
      ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
   end;
   [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Marginal Predictive Moments - Posterior Distribution','','',CurrINI);
elseif (isempty(CurrChain)==0)&(CurrChain>NumChains);
   action = 'cancel';
   SetLogText(['The selected chain number (' int2str(CurrChain) ') is greater than the available number of MCMC chains.']);
else;
   CurrChain = 1;
   action = 'ok';
end;
if strcmp(lower(action),'cancel')==1;
   SetLogText('Aborting calculation.');
   return;
end;
%
% check if the data actually exists
%
if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==0;
   SetLogText('Unable to locate the posterior parameter draws. Aborting calculation.');
   return;
end;
%
% check if sub-sample should be
%
if DSGEModel.PosteriorSampler<=7;
   if isempty(IsSubSample)==1;
      txt = 'Would you like to use a sub-sample of the posterior draws when estimating the marginal predictive moments?';
      answer = Query(txt,'question',400,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
      if strcmp(lower(answer),'yes')==1;
         IsSubSample = 1;
      else;
         IsSubSample = 0;
      end;
   end;
else;
   IsSubSample = 0;
end;
%
% load the data
%
[PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
clear('LogPost','AcceptedDraws','PostSample');
[NumPostDraws,NumParam] = size(thetaPostSample);
thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
NumDraws = NumPostDraws-NumBurnin;
if IsSubSample==1;
   UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
   thetaPostSample = thetaPostSample(UseDraws,:);
   NumDraws = length(UseDraws);
end;
%
% collect some required data
%
theta = ModeData.thetaMode;
thetaPositions = ModeData.thetaPositions;
thetaIndex = ModeData.thetaIndex;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
LowerBound = ModeData.LowerBound;
UniformBounds = ModeData.UniformBounds;
ModelParameters = ModeData.ModelParameters;
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
%
% check how many time periods to use for predictive distributions
%
MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% check the maximum when we take NaNs into account
%
YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
%MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
[MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
MaxFH = min(MaxFH,MaxPH);
YData = YData(:,1:MaxFH);
%
% matrix with exogenous variables
%
X = [];
if DSGEModel.k>0;
   if NumPredX<MaxFH;
      X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
   end;
   if NumPredX>0;
      X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
   end;
end;
%
% Determine the prediction sample
%
PredictionSampleStrMat = CreateFullSample(DSGEModel);
PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
%
% check if older calculations exists given that such may be used
%
tic;
MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-PredictiveMoments-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(MaxFH) '.' int2str(NumDraws) '-' int2str(NumPostDraws) '.mat']);
if (RecomputeMoments==0)&(FileExist(MatFile)==1);
   DoneCalc = 1;
else;
   %
   % compute the predictive moments
   %
   SetLogText(strvcat('Estimating the marginal predictive mean and covariance','for the observed variables...',' '));
   DoneCalc = DSGEPredictiveMoments(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxFH,NumPostDraws,CurrChain,1,Weights,SmpStr);
end;
clear('thetaPostSample');
if DoneCalc==0;
   SetLogText('Aborting calculation.');
   CompTime = toc;
   return;
end;
%
% load the results
%
CompTime = toc;
SetLogText(['Computation time: ' SecondsToHoursStr(CompTime) ' (hrs:min:sec)']);
MPM = load(MatFile);
PredY = MPM.MeanPredictY;
n = DSGEModel.n;
CovPred = zeros(n,n,MaxFH);
for i=1:MaxFH;
   CovPred(:,:,i) = MPM.MeasureCov+MPM.ShockCov(:,:,i)+MPM.StateCov(:,:,i)+MPM.ParameterCov(:,:,i);
end;
%
if isfield(MPM,'CovPred')==0;
   save(MatFile,'CovPred','PredictionSampleStrMat','-APPEND');
end;
if isfield(MPM,'YData')==0;
   save(MatFile,'YData','-APPEND');
end;
%
% write the results to log text and to file
%
ResFile = strrep(MatFile,'.mat','.txt');
PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,[],NumDraws,NumPostDraws,NumBurnin,1,CompTime,DSGEModel,CurrINI);
%
% write results to screen
%
if FileExist(ResFile)==1;
   PrtStr = ['%s%s   %10.4f   %10.4f'];
   VarLength = size(DSGEModel.VariableNames,2);
   if VarLength<15;
      AddStr = SpaceStr(15-VarLength);
   else;
      AddStr = '';
   end;
   FirstStr = MultiCharStr('=',42);
   DivStr = MultiCharStr('-',42);
   ResultsStr = '';
   for i=1:MaxFH;
      ResultsStr = strvcat(ResultsStr,FirstStr);
      ResultsStr = strvcat(ResultsStr,sprintf('Forecast period %1.0f: %s',i,PredictionSampleStrMat(i,:)));
      ResultsStr = strvcat(ResultsStr,'Variable              Mean      St. Dev.',DivStr);
      for j=1:DSGEModel.n;
         ResultsStr = strvcat(ResultsStr,sprintf(PrtStr,DSGEModel.VariableNames(j,:),AddStr,PredY(j,i),sqrt(CovPred(j,j,i))));
      end;
      ResultsStr = strvcat(ResultsStr,FirstStr,' ');
   end;
   SetLogText(strvcat(' ',ResultsStr,'Output written to:',['Text file: ' GetFilename(ResFile)],['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(ResFile)],' '));
   Status = 1;
end;

%
% end of SequentialPredictiveMoments.m
%
