function DSGEVARMenuFunctions(selector)
% DSGEVARMenuFunctions: switch function for running functions on the DSGE-VAR menu.
%
% USAGE:
%
%       DSGEVARMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 12, 2010.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-02-2010: Updated the documentation.
%
% * 16-02-2010: Changed the order of the input variables for "YADAEnableControls".
%
% * 17-02-2010: Fixed the "YADA" estimation log text.
%
% * 18-02-2010: Added the prior sampling code.
%
% * 19-02-2010: Added code for viewing the VAR equations at prior and posterior mode.
%
% * 20-02-2010: Added code for viewing posterior mode results.
%
% * 21-02-2010: Added code for suggesting the optimal lambda.
%
% * 22-02-2010: Added code for checking the posterior mode.
%
% * 23-02-2010: Added code for computing impulse responses for single parameter values
%               (posterior mode and initial values).
%
% * 24-02-2010: Finished the code for computing impulse respones for single parameter
%               values. Optimization error summary code was also added.
%
% * 25-02-2010: Added code for estimating the structural shocks for single parameters values.
%
% * 26-02-2010: Moved the call to YADAGUI('tab') to the posterior sampling function.
%
% * 01-03-2010: Enabled the call to "DSGEVARSlicePosteriorSampling".
%
% * 02-03-2010: Started work on the posterior sampling summary case.
%
% * 03-03-2010: Took into account that posterior samplign output for the DSGE-VAR no longer
%               uses the selected shocks as an indicator for the filename of result files
%               when dealing with the DSGE model parameters.
%
% * 05-03-2010: Checked the "PostModeResults" case.
%
% * 08-03-2010: The maximum impulse response horizon is not determined by the setting in the
%               posterior mode file, but by the dialog setting.
%
% * 09-03-2010: Added code for displaying the VAR posterior sampling summary, the VAR prior
%               and posterior densities.
%
% * 10-03-2010: Continued work on the prior and posterior densities of the VAR model parameters
%               of the DSGE-VAR. Made sure that controls.tab6 gets focus when the matlab version
%               supports this.
%
% * 19-03-2010: Began work on comparing marginal likelihoods when they've been estimated with
%               the modified harmonic mean. Added an additional input variable to the calls for
%               "GetDSGEVARPostDrawsSelection".
%
% * 12-04-2010: Began work on the function that compares estimated parameters for the posterior
%               distribution across different DSGE-VARs and the DSGE.
%
% * 15-04-2010: Began work on the impulse responses for posterior draws.
%
% * 16-04-2010: Added the impulse responses for prior draws.
%
% * 26-04-2010: Began work on estimating the structural shocks for posterior draws.
%
% * 27-04-2010: Finished the function for estimating the structural shocks for posterior draws.
%               Added the function for estimation these variables for prior draws.
%
% * 19-05-2010: Added the cases for computing forecast error variance decompositions.
%
% * 20-05-2010: Added variance decompositions at posterior mode values and at initial values.
%
% * 21-05-2010: Added the prior and posterior distribution of the variance decompositions.
%
% * 24-05-2010: Added the cases for the observed variable decompositions.
%
% * 25-05-2010: Added the observed variable decompositions for posterior mode values.
%
% * 26-05-2010: Added the observed variable decompositions for initial values, and the observed
%               variable decomposition paths for posterior mode and initial values.
%
% * 27-05-2010: Added the observed variable decomposition paths for the prior and the posterior draws.
%
% * 28-05-2010: Began work on spectral decompositions at posterior mode values.
%
% * 31-05-2010: Finished spectral decompositions at posterior mode and at initial values.
%
% * 01-06-2010: Updated the documentation.
%
% * 02-06-2010: Changed the mat-file names when saving the DSGE-VAR results to disk.
%
% * 08-06-2010: Lowered the minimum number of posterior draws of the DSGE-VAR parameters to 100.
%
% * 09-06-2010: Updated the documentation.
%
% * 22-06-2010: Fixed bug from not initializing "ParamStr".
%
% * 03-11-2010: Began work on simulating data with a DSGE-VAR at posterior mode values.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Fixed path errors for the posterior mode estimated when "PME" is 2 or 3.
%
% * 18-01-2011: Started work on the correlation decompositions.
%
% * 19-01-2011: Fisnihed the correlation decompositions.
%
% * 21-01-2011: Ensured that the "YADA - Bad MAT File" dialogs can be up to 300 pixels in height.
%
% * 07-02-2011: The function only allows plotting of the densities of update parameters when the
%               field "additional" of the structure "ParameterNames" is not empty.
%
% * 08-02-2011: Took into account that "GetUpdatePriorDensity" now provides the "UpdateParameters"
%               output variable.
%
% * 10-02-2011: The function now checks to make sure that "UpdateParameters" is valid for the current set of
%               parameters to update, i.e., that the underlying mat-file is not too old.
%
% * 27-02-2011: Added the "DSGE Posterior Mode" case to the "View|VAR Model" case.
%
% * 28-02-2011: Added additional "DSGE Posterior Mode" cases for the DSGE-VAR models.
%
% * 05-04-2011: Added support for the "ShockAliases" and "ShockColors" variables.
%
% * 08-04-2011: The "ShockAliases" variables is now also used for the names when plotting estimates of the
%               structural shocks.
%
% * 18-04-2011: The "ShockColors" variable can now be initialized based on the color of the shockgroups
%               it belongs to.
%
% * 31-05-2011: Updated the documentation.
%
% * 02-01-2012: Updated the documentation.
%
% * 06-01-2012: Made sure that ShockGroupsColors is only used when it has at least as many rows as
%               there are used shock groups.
%
% * 09-02-2012: began work on the eigenvalues functions.
%
% * 24-02-2012: Started work on the sequential estimation of marginal likelihood.
%
% * 27-02-2012: Continued work on the sequential marginal likelihood estimation.
%
% * 28-02-2012: Finished the sequential marginal likelihood estimation.
%
% * 29-02-2012: Adjusted the size of the wait dialog which is displayed during sequential estimation
%               of the marginal likelihood for the modified harmonic mean estimator.
%
% * 19-03-2012: Started work on the Laplace approximation for the marginal likelihood.
%
% * 20-03-2012: Finished coding the Laplace approximation.
%
% * 22-03-2012: The modified harmonic mean for the full sample can now be based on a subset of the
%               posterior draws.
%
% * 28-03-2012: Took into account that the MargLikeLaplace function accepts a third variable.
%
% * 05-04-2012: Made sure that the Laplace approximation at posterior mode works correctly also
%               when the sample changes, i.e., all sample information is recomputed.
%
% * 02-05-2012: The average value is now also shown for the modified harmonic mean estimator.
%
% * 07-05-2012: When plotting raw posterior draws the user can now select to plot a sub-sample.
%
% * 11-05-2012: Took into account that "LoadVARDSGEPosteriorDraws" now provides "thetaPostSample"
%               as an output variable.
%
% * 14-05-2012: Took into account that the above function now also provides "PostSample".
%
% * 15-05-2012: Changed "Average" to "Mean" and added "Median" estimate for the modified harmonic
%               mean.
%
% * 03-12-2012: Added some additional code to distinguish the SWZ's and Geweke's versions of the modified
%               harmonic mean estimator.
%
% * 06-12-2012: Took into account that "MargLikeModifiedHarmonic" now supports the output variable "NWStdErr".
%
% * 10-12-2012: Began work on SWZ's truncated elliptical version of the modified harmonic mean.
%
% * 11-12-2012: Finished the SWZ's truncated elliptical version of the modified harmonic mean.
%
% * 13-12-2012: Began work on comparing marginal likelihoods with the Sims, Waggoner and Zha version of the
%               modified harmonic mean estimator.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 21-02-2014: Added code for plotting the log posterior kernel values of the DSGE-VAR model for
%               when the marginalized likelihood function is used.
%
% * 24-02-2014: Added code for making scatter-plots of the psoterior draws of the DSGE model parameters.
%
% * 25-02-2014: Added code for plotting the path of the acceptance ratio, the posterior mean, and the
%               posterior median over the post burnin posterior draws. Added code for the CUSUM plots
%               of the log posterior kernel, the original and the transformed parameters, as well as on
%               the separated partial means test for the log posterior kernel.
%
% * 26-02-2014: Finished the code on the separated partial means test for the original and the transformed
%               parameters. began code for the MANOVA convergence statistics.
%
% * 27-02-2014: Finished the MANOVA code.
%
% * 07-03-2014: Added code for providing plots of the Laplace approximation of the posterior densities for
%               the DSGE model parameters based on the marginal or the joint posterior mode estimates.
%
% * 11-03-2014: Started work on the code for plotting the log-likelihood at posterior mode.
%
% * 12-03-2014: Continued working on code for plotting the log-likelihood function.
%
% * 13-03-2014: Finished the function for plotting the log-likelihood at posterior mode.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now takes into account that a Student-t proposal density can be used
%               with the random walk Metropolis posterior sampler of the DSGE model parameters.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 02-04-2017: The function is now partly addapted to running the fixed blocking RWM posterior
%               sampler with a normal proposal density.
%
% * 06-04-2017: The function is now fully adapted to running the fixed blocking RWM posterior sampler
%               with a normal proposal density.
%
% * 10-04-2017: The function is now fully adapted to running the fixed blocking RWM posterior sampler
%               with a Student-t proposal density.
%
% * 18-04-2017: The function is now partly adapted to running the random blocking RWM posterior sampler
%               with a normal proposal density.
%
% * 19-04-2017: The function is now fully adapted to the above sampler.
%
% * 20-04-2017: The function is now fully adapted to running the random blocking RWM posterior sampler
%               with a Student-t proposal density.
%
% * 12-06-2017: The function now provides some support for the system prior.
%
% * 16-06-2017: The Laplace approximations now support the system prior.
%
% * 26-06-2017: Took into account that DSGEVARPriorSampling now requires the input variables ParameterNames,
%               thetaIndex, UniformBounds, and controls.
%
% * 28-06-2017: The view posterior densities for DSGE model parameters function is adjusted to support the
%               system prior.
%
% * 14-08-2017: Added some initial code for using the SMC with likelihood tempering posterior sampler.
%
% * 24-08-2017: Took into account that "PosteriorMeanMedianPlots" takes the "Weights" input variable.
%
% * 25-08-2017: Took into account that "CUSUM" takes the input variable "Weights".
%
% * 30-08-2017: Took into account that "DSGEVARLogLikelihood", "logConcPosteriorPhiDSGEVAR", and 
%               "logPosteriorPhiDSGEVAR" require the input variable "NumWorkers".
%
% * 08-09-2017: Added a call to "DSGEVARSMCLikelihoodTemperingPosteriorSampler". Took into account that
%               "LoadDSGEVARPosteriorDraws" provides the "Weights" output variable.
%
% * 11-09-2017: Added more support for SMC with likelihood tempering with the Weights variable. Took into
%               account that "LoadVARDSGEPosteriorDraws" provides the output variable "Weights". In addition,
%               the functions "DSGEVAROVPsDistribution", "DSGEVARVDsDistribution", and "DSGEVARIRFsDistribution"
%               require the input variable "Weights".
%
% * 12-09-2017: Added function for comparing marginal likelihoods for DSGE-VAR models using the SMC based
%               estimates.
%
% * 15-09-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
lasterr = '';
%
switch selector
   case 'DSGEVARMargPostMode'
      if get(controls.about,'UserData')==0;
         %
         % This case attempts to find the posterior mode through numerical optimization of the joint
         % posterior distribution
         %
         if (strcmp(lower(DSGEModel.UpdateParameterFile),lower(pwd))==1)&(strcmp(lower(DSGEModel.InitializeParameterFile),lower(pwd))==1);
            txt = 'Are you sure that you have specified all the parameters that your DSGE model requires?';
            answer = Query(txt,'question',120,'No Additional Parameter Files',400,'no',CurrINI);
         else;
            answer = 'Yes';
         end;
         if strcmp(lower(answer),'yes')==1;
            oldtab = get(controls.tab1,'UserData');
            set(maingui,'CurrentObject',controls.tab6);
            YADAGUI('tab');
            if MatlabNumber>=7;
               uicontrol(controls.tab6);
            end;
            set(controls.about,'UserData',1);
            drawnow;
            %
            % check if we should add some diving line to the log-text control
            %
            LogText = get(controls.estimation_log,'String');
            if isempty(LogText)==0;
               SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
            end;
            %
            % Run the Posterior mode estimation routine
            %
            MarginalLambda = DSGEVARMargPosteriorModeEstimation(DSGEModel,CurrINI,controls);
            if isempty(MarginalLambda)==0;
               DSGEModel.MarginalLambda = MarginalLambda;
            end;
            %
            % we need to enable these things
            %
            set(controls.about,'UserData',0);
         end;
      end;
   case 'DSGEVARJointPostMode'
      if get(controls.about,'UserData')==0;
         %
         % This case attempts to find the posterior mode through numerical optimization of the joint
         % posterior distribution
         %
         if (strcmp(lower(DSGEModel.UpdateParameterFile),lower(pwd))==1)&(strcmp(lower(DSGEModel.InitializeParameterFile),lower(pwd))==1);
            txt = 'Are you sure that you have specified all the parameters that your DSGE model requires?';
            answer = Query(txt,'question',120,'No Additional Parameter Files',400,'no',CurrINI);
         else;
            answer = 'Yes';
         end;
         if strcmp(lower(answer),'yes')==1;
            oldtab = get(controls.tab1,'UserData');
            set(maingui,'CurrentObject',controls.tab6);
            YADAGUI('tab');
            if MatlabNumber>=7;
               uicontrol(controls.tab6);
            end;
            set(controls.about,'UserData',1);
            drawnow;
            %
            % check if we should add some diving line to the log-text control
            %
            LogText = get(controls.estimation_log,'String');
            if isempty(LogText)==0;
               SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
            end;
            %
            % Run the Posterior mode estimation routine
            %
            JointLambda = DSGEVARPosteriorModeEstimation(DSGEModel,CurrINI,controls);
            if isempty(JointLambda)==0;
               DSGEModel.JointLambda = JointLambda;
            end;
            %
            % we need to enable these things
            %
            set(controls.about,'UserData',0);
         end;
      end;
   case 'PostSampling'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % run the posterior sampler
         %
         if DSGEModel.PosteriorSampler==1;
            %
            % random walk Metropolis with normal proposal density
            %
            DSGEVARRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==2;
            %
            % slice sampler
            %
            DSGEVARSlicePosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==3;
            %
            % random walk Metropolis with a Student-t proposal density
            %
            DSGEVARRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==4;
            %
            % fixed blocking RWM with normal proposal
            %
            DSGEVARFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==5;
            %
            % fixed blocking RWM with a Student-t proposal
            %
            DSGEVARFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==6;
            %
            % random blocking RWM with a normal proposal
            %
            DSGEVARRandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==7;
            %
            % random blocking RWM with a Student-t proposal
            %
            DSGEVARRandomBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         elseif DSGEModel.PosteriorSampler==8;
            %
            % SMC with likelihood tempering
            %
            DSGEVARSMCLikelihoodTemperingPosteriorSampler(DSGEModel,CurrINI,maingui,controls);
         end;
         %
         set(controls.about,'UserData',0);
%         set(maingui,'CurrentObject',getfield(controls,['tab' num2str(oldtab)]));
%         YADAGUI('tab');
      end;
   case 'VARPostSampling'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % run the VAR posterior sampling function
         %
         DSGEVARPosteriorSampling(DSGEModel,CurrINI,maingui,controls);
         %
         set(controls.about,'UserData',0);
      end;
   case 'PriorSampling'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % try to locate a suitable posterior mode results file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            try;
               ModeData = load(ModeFile);
               Status = 1;
            catch;
               [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
               if Status==0;
                  [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
               end;
               ModeData = [];
            end;
         end;
         if Status==1;
            ErrorStr = '';
            if isempty(ModeData)==1;
               ModeFile = StringTrim(Files(1,:));
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the DSGE model prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'DSGE-VAR Prior Sampling',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  LowerBound = ModeData.LowerBound;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               if FileExist(DSGEModel.PriorFile)==1;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % suggest how many prior draws to compute
            %
            PriorDrawsStr = get(controls.posterior.postdrawsusage,'String');
            PostDrawsUsageValue = DSGEModel.PostDrawsUsageValue;
            if (PostDrawsUsageValue<10)&(isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0);
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  PostDrawsUsageValue = 10;
               end;
            end;
            %
            % open a selection dialog
            %
            [action,PriorDrawsValue] = SelectionDlg(PriorDrawsStr,PostDrawsUsageValue,'Select the number of draws from the prior distribution:','Sampling Selection','DSGE-VAR Prior Sampling','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               NumPriorDraws = str2num(strrep(StringTrim(PriorDrawsStr(PriorDrawsValue,:)),',',''));
               %
               % run the prior sampling function
               %
               DSGEVARPriorSampling(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,DSGEModel,CurrINI,controls);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'VARModelInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','DSGE-VAR Model - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                  if IdentifyStatus==0;
                     A0 = [];
                  end;
                  [ModelEquations,IndentStr] = DSGEVARModelEquations(Phi,A0,DSGEModel.VariableNames,DSGEModel.XVariableNames,ShockNames,DSGEModel.BVARLags);
                  SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                  SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                  %
                  % write the results to file
                  %
                  TextFileName = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Equations-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-InitialValues.txt']);
                  fid = fopen(TextFileName,'wt');
                  fprintf(fid,'*********************************************************************************************************\n');
                  fprintf(fid,'*                                                                                                       *\n');
                  fprintf(fid,'*                                  D S G E - V A R   E Q U A T I O N S                                  *\n');
                  fprintf(fid,'*                                                                                                       *\n');
                  fprintf(fid,'*********************************************************************************************************\n\n');
                  fprintf(fid,'Prior mode results for lambda = %0.4f:\n\n',lambdai);
                  %
                  for i=1:size(ModelEquations,1);
                     %
                     % make sure each equation is wrapped.
                     %
                     MEStr = StrWrap(StringTrim(ModelEquations(i,:)),80);
                     for j=1:size(MEStr,1);
                        if j>1;
                           fprintf(fid,'%s%s\n',IndentStr,MEStr(j,:));
                        else;
                           fprintf(fid,'%s\n',MEStr(1,:));
                        end;
                     end;
                     fprintf(fid,'\n');
                  end;
                  %
                  fprintf(fid,'=========================================================================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFileName));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFileName));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  % open the file
                  %
                  if FileExist(TextFileName)==1;
                     TextGUI(TextFileName,['DSGE-VAR Model At Prior Mode - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'VARModelPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         % check if joint or marginal mode
         %
         if gcbo==controls.dsgevarview.varmodel.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            FileAddOn = 'Marginal';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.varmodel.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            FileAddOn = 'Joint';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.varmodel.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            FileAddOn = 'DSGE';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['DSGE-VAR Model - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         %
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % collect the data we need
               %
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  IdentifyModeStatus = ModeData.IdentifyModeStatus;
                  if IdentifyModeStatus==1;
                     A0 = ModeData.A0Mode;
                  else;
                     A0 = [];
                  end;
                  ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
                  ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
               end;
               %
               if ModeStatus==1;
                  [ModelEquations,IndentStr] = DSGEVARModelEquations(Phi,A0,ModeData.DSGEModel.VariableNames,ModeData.DSGEModel.XVariableNames,ShockNames,DSGEModel.BVARLags);
                  SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                  SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                  %
                  % write to file
                  %
                  TextFileName = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Equations-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' FileAddOn 'PosteriorMode.txt']);
                  fid = fopen(TextFileName,'wt');
                  fprintf(fid,'*********************************************************************************************************\n');
                  fprintf(fid,'*                                                                                                       *\n');
                  fprintf(fid,'*                                  D S G E - V A R   E Q U A T I O N S                                  *\n');
                  fprintf(fid,'*                                                                                                       *\n');
                  fprintf(fid,'*********************************************************************************************************\n\n');
                  fprintf(fid,'%s posterior mode results for lambda = %0.4f:\n\n',FileAddOn,lambdai);
                  %
                  for i=1:size(ModelEquations,1);
                     %
                     % make sure each equation is wrapped.
                     %
                     MEStr = StrWrap(StringTrim(ModelEquations(i,:)),80);
                     for j=1:size(MEStr,1);
                        if j>1;
                           fprintf(fid,'%s%s\n',IndentStr,MEStr(j,:));
                        else;
                           fprintf(fid,'%s\n',MEStr(1,:));
                        end;
                     end;
                     fprintf(fid,'\n');
                  end;
                  %
                  fprintf(fid,'=========================================================================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFileName));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFileName));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  % open the file
                  %
                  if FileExist(TextFileName)==1;
                     TextGUI(TextFileName,['DSGE-VAR Model At ' ModeType ' - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'EigenvaluesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Eigenvaues - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the eigenvalues
                  %
                  EV = BVAREigenvaluesPostMode(Phi(:,DSGEModel.k+1:size(Phi,2)));
                  %
                  % display the eigenvalues
                  %
                  PlotBVAREigenvalues(EV,['DSGE-VAR(' num2str(lambdai) ') Model'],['DSGE-VAR(' num2str(lambdai) ') model with ' int2str(DSGEModel.BVARLags) ' lags'],'Initial Values',CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;         
   case 'EigenvaluesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         % check if joint or marginal mode
         %
         if gcbo==controls.dsgevarview.eigenvalues.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.eigenvalues.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.eigenvalues.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['DSGE-VAR Model - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         %
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % collect the data we need
               %
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               end;
               %
               if ModeStatus==1;
                  %
                  % compute the eigenvalues
                  %
                  EV = BVAREigenvaluesPostMode(Phi(:,DSGEModel.k+1:size(Phi,2)));
                  %
                  % display the eigenvalues
                  %
                  PlotBVAREigenvalues(EV,['DSGE-VAR(' num2str(lambdai) ') Model'],['DSGE-VAR(' num2str(lambdai) ') model with ' int2str(DSGEModel.BVARLags) ' lags'],ModeType,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;  
   case 'PostModeResults'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         % check if joint or marginal mode
         %
         Status = 0;
         if gcbo==controls.dsgevarview.postmoderes.margmode;
            [Status,lambda,Files] = VerifyDSGEVARPostModeTxtFiles(DSGEModel,'Marginal','Results');
            ModeType = 'Marginal Posterior Mode';
            FileAddOn = 'Results';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.postmodesum.margmode;
            [Status,lambda,Files] = VerifyDSGEVARPostModeTxtFiles(DSGEModel,'Marginal','Summary');
            ModeType = 'Marginal Posterior Mode';
            FileAddOn = 'Summary';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.postmoderes.jointmode;
            [Status,lambda,Files] = VerifyDSGEVARPostModeTxtFiles(DSGEModel,'Joint','Results');
            ModeType = 'Joint Posterior Mode';
            FileAddOn = 'Results';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.postmodesum.jointmode;
            [Status,lambda,Files] = VerifyDSGEVARPostModeTxtFiles(DSGEModel,'Joint','Summary');
            ModeType = 'Joint Posterior Mode';
            FileAddOn = 'Summary';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         end;
         %
         if Status==1;
            %
            % get suitable data for the selection dialog
            %
            if length(lambda)>1;
               [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
               [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['DSGE-VAR ' ModeType ' - ' FileAddOn],'','',CurrINI);
            else;
               action = 'OK';
               value = 1;
            end;
            if strcmp(lower(action),'ok')==1;
               ModeFile = StringTrim(Files(value,:));
               %
               % open file
               %
               if FileExist(ModeFile)==1;
                  TextGUI(ModeFile,['DSGE-VAR ' ModeType ' Estimation ' FileAddOn ' - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
               end;
            end;
         else;
            txt = ['YADA couldn''t locate any ' lower(ModeType) ' ' lower(FileAddOn) ' files in the "' DSGEModel.OutputDirectory '" directory. Please check if you have moved or deleted files.'];
            About(txt,'information',['DSGE-VAR ' ModeType ' - ' FileAddOn],200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OptErrorSummary'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         % check if joint or marginal mode
         %
         Status = 0;
         if gcbo==controls.dsgevarview.opterror.margmode;
            [Status,Lambda,Files] = VerifyDSGEVAROptErrorFiles(DSGEModel,'Marg');
            ModeType = 'Marginal Posterior Mode';
         elseif gcbo==controls.dsgevarview.opterror.jointmode;
            [Status,Lambda,Files] = VerifyDSGEVAROptErrorFiles(DSGEModel,'Joint');
            ModeType = 'Joint Posterior Mode';
         end;
         %
         if Status==1;
            OptErrorFile = '';
            if length(Lambda)>1;
               LambdaValuesStr = '';
               for i=1:length(Lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(Lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(Lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',['Optimization Error Summary - ' ModeType],'','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  OptErrorFile = StringTrim(Files(value,:));
               end;
            else;
               OptErrorFile = StringTrim(Files);
            end;
            if isempty(OptErrorFile)==0;
               TextGUI(OptErrorFile,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
            end;
         else;
            txt = ['YADA couldn''t locate any ' lower(ModeType) ' optimization error summary files in the "' DSGEModel.OutputDirectory '" directory. Please check if you have moved or deleted files.'];
            About(txt,'information',['DSGE-VAR ' ModeType ' - ' FileAddOn],200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorSamplingSummary'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we have multiple markov chains
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','DSGE-VAR Posterior Sampling Summary - DSGE','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         if strcmp(lower(action),'cancel')==0;
            %
            % determine which results are available
            %
            [status,PME] = VerifyDSGEVARSamplingTxtFiles(DSGEModel,CurrChain,controls);
            if status==1;
               %
               % check which posterior mode estimators the use can choose between
               %
               AvailablePME = 1-[isempty(PME(1).Lambda) isempty(PME(2).Lambda) isempty(PME(3).Lambda)];
               if sum(AvailablePME)>1;
                  PMEStr = '';
                  if AvailablePME(1)==1;
                     PMEStr = 'DSGE model posterior mode';
                  end;
                  if AvailablePME(2)==1;
                     PMEStr = strvcat(PMEStr,'DSGE-VAR marginal posterior mode');
                  end;
                  if AvailablePME(3)==1;
                     PMEStr = strvcat(PMEStr,'DSGE-VAR joint posterior mode');
                  end;
                  %
                  % display a selection dialog
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     [action,SelectedPME] = SelectionDlg(PMEStr,1,'Select the posterior mode estimator used in posterior sampling:','Estimator Selection','DSGE-VAR Posterior Sampling Summary - DSGE','','',CurrINI);
                  else;
                     action = 'OK';
                     SelectedPME = 1;
                  end;
               else;
                  UsePME = AvailablePME.*(1:3);
                  UsePME = UsePME(UsePME>0);
                  action = 'OK';
                  SelectedPME = [];
               end;
               if strcmp(lower(action),'ok')==1;
                  %
                  % check if we have multiple lambda values to choose between
                  %
                  if isempty(SelectedPME)==0;
                     PMEStr = StringTrim(PMEStr(SelectedPME,:));
                     if strcmp(PMEStr,'DSGE model posterior mode')==1;
                        UsePME = 1;
                     elseif strcmp(PMEStr,'DSGE-VAR marginal posterior mode')==1;
                        UsePME = 2;
                     else;
                        UsePME = 3;
                     end;
                  end;
                  Lambda = PME(UsePME).Lambda;
                  if length(Lambda)>1;
                     LambdaValuesStr = '';
                     for i=1:length(Lambda);
                        LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(Lambda(i),'%0.4f')]);
                     end;
                     [action,value] = SelectionDlg(LambdaValuesStr,length(Lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','DSGE-VAR Posterior Sampling Summary - DSGE','','',CurrINI);
                     if strcmp(lower(action),'ok')==0;
                        value = 0;
                     end;
                  else;
                     value = 1;
                  end;
                  if value>0;
                     lambda = Lambda(value);
                     %
                     DrawsStr = get(controls.posterior.draws,'String');
                     NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                     %
                     TempStagesStr = get(controls.posterior.temperingstages,'String');
                     TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
                     %
                     if DSGEModel.PosteriorSampler==1;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==2;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\SlicePosteriorSamplingSummary-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==3;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==4;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==5;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==6;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==7;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     elseif DSGEModel.PosteriorSampler==8;
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\SMCPosteriorSamplingSummary-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
                     end;
                     TextGUI(TextFile,['DSGE Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
                  end;
               end;
            else;
               txt = ['YADA couldn''t locate any posterior sampling summary files on disk for posterior chain number ' int2str(CurrChain) '. Please choose a different Markov chain.'];
               About(txt,'information','DSGE-VAR Posterior Sampling Summary - DSGE',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'VARPosteriorSamplingSummary'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we have multiple markov chains
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','DSGE-VAR Posterior Sampling Summary - VAR','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         if strcmp(lower(action),'cancel')==0;
            %
            % determine which results are available
            %
            [status,PME] = VerifyVARDSGESamplingTxtFiles(DSGEModel,CurrChain,controls);
            if status==1;
               %
               % check which posterior mode estimators the use can choose between
               %
               AvailablePME = 1-[isempty(PME(1).Lambda) isempty(PME(2).Lambda) isempty(PME(3).Lambda)];
               if sum(AvailablePME)>1;
                  PMEStr = '';
                  if AvailablePME(1)==1;
                     PMEStr = 'DSGE model posterior mode';
                  end;
                  if AvailablePME(2)==1;
                     PMEStr = strvcat(PMEStr,'DSGE-VAR marginal posterior mode');
                  end;
                  if AvailablePME(3)==1;
                     PMEStr = strvcat(PMEStr,'DSGE-VAR joint posterior mode');
                  end;
                  %
                  % display a selection dialog
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     [action,SelectedPME] = SelectionDlg(PMEStr,1,'Select the posterior mode estimator used in posterior sampling:','Estimator Selection','DSGE-VAR Posterior Sampling Summary - VAR','','',CurrINI);
                  else;
                     action = 'OK';
                     SelectedPME = 1;
                  end;
               else;
                  UsePME = AvailablePME.*(1:3);
                  UsePME = UsePME(UsePME>0);
                  action = 'OK';
                  SelectedPME = [];
               end;
               if strcmp(lower(action),'ok')==1;
                  %
                  % check if we have multiple lambda values to choose between
                  %
                  if isempty(SelectedPME)==0;
                     PMEStr = StringTrim(PMEStr(SelectedPME,:));
                     if strcmp(PMEStr,'DSGE model posterior mode')==1;
                        UsePME = 1;
                     elseif strcmp(PMEStr,'DSGE-VAR marginal posterior mode')==1;
                        UsePME = 2;
                     else;
                        UsePME = 3;
                     end;
                  end;
                  Lambda = PME(UsePME).Lambda;
                  if length(Lambda)>1;
                     LambdaValuesStr = '';
                     for i=1:length(Lambda);
                        LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(Lambda(i),'%0.4f')]);
                     end;
                     [action,value] = SelectionDlg(LambdaValuesStr,length(Lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','DSGE-VAR Posterior Sampling Summary - VAR','','',CurrINI);
                     if strcmp(lower(action),'ok')==0;
                        value = 0;
                     end;
                  else;
                     value = 1;
                  end;
                  if value>0;
                     lambda = Lambda(value);
                     %
                     DrawsStr = get(controls.posterior.draws,'String');
                     NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                     if DSGEModel.PosteriorSampler<=7;
                        BurninStr = get(controls.posterior.burnin,'String');
                        NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
                     else;
                        NumBurnin = 0;
                     end;
                     NumPostDraws = NumPostDraws-NumBurnin;
                     %
                     if DSGEModel.PosteriorSampler<=7;
                        DrawsPercentStr = get(controls.posterior.usepostdraws,'String');
                        DrawsPercent = str2num(strrep(StringTrim(DrawsPercentStr(DSGEModel.PostDrawsPercentValue,:)),'%',''));
                        UsedDraws = ceil(DrawsPercent*NumPostDraws/100);
                     else;
                        UsedDraws = NumPostDraws;
                     end;
                     %
                     SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                     SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                     %
                     % number of tempering stages
                     %
                     TempStagesStr = get(controls.posterior.temperingstages,'String');
                     TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
                     %
                     PostSampleStr = [int2str(DSGEModel.InverseHessianEstimator) '.' int2str(DSGEModel.PosteriorSampler)];
                     if DSGEModel.PosteriorSampler==3;
                        PostSampleStr = [PostSampleStr '-Student-' int2str(DSGEModel.StudenttDegFree)];
                     elseif DSGEModel.PosteriorSampler==4;
                        PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks)];
                     elseif DSGEModel.PosteriorSampler==5;
                        PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-Student-' int2str(DSGEModel.StudenttDegFree)];
                     elseif DSGEModel.PosteriorSampler==6;
                        PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2))];
                     elseif DSGEModel.PosteriorSampler==7;
                        PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-Student-' int2str(DSGEModel.StudenttDegFree)];
                     elseif DSGEModel.PosteriorSampler==8;
                        PostSampleStr = [int2str(DSGEModel.PosteriorSampler) '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm)];
                     end;
                     %
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsePME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(UsedDraws) '.' int2str(NumPostDraws) '.txt']);
                     if FileExist(TextFile)==1;
                        TextGUI(TextFile,['VAR Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
                     end;
                  end;
               end;
            else;
               txt = ['YADA couldn''t locate any posterior sampling summary files on disk for posterior chain number ' int2str(CurrChain) '. Please choose a different Markov chain.'];
               About(txt,'information','DSGE-VAR Posterior Sampling Summary - VAR',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'RawPosteriorDraws'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Raw Posterior Draws',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            NumPostDraws = length(LogPost);
            %
            % check which parameters to display
            %
            if gcbo==controls.dsgevarview.rawdraws.originalparameters;
               TypeStr = 'Original';
               ParamSample = thetaPostSample;
            elseif gcbo==controls.dsgevarview.rawdraws.transformedparameters;
               TypeStr = 'Transformed';
               ParamSample = PostSample;
            elseif gcbo==controls.dsgevarview.rawdraws.weights;
               TypeStr = 'Weights';
               ParamSample = Weights;
            else;
               ParamSample = LogPost;
               TypeStr = 'Log Posterior Kernel Values';
               thetaPost = thetaPostSample;
            end;
            %
            clear('PostSample','thetaPostSample','LogPost','AcceptedDraws');
            %
            % ask if the user wants to plot a subsample
            %
            if (gcbo==controls.dsgevarview.rawdraws.originalparameters)|(gcbo==controls.dsgevarview.rawdraws.transformedparameters);
               if DSGEModel.PosteriorSampler<=7;
                  txt = 'Would you like to plot a sub-sample of the posterior draws rather than the full sample?';
                  answer = Query(txt,'question',400,['DSGE-VAR(' num2str(lambda) ') Raw Posterior Draws - ' TypeStr ' Parameters'],600,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % check which draws we should use
                     %
                     ParamSample = ParamSample(NumBurnin+1:NumPostDraws,:);
                     NumDraws = NumPostDraws-NumBurnin;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     ParamSample = ParamSample(UseDraws,:);
                  end;
                  drawnow;
               end;
               %
               % send the data to the raw plotting function
               %
               RawDSGEVARPosteriorPlots(ParamSample,lambda,PME,NumBurnin,NumPostDraws,CurrChain,TypeStr,DSGEModel,CurrINI);
            elseif gcbo==controls.dsgevarview.rawdraws.weights;
               %
               % send the data to the raw plotting function
               %
               drawnow;
               RawWeightsPosteriorPlots(Weights,lambda,NumPostDraws,CurrChain,DSGEModel,CurrINI,controls);
            else;
               if DSGEModel.PosteriorSampler<=7;
                  txt = 'Would you like to plot a sub-sample of the log posterior kernel values for the posterior draws rather than the full sample?';
                  answer = Query(txt,'question',400,['DSGE-VAR(' num2str(lambda) ') Raw Posterior Draws - ' TypeStr],600,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % check which draws we should use
                     %
                     ParamSample = ParamSample(NumBurnin+1:NumPostDraws,:);
                     NumDraws = NumPostDraws-NumBurnin;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     ParamSample = ParamSample(UseDraws,:);
                     thetaPost = thetaPost(UseDraws,:);
                  end;
                  drawnow;
               end;
               %
               RawDSGEVARLogPosteriorPlot(ParamSample,thetaPost,lambda,PME,NumBurnin,NumPostDraws,CurrChain,DSGEModel,CurrINI);
               clear('thetaPost');
            end;
            %
            % clear the used parameter draws
            %
            clear('ParamSample');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ScatterPlots'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Scatter-Plot Posterior Draws',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
            SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
            if PME==1;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            elseif PME==2;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            elseif PME==3;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            end;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % load the posterior draws
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
               NumDraws = length(LogPost);
               %
               % check which parameters to display
               %
               if gcbo==controls.dsgevarview.scatterplots.originalparameters;
                  TypeStr = 'Original';
                  ParamSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  ParamMode = ModeData.thetaMode;
               else;
                  TypeStr = 'Transformed';
                  ParamSample = PostSample(NumBurnin+1:NumDraws,:);
                  ParamMode = ModeData.phiMode;
               end;
               NumDraws = NumDraws-NumBurnin;
               %
               % clear posterior sample data
               %
               clear('PostSample','thetaPostSample','AcceptedDraws');
               %
               % check which draws we should use
               %
               if DSGEModel.PosteriorSampler<=7;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                  if length(UseDraws)<NumDraws;
                     ParamSample = ParamSample(UseDraws,:);
                     LogPost = LogPost(UseDraws);
                     UsedDraws = length(UseDraws);
                  else;
                     UsedDraws = NumDraws;
                  end;
               end;
               %
               % send the data to the scatter-plotting function
               %
               drawnow;
               ScatterPlotPosteriorDraws('init',ParamSample,LogPost,ModeData.ParameterNames.estimated,ParamMode,CurrChain,['DSGE-VAR(' num2str(lambda) ')'],TypeStr,DSGEModel,CurrINI);
               %
               % clear the used parameter draws
               %
               clear('ParamSample','LogPost');
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CUSUMLogPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR CUSUM Plots - Log Posterior Kernel',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            Weights = [];
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('PostSample','AcceptedDraws','thetaPostSample');
            %
            % send the data to the cusum computation function
            %
            drawnow;
            [CUSUMPost,CUSUMAverage] = CUSUM(LogPost,Weights,NumBurnin,'Log Posterior Kernel',lambda,CurrINI);
            %
            if isempty(CUSUMPost)==0;
               %
               % display the results
               %
               CUSUMPlots(CUSUMPost,CUSUMAverage,NumBurnin,CurrChain,'logpost',lambda,PME,DSGEModel,CurrINI);
            end;
            % clear everything
            %
            clear('LogPost');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CUSUMParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevarview.convergence.cusumplots.original;
            ParamStr = 'Original';
         else;
            ParamStr = 'Transformed';
         end;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection(['DSGE-VAR CUSUM Plots - ' ParamStr ' Parameters'],DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            Weights = [];
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            if strcmp(lower(ParamStr),'original')==1;
               ParamDraws = thetaPostSample;
            else;
               ParamDraws = PostSample;
            end;
            clear('PostSample','AcceptedDraws','LogPost','thetaPostSample');
            %
            % send the data to the cusum computation function
            %
            drawnow;
            [CUSUMPost,CUSUMAverage] = CUSUM(ParamDraws,Weights,NumBurnin,strrep(strrep(get(gcbo,'Label'),'&',''),'...',''),lambda,CurrINI);
            %
            if isempty(CUSUMPost)==0;
               %
               % display the results
               %
               CUSUMPlots(CUSUMPost,CUSUMAverage,NumBurnin,CurrChain,ParamStr,lambda,PME,DSGEModel,CurrINI);
            end;
            %
            % clear everything
            %
            clear('ParamDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SPMTLogPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Separated Partial Means - Log Posterior Kernel',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('PostSample','AcceptedDraws','thetaPostSample');
            NumIter = size(LogPost,1);
            %
            % check how many separated partial means to compute
            %
            NumMeansStr = '';
            for i=2:20;
               if IsInteger((NumIter-NumBurnin)/(2*i))==1;
                  NumMeansStr = strvcat(NumMeansStr,int2str(i));
               end;
            end;
            %
            if isempty(NumMeansStr)==0;
               [action,NumPM] = SelectionDlg(NumMeansStr,1,'Select the number of partial means to allow for','Partial Means Selection','DSGE-VAR Separated Partial Means - Log Posterior Kernel','','',CurrINI);
               %
               % check if the action was cancelled
               %
               if strcmp(lower(action),'cancel')==0;
                  NumPM = str2num(StringTrim(NumMeansStr(NumPM,:)));
                  %
                  % check if we have already calculated the separated partial means test for the selected chain and number of partial means
                  %
                  SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                  SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                  ChainsStr = get(controls.posterior.chains,'String');
                  NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-SeparatedPartialMeans-LogPosteriorKernel-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' num2str(NumPM,'%0.0f') '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.txt']);
                  %
                  if FileExist(ResFile)==1;
                     txt = ['YADA has already computed the separated partial means test for ' num2str(NumPM,'%0.0f') ' separated partial means. Would you like to view this file? If you push the "No" button then YADA will recompute the tests.'];
                     GewekeStat = 1;
                     RunTest = 0;
                     answer = Query(txt,'question',200,'DSGE-VAR Separated Partial Means Test - Log Posterior Kernel',600,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        RunTest = 1;
                     end;
                  else;
                     RunTest = 1;
                  end;
                  %
                  if RunTest==1;
                     GewekeStat = SeparatedPartialMeansTest(LogPost(NumBurnin+1:NumIter,:),Weights,NumPM,'Log Posterior Kernel',lambda,CurrINI);
                     %
                     % write the results to file
                     %
                     if isempty(GewekeStat)==0;
                        PrintSeparatedPartialMeans(GewekeStat,NumPM,CurrChain,NumChains,NumIter,NumBurnin,'Log Posterior Kernel','log posterior',lambda,DSGEModel,CurrINI);
                     end;
                  end;
                  if (isempty(GewekeStat)==0)&(FileExist(ResFile)==1);
                     TextGUI(ResFile,['DSGE-VAR(' num2str(lambda) ') Separated Partial Means Test - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Log Posterior Kernel',50,CurrINI,1,1);
                  end;
               end;
            else;
               txt = ['YADA was unable to find any interger p between 2 and 20 such that ' num2str(NumIter-NumBurnin,'%0.0f') '/2p is an integer. YADA is therefore unable to calculate Geweke''s separated partial means test.'];
               About(txt,'information','DSGE-VAR Separated Partial Means - Log Posterior Kernel',200,600,CurrINI);
            end;
            %
            % clear everything
            %
            clear('LogPost');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SPMTParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevarview.convergence.spmt.original;
            ParamStr = 'Original';
         else;
            ParamStr = 'Transformed';
         end;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection(['DSGE-VAR Separated Partial Means - ' ParamStr ' Parameters'],DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % Try to load the posterior mode estimates
            %
            SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
            SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
            if PME==1;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            elseif PME==2;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            elseif PME==3;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            end;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            %
            % check if we could successfully load the data
            %
            if isempty(ErrorStr)==1;
               ParameterNames = ModeData.ParameterNames;
               %
               % load the posterior draws
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
               if strcmp(lower(ParamStr),'original')==1;
                  ParamDraws = thetaPostSample;
               else;
                  ParamDraws = PostSample;
               end;
               NumIter = size(LogPost,1);
               clear('PostSample','AcceptedDraws','LogPost','thetaPostSample');
               %
               % check how many separated partial means to compute
               %
               NumMeansStr = '';
               for i=2:20;
                  if IsInteger((NumIter-NumBurnin)/(2*i))==1;
                     NumMeansStr = strvcat(NumMeansStr,int2str(i));
                  end;
               end;
               %
               if isempty(NumMeansStr)==0;
                  [action,NumPM] = SelectionDlg(NumMeansStr,1,'Select the number of partial means to allow for','Partial Means Selection',['DSGE-VAR Separated Partial Means - ' ParamStr ' Parameters'],'','',CurrINI);
                  %
                  % check if the action was cancelled
                  %
                  if strcmp(lower(action),'cancel')==0;
                     NumPM = str2num(StringTrim(NumMeansStr(NumPM,:)));
                     %
                     % check if we have already calculated the separated partial means test for the selected chain and number of partial means
                     %
                     ChainsStr = get(controls.posterior.chains,'String');
                     NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-SeparatedPartialMeans-' ParamStr 'Parameters-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' num2str(NumPM,'%0.0f') '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.txt']);
                     if FileExist(ResFile)==1;
                        txt = ['YADA has already computed the separated partial means test for ' num2str(NumPM,'%0.0f') ' separated partial means. Would you like to view this file? If you push the "No" button then YADA will recompute the tests.'];
                        GewekeStat = 1;
                        RunTest = 0;
                        answer = Query(txt,'question',200,['DSGE-VAR Separated Partial Means Test - ' ParamStr ' Parameters'],600,'no',CurrINI);
                        if strcmp(lower(answer),'no')==1;
                           RunTest = 1;
                        end;
                     else;
                        RunTest = 1;
                     end;
                     %
                     if RunTest==1;
                        GewekeStat = SeparatedPartialMeansTest(ParamDraws(NumBurnin+1:NumIter,:),Weights,NumPM,[ParamStr ' Parameters'],lambda,CurrINI);
                        %
                        % write the results to file
                        %
                        if isempty(GewekeStat)==0;
                           PrintSeparatedPartialMeans(GewekeStat,NumPM,CurrChain,NumChains,NumIter,NumBurnin,[ParamStr ' Parameters'],ParameterNames.estimated,lambda,DSGEModel,CurrINI);
                        end;
                     end;
                     if (isempty(GewekeStat)==0)&(FileExist(ResFile)==1);
                        TextGUI(ResFile,['DSGE-VAR(' num2str(lambda) ') Separated Partial Means Test - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],[ParamStr ' Parameters'],50,CurrINI,1,1);
                     end;
                  end;
               else;
                  txt = ['YADA was unable to find any interger p between 2 and 20 such that ' num2str(NumIter-NumBurnin,'%0.0f') '/2p is an integer. YADA is therefore unable to calculate Geweke''s separated partial means test.'];
                  About(txt,'information',['DSGE-VAR Separated Partial Means - ' ParamStr ' Parameters'],200,600,CurrINI);
               end;
               %
               % clear everything
               %
               clear('ParamDraws');
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeAcceptance'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Posterior Sampler Acceptance Ratio',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('PostSample','LogPost','thetaPostSample');
            %
            % send the data to the acceptance ratio plotting function
            %
            drawnow;
            AcceptanceRatioPlot(AcceptedDraws,NumBurnin,CurrChain,lambda,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('AcceptedDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SMCAdapation'
   	 if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Posterior Sampler Acceptance Ratio',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            ChainsStr = get(controls.posterior.chains,'String');
         	NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         	%
				% Number of posterior draws
				%
				DrawsStr = get(controls.posterior.draws,'String');
				NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
				%
				% number of tempering stages
				%
				TempStagesStr = get(controls.posterior.temperingstages,'String');
				TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
				%
				if DSGEModel.PosteriorSampler==8;
					PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumPostDraws,'%0.0f') '.mat']);
				else;
					PostSampleFile = '';
				end;
				if FileExist(PostSampleFile)==1;
					PSData = load(PostSampleFile);
					NumberAcceptedDraws = PSData.NumberAcceptedDraws;
					ScaleParameter = PSData.ScaleParameter;
					EffectiveSampleSize = PSData.EffectiveSampleSize;
					pstar = PSData.pstar;
					ResamplingThreshold = PSData.ResamplingThreshold;
					clear('PSData');
					%
					% graph the results
					%
					GraphSMCAdaption(NumberAcceptedDraws,ScaleParameter,EffectiveSampleSize,ResamplingThreshold,pstar,lambda,NumPostDraws,CurrChain,NumChains,DSGEModel,CurrINI);
				else;
					txt = ['The file for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' is not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
				end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeMANOVA'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,PME,Lambda] = GetDSGEVARMultipleChainsPostDraws('DSGE-VAR Multivariate ANOVA',DSGEModel,controls,CurrINI,1);
         if value>0;
            lambda = Lambda(value);
            %
            % determine the current number of MCMC chains
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            % determine the number of posterior draws
            %
            DrawsStr = get(controls.posterior.draws,'String');
            NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
            %
            % determine the number of burn-in draws
            %
            BurninStr = get(controls.posterior.burnin,'String');
            NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
            %
            SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
            SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
            %
            % check the results file
            %
            ResFile = FixFilePath([DSGEModel.OutputDirectory '\manova\MANOVA-DSGE-VAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
            if FileExist(ResFile)==1;
               txt = ['You have already computed the multivariate potential scale reduction factors for ' num2str(NumChains,'%0.0f') ' MCMC chains with ' num2str(NumPostDraws,'%0.0f') ' posterior draws and ' num2str(NumBurnin,'%0.0f') ' burn-in draws. Would you like to use the old results?'];
               answer = Query(txt,'question',200,['DSGE-VAR(' num2str(lambda) ') Multivariate ANOVA'],500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  RunANOVA = 0;
                  ANOVAData = load(ResFile);
                  MPSRF = ANOVAData.MPSRF;
                  DoneCalc = 1;
               else;
                  RunANOVA = 1;
               end;
            else;
               RunANOVA = 1;
            end;
            if RunANOVA==1;
               %
               % run the multivariate ANOVA functions
               %
               [DoneCalc,MPSRF] = MultiANOVA(NumChains,1,lambda,PME,DSGEModel,CurrINI,controls);
            end;
            if DoneCalc==1;
               PlotMultiANOVA(MPSRF,NumBurnin,NumChains,lambda,DSGEModel,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeMean'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Sequential Posterior Mean',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            Weights = [];
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('PostSample','LogPost','AcceptedDraws');
            %
            % send the data to the acceptance ratio plotting function
            %
            drawnow;
            PosteriorMeanMedianPlots(thetaPostSample,Weights,NumBurnin,CurrChain,'Mean',lambda,PME,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('thetaPostSample');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeMedian'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Sequantial Posterior Median',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            Weights = [];
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('PostSample','LogPost','AcceptedDraws');
            %
            % send the data to the acceptance ratio plotting function
            %
            drawnow;
            PosteriorMeanMedianPlots(thetaPostSample,Weights,NumBurnin,CurrChain,'Median',lambda,PME,DSGEModel,CurrINI);
            %
            % clear everything
            %
            clear('thetaPostSample');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ChibJeliazkov'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Seq. Marginal Likelihood - Chib & Jeliazkov',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('thetaPostSample');
            NumPostDraws = length(LogPost);
            %
            % ask if the user wants to run the sequential estimation routine or just see the end point results
            %
            txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
            answer = Query(txt,'question',200,'DSGE-VAR Seq. Marginal Likelihood - Chib & Jeliazkov',500,'no',CurrINI);
            %
            % run the sequential estimation routine
            %
            drawnow;
            if strcmp(lower(answer),'no')==1;
               PlotMargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,NumBurnin,CurrChain,lambda,PME,DSGEModel,CurrINI);
            else;
               %
               SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
               SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
               if PME==1;
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
               elseif PME==2;
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
               elseif PME==3;
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
               end;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               %
               % check if the user wants to recompute the log posterior
               %
               if isempty(ErrorStr)==1;
                  txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                  answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'DSGE-VAR Seq. Marginal Likelihood - Chib & Jeliazkov',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               OldNumBurnin = NumBurnin;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % recompute the log posterior
                  %
                  PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                  AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                  NumBurnin = 0;
                  LogPost = DSGEVARLogPosterior(PostSample,AcceptedDraws,ModeData,lambda,'Chib & Jeliazkov',DSGEModel,CurrINI);
               end;
               %
               if isempty(LogPost)==0;
                  TmpDSGEModel = DSGEModel;
                  TmpDSGEModel.SequentialML = 0;
                  %
                  LogMargsCJ = PlotMargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,NumBurnin,CurrChain,lambda,PME,TmpDSGEModel,CurrINI);
                  clear('TmpDSGEModel');
                  %
                  % check if we got any results
                  %
                  if isempty(LogMargsCJ)==0;
                     NumBurnin = OldNumBurnin;
                     %
                     % check the total number of Markov chains
                     %
                     ChainsStr = get(controls.posterior.chains,'String');
                     NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
                     %
                     % retrieve estimation sample start year and period
                     %
                     [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                     %
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ChibJeliazkov-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                     fid = fopen(TextFile,'wt');
                     prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     %
                     fprintf(fid,'********************************************************************************\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'********************************************************************************\n\n');
                     fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                     fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                     if NumChains>1;
                        fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                     end;
                     fprintf(fid,'lambda:                           %0.4f\n',lambda);
                     fprintf(fid,'Number of lags:                   %0.0f\n',DSGEModel.BVARLags);
                     %
                     % sample information
                     %
                     fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'==========================================================\n');
                     fprintf(fid,'                          point estimate        std. error\n');
                     fprintf(fid,'----------------------------------------------------------\n');
                     fprintf(fid,['Chib & Jeliazkov         ' prt_val '   ' prt_val '\n'],LogMargsCJ(1,2),LogMargsCJ(1,3));
                     fprintf(fid,'==========================================================\n\n');
                     %
                     % Some information about the filename and its location
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     %
                     fclose(fid);
                     %
                     % show in TextGUI
                     %
                     TextGUI(TextFile,'DSGE-VAR Marginal Likelihood - Chib & Jeliazkov',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                  end;
               end;
            end;
            %
            % clear everything
            %
            clear('PostSample','LogPost','AcceptedDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'TruncatedElliptical'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Seq. Marginal Likelihood - SWZ''s Harmonic Mean',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
            SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
            if PME==1;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            elseif PME==2;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            elseif PME==3;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            end;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % load the posterior draws
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
               clear('thetaPostSample');
               NumPostDraws = length(LogPost);
               drawnow;
               %
               % ask if the user wants to run the sequential estimation routine or just see the end point results
               %
               txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
               answer = Query(txt,'question',200,'DSGE-VAR Seq. Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
               %
               % select the PValue
               %
               PValueStrMatrix = strvcat('90 percent of posterior draws','80 percent of posterior draws','70 percent of posterior draws','60 percent of posterior draws','50 percent of posterior draws');
               [action,SelectedOption] = SelectionDlg(PValueStrMatrix,1,'Select the truncation parameter of the elliptical density:','Truncation Parameter','DSGE-VAR Seq. Marginal Likelihood - SWZ''s Harmonic Mean',[],[],CurrINI);
               if strcmp(lower(action),'ok')==1;
                  %
                  PValue = str2num(StringTrim(strrep(PValueStrMatrix(SelectedOption,:),'percent of posterior draws','')));
                  %
                  % run the sequential estimation routine
                  %
                  if strcmp(lower(answer),'no')==1;
                     %
                     % sequential marginal likelihood start value and step length
                     %
                     SeqStartStr = get(controls.dsge.sequentialstart,'String');
                     DSGEModel.SequentialStartIterationValue = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
                     SeqStepStr = get(controls.dsge.sequentialstep,'String');
                     DSGEModel.SequentialStepLengthValue = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
                     %
                     % Open up a wait dialog.
                     %
                     txt = ['Please wait while YADA computes SWZ''s modified harmonic mean estimator of the log marginal likelihood for a DSGE-VAR(' num2str(lambda,'%0.4f') ') with ' int2str(DSGEModel.BVARLags) ' lags. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
                     WaitHandle = WaitDLG(txt,'information','DSGE-VAR Seq. Marginal Likelihood',600,300,CurrINI,1);
                     WaitControls = get(WaitHandle,'UserData');
                     %
                     [LogMargsSWZ,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample(NumBurnin+1:NumPostDraws,:),LogPost(NumBurnin+1:NumPostDraws,:),NumPostDraws,PValue,1,ModeData,lambda,DSGEModel,CurrINI,WaitControls.cancel);
                     %
                     % close the wait dialog
                     %
                     set(WaitControls.text,'UserData','done');
                     delete(WaitHandle);
                     drawnow;
                     pause(0.02);
                     %
                     if isempty(LogMargsSWZ)==0;
                        GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
                        FigHandle = figure('Visible','off', ...
                                  'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
                                  'Tag','MargLikeMHM', ...
                                  'Name','Sequential Marginal Likelihood - SWZ''s Harmonic Mean');
                        %
                        SetFigureProperties(FigHandle,CurrINI);
                        %
                        Xvalues = LogMargsSWZ(:,1)+NumBurnin;
                        Yvalues = LogMargsSWZ(:,2);
                        %
                        plot(Xvalues,Yvalues,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
                        hold('off');
                        %
                        if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
                           SetAxesFonts(CurrINI,'Title','SWZ''s Harmonic Mean Estimates','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
                        else;
                           SetAxesFonts(CurrINI,'Title','SWZ''s Harmonic Mean Estimates','XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
                        end;
                        axis('tight');
                        %
                        % check the YLim
                        %
                        ylim = get(gca,'YLim');
                        add_on = 0.1*(ylim(2)-ylim(1));
                        ylim = [ylim(1)-add_on ylim(2)+add_on];
                        set(gca,'YLim',ylim);
                        %
                        % save the grasph
                        %
                        if DirExist(GraphDirectory);
                           if DSGEModel.PosteriorSampler==1;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-RWM-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           elseif DSGEModel.PosteriorSampler==2;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Slice-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           elseif DSGEModel.PosteriorSampler==3;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           elseif DSGEModel.PosteriorSampler==4;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           elseif DSGEModel.PosteriorSampler==5;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           elseif DSGEModel.PosteriorSampler==6;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           elseif DSGEModel.PosteriorSampler==7;
                              file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                           end;
                           SetLogText(strvcat('   Saving plot to:',['   ' GraphDirectory '...']));
                           saveas(FigHandle,file);
                        end;
                        %
                        % display the graph
                        %
                        SetToolbarImages(FigHandle,CurrINI.images);
                        set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
                        drawnow;
                        pause(0.02);
                     end;
                  else;
                     %
                     % check if the user wants to recompute the log posterior
                     %
                     if isempty(ErrorStr)==1;
                        txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                        answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'DSGE-VAR Seq. Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     %
                     % select which posterior draws to use
                     %
                     LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
                     PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                     AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                     NumDraws = NumPostDraws-NumBurnin;
                     %
                     % check if the user wishes a sub-sample
                     %
                     txt = 'Would you like to use a sub-sample of the posterior draws?';
                     answersub = Query(txt,'question',400,'DSGE-VAR Seq. Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
                     if strcmp(lower(answersub),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        LogPost = LogPost(UseDraws);
                        PostSample = PostSample(UseDraws,:);
                        AcceptedDraws = AcceptedDraws(UseDraws);
                     end;
                     NumDraws = length(AcceptedDraws);
                     %
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % recompute the log posterior
                        %
                        LogPost = DSGEVARLogPosterior(PostSample,AcceptedDraws,ModeData,lambda,'SWZ''s Harmonic Mean',DSGEModel,CurrINI);
                     end;
                     %
                     if isempty(LogPost)==0;
                        [LogMargsSWZ,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample,LogPost,NumPostDraws,PValue,0,ModeData,lambda,DSGEModel,CurrINI);
                        if isempty(LogMargsSWZ)==0;
                           %
                           % check the total number of Markov chains
                           %
                           ChainsStr = get(controls.posterior.chains,'String');
                           NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
                           %
                           % create sample dates
                           %
                           [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                           KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
                           p = DSGEModel.BVARLags;
                           if FirstPeriod+KalmanFirstObservation-1<p+1;
                              KalmanFirstObservation = p+2-FirstPeriod;
                           end;
                           %
                           % actual sample that we will use
                           %
                           [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
                           %
                           TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-SWZ-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
                           fid = fopen(TextFile,'wt');
                           prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                           %
                           fprintf(fid,'********************************************************************************\n');
                           fprintf(fid,'*                                                                              *\n');
                           fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                           fprintf(fid,'*                                                                              *\n');
                           fprintf(fid,'********************************************************************************\n\n');
                           fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                           fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
                           fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                           if NumChains>1;
                              fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                           end;
                           fprintf(fid,'lambda:                           %0.4f\n',lambda);
                           fprintf(fid,'Number of lags:                   %0.0f\n',p);
                           %
                           % sample information
                           %
                           fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                           fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                           %
                           % SWZ's modified harmonic mean estimator
                           %
                           fprintf(fid,'===============================================================\n');
                           fprintf(fid,'                              point estimate        std. error\n');
                           fprintf(fid,'---------------------------------------------------------------\n');
                           fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],LogMargsSWZ(1,2),NWStdErr);
                           fprintf(fid,'---------------------------------------------------------------\n');
                           fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],LBound);
                           fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],qL);
                           fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
                           fprintf(fid,'===============================================================\n\n');
                           fprintf(fid,'NOTE: The "fraction of iid draws from elliptical > lower bound" notation refers to the fraction of\n      the parameter draws which have a log posterior kernel greater than the lower bound. The "fraction\n      of posterior draws > lower bound" notation also refers to the log posterior kernel values.\n\n');
                           %
                           fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                           fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                           fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                           %
                           fclose(fid);
                           %
                           % show in TextGUI
                           %
                           TextGUI(TextFile,'DSGE-VAR Marginal Likelihood - SWZ''s Modified Harmonic Mean',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                        end;
                     end;
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModifiedHarmonicMean'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Seq. Marginal Likelihood - Geweke''s Harmonic Mean',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            clear('thetaPostSample');
            NumPostDraws = length(LogPost);
            %
            % ask if the user wants to run the sequential estimation routine or just see the end point results
            %
            txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
            answer = Query(txt,'question',200,'DSGE-VAR Seq. Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
            %
            % run the sequential estimation routine
            %
            drawnow;
            if strcmp(lower(answer),'no')==1;
               %
               % Coverage probabilities for the Modified Harmonic Mean estimator of
               % the marginal likelihood
               %
               CovStartStr = get(controls.dsge.coveragestart,'String');
               DSGEModel.CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
               CovIncStr = get(controls.dsge.coverageincrement,'String');
               DSGEModel.CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
               CovEndStr = get(controls.dsge.coverageend,'String');
               DSGEModel.CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
               %
               % sequential marginal likelihood start value and step length
               %
               SeqStartStr = get(controls.dsge.sequentialstart,'String');
               DSGEModel.SequentialStartIterationValue = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
               SeqStepStr = get(controls.dsge.sequentialstep,'String');
               DSGEModel.SequentialStepLengthValue = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
               %
               % Open up a wait dialog.
               %
               txt = ['Please wait while YADA computes Geweke''s modified harmonic mean estimator of the log marginal likelihood for a DSGE-VAR(' num2str(lambda) ') with ' int2str(DSGEModel.BVARLags) ' lags. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
               WaitHandle = WaitDLG(txt,'information','DSGE-VAR Seq. Marginal Likelihood',600,300,CurrINI,1);
               WaitControls = get(WaitHandle,'UserData');
               %
               [LogMargsMH,CovProb,NWStdErr] = MargLikeModifiedHarmonic(PostSample(NumBurnin+1:NumPostDraws,:),LogPost(NumBurnin+1:NumPostDraws,:),1,[],DSGEModel,WaitControls.cancel);
               %
               % close the wait dialog
               %
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
               %
               if isempty(LogMargsMH)==0;
                  GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
                  FigHandle = figure('Visible','off', ...
                            'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
                            'Tag','MargLikeMHM', ...
                            'Name','Sequential Marginal Likelihood - Geweke''s Harmonic Mean');
                  %
                  SetFigureProperties(FigHandle,CurrINI);
                  %
                  Xvalues = LogMargsMH(:,1)+NumBurnin;
                  Yvalues = LogMargsMH(:,2:size(LogMargsMH,2));
                  CurrMap = colormap;
                  NumColor = 1;
                  NumStyle = 1;
                  ColorOrder = [(1:8:57) (2:8:58) (3:8:59) (4:8:60) (5:8:61) (6:8:62) (7:8:63) (8:8:64)];
                  LegendStr = '';
                  for i=1:size(Yvalues,2);
                     if NumColor<=length(CurrMap);
                        CurrColor = CurrMap(ColorOrder(i),:);
                     else;
                        CurrColor = CurrMap(ColorOrder(1),:);
                        NumColor = 1;
                     end;
                     if NumStyle==1;
                        CurrStyle = '-';
                     elseif NumStyle==2;
                        CurrStyle = '--';
                     elseif NumStyle==3;
                        CurrStyle = ':';
                     elseif NumStyle==4;
                        NumStyle = 0;
                        CurrStyle = '-.';
                     end;
                     plot(Xvalues,Yvalues(:,i),'LineStyle',CurrStyle,'Color',CurrColor,'LineWidth',1);
                     hold('on');
                     NumColor = NumColor+1;
                     NumStyle = NumStyle+1;
                     LegendStr = strvcat(LegendStr,[num2str(CovProb(i)*100,'%0.0f') ' percent coverage prob']);
                  end;
                  %
                  % plot the average
                  %
                  plot(Xvalues,(1/size(Yvalues,2))*sum(Yvalues,2),'LineStyle','-','Color',[0 0 0],'LineWidth',1);
                  LegendStr = strvcat(LegendStr,'Overall mean');
                  hold('off');
                  %
                  if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
                     SetAxesFonts(CurrINI,'Title','Geweke''s Harmonic Mean Estimates','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
                  else;
                     SetAxesFonts(CurrINI,'Title','Geweke''s Harmonic Mean Estimates','XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
                  end;
                  axis('tight');
                  %
                  % check the YLim
                  %
                  ylim = get(gca,'YLim');
                  add_on = 0.1*(ylim(2)-ylim(1));
                  ylim = [ylim(1)-add_on ylim(2)+add_on];
                  set(gca,'YLim',ylim);
                  %
                  legHandle = legend(LegendStr,0);
                  SetAxesFonts(CurrINI,legHandle);
                  %
                  % save the grasph
                  %
                  if DirExist(GraphDirectory);
                     if DSGEModel.PosteriorSampler==1;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-RWM-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     elseif DSGEModel.PosteriorSampler==2;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Slice-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     elseif DSGEModel.PosteriorSampler==3;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     elseif DSGEModel.PosteriorSampler==4;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     elseif DSGEModel.PosteriorSampler==5;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     elseif DSGEModel.PosteriorSampler==6;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     elseif DSGEModel.PosteriorSampler==7;
                        file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
                     end;
                     SetLogText(strvcat('   Saving plot to:',['   ' GraphDirectory '...']));
                     saveas(FigHandle,file);
                  end;
                  %
                  % display the graph
                  %
                  SetToolbarImages(FigHandle,CurrINI.images);
                  set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
                  drawnow;
                  pause(0.02);
               end;
            else;
               %
               % Coverage probabilities for Geweke''s Modified Harmonic Mean estimator of
               % the marginal likelihood
               %
               CovStartStr = get(controls.dsge.coveragestart,'String');
               CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
               CovIncStr = get(controls.dsge.coverageincrement,'String');
               CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
               CovEndStr = get(controls.dsge.coverageend,'String');
               CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
               CovProb = (CovStartValue:CovIncValue:CovEndValue);
               %
               SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
               SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
               if PME==1;
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
               elseif PME==2;
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
               elseif PME==3;
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
               end;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               %
               % check if the user wants to recompute the log posterior
               %
               if isempty(ErrorStr)==1;
                  txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                  answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'DSGE-VAR Seq. Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               %
               % select which posterior draws to use
               %
               LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
               PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
               AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
               NumDraws = NumPostDraws-NumBurnin;
               %
               % check if the user wishes a sub-sample
               %
               txt = 'Would you like to use a sub-sample of the posterior draws?';
               answersub = Query(txt,'question',400,'DSGE-VAR Seq. Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
               if strcmp(lower(answersub),'yes')==1;
                  %
                  % check which draws we should use
                  %
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                  LogPost = LogPost(UseDraws);
                  PostSample = PostSample(UseDraws,:);
                  AcceptedDraws = AcceptedDraws(UseDraws);
               end;
               NumDraws = length(AcceptedDraws);
               %
               if strcmp(lower(answer),'yes')==1;
                  %
                  % recompute the log posterior
                  %
                  LogPost = DSGEVARLogPosterior(PostSample,AcceptedDraws,ModeData,lambda,'Geweke''s Harmonic Mean',DSGEModel,CurrINI);
               end;
               if isempty(LogPost)==0;
                  [LogMargsMH,CovProb,NWStdErr] = MargLikeModifiedHarmonic(PostSample,LogPost,0,CovProb,DSGEModel);
                  %
                  % check the total number of Markov chains
                  %
                  ChainsStr = get(controls.posterior.chains,'String');
                  NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
                  %
                  % create sample dates
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
                  p = DSGEModel.BVARLags;
                  if FirstPeriod+KalmanFirstObservation-1<p+1;
                     KalmanFirstObservation = p+2-FirstPeriod;
                  end;
                  %
                  % actual sample that we will use
                  %
                  [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
                  fid = fopen(TextFile,'wt');
                  prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                  fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
                  fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                  if NumChains>1;
                     fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                  end;
                  fprintf(fid,'lambda:                           %0.4f\n',lambda);
                  fprintf(fid,'Number of lags:                   %0.0f\n',p);
                  %
                  % sample information
                  %
                  fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % Geweke's modified harmonic mean estimator
                  %
                  prt_str = 'Geweke''s Harmonic Mean';
                  top_str = '                      ';
                  std_str = 'Num. standard errors: ';
                  for i=1:length(CovProb);
                     top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
                     prt_str = [prt_str '   ' prt_val];
                     std_str = [std_str '   ' prt_val];
                  end;
                  TopStr = MultiCharStr('=',length(top_str));
                  MidStr = MultiCharStr('-',length(top_str));
                  fprintf(fid,[TopStr '\n']);
                  fprintf(fid,'                              Coverage probabilities for the estimator\n');
                  fprintf(fid,[top_str '\n']);
                  fprintf(fid,[MidStr '\n']);
                  fprintf(fid,[prt_str '\n'],LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
                  fprintf(fid,[std_str '\n'],NWStdErr');
                  fprintf(fid,[MidStr '\n']);
                  fprintf(fid,['Mean:                    ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
                  fprintf(fid,['Median:                  ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
                  fprintf(fid,[TopStr '\n\n']);
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  %
                  fclose(fid);
                  %
                  % show in TextGUI
                  %
                  TextGUI(TextFile,'DSGE-VAR Marginal Likelihood - Geweke''s Modified Harmonic Mean',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
               end;
            end;
            %
            % clear everything
            %
            clear('PostSample','LogPost','AcceptedDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplaceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
               thetaIndex = ModeData.thetaIndex;
               UniformBounds = ModeData.UniformBounds;
               if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
                  LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
                  IsSystemPrior = 1;
               else;
                  LogMargLikeSystemLaplace = 0;
                  IsSystemPrior = 0;
               end;
            else;
               LogMargLikeSystemLaplace = 0;
               IsSystemPrior = 0;
            end;
         else;
            LogMargLikeSystemLaplace = 0;
            IsSystemPrior = 0;
         end;
         %
         % check if system prior mode has been estimated
         %
         if (LogMargLikeSystemLaplace==0)&(FileExist(DSGEModel.SystemPriorFile)==1);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               SPModeData = load(ModeFile);
               LogMargLikeSystemLaplace = SPModeData.LogMargLikeSystemLaplace;
            end;
            IsSystemPrior = 1;
         end;
         if InitStatus==1;
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % select lambda
               %
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Laplace Approximation - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % need to compute certain values
                  %
                  n = DSGEModel.n;
                  k = DSGEModel.k;
                  p = DSGEModel.BVARLags;
                  npk = (n*p)+k;
                  [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
                  if k>0;
                     DetProductMoments = CalcDeterministicProductMoments(x,X);
                  else;
                     DetProductMoments = [];
                  end;
                  T = size(y,2);
                  GammaHatyy = (1/T)*y*y';
                  GammaHatyY = (1/T)*y*Y';
                  GammaHatYY = (1/T)*Y*Y';
                  if isinf(lambdai)==0;
                     logGPR = LogGammaProductRatio(n,((1+lambdai)*T)-npk,lambdai*T-npk);
                  else;
                     logGPR = [];
                  end;
                  phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
                  %
                  % we can now compute the log posterior and the inverse Hessian
                  %
                  NumParam = length(phi);
                  StepLengthStr = get(controls.optimize.steplength,'String');
                  StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                  LogPostDensity = -logPosteriorPhiDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  VechHessian = YADAHessianMatrix('logPosteriorPhiDSGEVAR',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                  if IsPositiveDefinite(HessianMat)==1;
                     LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                     %
                     PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     if FirstPeriod+KalmanFirstObservation-1<p+1;
                        KalmanFirstObservation = p+2-FirstPeriod;
                     end;
                     [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-LaplaceMarginalLikelihood-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-InitialValues.txt']);                  
                     %
                     fid = fopen(TextFile,'wt');
                     fprintf(fid,'********************************************************************************\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                     fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'********************************************************************************\n\n');
                     %
                     % DSGE-VAR model information
                     %
                     fprintf(fid,'Number of lags:                   %0.0f \n',p);
                     fprintf(fid,'Lambda:                           %0.4f \n\n',lambdai);
                     %
                     % sample information
                     %
                     fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % now the marginal likelihood
                     %
                     fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                     fprintf(fid,'where \n\n');
                     fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                     fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                     if IsSystemPrior==1;
                        if LogMargLikeSystemLaplace~=0;
                           fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                        else;
                           fprintf(fid,'   Correction for the log marginal\n   likelihood of the system prior: Not estimated yet \n\n');
                        end;
                     end;
                     fprintf(fid,'================================================================================\n\n');
                     %
                     % Some information about the filename and its location
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     fclose(fid);
                     %
                     % display the file
                     %
                     TextGUI(TextFile,['DSGE-VAR(' num2str(lambdai) ') Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Initial Values',100,CurrINI,0,1);
                  else;
                     txt = 'The estimated Hessian matrix based on finite differences is not positive definite. YADA is therefore unable to compute the Laplace approximation of the marginal likelihood.';
                     About(txt,'information','Laplace Approximation - Initial Values',300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplacePosteriorMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevarview.sequentialmarglike.laplace.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
            ModeValue = 1;
         elseif gcbo==controls.dsgevarview.sequentialmarglike.laplace.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            ModeValue = 2;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.sequentialmarglike.laplace.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            ModeValue = 3;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Laplace Approximation - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            HeaderStr = 'YADA - Bad MAT File';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            %
            if isempty(ErrorStr)==1;
               %
               % need to compute certain values
               %
               n = DSGEModel.n;
               k = DSGEModel.k;
               p = DSGEModel.BVARLags;
               npk = (n*p)+k;
               %
               phi = ModeData.phiMode;
               %
               % setup the sample
               %
               [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
               if DSGEModel.k>0;
                  DetProductMoments = CalcDeterministicProductMoments(x,X);
               else;
                  DetProductMoments = [];
               end;
               T = size(y,2);
               GammaHatyy = (1/T)*y*y';
               GammaHatyY = (1/T)*y*Y';
               GammaHatYY = (1/T)*Y*Y';
               if ModeValue<3;
                  if isinf(lambdai)==0;
                     logGPR = LogGammaProductRatio(n,((1+lambdai)*T)-npk,lambdai*T-npk);
                  else;
                     logGPR = [];
                  end;
               else;
                  if isinf(lambdai)==0;
                     logCPC = DSGEVARLogConcLikelihoodConstant(lambdai,T,n,npk);
                  else;
                     logCPC = [];
                  end;
               end;
               %
               % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
               %
               [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
               [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
               if FileExist(DSGEModel.UpdateParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
               end;
               if FileExist(DSGEModel.InitializeParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
               end;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
               end;
               %
               % compute the log posterior and the inverse Hessian at the mode
               %
               NumParam = length(phi);
               StepLengthStr = get(controls.optimize.steplength,'String');
               StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
               if ModeValue<3;
                  LogPostDensity = -logPosteriorPhiDSGEVAR(phi,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  VechHessian = YADAHessianMatrix('logPosteriorPhiDSGEVAR',phi,StepLength,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
               else;
                  LogPostDensity = -logConcPosteriorPhiDSGEVAR(phi,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  VechHessian = YADAHessianMatrix('logConcPosteriorPhiDSGEVAR',phi,StepLength,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,CurrINI.NumWorkers);
               end;
               HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
               if IsPositiveDefinite(HessianMat)==1;
                  if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
                     LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
                     IsSystemPrior = 1;
                  else;
                     LogMargLikeSystemLaplace = 0;
                     IsSystemPrior = 0;
                  end;
                  LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                  %
                  PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                  KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  if FirstPeriod+KalmanFirstObservation-1<p+1;
                     KalmanFirstObservation = p+2-FirstPeriod;
                  end;
                  [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-LaplaceMarginalLikelihood-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' ModeShort '.txt']);                  
                  %
                  fid = fopen(TextFile,'wt');
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                  fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  % DSGE-VAR model information
                  %
                  fprintf(fid,'Posterior mode estimate:          %s \n',ModeType);
                  fprintf(fid,'Number of lags:                   %0.0f \n',p);
                  fprintf(fid,'Lambda:                           %0.4f \n\n',lambdai);
                  %
                  % sample information
                  %
                  fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % now the marginal likelihood
                  %
                  fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                  fprintf(fid,'where \n\n');
                  fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                  fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                  if IsSystemPrior==1;
                     fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                  end;
                  fprintf(fid,'================================================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  % display the file
                  %
                  TextGUI(TextFile,['DSGE-VAR(' num2str(lambdai) ') Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],ModeType,100,CurrINI,0,1);
               else;
                  txt = 'The estimated Hessian matrix based on finite differences is not positive definite. YADA is therefore unable to compute the Laplace approximation of the marginal likelihood.';
                  About(txt,'information',['Laplace Approximation - ' ModeType],300,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModestyStatistics'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModestyResults = get(gcbo,'UserData');
         ChainStr = strrep(strrep(get(gcbo,'Label'),'&',''),'...','');
         %
         ShowResult = 0;
         if size(ModestyResults.Files,1)>1;
            %
            % open a selection dialog
            %
            [action,value] = SelectionDlg(ModestyResults.Information,size(ModestyResults.Files,1),'Select the modesty results you would like to view:','Modesty Results Selection',['DSGE-VAR Modesty Statistics - ' ChainStr],'','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               file = ModestyResults.Files(value,:);
               lambdai = ModestyResults.Lambda(value);
               ShowResult = 1;
            end;
         else;
            file = ModestyResults.Files;
            lambdai = ModestyResults.Lambda;
            ShowResult = 1;
         end;
         %
         % display the text file
         %
         if ShowResult==1;
            TextGUI(file,['DSGE-VAR(' num2str(lambdai) ') Modesty Statistics - ' ChainStr],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,1);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CompareEstimatedParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevarview.compareparameters.margmode;
            [status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal';
         elseif gcbo==controls.dsgevarview.compareparameters.jointmode;
            [status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint';
         end;
         %
         % display the estimated posterior modes and compare to DSGE model mode results
         % if they exist
         %
         GraphDSGEVARPostModeParams(ModeType,lambda,ModeFiles,DSGEModel,CurrINI);
         %
         set(controls.about,'UserData',0);
      end;
   case 'CEPPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('Compare Estimated Parameters',DSGEModel,controls,CurrINI,0);
         if value>0;
            %
            % check how many Lambda values we have
            %
            DoCMP = 0;
            if length(Lambda)>1;
               DoCMP = 1;
            else;
               %
               % check if we can compute the marginal likelihood for the DSGE model
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  DoCMP = 1;
               end;
            end;
            if DoCMP==1;
               %
               % run the marginal likelihood comparison
               %
               CompareEstimatedParameters(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda);
            else;
               txt = ['YADA has only been able to locate posterior draws for one model, the DSGE-VAR with lambda=' num2str(Lambda(value)) '. But you need at least two models to compare the estimated parameters. You may either run the sampler for another DSGE-VAR or for the DSGE model.'];
               About(txt,'information','Compare Estimated Parameters',200,500,CurrINI);
            end;
         end;
         %
         %
         set(controls.about,'UserData',0);
      end;
   case 'CheckPosteriorMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % run the DSGE-VAR check optimum function
         %
         DSGEVARCheckPosteriorMode(DSGEModel,CurrINI,controls);
         %
         set(controls.about,'UserData',0);
      end;
   case 'CompareChib'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('Compare Marginal Likelihood - Chib & Jeliazkov',DSGEModel,controls,CurrINI,0);
         if value>0;
            %
            % check how many Lambda values we have
            %
            DoCMP = 0;
            if length(Lambda)>1;
               DoCMP = 1;
            else;
               %
               % check if we can compute the marginal likelihood for the DSGE model
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  DoCMP = 1;
               end;
            end;
            if DoCMP==1;
               %
               % select the Lambda values to use
               %
               if length(Lambda)>1;
                  LambdaNames = '';
                  for i=1:length(Lambda);
                     LambdaNames = strvcat(LambdaNames,['lambda=' num2str(Lambda(i),'%0.4f')]);
                  end;
                  positions = (1:length(Lambda));
                  [PosAction,positions] = SelectCondVarShockDLG('init','SWZ''s Harmonic Mean','Lambda Values For',LambdaNames,positions,CurrINI);
                  if strcmp(lower(PosAction),'ok')==1;
                     if isempty(positions)==0;
                        Lambda = Lambda(positions);
                     else;
                        Lambda = [];
                        txt = 'You didn''t select any values for the lambda hyperparameter. YADA will therefore abort from the comparison of marginal likelihood value.';
                        About(txt,'information','Compare Marginal Likelihood - SWZ''s Harmonic Mean',200,500,CurrINI);
                     end;
                  else;
                     Lambda = [];
                  end;
               end;
               %
               % run the marginal likelihood comparison
               %
               if isempty(Lambda)==0;
                  CompareMarginalLikelihood(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda,3);
               end;
            else;
               txt = ['YADA has only been able to locate posterior draws for one model, the DSGE-VAR with lambda=' num2str(Lambda(value)) '. But you need at least two models to make marginal likelihood comparisons. You may either run the sampler for another DSGE-VAR or for the DSGE model.'];
               About(txt,'information','Compare Marginal Likelihood - Chib & Jeliazkov',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CompareSWZModifiedHarmonic'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('Compare Marginal Likelihood - SWZ''s Harmonic Mean',DSGEModel,controls,CurrINI,0);
         if value>0;
            %
            % check how many Lambda values we have
            %
            DoCMP = 0;
            if length(Lambda)>1;
               DoCMP = 1;
            else;
               %
               % check if we can compute the marginal likelihood for the DSGE model
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  DoCMP = 1;
               end;
            end;
            if DoCMP==1;
               %
               % select the fraction of posterior draws
               %
               PValueStrMatrix = strvcat('90 percent of posterior draws','80 percent of posterior draws','70 percent of posterior draws','60 percent of posterior draws','50 percent of posterior draws');
               [action,SelectedOption] = SelectionDlg(PValueStrMatrix,1,'Select the truncation parameter of the elliptical density:','Truncation Parameter','Compare Marginal Likelihood - SWZ''S Harmonic Mean',[],[],CurrINI);
               if strcmp(lower(action),'ok')==1;
                  %
                  PValue = str2num(StringTrim(strrep(PValueStrMatrix(SelectedOption,:),'percent of posterior draws','')));
                  %
                  % select the Lambda values to use
                  %
                  if length(Lambda)>1;
                     LambdaNames = '';
                     for i=1:length(Lambda);
                        LambdaNames = strvcat(LambdaNames,['lambda=' num2str(Lambda(i),'%0.4f')]);
                     end;
                     positions = (1:length(Lambda));
                     [PosAction,positions] = SelectCondVarShockDLG('init','SWZ''s Harmonic Mean','Lambda Values For',LambdaNames,positions,CurrINI);
                     if strcmp(lower(PosAction),'ok')==1;
                        if isempty(positions)==0;
                           Lambda = Lambda(positions);
                        else;
                           Lambda = [];
                           txt = 'You didn''t select any values for the lambda hyperparameter. YADA will therefore abort from the comparison of marginal likelihood value.';
                           About(txt,'information','Compare Marginal Likelihood - SWZ''s Harmonic Mean',200,500,CurrINI);
                        end;
                     else;
                        Lambda = [];
                     end;
                  end;
                  %
                  % run the marginal likelihood comparison
                  %
                  if isempty(Lambda)==0
                     CompareMarginalLikelihood(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda,2,PValue);
                  end;
               end;
            else;
               txt = ['YADA has only been able to locate posterior draws for one model, the DSGE-VAR with lambda=' num2str(Lambda(value)) '. But you need at least two models to make marginal likelihood comparisons. You may either run the sampler for another DSGE-VAR or for the DSGE model.'];
               About(txt,'information','Compare Marginal Likelihood - SWZ''s Harmonic Mean',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CompareSMCMargLike'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('Compare Marginal Likelihood - Sequential Monte Carlo',DSGEModel,controls,CurrINI,0);
         if value>0;
            %
            % check how many Lambda values we have
            %
            DoCMP = 0;
            if length(Lambda)>1;
               DoCMP = 1;
            else;
               %
               % check if we can compute the marginal likelihood for the DSGE model
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  DoCMP = 1;
               end;
            end;
            if DoCMP==1;
               %
               % select the Lambda values to use
               %
               if length(Lambda)>1;
                  LambdaNames = '';
                  for i=1:length(Lambda);
                     LambdaNames = strvcat(LambdaNames,['lambda=' num2str(Lambda(i),'%0.4f')]);
                  end;
                  positions = (1:length(Lambda));
                  [PosAction,positions] = SelectCondVarShockDLG('init','Sequential Monte Carlo','Lambda Values For',LambdaNames,positions,CurrINI);
                  if strcmp(lower(PosAction),'ok')==1;
                     if isempty(positions)==0;
                        Lambda = Lambda(positions);
                     else;
                        Lambda = [];
                        txt = 'You didn''t select any values for the lambda hyperparameter. YADA will therefore abort from the comparison of marginal likelihood value.';
                        About(txt,'information','Compare Marginal Likelihood - Sequential Monte Carlo',200,500,CurrINI);
                     end;
                  else;
                     Lambda = [];
                  end;
               end;
               %
               % run the marginal likelihood comparison
               %
               if isempty(Lambda)==0;
                  CompareMarginalLikelihood(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda,4);
               end;
            else;
               txt = ['YADA has only been able to locate posterior draws for one model, the DSGE-VAR with lambda=' num2str(Lambda(value)) '. But you need at least two models to make marginal likelihood comparisons. You may either run the sampler for another DSGE-VAR or for the DSGE model.'];
               About(txt,'information','Compare Marginal Likelihood - Sequential Monte Carlo',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CompareModifiedHarmonic'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('Compare Marginal Likelihood - Geweke''s Harmonic Mean',DSGEModel,controls,CurrINI,0);
         if value>0;
            %
            % check how many Lambda values we have
            %
            DoCMP = 0;
            if length(Lambda)>1;
               DoCMP = 1;
            else;
               %
               % check if we can compute the marginal likelihood for the DSGE model
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  DoCMP = 1;
               end;
            end;
            if DoCMP==1;
               %
               % select the Lambda values to use
               %
               if length(Lambda)>1;
                  LambdaNames = '';
                  for i=1:length(Lambda);
                     LambdaNames = strvcat(LambdaNames,['lambda=' num2str(Lambda(i),'%0.4f')]);
                  end;
                  positions = (1:length(Lambda));
                  [PosAction,positions] = SelectCondVarShockDLG('init','SWZ''s Harmonic Mean','Lambda Values For',LambdaNames,positions,CurrINI);
                  if strcmp(lower(PosAction),'ok')==1;
                     if isempty(positions)==0;
                        Lambda = Lambda(positions);
                     else;
                        Lambda = [];
                        txt = 'You didn''t select any values for the lambda hyperparameter. YADA will therefore abort from the comparison of marginal likelihood value.';
                        About(txt,'information','Compare Marginal Likelihood - SWZ''s Harmonic Mean',200,500,CurrINI);
                     end;
                  else;
                     Lambda = [];
                  end;
               end;
               %
               % run the marginal likelihood comparison
               %
               if isempty(Lambda)==0;
                  CompareMarginalLikelihood(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda,1);
               end;
            else;
               txt = ['YADA has only been able to locate posterior draws for one model, the DSGE-VAR with lambda=' num2str(Lambda(value)) '. But you need at least two models to make marginal likelihood comparisons. You may either run the sampler for another DSGE-VAR or for the DSGE model.'];
               About(txt,'information','Compare Marginal Likelihood - Geweke''s Harmonic Mean',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'VARPriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which function was selected
         %
         if gcbo==controls.dsgevarview.priordensity.varparameter;
            ParamStr = 'VAR';
            ParamType = 1;
         elseif gcbo==controls.dsgevarview.priordensity.sigmaparameter;
            ParamStr = 'Sigma';
            ParamType = 2;
         elseif gcbo==controls.dsgevarview.priordensity.a0parameter;
            ParamStr = 'A0';
            ParamType = 3;
         end;
         %
         % try to locate a suitable posterior mode results file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            try;
               ModeData = load(ModeFile);
               Status = 1;
            catch;
               [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
               if Status==0;
                  [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
               end;
               ModeData = [];
            end;
         end;
         if Status==1;
            ErrorStr = '';
            if isempty(ModeData)==1;
               ModeFile = StringTrim(Files(1,:));
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the DSGE model prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,[ParamStr ' Prior Densities'],500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  LowerBound = ModeData.LowerBound;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               if FileExist(DSGEModel.PriorFile)==1;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',[ParamStr ' Prior Densities'],'','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaModeEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  else;
                     SigmaModeEpsilon = SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  %
                  if (ParamType<3)|(IdentifyStatus==1);
                     %
                     % run the prior densities function
                     %
                     GraphVARDSGEPriorDensity(ParamStr,Phi,SigmaModeEpsilon,A0,T,lambdai,H,B0,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the posterior densities of the A0 parameters.'];
                     About(txt,'information',[ParamStr ' Prior Densities'],200,500,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DSGEPosteriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('DSGE-VAR Posterior Densities - DSGE Parameters',DSGEModel,controls,CurrINI,1);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
            %
            clear('PostSample','LogPost','AcceptedDraws');
            if PME==1;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            elseif PME==2;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            elseif PME==3;
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            end;
            %
            % try to load the ModeFile to get names of parameters
            %
            try;
               ModeData = load(ModeFile);
               ParameterNames = ModeData.ParameterNames;
            catch;
               ParameterNames.additional = []
            end;
            %
            % check if we have parameters that are functions of estimated parameters
            %
            if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
               %
               % ask if the user wants to load additional parameters
               %
               txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot posterior densities for these additional parameters?'];
               answer = Query(txt,'question',200,'DSGE-VAR Posterior Densities - Additional Parameters',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  %
                  % determine the selected shocks of the DSGE-VAR
                  %
                  SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                  SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                  %
                  % total number of Markov chains
                  %
                  ChainsStr = get(controls.posterior.chains,'String');
                  NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
                  %
                  [UpdatePostSample,UpdateMode,UpdateNames] = GetDSGEVARUpdatePosteriorDraws(thetaPostSample,lambda,PME,ModeFile,NumBurnin,CurrChain,NumChains,1,DSGEModel,CurrINI);
                  %
                  % check if the user has the necessary tools for computing prior densitites
                  % for the additional parameters through random draws from the prior of the
                  % relevant estimated parameters
                  %
                  if isempty(UpdateMode)==0;
                     if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
                        IsSystemPrior = 0;
                     else;
                        IsSystemPrior = 1;
                     end;
                     TmpCurrINI = CurrINI;
                     TmpCurrINI.UpdateNames = UpdateNames;
                     if IsSystemPrior==0;
                        [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                     else;
                        [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                     end;
                     %
                     % test if the results were obtained from an old mat-file
                     %
                     if max(UpdateParameters)>length(UpdateMode);
                        if DSGEModel.PosteriorSampler==1;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                        elseif DSGEModel.PosteriorSampler==2;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Slice-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                        elseif DSGEModel.PosteriorSampler==3;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
                        elseif DSGEModel.PosteriorSampler==4;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
                        elseif DSGEModel.PosteriorSampler==5;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
                        elseif DSGEModel.PosteriorSampler==6;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                        elseif DSGEModel.PosteriorSampler==7;
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                        elseif DSGEModel.PosteriorSampler==8;
                           TempStagesStr = get(controls.posterior.temperingstages,'String');
                           TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
                           AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-Additional-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(size(thetaPostSample,1)) '.mat']);
                        end;
                        delete(AddFile);
                        if IsSystemPrior==0;
                           [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                        else;
                           [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                        end;
                     end;
                     clear('TmpCurrINI');
                     if isempty(UpdatePriorX)==0;
                        if length(UpdateMode)>length(UpdateParameters);
                           UpdateMode = UpdateMode(UpdateParameters);
                           UpdateNames = UpdateNames(UpdateParameters,:);
                           UpdatePostSample = UpdatePostSample(:,UpdateParameters);
                        end;
                     else;
                        txt = 'YADA has determined that all parameters that are supposed to be functions of the estimated parameters are, in fact, fixed. YADA will therefore skip these parameters in the plots.';
                        About(txt,'information','DSGE-VAR Posterior Densities - Additional Parameters',300,500,CurrINI);
                        UpdatePostSample = [];
                        UpdateMode = [];
                        UpdateNames = '';
                        UpdatePriorX = [];
                        UpdatePriorY = [];
                        UpdatePriorBounds = [];
                     end;
                  else;
                     UpdatePostSample = [];
                     UpdateNames = '';
                     UpdatePriorX = [];
                     UpdatePriorY = [];
                     UpdatePriorBounds = [];
                  end;
               else;
                  UpdatePostSample = [];
                  UpdateMode = [];
                  UpdateNames = '';
                  UpdatePriorX = [];
                  UpdatePriorY = [];
                  UpdatePriorBounds = [];
               end;
            else;
               UpdatePostSample = [];
               UpdateMode = [];
               UpdateNames = '';
               UpdatePriorX = [];
               UpdatePriorY = [];
               UpdatePriorBounds = [];
            end;
            %
            % send the data to the posterior densities plotting function
            %
            drawnow;
            GraphDSGEVARPosteriorDensity(thetaPostSample,lambda,PME,ModeFile,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,DSGEModel,CurrINI,controls,1);
            %
            % clear everything
            %
            clear('thetaPostSample');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'VARPosteriorDensity'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check which function was selected
         %
         if gcbo==controls.dsgevarview.postdensity.varparameter;
            ParamStr = 'VAR';
            ParamType = 1;
         elseif gcbo==controls.dsgevarview.postdensity.sigmaparameter;
            ParamStr = 'Sigma';
            ParamType = 2;
         elseif gcbo==controls.dsgevarview.postdensity.a0parameter;
            ParamStr = 'A0';
            ParamType = 3;
         end;
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection(['DSGE-VAR Posterior Densities - ' ParamStr ' Parameters'],DSGEModel,controls,CurrINI);
         %
         % check if the user made a selection
         %
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample');
            NumDraws = size(PhiDraws,1);
            if NumDraws>=100;
               if (ParamType<3)|(isempty(A0Draws)==0);
                  %
                  % clean up memory from parameter draws not needed
                  %
                  if ParamType==1;
                     PostSample = PhiDraws;
                     ParamMode = PhiMode;
                  elseif ParamType==2;
                     PostSample = SigmaDraws;
                     ParamMode = SigmaModeEpsilon;
                  elseif ParamType==3;
                     PostSample = A0Draws;
                     ParamMode = A0Mode;
                  end;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  %
                  % run the graph posterior function
                  %
                  drawnow;
                  GraphVARDSGEPosteriorDensity(ParamStr,PostSample,ParamMode,lambda,PME,CurrChain,DSGEModel,CurrINI);
                  clear('PostSample');
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior densities of the A0 parameters.'];
                  About(txt,'information',['DSGE-VAR Posterior Densities - ' ParamStr ' Parameters'],200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior densities of these parameters. You currently only have ' int2str(NumDraws) ' posterior draws of the ' ParamStr ' parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information',['DSGE-VAR Posterior Densities - ' ParamStr ' Parameters'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewLaplacePosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevarview.viewlaplaceposterior.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevarview.viewlaplaceposterior.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Laplace Posterior Mode - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               thetaMode = ModeData.thetaMode;
               InverseHessian = ModeData.InverseHessian;
               if isfield(ModeData,'ThetaPhiPartial')==1;
                  ThetaPhiPartial = ModeData.ThetaPhiPartial;
               else;
                  ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
               end;
               if isfield(ModeData,'thetaAdditionalMode')==1;
                  thetaAdditionalMode = ModeData.thetaAdditionalMode;
               else;
                  [thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModeData.ModelParameters,ModeData.ParameterNames);
               end;
               if (isfield(ModeData,'phiInverseHessianFinDiff')==1)&(all(~isnan(ModeData.phiInverseHessianFinDiff))==1)&(size(ModeData.phiInverseHessianFinDiff,1)==length(thetaMode));
                  txt = 'When computing the Laplace approximation of the posterior densities, would you like to use the finite difference estimator of the inverse Hessian instead of the inverse Hessian from the optimization routine?';
                  answer = Query(txt,'question',200,['Laplace Posterior Densities - ' ModeType],500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     thetaInverseHessian = ThetaPhiPartial*ModeData.phiInverseHessianFinDiff*ThetaPhiPartial;
                  else;
                     thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
                  end;
               else;
                  thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
               end;
               %
               % we can now construct the plots
               %
               GraphLaplacePosteriorDensity(thetaMode,thetaInverseHessian,thetaAdditionalMode,ModeData.theta,ModeData.thetaDist,ModeData.LowerBound,ModeData.PriorDist,ModeData.ParameterNames,ModeData.thetaPositions,ModeData.ModelParameters,lambdai,ModeType,controls,DSGEModel,CurrINI);
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DSGEVARSimulateDataInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Simulate Data - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  if IsPositiveDefinite(SigmaEpsilon)==1;
                     %
                     % select the number of observations for the simulations
                     %
                     NumObs = (50:25:10000);
                     Tpos = (NumObs==DSGEModel.T) .* (1:length(NumObs));
                     Tpos = Tpos(Tpos>0);
                     if isempty(Tpos)==1;
                        Tpos = (NumObs>DSGEModel.T+1) .* (1:length(NumObs));
                        Tpos = Tpos(Tpos>0);
                        if isempty(Tpos)==0;
                           Tpos = Tpos(1);
                        end;
                        if Tpos==1;
                           NumObs = [DSGEModel.T NumObs];
                        elseif isempty(Tpos)==1;
                           NumObs = [NumObs DSGEModel.T];
                           Tpos = length(NumObs);
                        else;
                           NumObs = [NumObs(1:Tpos-1) DSGEModel.T NumObs(Tpos+1:length(NumObs))];
                        end;
                     end;
                     %
                     % create the string matrix with the number of observation
                     %
                     ObsStrMat = '';
                     for i=1:length(NumObs);
                        ObsStrMat = strvcat(ObsStrMat,[int2str(NumObs(i)) ' observations']);
                     end;
                     %
                     % display the selection dialog
                     %
                     [action,SelectedOption] = SelectionDlg(ObsStrMat,Tpos,'Select the number of observations to simulate','Simulate Data Selection','Simulate Data - Initial Values','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        %
                        % simulate data with the DSGE-VAR
                        %
                        SimulatedData = DSGEVARSimulateData(Phi,SigmaEpsilon,NumObs(SelectedOption),DSGEModel,CurrINI);
                        %
                        % ask if simulated data should be saved to file
                        %
                        txt = 'Would you like to save the simulated data for the initial parameter values to file?';
                        answer = Query(txt,'question',140,'Save - Simulate Data',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           Y = SimulatedData.Y;
                           X = SimulatedData.X;
                           Epsilon = SimulatedData.Epsilon;
                           muY = SimulatedData.Parameters.muY;
                           YNames = DSGEModel.VariableNames;
                           ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-SimulatedData-' DSGEModel.NameOfModel '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(NumObs(SelectedOption)) '-InitialValues.mat']);
                           if isempty(DSGEModel.X)==0;
                              XNames = DSGEModel.XVariableNames;
                              save(file,'Y','X','Epsilon','Phi','SigmaEpsilon','muY','YNames','XNames','ShockNames');
                              NumEntries = '9';
                           else;
                              save(file,'Y','Epsilon','Phi','SigmaEpsilon','muY','YNames','ShockNames');
                              NumEntries = '9';
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARSimData' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(NumObs(SelectedOption)) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% string matrix with the names of the exogenous variables\n%%\n');
                              fprintf(fid,'XNames = DataStructure.XNames;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the exonomic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% matrix with the generated exogenous variables\n%%\n');
                              fprintf(fid,'X = DataStructure.X;\n');
                              XNameStr = 'XNames (string matrix with names of the exogenous variables), ';
                              XStr = 'X (matrix with generated data for the exogenous variables), ';
                           else;
                              XNameStr = '';
                              XStr = '';
                           end;
                           fprintf(fid,'%%\n%% matrix with the simulated innovations\n%%\n');
                           fprintf(fid,'Epsilon = DataStructure.Epsilon;\n');
                           
                           fprintf(fid,'%%\n%% vector with stacked initial values for the observed variables: vec([y(0) ... y(1-p)])\n%%\n');
                           fprintf(fid,'muY = DataStructure.muY;\n');
                           fprintf(fid,'%%\n%% matrix with VAR parameters\n%%\n');
                           fprintf(fid,'Phi = DataStructure.Phi;\n');
                           fprintf(fid,'%%\n%% matrix with innovation covariances\n%%\n');
                           fprintf(fid,'SigmaEpsilon = DataStructure.SigmaEpsilon;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The simulated data from a DSGE-VAR with lambda=' num2str(lambdai) ' and p=' int2str(DSGEModel.BVARLags) ' lags have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), ' XNameStr 'ShockNames (string matrix with the names of the economic shocks), Y (matrix with simulated data for the observed variables), ' XStr 'Epsilon (matrix with simulated innovations), muY (vector with stacked initial values of the observed variables, Phi (matrix with VAR parameters), and SigmaEpsilon (covariance matrix of the innovations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Simulated Data - Initial Values',400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the generated data
                        %
                        GraphSimulatedData('init',CurrINI,DSGEModel,SimulatedData.Y,lambdai,'Initial Values');
                     end;
                  else;
                     txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive definite. YADA is therefore unable to simulate data.'];
                     About(txt,'information','Simulate Date - Initial Values',300,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'DSGEVARSimulateDataPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.simulatedata.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.simulatedata.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.simulatedata.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Simulate Data - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  SigmaEpsilon = ModeData.SigmaModeEpsilon;
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               end;
               if ModeStatus==1;
                  if IsPositiveDefinite(SigmaEpsilon)==1;
                     %
                     % select the number of observations for the simulations
                     %
                     NumObs = (50:25:10000);
                     Tpos = (NumObs==DSGEModel.T) .* (1:length(NumObs));
                     Tpos = Tpos(Tpos>0);
                     if isempty(Tpos)==1;
                        Tpos = (NumObs>DSGEModel.T+1) .* (1:length(NumObs));
                        Tpos = Tpos(Tpos>0);
                        if isempty(Tpos)==0;
                           Tpos = Tpos(1);
                        end;
                        if Tpos==1;
                           NumObs = [DSGEModel.T NumObs];
                        elseif isempty(Tpos)==1;
                           NumObs = [NumObs DSGEModel.T];
                           Tpos = length(NumObs);
                        else;
                           NumObs = [NumObs(1:Tpos-1) DSGEModel.T NumObs(Tpos+1:length(NumObs))];
                        end;
                     end;
                     %
                     % create the string matrix with the number of observation
                     %
                     ObsStrMat = '';
                     for i=1:length(NumObs);
                        ObsStrMat = strvcat(ObsStrMat,[int2str(NumObs(i)) ' observations']);
                     end;
                     %
                     % display the selection dialog
                     %
                     [action,SelectedOption] = SelectionDlg(ObsStrMat,Tpos,'Select the number of observations to simulate','Simulate Data Selection',['Simulate Data - ' ModeType],'','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        %
                        % simulate data with the DSGE-VAR
                        %
                        SimulatedData = DSGEVARSimulateData(Phi,SigmaEpsilon,NumObs(SelectedOption),DSGEModel,CurrINI);
                        %
                        % ask if simulated data should be saved to file
                        %
                        txt = ['Would you like to save the simulated data for the ' lower(ModeType) ' parameter estimate to file?'];
                        answer = Query(txt,'question',140,'Save - Simulate Data',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           Y = SimulatedData.Y;
                           X = SimulatedData.X;
                           Epsilon = SimulatedData.Epsilon;
                           muY = SimulatedData.Parameters.muY;
                           YNames = DSGEModel.VariableNames;
                           ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-SimulatedData-' DSGEModel.NameOfModel '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(NumObs(SelectedOption)) '-' ModeShort '.mat']);
                           if isempty(DSGEModel.X)==0;
                              XNames = DSGEModel.XVariableNames;
                              save(file,'Y','X','Epsilon','Phi','SigmaEpsilon','muY','YNames','XNames','ShockNames');
                              NumEntries = '9';
                           else;
                              save(file,'Y','Epsilon','Phi','SigmaEpsilon','muY','YNames','ShockNames');
                              NumEntries = '9';
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARSimData' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(NumObs(SelectedOption)) ModeShort '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% string matrix with the names of the exogenous variables\n%%\n');
                              fprintf(fid,'XNames = DataStructure.XNames;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the exonomic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% matrix with the generated exogenous variables\n%%\n');
                              fprintf(fid,'X = DataStructure.X;\n');
                              XNameStr = 'XNames (string matrix with names of the exogenous variables), ';
                              XStr = 'X (matrix with generated data for the exogenous variables), ';
                           else;
                              XNameStr = '';
                              XStr = '';
                           end;
                           fprintf(fid,'%%\n%% matrix with the simulated innovations\n%%\n');
                           fprintf(fid,'Epsilon = DataStructure.Epsilon;\n');
                           
                           fprintf(fid,'%%\n%% vector with stacked initial values for the observed variables: vec([y(0) ... y(1-p)])\n%%\n');
                           fprintf(fid,'muY = DataStructure.muY;\n');
                           fprintf(fid,'%%\n%% matrix with VAR parameters\n%%\n');
                           fprintf(fid,'Phi = DataStructure.Phi;\n');
                           fprintf(fid,'%%\n%% matrix with innovation covariances\n%%\n');
                           fprintf(fid,'SigmaEpsilon = DataStructure.SigmaEpsilon;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The simulated data from a DSGE-VAR with lambda=' num2str(lambdai) ' and p=' int2str(DSGEModel.BVARLags) ' lags have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), ' XNameStr 'ShockNames (string matrix with the names of the economic shocks), Y (matrix with simulated data for the observed variables), ' XStr 'Epsilon (matrix with simulated innovations), muY (vector with stacked initial values of the observed variables, Phi (matrix with VAR parameters), and SigmaEpsilon (covariance matrix of the innovations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',['Simulated Data - ' ModeType],400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the generated data
                        %
                        GraphSimulatedData('init',CurrINI,DSGEModel,SimulatedData.Y,lambdai,ModeType);
                     end;
                  else;
                     txt = ['The ' lower(ModeType) ' estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive definite. YADA is therefore unable to simulate data.'];
                     About(txt,'information',['Simulate Data - ' ModeType],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShocksInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Structural Shocks - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                     for j=1:DSGEModel.n;
                        ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                     end;
                     %
                     % get the data and sample information
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     % compute the shocks
                     %
                     Shocks = (A0\eye(size(y,1)))*(y-(Phi*Y));
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save the estimated structural shocks at initial values to file?';
                     answer = Query(txt,'question',140,'Save - Structural Shocks',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Shocks-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                        save(file,'ShockNames','Shocks','DateVector');
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARShocks' strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% matrix with the estimate shocks\n%%\n');
                        fprintf(fid,'Shocks = DataStructure.Shocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The estimated structural shocks at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ShockNames (string matrix with the names of the structural shocks), Shocks (matrix with the estimated structural shocks), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Structural Shocks - Initial Values',200,500,CurrINI);
                     end;
                     %
                     % display the shocks
                     %
                     DSGEVARShockGraphDLG('init',DSGEModel,CurrINI,ShockNames,Shocks,[],DateVector,SampleStrMat,'Initial Values');
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not estimate the structural shocks.'];
                     About(txt,'information','Structural Shocks - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ShocksPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine which model to consider
         %
         [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Structural Shocks - Prior',DSGEModel,CurrINI);
         if isempty(lambda)==0;
            %
            % load the prior draws data
            %
            [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
            UsedDraws = size(PhiDraws,1);
            if isempty(A0Draws)==0;
               clear('SigmaDraws');
               %
               % check if prior draws already exist on disk
               %
               if DSGEVARShocksExist(UsedDraws,[],0,[],lambda,DSGEModel,controls)==1;
                  txt = 'YADA has located results for the structural shocks on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Structural Shocks - Prior',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the structural shocks
                  %
                  DoneCalc = DSGEVARStructuralShocks(PhiDraws,A0Draws,[],0,[],lambda,DSGEModel,CurrINI,controls);
               end;
               %
               % clear everything
               %
               clear('PhiDraws','A0Draws');
               if DoneCalc==1;
                  %
                  % sort the results
                  %
                  [Shocks,DateVector,SampleStrMat] = DSGEVARShocksDistribution(DSGEModel,CurrINI,controls,UsedDraws,[],0,[],lambda);
                  ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                  for j=1:DSGEModel.n;
                     ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                  end;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the structural shocks distribution to file?';
                  answer = Query(txt,'question',140,'Save - Structural Shocks',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     DrawsStr = get(controls.posterior.draws,'String');
                     NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ShocksPriorDist-' num2str(lambda,'%0.4f') '-' int2str(UsedDraws) '.mat']);
                     %
                     save(file,'ShockNames','Shocks','DateVector');
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARShocksPriorDist' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(UsedDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     fprintf(fid,'%%\n%% structure with fields u containing the prior mean\n%% \n%% of the structural shocks, and vector field Quantiles\n%% with subfields u and percent holding quantile data and quantile percent.\n%%\n');
                     fprintf(fid,'Shocks = DataStructure.Shocks;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The structural shock distribution has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ShockNames (string matrix with the names of the structural shocks), Shocks (a structure with prior mean and quantile data for the structural shocks), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Structural Shocks - Prior',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  DSGEVARShockGraphDLG('init',DSGEModel,CurrINI,ShockNames,Shocks.u,Shocks.Quantiles,DateVector,SampleStrMat,'Prior Distribution');
               end;
            else;
               clear('PhiDraws','SigmaDraws');
               txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the prior distribution of the structural shocks.'];
               About(txt,'information','Structural Shocks - Prior',200,500,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ShocksPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.shocks.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.shocks.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.shocks.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Structural Shocks - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     end;
                     %
                     % locate the positions in DSGEModel.StateShockNames that correspond to ShockNames
                     %
                     Positions = zeros(DSGEModel.n,1);
                     for j=1:DSGEModel.n;
                        Positions(j) = LocateStr(DSGEModel.StateShockNames,StringTrim(ShockNames(j,:)));
                     end;
                     Positions = Positions(Positions>0);
                     if length(Positions)==DSGEModel.n;
                        ShockNames = DSGEModel.ShockAliases(Positions,:);
                        for j=1:DSGEModel.n;
                           ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                        end;
                     end;
                     %
                     % get the data and sample information
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     % compute the shocks
                     %
                     Shocks = (A0\eye(size(y,1)))*(y-(Phi*Y));
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the estimated structural shocks at ' lower(ModeType) ' to file?'];
                     answer = Query(txt,'question',140,'Save - Structural Shocks',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Shocks-' num2str(lambdai,'%0.4f') '-' strrep(ModeType,' ','') '.mat']);
                        save(file,'ShockNames','Shocks','DateVector');
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARShocks' strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% matrix with the estimate shocks\n%%\n');
                        fprintf(fid,'Shocks = DataStructure.Shocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The estimated structural shocks at ' lower(ModeType) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ShockNames (string matrix with the names of the structural shocks), Shocks (matrix with the estimated structural shocks), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',['Structural Shocks - ' ModeType],200,500,CurrINI);
                     end;
                     %
                     % display the shocks
                     %
                     DSGEVARShockGraphDLG('init',DSGEModel,CurrINI,ShockNames,Shocks,[],DateVector,SampleStrMat,ModeType);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not estimate the structural shocks.'];
                     About(txt,'information',['Structural Shocks - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShocksPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % select DSGE-VAR model
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Structural Shocks - Posterior',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if isempty(A0Draws)==0;
                  %
                  % clear the sigma parameter draws from memory
                  %
                  clear('SigmaModeEpsilon');
                  %
                  % check if posterior draws already exist on disk
                  %
                  if DSGEVARShocksExist(UsedDraws,CurrChain,1,PME,lambda,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the structural shocks on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,'Structural Shocks - Posterior',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the structural shocks
                     %
                     DoneCalc = DSGEVARStructuralShocks(PhiDraws,A0Draws,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                  end;
                  %
                  % clear everything
                  %
                  clear('PhiDraws','A0Draws');
                  if DoneCalc==1;
                     %
                     % sort the results
                     %
                     [Shocks,DateVector,SampleStrMat] = DSGEVARShocksDistribution(DSGEModel,CurrINI,controls,UsedDraws,CurrChain,1,PME,lambda);
                     ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                     for j=1:DSGEModel.n;
                        ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                     end;
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the structural shocks distribution to file?';
                     answer = Query(txt,'question',140,'Save - Structural Shocks',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        DrawsStr = get(controls.posterior.draws,'String');
                        NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ShocksDist-' num2str(lambda,'%0.4f') '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
                        %
                        save(file,'ShockNames','Shocks','DateVector');
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARShocksDist' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% structure with fields u containing the posterior mean\n%% \n%% of the structural shocks, and vector field Quantiles\n%% with subfields u and percent holding quantile data and quantile percent.\n%%\n');
                        fprintf(fid,'Shocks = DataStructure.Shocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The structural shock distribution has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ShockNames (string matrix with the names of the structural shocks), Shocks (a structure with posterior mean and quantile data for the structural shocks), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Structural Shocks - Posterior',200,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     DSGEVARShockGraphDLG('init',DSGEModel,CurrINI,ShockNames,Shocks.u,Shocks.Quantiles,DateVector,SampleStrMat,'Posterior Distribution');
                  end;
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior distribution of the structural shocks.'];
                  About(txt,'information','Structural Shocks - Posterior',200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior distribution of the structural shocks. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Structural Shocks - Posterior',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
            end;
         end;
         if InitStatus==1;
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % select lambda
               %
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Log-Likelihood Function - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  [lnLt,mcode,Status,CalcTime] = DSGEVARLogLikelihood(theta,ModelParameters,thetaPositions,DSGEModel,AIMData,lambdai,1,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  if mcode==1;
                     GraphLogLikelihood(lnLt,CalcTime,DSGEModel,CurrINI,['DSGE-VAR(' num2str(lambdai) ') '],'Initial Values');
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.loglikelihood.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            PME = 2;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.loglikelihood.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            PME = 3;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.loglikelihood.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            PME = 1;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Log-Likelihood Function - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % compute the log-likelihood
               %
               [lnLt,mcode,Status,CalcTime] = DSGEVARLogLikelihood(ModeData.thetaMode,ModeData.ModelParameters,ModeData.thetaPositions,DSGEModel,ModeData.AIMData,lambdai,PME,CurrINI.OrderQZ,CurrINI.NumWorkers);
               if mcode==1;
                  GraphLogLikelihood(lnLt,CalcTime,DSGEModel,CurrINI,['DSGE-VAR(' num2str(lambdai) ') '],ModeType);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarDecompInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            if gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.initialvalues;
               DataType = 'Original';
               DecType = 1;
            elseif gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.initialvalues;
               DataType = 'Annualized';
               DecType = 2;
            end;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Historical Decompositions - Initial Values'],'','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     % compute the observed variable decompositions decompositions
                     %
                     [yMean,yInitial,yShocks] = DSGEVARObsVarDecomp(Phi,A0,DSGEModel.BVARLags,DSGEModel.k,y,Y,(DSGEModel.k==1)&(max(Y(1,:))==min(Y(1,:))));
                     %
                     % check if we should annualize
                     %
                     T = size(y,2);
                     if DecType==2;
                        TOld = T;
                        y = Annualize(y,DSGEModel.annual,DSGEModel.annualscale);
                        yMean = Annualize(yMean,DSGEModel.annual,DSGEModel.annualscale);
                        yInitial = Annualize(yInitial,DSGEModel.annual,DSGEModel.annualscale);
                        T = size(y,2);
                        Shocks = zeros(DSGEModel.n,T,DSGEModel.n);
                        for j=1:DSGEModel.n;
                           Shocks(:,:,j) = Annualize(yShocks(:,:,j),DSGEModel.annual,DSGEModel.annualscale);
                        end;
                        yShocks = Shocks;
                        DateVector = DateVector(TOld-T+1:TOld);
                        SampleStrMat = SampleStrMat(TOld-T+1:TOld,:);
                     end;
                     %
                     % ask if we should consider shock groups instead of individual shocks
                     %
                     ShockType = 'Individual';
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the historical observed variable decompositions for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Historical Decompositions - Initial Values'],500,'yes',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           q = size(IMatrix,2);
                           %
                           % shift dimension of yShocks from (n x T x n) into (T x n x n)
                           %
                           Shocks = shiftdim(yShocks,1);
                           %
                           % apply IMatrix to each Txn matrix yielding T x q x n
                           %
                           yShocks = zeros(T,q,DSGEModel.n);
                           for j=1:DSGEModel.n;
                              yShocks(:,:,j) = Shocks(:,:,j)*IMatrix;
                           end;
                           %
                           % shift dimensions of yShocks from (T x q x n) into (n x T x q)
                           %
                           yShocks = shiftdim(yShocks,2);
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the historical observed variable decompositions for the ' lower(DataType) ' data at initial values to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Historical Decompositions'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ObsVarDecompositions-' ShockType '-' DataType '-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'YNames','ShockNames','y','yMean','yInitial','yShocks','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAROVDec' ShockType(1) DataType(1)  strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% data on the observed variables\n%%\n');
                        fprintf(fid,'y = DataStructure.y;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the exogenous variable term of the historical decomposition\n%%\n');
                        fprintf(fid,'yMean = DataStructure.yMean;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the initial values term of the historical decomposition\n%%\n');
                        fprintf(fid,'yInitial = DataStructure.yInitial;\n');
                        if strcmp(ShockType,'Individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxn with the shock terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the n structural shocks\n%%\n');
                           GrpStr = '';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxg with the shock group terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the g shock groups\n%%\n');
                           GrpStr = ' group';
                        end;
                        fprintf(fid,'yShocks = DataStructure.yShocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The historical observed variable decomposition at initial values has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 7 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), y (matrix with the data on the observed variables), yMean (exogenous variables term in the historical decomposition), yInitial (initial values term in the historical decomposition), yShocks (shock' GrpStr ' terms in the historical decomposition), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Historical Decompositions - Initial Values'],400,500,CurrINI);
                     end;
                     %
                     if size(ShockNames,1)==DSGEModel.n;
                        ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                        for j=1:size(ShockNames,1);
                           ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                        end;
                     end;
                     %
                     % display the results
                     %
                     DSGEVARObsVarDecompDLG('init',y,yMean,yInitial,yShocks,DateVector,ShockNames,DSGEModel.DSGEVARShocks,'Initial Values',DecType,lambdai,SampleStrMat,CurrINI,DSGEModel);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the historical observed variable decompositions.'];
                     About(txt,'information',[DataType ' Historical Decompositions - Initial Values'],200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarDecompPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.margmode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.jointmode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.dsgemode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % check which data to compute variance decompositions for
         %
         if (gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.margmode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.jointmode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.original.dsgemode);
            DataType = 'Original';
            DecType = 1;
         elseif (gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.margmode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.jointmode)|(gcbo==controls.dsgevartools.decomposition.ovd.decomp.annual.dsgemode);
            DataType = 'Annualized';
            DecType = 2;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Historical Decompositions - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(ModeData.DSGEModel.DSGEVARShocks);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     end;
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     % compute the observed variable decompositions decompositions
                     %
                     [yMean,yInitial,yShocks] = DSGEVARObsVarDecomp(Phi,A0,DSGEModel.BVARLags,DSGEModel.k,y,Y,(DSGEModel.k==1)&(max(Y(1,:))==min(Y(1,:))));
                     %
                     % check if we should annualize
                     %
                     T = size(y,2);
                     if DecType==2;
                        TOld = T;
                        y = Annualize(y,DSGEModel.annual,DSGEModel.annualscale);
                        yMean = Annualize(yMean,DSGEModel.annual,DSGEModel.annualscale);
                        yInitial = Annualize(yInitial,DSGEModel.annual,DSGEModel.annualscale);
                        T = size(y,2);
                        Shocks = zeros(DSGEModel.n,T,DSGEModel.n);
                        for j=1:DSGEModel.n;
                           Shocks(:,:,j) = Annualize(yShocks(:,:,j),DSGEModel.annual,DSGEModel.annualscale);
                        end;
                        yShocks = Shocks;
                        DateVector = DateVector(TOld-T+1:TOld);
                        SampleStrMat = SampleStrMat(TOld-T+1:TOld,:);
                     end;
                     %
                     % ask if we should consider shock groups instead of individual shocks
                     %
                     ShockType = 'Individual';
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the historical observed variable decompositions for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Historical Decompositions - ' ModeType],500,'yes',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           q = size(IMatrix,2);
                           %
                           % shift dimension of yShocks from (n x T x n) into (T x n x n)
                           %
                           Shocks = shiftdim(yShocks,1);
                           %
                           % apply IMatrix to each Txn matrix yielding T x q x n
                           %
                           yShocks = zeros(T,q,DSGEModel.n);
                           for j=1:DSGEModel.n;
                              yShocks(:,:,j) = Shocks(:,:,j)*IMatrix;
                           end;
                           %
                           % shift dimensions of yShocks from (T x q x n) into (n x T x q)
                           %
                           yShocks = shiftdim(yShocks,2);
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the historical observed variable decompositions for the ' lower(DataType) ' data at ' lower(ModeType) ' to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Historical Decompositions'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ObsVarDecompositions-' ShockType '-' DataType '-' num2str(lambdai,'%0.4f') '-' ModeShort '.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'YNames','ShockNames','y','yMean','yInitial','yShocks','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAROVDec' ShockType(1) DataType(1)  strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% data on the observed variables\n%%\n');
                        fprintf(fid,'y = DataStructure.y;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the exogenous variable term of the historical decomposition\n%%\n');
                        fprintf(fid,'yMean = DataStructure.yMean;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the initial values term of the historical decomposition\n%%\n');
                        fprintf(fid,'yInitial = DataStructure.yInitial;\n');
                        if strcmp(ShockType,'Individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxn with the shock terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the n structural shocks\n%%\n');
                           GrpStr = '';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxg with the shock group terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the g shock groups\n%%\n');
                           GrpStr = ' group';
                        end;
                        fprintf(fid,'yShocks = DataStructure.yShocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The historical observed variable decomposition at ' lower(ModeType) ' values has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 7 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), y (matrix with the data on the observed variables), yMean (exogenous variables term in the historical decomposition), yInitial (initial values term in the historical decomposition), yShocks (shock' GrpStr ' terms in the historical decomposition), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Historical Decompositions - ' ModeType],400,500,CurrINI);
                     end;
                     %
                     if size(ShockNames,1)==DSGEModel.n;
                        ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                        for j=1:size(ShockNames,1);
                           ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                        end;
                     end;
                     %
                     % display the results
                     %
                     DSGEVARObsVarDecompDLG('init',y,yMean,yInitial,yShocks,DateVector,ShockNames,ModeData.DSGEModel.DSGEVARShocks,ModeType,DecType,lambdai,SampleStrMat,CurrINI,DSGEModel);
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to compute the historical observed variable decompositions.'];
                     About(txt,'information',[DataType ' Historical Decompositions - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarPathsInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            if gcbo==controls.dsgevartools.decomposition.ovd.paths.original.initialvalues;
               DataType = 'Original';
               DecType = 1;
            elseif gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.initialvalues;
               DataType = 'Annualized';
               DecType = 2;
            end;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Decomposition Paths - Initial Values'],'','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     % compute the observed variable decompositions decompositions
                     %
                     [yMean,yInitial,yShocks] = DSGEVARObsVarDecomp(Phi,A0,DSGEModel.BVARLags,DSGEModel.k,y,Y,(DSGEModel.k==1)&(max(Y(1,:))==min(Y(1,:))));
                     %
                     % check if we should annualize
                     %
                     T = size(y,2);
                     if DecType==2;
                        TOld = T;
                        y = Annualize(y,DSGEModel.annual,DSGEModel.annualscale);
                        yMean = Annualize(yMean,DSGEModel.annual,DSGEModel.annualscale);
                        yInitial = Annualize(yInitial,DSGEModel.annual,DSGEModel.annualscale);
                        T = size(y,2);
                        Shocks = zeros(DSGEModel.n,T,DSGEModel.n);
                        for j=1:DSGEModel.n;
                           Shocks(:,:,j) = Annualize(yShocks(:,:,j),DSGEModel.annual,DSGEModel.annualscale);
                        end;
                        yShocks = Shocks;
                        DateVector = DateVector(TOld-T+1:TOld);
                        SampleStrMat = SampleStrMat(TOld-T+1:TOld,:);
                     end;
                     %
                     % ask if we should consider shock groups instead of individual shocks
                     %
                     ShockType = 'Individual';
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the historical observed variable decomposition paths for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Decomposition Paths - Initial Values'],500,'yes',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           q = size(IMatrix,2);
                           %
                           % shift dimension of yShocks from (n x T x n) into (T x n x n)
                           %
                           Shocks = shiftdim(yShocks,1);
                           %
                           % apply IMatrix to each Txn matrix yielding T x q x n
                           %
                           yShocks = zeros(T,q,DSGEModel.n);
                           for j=1:DSGEModel.n;
                              yShocks(:,:,j) = Shocks(:,:,j)*IMatrix;
                           end;
                           %
                           % shift dimensions of yShocks from (T x q x n) into (n x T x q)
                           %
                           yShocks = shiftdim(yShocks,2);
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the historical observed variable decomposition paths for the ' lower(DataType) ' data at initial values to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Decomposition Paths'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ObsVarDecompositions-' ShockType '-' DataType '-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'YNames','ShockNames','y','yMean','yInitial','yShocks','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAROVDec' ShockType(1) DataType(1)  strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% data on the observed variables\n%%\n');
                        fprintf(fid,'y = DataStructure.y;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the exogenous variable term of the historical decomposition\n%%\n');
                        fprintf(fid,'yMean = DataStructure.yMean;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the initial values term of the historical decomposition\n%%\n');
                        fprintf(fid,'yInitial = DataStructure.yInitial;\n');
                        if strcmp(ShockType,'Individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxn with the shock terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the n structural shocks\n%%\n');
                           GrpStr = '';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxg with the shock group terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the g shock groups\n%%\n');
                           GrpStr = ' group';
                        end;
                        fprintf(fid,'yShocks = DataStructure.yShocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The historical observed variable decomposition paths at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 7 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), y (matrix with the data on the observed variables), yMean (exogenous variables term in the historical decomposition), yInitial (initial values term in the historical decomposition), yShocks (shock' GrpStr ' terms in the historical decomposition), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Decomposition Paths - Initial Values'],400,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     DSGEVARObsVarDecPathsDLG('init',y,yMean,yInitial,yShocks,[],DateVector,ShockNames,'Initial Values',DecType,lambdai,SampleStrMat,CurrINI,DSGEModel);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the historical observed variable decompositions.'];
                     About(txt,'information',[DataType ' Decomposition Paths - Initial Values'],200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarPathsPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % are we using annualized data, levels, or the original
         %
         if gcbo==controls.dsgevartools.decomposition.ovd.paths.original.prior;
            DataType = 'Original';
            DecType = 1;
         elseif gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.prior;
            DataType = 'Annualized';
            DecType = 2;
         end;
         %
         % determine which model to consider
         %
         [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws([DataType ' Decomposition Paths - Prior'],DSGEModel,CurrINI);
         if isempty(lambda)==0;
            %
            % load the prior draws data
            %
            [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
            UsedDraws = size(PhiDraws,1);
            if isempty(A0Draws)==0;
               clear('SigmaDraws');
               %
               ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
               %
               % check if the results already exist on disk
               %
               if DSGEVAROVPsExist(UsedDraws,DecType,[],0,[],lambda,DSGEModel,controls)==1;
                  txt = 'YADA has located results for the historical observed variable decomposition paths on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,[DataType ' Decomposition Paths - Prior'],500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               %
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the observed variable decomposition paths
                  %
                  DoneCalc = DSGEVARObsVarDecompDistribution(PhiDraws,A0Draws,DecType,[],0,[],lambda,DSGEModel,CurrINI,controls);
               end;
               %
               % clear everything
               %
               clear('PhiDraws','A0Draws');
               if DoneCalc==1;
                  %
                  % get access to the data and sample information
                  %
                  [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                  DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                  DateVector = DateVector(HSample(1):HSample(2));
                  SampleStrMat = CreateFullSample(DSGEModel);
                  SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                  if DecType==2;
                     %
                     % annualize
                     %
                     TOld = size(y,2);
                     y = Annualize(y,DSGEModel.annual,DSGEModel.annualscale);
                     T = size(y,2);
                     DateVector = DateVector(TOld-T+1:TOld);
                     SampleStrMat = SampleStrMat(TOld-T+1:TOld,:);
                  end;
                  %
                  % ask if shock groups should be considered
                  %
                  ShockType = 'Individual';
                  PrintGroups = 0;
                  ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                  if length(ShockGroups)>max(ShockGroups);
                     txt = 'Would you like to view results of the historical observed variable decomposition paths for shock groups instead of for the individual shocks?';
                     answer = Query(txt,'question',200,[DataType ' Decomposition Paths - Prior'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        ShockType = 'Group';
                        PrintGroups = 1;
                        ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                     end;
                  end;
                  %
                  % sort the results
                  %
                  OVDPs = DSGEVAROVPsDistribution(DSGEModel,CurrINI,controls,lower(ShockType),UsedDraws,DecType,[],0,[],[],lambda,size(y,2));
                  if isempty(OVDPs)==0;
                     %
                     % check if the mean decomposition and the data comply
                     %
                     yCheck = OVDPs.yMean+OVDPs.yInitial+sum(OVDPs.yShocks,3);
                     if max(max(abs(y-yCheck)))<=1.0e-05;
                        answer = 'Yes';
                     else;
                        txt = ['The largest absolute error for the mean historical observed variable decomposition is ' num2str(max(max(abs(y-yCheck)))) '. Are you sure you want to continue?'];
                        answer = Query(txt,'question',200,[DataType ' Decomposition Paths - Prior'],500,'no',CurrINI);
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % save results to disk
                        %
                        txt = ['Would you like to save the prior distribution of the historical observed variable decomposition paths for the ' lower(DataType) ' data to file?'];
                        answer = Query(txt,'question',140,['Save - ' DataType ' Decomposition Paths'],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ObsVarDecompositions-PriorDist-' ShockType '-' DataType '-' num2str(lambda,'%0.4f') '.mat']);
                           YNames = DSGEModel.VariableNames;
                           yMean = OVDPs.yMean;
                           yInitial = OVDPs.yInitial;
                           yShocks = OVDPs.yShocks;
                           Quantiles = OVDPs.Quantiles;
                           save(file,'YNames','ShockNames','y','yMean','yInitial','yShocks','Quantiles','DateVector');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAROVDecPriorDist' ShockType(1) DataType(1)  strrep(num2str(lambda,'%0.4f'),'.','_') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% data on the observed variables\n%%\n');
                           fprintf(fid,'y = DataStructure.y;\n');
                           fprintf(fid,'%%\n%% matrix of dimension nxT with the prior mean of the exogenous variable term of the historical decomposition\n%%\n');
                           fprintf(fid,'yMean = DataStructure.yMean;\n');
                           fprintf(fid,'%%\n%% matrix of dimension nxT with the prior mean of the initial values term of the historical decomposition\n%%\n');
                           fprintf(fid,'yInitial = DataStructure.yInitial;\n');
                           if strcmp(ShockType,'Individual')==1;
                              fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxn with the prior mean of the shock terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the n structural shocks\n%%\n');
                              GrpStr = '';
                           else;
                              fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxg with the prior mean of the shock group terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the g shock groups\n%%\n');
                              GrpStr = ' group';
                           end;
                           fprintf(fid,'yShocks = DataStructure.yShocks;\n');
                           fprintf(fid,'%%\n%% structure with quantile data\n%%\n');
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The prior distribution of the historical observed variable decomposition paths have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), y (matrix with the data on the observed variables), yMean (exogenous variables term in the historical decomposition), yInitial (initial values term in the historical decomposition), yShocks (shock' GrpStr ' terms in the historical decomposition), Quantiles (structure with quantile data on the decomposition of the observed variables for the sum of the initial value, exogenous variable, and shock' GrpStr ' terms), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',[DataType ' Decomposition Paths - Prior'],400,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        DSGEVARObsVarDecPathsDLG('init',y,OVDPs.yMean,OVDPs.yInitial,OVDPs.yShocks,OVDPs.Quantiles,DateVector,ShockNames,'Prior Distribution',DecType,lambda,SampleStrMat,CurrINI,DSGEModel);
                     end;
                  end;
               end;
            else;
               clear('PhiDraws','SigmaDraws');
               txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the prior distribution of the historical observed variable decomposition paths.'];
               About(txt,'information',[DataType ' Decomposition Paths - Prior'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarPathsPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.decomposition.ovd.paths.original.margmode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.decomposition.ovd.paths.original.jointmode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.decomposition.ovd.paths.original.dsgemode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % check which data to compute variance decompositions for
         %
         if (gcbo==controls.dsgevartools.decomposition.ovd.paths.original.margmode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.original.jointmode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.original.dsgemode);
            DataType = 'Original';
            DecType = 1;
         elseif (gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.margmode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.jointmode)|(gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.dsgemode);
            DataType = 'Annualized';
            DecType = 2;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Decomposition Paths - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(ModeData.DSGEModel.DSGEVARShocks);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     end;
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     % compute the observed variable decompositions decompositions
                     %
                     [yMean,yInitial,yShocks] = DSGEVARObsVarDecomp(Phi,A0,DSGEModel.BVARLags,DSGEModel.k,y,Y,(DSGEModel.k==1)&(max(Y(1,:))==min(Y(1,:))));
                     %
                     % check if we should annualize
                     %
                     T = size(y,2);
                     if DecType==2;
                        TOld = T;
                        y = Annualize(y,DSGEModel.annual,DSGEModel.annualscale);
                        yMean = Annualize(yMean,DSGEModel.annual,DSGEModel.annualscale);
                        yInitial = Annualize(yInitial,DSGEModel.annual,DSGEModel.annualscale);
                        T = size(y,2);
                        Shocks = zeros(DSGEModel.n,T,DSGEModel.n);
                        for j=1:DSGEModel.n;
                           Shocks(:,:,j) = Annualize(yShocks(:,:,j),DSGEModel.annual,DSGEModel.annualscale);
                        end;
                        yShocks = Shocks;
                        DateVector = DateVector(TOld-T+1:TOld);
                        SampleStrMat = SampleStrMat(TOld-T+1:TOld,:);
                     end;
                     %
                     % ask if we should consider shock groups instead of individual shocks
                     %
                     ShockType = 'Individual';
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the historical observed variable decomposition paths for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Decomposition Paths - ' ModeType],500,'yes',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           q = size(IMatrix,2);
                           %
                           % shift dimension of yShocks from (n x T x n) into (T x n x n)
                           %
                           Shocks = shiftdim(yShocks,1);
                           %
                           % apply IMatrix to each Txn matrix yielding T x q x n
                           %
                           yShocks = zeros(T,q,DSGEModel.n);
                           for j=1:DSGEModel.n;
                              yShocks(:,:,j) = Shocks(:,:,j)*IMatrix;
                           end;
                           %
                           % shift dimensions of yShocks from (T x q x n) into (n x T x q)
                           %
                           yShocks = shiftdim(yShocks,2);
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the historical observed variable decomposition paths for the ' lower(DataType) ' data at ' lower(ModeType) ' to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Decomposition Paths'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ObsVarDecompositions-' ShockType '-' DataType '-' num2str(lambdai,'%0.4f') '-' ModeShort '.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'YNames','ShockNames','y','yMean','yInitial','yShocks','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAROVDec' ShockType(1) DataType(1)  strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% data on the observed variables\n%%\n');
                        fprintf(fid,'y = DataStructure.y;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the exogenous variable term of the historical decomposition\n%%\n');
                        fprintf(fid,'yMean = DataStructure.yMean;\n');
                        fprintf(fid,'%%\n%% matrix of dimension nxT with the initial values term of the historical decomposition\n%%\n');
                        fprintf(fid,'yInitial = DataStructure.yInitial;\n');
                        if strcmp(ShockType,'Individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxn with the shock terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the n structural shocks\n%%\n');
                           GrpStr = '';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxg with the shock group terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the g shock groups\n%%\n');
                           GrpStr = ' group';
                        end;
                        fprintf(fid,'yShocks = DataStructure.yShocks;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The historical observed variable decomposition paths at ' lower(ModeType) ' values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 7 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), y (matrix with the data on the observed variables), yMean (exogenous variables term in the historical decomposition), yInitial (initial values term in the historical decomposition), yShocks (shock' GrpStr ' terms in the historical decomposition), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Decomposition Paths - ' ModeType],400,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     DSGEVARObsVarDecPathsDLG('init',y,yMean,yInitial,yShocks,[],DateVector,ShockNames,ModeType,DecType,lambdai,SampleStrMat,CurrINI,DSGEModel);
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to compute the historical observed variable decomposition paths.'];
                     About(txt,'information',[DataType ' Decomposition Paths - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarPathsPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevartools.decomposition.ovd.paths.original.posterior;
            DataType = 'Original';
            DecType = 1;
         elseif gcbo==controls.dsgevartools.decomposition.ovd.paths.annual.posterior;
            DataType = 'Annualized';
            DecType = 2;
         end;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataType ' Decomposition Paths - Posterior'],DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if isempty(A0Draws)==0;
                  %
                  % we don't need the posterior draws of SigmaEpsilon
                  %
                  clear('SigmaDraws');
                  %
                  ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                  %
                  % check if the results already exist on disk
                  %
                  if DSGEVAROVPsExist(UsedDraws,DecType,CurrChain,1,PME,lambda,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the historical observed variable decomposition paths on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,[DataType ' Decomposition Paths - Posterior'],500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the observed variable decomposition paths
                     %
                     DoneCalc = DSGEVARObsVarDecompDistribution(PhiDraws,A0Draws,DecType,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                  end;
                  %
                  % clear everything
                  %
                  clear('PhiDraws','A0Draws');
                  if DoneCalc==1;
                     %
                     % get access to the data and sample information
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):HSample(2));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     if DecType==2;
                        %
                        % annualize
                        %
                        TOld = size(y,2);
                        y = Annualize(y,DSGEModel.annual,DSGEModel.annualscale);
                        T = size(y,2);
                        DateVector = DateVector(TOld-T+1:TOld);
                        SampleStrMat = SampleStrMat(TOld-T+1:TOld,:);
                     end;
                     %
                     % ask if shock groups should be considered
                     %
                     ShockType = 'Individual';
                     PrintGroups = 0;
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the historical observed variable decomposition paths for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',200,[DataType ' Decomposition Paths - Posterior'],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           ShockType = 'Group';
                           PrintGroups = 1;
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                        end;
                     end;
                     %
                     % sort the results
                     %
                     OVDPs = DSGEVAROVPsDistribution(DSGEModel,CurrINI,controls,lower(ShockType),UsedDraws,DecType,CurrChain,1,Weights,PME,lambda,size(y,2));
                     if isempty(OVDPs)==0;
                        %
                        % check if the mean decomposition and the data comply
                        %
                        yCheck = OVDPs.yMean+OVDPs.yInitial+sum(OVDPs.yShocks,3);
                        if max(max(abs(y-yCheck)))<=1.0e-05;
                           answer = 'Yes';
                        else;
                           txt = ['The largest absolute error for the mean historical observed variable decomposition is ' num2str(max(max(abs(y-yCheck)))) '. Are you sure you want to continue?'];
                           answer = Query(txt,'question',200,[DataType ' Decomposition Paths - Posterior'],500,'no',CurrINI);
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % save results to disk
                           %
                           txt = ['Would you like to save the posterior distribution of the historical observed variable decomposition paths for the ' lower(DataType) ' data to file?'];
                           answer = Query(txt,'question',140,['Save - ' DataType ' Decomposition Paths'],500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-ObsVarDecompositions-PosteriorDist-' ShockType '-' DataType '-' num2str(lambda,'%0.4f') '.mat']);
                              YNames = DSGEModel.VariableNames;
                              yMean = OVDPs.yMean;
                              yInitial = OVDPs.yInitial;
                              yShocks = OVDPs.yShocks;
                              Quantiles = OVDPs.Quantiles;
                              save(file,'YNames','ShockNames','y','yMean','yInitial','yShocks','Quantiles','DateVector');
                              %
                              % create a matlab file to read the data
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAROVDecPostDist' ShockType(1) DataType(1)  strrep(num2str(lambda,'%0.4f'),'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                              fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                              fprintf(fid,'%%\n%% data on the observed variables\n%%\n');
                              fprintf(fid,'y = DataStructure.y;\n');
                              fprintf(fid,'%%\n%% matrix of dimension nxT with the posterior mean of the exogenous variable term of the historical decomposition\n%%\n');
                              fprintf(fid,'yMean = DataStructure.yMean;\n');
                              fprintf(fid,'%%\n%% matrix of dimension nxT with the posterior mean of the initial values term of the historical decomposition\n%%\n');
                              fprintf(fid,'yInitial = DataStructure.yInitial;\n');
                              if strcmp(ShockType,'Individual')==1;
                                 fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxn with the posterior mean of the shock terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the n structural shocks\n%%\n');
                                 GrpStr = '';
                              else;
                                 fprintf(fid,'%%\n%% a 3D matrix of dimension nxTxg with the posterior mean of the shock group terms of the historical decomposition\n%% of the n observed variables for the T time periods, and the g shock groups\n%%\n');
                                 GrpStr = ' group';
                              end;
                              fprintf(fid,'yShocks = DataStructure.yShocks;\n');
                              fprintf(fid,'%%\n%% structure with quantile data\n%%\n');
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              txt = ['The posterior distribution of the historical observed variable decomposition paths have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), y (matrix with the data on the observed variables), yMean (exogenous variables term in the historical decomposition), yInitial (initial values term in the historical decomposition), yShocks (shock' GrpStr ' terms in the historical decomposition), Quantiles (structure with quantile data on the decomposition of the observed variables for the sum of the initial value, exogenous variable, and shock' GrpStr ' terms), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information',[DataType ' Decomposition Paths - Posterior'],400,500,CurrINI);
                           end;
                           %
                           % display the results
                           %
                           DSGEVARObsVarDecPathsDLG('init',y,OVDPs.yMean,OVDPs.yInitial,OVDPs.yShocks,OVDPs.Quantiles,DateVector,ShockNames,'Posterior Distribution',DecType,lambda,SampleStrMat,CurrINI,DSGEModel);
                        end;
                     end;
                  end;
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior distribution of the historical observed variable decomposition paths.'];
                  About(txt,'information',[DataType ' Decomposition Paths - Posterior'],200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior distribution of the historical observed variable decomposition paths. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information',[DataType ' Decomposition Paths - Posterior'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CorrDecompInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Correlation Decompositions - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     %
                     % compute the correlation decompositions
                     %
                     CorrDec = DSGEVARCorrDecompParam(Phi,A0,'Initial Values',DSGEModel,CurrINI);
                     if isempty(CorrDec)==0;
                        %
                        % add shock and group information
                        %
                        CorrDec.ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                        CorrDec.ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                        CorrDec.ShockGroupNames = DSGEModel.ShockGroupNames(unique(CorrDec.ShockGroups),:);
                        if (isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)>=length(unique(CorrDec.ShockGroups)));
                           CorrDec.ShockGroupColors = DSGEModel.ShockGroupColors(unique(CorrDec.ShockGroups),:);
                        else;
                           CorrDec.ShockGroupColors = CreateDefaultColors(length(unique(CorrDec.ShockGroups)));
                        end;
                        CorrDec.ShockAliases = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                        if isempty(DSGEModel.ShockColors)==0;
                           CorrDec.ShockColors = DSGEModel.ShockColors(DSGEModel.DSGEVARShocks,:);
                        else;
                           if max(DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks))==DSGEModel.n
                              CorrDec.ShockColors = CreateDefaultColors(DSGEModel.n);
                           else;
                              CorrDec.ShockColors = CreateShockColors(DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks),DSGEModel.ShockGroupColors);
                           end;
                        end;
                        %
                        % save the results to disk
                        %
                        txt = 'Would you like to save the correlation decompositions of the observed variables at initial values to disk?';
                        answer = Query(txt,'question',200,'Save - Correlation Decompositions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           YDecomp = CorrDec.Y;
                           YNames = DSGEModel.VariableNames;
                           ShockNames = CorrDec.ShockNames;
                           AutoCovHorizon = CorrDec.AutoCovHorizon;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CorrDec-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-InitialValues.mat']);
                           save(file,'YNames','ShockNames','YDecomp','AutoCovHorizon');
                           %
                           % create the script file
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCorrDec' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) SelectedShockStr 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with names of the n observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with names of the n shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% 3D [nx(2h+1)xn] matrix with correlation decompositions of the observed variables\n%%\n');
                           fprintf(fid,'YDecomp = DataStructure.YDecomp;\n');
                           fprintf(fid,'%%\n%% length of the autocovariance horizon (h)\n%%\n');
                           fprintf(fid,'AutoCovHorizon = DataStructure.AutoCovHorizon;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The correlation decompositions of the observed variables at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the shocks), YDecomp (an nx(2h+1)xn matrix with the correlation decompositions of the observed variables for autocovariance horizons -h until h of the n shocks), and AutoCovHorizon (integer with the length of the autocovariance horizon, i.e. h). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Correlation Decompositions - Initial Values',500,600,CurrINI);
                           drawnow;
                           pause(0.02);
                        end;
                        %
                        % display the results
                        %
                        CorrelationDecompositionDLG('init',CorrDec,DSGEModel.VariableNames,'Observed Variables','DSGE-VAR','Initial Values',DSGEModel,CurrINI);
                     end;
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the correlation decompositions.'];
                     About(txt,'information','Correlation Decompositions - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CorrDecompPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.decomposition.corrdecomp.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.decomposition.corrdecomp.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.decomposition.corrdecomp.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Correlation Decompositions - ' ModeType],'','',CurrINI);
            drawnow;
            pause(0.02);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     A0 = ModeData.A0Mode;
                     %
                     % compute the correlation decompositions
                     %
                     CorrDec = DSGEVARCorrDecompParam(Phi,A0,ModeType,DSGEModel,CurrINI);
                     if isempty(CorrDec)==0;
                        %
                        % add shock and group information
                        %
                        if isfield(ModeData,'SigmaModeEpsilon')==1;
                           CorrDec.ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                           CorrDec.ShockGroups = DSGEModel.ShockGroups(ModeData.DSGEModel.DSGEVARShocks);
                        else;
                           CorrDec.ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                           CorrDec.ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                        end;
                        CorrDec.ShockGroupNames = DSGEModel.ShockGroupNames(unique(CorrDec.ShockGroups),:);
                        if (isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)>=length(unique(CorrDec.ShockGroups)));
                           CorrDec.ShockGroupColors = DSGEModel.ShockGroupColors(unique(CorrDec.ShockGroups),:);
                        else;
                           CorrDec.ShockGroupColors = CreateDefaultColors(length(unique(CorrDec.ShockGroups)));
                        end;
                        CorrDec.ShockAliases = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                        if isempty(DSGEModel.ShockColors)==0;
                           CorrDec.ShockColors = DSGEModel.ShockColors(DSGEModel.DSGEVARShocks,:);
                        else;
                           if max(CorrDec.ShockGroups)==DSGEModel.n;
                              CorrDec.ShockColors = CreateDefaultColors(DSGEModel.n);
                           else;
                              CorrDec.ShockColors = CreateShockColors(CorrDec.ShockGroups,DSGEModel.ShockGroupColors);
                           end;
                        end;
                        %
                        % save the results to disk
                        %
                        txt = ['Would you like to save the correlation decompositions of the observed variables at ' lower(ModeType) ' to disk?'];
                        answer = Query(txt,'question',200,'Save - Correlation Decompositions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           YDecomp = CorrDec.Y;
                           YNames = DSGEModel.VariableNames;
                           ShockNames = CorrDec.ShockNames;
                           AutoCovHorizon = CorrDec.AutoCovHorizon;
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CorrDec-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ModeShort '.mat']);
                           save(file,'YNames','ShockNames','YDecomp','AutoCovHorizon');
                           %
                           % create the script file
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCorrDec' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) SelectedShockStr ModeShort '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with names of the n observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with names of the n shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% 3D [nx(2h+1)xn] matrix with correlation decompositions of the observed variables\n%%\n');
                           fprintf(fid,'YDecomp = DataStructure.YDecomp;\n');
                           fprintf(fid,'%%\n%% length of the autocovariance horizon (h)\n%%\n');
                           fprintf(fid,'AutoCovHorizon = DataStructure.AutoCovHorizon;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The correlation decompositions of the observed variables at ' lower(ModeType) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the shocks), YDecomp (an nx(2h+1)xn matrix with the correlation decompositions of the observed variables for autocovariance horizons -h until h of the n shocks), and AutoCovHorizon (integer with the length of the autocovariance horizon, i.e. h). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',['Correlation Decompositions - ' ModeType],500,600,CurrINI);
                           drawnow;
                           pause(0.02);
                        end;
                        %
                        % display the results
                        %
                        CorrelationDecompositionDLG('init',CorrDec,DSGEModel.VariableNames,'Observed Variables','DSGE-VAR',ModeType,DSGEModel,CurrINI);
                     end;
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to compute the correlation decompositions.'];
                     About(txt,'information',['Correlation Decompositions - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SpectralDecompInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Spectral Decompositions - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     %
                     % compute the spectral decomposition
                     %
                     [SyOmega,SyOmegaAnnual,OriginalVariance,AnnualVariance,Omega] = DSGEVARSpectralDecomposition(Phi,A0,lambdai,'Initial Values',DSGEModel,CurrINI);
                     %
                     % save the results to disk
                     %
                     txt = 'Would you like to save the spectral decompositions of the observed variables at initial values to disk?';
                     answer = Query(txt,'question',200,'Save - Spectral Decompositions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        YNames = DSGEModel.VariableNames;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-SpecDec-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                        if isempty(SyOmegaAnnual)==0;
                           save(file,'YNames','ShockNames','SyOmega','SyOmegaAnnual','OriginalVariance','AnnualVariance','Omega');
                        else;
                           save(file,'YNames','ShockNames','SyOmega','OriginalVariance','Omega');
                        end;
                        %
                        % create the script file
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARSpecDec' strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with names of the n observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with names of the q shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% f dimensional vector with individual frequencies\n%%\n');
                        fprintf(fid,'Omega = DataStructure.Omega;\n');
                        fprintf(fid,'%%\n%% nxf cell array with nxn matrices with conditional\n%% spectral density for a given shock and frequency\n%%\n');
                        fprintf(fid,'SyOmega = DataStructure.SyOmega;\n');
                        fprintf(fid,'%%\n%% vector with population variances of the observed variables\n%%\n');
                        fprintf(fid,'OriginalVariance = DataStructure.OriginalVariance;\n');
                        if isempty(SyOmegaAnnual)==0;
                           fprintf(fid,'%%\n%% nxf cell array with nxn matrices with annualized conditional\n%% spectral density for a given shock and frequency\n%%\n');
                           fprintf(fid,'SyOmegaAnnual = DataStructure.SyOmegaAnnual;\n');
                           fprintf(fid,'%%\n%% vector with population variances of the annualized observed variables\n%%\n');
                           fprintf(fid,'AnnualVariance = DataStructure.AnnualVariance;\n');
                           NumEntries = 7;
                           AddText = 'OriginalVariance (vector with the population variances of the observed variables), SyOmegaAnnual (an nxf dimenional cell array with nxn matrices of the population spectrum of the annualized observed variables for each shock at frequencies 0 until pi), and AnnualVariance (vector with population variances of the annualized observed variables)';
                        else;
                           NumEntries = 5;
                           AddText = 'and OriginalVariance (vector with the population variances of the observed variables)';
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The spectral decompositions of the observed variables at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the shocks), Omega (an f dimensional vector with the individual frequencies for the spectral decomposition), SyOmega (an nxf dimenional cell array with nxn matrices of the population spectrum of the observed variables for each shock at frequencies 0 until pi), ' AddText '. To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Spectral Decompositions - Initial Values',500,600,CurrINI);
                        drawnow;
                        pause(0.02);
                     end;
                     %
                     % display the results
                     %
                     DSGEVARSpectralDecompDLG('init',Omega,SyOmega,OriginalVariance,SyOmegaAnnual,AnnualVariance,lambdai,ShockNames,ShockGroups,'Initial Values',DSGEModel,CurrINI);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the spectral decompositions.'];
                     About(txt,'information','Spectral Decompositions - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SpectralDecompPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if gcbo==controls.dsgevartools.decomposition.spectraldecomp.margmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.decomposition.spectraldecomp.jointmode;
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif gcbo==controls.dsgevartools.decomposition.spectraldecomp.dsgemode;
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Spectral Decompositions - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(ModeData.DSGEModel.DSGEVARShocks);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     end;
                     %
                     % compute the spectral decomposition
                     %
                     [SyOmega,SyOmegaAnnual,OriginalVariance,AnnualVariance,Omega] = DSGEVARSpectralDecomposition(Phi,A0,lambdai,ModeType,DSGEModel,CurrINI);
                     %
                     % save the results to disk
                     %
                     txt = ['Would you like to save the spectral decompositions of the observed variables at ' lower(ModeType) ' to disk?'];
                     answer = Query(txt,'question',200,'Save - Spectral Decompositions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        YNames = DSGEModel.VariableNames;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-SpecDec-' num2str(lambdai,'%0.4f') '-' ModeShort '.mat']);
                        if isempty(SyOmegaAnnual)==0;
                           save(file,'YNames','ShockNames','SyOmega','SyOmegaAnnual','OriginalVariance','AnnualVariance','Omega');
                        else;
                           save(file,'YNames','ShockNames','SyOmega','OriginalVariance','Omega');
                        end;
                        %
                        % create the script file
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARSpecDec' strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with names of the n observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with names of the q shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% f dimensional vector with individual frequencies\n%%\n');
                        fprintf(fid,'Omega = DataStructure.Omega;\n');
                        fprintf(fid,'%%\n%% nxf cell array with nxn matrices with conditional\n%% spectral density for a given shock and frequency\n%%\n');
                        fprintf(fid,'SyOmega = DataStructure.SyOmega;\n');
                        fprintf(fid,'%%\n%% vector with population variances of the observed variables\n%%\n');
                        fprintf(fid,'OriginalVariance = DataStructure.OriginalVariance;\n');
                        if isempty(SyOmegaAnnual)==0;
                           fprintf(fid,'%%\n%% nxf cell array with nxn matrices with annualized conditional\n%% spectral density for a given shock and frequency\n%%\n');
                           fprintf(fid,'SyOmegaAnnual = DataStructure.SyOmegaAnnual;\n');
                           fprintf(fid,'%%\n%% vector with population variances of the annualized observed variables\n%%\n');
                           fprintf(fid,'AnnualVariance = DataStructure.AnnualVariance;\n');
                           NumEntries = 7;
                           AddText = 'OriginalVariance (vector with the population variances of the observed variables), SyOmegaAnnual (an nxf dimenional cell array with nxn matrices of the population spectrum of the annualized observed variables for each shock at frequencies 0 until pi), and AnnualVariance (vector with population variances of the annualized observed variables)';
                        else;
                           NumEntries = 5;
                           AddText = 'and OriginalVariance (vector with the population variances of the observed variables)';
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The spectral decompositions of the observed variables at ' lower(ModeType) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the shocks), Omega (an f dimensional vector with the individual frequencies for the spectral decomposition), SyOmega (an nxf dimenional cell array with nxn matrices of the population spectrum of the observed variables for each shock at frequencies 0 until pi), ' AddText '. To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',['Spectral Decompositions - ' ModeType],500,600,CurrINI);
                        drawnow;
                        pause(0.02);
                     end;
                     %
                     % display the results
                     %
                     DSGEVARSpectralDecompDLG('init',Omega,SyOmega,OriginalVariance,SyOmegaAnnual,AnnualVariance,lambdai,ShockNames,ShockGroups,ModeType,DSGEModel,CurrINI);
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to compute the spectral decompositions.'];
                     About(txt,'information',['Spectral Decompositions - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'FEVDInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            if gcbo==controls.dsgevartools.decomposition.fevd.original.initialvalues;
               DataType = 'Original';
               VDType = 1;
            elseif gcbo==controls.dsgevartools.decomposition.fevd.levels.initialvalues;
               DataType = 'Levels';
               VDType = 2;
            end;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Variance Decompositions - Initial Values'],'','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     %
                     if strcmp(DSGEModel.DataFrequency,'q')==1;
                        SampleFreq = 4;
                     elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                        SampleFreq = 12;
                     else;
                        SampleFreq = 1;
                     end;
                     h = SampleFreq*DSGEModel.IRHorizon;
                     %
                     % compute the variance decompositions
                     %
                     [FEVDs,LRVD,VarShares] = DSGEVARVarianceDecompositions(Phi,A0,DSGEModel.BVARLags,h,VDType,DSGEModel.levels);
                     %
                     % determine string with 1's and 0's for the selected shocks
                     %
                     SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                     SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                     %
                     % check if we should consider shock groups instead of the individual shocks
                     %
                     ShockType = 'individual';
                     AddName = 'Ind';
                     ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     PrintGroups = 0;
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the variance decompositions for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Variance Decompositions - Initial Values'],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           FEVDGs = zeros(DSGEModel.n,size(IMatrix,2),h);
                           for i=1:h;
                              FEVDGs(:,:,i) = FEVDs(:,:,i)*IMatrix;
                           end;
                           ShockType = 'group';
                           AddName = 'Group';
                           FEVDs = FEVDGs;
                           LRVD = LRVD*IMatrix;
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           PrintGroups = 1;
                        end;
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the variance decompositions for the ' lower(DataType) ' data at initial values to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Variance Decompositions'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-VarianceDecompositions-' AddName '-' DataType '-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'FEVDs','LRVD','YNames','ShockNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARVD' AddName DataType strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',ShockType);
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        if strcmp(ShockType,'individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxnxh with the variance decompositions\n%% for the n observed variables, the n structural shocks, and the h forecast horizons\n%%\n');
                           ShockNumStr = 'n';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxgxh with the variance decompositions\n%% for the n observed variables, the g shock groups, and the h forecast horizons\n%%\n');
                           ShockNumStr = 'g';
                        end;
                        fprintf(fid,'FEVDs = DataStructure.FEVDs;\n');
                        fprintf(fid,'%%\n%% long run variance decomposition\n%%\n');
                        fprintf(fid,'LRVD = DataStructure.LRVD;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The variance decompositions for the ' lower(DataType) ' data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' ShockType ' DSGE-VAR shocks), FEVDs (a 3D matrix of dimension nx' ShockNumStr 'xh with the variance decompositions for the n observed variables), and LRVD (matrix of dimension nx' ShockNumStr ' with the long run variance decompositions). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Variance Decompositions - Initial Values'],200,500,CurrINI);
                     end;
                     %
                     % print the results to file
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-FEVDs-' AddName '-' DataType '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-InitialValues.txt']);
                     PrintDSGEVARVarianceDecomposition(ResFile,FEVDs,LRVD,VarShares,[],lambdai,ShockNames,PrintGroups,VDType,'Initial Values',DSGEModel,CurrINI);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,[DataType ' Variance Decompositions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                     end;
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the variance decompositions.'];
                     About(txt,'information',[DataType ' Variance Decompositions - Initial Values'],200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'FEVDPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % are we using annualized data, levels, or the original
         %
         if gcbo==controls.dsgevartools.decomposition.fevd.original.prior;
            DataType = 'Original';
            VDType = 1;
         elseif gcbo==controls.dsgevartools.decomposition.fevd.levels.prior;
            DataType = 'Levels';
            VDType = 2;
         end;
         %
         % determine which model to conider
         %
         [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws([DataType ' Variance Decompositions - Prior'],DSGEModel,CurrINI);
         if isempty(lambda)==0;
            %
            % load the prior draws data
            %
            [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
            UsedDraws = size(PhiDraws,1);
            if isempty(A0Draws)==0;
               clear('SigmaDraws');
               %
               ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
               %
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  SampleFreq = 4;
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  SampleFreq = 12;
               else;
                  SampleFreq = 1;
               end;
               h = SampleFreq*DSGEModel.IRHorizon;
               %
               % check if results already exist on disk
               %
               if DSGEVARVDsExist(UsedDraws,h,VDType,[],0,[],lambda,DSGEModel,controls)==1;
                  txt = 'YADA has located results for the variance decompositions on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,[DataType ' Variance Decompositions - Prior'],500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the variance decompositions
                  %
                  DoneCalc = DSGEVARVarDecompDistribution(PhiDraws,A0Draws,h,VDType,[],0,[],lambda,DSGEModel,CurrINI,controls);
               end;
               %
               % clear everything
               %
               clear('PhiDraws','A0Draws');
               if DoneCalc==1;
                  ShockType = 'Individual';
                  PrintGroups = 0;
                  ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                  if length(ShockGroups)>max(ShockGroups);
                     txt = 'Would you like to view results of the variance decompositions for shock groups instead of for the individual shocks?';
                     answer = Query(txt,'question',200,[DataType ' Variance Decompositions - Prior'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        ShockType = 'Group';
                        PrintGroups = 1;
                        ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                     end;
                  end;
                  %
                  % sort the results
                  %
                  VDs = DSGEVARVDsDistribution(DSGEModel,CurrINI,controls,lower(ShockType),UsedDraws,h,VDType,[],0,[],[],lambda);
                  %
                  % ask if the results should be stored to disk
                  %
                  txt = ['Would you like to save the prior distributions of the variance decompositions for the ' lower(DataType) ' data to file?'];
                  answer = Query(txt,'question',140,['Save - ' DataType ' Variance Decompositions'],500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-VarDecomp-PriorDist-' ShockType '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(UsedDraws) '.mat']);
                     YNames = DSGEModel.VariableNames;
                     save(file,'VDs','YNames','ShockNames');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARVDPriorDist' ShockType DataType strrep(num2str(lambda,'%0.4f'),'.','_') int2str(UsedDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     fprintf(fid,'%%\n%% structure with fields FEVDs containing the prior mean varaince\n%% decomposition, LRVD with the prior mean long-run variance\n%% decompositions, and the vector field Quantiles with\n%% subfields FEVDs, LRVD and percent holding quantile data\n%% and quantile percent.\n%%\n');
                     fprintf(fid,'VDs = DataStructure.VDs;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The prior distributions of the variance decompositions for the ' lower(DataType) ' data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), and VDs (structure with fields holding the mean variance decompositions, the mean long-run variance decompositions, and quantile data). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information',[DataType ' Variance Decompositions - Prior'],200,500,CurrINI);
                  end;
                  %
                  % determine string with 1's and 0's for the selected shocks
                  %
                  SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                  SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                  %
                  % print the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Prior-FEVDs-' ShockType '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
                  PrintDSGEVARVarianceDecomposition(ResFile,VDs.FEVDs,VDs.LRVD,[],VDs.Quantiles,lambda,ShockNames,PrintGroups,VDType,'Prior Distribution',DSGEModel,CurrINI);
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,[DataType ' Variance Decompositions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,1);
                  end;
               end;
            else;
               clear('PhiDraws','SigmaDraws');
               txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the prior distribution of the variance decompositions.'];
               About(txt,'information',[DataType ' Variance Decompositions - Prior'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'FEVDPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.decomposition.fevd.original.margmode)|(gcbo==controls.dsgevartools.decomposition.fevd.levels.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.decomposition.fevd.original.jointmode)|(gcbo==controls.dsgevartools.decomposition.fevd.levels.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.decomposition.fevd.original.dsgemode)|(gcbo==controls.dsgevartools.decomposition.fevd.levels.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % check which data to compute variance decompositions for
         %
         if (gcbo==controls.dsgevartools.decomposition.fevd.original.margmode)|(gcbo==controls.dsgevartools.decomposition.fevd.original.jointmode)|(gcbo==controls.dsgevartools.decomposition.fevd.original.dsgemode);
            DataType = 'Original';
            VDType = 1;
         elseif (gcbo==controls.dsgevartools.decomposition.fevd.levels.margmode)|(gcbo==controls.dsgevartools.decomposition.fevd.levels.jointmode)|(gcbo==controls.dsgevartools.decomposition.fevd.levels.dsgemode);
            DataType = 'Levels';
            VDType = 2;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Variance Decompositions - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(ModeData.DSGEModel.DSGEVARShocks);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     end;
                     %
                     if strcmp(ModeData.DSGEModel.DataFrequency,'q')==1;
                        SampleFreq = 4;
                     elseif strcmp(ModeData.DSGEModel.DataFrequency,'m')==1;
                        SampleFreq = 12;
                     else;
                        SampleFreq = 1;
                     end;
                     h = SampleFreq*DSGEModel.IRHorizon;
                     %
                     % compute the variance decompositions
                     %
                     [FEVDs,LRVD,VarShares] = DSGEVARVarianceDecompositions(Phi,A0,DSGEModel.BVARLags,h,VDType,DSGEModel.levels);
                     %
                     % check if we should consider shock groups instead of the individual shocks
                     %
                     ShockType = 'individual';
                     AddName = 'Ind';
                     PrintGroups = 0;
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the variance decompositions for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Variance Decompositions - ' ModeType],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           FEVDGs = zeros(DSGEModel.n,size(IMatrix,2),h);
                           for i=1:h;
                              FEVDGs(:,:,i) = FEVDs(:,:,i)*IMatrix;
                           end;
                           ShockType = 'group';
                           AddName = 'Group';
                           FEVDs = FEVDGs;
                           LRVD = LRVD*IMatrix;
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           PrintGroups = 1;
                        end;
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = ['Would you like to save the variance decompositions for the ' lower(DataType) ' data at ' lower(ModeType) ' to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Variance Decompositions'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-VarianceDecompositions-' AddName '-' DataType '-' num2str(lambdai,'%0.4f') '-' ModeShort '.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'FEVDs','LRVD','YNames','ShockNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARVD' AddName DataType strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',ShockType);
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        if strcmp(ShockType,'individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxnxh with the variance decompositions\n%% for the n observed variables, the n structural shocks, and the h forecast horizons\n%%\n');
                           ShockNumStr = 'n';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxgxh with the variance decompositions\n%% for the n observed variables, the g shock groups, and the h forecast horizons\n%%\n');
                           ShockNumStr = 'g';
                        end;
                        fprintf(fid,'FEVDs = DataStructure.FEVDs;\n');
                        fprintf(fid,'%%\n%% long run variance decomposition\n%%\n');
                        fprintf(fid,'LRVD = DataStructure.LRVD;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The variance decompositions for the ' lower(DataType) ' data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' ShockType ' DSGE-VAR shocks), FEVDs (a 3D matrix of dimension nx' ShockNumStr 'xh with the variance decompositions for the n observed variables), and LRVD (matrix of dimension nx' ShockNumStr ' with the long run variance decompositions). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Variance Decompositions - ' ModeType],200,500,CurrINI);
                     end;
                     %
                     % print the results to file
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-FEVDs-' AddName '-' DataType '-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' ModeShort '.txt']);
                     PrintDSGEVARVarianceDecomposition(ResFile,FEVDs,LRVD,VarShares,[],lambdai,ShockNames,PrintGroups,VDType,ModeType,DSGEModel,CurrINI);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,[DataType ' Variance Decompositions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],[ModeType ' Parameter Values'],50,CurrINI,1,1);
                     end;
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the variance decompositions.'];
                     About(txt,'information',[DataType ' Variance Decompositions - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'FEVDPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevartools.decomposition.fevd.original.posterior;
            DataType = 'Original';
            VDType = 1;
         elseif gcbo==controls.dsgevartools.decomposition.fevd.levels.posterior;
            DataType = 'Levels';
            VDType = 2;
         end;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataType ' Variance Decompositions - Posterior'],DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if isempty(A0Draws)==0;
                  %
                  % we don't need the posterior draws of SigmaEpsilon
                  %
                  clear('SigmaDraws');
                  %
                  ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                  %
                  if strcmp(DSGEModel.DataFrequency,'q')==1;
                     SampleFreq = 4;
                  elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                     SampleFreq = 12;
                  else;
                     SampleFreq = 1;
                  end;
                  h = SampleFreq*DSGEModel.IRHorizon;
                  %
                  % check if results already exist on disk
                  %
                  if DSGEVARVDsExist(UsedDraws,h,VDType,CurrChain,1,PME,lambda,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the variance decompositions on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,[DataType ' Variance Decompositions - Posterior'],500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the variance decompositions
                     %
                     DoneCalc = DSGEVARVarDecompDistribution(PhiDraws,A0Draws,h,VDType,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                  end;
                  %
                  % clear everything
                  %
                  clear('PhiDraws','A0Draws');
                  if DoneCalc==1;
                     ShockType = 'Individual';
                     PrintGroups = 0;
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view results of the variance decompositions for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',200,[DataType ' Variance Decompositions - Posterior'],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           ShockType = 'Group';
                           PrintGroups = 1;
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                        end;
                     end;
                     %
                     % sort the results
                     %
                     VDs = DSGEVARVDsDistribution(DSGEModel,CurrINI,controls,lower(ShockType),UsedDraws,h,VDType,CurrChain,1,Weights,PME,lambda);
                     %
                     % ask if the results should be stored to disk
                     %
                     txt = ['Would you like to save the posterior distributions of the variance decompositions for the ' lower(DataType) ' data to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Variance Decompositions'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        DrawsStr = get(controls.posterior.draws,'String');
                        NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-VarDecomp-PosteriorDist-' ShockType '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws)  '.mat']);
                        YNames = DSGEModel.VariableNames;
                        save(file,'VDs','YNames','ShockNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARVDPostDist' ShockType DataType strrep(num2str(lambda,'%0.4f'),'.','_') int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the %s DSGE-VAR shocks\n%%\n',lower(ShockType));
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% structure with fields FEVDs containing the posterior mean\n%% variance decomposition, LRVD with the posterior mean long-run\n%% variance decompositions, and the vector field Quantiles\n%% with subfields FEVDs, LRVD and percent holding quantile\n%% data and quantile percent.\n%%\n');
                        fprintf(fid,'VDs = DataStructure.VDs;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The posterior distributions of the variance decompositions for the ' lower(DataType) ' data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), ShockNames (string matrix with the names of the ' lower(ShockType) ' DSGE-VAR shocks), and VDs (structure with fields holding the mean variance decompositions, the mean long-run variance decompositions, and quantile data). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Variance Decompositions - Posterior'],200,500,CurrINI);
                     end;
                     %
                     % determine string with 1's and 0's for the selected shocks
                     %
                     SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                     SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                     %
                     % print the results to file
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-Posterior-FEVDs-' ShockType '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.txt']);
                     PrintDSGEVARVarianceDecomposition(ResFile,VDs.FEVDs,VDs.LRVD,[],VDs.Quantiles,lambda,ShockNames,PrintGroups,VDType,'Posterior Distribution',DSGEModel,CurrINI);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,[DataType ' Variance Decompositions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior distribution of the variance decompositions.'];
                  About(txt,'information',[DataType ' Variance Decompositions - Posterior'],200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior distribution of the variance decompositions. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information',[DataType ' Variance Decompositions - Posterior'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRFunctionInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  try;
                     ModeData = load(ModeFile);
                     InitStatus = 1;
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                     ModeData = [];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            if gcbo==controls.dsgevartools.irfunction.original.initialvalues;
               DataType = 'Original';
               IRType = 1;
            elseif gcbo==controls.dsgevartools.irfunction.annual.initialvalues;
               DataType = 'Annualized';
               IRType = 2;
            elseif gcbo==controls.dsgevartools.irfunction.levels.initialvalues;
               DataType = 'Levels';
               IRType = 3;
            end;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Impulse Responses - Initial Values'],'','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                  if IdentifyStatus==1;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
                     for j=1:DSGEModel.n;
                        ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                     end;
                     %
                     if strcmp(DSGEModel.DataFrequency,'q')==1;
                        SampleFreq = 4;
                     elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                        SampleFreq = 12;
                     else;
                        SampleFreq = 1;
                     end;
                     h = SampleFreq*DSGEModel.IRHorizon;
                     %
                     % compute impulse responses
                     %
                     IRF = DSGEVARImpulseResponses(Phi,A0,DSGEModel.BVARLags,h,IRType,DSGEModel.levels,DSGEModel.annual,DSGEModel.annualscale);
                     %
                     % check if we should consider shock groups instead of the individual shocks
                     %
                     ShockType = 'individual';
                     AddName = 'Ind';
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Impulse Responses - Initial Values'],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           IRG = zeros(DSGEModel.n,size(IMatrix,2),h+1);
                           for i=1:h+1;
                              IRG(:,:,i) = IRF(:,:,i)*IMatrix;
                           end;
                           ShockType = 'group';
                           AddName = 'Group';
                           IRF = IRG;
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                        end;
                     end;
                     %
                     % save to disk
                     %
                     txt = ['Would you like to save the ' lower(DataType) ' impulse response functions at initial values to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Impulse Responses'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        YNames = DSGEModel.VariableNames;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-' DataType 'ImpulseResponses-' AddName '-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                        save(file,'YNames','ShockNames','IRF');
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' DataType 'IR' AddName strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        if strcmp(ShockType,'individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxnxh with the responses for the n observed variables\n%%\n');
                           ShockNumStr = 'n';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxgxh with the responses for the n observed variables\n%%\n');
                           ShockNumStr = 'g';
                        end;
                        fprintf(fid,'IRF = DataStructure.IRF;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the endogenous variables), ShockNames (string matrix with the names of the ' ShockType ' DSGE-VAR shocks), and IRF (a 3D matrix of dimension nxnxh with the responses for the n observed variables to the ' ShockNumStr ' shocks from period 0 until period h-1). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Impulse Responses - Initial Values'],300,500,CurrINI);
                     end;
                     %
                     % display the impulse responses
                     %
                     IRDSGEVARGraphDLG('init',CurrINI,DSGEModel,IRF,ShockNames,lambdai,'Initial Values',DataType,ShockType);
                  else;
                     txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the impulse responses.'];
                     About(txt,'information',[DataType ' Impulse Responses - Initial Values'],200,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'IRFunctionPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % are we using annualized data, levels, or the original
         %
         if gcbo==controls.dsgevartools.irfunction.original.prior;
            DataType = 'Original';
            IRType = 1;
         elseif gcbo==controls.dsgevartools.irfunction.annual.prior;
            DataType = 'Annualized';
            IRType = 2;
         elseif gcbo==controls.dsgevartools.irfunction.levels.prior;
            DataType = 'Levels';
            IRType = 3;
         end;
         %
         % determine which model to conider
         %
         [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws([DataType ' Impulse Responses - Prior'],DSGEModel,CurrINI);
         if isempty(lambda)==0;
            %
            % load the prior draws data
            %
            [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
            UsedDraws = size(PhiDraws,1);
            if isempty(A0Draws)==0;
               clear('SigmaDraws');
               %
               ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
               %
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  SampleFreq = 4;
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  SampleFreq = 12;
               else;
                  SampleFreq = 1;
               end;
               h = SampleFreq*DSGEModel.IRHorizon;
               %
               % check if posterior draws already exist on disk
               %
               if DSGEVARIRFsExist(UsedDraws,h,IRType,[],0,[],lambda,DSGEModel,controls)==1;
                  txt = 'YADA has located results for the impulse responses on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,[DataType ' Impulse Responses - Prior'],500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the impulse responses
                  %
                  DoneCalc = DSGEVARImpulseRespDistribution(PhiDraws,A0Draws,h,IRType,[],0,[],lambda,DSGEModel,CurrINI,controls);
               end;
               %
               % clear everything
               %
               clear('PhiDraws','A0Draws');
               if DoneCalc==1;
                  ShockType = 'Individual';
                  ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                  if length(ShockGroups)>max(ShockGroups);
                     txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                     answer = Query(txt,'question',200,[DataType ' Impulse Responses - Prior'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        ShockType = 'Group';
                     end;
                  end;
                  %
                  % sort the results
                  %
                  IRFs = DSGEVARIRFsDistribution(DSGEModel,CurrINI,controls,lower(ShockType),UsedDraws,h,IRType,[],0,[],[],lambda);
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the impulse response functions to file?';
                  answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     YNames = DSGEModel.VariableNames;
                     file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-' DataType '-ImpulseResponsePriorDist-' num2str(lambda,'%0.4f') '-' ShockType '-' int2str(UsedDraws) '.mat']);
                     if strcmp(lower(ShockType),'individual')==1;
                        save(file,'YNames','ShockNames','IRFs');
                     else;
                        ShockGroupNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                        save(file,'YNames','ShockGroupNames','IRFs');
                     end;
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' DataType 'IRPriorDist' strrep(num2str(lambda,'%0.4f'),'.','_') ShockType int2str(UsedDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     if strcmp(lower(ShockType),'individual')==1;
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        ShockStr = 'ShockNames (string matrix with the names of the economic shocks)';
                     else;
                        fprintf(fid,'%%\n%% string matrix with the names of the shock groups\n%%\n');
                        fprintf(fid,'ShockGroupNames = DataStructure.ShockGroupNames;\n');
                        ShockStr = 'ShockGroupNames (string matrix with the names of the shock groups)';
                     end;
                     fprintf(fid,'%%\n%% structure with fields Y containing the prior mean\n%% impulse responses, and vector field Quantiles\n%% with subfields Y and percent holding quantile data and quantile percent.\n%%\n');
                     fprintf(fid,'IRFs = DataStructure.IRFs;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), ' ShockStr ', and IRFs (a structure with prior mean and quantile data for the impulse responses). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information',[DataType ' Impulse Responses - Prior'],200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  IRDistributionGraphDLG('init',DSGEModel,CurrINI,IRFs,DataType,lower(ShockType),0,0,lambda);
               end;
            else;
               clear('PhiDraws','SigmaDraws');
               txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the prior distribution of the impulse responses.'];
               About(txt,'information',[DataType ' Impulse Responses - Prior'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRFunctionPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.irfunction.original.margmode)|(gcbo==controls.dsgevartools.irfunction.annual.margmode)|(gcbo==controls.dsgevartools.irfunction.levels.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.irfunction.original.jointmode)|(gcbo==controls.dsgevartools.irfunction.annual.jointmode)|(gcbo==controls.dsgevartools.irfunction.levels.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.irfunction.original.dsgemode)|(gcbo==controls.dsgevartools.irfunction.annual.dsgemode)|(gcbo==controls.dsgevartools.irfunction.levels.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % check which data to compute impulse responses for
         %
         if (gcbo==controls.dsgevartools.irfunction.original.margmode)|(gcbo==controls.dsgevartools.irfunction.original.jointmode)|(gcbo==controls.dsgevartools.irfunction.original.dsgemode);
            DataType = 'Original';
            IRType = 1;
         elseif (gcbo==controls.dsgevartools.irfunction.annual.margmode)|(gcbo==controls.dsgevartools.irfunction.annual.jointmode)|(gcbo==controls.dsgevartools.irfunction.annual.dsgemode);
            DataType = 'Annualized';
            IRType = 2;
         elseif (gcbo==controls.dsgevartools.irfunction.levels.margmode)|(gcbo==controls.dsgevartools.irfunction.levels.jointmode)|(gcbo==controls.dsgevartools.irfunction.levels.dsgemode);
            DataType = 'Levels';
            IRType = 3;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Impulse Responses - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==0;
                  [ModeData.PhiMode,SigmaEpsilon,ModeData.A0Mode,ModeData.IdentifyModeStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               else;
                  ModeStatus = 1;
               end;
               if ModeStatus==1;
                  if (isfield(ModeData,'IdentifyModeStatus')==1)&(ModeData.IdentifyModeStatus==1);
                     %
                     % load data from file
                     %
                     Phi = ModeData.PhiMode;
                     if DSGEModel.k>0;
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                     end;
                     A0 = ModeData.A0Mode;
                     if isfield(ModeData,'SigmaModeEpsilon')==1;
                        ShockNames = ModeData.DSGEModel.StateShockNames(ModeData.DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(ModeData.DSGEModel.DSGEVARShocks);
                     else;
                        ShockNames = ModeData.DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                        ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     end;
                     %
                     % locate the positions in DSGEModel.StateShockNames that correspond to ShockNames
                     %
                     Positions = zeros(DSGEModel.n,1);
                     for j=1:DSGEModel.n;
                        Positions(j) = LocateStr(DSGEModel.StateShockNames,StringTrim(ShockNames(j,:)));
                     end;
                     Positions = Positions(Positions>0);
                     if length(Positions)==DSGEModel.n;
                        ShockNames = DSGEModel.ShockAliases(Positions,:);
                        for j=1:DSGEModel.n;
                           ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
                        end;
                     end;
                     %
                     if strcmp(ModeData.DSGEModel.DataFrequency,'q')==1;
                        SampleFreq = 4;
                     elseif strcmp(ModeData.DSGEModel.DataFrequency,'m')==1;
                        SampleFreq = 12;
                     else;
                        SampleFreq = 1;
                     end;
                     h = SampleFreq*DSGEModel.IRHorizon;
                     %
                     % compute impulse responses
                     %
                     IRF = DSGEVARImpulseResponses(Phi,A0,ModeData.DSGEModel.BVARLags,h,IRType,DSGEModel.levels,DSGEModel.annual,DSGEModel.annualscale);
                     %
                     % check if we should consider shock groups instead of the individual shocks
                     %
                     ShockType = 'individual';
                     AddName = 'Ind';
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',140,[DataType ' Impulse Responses - ' ModeType],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           IMatrix = SubIdentityMatrix(ShockGroups);
                           IRG = zeros(DSGEModel.n,size(IMatrix,2),h+1);
                           for i=1:h+1;
                              IRG(:,:,i) = IRF(:,:,i)*IMatrix;
                           end;
                           ShockType = 'group';
                           AddName = 'Group';
                           IRF = IRG;
                           ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                        end;
                     end;
                     %
                     % save to disk
                     %
                     txt = ['Would you like to save the ' lower(DataType) ' impulse response functions at ' lower(ModeType) ' to file?'];
                     answer = Query(txt,'question',140,['Save - ' DataType ' Impulse Responses'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        YNames = ModeData.DSGEModel.VariableNames;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-' DataType 'ImpulseResponses-' AddName '-' num2str(lambdai,'%0.4f') '-' strrep(ModeType,' ','') '.mat']);
                        save(file,'YNames','ShockNames','IRF');
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' DataType 'IR' AddName strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        if strcmp(ShockType,'individual')==1;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxnxh with the responses for the n observed variables\n%%\n');
                           ShockNumStr = 'n';
                        else;
                           fprintf(fid,'%%\n%% a 3D matrix of dimension nxgxh with the responses for the n observed variables\n%%\n');
                           ShockNumStr = 'g';
                        end;
                        fprintf(fid,'IRF = DataStructure.IRF;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the endogenous variables), ShockNames (string matrix with the names of the ' ShockType ' DSGE-VAR shocks), and IRF (a 3D matrix of dimension nxnxh with the responses for the n observed variables to the ' ShockNumStr ' shocks from period 0 until period h-1). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Impulse Responses - ' ModeType],300,500,CurrINI);
                     end;
                     %
                     % display the impulse responses
                     %
                     IRDSGEVARGraphDLG('init',CurrINI,DSGEModel,IRF,ShockNames,lambdai,ModeType,DataType,ShockType);
                  else;
                     txt = ['YADA is unable to identify the shocks of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA can therefore not compute the impulse responses.'];
                     About(txt,'information',[DataType ' Impulse Responses - ' ModeType],200,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRFunctionPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevartools.irfunction.original.posterior;
            DataType = 'Original';
            IRType = 1;
         elseif gcbo==controls.dsgevartools.irfunction.annual.posterior;
            DataType = 'Annualized';
            IRType = 2;
         elseif gcbo==controls.dsgevartools.irfunction.levels.posterior;
            DataType = 'Levels';
            IRType = 3;
         end;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataType ' Impulse Responses - Posterior'],DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            clear('thetaPostSample');
            UsedDraws = size(PhiDraws,1);
            if UsedDraws>=100;
               if isempty(A0Draws)==0;
                  %
                  % we don't need the posterior draws of SigmaEpsilon
                  %
                  clear('SigmaDraws');
                  %
                  ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
                  %
                  if strcmp(DSGEModel.DataFrequency,'q')==1;
                     SampleFreq = 4;
                  elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                     SampleFreq = 12;
                  else;
                     SampleFreq = 1;
                  end;
                  h = SampleFreq*DSGEModel.IRHorizon;
                  %
                  % check if posterior draws already exist on disk
                  %
                  if DSGEVARIRFsExist(UsedDraws,h,IRType,CurrChain,1,PME,lambda,DSGEModel,controls)==1;
                     txt = 'YADA has located results for the impulse responses on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,[DataType ' Impulse Responses - Posterior'],500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the impulse responses
                     %
                     DoneCalc = DSGEVARImpulseRespDistribution(PhiDraws,A0Draws,h,IRType,CurrChain,1,PME,lambda,DSGEModel,CurrINI,controls);
                  end;
                  %
                  % clear everything
                  %
                  clear('PhiDraws','A0Draws');
                  if DoneCalc==1;
                     ShockType = 'Individual';
                     ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
                     if length(ShockGroups)>max(ShockGroups);
                        txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',200,[DataType ' Impulse Responses - Posterior'],500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % sort the results
                     %
                     IRFs = DSGEVARIRFsDistribution(DSGEModel,CurrINI,controls,lower(ShockType),UsedDraws,h,IRType,CurrChain,1,Weights,PME,lambda);
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the impulse response functions to file?';
                     answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        DrawsStr = get(controls.posterior.draws,'String');
                        NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                        %
                        YNames = DSGEModel.VariableNames;
                        file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-' DataType '-ImpulseResponseDist-' num2str(lambda,'%0.4f') '-' ShockType '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
                        if strcmp(lower(ShockType),'individual')==1;
                           save(file,'YNames','ShockNames','IRFs');
                        else;
                           ShockGroupNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
                           save(file,'YNames','ShockGroupNames','IRFs');
                        end;
                        %
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' DataType 'IRDist' strrep(num2str(lambda,'%0.4f'),'.','_') ShockType int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        if strcmp(lower(ShockType),'individual')==1;
                           fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           ShockStr = 'ShockNames (string matrix with the names of the economic shocks)';
                        else;
                           fprintf(fid,'%%\n%% string matrix with the names of the shock groups\n%%\n');
                           fprintf(fid,'ShockGroupNames = DataStructure.ShockGroupNames;\n');
                           ShockStr = 'ShockGroupNames (string matrix with the names of the shock groups)';
                        end;
                        fprintf(fid,'%%\n%% structure with fields Y containing the posterior mean\n%% impulse responses, and vector field Quantiles\n%% with subfields Y and percent holding quantile data and quantile percent.\n%%\n');
                        fprintf(fid,'IRFs = DataStructure.IRFs;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), ' ShockStr ', and IRFs (a structure with posterior mean and quantile data for the impulse responses). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',[DataType ' Impulse Responses - Posterior'],200,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     IRDistributionGraphDLG('init',DSGEModel,CurrINI,IRFs,DataType,lower(ShockType),1,0,lambda);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws');
                  txt = ['YADA is unable to identify the structural shocks of the DSGE-VAR model with lambda=' num2str(lambda) '. YADA can therefore not compute the posterior distribution of the impulse responses.'];
                  About(txt,'information',[DataType ' Impulse Responses - Posterior'],200,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute posterior distribution of the impulse reponses. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information',[DataType ' Impulse Responses - Posterior'],200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetDSGEVARShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         if DSGEModel.n<length(DSGEModel.StateShockPositions);
            %
            % need to make a selection
            %
            if length(DSGEModel.DSGEVARShocks)>DSGEModel.n;
               DSGEModel.DSGEVARShocks = (1:DSGEModel.n);
            end;
            %
            % display a selection dialog
            %
            [action,NewDSGEVARShocks] = SelectCondVarShockDLG('init','Shocks','DSGE-VAR Model',DSGEModel.StateShockNames,DSGEModel.DSGEVARShocks,CurrINI);
            if strcmp(lower(action),'ok')==1;
               if length(NewDSGEVARShocks)==DSGEModel.n;
                  %
                  % check if the B0 matrix has full column rank
                  %
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
                     ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
                     if FileExist(ModeFile)==1;
                        ErrorStr = '';
                        try;
                           ModeData = load(ModeFile);
                        catch;
                           ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                        end;
                        if isempty(ErrorStr)==1;
                           theta = ModeData.theta;
                           thetaPositions = ModeData.thetaPositions;
                           ModelParameters = ModeData.ModelParameters;
                           ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                           InitStatus = 1;
                        else;
                           About(ErrorStr,'information','YADA - Bad MAT File',300,500,CurrINI);
                        end;
                     end;
                  end;
                  if InitStatus==1;
                     %
                     % load the AIMData file
                     %
                     AIMData = load(DSGEModel.AIMDataFile);
                     %
                     % check if we can solve the model
                     %
                     ErrorStr = '';
                     try;
                        [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                        if isempty(ErrMsg)==0;
                           ErrorStr = ['YADA caught an error when trying to solve the DSGE model. It is therefore not possible to test if the shocks you selected can be used for identification purpose of the DSGE-VAR model. Message caught is: ' ErrMsg];
                        end;
                     catch;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. It is therefore not possible to test if the shocks you selected can be used for identification purpose of the DSGE-VAR model. Message caught is: ' lasterr];
                     end;
                     if isempty(ErrorStr)==1;
                        %
                        % check the rank of A0 at initial values under the selected shocks
                        %
                        if length(size(H))==2;
                           B = H'*B0(:,NewDSGEVARShocks);
                        else;
                           B = ((1/DSGEModel.T)*sum(H,3))'*B0(:,NewDSGEVARShocks);
                        end;
                        if rank(B)==length(NewDSGEVARShocks);
                           DSGEModel.DSGEVARShocks = NewDSGEVARShocks;
                        else;
                           txt = ['The shocks you selected for the DSGE-VAR result in a singular contemporaneous response matrix of the DSGE model at the initial parameter values when using only these shocks. The rank of the matrix is ' int2str(rank(B)) ', while its dimension is ' int2str(size(B,1)) 'x' int2str(size(B,2)) '. The contemporaneous response matrix must be nonsingular for the shocks of the DSGE-VAR to be identified from the DSGE model. Please consider a different selection.'];
                           About(txt,'information','YADA - Bad DSGE-VAR Shock Selection',300,500,CurrINI);
                        end;
                     else;
                        About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
                     end;
                  end;
               else;
                  txt = ['The number of shocks you selected (' int2str(length(NewDSGEVARShocks)) ') doesn''t match the number of endogenous variables (' int2str(DSGEModel.n) '). The selection is therefore not valid.'];
                  About(txt,'information','YADA - Bad DSGE-VAR Shock Selection',200,500,CurrINI);
               end;
            end;
         elseif DSGEModel.n==length(DSGEModel.StateShockPositions);
            txt = strvcat(['The number of shocks in the DSGE model (' int2str(length(DSGEModel.StateShockPositions)) ') is equal to the number of endogenous variables (' int2str(DSGEModel.n) '). The shocks are given by:'],' ',StringTrim(vecstr(DSGEModel.StateShockNames,' ')),' ','There is therefore no need to select a subset of the DSGE model shocks to match the number of shocks in the DSGE-VARs.');
            About(txt,'information','Set DSGE-VAR Shocks',300,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'DSGEVARLagOrder'
      set(maingui,'CurrentObject',controls.tab5);
      YADAGUI('tab');
      %
      txt = ['You can change the lag order of DSGE-VARs through the "Lag order of VAR and DSGE-VAR models" popup control in the "Lag Order Selection" frame on the "Bayesian VAR" tab. To serve you well, YADA has already switched to this tab and the current lag order is: ' int2str(DSGEModel.BVARLags) '.'];
      About(txt,'information','DSGE-VAR - Lag Order Selection',300,500,CurrINI);
      if MatlabNumber>=7;
         uicontrol(controls.bvar.lag);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of DSGEVARMenuFunctions.m
%
