function ViewMenuFunctions(selector)
% ViewMenuFunctions: switch function for running the function on the view menu.
%
% USAGE:
%
%       ViewMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 14, 2010.
%                        This version: September 15, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-02-2010: Updated the documentation.
%
% * 16-02-2010: Changed the order of the input variables for "YADAEnableControls".
%
% * 02-03-2010: Updated the documentation.
%
% * 07-03-2010: Changed cases of the cases.
%
% * 21-04-2010: Added the case of displaying the structural form.
%
% * 22-04-2010: Made sure that the structural form cases requires the number of leads
%               to be at most 1.
%
% * 07-05-2010: Began changing the structural form code such that the number of leads
%               can exceed 1.
%
% * 09-07-2010: Added the "propeties" cases for viewing the controllability and
%               observability results.
%
% * 29-11-2010: The "View Transformed Data" function can now optionally save the transformed
%               data to disk.
%
% * 01-12-2010: Moved the "SaveAnnualData" functionality to "ViewAnnualData". This gives way
%               for the "ViewLaplacePosterior" function.
%
% * 15-12-2010: Updated the documentation.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 07-02-2011: The function only allows plotting of the densities of update parameters when the
%               field "additional" of the structure "ParameterNames" is not empty.
%
% * 08-02-2011: Took into account that "GetUpdatePriorDensity" now provides the "UpdateParameters"
%               output variable.
%
% * 10-02-2011: The function now checks to make sure that "UpdateParameters" is valid for the current set of
%               parameters to update, i.e., that the underlying mat-file is not too old.
%
% * 17-06-2011: Removed unused output variables from calls to "InitializeDSGEModelSimulation".
%
% * 20-06-2011: Added the optional input argument "CheckModelSolution" when determining the structural form
%               at initial values. This has been added to allow the structural form to be displayed also when
%               the model doesn't have a unique convergent solution.
%
% * 21-06-2011: Made sure that the structural form can be written to file and displayed also when the
%               model doesn't have a unique convergent solution. Added functions for displaying all parameter
%               values in the "ModelParameters" structure at initial values or posterior mode values.
%
% * 02-01-2012: Updated the documentation.
%
% * 21-02-2012: Added code for recomputing the log posterior when computing the marginal likelihood
%               using the modefied harmonic mean estimator.
%
% * 22-02-2012: Did the same for the Chib and Jeliazkov estimator.
%
% * 28-02-2012: Fixed some minor issues regarding the number of burn-in draws and posterior draws when
%               writing marginal likelihood estimation results for the Chib and Jelizkov case to file.
%
% * 02-03-2012: Added the functions for displaying the prior and posterior distributions (mean and
%               and standard deviation) of the state-space form.
%
% * 06-03-2012: Added function for showing the Laplace approximation of the marginal likelihood at
%               the posterior mode values.
%
% * 07-03-2012: Added function for showing the Laplace approximation of the marginal likelihood at
%               the initial values.
%
% * 22-03-2012: The function now tries to load "phiInverseHessianFinDiff" when computing the marginal
%               likelihood at the posterior mode with the Laplace approximation. The modified harmonic
%               mean estimate for the full sample can now also be based on a subset of the posterior
%               draws.
%
% * 28-03-2012: Took into account that "MargLikeLaplace" now accepts a third input variable.
%
% * 02-05-2012: Added average values for the modified harmonic mean.
%
% * 07-05-2012: When plotting raw posterior draws the user can now select to plot a sub-sample.
%
% * 15-05-2012: Changed "Average" to "Mean" and added "Median" estimate of the modified harmonic mean.
%
% * 05-06-2012: Took into account that "DSGEModel.StateEquationPositions" determines the rows from the AiM
%               output of the state equations.
%
% * 16-11-2012: Took into account that "CalculateDSGESquareRootFilter" now provides three required and two
%               optional output variables.
%
% * 30-11-2012: Added function case for computing the marginal likelihood with the truncated elliptical
%               distribution in the modified harmonic mean estimator, as in Sims, Waggoner and Zha.
%
% * 03-12-2012: Continued work on the SWZ estimator of the marginal likelihood.
%
% * 04-12-2012: Continued work on the SWZ estimator of the marginal likelihood.
%
% * 06-12-2012: Continued the SWZ estimator of the marginal likelihood. Took into account that "MargLikeModifiedHarmonic"
%               now supports the output variable "NWStdErr".
%
% * 07-12-2012: Finished the SWZ estimator of the marginal likelihood.
%
% * 10-12-2012: Fixed a minor beauty coding spot. Took into account that "MargLikeSWZModifiedHarmonic" now requires the
%               lambda input variable.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 27-03-2013: Added possibility to save observed variables to disk when viewing them.
%
% * 28-01-2014: Updated the documentation.
%
% * 20-02-2014: Added code for plotting the log posterior kernel values for the raw draws.
%
% * 21-02-2014: Streamlined the code for plotting the log posterior kernel values.
%
% * 24-02-2014: Fixed a dialog name when having multiple chains and wishing to display scatter-plots
%               of the posterior draws.
%
% * 25-02-2014: Took into account that the "AcceptanceRatioPlot" function now accepts the "lambda"
%               input variable to allow it to properly plot DSGE-VAR model results in addition to
%               the DSGE model results. The function "PosteriorMeanMedianPlots" also accepts the
%               input variable "lambda" and "PME" for the same reason. The function "CUSUM" now
%               accepts the "lambda" input variable.
%
% * 26-02-2014: Made sure that the variable "RunTest" is initialized when computing the separated
%               partial means test for the log posterior kernel values as well as for the original
%               and transformed parameters.
%
% * 27-02-2014: Took into account that the "MultiANOVA" function accepts the "lambda" and "PME"
%               input variables.
%
% * 07-03-2014: Fixed a bug when attempting to plot the Laplace approximation of the posterior densities
%               and the finite difference inverse Hessian is selected.
%
% * 15-08-2014: Added information about the zero lower bound to the data construction file information
%               function.
%
% * 14-01-2015: Updated the documentation.
%
% * 15-06-2015: Added information about actuals data when showing the data construction information.
%
% * 27-08-2015: The function now supports the use of a Student-t proposal density for the RWM posterior sampler.
%
% * 02-09-2015: Fixed the information displayed when the kalmanstatus is -1.
%
% * 26-11-2015: Began updating the "Iterated Parameter Estimates" function is updated to also support
%               showing the original parameters at a particular step in the iterations.
%
% * 27-11-2015: Finished the addition to the "Iterated Parameter Estimates" function.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 31-05-2017: The View prior and log Jacobian value function now supports the system prior.
%
% * 01-06-2017: The print prior information and graph prior densities function now support the system prior.
%
% * 02-06-2017: The viewing of the Laplace approximation at initial and posterior mode values now support
%               the system prior.
%
% * 08-06-2017: The viewing of the posterior densities now support the system prior.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 14-08-2017: The function now supports the SMC likelihood tempering posterior sampler.
%
% * 23-08-2017: Took into account that LoadDSGEPosteriorDraws now provides the output variable "Weights".
%
% * 24-08-2017: Took into account that "PosteriorMeanMedianPlots" takes the "Weights" input variable.
%
% * 25-08-2017: Took into account that "DSGEStateSpaceMoments", "CUSUM", and "SeparatedPartialMeansTest"
%               require the "Weights" input variable.
%
% * 28-08-2017: The "RawPlots" case now allows for plotting the importance weights for the SMC with likelihood
%               tempering posterior sampler.
%
% * 29-08-2017: Took into account that "logPosteriorPhiDSGE" requires the input variable "NumWorkers".
%
% * 04-09-2017: Speeded up the Weights computations.
%
% * 11-09-2017: Took into account that "RawWeightsPosteriorPlots" takes the lambda input variable.
%
% * 15-09-2017: Added SMC adaption function.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
lasterr = '';
%
switch selector
   case 'ViewGraphics'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Open a Graphics file dialog and the load the graphics file.
         %
         GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
         if DirExist(GraphDirectory)==0;
            GraphDirectory = pwd;
         end;
         OpenGraphics(GraphDirectory,CurrINI);
         set(controls.about,'UserData',0);
      end;
   case 'DataInfo'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % setup the txt string matrix with all information taken from the data constructions file
         %
         txt = strvcat(' ','DATA CONSTRUCTION INFORMATION','=============================',' ');
         txt = strvcat(txt,['FILE:                          ' GetFilename(DSGEModel.DataConstructionFile)],' ');
         txt = strvcat(txt,['FULL SAMPLE:                   ' DSGEModel.BeginYear ':' DSGEModel.BeginPeriod ' - ' DSGEModel.EndYear ':' DSGEModel.EndPeriod]);
         txt = strvcat(txt,['DATA FREQUENCY:                ' DSGEModel.DataFrequency]);
         txt = strvcat(txt,['NUMBER OF OBSERVATIONS:        ' int2str(DSGEModel.T)]);
         txt = strvcat(txt,['NUMBER OF OBSERVED VARIABLES:  ' int2str(DSGEModel.n)]);
         txt = strvcat(txt,['NUMBER OF EXOGENOUS VARIABLES: ' int2str(DSGEModel.k)],' ');
         txt = strvcat(txt,['OBSERVED VARIABLES:            ' vecstr(DSGEModel.VariableNames,' ')]);
         if isempty(DSGEModel.Actuals)==0;
            txt = strvcat(txt,' ','ALTERNATIVE ACTUALS DATASETS OF THE OBSERVED VARIABLES','======================================================',' ');
            m = length(DSGEModel.Actuals);
            for i=1:m;
               txt = strvcat(txt,['   ' DSGEModel.Actuals(i).title ' (' int2str(DSGEModel.n) 'x' int2str(size(DSGEModel.Actuals(i).data,2)) ')']);
            end;
            txt = strvcat(txt,' ','These data can be used when computing the predictive likelihood.',' ');
         end;
         if isempty(DSGEModel.YTransformation)==0;
            txt = strvcat(txt,' ','TRANSFORMATIONS OF OBSERVED VARIABLES','-------------------------------------',' ');
            AddTxt = '';
            for i=1:size(DSGEModel.VariableNames,1);
               CurrField = getfield(DSGEModel.YTransformation,StringTrim(DSGEModel.VariableNames(i,:)));
               if (isfield(CurrField,'fcn')==1)&(isfield(CurrField,'partial')==1)&(isfield(CurrField,'annualizefcn')==1)&(isfield(CurrField,'annualizepartial')==1)&(isfield(CurrField,'initial')==1)&(isfield(CurrField,'x')==1);
                  AddTxt = strvcat(AddTxt,[' - ' DSGEModel.VariableNames(i,:) ' general function:   ' CurrField.fcn],['   ' SpaceStr(size(DSGEModel.VariableNames,2)) ' - partial function: ' CurrField.partial],['   ' SpaceStr(size(DSGEModel.VariableNames,2)) ' annualize function: ' CurrField.annualizefcn],['   ' SpaceStr(size(DSGEModel.VariableNames,2)) ' - partial function: ' CurrField.annualizepartial]);
               end;
               if (isfield(CurrField,'invertfcn')==1)&(isfield(CurrField,'invertinitial')==1)&(isfield(CurrField,'invertx')==1);
                  AddTxt = strvcat(AddTxt,['   ' SpaceStr(size(DSGEModel.VariableNames,2)) ' inversion function: ' CurrField.invertfcn]);
               end;
               if (isfield(CurrField,'exportfcn')==1)&(isfield(CurrField,'exportinitial')==1)&(isfield(CurrField,'exportx')==1);
                  AddTxt = strvcat(AddTxt,['   ' SpaceStr(size(DSGEModel.VariableNames,2)) ' export function:    ' CurrField.exportfcn]);
               end;
            end;
            if isempty(AddTxt)==0;
               txt = strvcat(txt,AddTxt,' ');
            else;
               txt = strvcat(txt,'                               No information is available.',' ');
            end;
         end;
         txt = strvcat(txt,['LEVELS VARIABLES:              ' num2str(DSGEModel.levels)],'                               0 means first difference','                               1 means levels',' ');
         txt = strvcat(txt,['ANNUALIZATION:                 ' num2str(DSGEModel.annual)],'                               1 means that variable is in annual changes or levels','                               4 means that variable is in quarterly changes','                               12 means that variable is in monthly changes',['SCALE FACTORS:                 ' num2str(DSGEModel.annualscale)],' ');
         if DSGEModel.k>0;
            xvars = vecstr(DSGEModel.XVariableNames,' ');
         else;
            xvars = 'None';
         end;
         txt = strvcat(txt,['EXOGENOUS VARIABLES:           ' xvars]);
         %
         % DSGE-VAR information
         %
         txt = strvcat(txt,' ','DSGE-VAR SETTINGS','-----------------',' ',['Feasible lambda values: ' num2str(GetLambda(DSGEModel),'%9.4f')]);
         %
         % check if there are conditional variables
         %
         if isempty(DSGEModel.ZVariableNames)==0;
            zvars = vecstr(DSGEModel.ZVariableNames,' ');
         else;
            zvars = 'None';
         end;
         txt = strvcat(txt,' ',['CONDITIONAL VARIABLES:         ' zvars]);
         if isempty(DSGEModel.ZTransformation)==0;
            txt = strvcat(txt,' ','TRANSFORMATIONS OF CONDITIONAL VARIABLES','----------------------------------------',' ');
            AddTxt = '';
            for i=1:size(DSGEModel.ZVariableNames,1);
               CurrField = getfield(DSGEModel.ZTransformation,StringTrim(DSGEModel.ZVariableNames(i,:)));
               if (isfield(CurrField,'fcn')==1)&(isfield(CurrField,'partial')==1)&(isfield(CurrField,'annualizefcn')==1)&(isfield(CurrField,'annualizepartial')==1)&(isfield(CurrField,'initial')==1)&(isfield(CurrField,'x')==1);
                  AddTxt = strvcat(AddTxt,[' - ' DSGEModel.ZVariableNames(i,:) ' general function:   ' CurrField.fcn],['   ' SpaceStr(size(DSGEModel.ZVariableNames,2)) ' - partial function: ' CurrField.partial],['   ' SpaceStr(size(DSGEModel.ZVariableNames,2)) ' annualize function: ' CurrField.annualizefcn],['   ' SpaceStr(size(DSGEModel.ZVariableNames,2)) ' - partial function: ' CurrField.annualizepartial]);
               end;
               if (isfield(CurrField,'invertfcn')==1)&(isfield(CurrField,'invertinitial')==1)&(isfield(CurrField,'invertx')==1);
                  AddTxt = strvcat(AddTxt,['   ' SpaceStr(size(DSGEModel.ZVariableNames,2)) ' inversion function: ' CurrField.invertfcn]);
               end;
               if (isfield(CurrField,'exportfcn')==1)&(isfield(CurrField,'exportinitial')==1)&(isfield(CurrField,'exportx')==1);
                  AddTxt = strvcat(AddTxt,['   ' SpaceStr(size(DSGEModel.ZVariableNames,2)) ' export function:    ' CurrField.exportfcn]);
               end;
            end;
            if isempty(AddTxt)==0;
               txt = strvcat(txt,AddTxt,' ');
            else;
               txt = strvcat(txt,'                               No information is available.',' ');
            end;
         end;
         %
         % check if there are observed variable scenario variables
         %
         if isempty(DSGEModel.CVariableNames)==0;
            cvars = vecstr(DSGEModel.CVariableNames,' ');
         else;
            cvars = 'None';
         end;
         txt = strvcat(txt,' ',['SCENARIO VARIABLES:            ' cvars]);
         if isempty(DSGEModel.CTransformation)==0;
            txt = strvcat(txt,' ','TRANSFORMATIONS OF OBSERVED SCENARIO VARIABLES','----------------------------------------------',' ');
            AddTxt = '';
            for i=1:size(DSGEModel.CVariableNames,1);
               CurrField = getfield(DSGEModel.CTransformation,StringTrim(DSGEModel.CVariableNames(i,:)));
               if (isfield(CurrField,'fcn')==1)&(isfield(CurrField,'partial')==1)&(isfield(CurrField,'annualizefcn')==1)&(isfield(CurrField,'annualizepartial')==1)&(isfield(CurrField,'initial')==1)&(isfield(CurrField,'x')==1);
                  AddTxt = strvcat(AddTxt,[' - ' DSGEModel.CVariableNames(i,:) ' general function:   ' CurrField.fcn],['   ' SpaceStr(size(DSGEModel.CVariableNames,2)) ' - partial function: ' CurrField.partial],['   ' SpaceStr(size(DSGEModel.CVariableNames,2)) ' annualize function: ' CurrField.annualizefcn],['   ' SpaceStr(size(DSGEModel.CVariableNames,2)) ' - partial function: ' CurrField.annualizepartial]);
               end;
               if (isfield(CurrField,'invertfcn')==1)&(isfield(CurrField,'invertinitial')==1)&(isfield(CurrField,'invertx')==1);
                  AddTxt = strvcat(AddTxt,['   ' SpaceStr(size(DSGEModel.CVariableNames,2)) ' inversion function: ' CurrField.invertfcn]);
               end;
               if (isfield(CurrField,'exportfcn')==1)&(isfield(CurrField,'exportinitial')==1)&(isfield(CurrField,'exportx')==1);
                  AddTxt = strvcat(AddTxt,['   ' SpaceStr(size(DSGEModel.CVariableNames,2)) ' export function:    ' CurrField.exportfcn]);
               end;
            end;
            if isempty(AddTxt)==0;
               txt = strvcat(txt,AddTxt,' ');
            else;
               txt = strvcat(txt,'                               No information is available.',' ');
            end;
         end;
         %
         pStr = num2str(DSGEModel.Percentiles(1),'%0.2f');
         for i=2:length(DSGEModel.Percentiles);
            pStr = [pStr '  ' num2str(DSGEModel.Percentiles(i),'%0.2f')];
         end;
         txt = strvcat(txt,' ',['PERCENTILES:                   ' pStr]);
         %
         % zero lower bound information
         %
         if isempty(DSGEModel.ZLBPosition)==0;
            txt = strvcat(txt,' ','ZERO LOWER BOUND','----------------',' ',['Monetary policy rate:   ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:))]);
            if length(DSGEModel.ZLBData)==1;
               txt = strvcat(txt,['Zero lower bound value: ' num2str(DSGEModel.ZLBData)]);
            end;
         end;
         %
         % Collect information about a Bayesian VAR
         %
         if isempty(DSGEModel.BVARY)==0;
            txt = strvcat(txt,' ','BAYESIAN VAR VARIABLES','----------------------',' ',['ENDOGENOUS:                    ' vecstr(DSGEModel.VariableNames(DSGEModel.BVARY,:),' ')]);
            if isempty(DSGEModel.BVARX)==0;
               txt = strvcat(txt,['EXOGENOUS:                     ' vecstr(DSGEModel.XVariableNames(DSGEModel.BVARX,:),' ')],' ');
            else;
               txt = strvcat(txt,'EXOGENOUS:                     None',' ');
            end;
         end;
         %
         % display the information in a text dialog
         %
         TextGUI(txt,['Data Construction Information - ' GetName(DSGEModel.DataConstructionFile)],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Information',40,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'AiMModelInfo'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         TextGUI(DSGEModel.AIMFile,['AiM Model File - ' GetFilename(DSGEModel.AIMFile)],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'DSGE Model Specification',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'PriorInfo'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % collect the information
         %
         [InitStatus,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI);
         if InitStatus==1;
            %
            % print to file
            %
            TextFile = PrintPriorInformation(ParameterNames,theta,thetaDist,PriorDist,DSGEModel,CurrINI);
            %
            % check if file exists and display the file in dialog.
            %
            if FileExist(TextFile)==1;
               TextGUI(TextFile,['Prior Distribution - ' GetFilename(DSGEModel.PriorFile)],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Information',40,CurrINI,1,1);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ParameterValuesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % We need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames] = InitializeDSGEModelSimulation(DSGEModel,CurrINI,0);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  ParameterNames = ModeData.ParameterNames;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % write the result to a text file
            %
            ResFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-' DSGEModel.NameOfModel '-InitialValues.txt']);
            PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,'Initial values');
            %
            if FileExist(ResFile)==1;
               TextGUI(ResFile,['Parameter Values - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ParameterValuesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
            ParameterNames = ModeData.ParameterNames;
            %
            % write the result to a text file
            %
            ResFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-' DSGEModel.NameOfModel '-PosteriorMode.txt']);
            PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,'Posterior mode');
            %
            if FileExist(ResFile)==1;
               TextGUI(ResFile,['Parameter Values - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Values',50,CurrINI,1,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PropertiesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     %
                     % compute the controllability and observability matrices
                     %
                     [r,q] = size(B0);
                     n = DSGEModel.n;
                     C = zeros(r,r*q);
                     O = zeros(r,n*r);
                     Fi = eye(r);
                     for i=1:r;
                        Ind = (1+(q*(i-1)):q*i);
                        C(:,Ind) = Fi*B0;
                        O(:,Ind) = Fi'*H;
                        Fi = Fi*F;
                     end;
                     if rank(C)<r;
                        ConStr = 'not controllable';
                     else;
                        ConStr = 'controllable';
                     end;
                     if rank(O)<r;
                        ObsStr = 'not observable';
                     else;
                        ObsStr = 'observable';
                     end;
                     txt = ['The rank of the controllability matrix is ' int2str(rank(C)) ', while the rank of the observability matrix is ' int2str(rank(O)) '. This should be compared with the number of state variables, i.e., ' int2str(r) '. This means that the state-space form is ' ConStr ' and ' ObsStr '.'];
                     About(txt,'information','Properties - Initial Values',300,500,CurrINI);
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','Properties - Initial Values',200,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PropertiesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % copy files to the tmp folder
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     %
                     % compute the controllability and observability matrices
                     %
                     [r,q] = size(B0);
                     n = DSGEModel.n;
                     C = zeros(r,r*q);
                     O = zeros(r,n*r);
                     Fi = eye(r);
                     for i=1:r;
                        Ind = (1+(q*(i-1)):q*i);
                        C(:,Ind) = Fi*B0;
                        O(:,Ind) = Fi'*H;
                        Fi = Fi*F;
                     end;
                     if rank(C)<r;
                        ConStr = 'not controllable';
                     else;
                        ConStr = 'controllable';
                     end;
                     if rank(O)<r;
                        ObsStr = 'not observable';
                     else;
                        ObsStr = 'observable';
                     end;
                     txt = ['The rank of the controllability matrix is ' int2str(rank(C)) ', while the rank of the observability matrix is ' int2str(rank(O)) '. This should be compared with the number of state variables, i.e., ' int2str(r) '. This means that the state-space form is ' ConStr ' and ' ObsStr '.'];
                     About(txt,'information','Properties - Posterior Mode',300,500,CurrINI);
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','Properties - Posterior Mode',200,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StructuralFormInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI,0);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  %
                  % get the structural form matrices
                  %
                  [GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
                  [rh,ch] = size(HMatrix);
                  if ((2+AIMData.nlead)*rh)==ch;
                     [rg,cg] = size(GMatrix);
                     H = zeros(rh,ch);
                     H(1:rg,1:cg) = GMatrix;
                     H = H+HMatrix;
                     Hlag = H(:,1:rh);
                     Hexp = H(:,(2*rh)+1:(2+AIMData.nlead)*rh);
                     H = H(:,rh+1:2*rh);
                     %
                     % now we select the variables
                     %
                     StateVarPos = DSGEModel.StateVariablePositions;
                     if AIMData.nlead>1;
                        for j=2:AIMData.nlead;
                           StateVarPos = [StateVarPos DSGEModel.StateVariablePositions+(rh*(j-1))];
                        end;
                     end;
                     Hlag = Hlag(DSGEModel.StateEquationPositions,DSGEModel.StateVariablePositions);
                     Hexp = Hexp(DSGEModel.StateEquationPositions,StateVarPos);
                     D = -H(DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
                     H = H(DSGEModel.StateEquationPositions,DSGEModel.StateVariablePositions);
                     if mcode==1;
                        %
                        % checking the solution
                        %
                        S0 = H;
                        Hcheck = Hlag+(H*F);
                        r = length(DSGEModel.StateVariablePositions);
                        Fmat = F;
                        for j=1:AIMData.nlead;
                           ind = (1:r)+(r*(j-1));
                           S0 = S0+(Hexp(:,ind)*Fmat);
                           Fmat = Fmat*F;
                           Hcheck = Hcheck+(Hexp(:,ind)*Fmat);
                        end;
                        Bcheck = B0-(S0\D);
                        %
                        SolutionError = max(max(abs([Hcheck Bcheck])));
                        SolverMessage = '';
                     else;
                        SolutionError = NaN;
                        if mcode==2;
                           mcodeStr = 'Roots are not correctly computed by real_schur.'
                        elseif mcode==3;
                           mcodeStr = 'Too many big roots.';
                        elseif mcode==35;
                           mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                        elseif mcode==4;
                           mcodeStr = 'Too few big roots.';
                        elseif mcode==45;
                           mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                        elseif mcode==5;
                           mcodeStr = 'q(:,right) is singular.';
                        elseif mcode==61;
                           mcodeStr = 'Too many exact shiftrights.';
                        elseif mcode==62;
                           mcodeStr = 'Too many numeric shiftrights.';
                        elseif mcode==7;
                           mcodeStr = 'Infinite or NaN values detected.';
                        elseif mcode==8;
                           mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                        elseif mcode==-1;
                           mcodeStr = 'No stable solution.';
                        elseif mcode==-2;
                           mcodeStr = 'Too many large generalized eigenvalues.';
                        elseif mcode==-3;
                           mcodeStr = 'Too few large generalized eigenvalues.';
                        else;
                           mcodeStr = 'Return code not properly specified.';
                        end;
                        if DSGEModel.ModelSolver==1;
                           SolverStr = 'AiM';
                        elseif DSGEModel.ModelSolver==2;
                           SolverStr = 'Klein';
                        elseif DSGEModel.ModelSolver==3;
                           SolverStr = 'Gensys';
                        end;
                        SolverMessage = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     end;
                     %
                     % ask if the user would like to save the results to file
                     %
                     txt = 'Would you like to save the structural form matrices at initial values to file?';
                     answer = Query(txt,'question',200,'Structural Form - Initial Values',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        ShockNames = DSGEModel.StateShockNames;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\StructuralForm-' DSGEModel.NameOfModel '-InitialValues.mat']);
                        if mcode==1;
                           save(file,'StateVariableNames','ShockNames','Hlag','H','Hexp','D','F','B0');
                        else;
                           save(file,'StateVariableNames','ShockNames','Hlag','H','Hexp','D');
                        end;
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SF' strrep(DSGEModel.NameOfModel,'-','') 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with names of the state variables \n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% string matrix with names of the economic shocks \n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%% S T R U C T U R A L   F O R M\n');
                        fprintf(fid,'%%\n%%    Hlag*xi(t-1) + H*xi(t) + sum(j=1,%0.0f)Hexp*E_t[xi(t+j)] = D*eta(t)\n',AIMData.nlead);
                        fprintf(fid,'%%\n%% matrix with t-1 coefficients on the state variables\n%%\n');
                        fprintf(fid,'Hlag = DataStructure.Hlag;\n');
                        fprintf(fid,'%%\n%% matrix with t coefficients on the state variables\n%%\n');
                        fprintf(fid,'H = DataStructure.H;\n');
                        fprintf(fid,'%%\n%% matrix with t+1,...,t+%0.0f coefficients on expected values of the state variables\n%%\n',AIMData.nlead);
                        fprintf(fid,'Hexp = DataStructure.Hexp;\n');
                        fprintf(fid,'%%\n%% matrix with t coefficients on the economic shocks\n%%\n');
                        fprintf(fid,'D = DataStructure.D;\n');
                        fprintf(fid,'%%\n%% S O L U T I O N   (R E D U C E D   F O R M)\n');
                        fprintf(fid,'%%\n%%    xi(t) = F*xi(t-1) + B0*eta(t)\n');
                        if mcode==1;
                           fprintf(fid,'%%\n%% matrix with state transition parameters\n%%\n');
                           fprintf(fid,'F = DataStructure.F;\n');
                           fprintf(fid,'%%\n%% matrix with shock parameters\n%%\n');
                           fprintf(fid,'B0 = DataStructure.B0;\n');
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        if mcode==1;
                           txt = ['The structural form at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: StateVariableNames (string matrix with names of the state variables), ShockNames (string matrix with names of the economic shocks), Hlag (structural form matrix with coefficients on lagged state variables), H (structural form matrix with coefficients on current state variables), Hexp (structural form matrix with coefficients on expected values of future state variables), D (structural form matrix with coefficients on current economic shocks), F (reduced form matrix with parameters on lagged state variables), and B0 (reduced form matrix with parameters on the shocks). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        else;
                           txt = ['The structural form at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: StateVariableNames (string matrix with names of the state variables), ShockNames (string matrix with names of the economic shocks), Hlag (structural form matrix with coefficients on lagged state variables), H (structural form matrix with coefficients on current state variables), Hexp (structural form matrix with coefficients on expected values of future state variables), and D (structural form matrix with coefficients on current economic shocks). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        end;
                        About(txt,'information','Save Structural Form - Initial Values',500,600,CurrINI);
                     end;
                     %
                     % print the structural form to file and report the solution error
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\StructuralForm-' DSGEModel.NameOfModel '-InitialValues.txt']);
                     PrintStructuralForm(ResFile,DSGEModel,CurrINI,Hlag,H,Hexp,D,AIMData.nlead,SolutionError,SolverMessage,'Initial values');
                     %
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,['Structural Form - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                     end;
                  else;
                     txt = 'The numbers of leads (1) and lags (1) don''t match with the dimension of the structural form matrix supplied through AiM.';
                     About(txt,'information','YADA - Error Message',150,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StructuralFormPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % copy files to the tmp folder
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     %
                     % get the structural form matrices
                     %
                     [GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
                     [rh,ch] = size(HMatrix);
                     if ((2+AIMData.nlead)*rh)==ch;
                        [rg,cg] = size(GMatrix);
                        H = zeros(rh,ch);
                        H(1:rg,1:cg) = GMatrix;
                        H = H+HMatrix;
                        Hlag = H(:,1:rh);
                        Hexp = H(:,(2*rh)+1:(2+AIMData.nlead)*rh);
                        H = H(:,rh+1:2*rh);
                        %
                        % now we select the variables
                        %
                        StateVarPos = DSGEModel.StateVariablePositions;
                        if AIMData.nlead>1;
                           for j=2:AIMData.nlead;
                              StateVarPos = [StateVarPos DSGEModel.StateVariablePositions+(rh*(j-1))];
                           end;
                        end;
                        Hlag = Hlag(DSGEModel.StateEquationPositions,DSGEModel.StateVariablePositions);
                        Hexp = Hexp(DSGEModel.StateEquationPositions,StateVarPos);
                        D = -H(DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
                        H = H(DSGEModel.StateEquationPositions,DSGEModel.StateVariablePositions);
                        %
                        % checking the solution
                        %
                        S0 = H;
                        Hcheck = Hlag+(H*F);
                        r = length(DSGEModel.StateVariablePositions);
                        Fmat = F;
                        for j=1:AIMData.nlead;
                           ind = (1:r)+(r*(j-1));
                           S0 = S0+(Hexp(:,ind)*Fmat);
                           Fmat = Fmat*F;
                           Hcheck = Hcheck+(Hexp(:,ind)*Fmat);
                        end;
                        Bcheck = B0-(S0\D);
                        %
                        SolutionError = max(max(abs([Hcheck Bcheck])));
                        %
                        % ask if the user would like to save the results to file
                        %
                        txt = 'Would you like to save the structural form matrices at posterior mode to file?';
                        answer = Query(txt,'question',200,'Structural Form - Posterior Mode',500,'no',CurrINI);
                        %
                        if strcmp(lower(answer),'yes')==1;
                           %
                           ShockNames = DSGEModel.StateShockNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\StructuralForm-' DSGEModel.NameOfModel '-PosteriorMode.mat']);
                           save(file,'StateVariableNames','ShockNames','Hlag','H','Hexp','D','F','B0');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SF' strrep(DSGEModel.NameOfModel,'-','') 'PostMode.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with names of the state variables \n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% string matrix with names of the economic shocks \n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% S T R U C T U R A L   F O R M\n');
                           fprintf(fid,'%%\n%%    Hlag*xi(t-1) + H*xi(t) + sum(j=1,%0.0f)Hexp*E_t[xi(t+j)] = D*eta(t)\n',AIMData.nlead);
                           fprintf(fid,'%%\n%% matrix with t-1 coefficients on the state variables\n%%\n');
                           fprintf(fid,'Hlag = DataStructure.Hlag;\n');
                           fprintf(fid,'%%\n%% matrix with t coefficients on the state variables\n%%\n');
                           fprintf(fid,'H = DataStructure.H;\n');
                           fprintf(fid,'%%\n%% matrix with t+1,...,t+%0.0f coefficients on expected values of the state variables\n%%\n',AIMData.nlead);
                           fprintf(fid,'Hexp = DataStructure.Hexp;\n');
                           fprintf(fid,'%%\n%% matrix with t coefficients on the economic shocks\n%%\n');
                           fprintf(fid,'D = DataStructure.D;\n');
                           fprintf(fid,'%%\n%% S O L U T I O N   (R E D U C E D   F O R M)\n');
                           fprintf(fid,'%%\n%%    xi(t) = F*xi(t-1) + B0*eta(t)\n');
                           fprintf(fid,'%%\n%% matrix with state transition parameters\n%%\n');
                           fprintf(fid,'F = DataStructure.F;\n');
                           fprintf(fid,'%%\n%% matrix with shock parameters\n%%\n');
                           fprintf(fid,'B0 = DataStructure.B0;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The structural form at posterior mode parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: StateVariableNames (string matrix with names of the state variables), ShockNames (string matrix with names of the economic shocks), Hlag (structural form matrix with coefficients on lagged state variables), H (structural form matrix with coefficients on current state variables), Hexp (structural form matrix with coefficients on expected values of future state variables), D (structural form matrix with coefficients on current economic shocks), F (reduced form matrix with parameters on lagged state variables), and B0 (reduced form matrix with parameters on the shocks). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Save Structural Form - Posterior Mode',500,600,CurrINI);
                        end;
                        %
                        % print the structural form to file and report the solution error
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\StructuralForm-' DSGEModel.NameOfModel '-PosteriorMode.txt']);
                        PrintStructuralForm(ResFile,DSGEModel,CurrINI,Hlag,H,Hexp,D,AIMData.nlead,SolutionError,'','Posterior mode');
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Structural Form - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                        end;
                     else;
                        txt = ['The numbers of leads (' int2str(AIMData.nlead) ') and lags (1) don''t match with the dimension of the structural form matrix supplied through AiM.'];
                        About(txt,'information','YADA - Error Message',150,500,CurrINI);
                     end;
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','Save State-Space Matrices - Posterior Mode',200,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateSpaceSaveInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % load the AIMData file
            %
            AIMData = load(DSGEModel.AIMDataFile);
            %
            % check if we can solve the model
            %
            ErrorStr = '';
            try;
               [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
               if isempty(ErrMsg)==0;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
               end;
            catch;
               ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
            end;
            if strcmp(ErrorStr,'')==1;
               if mcode==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\StateSpaceMatrices-' DSGEModel.NameOfModel '-InitialValues.mat']);
                  save(file,'A','H','R','F','B0');
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\SSM' strrep(DSGEModel.NameOfModel,'-','') 'InitVal.m']);
                  %
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                  fprintf(fid,'%%\n%% matrix with parameters on exogenous variables\n%%\n');
                  fprintf(fid,'A = DataStructure.A;\n');
                  fprintf(fid,'%%\n%% matrix with parameters on state variables\n%%\n');
                  fprintf(fid,'H = DataStructure.H;\n');
                  fprintf(fid,'%%\n%% matrix with measurement error covariances\n%%\n');
                  fprintf(fid,'R = DataStructure.R;\n');
                  fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                  fprintf(fid,'%%\n%% matrix with state transition parameters\n%%\n');
                  fprintf(fid,'F = DataStructure.F;\n');
                  fprintf(fid,'%%\n%% matrix with shock parameters\n%%\n');
                  fprintf(fid,'B0 = DataStructure.B0;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  %
                  txt = ['The state-space matrices at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: A (matrix with parameters on exogenous variables in the measurement equation), H (matrix with parameters on state variables in the measurement equation), R (matrix with measurement error covariances), F (matrix with state transition parameters), and B0 (matrix with shock parameters). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Save State-Space Matrices - Initial Values',200,500,CurrINI);
                  drawnow;
               else;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  end;
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                  About(txt,'information','Save State-Space Model - Initial Values',120,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateSpaceSavePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % copy files to the tmp folder
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateSpaceMatrices-' DSGEModel.NameOfModel '-PosteriorMode.mat']);
                     save(file,'A','H','R','F','B0');
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SSM' strrep(DSGEModel.NameOfModel,'-','') 'PostMode.m']);
                     %
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                     fprintf(fid,'%%\n%% matrix with parameters on exogenous variables\n%%\n');
                     fprintf(fid,'A = DataStructure.A;\n');
                     fprintf(fid,'%%\n%% matrix with parameters on state variables\n%%\n');
                     fprintf(fid,'H = DataStructure.H;\n');
                     fprintf(fid,'%%\n%% matrix with measurement error covariances\n%%\n');
                     fprintf(fid,'R = DataStructure.R;\n');
                     fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                     fprintf(fid,'%%\n%% matrix with state transition parameters\n%%\n');
                     fprintf(fid,'F = DataStructure.F;\n');
                     fprintf(fid,'%%\n%% matrix with shock parameters\n%%\n');
                     fprintf(fid,'B0 = DataStructure.B0;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     txt = ['The state-space matrices at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: A (matrix with parameters on exogenous variables in the measurement equation), H (matrix with parameters on state variables in the measurement equation), R (matrix with measurement error covariances), F (matrix with state transition parameters), and B0 (matrix with shock parameters). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Save State-Space Matrices - Posterior Mode',200,500,CurrINI);
                     drawnow;
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','Save State-Space Matrices - Posterior Mode',120,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateSpaceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  ParameterNames = ModeData.ParameterNames;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % load the AIMData file
            %
            AIMData = load(DSGEModel.AIMDataFile);
            %
            % check if we can solve the model
            %
            ErrorStr = '';
            try;
               [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
               if isempty(ErrMsg)==0;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
               end;
            catch;
               ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
            end;
            if strcmp(ErrorStr,'')==1;
               if mcode==1;
                  if length(size(H))==3;
                     %
                     % check if H has the required size
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     if size(H,3)>=LastPeriod;
                        %
                        % pick a certain period for the H matrix
                        %
                        SubSampleMat = get(controls.dsge.initializekalman,'String');
                        SubSampleMat = SubSampleMat(DSGEModel.KalmanFirstObservation:size(SubSampleMat,1),:);
                        [action,SelectedPosition] = SelectionDlg(SubSampleMat,size(SubSampleMat,1),'Select time period for the measurement matrix H','Measurement Matrix','State-Space Form - Initial Values','','',CurrINI);
                        %
                        SelPos = FirstPeriod+DSGEModel.KalmanFirstObservation-1+(SelectedPosition-1);
                        H = H(:,:,SelPos);
                     else;
                        action = 'Cancel';
                        ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from showing the state-space form.'];
                        About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
                     end;
                  else;
                     action = 'OK';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     %
                     % we can now setup the state-space form
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateSpaceForm-' DSGEModel.NameOfModel '-InitValues.txt']);
                     %
                     PrintStateSpaceForm(file,DSGEModel,CurrINI,A,H,R,F,B0,length(theta),size(ParameterNames.all,1),'Initial Values');
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['State-Space Form - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  end;
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                  About(txt,'information','State-Space Form - Initial Values',120,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateSpacePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'State-Space Form - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % First we check if the calculations have already been performed
            %
            MatFile = [DSGEModel.OutputDirectory '\statespace\DSGE-StateSpace-PriorMoments-' DSGEModel.NameOfModel '-' int2str(NumPriorDraws) '.mat'];
            if FileExist(MatFile)==1;
               txt = 'YADA has located results for the prior distribution on disk. Would you like to use these results?';
               answer = Query(txt,'question',150,'State Space Form - Prior Distribution',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               StateSpaceData = load(MatFile);
               A = StateSpaceData.A;
               H = StateSpaceData.H;
               R = StateSpaceData.R;
               F = StateSpaceData.F;
               B0 = StateSpaceData.B0;
            else;
               %
               % compute the estimated
               %
               [A,H,R,F,B0] = DSGEStateSpaceMoments(theta,thetaDraws',thetaPositions,ModelParameters,0,[],NumPriorDraws,[],DSGEModel,CurrINI);
            end;
            if isempty(H)==0;
               %
               % write the results to file
               %
               file = FixFilePath([DSGEModel.OutputDirectory '\StateSpaceForm-PriorMoments-' DSGEModel.NameOfModel '-' int2str(NumPriorDraws) '.txt']);
               %
               PrintStateSpaceFormMoments(file,DSGEModel,CurrINI,A,H,R,F,B0,length(theta),size(ParameterNames.all,1),'Prior Distribution');
               %
               if FileExist(file)==1;
                  TextGUI(file,['State-Space Form - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Prior Mean and Standard Deviation',50,CurrINI,1,1);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateSpacePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % copy files to the tmp folder
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     if length(size(H))==3;
                        %
                        % check if H has the required size
                        %
                        [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                        if size(H,3)>=LastPeriod;
                           %
                           % pick a certain period for the H matrix
                           %
                           SubSampleMat = get(controls.dsge.initializekalman,'String');
                           SubSampleMat = SubSampleMat(DSGEModel.KalmanFirstObservation:size(SubSampleMat,1),:);
                           [action,SelectedPosition] = SelectionDlg(SubSampleMat,size(SubSampleMat,1),'Select time period for the measurement matrix H','Measurement Matrix','State-Space Form - Posterior Mode','','',CurrINI);
                           %
                           SelPos = FirstPeriod+DSGEModel.KalmanFirstObservation-1+(SelectedPosition-1);
                           H = H(:,:,SelPos);
                        else;
                           action = 'Cancel';
                           ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from showing the state-space form.'];
                           About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
                        end;
                     else;
                        action = 'OK';
                     end;
                     if strcmp(lower(action),'ok')==1;
                        %
                        % we can now setup the state-space form
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateSpaceForm-' DSGEModel.NameOfModel '-PosteriorMode.txt']);
                        %
                        PrintStateSpaceForm(file,DSGEModel,CurrINI,A,H,R,F,B0,length(ModeData.theta),size(ModeData.ParameterNames.estimated,1)+size(ModeData.ParameterNames.calibrated,1)+size(ModeData.ParameterNames.additional,1),'Posterior Mode');
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['State-Space Form - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                        end;
                     end;
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','State-Space Form - Posterior Mode',120,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateSpacePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','State-Space Form - Posterior','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % First we check if the calculations have already been performed
                  %
                  MatFile = [DSGEModel.OutputDirectory '\statespace\DSGE-StateSpace-PosteriorMoments-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws) '.mat'];
                  if FileExist(MatFile)==1;
                     txt = 'YADA has located results for the posterior distribution on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,'State Space Form - Posterior Distribution',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     StateSpaceData = load(MatFile);
                     A = StateSpaceData.A;
                     H = StateSpaceData.H;
                     R = StateSpaceData.R;
                     F = StateSpaceData.F;
                     B0 = StateSpaceData.B0;
                  else;
                     %
                     % compute the estimated
                     %
                     [A,H,R,F,B0] = DSGEStateSpaceMoments(theta,thetaPostSample,thetaPositions,ModelParameters,1,Weights,NumDraws,CurrChain,DSGEModel,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
                  %
                  if isempty(H)==0;
                     %
                     % write the results to file
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateSpaceForm-PosteriorMoments-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws) '.txt']);
                     %
                     PrintStateSpaceFormMoments(file,DSGEModel,CurrINI,A,H,R,F,B0,length(theta),size(ParameterNames.estimated,1)+size(ParameterNames.calibrated,1)+size(ParameterNames.additional,1),'Posterior Distribution');
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['State-Space Form - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterrior Mean and Standard Deviation',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PlotAPrior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the plot-a-prior dialog
         %
         PlotAPrior('init',DSGEModel,CurrINI);
         %
         set(controls.about,'UserData',0);
      end;
   case 'EstimateParamCovMat'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            %
            % get the parameter names
            %
            ParameterNames = ModeData.ParameterNames.estimated;
            InverseHessian = ModeData.InverseHessian;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Estimate Parameter Covariance Matrix','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            if strcmp(lower(action),'cancel')==0;
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('thetaPostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % skip the burnin draws
                  %
                  PostSample = PostSample(NumBurnin+1:NumDraws,:)';
                  %
                  % compute the mean
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     ParamMean = (1/(NumDraws-NumBurnin))*sum(PostSample,2);
                     %
                     % and now the covariance matrix
                     %
                     PostSample = PostSample-(ParamMean*ones(1,NumDraws-NumBurnin));
                     ParameterCovarianceMatrix = (1/(NumDraws-NumBurnin))*PostSample*PostSample';
                  else;
                     NumParam = size(PostSample,1);
                     Weightsnp = (Weights*ones(1,NumParam))';
                     ParamMean = (1/NumDraws)*sum(PostSample.*Weightsnp,2);
                     PostSample = PostSample-(ParamMean*ones(1,NumDraws));
                     ParameterCovarianceMatrix = (1/NumDraws)*(PostSample.*Weightsnp)*PostSample';
                  end;
                  clear('PostSample');
                  %
                  % now we save the matrix
                  %
                  MatFile = FixFilePath([DSGEModel.OutputDirectory '\' DSGEModel.NameOfModel '-EstParamCovarianceMatrix-' int2str(NumBurnin) '.' int2str(NumDraws) '.' int2str(CurrChain) '.mat']);
                  save(MatFile,'ParameterCovarianceMatrix');
                  drawnow;
                  %
                  % provide a message box
                  %
                  txt = ['YADA has estimated a parameter covariance matrix based on ' int2str(NumDraws-NumBurnin) ' post-burnin-period draws from the posterior distribution. The variable containing the matrix, "ParameterCovarianceMatrix", has been saved to disk in the file "' GetFilename(MatFile) '", located in the directory "' GetPath(MatFile) '". To use this matrix in the proposal density for the random-walk Metropolis algorithm you need to select "My estimate" for the option "Method for estimating the inverse Hessian" on the "Options" tab.'];
                  About(txt,'information','Estimate Parameter Covariance Matrix',400,500,CurrINI);
                  %
                  % now print the results to file and compare with the inverse Hessian
                  %
                  TxtFile = FixFilePath([DSGEModel.OutputDirectory '\' DSGEModel.NameOfModel '-EstParamCovarianceMatrix-' int2str(NumBurnin) '.' int2str(NumDraws) '.' int2str(CurrChain) '.txt']);
                  %
                  PrintParameterCovarianceMatrix(TxtFile,ParameterNames,ParameterCovarianceMatrix,InverseHessian,NumBurnin,NumDraws,CurrINI);
                  %
                  % display the results
                  %
                  if FileExist(TxtFile);
                     TextGUI(TxtFile,['Estimated Parameter Covariance Matrix - ' GetFilename(MatFile)],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Parameter Covariance Matrix - Transformed Parameters',50,CurrINI,1,1);
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModeInfo'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeTextFile = get(controls.modeinfo,'UserData');
         TextGUI(ModeTextFile,['Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'ModeSummaryInfo'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeTextFile = get(controls.modesummary,'UserData');
         TextGUI(ModeTextFile,['Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'OptErrorSummaryInfo'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ErrorTextFile = get(controls.opterrorsummary,'UserData');
         TextGUI(ErrorTextFile,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'PostSampleSummary'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         PostSampleTextFile = get(gcbo,'UserData');
         TextGUI(PostSampleTextFile,['Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'RawPlots'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Raw Posterior Draws','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               NumPostDraws = length(LogPost);
               %
               % check which parameters to use
               %
               if gcbo==controls.postsample.rawplots_original;
                  ParamSample = thetaPostSample;
                  TypeStr = 'Original';
               elseif gcbo==controls.postsample.rawplots_transformed;
                  ParamSample = PostSample;
                  TypeStr = 'Transformed';
               elseif gcbo==controls.postsample.rawplots_weights;
                  ParamSample = Weights;
                  TypeStr = 'Weights';
               else;
                  ParamSample = LogPost;
                  TypeStr = 'Log Posterior Kernel Values';
                  thetaPost = thetaPostSample;
               end;
               %
               % clear posterior sample data
               %
               clear('PostSample','thetaPostSample','LogPost','AcceptedDraws','Weights');
               %
               % ask if the user wants to plot a subsample
               %
               if (gcbo==controls.postsample.rawplots_original)|(gcbo==controls.postsample.rawplots_transformed);
                  if DSGEModel.PosteriorSampler<=7;
                     txt = 'Would you like to plot a sub-sample of the posterior draws rather than the full sample?';
                     answer = Query(txt,'question',400,['Raw Posterior Draws - ' TypeStr ' Parameters'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        ParamSample = ParamSample(NumBurnin+1:NumPostDraws,:);
                        NumDraws = NumPostDraws-NumBurnin;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        ParamSample = ParamSample(UseDraws,:);
                     end;
                  end;
                  %
                  % send the data to the raw plotting function
                  %
                  drawnow;
                  RawPosteriorPlots(ParamSample,NumBurnin,NumPostDraws,CurrChain,TypeStr,DSGEModel,CurrINI);
               elseif gcbo==controls.postsample.rawplots_weights;
                  %
                  % send the data to the raw plotting function
                  %
                  drawnow;
                  RawWeightsPosteriorPlots(ParamSample,[],NumPostDraws,CurrChain,DSGEModel,CurrINI,controls);
               else;
                  if DSGEModel.PosteriorSampler<=7;
                     txt = 'Would you like to plot a sub-sample of the log posterior kernel values for the posterior draws rather than the full sample?';
                     answer = Query(txt,'question',400,['Raw Posterior Draws - ' TypeStr],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        ParamSample = ParamSample(NumBurnin+1:NumPostDraws,:);
                        NumDraws = NumPostDraws-NumBurnin;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        ParamSample = ParamSample(UseDraws,:);
                        thetaPost = thetaPost(UseDraws,:);
                        drawnow;
                     end;
                  end;
                  %
                  RawLogPosteriorPlot(ParamSample,thetaPost,NumBurnin,NumPostDraws,CurrChain,DSGEModel,CurrINI);
                  clear('thetaPost');
               end;
               %
               % clear the used parameter draws
               %
               clear('ParamSample');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ScatterPlots'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Scatter-Plot Posterior Draws','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            %
            % check if the data actually exists
            %
            if isempty(ErrorStr)==1;
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  PostSample = PostSample(NumBurnin+1:NumDraws,:);
                  LogPost = LogPost(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which parameters to use
                  %
                  if gcbo==controls.postsample.scatterplots_original;
                     ParamSample = thetaPostSample;
                     TypeStr = 'Original';
                     ParamMode = ModeData.thetaMode;
                  else;
                     ParamSample = PostSample;
                     TypeStr = 'Transformed';
                     ParamMode = ModeData.phiMode;
                  end;
                  %
                  % clear posterior sample data
                  %
                  clear('PostSample','thetaPostSample','AcceptedDraws');
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        ParamSample = ParamSample(UseDraws,:);
                        LogPost = LogPost(UseDraws);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % send the data to the scatter-plotting function
                  %
                  drawnow;
                  ScatterPlotPosteriorDraws('init',ParamSample,LogPost,ModeData.ParameterNames.estimated,ParamMode,CurrChain,'DSGE',TypeStr,DSGEModel,CurrINI);
                  %
                  % clear the used parameter draws
                  %
                  clear('ParamSample','LogPost');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ChibJeliazkov'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Sequential Marginal Likelihood - Chib & Jeliazkov','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('thetaPostSample');
               NumPostDraws = length(LogPost);
               %
               % as if the user wants to run the sequential estimation routine or just see the end point results
               %
               txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
               answer = Query(txt,'question',200,'Marginal Likelihood - Chib & Jeliazkov',500,'no',CurrINI);
               %
               % run the sequential estimation routine
               %
               drawnow;
               if strcmp(lower(answer),'no')==1;
                  PlotMargLikeChibJeliazkov(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI);
               else;
                  %
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
                  ErrorStr = '';
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                  end;
                  %
                  % check if the user wants to recompute the log posterior
                  %
                  if isempty(ErrorStr)==1;
                     txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                     answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood - Chib & Jeliazkov',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  OldNumBurnin = NumBurnin;
                  if strcmp(lower(answer),'yes')==1;
                     NumPostDraws = length(LogPost);
                     PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                     AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                     NumBurnin = 0;
                     LogPost = DSGELogPosterior(PostSample,AcceptedDraws,ModeData,'Chib & Jeliazkov',DSGEModel,CurrINI);
                  end;
                  %
                  if isempty(LogPost)==0;
                     TmpDSGEModel = DSGEModel;
                     TmpDSGEModel.SequentialML = 0;
                     %
                     LogMargsCJ = PlotMargLikeChibJeliazkov(PostSample,LogPost,NumBurnin,CurrChain,TmpDSGEModel,CurrINI);
                     clear('TmpDSGEModel');
                     %
                     % check if we got any results
                     %
                     if isempty(LogMargsCJ)==0;
                        NumBurnin = OldNumBurnin;
                        %
                        % retrieve estimation sample start year and period
                        %
                        [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ChibJeliazkov-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                        fid = fopen(TextFile,'wt');
                        prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                        fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                        if NumChains>1;
                           fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                        end;
                        %
                        % sample information
                        %
                        fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'==========================================================\n');
                        fprintf(fid,'                          point estimate        std. error\n');
                        fprintf(fid,'----------------------------------------------------------\n');
                        fprintf(fid,['Chib & Jeliazkov         ' prt_val '   ' prt_val '\n'],LogMargsCJ(1,2),LogMargsCJ(1,3));
                        fprintf(fid,'==========================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        %
                        fclose(fid);
                        %
                        % show in TextGUI
                        %
                        TextGUI(TextFile,'Marginal Likelihood - Chib & Jeliazkov',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                     end;
                  end;
               end;
               %
               % clear everything
               %
               clear('PostSample','LogPost','AcceptedDraws');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'TruncatedElliptical'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load data from file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Sequential Marginal Likelihood - SWZ Harmonic Mean','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('thetaPostSample');
                  %
                  % asK if the user wants to run the sequential estimation routine or just see the end point results
                  %
                  txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
                  answer = Query(txt,'question',200,'Marginal Likelihood - SWZ''S Harmonic Mean',500,'no',CurrINI);
                  %
                  % select the PValue
                  %
                  PValueStrMatrix = strvcat('90 percent of posterior draws','80 percent of posterior draws','70 percent of posterior draws','60 percent of posterior draws','50 percent of posterior draws');
                  [action,SelectedOption] = SelectionDlg(PValueStrMatrix,1,'Select the truncation parameter of the elliptical density:','Truncation Parameter','Marginal Likelihood - SWZ''S Harmonic Mean',[],[],CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     PValue = str2num(StringTrim(strrep(PValueStrMatrix(SelectedOption,:),'percent of posterior draws','')));
                     %
                     % run the sequential estimation routine
                     %
                     drawnow;
                     if strcmp(lower(answer),'no')==1;
                        PlotMargLikeSWZModifiedHarmonic(PostSample,LogPost,PValue,ModeData,NumBurnin,CurrChain,DSGEModel,CurrINI);
                     else;
                        NumPostDraws = length(LogPost);
                        %
                        % Coverage probabilities for Geweke''s Harmonic Mean estimator of
                        % the marginal likelihood
                        %
                        CovStartStr = get(controls.dsge.coveragestart,'String');
                        CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
                        CovIncStr = get(controls.dsge.coverageincrement,'String');
                        CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
                        CovEndStr = get(controls.dsge.coverageend,'String');
                        CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
                        CovProb = (CovStartValue:CovIncValue:CovEndValue);
                        %
                        % check if the user wants to recompute the log posterior
                        %
                        txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                        answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
                        %
                        % select which posterior draws to use
                        %
                        LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
                        PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                        AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                        NumDraws = NumPostDraws-NumBurnin;
                        %
                        % check if the user wishes a sub-sample
                        %
                        txt = 'Would you like to use a sub-sample of the posterior draws?';
                        answersub = Query(txt,'question',400,'Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
                        if strcmp(lower(answersub),'yes')==1;
                           %
                           % check which draws we should use
                           %
                           UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                           LogPost = LogPost(UseDraws);
                           PostSample = PostSample(UseDraws,:);
                           AcceptedDraws = AcceptedDraws(UseDraws);
                        end;
                        NumDraws = length(AcceptedDraws);
                        %
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % recompute the log posterior
                           %
                           LogPost = DSGELogPosterior(PostSample,AcceptedDraws,ModeData,'SWZ''s Harmonic Mean',DSGEModel,CurrINI);
                        end;
                        %
                        if isempty(LogPost)==0;
                           [LogMargsSWZ,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample,LogPost,NumPostDraws,PValue,0,ModeData,[],DSGEModel,CurrINI);
                           if isempty(LogMargsSWZ)==0;
                              %
                              % write results to a text file
                              %
                              [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                              %
                              TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-SWZ-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
                              fid = fopen(TextFile,'wt');
                              prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                              %
                              fprintf(fid,'********************************************************************************\n');
                              fprintf(fid,'*                                                                              *\n');
                              fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                              fprintf(fid,'*                                                                              *\n');
                              fprintf(fid,'********************************************************************************\n\n');
                              fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                              fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
                              fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                              if NumChains>1;
                                 fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                              end;
                              %
                              % sample information
                              %
                              fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                              fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                              %
                              % SWZ's modified harmonic mean estimator
                              %
                              fprintf(fid,'===============================================================\n');
                              fprintf(fid,'                              point estimate        std. error\n');
                              fprintf(fid,'---------------------------------------------------------------\n');
                              fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],LogMargsSWZ(1,2),NWStdErr);
                              fprintf(fid,'---------------------------------------------------------------\n');
                              fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],LBound);
                              fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],qL);
                              fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
                              fprintf(fid,'===============================================================\n\n');
                              fprintf(fid,'NOTE: The "fraction of iid draws from elliptical > lower bound" notation refers to the fraction of\n      the parameter draws which have a log posterior kernel greater than the lower bound. The "fraction\n      of posterior draws > lower bound" notation also refers to the log posterior kernel values.\n\n');
                              %
                              fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                              fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                              fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                              %
                              fclose(fid);
                              %
                              % show in TextGUI
                              %
                              TextGUI(TextFile,'Marginal Likelihood - SWZ''s Modified Harmonic Mean',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                           end;
                        end;
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModifiedHarmonicMean'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Sequential Marginal Likelihood - Geweke''s Harmonic Mean','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('thetaPostSample');
               %
               % asK if the user wants to run the sequential estimation routine or just see the end point results
               %
               txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
               answer = Query(txt,'question',200,'Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
               %
               % run the sequential estimation routine
               %
               drawnow;
               if strcmp(lower(answer),'no')==1;
                  PlotMargLikeModifiedHarmonic(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI);
               else;
                  NumPostDraws = length(LogPost);
                  %
                  % Coverage probabilities for Geweke''s Harmonic Mean estimator of
                  % the marginal likelihood
                  %
                  CovStartStr = get(controls.dsge.coveragestart,'String');
                  CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
                  CovIncStr = get(controls.dsge.coverageincrement,'String');
                  CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
                  CovEndStr = get(controls.dsge.coverageend,'String');
                  CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
                  CovProb = (CovStartValue:CovIncValue:CovEndValue);
                  %
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
                  ErrorStr = '';
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                  end;
                  %
                  % check if the user wants to recompute the log posterior
                  %
                  if isempty(ErrorStr)==1;
                     txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                     answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  % select which posterior draws to use
                  %
                  LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
                  PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                  AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws?';
                  answersub = Query(txt,'question',400,'Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
                  if strcmp(lower(answersub),'yes')==1;
                     %
                     % check which draws we should use
                     %
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     LogPost = LogPost(UseDraws);
                     PostSample = PostSample(UseDraws,:);
                     AcceptedDraws = AcceptedDraws(UseDraws);
                  end;
                  NumDraws = length(AcceptedDraws);
                  %
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % recompute the log posterior
                     %
                     LogPost = DSGELogPosterior(PostSample,AcceptedDraws,ModeData,'Geweke''s Harmonic Mean',DSGEModel,CurrINI);
                  end;
                  %
                  if isempty(LogPost)==0;
                     [LogMargsMH,CovProb,NWStdErr] = MargLikeModifiedHarmonic(PostSample,LogPost,0,CovProb,DSGEModel);
                     %
                     % write results to a text file
                     %
                     [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                     %
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
                     fid = fopen(TextFile,'wt');
                     prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     %
                     fprintf(fid,'********************************************************************************\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'********************************************************************************\n\n');
                     fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                     fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
                     fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                     if NumChains>1;
                        fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                     end;
                     %
                     % sample information
                     %
                     fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % Geweke's modified harmonic mean estimator
                     %
                     prt_str = 'Geweke''s Harmonic Mean';
                     top_str = '                      ';
                     std_str = 'Num. standard errors: ';
                     for i=1:length(CovProb);
                        top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
                        prt_str = [prt_str '   ' prt_val];
                        std_str = [std_str '   ' prt_val];
                     end;
                     TopStr = MultiCharStr('=',length(top_str));
                     MidStr = MultiCharStr('-',length(top_str));
                     fprintf(fid,[TopStr '\n']);
                     fprintf(fid,'                              Coverage probabilities for the estimator\n');
                     fprintf(fid,[top_str '\n']);
                     fprintf(fid,[MidStr '\n']);
                     fprintf(fid,[prt_str '\n'],LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
                     fprintf(fid,[std_str '\n'],NWStdErr');
                     fprintf(fid,[MidStr '\n']);
                     fprintf(fid,['Mean:                    ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
                     fprintf(fid,['Median:                  ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
                     fprintf(fid,[TopStr '\n\n']);
                     %
                     % Some information about the filename and its location
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     %
                     fclose(fid);
                     %
                     % show in TextGUI
                     %
                     TextGUI(TextFile,'Marginal Likelihood - Geweke''s Modified Harmonic Mean',[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                  end;
               end;
               %
               % clear everything
               %
               clear('PostSample','LogPost','AcceptedDraws');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplaceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % get the parameter data
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  UniformBounds = ModeData.UniformBounds;
                  thetaIndex = ModeData.thetaIndex;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         %
         if InitStatus==1;
            ErrorStr = '';
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
            if isempty(ErrorStr)==1
               %
               % we can now calculate the log-likelihood
               %
               if DSGEModel.KalmanAlgorithm~=2;
                  [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
               else;
                  [StateVarStructure.lnLt,StateVarStructure.nonsingular,StateVarStructure.KalmanTime,mcode,kalmanstatus] = CalculateDSGESquareRootFilter(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
               end;
               if (mcode==1)&(kalmanstatus==0);
                  %
                  % test the rest
                  %
                  if (isfield(StateVarStructure,'lnLt')==1)&(isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                     if StateVarStructure.nonsingular==1;
                        %
                        % compute the value of the prior
                        %
                        phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
                        LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
                        LogPriorIV = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
                        IsSystemPrior = 0;
                        if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
                           LogSPIV = 0;
                           LogMargLikeSystemLaplace = 0;
                        else;
                           if FileExist(DSGEModel.SystemPriorFile)==1;
                              %
                              % copy file to tmp dir
                              %
                              [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                              try;
                                 eval(['LogSPIV = ' GetName(DSGEModel.SystemPriorFile) '(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
                                 IsSystemPrior = 1;
                                 if (exist('ModeData')==1)&(isfield(ModeData,'LogMargLikeSystemLaplace')==1);
                                    LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
                                 else;
                                    %
                                    % check if the system prior mode file exist
                                    %
                                    SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
                                    if FileExist(SystemPriorModeFile)==1;
                                       SystemPriorModeData = load(SystemPriorModeFile);
                                       %
                                       % we only need the Laplace approximation of the log marginal likelihood of the system prior
                                       %
                                       LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
                                    else;
                                       %
                                       % need to compute the prior mode of the system prior first
                                       %
                                       LogMargLikeSystemLaplace = NaN;
                                    end;
                                 end;
                              catch;
                                 LogSPIV = 0;
                                 LogMargLikeSystemLaplace = 0;
                              end;
                           else;
                              LogSPIV = 0;
                              LogMargLikeSystemLaplace = 0;
                           end;
                        end;
                        %
                        NumParam = length(theta);
                        StepLengthStr = get(controls.optimize.steplength,'String');
                        StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                        %
                        % set the InitialStateVector value
                        %
                        if DSGEModel.UseOwnInitialState==1;
                           if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
                              DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
                           else;
                              DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                           end;
                        else;
                           DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                        end;
                        [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                        %
                        VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phi,StepLength,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
                        HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                        %
                        LogPostDensity = sum(StateVarStructure.lnLt)+LogJacobianIV+LogPriorIV+LogSPIV;
                        %
                        if isnan(LogMargLikeSystemLaplace)==0;
                           LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                        else;
                           LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0);
                        end;
                        %
                        PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                        [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-InitialValues.txt']);
                        fid = fopen(TextFile,'wt');
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                        fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        % sample information
                        %
                        fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        %
                        % now the marginal likelihood
                        %
                        fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                        fprintf(fid,'where \n\n');
                        fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                        if isnan(LogMargLikeSystemLaplace)==0;
                           fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                        else;
                           fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity);
                        end;
                        if IsSystemPrior==1;
                           if isnan(LogMargLikeSystemLaplace)==0;
                              fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                           else;
                              fprintf(fid,'   The correction term for the log\n   marginal likelihood of the system\n   prior has not been computed yet.\n   Please run prior sampling.\n\n');
                           end;
                        end;
                        fprintf(fid,'================================================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        %
                        % display the results
                        %
                        TextGUI(TextFile,['Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Initial Values',100,CurrINI,0,1);
                     elseif sum(isinf(StateVarStructure.lnLt))>0;
                        txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                        About(txt,'information','Laplace Approximation - Initial Values',300,500,CurrINI);
                     end;
                  end;
               elseif (mcode==1)&(kalmanstatus~=0);
                  if kalmanstatus==1;
                     txt = 'The state variables are not stationary at the initial values.';
                  else;
                     if StateVarStructure.nonsingular==1;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     else;
                        txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                     end;
                  end;
                  About(txt,'information','Laplace Approximation - Initial Values',200,500,CurrINI);
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  end;
                  txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Laplace Approximation - Initial Values',200,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplacePosteriorMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load data from file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if isempty(ErrorStr)==1;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % we can now calculate the log-likelihood
            %
            if DSGEModel.KalmanAlgorithm~=2;
               StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
            else;
               [StateVarStructure.lnLt,StateVarStructure.nonsingular,StateVarStructure.KalmanTime] = CalculateDSGESquareRootFilter(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
            end;
            if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'lnLt')==1);
               if (isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                  if StateVarStructure.nonsingular==1;
                     %
                     % compute the value of the prior
                     %
                     LogJacobianMode = logJacobian(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
                     LogPriorMode = logPriorDSGE(ModeData.thetaMode,ModeData.thetaDist,ModeData.PriorDist,ModeData.LowerBound);
                     IsSystemPrior = 0;
                     if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
                        LogSPMode = 0;
                        LogMargLikeSystemLaplace = 0;
                     else;
                        if FileExist(DSGEModel.SystemPriorFile)==1;
                           %
                           % copy file to tmp dir
                           %
                           [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                           try;
                              eval(['LogSPMode = ' GetName(DSGEModel.SystemPriorFile) '(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);']);
                              LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
                              IsSystemPrior = 1;
                           catch;
                              LogSPMode = 0;
                              LogMargLikeSystemLaplace = 0;
                           end;
                        else;
                           LogSPMode = 0;
                           LogMargLikeSystemLaplace = 0;
                        end;
                     end;
                     %
                     % ask if the inverse Hessian at the mode should be recalculated
                     %
                     txt = 'Do you want YADA to recalculate the inverse Hessian at the posterior mode for the Laplace approximation of the marginal likelihood?';
                     answer = Query(strvcat(txt,' ','NOTE: The recalculation will use finite differences for the inverse Hessian. YADA will otherwise use the inverse Hessian from the posterior mode estimation output for the transformed parameters.'),'question',400,'Laplace Approximation - Posterior Mode',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        if isnan(ModeData.phiInverseHessianFinDiff)==0;
                           InverseHessian = ModeData.phiInverseHessianFinDiff;
                        else;
                           InverseHessian = ModeData.InverseHessian;
                        end;
                        UseOld = 1;
                     else;
                        NumParam = length(theta);
                        StepLengthStr = get(controls.optimize.steplength,'String');
                        StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                        %
                        % set the InitialStateVector value
                        %
                        if DSGEModel.UseOwnInitialState==1;
                           if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
                              DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
                           else;
                              DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                           end;
                        else;
                           DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                        end;
                        [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                        %
                        VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',ModeData.phiMode,StepLength,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
                        HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                        UseOld = 0;
                     end;
                     LogPostDensity = sum(StateVarStructure.lnLt)+LogJacobianMode+LogPriorMode+LogSPMode;
                     %
                     if UseOld==1;
                        LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
                     else;
                        LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                     end;
                     %
                     PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                     %
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-PostMode.txt']);
                     fid = fopen(TextFile,'wt');
                     fprintf(fid,'********************************************************************************\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                     fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'********************************************************************************\n\n');
                     %
                     % sample information
                     %
                     fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % now the marginal likelihood
                     %
                     fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                     fprintf(fid,'where \n\n');
                     fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                     fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                     if IsSystemPrior==1;
                        fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                     end;
                     fprintf(fid,'================================================================================\n\n');
                     %
                     % Some information about the filename and its location
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     fclose(fid);
                     %
                     % display the results
                     %
                     TextGUI(TextFile,['Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Posterior Mode',100,CurrINI,0,1);
                  else;
                     txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                     About(txt,'information','Laplace Approximation - Posterior Mode',300,500,CurrINI);
                  end;
               elseif sum(isinf(StateVarStructure.lnLt))>0;
                  txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                  About(txt,'information','Laplace Approximation - Posterior Mode',300,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CUSUMLogPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','CUSUM Plots - Log Posterior Kernel','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','AcceptedDraws','thetaPostSample');
               %
               % send the data to the cusum computation function
               %
               drawnow;
               [CUSUMPost,CUSUMAverage] = CUSUM(LogPost,Weights,NumBurnin,'Log Posterior Kernel',[],CurrINI);
               %
               if isempty(CUSUMPost)==0;
                  %
                  % display the results
                  %
                  CUSUMPlots(CUSUMPost,CUSUMAverage,NumBurnin,CurrChain,'logpost',[],[],DSGEModel,CurrINI);
               end;
               %
               % clear everything
               %
               clear('LogPost');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CUSUMParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if gcbo==controls.postsample.cusumplots.original;
            ParamStr = 'Original';
         else;
            ParamStr = 'Transformed';
         end;
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection',['CUSUM Plots - ' ParamStr ' Parameters'],'','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               if strcmp(lower(ParamStr),'original')==1;
                  ParamDraws = thetaPostSample;
               else;
                  ParamDraws = PostSample;
               end;
               clear('PostSample','AcceptedDraws','LogPost','thetaPostSample');
               %
               % send the data to the cusum computation function
               %
               drawnow;
               [CUSUMPost,CUSUMAverage] = CUSUM(ParamDraws,Weights,NumBurnin,strrep(strrep(get(gcbo,'Label'),'&',''),'...',''),[],CurrINI);
               %
               if isempty(CUSUMPost)==0;
                  %
                  % display the results
                  %
                  CUSUMPlots(CUSUMPost,CUSUMAverage,NumBurnin,CurrChain,ParamStr,[],[],DSGEModel,CurrINI);
               end;
               %
               % clear everything
               %
               clear('ParamDraws');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SPMTLogPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Separated Partial Means - Log Posterior Kernel','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','AcceptedDraws','thetaPostSample');
               drawnow;
               NumIter = size(LogPost,1);
               %
               % check how many separated partial means to compute
               %
               NumMeansStr = '';
               for i=2:20;
                  if IsInteger((NumIter-NumBurnin)/(2*i))==1;
                     NumMeansStr = strvcat(NumMeansStr,int2str(i));
                  end;
               end;
               %
               if isempty(NumMeansStr)==0;
                  [action,NumPM] = SelectionDlg(NumMeansStr,1,'Select the number of partial means to allow for','Partial Means Selection','Separated Partial Means - Log Posterior Kernel','','',CurrINI);
                  %
                  % check if the action was cancelled
                  %
                  if strcmp(lower(action),'cancel')==0;
                     NumPM = str2num(StringTrim(NumMeansStr(NumPM,:)));
                     %
                     % check if we have already calculated the separated partial means test for the selected chain and number of partial means
                     %
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\SeparatedPartialMeans-LogPosteriorKernel-' DSGEModel.NameOfModel '-' num2str(NumPM,'%0.0f') '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.txt']);
                     %
                     if FileExist(ResFile)==1;
                        txt = ['YADA has already computed the separated partial means test for ' num2str(NumPM,'%0.0f') ' separated partial means. Would you like to view this file? If you push the "No" button then YADA will recompute the tests.'];
                        GewekeStat = 1;
                        RunTest = 0;
                        answer = Query(txt,'question',200,'Separated Partial Means Test - Log Posterior Kernel',500,'no',CurrINI);
                        if strcmp(lower(answer),'no')==1;
                           RunTest = 1;
                        end;
                     else;
                        RunTest = 1;
                     end;
                     if RunTest==1;
                        GewekeStat = SeparatedPartialMeansTest(LogPost(NumBurnin+1:NumIter,:),Weights,NumPM,'Log Posterior Kernel',[],CurrINI);
                        %
                        % write the results to file
                        %
                        if isempty(GewekeStat)==0;
                           PrintSeparatedPartialMeans(GewekeStat,NumPM,CurrChain,NumChains,NumIter,NumBurnin,'Log Posterior Kernel','log posterior',[],DSGEModel,CurrINI);
                        end;
                     end;
                     if (isempty(GewekeStat)==0)&(FileExist(ResFile)==1);
                        TextGUI(ResFile,['Separated Partial Means Test - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Log Posterior Kernel',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  txt = ['YADA was unable to find any interger p between 2 and 20 such that ' num2str(NumIter-NumBurnin,'%0.0f') '/2p is an integer. YADA is therefore unable to calculate Geweke''s separated partial means test.'];
                  About(txt,'information','Separated Partial Means - Log Posterior Kernel',200,600,CurrINI);
               end;
               %
               % clear everything
               %
               clear('LogPost');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SPMTParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if gcbo==controls.postsample.spmt.original;
            ParamStr = 'Original';
         else;
            ParamStr = 'Transformed';
         end;
         %
         % Try to load the posterior mode estimates
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         %
         % check if we could successfully load the data
         %
         if isempty(ErrorStr)==1;
            ParameterNames = ModeData.ParameterNames;
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection',['Separated Partial Means - ' ParamStr ' Parameters'],'','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumIter = size(LogPost,1);
                  clear('AcceptedDraws','LogPost');
                  drawnow;
                  %
                  if strcmp(lower(ParamStr),'original')==1;
                     ParamDraws = thetaPostSample;
                  else;
                     ParamDraws = PostSample;
                  end;
                  clear('thetaPostSample','PostSample');
                  %
                  % check how many separated partial means to compute
                  %
                  NumMeansStr = '';
                  for i=2:20;
                     if IsInteger((NumIter-NumBurnin)/(2*i))==1;
                        NumMeansStr = strvcat(NumMeansStr,num2str(i,'%0.0f'));
                     end;
                  end;
                  %
                  if isempty(NumMeansStr)==0;
                     [action,NumPM] = SelectionDlg(NumMeansStr,1,'Select the number of partial means to allow for','Partial Means Selection',['Separated Partial Means - ' ParamStr ' Parameters'],'','',CurrINI);
                     %
                     % check if the action was cancelled
                     %
                     if strcmp(lower(action),'cancel')==0;
                        NumPM = str2num(StringTrim(NumMeansStr(NumPM,:)));
                        %
                        % check if we have already calculated the separated partial means test for the selected chain and number of partial means
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\SeparatedPartialMeans-' ParamStr 'Parameters-' DSGEModel.NameOfModel '-' num2str(NumPM,'%0.0f') '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.txt']);
                        %
                        if FileExist(ResFile)==1;
                           txt = ['YADA has already computed the separated partial means test for ' num2str(NumPM,'%0.0f') ' separated partial means. Would you like to view this file? If you push the "No" button then YADA will recompute the tests.'];
                           GewekeStat = 1;
                           RunTest = 0;
                           answer = Query(txt,'question',200,['Separated Partial Means Test - ' ParamStr ' Parameters'],500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RunTest = 1;
                           end;
                        else;
                           RunTest = 1;
                        end;
                        if RunTest==1;
                           GewekeStat = SeparatedPartialMeansTest(ParamDraws(NumBurnin+1:NumIter,:),Weights,NumPM,[ParamStr ' Parameters'],[],CurrINI);
                           %
                           % write the results to file
                           %
                           if isempty(GewekeStat)==0;
                              PrintSeparatedPartialMeans(GewekeStat,NumPM,CurrChain,NumChains,NumIter,NumBurnin,[ParamStr ' Parameters'],ParameterNames.estimated,[],DSGEModel,CurrINI);
                           end;
                        end;
                        if (isempty(GewekeStat)==0)&(FileExist(ResFile)==1);
                           TextGUI(ResFile,['Separated Partial Means Test - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],[ParamStr ' Parameters'],50,CurrINI,1,1);
                        end;
                     end;
                  else;
                     txt = ['YADA was unable to find any interger p between 2 and 20 such that ' num2str(NumIter-NumBurnin,'%0.0f') '/2p is an integer. YADA is therefore unable to calculate Geweke''s separated partial means test.'];
                     About(txt,'information',['Separated Partial Means - ' ParamStr ' Parameters'],200,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('ParamDraws');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeAcceptance'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Acceptance Ratio','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','LogPost','thetaPostSample','Weights');
               %
               % send the data to the acceptance ratio plotting function
               %
               drawnow;
               AcceptanceRatioPlot(AcceptedDraws,NumBurnin,CurrChain,[],DSGEModel,CurrINI);
               %
               % clear everything
               %
               clear('AcceptedDraws');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SMCAdapation'
   	if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Acceptance Ratio','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
				% Number of posterior draws
				%
				DrawsStr = get(controls.posterior.draws,'String');
				NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
				%
				% number of tempering stages
				%
				TempStagesStr = get(controls.posterior.temperingstages,'String');
				TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
				%
				if DSGEModel.PosteriorSampler==8;
					PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumPostDraws,'%0.0f') '.mat']);
				else;
					PostSampleFile = '';
				end;
				if FileExist(PostSampleFile)==1;
					PSData = load(PostSampleFile);
					NumberAcceptedDraws = PSData.NumberAcceptedDraws;
					ScaleParameter = PSData.ScaleParameter;
					EffectiveSampleSize = PSData.EffectiveSampleSize;
					pstar = PSData.pstar;
					ResamplingThreshold = PSData.ResamplingThreshold;
					clear('PSData');
					%
					% graph the results
					%
					GraphSMCAdaption(NumberAcceptedDraws,ScaleParameter,EffectiveSampleSize,ResamplingThreshold,pstar,[],NumPostDraws,CurrChain,NumChains,DSGEModel,CurrINI);
				else;
					txt = ['The file for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' is not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
				end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeMANOVA'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine the current number of MCMC chains
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         % determine the number of posterior draws
         %
         DrawsStr = get(controls.posterior.draws,'String');
         NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
         %
         % determine the number of burn-in draws
         %
         BurninStr = get(controls.posterior.burnin,'String');
         NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
         %
         % check the results file
         %
         ResFile = FixFilePath([DSGEModel.OutputDirectory '\manova\MANOVA-' DSGEModel.NameOfModel '-' num2str(NumChains,'%0.0f') '.' num2str(NumBurnin,'%0.0f') '.' num2str(NumPostDraws,'%0.0f') '.mat']);
         if FileExist(ResFile)==1;
            txt = ['You have already computed the multivariate potential scale reduction factors for ' num2str(NumChains,'%0.0f') ' MCMC chains with ' num2str(NumPostDraws,'%0.0f') ' posterior draws and ' num2str(NumBurnin,'%0.0f') ' burn-in draws. Would you like to use the old results?'];
            answer = Query(txt,'question',200,'Multivariate ANOVA',500,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               RunANOVA = 0;
               ANOVAData = load(ResFile);
               MPSRF = ANOVAData.MPSRF;
               DoneCalc = 1;
            else;
               RunANOVA = 1;
            end;
         else;
            RunANOVA = 1;
         end;
         if RunANOVA==1;
            %
            % run the multivariate ANOVA functions
            %
            [DoneCalc,MPSRF] = MultiANOVA(NumChains,1,[],[],DSGEModel,CurrINI,controls);
         end;
         if DoneCalc==1;
            PlotMultiANOVA(MPSRF,NumBurnin,NumChains,[],DSGEModel,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeMean'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Posterior Mean','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','LogPost','AcceptedDraws');
               %
               % send the data to the posterior mean and median plotting function
               %
               drawnow;
               PosteriorMeanMedianPlots(thetaPostSample,Weights,NumBurnin,CurrChain,'Mean',[],[],DSGEModel,CurrINI);
               %
               % clear everything
               %
               clear('thetaPostSample');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConvergeMedian'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Posterior Median','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','LogPost','AcceptedDraws');
               %
               % send the data to the posterior mean and median plotting function
               %
               drawnow;
               PosteriorMeanMedianPlots(thetaPostSample,Weights,NumBurnin,CurrChain,'Median',[],[],DSGEModel,CurrINI);
               %
               % clear everything
               %
               clear('thetaPostSample');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewModestyStat'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         file = get(gcbo,'UserData');
         ChainStr = strrep(strrep(get(gcbo,'Label'),'&',''),'...','');
         %
         % display the text file
         %
         TextGUI(file,['Modesty Statistics - ' ChainStr],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',40,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'IteratedParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if isempty(ErrorStr)==1;
            if isfield(ModeData,'phiEstimates')==1;
               phiEstimates = ModeData.phiEstimates;
               thetaEstimates = zeros(size(phiEstimates));
               for i=1:size(phiEstimates,2);
                  thetaEstimates(:,i) = PhiToTheta(phiEstimates(:,i),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
               end;
					txt = strvcat('Would you like to save to a mat-file the iterated parameter estimates at a particular iteration of the posterior mode estimation run?',' ','NOTE: The alternative is to view plots of the iterated estimates of the individual parameters.');
					answer = Query(txt,'question',300,'Iterated Parameter Estimates',600,'no',CurrINI);
					if strcmp(lower(answer),'yes')==1;
						if isfield(ModeData,'OptResults')==1;
							%
							% the log posterior kernel estimates are located in the second column of OptResults
							%
							LogPostKernel = ModeData.OptResults(:,2);
						else;
							%
							% we need to recompute the log posterior kernel for all phi estimates
							%
							LogPostKernel = DSGELogPosteriorKernel(phiEstimates,ModeData,DSGEModel,CurrINI);
							%
							% save the results to the mat-file
							%
							OptResults = zeros(length(LogPostKernel),4);
							OptResults(:,2) = LogPostKernel;
							save(ModeFile,'OptResults','-APPEND')
						end;
						NumIter = size(phiEstimates,2);
						if NumIter>1;
							ParamSample = thetaEstimates(:,2:NumIter);
						else;
							ParamSample = thetaEstimates;
						end;
						%
						% select the parameter value to use
						%
						SelStr = ['Iteration   1: logPost = ' num2str(LogPostKernel(1),'%0.6f')];
						for i=2:NumIter-1;
							SelStr = strvcat(SelStr,sprintf('Iteration %3.0f: logPost = %0.6f',i,LogPostKernel(i)));
						end;
						[action,SelValue] = SelectionDlg(SelStr,1,'Select the iteration number to use for showing parameter values','Parameter Value Selection','Iterated Parameter Estimates','','',CurrINI);
						if strcmp(lower(action),'ok')==1;
							ParamValue = ParamSample(:,SelValue);
							LPK = LogPostKernel(SelValue);
							%
							% save to mat-file
							%
							MatFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-' DSGEModel.NameOfModel '-Iteration-' int2str(SelValue) '.mat']);
							ParameterNames = ModeData.ParameterNames.estimated;
							save(MatFile,'ParamValue','LPK','ParameterNames');
							%
							% write an m-file to extract the data
							%
							mfile = FixFilePath([DSGEModel.OutputDirectory '\ParamVals_' DSGEModel.NameOfModel '_Iteration_' int2str(SelValue) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                     fprintf(fid,'%%\n%% string matrix with the parameter names\n%%\n');
                     fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
                     fprintf(fid,'%%\n%% vector with values for the original parameters\n%%\n');
                     fprintf(fid,'ParamValue = DataStructure.ParamValue;\n');
                     fprintf(fid,'%%\n%% value of the log posterior kernel\n%%\n');
                     fprintf(fid,'LPK = DataStructure.LPK;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The values for the original parameters have been saved to the file "' GetFilename(MatFile) '" in the directory "' GetPath(MatFile) '". The file contains 3 entries: ParameterNames (string matrix with the names of the parameters), ParamValue (vector with the values of the parameters at iteration no ' int2str(SelValue) ' of the posterior mode estimation run), and LPK (scalar with the value of the log posterior kernel at the selected parameter values). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Iterated Parameter Estimates',400,500,CurrINI);
                     drawnow;
						end;
					else;
	               %
	               % open the graph iterated parameters dialog
	               %
	               IteratedParamsGraphDLG('init',CurrINI,ModeData.ParameterNames.estimated,[ModeData.theta thetaEstimates]);
	            end;
            else;
               txt = 'The data file with the posterior mode estimates didn''t include the iterated parameter estimates. YADA is therefore unable to display them.';
               About(txt,'information','YADA - Old MAT File',150,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CheckPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % run the check optimum function
         %
         CheckPosteriorMode(DSGEModel,CurrINI,controls);
         set(controls.about,'UserData',0);
      end;
   case 'ThreeDeePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if isempty(ErrorStr)==1;
            if gcbo==controls.threedeepostmode.transformed;
               UseTransformed = 1;
            else;
               UseTransformed = 0;
            end;
            %
            % display the dialog for selecting parameters
            %
            ThreeDeeParamsGraphDLG('init',DSGEModel,CurrINI,ModeData,UseTransformed);
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ViewPrior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Prior Density Data',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % we can now construct the plots
            %
            if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
               GraphPriorDensity(theta,thetaDist,LowerBound,PriorDist,ParameterNames,thetaPositions,ModelParameters,controls,DSGEModel,CurrINI);
            else;
               GraphSystemPriorDensity(DSGEModel,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ViewLaplacePosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load data from file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            thetaMode = ModeData.thetaMode;
            InverseHessian = ModeData.InverseHessian;
            if isfield(ModeData,'ThetaPhiPartial')==1;
               ThetaPhiPartial = ModeData.ThetaPhiPartial;
            else;
               ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
            end;
            if isfield(ModeData,'thetaAdditionalMode')==1;
               thetaAdditionalMode = ModeData.thetaAdditionalMode;
            else;
               [thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModeData.ModelParameters,ModeData.ParameterNames);
            end;
            if (isfield(ModeData,'phiInverseHessianFinDiff')==1)&(all(~isnan(ModeData.phiInverseHessianFinDiff))==1)&(size(ModeData.phiInverseHessianFinDiff,1)==length(thetaMode));
               txt = 'When computing the Laplace approximation of the posterior densities, would you like to use the finite difference estimator of the inverse Hessian instead of the inverse Hessian from the optimization routine?';
               answer = Query(txt,'question',200,'Laplace Posterior Densities',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  thetaInverseHessian = ThetaPhiPartial*ModeData.phiInverseHessianFinDiff*ThetaPhiPartial;
               else;
                  thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
               end;
            else;
               thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
            end;
            %
            % we can now construct the plots
            %
            GraphLaplacePosteriorDensity(thetaMode,thetaInverseHessian,thetaAdditionalMode,ModeData.theta,ModeData.thetaDist,ModeData.LowerBound,ModeData.PriorDist,ModeData.ParameterNames,ModeData.thetaPositions,ModeData.ModelParameters,[],'DSGE Posterior Mode',controls,DSGEModel,CurrINI);
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Posterior Densities','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','LogPost','AcceptedDraws');
               %
               % try to load the ModeFile to get names of parameters
               %
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
               try;
                  ModeData = load(ModeFile);
                  ParameterNames = ModeData.ParameterNames;
               catch;
                  ParameterNames.additional = []
               end;
               %
               % check if we have parameters that are functions of estimated parameters
               %
               if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
                  %
                  % ask if the user wants to load additional parameters
                  %
                  txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot posterior densities for these additional parameters?'];
                  answer = Query(txt,'question',200,'Posterior Densities - Additional Parameters',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     [UpdatePostSample,UpdateMode,UpdateNames] = GetUpdatePosteriorDraws(thetaPostSample,NumBurnin,CurrChain,NumChains,1,DSGEModel,CurrINI);
                     %
                     % check if the user has the necessary tools for computing prior densitites
                     % for the additional parameters through random draws from the prior of the
                     % relevant estimated parameters
                     %
                     if isempty(UpdateMode)==0;
                        if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
                           IsSystemPrior = 0;
                        else;
                           IsSystemPrior = 1;
                        end;
                        TmpCurrINI = CurrINI;
                        TmpCurrINI.UpdateNames = UpdateNames;
                        if IsSystemPrior==0;
                           [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                        else;
                           [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                        end;
                        %
                        % test if the results were obtained from an old mat-file
                        %
                        if max(UpdateParameters)>length(UpdateMode);
                           if DSGEModel.PosteriorSampler==1;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==2;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Slice-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==3;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==4;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==5;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==6;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==7;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==8;
                              TempStagesStr = get(controls.posterior.temperingstages,'String');
                              TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-Additional-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(size(thetaPostSample,1)) '.mat']);
                           end;
                           delete(AddFile);
                           if IsSystemPrior==0;
                              [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                           else;
                              [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = GetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                           end;
                        end;
                        clear('TmpCurrINI');
                        if isempty(UpdatePriorX)==0;
                           if length(UpdateMode)>length(UpdateParameters);
                              UpdateMode = UpdateMode(UpdateParameters);
                              UpdateNames = UpdateNames(UpdateParameters,:);
                              UpdatePostSample = UpdatePostSample(:,UpdateParameters);
                           end;
                        else;
                           txt = 'YADA has determined that all parameters that are supposed to be functions of the estimated parameters are, in fact, fixed. YADA will therefore skip these parameters in the plots.';
                           About(txt,'information','Posterior Densities - Additional Parameters',300,500,CurrINI);
                           UpdatePostSample = [];
                           UpdateMode = [];
                           UpdateNames = '';
                           UpdatePriorX = [];
                           UpdatePriorY = [];
                           UpdatePriorBounds = [];
                        end;
                     else;
                        UpdatePostSample = [];
                        UpdateNames = '';
                        UpdatePriorX = [];
                        UpdatePriorY = [];
                        UpdatePriorBounds = [];
                     end;
                  else;
                     UpdatePostSample = [];
                     UpdateMode = [];
                     UpdateNames = '';
                     UpdatePriorX = [];
                     UpdatePriorY = [];
                     UpdatePriorBounds = [];
                  end;
               else;
                  UpdatePostSample = [];
                  UpdateMode = [];
                  UpdateNames = '';
                  UpdatePriorX = [];
                  UpdatePriorY = [];
                  UpdatePriorBounds = [];
               end;
               %
               % send the data to the posterior densities plotting function
               %
               drawnow;
               GraphPosteriorDensity(thetaPostSample,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,DSGEModel,CurrINI,controls);
               %
               % clear everything
               %
               clear('thetaPostSample');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPriorValueInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check the status of the DSGE model files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         %
         if InitStatus==1;
            logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
            lnjac = logJacobian(ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound),thetaIndex,UniformBounds);
            Status = 1;
            if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
               %
               % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
               %
               [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
               [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
               if FileExist(DSGEModel.UpdateParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
               end;
               if FileExist(DSGEModel.InitializeParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
               end;
               [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
               ErrorStr = '';
               ErrorFile = '';
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
                  ErrorFile = DSGEModel.AIMDataFile;
               end;
               %
               if isempty(ErrorStr)==1;
                  %
                  % compute the system prior
                  %
                  [logPrior,logSP] = logSystemPriorPhiDSGE(ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound),thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,AIMData,DSGEModel,CurrINI);
               else;
                  About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,500,CurrINI);
                  drawnow;
                  Status = 0;
               end;
            else;
               logSP = [];
            end;
            if Status==1;
               txt = '********************************************************************************';
               txt = strvcat(txt,'*                                                                              *');
               txt = strvcat(txt,'*        V A L U E S   O F   T H E   L O G   P R I O R   D E N S I T Y         *');
               txt = strvcat(txt,'*                A N D   O F   T H E   L O G   J A C O B I A N                 *');
               txt = strvcat(txt,'*                                                                              *');
               txt = strvcat(txt,'********************************************************************************',' ');
               txt = strvcat(txt,'Parameters evaluated at the initial parameter values',' ');
               prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
               if isempty(logSP)==1;
                  txt = strvcat(txt,sprintf(['log prior for theta:        ' prt_val],logPrior));
                  txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi): ' prt_val],lnjac));
                  txt = strvcat(txt,sprintf(['log prior for phi:          ' prt_val],logPrior+lnjac),' ');
                  txt = strvcat(txt,'NOTE: The sum of the log prior for theta (original parameters) and the log','      Jacobian is equal to the log prior of phi (transformed parameters).',' ');
               else;
                  txt = strvcat(txt,sprintf(['marginal log prior for theta:        ' prt_val],logPrior));
                  txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi):          ' prt_val],lnjac));
                  txt = strvcat(txt,sprintf(['marginal log prior for phi:          ' prt_val],logPrior+lnjac));
                  txt = strvcat(txt,sprintf(['conditional log system prior:        ' prt_val],logSP),' ');
                  txt = strvcat(txt,'NOTE: The sum of the marginal log prior for theta (original parameters) and','      the log Jacobian is equal to the marginal log prior of phi (transformed','      parameters). The conditional log system prior is added to get the full','      prior up to a constant, determined by the log marginal likelihood of','      the system prior.',' ');
               end;
               %
               % write to file
               %
               PriorJacValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Prior-Jacobian-' DSGEModel.NameOfModel '-InitialValues.txt']);
               fid = fopen(PriorJacValueFile,'wt');
               for i=1:size(txt,1);
                  fprintf(fid,'%s\n',txt(i,:));
               end;
               fprintf(fid,'================================================================================\n\n');
               %
               % bottom file data
               %
               fprintf(fid,'Directory for file: %s\n',GetPath(PriorJacValueFile));
               fprintf(fid,'Name of file:       %s\n',GetFilename(PriorJacValueFile));
               fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
               fclose(fid);
               %
               if FileExist(PriorJacValueFile)==1;
                  TextGUI(PriorJacValueFile,['DSGE Model Prior and Jacobian - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Initial Parameter Values',50,CurrINI,1,1);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPriorValuePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load data from file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            lnjac = logJacobian(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
            logPrior = logPriorDSGE(ModeData.thetaMode,ModeData.thetaDist,ModeData.PriorDist,ModeData.LowerBound);
            Status = 1;
            if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
               %
               % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
               %
               [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
               [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
               if FileExist(DSGEModel.UpdateParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
               end;
               if FileExist(DSGEModel.InitializeParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
               end;
               [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
               [logPrior,logSP] = logSystemPriorPhiDSGE(ModeData.phiMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.thetaDist,ModeData.PriorDist,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.AIMData,DSGEModel,CurrINI);
            else;
               logSP = [];
            end;
            txt = '********************************************************************************';
            txt = strvcat(txt,'*                                                                              *');
            txt = strvcat(txt,'*        V A L U E S   O F   T H E   L O G   P R I O R   D E N S I T Y         *');
            txt = strvcat(txt,'*                A N D   O F   T H E   L O G   J A C O B I A N                 *');
            txt = strvcat(txt,'*                                                                              *');
            txt = strvcat(txt,'********************************************************************************',' ');
            txt = strvcat(txt,'Parameters evaluated at the posterior mode parameter values',' ');
            prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
            if isempty(logSP)==1;
               txt = strvcat(txt,sprintf(['log prior for theta:        ' prt_val],logPrior));
               txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi): ' prt_val],lnjac));
               txt = strvcat(txt,sprintf(['log prior for phi:          ' prt_val],logPrior+lnjac),' ');
               txt = strvcat(txt,'NOTE: The sum of the log prior for theta (original parameters) and the log','      Jacobian is equal to the log prior of phi (transformed parameters).',' ');
            else;
               txt = strvcat(txt,sprintf(['marginal log prior for theta:        ' prt_val],logPrior));
               txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi):          ' prt_val],lnjac));
               txt = strvcat(txt,sprintf(['marginal log prior for phi:          ' prt_val],logPrior+lnjac));
               txt = strvcat(txt,sprintf(['conditional log system prior:        ' prt_val],logSP),' ');
               txt = strvcat(txt,'NOTE: The sum of the marginal log prior for theta (original parameters) and','      the log Jacobian is equal to the marginal log prior of phi (transformed','      parameters). The conditional log system prior is added to get the full','      prior up to a constant, determined by the log marginal likelihood of','      the system prior.',' ');
            end;
            %
            % write to file
            %
            PriorJacValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Prior-Jacobian-' DSGEModel.NameOfModel '-PostMode.txt']);
            fid = fopen(PriorJacValueFile,'wt');
            for i=1:size(txt,1);
               fprintf(fid,'%s\n',txt(i,:));
            end;
            fprintf(fid,'================================================================================\n\n');
            %
            % bottom file data
            %
            fprintf(fid,'Directory for file: %s\n',GetPath(PriorJacValueFile));
            fprintf(fid,'Name of file:       %s\n',GetFilename(PriorJacValueFile));
            fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
            fclose(fid);
            %
            if FileExist(PriorJacValueFile)==1;
               TextGUI(PriorJacValueFile,['DSGE Model Prior and Jacobian - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Posterior Mode Parameter Values',50,CurrINI,1,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewData'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % ask if user wants to save to disk
         %
         txt = 'Would you like to save the observed variables to disk?';
         answer = Query(txt,'question',200,'Save - Observed Variables',500,'no',CurrINI);
         if strcmp(lower(answer),'yes')==1;
            %
            % create the annualized data
            %
            DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
            %
            % determine the sub-sample from the full sample
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            Y = DSGEModel.Y(:,FirstPeriod:LastPeriod);
            YNames = DSGEModel.VariableNames;
            %
            %
            % setup a suitable file
            %
            OutputFile = FixFilePath([DSGEModel.OutputDirectory '\ObservedData-' DSGEModel.NameOfModel '.mat']);
            save(OutputFile,'YNames','Y','DateVector');
            %
            % create an m-file to acces the data
            %
            MFile = FixFilePath([DSGEModel.OutputDirectory '\OD' strrep(DSGEModel.NameOfModel,'-','') '.m']);
            fid = fopen(MFile,'wt');
            fprintf(fid,'%%\n%% load the data from "%s"\n%%\n',GetFilename(OutputFile));
            fprintf(fid,'ObsData = load(''%s'');\n',GetFilename(OutputFile));
            fprintf(fid,'%%\n%% the observed data\n%%\n');
            fprintf(fid,'Y = ObsData.Y;\n');
            fprintf(fid,'%%\n%% names of the observed variables\n%%\n');
            fprintf(fid,'YNames = ObsData.YNames;\n');
            fprintf(fid,'%%\n%% and a vector with numerical dates for the data\n%%\n');
            fprintf(fid,'DateVector = ObsData.DateVector;\n');
            fprintf(fid,'%%\n%% add your own code here\n%%\n\n\n');
            %
            fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
            fclose(fid);
            txt = ['YADA has stored the observed variables in the file "' GetFilename(OutputFile) '". It is located in the directory "' DSGEModel.OutputDirectory '". You may access this data by running the script file "' GetFilename(MFile) '" that is located in the same directory.'];
            About(txt,'information','Observed Variables',200,500,CurrINI);
         end;
         %
         % Open the Graph data dialog
         %
         GraphDataDLG('init',CurrINI,DSGEModel,'Original');
         set(controls.about,'UserData',0);
      end;
   case 'ViewAnnualData'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % ask if user wants to save to disk
         %
         txt = 'Would you like to save the annualized observed variables to disk?';
         answer = Query(txt,'question',200,'Save - Annualized Observed Variables',500,'no',CurrINI);
         if strcmp(lower(answer),'yes')==1;
            %
            % create the annualized data
            %
            DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
            %
            % determine the sub-sample from the full sample
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % annualize the data
            %
            Y = Annualize(DSGEModel.Y(:,FirstPeriod:LastPeriod),DSGEModel.annual,DSGEModel.annualscale);
            YNames = DSGEModel.VariableNames;
            DateVector = DateVector(1,length(DateVector)-size(Y,2)+1:length(DateVector));
            %
            % setup a suitable file
            %
            OutputFile = FixFilePath([DSGEModel.OutputDirectory '\AnnualizedData-' DSGEModel.NameOfModel '.mat']);
            save(OutputFile,'YNames','Y','DateVector');
            %
            % create an m-file to acces the data
            %
            MFile = FixFilePath([DSGEModel.OutputDirectory '\AD' strrep(DSGEModel.NameOfModel,'-','') '.m']);
            fid = fopen(MFile,'wt');
            fprintf(fid,'%%\n%% load the data from "%s"\n%%\n',GetFilename(OutputFile));
            fprintf(fid,'AnnualData = load(''%s'');\n',GetFilename(OutputFile));
            fprintf(fid,'%%\n%% the annualized data\n%%\n');
            fprintf(fid,'Y = AnnualData.Y;\n');
            fprintf(fid,'%%\n%% names of the observed variables\n%%\n');
            fprintf(fid,'YNames = AnnualData.YNames;\n');
            fprintf(fid,'%%\n%% and a vector with numerical dates for the data\n%%\n');
            fprintf(fid,'DateVector = AnnualData.DateVector;\n');
            fprintf(fid,'%%\n%% add your own code here\n%%\n\n\n');
            %
            fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
            fclose(fid);
            txt = ['YADA has stored the annualized observed variables in the file "' GetFilename(OutputFile) '". It is located in the directory "' DSGEModel.OutputDirectory '". You may access this data by running the script file "' GetFilename(MFile) '" that is located in the same directory.'];
            About(txt,'information','Annualized Observed Variables',200,500,CurrINI);
         end;
         %
         % Open the Graph data dialog
         %
         GraphDataDLG('init',CurrINI,DSGEModel,'Annualized');
         set(controls.about,'UserData',0);
      end;
   case 'ViewTransData'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % try to transform the data
         %
         DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
         if DTStatus==1;
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         else;
            ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
            HeaderStr = 'Missing Transformation Information';
         end;
         %
         % Open the Graph data dialog
         %
         if isempty(ErrorStr)==1;
            %
            % ask if user wants to save to disk
            %
            txt = 'Would you like to save the transformed observed variables to disk?';
            answer = Query(txt,'question',200,'Save - Transformed Observed Variables',500,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               %
               % save transformed data to disk
               %
               TransformedY = YData';
               if size(TransformedY,2)<size(DSGEModel.Y,2);
                  TransformedY = [NaN*ones(DSGEModel.n,size(DSGEModel.Y,2)-size(TransformedY,2)) TransformedY];
               end;
               YNames = DSGEModel.VariableNames;
               DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               TransformedY = TransformedY(:,FirstPeriod:LastPeriod);
               %
               OutputFile = FixFilePath([DSGEModel.OutputDirectory '\TransformedData-' DSGEModel.NameOfModel '.mat']);
               save(OutputFile,'YNames','TransformedY','DateVector');
               %
               % create an m-file to acces the data
               %
               MFile = FixFilePath([DSGEModel.OutputDirectory '\TD' strrep(DSGEModel.NameOfModel,'-','') '.m']);
               fid = fopen(MFile,'wt');
               fprintf(fid,'%%\n%% load the data from "%s"\n%%\n',GetFilename(OutputFile));
               fprintf(fid,'TransData = load(''%s'');\n',GetFilename(OutputFile));
               fprintf(fid,'%%\n%% the transformed data\n%%\n');
               fprintf(fid,'TransformedY = TransData.TransformedY;\n');
               fprintf(fid,'%%\n%% names of the observed variables\n%%\n');
               fprintf(fid,'YNames = TransData.YNames;\n');
               fprintf(fid,'%%\n%% and a vector with numerical dates for the data\n%%\n');
               fprintf(fid,'DateVector = TransData.DateVector;\n');
               fprintf(fid,'%%\n%% add your own code here\n%%\n\n\n');
               %
               fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
               fclose(fid);
               txt = ['YADA has stored the transformed observed variables in the file "' GetFilename(OutputFile) '". It is located in the directory "' DSGEModel.OutputDirectory '". You may access this data by running the script file "' GetFilename(MFile) '" that is located in the same directory.'];
               About(txt,'information','Transformed Observed Variables',200,500,CurrINI);
            end;
            %
            % display the data
            %
            GraphDataDLG('init',CurrINI,DSGEModel,'Transformed');
         else;
            About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of ViewMenuFunctions.m
%
